Package: apsimx
Title: Inspect, Read, Edit and Run 'APSIM' "Next Generation" and
        'APSIM' Classic
Version: 2.8.235
Authors@R: person("Fernando", "Miguez", email = "femiguez@iastate.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4627-8329"))
Description: The functions in this package inspect, read, edit and run files for 'APSIM' "Next Generation" ('JSON')
             and 'APSIM' "Classic" ('XML'). The files with an 'apsim' extension correspond to
	     'APSIM' Classic (7.x) - Windows only - and the ones with an 'apsimx' extension correspond to 'APSIM' "Next Generation".
	     For more information about 'APSIM' see (<https://www.apsim.info/>) and for 'APSIM'
	     next generation (<https://apsimnextgeneration.netlify.app/>). 
Depends: R (>= 4.0.0)
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/femiguez/apsimx/issues
Imports: DBI, jsonlite, knitr, RSQLite, tools, utils, xml2
Suggests: BayesianTools, chirps, datasets, daymetr, future, ggplot2,
        GSODR, listviewer, maps, metrica, mvtnorm, nasapower, nloptr,
        parallelly, reactR, rmarkdown, sensitivity, soilDB, sp, spData,
        sf, ucminf
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-03-09 22:04:37 UTC; fernandomiguez
Author: Fernando Miguez [aut, cre] (<https://orcid.org/0000-0002-4627-8329>)
Maintainer: Fernando Miguez <femiguez@iastate.edu>
Repository: CRAN
Date/Publication: 2025-03-10 05:40:02 UTC
Built: R 4.3.3; ; 2025-03-10 06:13:24 UTC; unix

Package: apsimx
Title: Inspect, Read, Edit and Run 'APSIM' "Next Generation" and
        'APSIM' Classic
Version: 2.8.271
Authors@R: person("Fernando", "Miguez", email = "femiguez@iastate.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4627-8329"))
Description: The functions in this package inspect, read, edit and run files for 'APSIM' "Next Generation" ('JSON')
             and 'APSIM' "Classic" ('XML'). The files with an 'apsim' extension correspond to
	     'APSIM' Classic (7.x) - Windows only - and the ones with an 'apsimx' extension correspond to 'APSIM' "Next Generation".
	     For more information about 'APSIM' see (<https://www.apsim.info/>) and for 'APSIM'
	     next generation (<https://apsimnextgeneration.netlify.app/>). 
Depends: R (>= 4.0.0)
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/femiguez/apsimx/issues
Imports: DBI, jsonlite, knitr, RSQLite, tools, utils, xml2
Suggests: BayesianTools, chirps, datasets, daymetr, future, ggplot2,
        GSODR, listviewer, maps, metrica, mgcv, mvtnorm, nasapower,
        nloptr, parallelly, reactR, rmarkdown, sensitivity, soilDB, sp,
        spData, sf, ucminf
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-08 17:02:15 UTC; femiguez
Author: Fernando Miguez [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4627-8329>)
Maintainer: Fernando Miguez <femiguez@iastate.edu>
Repository: CRAN
Date/Publication: 2026-07-13 23:40:56 UTC

# Generated by roxygen2: do not edit by hand

S3method("$<-",met)
S3method(coef,optim_apsim)
S3method(confint,optim_apsim)
S3method(plot,met)
S3method(plot,met_mrg)
S3method(plot,out_mrg)
S3method(plot,soil_profile)
S3method(plot,soil_profile_mrg)
S3method(print,met)
S3method(print,met_mrg)
S3method(print,optim_apsim)
S3method(print,out_mrg)
S3method(print,sens_apsim)
S3method(print,soil_profile_mrg)
S3method(summary,met)
S3method(summary,sens_apsim)
S3method(vcov,optim_apsim)
export(add_column_apsim_met)
export(amp_apsim_met)
export(apsim)
export(apsim.options)
export(apsim_example)
export(apsim_options)
export(apsim_version)
export(apsimx)
export(apsimx.options)
export(apsimx_example)
export(apsimx_filetype)
export(apsimx_options)
export(apsimx_soil_profile)
export(as_apsim_met)
export(auto_detect_apsim_examples)
export(auto_detect_apsimx_examples)
export(available_water_content)
export(carbon_stocks)
export(check_apsim_met)
export(check_apsimx)
export(check_apsimx_soil_profile)
export(compare_apsim)
export(compare_apsim_met)
export(compare_apsim_soil_profile)
export(date2doy)
export(doy2date)
export(edit_apsim)
export(edit_apsim_replace_soil_profile)
export(edit_apsim_xml)
export(edit_apsimx)
export(edit_apsimx_batch)
export(edit_apsimx_replace_soil_profile)
export(edit_apsimx_replacement)
export(extract_data_apsimx)
export(extract_values_apsimx)
export(get_apsimx_json)
export(get_chirps_apsim_met)
export(get_daymet2_apsim_met)
export(get_daymet_apsim_met)
export(get_gsod_apsim_met)
export(get_iem_apsim_met)
export(get_iemre_apsim_met)
export(get_isric_soil_profile)
export(get_power_apsim_met)
export(get_slga_soil)
export(get_slga_soil_profile)
export(get_ssurgo_soil_profile)
export(get_ssurgo_tables)
export(get_worldmodeler_apsim_met)
export(get_worldmodeler_soil_profile)
export(grep_json_list)
export(impute_apsim_met)
export(initialwater_parms)
export(insert_replacement_node)
export(inspect_apsim)
export(inspect_apsim_xml)
export(inspect_apsimx)
export(inspect_apsimx_json)
export(inspect_apsimx_replacement)
export(mcmc.apsim.env)
export(mcmc.apsimx.env)
export(napad_apsim_met)
export(optim_apsim)
export(optim_apsimx)
export(read_apsim)
export(read_apsim_all)
export(read_apsim_met)
export(read_apsim_soils)
export(read_apsimx)
export(read_apsimx_all)
export(remove_column_apsim_met)
export(sens_apsim)
export(sens_apsimx)
export(soilorganicmatter_parms)
export(soilwat_parms)
export(solutes_parms)
export(ssurgo2sp)
export(swim_parms)
export(tav_apsim_met)
export(tt_apsim_met)
export(unit_conv)
export(view_apsim)
export(view_apsim_xml)
export(view_apsimx)
export(write_apsim_met)
export(xargs_apsimx)
import(DBI)
import(RSQLite)
import(jsonlite)
import(knitr)
import(xml2)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,deviance)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,sigma)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,write.table)

**apsimx: R package for APSIM-X (NextGen) and APSIM Classic (7.x)**

[![CRAN](http://www.r-pkg.org/badges/version/apsimx)](https://CRAN.R-project.org/package=apsimx)
[![CRAN
downloads total](https://cranlogs.r-pkg.org/badges/grand-total/apsimx)](https://github.com/r-hub/cranlogs.app)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/apsimx)](https://cran.r-project.org/package=apsimx)


This package allows for interaction with APSIM-X ("Next Generation")
and/or APSIM 'Classic' (7.10). It can inspect, edit, run and read
APSIM files in both platforms. The format is JSON for APSIM-X and XML
for 'Classic'.

**Package requirements**

* Imported R packages: DBI, jsonlite, knitr, RSQLite, tools, utils, xml2

* Suggested R packages: BayesianTools, datasets, daymetr, FedData, future, ggplot2,
  GSODR, listviewer, maps, mvtnorm, nasapower, nloptr, reactR, rmarkdown, soilDB, sp, spData, sf, ucminf

* APSIMX:

* and/or APSIM (7.x) 'Classic'

NOTE ON REQUIREMENTS:

* Current versions of APSIM Next Gen do not require additional software
to run on Mac or Linux. The dotnet runtime environment is shipped with the image/package.

* For current versions, on Windows, the 'Microsoft Windows Desktop Runtime' 3.1.14 is required. The installer also updated GTK3 to 3.24.20 during the installation.

* For older versions of APSIM Next Gen (before Sept 2021) the mono framework was required to run on Mac and Linux (Debian). Mono should be installed first (in Mac and Linux). 

**If you are running the latest version of APSIM Next Gen, you do 
not need to install the Mono Framework**.

**Mono framework download**:
https://www.mono-project.com/download/stable/

**.Net Core download for Mac**:
https://dotnet.microsoft.com/download

**APSIMX download**:
https://www.apsim.info/download-apsim/

Since APSIM Next Gen 2021.04.01 (at least) .NET framework 4.6 or higher is 
required for Windows and I have had to update the Mono framework to 
6.12 on Mac (again this applies to pre Sep 2021).

If you want to install this package from github try in R:

> library(devtools) \
> devtools::install_github("femiguez/apsimx") \
> library(apsimx)

or the lightweight 'remotes' package

> library(remotes) \
> remotes::install_github("femiguez/apsimx") \
> library(apsimx)

It is **not** necessary to build the vignettes as they are also at:
https://femiguez.github.io/apsimx-docs/

*Note*: Building the vignettes does not require the presence of APSIM-X as I
have recently eliminated the APSIM-X dependency.

Still, if you want to build the vignettes, then use this instead
of the second line above:

> devtools::install_github("femiguez/apsimx", build_vignettes = TRUE,
> build_opts = c("--no-resave-data", "--no-manual")) 

If you do build the vignettes, there is an introduction to the package

> vignette("apsimx")

and a document which might help you build your own scripts either in R
or other languages

> vignette("apsimx-scripts")

If you have any questions contact Fernando E. Miguez (femiguez *at* iastate.edu)

Some papers that cite this package:

14\. van Versendaal et al.  Integrating Field Data and a Modeling Approach to Inform Optimum Planting Date × Maturity Group for Soybeans under Current and Future Weather Conditions in Kansas. Sustainability 2023, 15(2), 1081; https://doi.org/10.3390/su15021081

13\. R. H. K. Rathnappriya et al. Global Sensitivity Analysis of Key Parameters in the APSIMX-Sugarcane Model to Evaluate Nitrate Balance via Treed Gaussian Process. Agronomy 2022, 12(8), 1979; https://doi.org/10.3390/agronomy12081979

12\. Francisco Palmero, Ana J.P. Carcedo, Ricardo J. Haro, Ezequiel D. Bigatton, Fernando Salvagiotti, Ignacio A. Ciampitti. Modeling drought stress impacts under current and future climate for peanut in the semiarid pampas region of Argentina. Field Crops Research. 2022. https://doi.org/10.1016/j.fcr.2022.108615.

11\. Elsa Lagerquist, Iris Vogeler, Uttam Kumar, Göran Bergkvist, Marcos Lana, Christine A. Watson, David Parsons,
Assessing the effect of intercropped leguminous service crops on main crops and soil processes using APSIM NG.
Agricultural Systems. 2024. https://doi.org/10.1016/j.agsy.2024.103884.

10\. Daniel Pasquel, Davide Cammarano, Sébastien Roux, Annamaria Castrignanò, Bruno Tisseyre, Michele Rinaldi, Antonio Troccoli, James A. Taylor. Downscaling the APSIM crop model for simulation at the within-field scale,
Agricultural Systems. 2023. https://doi.org/10.1016/j.agsy.2023.103773.

9\. Tommaso Tadiello, Mara Gabbrielli, Marco Botta, Marco Acutis, Luca Bechini, Giorgio Ragaglini, Andrea Fiorini, Vincenzo Tabaglio, Alessia Perego. A new module to simulate surface crop residue decomposition: Description and sensitivity analysis. Ecological Modelling. Volume 480. 2023. https://doi.org/10.1016/j.ecolmodel.2023.110327.

8\. Lopez-Cruz, M., Aguate, F.M., Washburn, J.D. et al. Leveraging data from the Genomes-to-Fields Initiative to investigate genotype-by-environment interactions in maize in North America. Nat Commun 14, 6904 (2023). https://doi.org/10.1038/s41467-023-42687-4

7\. Ignacio Massigoge, Ana Carcedo, Jane Lingenfelser, Trevor Hefley, P.V. Vara Prasad, Dan Berning, Sara Lira, Carlos D. Messina, Charles W. Rice, Ignacio Ciampitti. Maize planting date and maturity in the US central Great Plains: Exploring windows for maximizing yields. European Journal of Agronomy. Volume 149. 2023. 
https://doi.org/10.1016/j.eja.2023.126905.

6\. Yang, Xuening and Zhang, Xuanze and Zhao, Zhigan and Ma, Ning and Tian, Jing and Xu, Zhenwu and Zhang, Junmei and Zhang, Yongqiang, Rainfall and Maximum Temperature are Dominant Climatic Factors Influencing Apsim-Maize Cultivar Parameters Sensitivity in Semiarid Regions. Available at SSRN: https://ssrn.com/abstract=4693866 or http://dx.doi.org/10.2139/ssrn.4693866

5\. Determining site-specific corn nitrogen rate over time with APSIM model. L.J. Thompson, S. Archontoulis, and L.A. Puntel. https://doi.org/10.3920/978-90-8686-947-3_138

4\. Kheir, A. M. S., Mkuhlani, S., Mugo, J. W., Elnashar, A., Nangia, V., Devare, M., & Govind, A. (2023). Integrating APSIM model with machine learning to predict wheat yield spatial distribution. Agronomy Journal, 115, 3188–3196. https://doi.org/10.1002/agj2.21470

3\. Acceptability and Evaluation of APSIM-Qryza for Promoting Water and Nitrogen Productivity in Paddy Fields. 
Shayan Hosseinpour, Hemmatollah Pirdashti, Mohammad Kaveh, Hamze Dokoohaki. https://doi.org/10.21203/rs.3.rs-2677879/v1

2\. Augmenting agroecosystem models with remote sensing data and machine learning increases overall estimates of nitrate-nitrogen leaching. Matthew Nowatzke, Luis Damiano, Fernando E Miguez, Gabe S McNunn, Jarad Niemi, Lisa A Schulte, Emily A Heaton and Andy VanLoocke. 20 October 2022. Environmental Research Letters, Volume 17, Number 11
DOI: 10.1088/1748-9326/ac998b

1\. Laurent, A., Cleveringa, A., Fey, S. et al. Late-season corn stalk nitrate measurements across the US Midwest from 2006 to 2018. Sci Data 10, 192 (2023). https://doi.org/10.1038/s41597-023-02071-9

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)

## ----apsimx-options, echo = FALSE, eval = FALSE-------------------------------
# apsimx_options(warn.versions = FALSE)
# ava <- apsim_version(verbose = FALSE)
# aiu <- apsim_version(which = "inuse")
# ##if(is.na(ava[2,2])) stop("Need APSIM-X to create this vignette")

## ----create-temp-dir, echo = TRUE, eval = TRUE--------------------------------
## Will only write file to a temporary directory for these Examples
tmp.dir <- tempdir()

## ----apsimx-example, eval = FALSE---------------------------------------------
# maize <- apsimx_example("Maize")

## ----read-in-maize-example, eval = TRUE, echo = FALSE-------------------------
extd.dir <- system.file("extdata", package = "apsimx")
## maize <- read_apsimx("Maize.db", src.dir = extd.dir)
maize <- read.csv(paste0(extd.dir, "/Maize.csv"))
maize$Date <- as.Date(maize$Date)

## ----summary-ggplot-maize-----------------------------------------------------
summary(maize)
## Simple data plotting
ggplot(data = maize , aes(x = Date, y = Maize.AboveGround.Wt)) + 
  geom_point()

## ----inspect-apsimx-----------------------------------------------------------
extd.dir <- system.file("extdata", package = "apsimx")
inspect_apsimx("Maize.apsimx", src.dir = extd.dir, node = "Weather")

## ----inspect-maize-oc---------------------------------------------------------
inspect_apsimx("Maize.apsimx", src.dir = extd.dir, 
               node = "Soil", soil.child = "Organic")

## ----edit-maize-oc------------------------------------------------------------
ocs <- c(1.5, 1.4, 1.3, 1.2, 1.1, 1.0, 0.9)
edit_apsimx("Maize.apsimx", 
            src.dir = extd.dir,
            wrt.dir = ".",
            node = "Soil",
            soil.child = "Organic", 
            parm = "Carbon", value = ocs)

## ----inspect-maize-oc-check---------------------------------------------------
inspect_apsimx("Maize-edited.apsimx", src.dir = ".", 
               node = "Soil", soil.child = "Organic")

## ----apsimx-wheat-no-run, eval = FALSE----------------------------------------
# sim <- apsimx("Wheat.apsimx", src.dir = extd.dir, value = "report")

## ----apsimx-wheat-no-run-two, eval = FALSE------------------------------------
# ex.dir <- auto_detect_apsimx_examples()
# ## Copy 'Wheat' file to a temporary directory
# ## (or change as needed)
# tmp.dir2 <- tempdir()
# file.copy(paste0(ex.dir, "/", "Wheat.apsimx"), tmp.dir2)
# sim <- apsimx("Wheat.apsimx", src.dir = tmp.dir2, value = "report")

## ----apsimx-wheat-read, echo = FALSE------------------------------------------
## sim <- read_apsimx("Wheat.db", src.dir = extd.dir)
sim <- read.csv(paste0(extd.dir, "/Wheat.csv"))
sim$Date <- as.Date(sim$Date)

## ----apsimx-wheat-summary-----------------------------------------------------
## Calcualte summary statistics on all variables
summary(sim)
## Plot data
ggplot(data = sim, aes(x = Date, y = Yield)) + geom_point()
## Inspect the Wheat .apsimx file
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop")
## This only displays the available 'Manager' components
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager")
## Looking more in-depth into 'SowingRule1'
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", NA))

## ----apsimx-wheat-cultivar-only-----------------------------------------------
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", 6))
## We can print and store the path to this parameter
pp <- inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", 6),
               print.path = TRUE)

## ----Millet-------------------------------------------------------------------
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager")

## ----Millet-sow---------------------------------------------------------------
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager",
              parm = list("Sow on a fixed date", NA))

## ----Millet-sow-plt-dens------------------------------------------------------
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager",
              parm = list("Sow on a fixed date",5), print.path = TRUE)
## Or store it in an object for later editing
pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Manager",
              parm = list("Sow on a fixed date", 5), print.path = TRUE)

## ----Millet-edit--------------------------------------------------------------
edit_apsim("Millet.apsim", src.dir = extd.dir, wrt.dir = tmp.dir,
           node = "Other", parm.path = pp, value = 8, 
           edit.tag = "-pp")

## ----Millet-edit-inspect------------------------------------------------------
inspect_apsim("Millet-pp.apsim", src.dir = tmp.dir,
              node = "Manager",
              parm = list("Sow on a fixed date", NA))

## ----removing-Millet-pp, echo = FALSE, eval = TRUE----------------------------
## Apparently this is not needed
file.remove(paste0(tmp.dir, "/Millet-pp.apsim"))

## ----inspect-replacement-node-------------------------------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", display.available = TRUE)

## ----inspect-replacement-node-child-------------------------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           display.available = TRUE)

## ----inspect-replacement-node-subchild----------------------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime",
                           display.available = TRUE)

## ----inspect-replacement-node-subchild-subsubchild----------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           display.available = TRUE) 

## ----inspect-replacement-node-subchild-subsubchild-temp-response--------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response") 

## ----inspect-replacement-node-subchild-subsubchild-temp-response-parm---------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response",
                           parm = "Y") 

## ----inspect-replacement-soybean-cultivar-node--------------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Soybean", display.available = TRUE) 

## ----inspect-replacement-soybean-cultivar-node-child--------------------------
inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                           src.dir = extd.dir,
                           node = "Soybean", 
                           node.child = "Cultivars",
                           node.subchild = "USA",
                           node.subsubchild = "PioneerP22T61_MG22",
                           display.available = FALSE) 

## ----edit-replacement---------------------------------------------------------
edit_apsimx_replacement("MaizeSoybean.apsimx", 
                        src.dir = extd.dir, wrt.dir = tmp.dir,
                        node = "Maize", node.child = "Phenology",
                        node.subchild = "ThermalTime", 
                        node.subsubchild = "BaseThermalTime",
                        node.sub3child = "Response",
                        parm = "Y", value = c(0, 12, 20, 28, 0))

## ----inspect-edit-replacement-------------------------------------------------
inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", 
                           src.dir = tmp.dir,
                           node = "Maize", 
                           node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response",
                           parm = "Y") 

## ----inspect-factorial-0, eval = FALSE----------------------------------------
# ## There are multiple 'Experiments' so we need to pick one
# inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
#                            root = list("Experiment", NA))
# ##These positions matched  Experiment   1 2 3 4 5
# ##Error in inspect_apsimx_replacement("Factorial", src.dir = ex.dir, root = ##list("Experiment", : Multiple root nodes found. Please provide a position

## ----inspect-factorial--------------------------------------------------------
## There are multiple 'Experiments' so we need to pick one
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1))
## We need to provide a node
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1), 
                           node = "Base", display.available = TRUE)
## We need to provide a node child
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1), 
                           node = "Base", node.child = "Clock",
                           display.available = TRUE)

## ----inspect-apsimx-factorial-1, eval = FALSE---------------------------------
# inspect_apsimx("Factorial.apsimx", src.dir = extd.dir)
# ## Simulation structure:
# ## list Name: Simulations
# # list length: 8
# # list names: $type ExplorerWidth Version Name Children IncludeInDocumentation Enabled ReadOnly
# # Children: Yes
# # Children length: 6
# # Children Names: Experiment RangeExperiment OperationsExpt Compound ManagerExpt DataStore
# # Error in inspect_apsimx("Factorial", src.dir = ex.dir) :
# #   more than one simulation found and no root node label has been specified
# #  select one of the children names above

## ----inspect-apsimx-factorial-2-----------------------------------------------
inspect_apsimx("Factorial.apsimx", src.dir = extd.dir,
               root = c("RangeExperiment", "Base2"),
               node = "Weather")

## ----edit-apsimx-factorial-2--------------------------------------------------
edit_apsimx("Factorial.apsimx", 
            src.dir = extd.dir, wrt.dir = tmp.dir,
            root = c("RangeExperiment", "Base2"),
            node = "Weather", 
            value = "Ames.met")

## ----apsim-verions-tail, eval = FALSE-----------------------------------------
# ava <- apsim_version()
# aiu <- apsim_version(which = "inuse")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)
apsimx_options(warn.versions = FALSE)
tmp.dir <- tempdir()

## ----sens0, eval = FALSE------------------------------------------------------
# ## Create a temporary directory and copy the Wheat.apsimx file
# tmp.dir <- tempdir()
# extd.dir <- system.file("extdata", package = "apsimx")
# file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
# ## Identify a parameter of interest
# ## In this case we want to know the impact of varying the fertilizer amount
# pp <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#                       node = "Manager", parm = list("SowingFertiliser", 1))
# ## For simplicity, we create a vector of fertilizer amounts (instead of sampling)
# ferts <- seq(5, 200, length.out = 7)
# col.res <- NULL
# for(i in seq_along(ferts)){
# 
#   edit_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#               node = "Other",
#               parm.path = pp, parm = "Amount",
#               value = ferts[i])
# 
#   sim <- apsimx("Wheat.apsimx", src.dir = tmp.dir)
#   col.res <- rbind(col.res, data.frame(fertilizer.amount = ferts[i],
#                                        wheat.aboveground.wt = mean(sim$Wheat.AboveGround.Wt, na.rm = TRUE)))
# }

## ----sens_apsimx, eval = FALSE------------------------------------------------
# tmp.dir <- tempdir()
# extd.dir <- system.file("extdata", package = "apsimx")
# file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
# ## Identify a parameter of interest
# ## In this case we want to know the impact of varying the fertilizer amount
# ## and the plant population
# pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#                       node = "Manager", parm = list("SowingFertiliser", 1))
# pp1 <- paste0(pp1, ".Amount")
# 
# pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#                       node = "Manager", parm = list("SowingRule1", 9))
# pp2 <- paste0(pp2, ".Population")
# 
# ## The names in the grid should (partially) match the parameter path names
# grd <- expand.grid(Fertiliser = c(50, 100, 150), Population = c(100, 200, 300))
# 
# ## This takes 2-3 minutes
# sns <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#                     parm.paths = c(pp1, pp2),
#                     grid = grd)

## ----sensitivity-parameterSets------------------------------------------------
library(sensitivity)
## Simple example: see documentation for other options
X.grid <- parameterSets(par.ranges = list(Fertiliser = c(1, 300), 
                                          Population = c(1, 300)),
                        samples = c(3,3), method = "grid")
X.grid

## ----sensitivity-morris-------------------------------------------------------
X.mrrs <- morris(factors = c("Fertiliser", "Population"),
                   r = 3, design = list(type = "oat", levels = 3, grid.jump = 1),
                   binf = c(0, 5), bsup = c(200, 300))
X.mrrs$X

## ----sensitivity-decoupling, eval = FALSE-------------------------------------
# ## This takes 2-3 minutes
# sns2 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
#                      parm.paths = c(pp1, pp2),
#                      grid = X.mrrs$X)
# ## These are the sensitivity results for AboveGround.Wt only
# sns.res.ag <- tell(X.mrrs, sns2$grid.sims$Wheat.AboveGround.Wt)
# sns.res.ag
# ## Call: morris(factors = c("Fertiliser", "Population"),
# ## r = 3, design = list(type = "oat", levels = 3, grid.jump = 1),
# ## binf = c(0, 5), bsup = c(200, 300))
# ##
# ## Model runs: 9
# ##                 mu  mu.star    sigma
# ## Fertiliser 916.6674 916.6674 662.8798
# ## Population 448.4530 448.4530 640.6807
# plot(sns.res.ag)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)
apsimx_options(warn.versions = FALSE)
tmp.dir <- tempdir()

## ----inspect-apsimx-xml-maize-------------------------------------------------
extd.dir <- system.file("extdata", package = "apsimx")
rue.pth <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir, parm = "rue")
ext.pth <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir, parm = "y_extinct_coef")
## To pass them to optim_apsim, combine them
pp <- c(rue.pth, ext.pth)

## ----obsWheat-----------------------------------------------------------------
data(obsWheat)
## See the structure
head(obsWheat)
## Only 10 observations
dim(obsWheat)
## Visualize the data
ggplot(obsWheat, aes(Date, Wheat.AboveGround.Wt)) + 
  geom_line() + 
  ggtitle("Biomass (g/m2)")
  
ggplot(obsWheat, aes(Date, Wheat.Leaf.LAI)) + 
  geom_line() +
  ggtitle("LAI")
  
ggplot(obsWheat, aes(Date, Wheat.Phenology.Stage)) + 
  geom_line() +
  ggtitle("Phenology Stages")

## ----wheat-sim-b4-opt, echo = FALSE-------------------------------------------
sim0 <- read.csv(file.path(extd.dir, "wheat-sim-b4-opt.csv"))
sim0$Date <- as.Date(sim0$Date)

## ----sim0-wheat-sim, eval = FALSE---------------------------------------------
# sim0 <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir, value = "report")

## ----sim0-viz-----------------------------------------------------------------
## Select 
sim0.s <- subset(sim0, 
                 Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))
## Visualize before optimization
## phenology
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Stage)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.Phenology.Stage)) + 
  ggtitle("Phenology")
## LAI
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
  ggtitle("LAI")
## Biomass
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")

## ----inspect------------------------------------------------------------------
## Finding RUE
inspect_apsimx_replacement("Wheat-opt-ex.apsimx", src.dir = extd.dir,
                           node = "Wheat", 
                           node.child = "Leaf",
                           node.subchild = "Photosynthesis",
                           node.subsubchild = "RUE", 
                           parm = "FixedValue",
                           verbose = FALSE)
## Finding BasePhyllochron
inspect_apsimx_replacement("Wheat-opt-ex.apsimx", src.dir = extd.dir,
                           node = "Wheat", 
                           node.child = "Cultivars",
                           node.subchild = "USA",
                           node.subsubchild = "Yecora", 
                           verbose = FALSE)
## Constructing the paths is straight-forward
pp1 <- "Wheat.Leaf.Photosynthesis.RUE.FixedValue"
pp2 <- "Wheat.Cultivars.USA.Yecora.BasePhyllochron"

## ----optim-apsimx, eval = FALSE-----------------------------------------------
# ## wop is for wheat optimization
# wop <- optim_apsimx("Wheat-opt-ex.apsimx",
#                     src.dir = extd.dir,
#                     parm.paths = c(pp1, pp2),
#                     data = obsWheat,
#                     weights = "mean",
#                     replacement = c(TRUE, TRUE),
#                     initial.values = c(1.2, 120))

## ----load-wop, echo = FALSE---------------------------------------------------
data("wheat-opt-ex", package = "apsimx")

## ----wop-result---------------------------------------------------------------
wop

## ----optim-apsimx-hessian, eval = FALSE---------------------------------------
# ## wop is for wheat optimization
# wop.h <- optim_apsimx("Wheat-opt-ex.apsimx",
#                       src.dir = extd.dir,
#                       parm.paths = c(pp1, pp2),
#                       data = obsWheat,
#                       weights = "mean",
#                       replacement = c(TRUE, TRUE),
#                       initial.values = c(1.2, 120),
#                       hessian = TRUE)

## ----wop-result-hessian-------------------------------------------------------
wop.h

## ----wop-result-opt, eval = FALSE---------------------------------------------
# ## We re-run the model because the Wheat-opt-ex.apsimx file
# ## is already edited
# sim.opt <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir, value = "report")
# sim.opt.s <- subset(sim.opt,
#                     Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))

## ----import-wop-result, echo = FALSE------------------------------------------
sim.opt.s <- read.csv(file.path(extd.dir, "wheat-sim-opt.csv"))
sim.opt.s$Date <- as.Date(sim.opt.s$Date)

## ----wop-result-opt-visualize-------------------------------------------------
  ## phenology
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Stage)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Phenology.Stage)) + 
    ggtitle("Phenology")
  ## LAI
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
    ggtitle("LAI")
  ## Biomass
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
##require(apsimx)

## ----apsimx-windows, eval = FALSE---------------------------------------------
# run.strng <- "C:/PROGRA~1/APSIM2024.2.7382.0/bin/Models.exe ./Wheat.apsimx"
# shell(cmd = run.strng, translate = TRUE, intern = TRUE)

## ----run-apsimx, eval = FALSE-------------------------------------------------
# ## Mac
# run.string.mac <- "mono /Applications/APSIM2021.01.14.6010.app/Contents/Resources/Bin/Models.exe ./Wheat.apsimx"
# system(command = run.string.mac, intern = TRUE)
# ## Debian
# run.string.debian <- "mono /usr/local/lib/apsim/2019.10.04.4236/Bin/Models.exe ./Wheat.apsimx"
# system(command = run.string.debian, intern = TRUE)

## ----sisd, eval = FALSE-------------------------------------------------------
# sessionInfo()
# Sys.info()[["sysname"]]
# Sys.info()[["release"]]
# Sys.info()[["version"]]
# if(.Platform$OS.type == "unix") system(command = "mono --version", intern = TRUE)
# R.version

#' 
#' @title Edit a replacement component in an .apsimx (JSON) file
#' @name edit_apsimx_replacement
#' @description edit the replacement componenet of an JSON apsimx file. It does not replace the GUI, but it can save time by quickly editing parameters and values.
#' @param file file ending in .apsimx to edit (JSON)
#' @param src.dir directory containing the .apsimx file; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param node specific node to edit
#' @param node.child specific node child component to edit.
#' @param node.subchild specific node sub-child to edit.
#' @param node.subsubchild specific node sub-subchild to edit.
#' @param node.sub3child specific node sub-sub-subchild to edit.
#' @param node.sub4child specific node sub-sub-sub-subchild to edit.
#' @param node.sub5child specific node sub-sub-sub-sub-subchild to edit.
#' @param node.string passing of a string instead of the node hierarchy. It can either start with a dot or not.
#' However, the \sQuote{best} form is not to start with a dot as it should be a more convenient form of passing
#' the nodes and their childs and not a real \sQuote{jsonpath}.
#' @param root \sQuote{root} node to explore (default = \dQuote{Models.Core.Replacements})
#' @param parm specific parameter to edit
#' @param value new values for the parameter
#' @param overwrite logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise
#' @param edit.tag if the file is edited a different tag from the default \sQuote{-edited} can be used.
#' @param verbose whether to print information about successful edit
#' @param grep.options Additional options for grep. To be passed as a list.
#' @details This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
#' @return (when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
#' As a side effect this function creates a new (JSON) .apsimx file.
#' @note The components that can be edited are restricted becuase this is better in preventing
#' errors of editing unintended parts of the file.
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## Writing to a temp directory, but change as needed
#' tmp.dir <- tempdir()
#' 
#' ## Inspect original values
#' inspect_apsimx_replacement("MaizeSoybean.apsimx", 
#'                            src.dir = extd.dir,
#'                            node = "Maize", 
#'                            node.child = "Phenology",
#'                            node.subchild = "ThermalTime", 
#'                            node.subsubchild = "BaseThermalTime",
#'                            node.sub3child = "Response")
#' 
#' edit_apsimx_replacement("MaizeSoybean.apsimx", 
#'                         src.dir = extd.dir, wrt.dir = tmp.dir,
#'                         node = "Maize", 
#'                         node.child = "Phenology",
#'                         node.subchild = "ThermalTime", 
#'                         node.subsubchild = "BaseThermalTime",
#'                         node.sub3child = "Response",
#'                         parm = "X",
#'                         value = c(10, 20, 30, 40, 50)) 
#' ## inspect it
#' inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", 
#'                            src.dir = tmp.dir,
#'                            node = "Maize", 
#'                            node.child = "Phenology",
#'                            node.subchild = "ThermalTime", 
#'                            node.subsubchild = "BaseThermalTime",
#'                            node.sub3child = "Response")
#' 
#' ## Illustrating using 'node.string'
#' ## Equivalent to the code to edit above
#' 
#' edit_apsimx_replacement("MaizeSoybean-edited.apsimx", 
#'                         src.dir = tmp.dir, wrt.dir = tmp.dir,
#'                         node.string = "Maize.Phenology.ThermalTime.BaseThermalTime.Response",
#'                         parm = "X",
#'                         value = c(11, 21, 31, 41, 51),
#'                         edit.tag = "-ns") 
#'                         
#' inspect_apsimx_replacement("MaizeSoybean-edited-ns.apsimx", 
#'                            src.dir = tmp.dir,
#'                            node = "Maize", 
#'                            node.child = "Phenology",
#'                            node.subchild = "ThermalTime", 
#'                            node.subsubchild = "BaseThermalTime",
#'                            node.sub3child = "Response")
#' }
#'

edit_apsimx_replacement <- function(file = "", src.dir = ".", wrt.dir = ".",
                                    node = NULL, node.child = NULL,
                                    node.subchild = NULL, node.subsubchild = NULL,
                                    node.sub3child = NULL, node.sub4child = NULL, 
                                    node.sub5child = NULL, node.string = NULL,
                                    root = list("Models.Core.Replacements", NA),
                                    parm = NULL, value = NULL, overwrite = FALSE,
                                    edit.tag = "-edited", verbose = TRUE, grep.options){
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }
  
  file.names <- dir(path = src.dir, pattern=".apsimx$",ignore.case=TRUE)
  
  if(length(file.names)==0){
    stop("There are no .apsimx files in the specified directory to edit.")
  }
  
  ## This matches the specified file from a list of files
  ## Notice that the .apsimx extension will be added here
  file <- match.arg(file, file.names, several.ok=FALSE)
  
  if(missing(parm) || missing(value)) stop("'parm' and/or 'value' are missing")
  
  apsimx_json <- jsonlite::read_json(paste0(src.dir, "/", file))
  
  ## Select Replacements node
  if(missing(grep.options)){
    frn <- grep(root[[1]], apsimx_json$Children, fixed = TRUE)  
  }else{
    gfixed <- ifelse(is.null(grep.options$fixed), FALSE, grep.options$fixed)
    gignore.case <- ifelse(is.null(grep.options$ignore.case), FALSE, grep.options$ignore.case)
    frn <- grep(root[[1]], apsimx_json$Children, fixed = gfixed, ignore.case = gignore.case)  
  }
  
  if(length(frn) == 0){
    ### Here we try a different root
    root <- list('Models.Core.Folder', NA)
    frn <- grep(root[[1]], apsimx_json$Children, fixed = TRUE)  
    if(length(frn) == 0)
      stop("Could not find 'root' for replacements", call. = FALSE)
  }
  
  if(length(frn) > 1){
    if(is.na(root[[2]])){
      cat("These positions matched ",root[[1]]," ",frn, "\n")
      stop("Multiple root nodes found. Please provide a position")
    }else{
      replacements.node <- apsimx_json$Children[[frn[root[[2]]]]]
    }
  }else{
    replacements.node <- apsimx_json$Children[[frn]]
  }
  
  if(!missing(grep.options)){
    gfixed <- grep.options$fixed
    gignore.case <- grep.options$ignore.case
  }else{
    gfixed <- FALSE
    gignore.case <- FALSE
  }
  
  ## Print names of replacements
  replacements.node.names <- vapply(replacements.node$Children, function(x) x$Name, 
                                    FUN.VALUE = "character")
  if(verbose) cat("Replacements: ", replacements.node.names, "\n")
  
  if(!missing(node.string)){
    ## strip the dot automatically if it is included
    if(strsplit(node.string, "")[[1]][1] == "."){
      node.string <- substring(node.string, 2)
    }
    ## If I want to provide the path returned by inspect_apsimx_replacement
    ## directly I need to strip the first part which is likely to be '.Simulations.Replacement.'
    if(grepl(".Replacements.", node.string)){
      node.string <- strsplit(node.string, ".Replacements.", fixed = TRUE)[[1]][2]
    }
    nodes <- strsplit(node.string, ".", fixed = TRUE)[[1]]
    node <- nodes[1]
    if(!is.na(nodes[2])) node.child <- nodes[2]
    if(!is.na(nodes[3])) node.subchild <- nodes[3]
    if(!is.na(nodes[4])) node.subsubchild <- nodes[4]
    if(!is.na(nodes[5])) node.sub3child <- nodes[5]
    if(!is.na(nodes[6])) node.sub4child <- nodes[6]
    if(!is.na(nodes[7])) node.sub5child <- nodes[7]
  }
  
  if(missing(node)) return(cat("Please provide a node \n"))
  ## Let's call this level = 0, at the 'node' level (nothing to edit)
  lvl <- -1
  wrn <- grep(node, replacements.node.names)
  rep.node <- replacements.node$Children[[wrn]]
  
  if(!is.null(rep.node$CropType) && verbose) cat("CropType", rep.node$CropType,"\n")
  
  if(parm %in% names(rep.node)){
    lvl <- 0
    rep.node <- edit_node(rep.node, parm = parm, value = value)
    replacements.node$Children[[wrn]] <- rep.node
    if(length(frn) == 1){
      apsimx_json$Children[[frn]] <- replacements.node
    }else{
      apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
    }
  } 
  
  rep.node.children.names <- sapply(rep.node$Children, function(x) x$Name)
  if(verbose) cat("Available node children: ", rep.node.children.names, "\n")
  
  ## Select a specific node.child
  if(missing(node.child) && lvl == -1) return(cat("missing node.child\n"))
  ## Node.child would be level = 1, still nothing to edit
  if(!missing(node.child)){
    wrnc <- grep(node.child, rep.node.children.names)
    rep.node.child <- rep.node$Children[[wrnc]]

    if(any(parm %in% names(rep.node.child))){
      lvl <- 1
      rep.node.child <- edit_node(rep.node.child, parm = parm, value = value)
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.child$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.child$Command)))){
        lvl <- 2
        wrnsc <- grep(parm, unlist(rep.node.child$Command), fixed = gfixed, ignore.case = gignore.case)
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.child$Command[[wrnsc]], "=")[[1]][1]
        rep.node.child$Command[[wrnsc]] <- paste0(cmdstrng, "=", value)
        ## Now write back to list
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(missing(node.subchild) && lvl == -1)
      stop("Failed to find and edit parameter")
    
    rep.node.subchildren.names <- sapply(rep.node.child$Children, function(x) x$Name)
  }
  
  ## Select a specific node.subchild
  if(missing(node.subchild) && verbose && missing(parm)) cat("missing node.subchild\n")
  
  if(!missing(node.subchild)){
    wrnsc <- grep(node.subchild, rep.node.subchildren.names)
    rep.node.subchild <- rep.node.child$Children[[wrnsc]]
  
    if(verbose) cat("Subchild Name: ", rep.node.subchild$Name,"\n")
  
    if(parm %in% names(rep.node.subchild)){
      rep.node.subchild <- edit_node(rep.node.subchild, parm = parm, value = value)
      lvl <- 3
      rep.node.child$Children[[wrnsc]] <- rep.node.subchild
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
      ## apsimx_json is ready to be written back to file
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.subchild$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.subchild$Command)))){
        lvl <- 3
        wrnscc <- grep(parm, unlist(rep.node.subchild$Command), fixed = gfixed, ignore.case = gignore.case)
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.subchild$Command[[wrnscc]], "=")[[1]][1]
        rep.node.subchild$Command[[wrnscc]] <- paste0(cmdstrng, "=", value)
        ## Now write back to list
        rep.node.child$Children[[wrnsc]] <- rep.node.subchild
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(missing(node.subsubchild) && lvl == -1)
      stop("Failed to find and edit parameter")
  }
  
  if(missing(node.subsubchild) && verbose && missing(parm)) cat("missing node.subsubchild\n")

  if(!missing(node.subsubchild)){
    rep.node.subsubchildren.names <- vapply(rep.node.subchild$Children, function(x) x$Name,
                                            FUN.VALUE = "character")
    ## Select a specific node.subsubchild
    wrnssc <- grep(node.subsubchild, rep.node.subsubchildren.names)
    if(length(wrnssc) == 0) stop("node.subsubchild not found")
    if(length(wrnssc) > 1) stop("More than one subsubchild found. Make it unique (see regular expressions)")
    rep.node.subsubchild <- rep.node.subchild$Children[[wrnssc]]
  
    if(verbose) cat("Subsubchild Name: ", rep.node.subsubchild$Name,"\n")
  
    if(parm %in% names(rep.node.subsubchild)){
      rep.node.subsubchild <- edit_node(rep.node.subsubchild, parm = parm, value = value)
      lvl <- 4
      rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
      rep.node.child$Children[[wrnsc]] <- rep.node.subchild
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
      ## apsimx_json is ready to be written back to file
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.subsubchild$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.subsubchild$Command)))){
        lvl <- 5
        wrnsscc <- grep(parm, unlist(rep.node.subsubchild$Command), fixed = gfixed, ignore.case = gignore.case)
        if(length(wrnsscc) == 0) stop("node.subsubchild Command not found")
        if(length(wrnsscc) > 1) stop("More than one subsubchild Command found. Make it unique (see regular expressions)")
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.subsubchild$Command[[wrnsscc]],"=")[[1]][1]
        rep.node.subsubchild$Command[[wrnsscc]] <- paste0(cmdstrng,"=",value)
        ## cat("New string:", rep.node.subsubchild$Command[[wrnsscc]], "\n")
        ## Now write back to list
        rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
        rep.node.child$Children[[wrnsc]] <- rep.node.subchild
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(missing(node.sub3child) && lvl == -1)
      stop("Failed to find and edit parameter")
  }

  ## Inserting deeper level June 15th 2020
  if(missing(node.sub3child) && verbose && missing(parm)) cat("missing node.sub3child\n")  
  
  if(!missing(node.sub3child)){
    rep.node.sub3children.names <- vapply(rep.node.subsubchild$Children, function(x) x$Name,
                                            FUN.VALUE = "character")
    ## Select a specific node.subchild
    wrnsssc <- grep(node.sub3child, rep.node.sub3children.names)
    if(length(wrnsssc) == 0) stop("node.sub4child not found")
    if(length(wrnsssc) > 1) stop("More than one sub3child found. Make it unique (see regular expressions)")
    
    rep.node.sub3child <- rep.node.subsubchild$Children[[wrnsssc]]
    
    if(verbose) cat("Sub-sub-subchild Name: ", rep.node.sub3child$Name,"\n")
    
    if(any(parm %in% names(rep.node.sub3child))){
      rep.node.sub3child <- edit_node(rep.node.sub3child, parm = parm, value = value)
      lvl <- 6
      # node is edited, now put it back in place
      rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
      rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
      rep.node.child$Children[[wrnsc]] <- rep.node.subchild
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
      ## apsimx_json is ready to be written back to file
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.sub3child$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.sub3child$Command)))){
        lvl <- 7
        wrnssscc <- grep(parm, unlist(rep.node.sub3child$Command), fixed = gfixed, ignore.case = gignore.case)
        if(length(wrnssscc) == 0) stop("node.sub3child Command not found")
        if(length(wrnssscc) > 1) stop("More than one sub3child Command found. Make it unique (see regular expressions)")
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.sub3child$Command[[wrnssscc]],"=")[[1]][1]
        rep.node.sub3child$Command[[wrnssscc]] <- paste0(cmdstrng,"=",value)
        ## Now write back to list
        rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
        rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
        rep.node.child$Children[[wrnsc]] <- rep.node.subchild
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(missing(node.sub4child) && lvl == -1)
      stop("Failed to find and edit parameter")
  }
  
  if(missing(node.sub4child) && verbose && missing(parm)) cat("missing node.sub4child\n")  

  #### Inserting node.sub4child option March 5th 2021
  if(!missing(node.sub4child)){
    rep.node.sub4children.names <- vapply(rep.node.sub3child$Children, function(x) x$Name,
                                          FUN.VALUE = "character")
    ## Select a specific node.subchild
    wrnssssc <- grep(node.sub4child, rep.node.sub4children.names, fixed = gfixed, ignore.case = gignore.case)
    if(length(wrnssssc) == 0) stop("node.sub4child not found")
    if(length(wrnssssc) > 1) stop("More than one sub4child found. Make it unique (see regular expressions)")
    
    rep.node.sub4child <- rep.node.sub3child$Children[[wrnssssc]]
    
    if(verbose) cat("Sub-sub-sub-subchild Name: ", rep.node.sub4child$Name,"\n")
    
    if(any(parm %in% names(rep.node.sub4child))){
      rep.node.sub4child <- edit_node(rep.node.sub4child, parm = parm, value = value)
      lvl <- 8
      # node is edited, now put it back in place
      rep.node.sub3child$Children[[wrnssssc]] <- rep.node.sub4child
      rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
      rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
      rep.node.child$Children[[wrnsc]] <- rep.node.subchild
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
      ## apsimx_json is ready to be written back to file
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.sub4child$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.sub4child$Command)))){
        lvl <- 9
        wrnsssscc <- grep(parm, unlist(rep.node.sub4child$Command), fixed = gfixed, ignore.case = gignore.case)
        if(length(wrnsssscc) == 0) stop("node.sub4child Command not found")
        if(length(wrnsssscc) > 1) stop("More than one sub4child Command found. Make it unique (see regular expressions)")
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.sub4child$Command[[wrnsssscc]], "=")[[1]][1]
        rep.node.sub4child$Command[[wrnsssscc]] <- paste0(cmdstrng, "=", value)
        ## Now write back to list
        rep.node.sub3child$Children[[wrnssssc]] <- rep.node.sub4child
        rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
        rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
        rep.node.child$Children[[wrnsc]] <- rep.node.subchild
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(missing(node.sub5child) && lvl == -1)
      stop("Failed to find and edit parameter")
  }  
  
  if(missing(node.sub5child) && verbose && missing(parm)) cat("missing node.sub5child\n")  
  
  #### Inserting node.sub5child option March 5th 2021
  if(!missing(node.sub5child)){
    rep.node.sub5children.names <- vapply(rep.node.sub4child$Children, function(x) x$Name,
                                          FUN.VALUE = "character")
    ## Select a specific node.subchild
    wrnsssssc <- grep(node.sub5child, rep.node.sub5children.names, fixed = gfixed, ignore.case = gignore.case)
    if(length(wrnsssssc) == 0) stop("node.subs5child not found")
    if(length(wrnsssssc) > 1) stop("More than one sub5child found. Make it unique (see regular expressions)")
    rep.node.sub5child <- rep.node.sub4child$Children[[wrnsssssc]]
    
    if(verbose) cat("Sub-sub-sub-sub-subchild Name: ", rep.node.sub5child$Name,"\n")
    
    if(any(parm %in% names(rep.node.sub5child))){
      rep.node.sub5child <- edit_node(rep.node.sub5child, parm = parm, value = value)
      lvl <- 10
      # node is edited, now put it back in place
      rep.node.sub4child$Children[[wrnsssssc]] <- rep.node.sub5child
      rep.node.sub3child$Children[[wrnssssc]] <- rep.node.sub4child
      rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
      rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
      rep.node.child$Children[[wrnsc]] <- rep.node.subchild
      rep.node$Children[[wrnc]] <- rep.node.child
      replacements.node$Children[[wrn]] <- rep.node
      if(length(frn) == 1){
        apsimx_json$Children[[frn]] <- replacements.node
      }else{
        apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
      }
      ## apsimx_json is ready to be written back to file
    }
    
    ## Cultivar parameters can be in 'Command'
    if(length(rep.node.sub5child$Command) > 0){
      if(any(grepl(parm, unlist(rep.node.sub5child$Command)))){
        lvl <- 11
        wrnssssscc <- grep(parm, unlist(rep.node.sub5child$Command), fixed = gfixed, ignore.case = gignore.case)
        if(length(wrnssssscc) == 0) stop("node.subs5child Command not found")
        if(length(wrnssssscc) > 1) stop("More than one sub5child Command found. Make it unique (see regular expressions)")
        ## Break it up and reassemble
        cmdstrng <- strsplit(rep.node.sub5child$Command[[wrnssssscc]], "=")[[1]][1]
        rep.node.sub5child$Command[[wrnssssscc]] <- paste0(cmdstrng, "=", value)
        ## Now write back to list
        rep.node.sub4child$Children[[wrnsssssc]] <- rep.node.sub5child
        rep.node.sub3child$Children[[wrnssssc]] <- rep.node.sub4child
        rep.node.subsubchild$Children[[wrnsssc]] <- rep.node.sub3child
        rep.node.subchild$Children[[wrnssc]] <- rep.node.subsubchild
        rep.node.child$Children[[wrnsc]] <- rep.node.subchild
        rep.node$Children[[wrnc]] <- rep.node.child
        replacements.node$Children[[wrn]] <- rep.node
        if(length(frn) == 1){
          apsimx_json$Children[[frn]] <- replacements.node
        }else{
          apsimx_json$Children[[frn[root[[2]]]]] <- replacements.node
        }
      }
    }
    
    if(lvl == -1)
      stop("Failed to find and edit parameter")
  }
  
  ## Write back to a file
  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir,"/",
                      strsplit(file,".",fixed = TRUE)[[1]][1],
                      edit.tag,".apsimx")
  }else{
    wr.path <- paste0(wrt.dir,"/",file)
  }
  
  jsonlite::write_json(apsimx_json, path = wr.path, 
                       pretty = TRUE, digits = NA, 
                       auto_unbox = TRUE, null = "null")
  
  if(verbose){
    if(!missing(node)) cat("Edited (node): ",node, "\n")
    if(!missing(node.child)) cat("Edited (node.child): ", node.child,"\n")
    if(!missing(node.subchild)) cat("Edited (node.subchild): ", node.subchild,"\n")
    if(!missing(node.subsubchild)) cat("Edited (node.subsubchild): ", node.subsubchild,"\n")
    if(!missing(node.sub3child)) cat("Edited (node.sub3child): ", node.sub3child,"\n")
    cat("Edit (level): ", lvl,"\n")
    cat("Edited parameter: ",parm, "\n")
    if(inherits(value, "factor")){
      cat("New values: ",as.character(value), "\n")  
    }else{
      cat("New values: ",value, "\n")  
    }
    cat("Created: ",wr.path,"\n")
  }
}

edit_node <- function(x, parm = NULL, value = NULL){
  
  ## parm is presumably a component in names of x
  
  if(length(names(x)) == 0) stop("no 'names' in node to edit")
  
  if(is.null(parm) | is.null(value)) stop("parm or value are missing")
  
  if(length(parm) == 1){
    x.nms <- names(x)
##    print(x.nms)
    ## wne <- which(x.nms == parm)
    wne <- grep(parm, x.nms)
##    if(length(x[[wne]]) != 1) stop("value should be of length = 1")
    x[[wne]] <- value
  }
  
  if(length(parm) > 1){
    x.nms <- names(x)
    wne <- grepl(parm, x.nms)
    j <- 1
    for(i in wne){
      x[[i]] <- value[j]
      j <- j + 1
    }
  }
  return(x)
}

## Example simulation for a point using APSIM
## Author: Fernando E. Miguez
## Date: 2021-06-08
##
## Required: R package apsimx 1.992
require(apsimx)
require(ggplot2)

apsimx_options(warn.versions = FALSE)

run.apsimx.point.sim <- get(".run.local.tests", envir = apsimx.options)

tmpd <- tempdir()
## tmpd <- "../tests"
## setwd(tmpd)

if(run.apsimx.point.sim && FALSE){

  site.name <- "ARK1"
  site.coords <- c(-90.75973, 34.72834)
  
  ## Copy Maize from Examples 
  exdir <- auto_detect_apsimx_examples()
  file.copy(file.path(exdir, "Maize.apsimx"), tmpd)
  
  ## Get weather from NASAPOWER
  ## Important: this is just an example. I recommend looking into
  ## whether this is a good source of weather data for this location.
  ## This takes a few seconds
  pmet <- get_power_apsim_met(site.coords, 
                              dates = c("2000-01-01", "2020-12-31"))
  check_apsim_met(pmet) ## Some missing values for solar radiation
  pmet <- impute_apsim_met(pmet) ## Automatically impute 
  check_apsim_met(pmet) ## Now it is okay
  write_apsim_met(pmet, wrt.dir = ".", filename = paste0(site.name, ".met")) ## Write to file
  
  ## Edit the filename for weather
  edit_apsimx("Maize.apsimx", node = "Weather",
              value = paste0(site.name, ".met"),
              overwrite = TRUE)
  
  inspect_apsimx("Maize.apsimx", node = "Weather")
  
  ## Also change the clock
  edit_apsimx("Maize.apsimx", node = "Clock",
              parm = c("Start", "End"),
              value = c("2000-01-01T00:00:00", "2020-12-31T00:00:00"),
              overwrite = TRUE)
  
  inspect_apsimx("Maize.apsimx")
  
  ## Change planting and harvest date
  edit_apsimx("Maize.apsimx", node = "Crop", 
              parm = "StartDate", value = "1-may", 
              overwrite = TRUE)
  
  edit_apsimx("Maize.apsimx", node = "Crop", 
              parm = "EndDate", value = "1-oct", 
              overwrite = TRUE)

  inspect_apsimx("Maize.apsimx", node = "Crop")
  
  sim0 <- apsimx("Maize.apsimx")
  
  ## Results with default soil
  ggplot(sim0, aes(x = Date, y = Maize.AboveGround.Wt)) + geom_point()
  
  ## Now let's get the soil data from SSURGO
  sp0 <- get_ssurgo_soil_profile(site.coords)
  
  ## Very important: these are just 'template' soil profiles from 
  ## where to start. These need to be checked and if you have any site 
  ## specific information you should use it
  plot(sp0[[1]]) ## Visualize properties
  plot(sp0[[1]], property = "water")
  ## You might want to use the APSIM-X GUI in addition
  
  ## Also, here you can modify the metadata, since it is a list
  sp0[[1]]$metadata$NearestTown <- "Marianna"
  ## You might want to populate the soilwat or swim options
  ## using the helper functions soilwat_parms or swim_parms
  
  edit_apsimx_replace_soil_profile("Maize.apsimx",
                                   soil.profile = sp0[[1]],
                                   overwrite = TRUE)  

  sim1 <- apsimx("Maize.apsimx")  
  
  ## Results with ssurgo soil
  ggplot(sim1, aes(x = Date, y = Maize.AboveGround.Wt)) + geom_point()
  
  cmp <- compare_apsim(sim0, sim1, variable = "Maize.AboveGround.Wt")
  plot(cmp, variable = "Maize.AboveGround.Wt") ## similar but not identical
  
  ## There is no site information
  inspect_apsimx("Maize.apsimx", node = "Soil", parm = "Site")
  ## However, the nearest town was edited as expected
  inspect_apsimx("Maize.apsimx", node = "Soil", parm = "NearestTown")
  
  ## If we still want to edit the Site
  edit_apsimx("Maize.apsimx", node = "Soil", 
              parm = "Site", value = "AK1", 
              overwrite = TRUE)
  
  inspect_apsimx("Maize.apsimx", node = "Soil")
}

## Testing whether this function works
require(apsimx)
require(sf)

## Even though I can run this externally, I will set it so it only
## runs locally
run.soil.profile <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.soil.profile){
## First create a soil profile
sp <- apsimx_soil_profile(nlayers = 15, 
                          crops = c("Barley", "Chickpea","Lucerne","Maize",
                                    "Perennial Grass","Sorghum","Wheat","Millet"))

## Find a file from 'Classic' such as 'Millet'
extd.dir <- system.file("extdata", package = "apsimx")

inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
              soil.child = "Water")

inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
              soil.child = "OrganicMatter")

inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
              soil.child = "Analysis")

inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
              soil.child = "Sample", parm = "Thickness")

## Edit it, replacing with information from the soil profile
edit_apsim_replace_soil_profile("Millet.apsim", soil.profile = sp, 
                                edit.tag = "-newsoil",
                                src.dir = extd.dir, wrt.dir = tmp.dir)

inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir,
                  node = "Soil")

inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir, 
              node = "Soil", soil.child = "Water")

inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir, 
              node = "Soil", soil.child = "OrganicMatter")

inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir, 
              node = "Soil", soil.child = "Analysis")

inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir, 
              node = "Soil", soil.child = "Sample")

## Should add another example using SSURGO data

## Read the main files
chorizon <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_chorizon.csv"))
component <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_component.csv"))
mapunit <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_mapunit.csv"))
mapunit.shp <- st_read(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_Mapunits.shp"), quiet = TRUE)

sp <- ssurgo2sp(mapunit = mapunit, 
                component = component, 
                chorizon = chorizon,
                mapunit.shp = mapunit.shp,
                nlayers = 20, nmapunit = 2)

## For first soil
metadata <- attributes(sp[[1]])
metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL;

asp <- apsimx_soil_profile(nlayers = 20,
                           Thickness = sp[[1]]$Thickness * 10,
                           BD = sp[[1]]$BD,
                           AirDry = sp[[1]]$AirDry,
                           LL15 = sp[[1]]$LL15,
                           DUL = sp[[1]]$DUL,
                           SAT = sp[[1]]$SAT,
                           KS = sp[[1]]$KS,
                           Carbon = sp[[1]]$Carbon,
                           crop.LL = sp[[1]]$LL15,
                           soil.bottom = 200,
                           metadata = metadata,
                           crops = c("Barley", "Chickpea","Lucerne","Maize",
                                     "Perennial Grass","Sorghum","Wheat","Millet"))

## Edit it, replacing with information from the soil profile
edit_apsim_replace_soil_profile("Millet.apsim", soil.profile = asp, 
                                edit.tag = "-newsoil-ssurgo",
                                src.dir = extd.dir, wrt.dir = tmp.dir)

}

## Testing the feature for APSIM Next Gen

run.soil.profile <- get(".run.local.tests", envir = apsimx.options)

if(run.soil.profile){
 
  extd.dir <- system.file("extdata", package = "apsimx")
  
  sp <- apsimx_soil_profile()
  
  swat <- soilwat_parms(Thickness = rep(50, 10), SWCON = runif(10, 0.2, 0.4))
  
  sp$soilwat <- swat
  
  list.files(tmp.dir)
  
  file.copy(file.path(extd.dir, "MaizeSoybean.apsimx"), tmp.dir)
 
  edit_apsimx_replace_soil_profile("MaizeSoybean.apsimx", soil.profile = sp,
                                   src.dir = tmp.dir, wrt.dir = tmp.dir,
                                   root = "SimulationSoybean") 

  inspect_apsimx("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil", soil.child = "Physical",
                 root = "SimulationSoybean")
    
  inspect_apsimx("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil", soil.child = "Physical", parm = "DUL",
                 root = "SimulationSoybean")
  
  inspect_apsimx("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil", soil.child = "SoilWater",
                 root = "SimulationSoybean")
  
  inspect_apsimx("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil", soil.child = "Physical", parm = "DUL",
                 root = "SimulationMaize")
  
}

#'
#' This function requires the \CRANpkg{GSODR} package.
#'
#' @title Get GSOD data for an APSIM met file
#' @description Uses \code{\link[GSODR]{get_GSOD}} from the \CRANpkg{GSODR} package to download data to create an APSIM met file.
#' @name get_gsod_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param dates date ranges
#' @param wrt.dir write directory
#' @param filename file name for writing out to disk
#' @param distance distance in kilometers for the nearest station
#' @param station choose the station either by index or character
#' @param fillin.radn whether to fill in radiation data using the nasapower pacakge. Default is FALSE.
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk.
#' @note This source of data does not provide solar radiation. If \sQuote{fillin.radn} is
#' TRUE it fill in radiation data using the nasapower package.
#' @return returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @export
#' @examples 
#' \dontrun{
#' require(GSODR)
#' ## This will not write a file to disk
#' gsd <- get_gsod_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"),
#'                           fillin.radn = TRUE)
#' summary(gsd)
#' ## Check for reasonable ranges
#' check_apsim_met(gsd)
#' }
#' 

get_gsod_apsim_met <- function(lonlat, dates, wrt.dir = ".", filename = NULL, 
                               distance = 100, station, fillin.radn = FALSE){
  
  if(!requireNamespace("GSODR", quietly = TRUE)){
    warning("The GSODR package is required for this function")
    return(NULL)
  }
  
  if(missing(filename)) filename <- "noname.met"
  
  if(!grepl(".met", filename, fixed = TRUE)) stop("filename should end in .met")
  
  yr1 <- as.numeric(format(as.Date(dates[1]), "%Y"))
  yr2 <- as.numeric(format(as.Date(dates[2]), "%Y"))
  
  ## Get the station
  nr.st <- GSODR::nearest_stations(LAT = lonlat[2], LON = lonlat[1], distance = distance)
  
  if(length(nr.st) == 0) stop("No stations found. Try increasing the distance.")
  
  if(!missing(station)){
    if(is.numeric(station)){
      nr.st1 <- nr.st[station]
    }else{
      wst <- which(nr.st %in% station)
      if(length(wst) == 0){
        stop("'station' not found within the range of near stations")
      }else{
        nr.st1 <- nr.st[wst]
      }
    }
  }else{
    nr.st1 <- nr.st[1] 
  }
  
  dts <- as.numeric(format(as.Date(dates), "%Y"))
  yrs <- seq(from = dts[1], to = dts[2])
  
  gsd <- GSODR::get_GSOD(years = yrs, station = nr.st1$STNID)

  if(nrow(gsd) == 0)
    stop("No data returned by the GDODR::get_GSOD function", call. = FALSE)
  
  stnid <- gsd$STNID[1]
  lati <- gsd$LATITUDE[1]
  longi <- gsd$LONGITUDE[1]
  
  if(fillin.radn){
    if(!requireNamespace("nasapower", quietly = TRUE)){
      warning("The nasapower package is required for this function")
      return(NULL)
    }
    pwr <- get_power_apsim_met(lonlat = lonlat, 
                               dates = c(gsd$YEARMODA[1], gsd$YEARMODA[nrow(gsd)]))
    pwr <- add_column_apsim_met(pwr, 
                                value = as.Date(c(1:nrow(pwr)-1), origin = paste0(yr1,"-01-01")),
                                name = "date", units = "()")
    pwr <- subset(pwr, select = c("date", "radn"))
    names(pwr) <- c("date", "RADN")
    gsd$date <- gsd$YEARMODA
    gsd <- merge(gsd, pwr, by = "date")
  }else{
    gsd$RADN <- NA  
  }

  gsd <- subset(as.data.frame(gsd), select = c("YEAR", "YDAY", "RADN",
                                               "MAX", "MIN", "PRCP", "RH", "WDSP"))
  
  
  names(gsd) <- c("year", "day", "radn", "maxt", "mint", "rain", "rh", "windspeed")
  units <- c("()", "()", "(MJ/m2/day)", "(oC)", "(oC)", "(mm)", "(%)", "(m/s)")
  
  if(fillin.radn){
    comments <- paste("!data from GSODR R package. Radiation from nasapower R. retrieved: ", Sys.time())
  }else{
    comments <- paste("!data from GSODR R package. retrieved: ", Sys.time())  
  }
  
  attr(gsd, "filename") <- filename
  attr(gsd, "site") <- paste("site =", "station-ID", stnid)
  attr(gsd, "latitude") <- paste("latitude =", lati)
  attr(gsd, "longitude") <- paste("longitude =", longi)
  attr(gsd, "tav") <- paste("tav =", mean(colMeans(gsd[,c("maxt","mint")], na.rm=TRUE), na.rm=TRUE))
  attr(gsd, "colnames") <- names(gsd)
  attr(gsd, "units") <- units
  attr(gsd, "comments") <- comments
  ## No constants
  class(gsd) <- c("met", "data.frame")
  
  gsd <- amp_apsim_met(gsd)
  
  if(filename != "noname.met"){
    write_apsim_met(gsd, wrt.dir = wrt.dir, filename = filename)
  }
  return(invisible(gsd))
}

#' Edit an APSIM-X (JSON) Simulation in Batch mode
#' 
#' This function allows editing of an APSIM-X (JSON) simulation file in batch mode.
#' 
#' from hol430
#' 
#' This allows the user to specify an .apsimx file and a config file when running Models.exe. The .apsimx file will not be run but instead, the changes listed in the config file will be applied to the .apsimx file, which will then be written to disk under the same filename.
#' 
#' The config file should contain lines of the form \sQuote{path = value}
#'
#' e.g.
#' 
#' [Clock].StartDate = 2019-1-20
#' .Simulations.Sim1.Name = SimulationVariant35
#' .Simulations.Sim2.Enabled = false
#' .Simulations.Sim1.Paddock.Soil.Thickness[1] = 50
#' Notes:
#'   
#'   Command line arguments should look like:
#'  Models.exe file.apsimx /Edit /path/to/config/file.conf
#' 
#' Relative paths will be resolved to the first match. ie
#' [Clock].StartDate will match the first clock found in the file.
#'
#' Dates can be specified as yyyy-mm-dd or mm/dd/yyyy.
#'
#' Strings should not be quoted
#' 
#' Array indices will be interpted as 1-indexed (mad face). So the first
#' element in the array should have index 1 in the config file.
#'
#' The file will be upgraded to the latest file version as part of this
#' process.
#' 
#' @name edit_apsimx_batch
#' @param file file ending in .apsimx to be edited (JSON)
#' @param src.dir directory containing the .apsimx file to be edited; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param parms parameter to be edited in the for of \sQuote{key = value}
#' @param silent controls the output of running APSIM at the command line
#' @param verbose whether to print information about successful edit
#' @return (when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
#' As a side effect this function creates a new (JSON) .apsimx file.
#' @export
#' @examples 
#' \donttest{
#' ## This example will read one of the examples distributed with APSIM-X
#' ## but write to a temporary directory
#' 
#' tmp.dir <- tempdir()
#' 
#' ## Edit InitialResidueMass
#' extd.dir <- system.file("extdata", package = "apsimx")
#' parms <- list(`.Simulations.Simulation.Field.SurfaceOrganicMatter.InitialResidueMass` = 600)
#' edit_apsimx_batch("Wheat.apsimx", src.dir = extd.dir, wrt.dir = tmp.dir, parms = parms)
#' }
#' 

edit_apsimx_batch <- function(file, 
                              src.dir = ".", 
                              wrt.dir = NULL,
                              parms = NULL, 
                              silent = FALSE,
                              verbose = TRUE){
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to edit.")
  }
  
  if(missing(parms)) stop("parms is missing")
  
  if(length(grep(".apsimx$", file)) != 0){
    ## I assume the extention was included
    ## Only use the name from here 
    ## file <- strsplit(file, ".", fixed = TRUE)[[1]][1]
    file <- tools::file_path_sans_ext(file)
  }
  
  file.name.path <- paste0(src.dir, "/", file, ".apsimx")
  
  if(!is.null(wrt.dir)){
    ## Need to copy the file to wrt.dir and then clean up
    file.copy(from = file.name.path, 
              to = wrt.dir)
  }
  
  fns <- strsplit(file, ".", fixed = TRUE)[[1]][1]
  fn.po <- paste0(fns, "-prms.conf")
  write("", file = paste0(wrt.dir, "/", fn.po))
  
  prm.out <- character(length(parms))
  
  for(i in seq_along(parms)){
    prmo <- paste(names(parms[i]), "=", parms[[i]])
    write(prmo, file = paste0(wrt.dir, "/", fn.po), append = TRUE)
  }
  
  ## Auto detect apsimx
  ada <- auto_detect_apsimx()
  
  if(.Platform$OS.type == "unix"){
    mono <- system("which mono", intern = TRUE)
    run.strng <- paste0(mono, " ", ada, " ", paste0(wrt.dir, "/", file, ".apsimx"), " /Edit ", paste0(wrt.dir, "/", fn.po))
    ## Run APSIM-X on the command line
    system(command = run.strng, ignore.stdout = silent)
  }
  
  if(.Platform$OS.type == "windows"){
    ada <- auto_detect_apsimx()
    run.strng <- paste0(ada, " ", paste0(wrt.dir, "/", file, ".apsimx")," /Edit ", paste0(wrt.dir, "/", fn.po))
    cmd.out <- shell(cmd = run.strng, translate = FALSE, intern = TRUE)
  }
  
  if(verbose){
    cat("Edited parameters file: ", paste0(wrt.dir, "/", fn.po), "\n")
    cat("Edited parameters: ", names(parms), "\n")
    cat("Created new: ", paste0(wrt.dir, "/", file, ".apsimx"), "\n")
    if(.Platform$OS.type == "windows") print(cmd.out)
  }
}

#' Source: https://esoil.io/TERNLandscapes/Public/Pages/SLGA/index.html
#' 
#' @title Generate a synthetic APSIM soil profile from the SLGA soil database
#' @description Retrieves soil data from the SLGA database (Australia) and converts it to an APSIM soil_profile object
#' @name get_slga_soil_profile
#' @param lonlat Longitude and latitude vector (e.g. c(151.8306, -27.4969)).
#' @param statistic default is the mean
#' @param soil.profile a soil profile to fill in in case the default one is not appropriate
#' @param find.location.name default is TRUE. Use either maps package or photon API to find Country/State.
#' If you are running this function many times it might be better to set this to FALSE.
#' @param fix whether to fix compatibility between saturation and bulk density (default is FALSE).
#' @param verbose argument passed to the fix function.
#' @param check whether to check the soil profile (default is TRUE)
#' @param physical whether soil physical properties are obtained from the data base or through \sQuote{SR}, Saxton and Rawls pedotransfer functions.
#' @param xargs additional arguments passed to \code{\link{apsimx_soil_profile}} or \sQuote{apsimx:::approx_soil_variable} function. At the moment these are:
#' \sQuote{soil.bottom}, \sQuote{method} and \sQuote{nlayers}.
#' @return it generates an object of class \sQuote{soil_profile}.
#' @seealso \code{\link{apsimx_soil_profile}}, \code{\link{edit_apsim_replace_soil_profile}}, \code{\link{edit_apsimx_replace_soil_profile}}.
#' @export
#' @author Fernando E. Miguez, Chloe (Yunru Lai), Eric Zurcher (CSIRO) and Andrew Moore (CSIRO)
#' @examples 
#' \dontrun{
#' ## Get soil profile properties for a single point
#'   sp1 <- get_slga_soil_profile(lonlat = c(151.8306, -27.4969), fix = TRUE, verbose = FALSE)
#'   ## Visualize
#'   plot(sp1)
#'   plot(sp1, property = "water")
#' }


# this is based on the get_isric_soil_profile function of Fernando Miguez, Eric Zurcher and Andrew Moore
get_slga_soil_profile <- function(lonlat, 
                                   statistic = c("mean", "Q0.5"),
                                   soil.profile,
                                   find.location.name = TRUE,
                                   fix = FALSE,
                                   verbose = TRUE,
                                   check = TRUE,
                                   physical = c("default", "SR"),
                                   xargs = NULL){

  statistic <- match.arg(statistic)
  physical <- match.arg(physical)
  
  #### Create extent step ####
  lon <- as.numeric(lonlat[1])
  lat <- as.numeric(lonlat[2])
  
  ## Determine if the location is in Australia
  if(requireNamespace("maps", quietly = TRUE)){
    country <- maps::map.where(x = lon, y = lat)
    if(country != "Australia" || is.na(country))
      stop("These coordinates do not correspond to a location in Australia. \n Did you specify the coordinates correctly?")
  }
  
  ## This code is now unnecessay, I think 
  if (lon < 112 || lon > 154) stop("Longitude should be between 112 and 154 for the extent of Australia")
  if (lat < -44 || lat > -10) stop("Latitude should be between -44 and -10 for the extent of Australia")
  
  slga <- get_slga_soil(lonlat = lonlat)
  
  ## These are the default thicknesses in ISRIC
  ## They also match thickness values in SLGA
  ## thcknss <- c(50, 100, 150, 300, 400, 1000) ## in mm
  thcknss <- slga$thickness * 10 ## converts cm to mm
  o.thcknss <- thcknss
  
  ## Some variables can be passed to apsimx:::approx_soil_variable
  soil.bottom <- 200
  method <- "constant"
  nlayers <- 6
  crps <- c("Maize", "Soybean", "Wheat")
  if(!is.null(xargs)){
    ### Soil bottom
    if(!is.null(xargs$soil.bottom) && is.null(xargs$nlayers)){
      soil.bottom <- xargs$soil.bottom
      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
    }
    ### Method
    if(!is.null(xargs$method)){
      method <- xargs$method
    }
    ### Number of layers
    if(!is.null(xargs$nlayers) && is.null(xargs$soil.bottom)){
      nlayers <- xargs$nlayers
      ### Need to redefine 'Thickness'
      thcknss <- apsimx_soil_profile(nlayers = nlayers)$soil$Thickness
    }
    ### Do I need this?
    if(!is.null(xargs$nlayers) && !is.null(xargs$soil.bottom)){
      nlayers <- xargs$nlayers
      soil.bottom <- xargs$soil.bottom
      ### Need to redefine 'Thickness'
      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
    }
    ### Crops
    if(!is.null(xargs$crops)){
      crps <- xargs$crops
    }
  }

  ## Create the empty soil profile
  if(missing(soil.profile)){
    if(is.null(xargs)){
      new.soil <- FALSE
    }else{
      new.soil <- TRUE      
    }
    soil_profile <- apsimx_soil_profile(nlayers = nlayers, Thickness = thcknss, soil.bottom = soil.bottom, crops = crps) 
    soil_profile$soil$ParticleSizeClay <- NA
    soil_profile$soil$ParticleSizeSilt <- NA
    soil_profile$soil$ParticleSizeSand <- NA
    soil_profile$soil$CEC <- NA  
    soil_profile$soil$Nitrogen <- NA
    soil_profile$soil$DUL <- NA
    soil_profile$soil$LL15 <- NA
    soil_profile$soil$SAT <- NA
    
  }else{
    soil_profile <- soil.profile
    new.soil <- TRUE
  }
  
  # Calculate SAT (saturation) for each depth layer based on BD and particle density (2.65 g/cm³)
  particle_density <- 2.65  # Particle density (g/cm³)
  
  if (new.soil) {
    sp.xout <- cumsum(soil_profile$soil$Thickness)
    
    ### Will this take care of both number of layers and soil.bottom?
    if(length(cumsum(thcknss)) != length(slga[["bdod"]])){
      bdod.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["bdod"]]), xout = sp.xout)
      soc.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["soc"]]), xout = sp.xout)
      phh2o.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["phh2o"]]), xout = sp.xout)
      clay.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["clay"]]), xout = sp.xout)
      sand.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["sand"]]), xout = sp.xout)
      nitrogen.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["nitrogen"]]), xout = sp.xout)
      cec.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["cec"]]), xout = sp.xout)
      ## wv0010.dat <- approx(data.frame(cumsum(o.thcknss), wv0010[[1]]), xout = sp.xout)
      wv0033.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["wv0033"]]), xout = sp.xout)
      wv1500.dat <- stats::approx(data.frame(cumsum(o.thcknss), slga[["wv1500"]]), xout = sp.xout)
      sat.tmp <- (1- (slga[["bdod"]]/particle_density))*0.99
      sat.dat <- stats::approx(data.frame(cumsum(o.thcknss), sat.tmp), xout = sp.xout)
    }else{
      bdod.dat <- data.frame(x = cumsum(thcknss), y = slga[["bdod"]])
      soc.dat <- data.frame(x = cumsum(thcknss), y = slga[["soc"]])
      phh2o.dat <- data.frame(x = cumsum(thcknss), y = slga[["phh2o"]])
      clay.dat <- data.frame(x = cumsum(thcknss), y = slga[["clay"]])
      sand.dat <- data.frame(x = cumsum(thcknss), y = slga[["sand"]])
      nitrogen.dat <- data.frame(x = cumsum(thcknss), y = slga[["nitrogen"]])
      cec.dat <- data.frame(x = cumsum(thcknss), y = slga[["cec"]])
      wv0033.dat <- data.frame(x = cumsum(thcknss), y = slga[["wv0033"]])
      wv1500.dat <- data.frame(x = cumsum(thcknss), y = slga[["wv1500"]])
      sat.dat <- data.frame(x = cumsum(thcknss), y = (1- (slga[["bdod"]]/particle_density))*0.99)
    }
    
    soil_profile$soil$BD <- approx_soil_variable(bdod.dat, 
                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$Carbon <- approx_soil_variable(soc.dat, 
                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$PH <- approx_soil_variable(phh2o.dat, 
                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$ParticleSizeClay <- approx_soil_variable(clay.dat, 
                                                               xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$ParticleSizeSand <- approx_soil_variable(sand.dat, 
                                                               xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$Nitrogen <- approx_soil_variable(nitrogen.dat, 
                                                       xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$CEC <- approx_soil_variable(cec.dat, 
                                                  xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$SAT <- approx_soil_variable(sat.dat, 
                                                  xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3
    soil_profile$soil$DUL <- approx_soil_variable(wv0033.dat, 
                                                  xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3  
    soil_profile$soil$LL15 <- approx_soil_variable(wv1500.dat, 
                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3
    
  } else {
    soil_profile$soil$BD <- slga[["bdod"]]  # Bulk density
    soil_profile$soil$Carbon <- slga[["soc"]]  # Soil organic carbon
    soil_profile$soil$PH <- slga[["phh2o"]]  # pH
    soil_profile$soil$ParticleSizeClay <- slga[["clay"]]  # Clay percentage
    soil_profile$soil$ParticleSizeSand <- slga[["sand"]]  # Sand percentage
    soil_profile$soil$ParticleSizeSilt <- slga[["silt"]]  # Silt percentage
    soil_profile$soil$Nitrogen <- slga[["nitrogen"]]  # Nitrogen content
    soil_profile$soil$CEC <- slga[["cec"]]  # Cation exchange capacity
    soil_profile$soil$DUL <- slga[["wv0033"]] * 1e-2  # Drained upper limit
    soil_profile$soil$LL15 <- slga[["wv1500"]] * 1e-2  # Lower limit (wilting point)
    soil_profile$soil$SAT <- (1 - (soil_profile$soil$BD / particle_density))*0.99 # This is Chloe's method
  }
  
  soil_profile$soil$ParticleSizeSilt <- 100 - (soil_profile$soil$ParticleSizeSand + soil_profile$soil$ParticleSizeClay)
  
  # Ensure SAT is greater than DUL; if not, set SAT to 1.1 times DUL
  soil_profile$soil$SAT <- ifelse(soil_profile$soil$SAT > soil_profile$soil$DUL, 
                                  soil_profile$soil$SAT, 
                                  1.1 * soil_profile$soil$DUL)
  
## Populating DUL and LL. These are equations from Table 1 in Saxton and Rawls 2006
  if(physical == "SR"){
    soil_profile$soil$DUL <- sr_dul(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    soil_profile$soil$LL15 <- sr_ll(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    DUL_S <- sr_dul_s(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    soil_profile$soil$SAT <- sr_sat(soil_profile$soil$ParticleSizeSand, soil_profile$soil$DUL, DUL_S)    
  }

  ## This calculation of KS is a new addition (2025-01-22)
  soil_profile$soil$KS <- sr_ks(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
  
  soil_profile$soil$AirDry <- soil_profile$soil$LL15
  soil_profile$soil$AirDry[1] <- soil_profile$soil$LL15[1] * 0.5 ## AirDry is half of LL for the first layer
  
  for(i in soil_profile$crops){
    soil_profile$soil[[paste0(i,".LL")]] <- soil_profile$soil$LL15 ## Without better information  
  }
  
  #### Passing soil properties from soilwat
  ## The soil texture class in the metadata will be based on the first layer only
  # note that this is the USA texture triangle - will need to update to the Australian one later
  txt_clss <- texture_class_slga(soil_profile$soil$ParticleSizeClay * 1e-2, soil_profile$soil$ParticleSizeSilt * 1e-2)
  # Get soil parameters
  t2sp <- texture2soilParms_slga(txt_clss)
  
  if(missing(soil.profile)){
    soil_profile$soilwat <- soilwat_parms(Salb = t2sp$Albedo, CN2Bare = t2sp$CN2, 
                                          SWCON = t2sp$SWCON,
                                          Thickness = thcknss)    
  }else{
    soil_profile$soilwat <- soilwat_parms(Salb = t2sp$Albedo, CN2Bare = t2sp$CN2, 
                                          SWCON = t2sp$SWCON,
                                          Thickness = soil_profile$soil$Thickness)    
  }
  
  ### Passing the initial soil water?
  if(new.soil){
    ini.wat <- (soil_profile$soil$DUL + soil_profile$soil$LL15) / 2
    isw <- initialwater_parms(Thickness = soil_profile$soil$Thickness, 
                              InitialValues = ini.wat)
  }else{
    ini.wat <- (soil_profile$soil$DUL + soil_profile$soil$LL15) / 2
    isw <- initialwater_parms(Thickness = thcknss, InitialValues = ini.wat)    
  }

  soil_profile$initialwater <- isw

  ## Will this always be Australia?
  ## photon might be nice for location
  if(find.location.name){
    if(requireNamespace("maps", quietly = TRUE)){
      country <- maps::map.where(x = lon, y = lat)
      if(country == "USA"){
        state <- toupper(maps::map.where(database = "county", x = lon, y = lat)) 
      }else{
        url <- paste0("https://photon.komoot.io/reverse?lon=", lon, "&lat=", lat)
        fgeo <- jsonlite::fromJSON(url)
        state <- fgeo$feature$properties$state
      }
    }else{
      url <- paste0("https://photon.komoot.io/reverse?lon=", lon, "&lat=", lat)
      fgeo <- jsonlite::fromJSON(url)
      state <- fgeo$feature$properties$state
      country <- fgeo$features$properties$country
    }    
  }else{
    state <- ""
    country <- ""
  }

  #### Attributes ####
  alist <- list()
  alist$SoilType <- paste("SoilType = ", txt_clss)
  alist$State <- state
  alist$Country <- country
  alist$Longitude <- lon
  alist$Latitude <- lat
  alist$DataSource <- paste("Original source is Soil and Landscape Grid of Australia. See: https://esoil.io/TERNLandscapes/Public/Pages/SLGA/index.html", Sys.time())
  alist$Comments <- paste("resolution = 90 m",
                          "- taxonomic classification name =", txt_clss,
                          "- drainage class =", NA, 
                          "- elevation =", NA,
                          "- slope =", NA,
                          "- geomdesc =", NA)
  
  soil_profile$metadata <- alist
  
  if(fix) soil_profile <- fix_apsimx_soil_profile(soil_profile, verbose = verbose)
  
  if(check) check_apsimx_soil_profile(soil_profile)
  
  return(soil_profile)  
}

## Texture to other soil properties

## This function was vectorized by Chloe
## If more than one textured is supplied only the first one is used
## for CN2 and Albedo, but different SWCON values can be provided
## assuming that this is for deeper depths. This only makes sense
## if the first layer is the top layer and the other ones are the 
## deeper layers of the soil profile
texture2soilParms_slga <- function(texture.class = "NO DATA") { 
  # Define texture classes and associated parameters
  textureClasses <- c("clay", "silty clay", "sandy clay", "clay loam", "silty clay loam", 
                      "sandy clay loam", "loam", "silty loam", "sandy loam", "silt", 
                      "loamy sand", "sand", "NO DATA")  
  Albedo <- c(0.12, 0.12, 0.13, 0.13, 0.12, 0.13, 0.13, 0.14, 0.13, 0.13, 0.16, 0.19, 0.13)
  CN2 <- c(73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 68.0, 73.0, 68.0, 68.0, 73.0)
  SWCON <- c(0.25, 0.3, 0.3, 0.4, 0.5, 0.5, 0.5, 0.5, 0.6, 0.5, 0.6, 0.75, 0.5)
  
  # Match the first layer of texture.class
  wtc <- match(texture.class[1], textureClasses, nomatch = length(textureClasses))
  
  # Match all layers of texture.class for SWCON
  swcon <- SWCON[match(texture.class, textureClasses, nomatch = length(textureClasses))]
  
  # Return results
  ans <- list(
    textureClasses = textureClasses[wtc],  # Only for the first layer
    Albedo = Albedo[wtc],                  # Only for the first layer
    CN2 = CN2[wtc],                        # Only for the first layer
    SWCON = swcon                          # Vector for all layers
  )
  return(ans)
}


## Potentially useful function provided by Eric Zurcher
## written by Andrew Moore
## I think the values should be in the 0-1 range
#==========================================================================
# Texture class mapping function
#==========================================================================

# Re-express the PSD in terms of the International system, using an equation from Minasny et al. (2001)

# intl_clay_propn <- function( usda_clay, usda_silt ) { 
#   return( usda_clay) 
# }
# 
# intl_silt_propn <- function( usda_clay, usda_silt ) { 
#   return( max( 0.0, -0.0041 - 0.127*usda_clay + 0.553*usda_silt + 0.17*usda_clay^2 - 0.19*usda_silt^2 + 0.59*usda_clay*usda_silt ) ) 
# }
# 
# intl_sand_propn <- function( usda_clay, usda_silt ) { 
#   return( 1.0 - intl_clay_propn( usda_clay, usda_silt ) - intl_silt_propn( usda_clay, usda_silt ) )
# }  

# Texture triangle as equations

## The function below was written by Andrew Moore and/or Eric Zurcher and
## modified by Fernando Miguez and Chloe
# redefine the texture_class function to accept vectors
texture_class_slga <- function(usda_clay, usda_silt) {

  if(length(usda_clay) != length(usda_silt))
    stop("length of clay should be equal to length of silt", call. = FALSE)
  if (any(usda_clay < 0 | usda_clay > 1)) stop("All values in usda_clay should be between 0 and 1")
  if (any(usda_silt < 0 | usda_silt > 1)) stop("All values in usda_silt should be between 0 and 1")
  
  if(any(usda_clay + usda_silt > 1))
    stop("clay plus silt should be less than one", call. = FALSE)
  
  intl_clay <- usda_clay
  intl_silt <- usda_silt
  intl_sand <- 1.0 - intl_clay - intl_silt
  
  # Initialize result vector
  classes <- rep(NA, length = length(usda_clay))
  
  # Apply texture triangle rules
  classes[(intl_sand < 0.75 - intl_clay) & (intl_clay >= 0.40)] <- "silty clay"
  classes[(intl_sand < 0.75 - intl_clay) & (intl_clay >= 0.26) & is.na(classes)] <- "silty clay loam"
  classes[(intl_sand < 0.75 - intl_clay) & is.na(classes)] <- "silty loam"
  classes[(intl_clay >= 0.40 + (0.305 - 0.40) / (0.635 - 0.35) * (intl_sand - 0.35)) & 
            (intl_clay < 0.50 + (0.305 - 0.50) / (0.635 - 0.50) * (intl_sand - 0.50)) & is.na(classes)] <- "clay"
  classes[(intl_clay >= 0.26 + (0.305 - 0.26) / (0.635 - 0.74) * (intl_sand - 0.74)) & is.na(classes)] <- "sandy clay"
  classes[(intl_clay >= 0.26 + (0.17 - 0.26) / (0.83 - 0.49) * (intl_sand - 0.49)) & 
            (intl_clay < 0.10 + (0.305 - 0.10) / (0.635 - 0.775) * (intl_sand - 0.775)) & is.na(classes)] <- "clay loam"
  classes[(intl_clay >= 0.26 + (0.17 - 0.26) / (0.83 - 0.49) * (intl_sand - 0.49)) & is.na(classes)] <- "sandy clay loam"
  classes[(intl_clay >= 0.10 + (0.12 - 0.10) / (0.63 - 0.775) * (intl_sand - 0.775)) & 
            (intl_clay < 0.10 + (0.305 - 0.10) / (0.635 - 0.775) * (intl_sand - 0.775)) & is.na(classes)] <- "loam"
  classes[(intl_clay >= 0.10 + (0.12 - 0.10) / (0.63 - 0.775) * (intl_sand - 0.775)) & is.na(classes)] <- "sandy loam"
  classes[(intl_clay < 0.00 + (0.08 - 0.00) / (0.88 - 0.93) * (intl_sand - 0.93)) & is.na(classes)] <- "loamy sand"
  classes[is.na(classes)] <- "sand"
  
  return(classes)
}

### The function below was created by Chloe (Yunru Lai)
### Modified by Fernando Miguez 2025-01-11

#' The data comes from https://esoil.io/TERNLandscapes/Public/Pages/SLGA/index.html
#' 
#' @title Retrieve soil profile data from SLGA (Soils for Australia)
#' @description This function gets a soil profile for the Australia extent
#' @name get_slga_soil
#' @param lonlat Longitude and latitude vector (e.g. c(151.8306, -27.4969))
#' @return a data.frame with elements: depth (midpoint in cm), depths (as character in cm), thickness (cm), clay, sand, silt, wv1500, wv0033, bdod, nitrogen, phh2o, cec, soc
#' @author Chloe (Yunru Lai) and Fernando E. Miguez
#' @export
#' @examples
#' \dontrun{
#' ## retrieve data from longitude and latitude 151.8305805675806 and -27.496873026858598
#' ## Note: This can take a couple of minutes
#' slga.soil <- get_slga_soil(lonlat = c(151.8306, -27.4969)) 
#' 
#' }

get_slga_soil <- function(lonlat) {
  
  if(missing(lonlat))
    stop("lonlat is missing without default", call. = FALSE)
  
  #### Create extent step ####
  longitude <- as.numeric(lonlat[1])
  latitude <- as.numeric(lonlat[2])
  
  if (longitude < 112 || longitude > 154) stop("Longitude should be between 112 and 154 for the extent of Australia")
  if (latitude < -44 || latitude > -10) stop("Latitude should be between -44 and -10 for the extent of Australia")
  
  ### API endpoint
  root.url <- "https://esoil.io/TERNLandscapes/Public/Products/TERN/SLGA/"
    
  # Define properties
  properties <- list(
    clay = paste0(root.url, "CLY/CLY_000_005_EV_N_P_AU_TRN_N_20210902.tif"),
    sand = paste0(root.url, "SND/SND_000_005_EV_N_P_AU_TRN_N_20210902.tif"),
    silt = paste0(root.url, "SLT/SLT_000_005_EV_N_P_AU_TRN_N_20210902.tif"),
    wv1500 = paste0(root.url, "L15/L15_000_005_EV_N_P_AU_TRN_N_20210614.tif"),
    wv0033 = paste0(root.url, "DUL/DUL_000_005_EV_N_P_AU_TRN_N_20210614.tif"),
    bdod = paste0(root.url, "BDW/BDW_000_005_EV_N_P_AU_TRN_N_20230607.tif"),
    nitrogen = paste0(root.url, "NTO/NTO_000_005_EV_N_P_AU_NAT_C_20231101.tif"),
    phh2o = paste0(root.url, "PHW/PHW_000_005_EV_N_P_AU_TRN_N_20220520.tif"),
    cec = paste0(root.url, "CEC/CEC_000_005_EV_N_P_AU_TRN_N_20220826.tif"),
    soc = paste0(root.url, "SOC/SOC_000_005_EV_N_P_AU_TRN_N_20220727.tif")
  )
  ## Previously used property
  # des = "https://esoil.io/TERNLandscapes/Public/Products/TERN/SLGA/DES/DES_000_200_EV_N_P_AU_TRN_C_20190901.tif",
  
  # Define depths
  depths <- c("000_005", "005_015", "015_030", "030_060", "060_100", "100_200")
  depth.midpoint <- c(2.5, 10, 22.5, 45, 80, 150) ## In cm 
  thickness <- c(5, 10, 15, 30, 40, 100) ## In cm
  # Initialize results list
  results <- list()
  
  # Loop through each property and depth
  for (property_name in names(properties)) {
    property_url <- properties[[property_name]]
    property_results <- numeric(length(depths))  # Initialize vector of length 6
    
    for (i in seq_along(depths)) {
      depth <- depths[i]
      
      # Replace "000_005" with the current depth
      cog_path <- gsub("000_005", depth, property_url)
      
      # Construct the API request URL
      api_url <- paste0(
        "https://esoil.io/TERNLandscapes/RasterProductsAPI/Drill?",
        "format=json&verbose=false&COGPath=", cog_path,
        "&latitude=", latitude, "&longitude=", longitude
      )
      
      # Make the API request
      response <- jsonlite::fromJSON(api_url)
      ## This returns a data.frame when it works
      
      if(!inherits(response, 'try-error')){
        property_results[i] <- response$Value  # Assign to vector
      }else{
        message("Could not retrieve data for", cog_path, latitude, longitude)
        property_results[i] <- NA  # Handle errors gracefully
      }
    }
    
    # Store the results for the property
    results[[property_name]] <- property_results
  }
  
  ans <- data.frame(depth = depth.midpoint, depths = depths, thickness = thickness,
                    as.data.frame(results))
  return(ans)
}

## Test APSIM SWIM code for 'Classic'
require(apsimx)

## This test does not need APSIM Classic to run
run.test.apsim.swim <- TRUE

## Only write to a temp directory
tmp.dir <- tempdir()

extd.dir <- system.file("extdata", package = "apsimx")

if(run.test.apsim.swim){

  inspect_apsim("maize-swim.apsim", src.dir = extd.dir)  
  inspect_apsim("maize-swim.apsim", src.dir = extd.dir, node = "Weather")  
  inspect_apsim("maize-swim.apsim", src.dir = extd.dir, 
                node = "Soil", soil.child = "Water") 
  inspect_apsim("maize-swim.apsim", src.dir = extd.dir, 
                node = "Soil", soil.child = "SWIM") 
  edit_apsim("maize-swim.apsim", 
             src.dir = extd.dir, wrt.dir = tmp.dir,
             node = "Soil", soil.child = "SWIM",
             parm = "Salb", value = 0.10) 
  inspect_apsim("maize-swim-edited.apsim", src.dir = tmp.dir, 
                node = "Soil", soil.child = "SWIM") 
  edit_apsim("maize-swim-edited.apsim", 
             src.dir = tmp.dir, wrt.dir = tmp.dir,
             overwrite = TRUE,
             node = "Soil", soil.child = "SWIM",
             parm = "WaterTableDepth", value = 1500)
  inspect_apsim("maize-swim-edited.apsim", src.dir = tmp.dir, 
                node = "Soil", soil.child = "SWIM") 
  
}

## Function to check whether mono (unix only) and APSIM-X are installed

.onAttach <- function(libname, pkgname){
  
  find.mono <- NULL
  
  find.apsim <- NULL
  ## For unix
  if(.Platform$OS.type == "unix"){
    ## Check for mono
    mono <- suppressWarnings(system("which mono", intern = TRUE, ignore.stderr = TRUE))
    if(length(mono) == 0){
      find.mono <- "Mono framework not found (only needed for pre-Sept 2021 versions of APSIM Next Gen) \n"
      ## In Ubuntu mono seems to be in /usr/bin/mono
      ## In Mac mono seems to be installed in:
      ## "/Library/Frameworks/Mono.framework/Versions/Current/Commands/mono"
      ## In both of them the command 'mono' seems to work
    }
    ## Check for dotnet
    ## This is not necessary any more
    # dotnet <- system("which dotnet", intern = TRUE)
    # if(length(dotnet) == 0){
    #   find.dotnet <- "dotnet core framework not found (needed Sept 2021 -or later- versions of APSIM Next Gen) \n"
    #   ## In Ubuntu mono seems to be in /usr/bin/mono
    #   ## In Mac mono seems to be installed in:
    #   ## "/usr/local/bin/dotnet"
    #   ## In both of them the command 'mono' seems to work
    # }
    
    ## This is for solaris or any other flavor of unix
    laf <- character(0)
    ## List of application files in Darwin (Mac)
    if(grepl("Darwin", Sys.info()[["sysname"]])){
      laf <- list.files("/Applications/")
    }
    ## Linux (Debian)
    if(grepl("Linux", Sys.info()[["sysname"]])){
      laf <- list.files("/usr/local/lib")
    }
  }else{
        if(grepl("Windows", Sys.info()[["sysname"]])){
          laf <- list.files("C:/PROGRA~1")
          laf2 <- list.files("C:/PROGRA~2")
        }
  }
  ## Check whether APSIM-X is detected
  if(!grepl("Windows", Sys.info()[["sysname"]])){
    find.apsimx <- laf[grepl("APSIM", laf, ignore.case = TRUE)]
  }else{
    ## This is the location for APSIM-X
    find.apsimx <- laf[grepl("APSIM", laf, ignore.case = TRUE)]
    ## This is the location for APSIM "Classic"
    find.apsim <- laf2[grepl("APSIM", laf2, ignore.case = TRUE)]
  }
  
  if(length(find.apsimx) > 0 || length(find.apsim) > 0){
    
    fax <- paste0("Found APSIM or APSIM-X")
    ## Won't print ApsimX at the moment but might change it in the future
    ## packageStartupMessage(fax)
    
  }else{
    
    apsim.not.found <- "APSIM(X) not found.
                        If APSIM(X) is installed in an alternative location,
                        set paths manually using 'apsimx_options' or 'apsim_options'.
                        You can still try as the package will look into the registry (under Windows)"
    
    ## if(length(find.mono) == 0) apsim.not.found <- c(find.mono, apsim.not.found)

    packageStartupMessage(apsim.not.found)
  }
}

#### Testing get_worldmodeler_soil_profile
require(apsimx)
apsimx_options(warn.versions = FALSE)

run.test.worldmodeler <- get(".run.local.tests", envir = apsimx.options)

if(run.test.worldmodeler && Sys.info()[["user"]] == "fernandomiguez"){
  
  ### Simple example which cleans up
  sp0 <- get_worldmodeler_soil_profile(lonlat = c(-93, 42))
  plot(sp0[[1]], property = "water")
  
  tdir <- "~/Dropbox/apsimx-other/worldmodeler"
  get_worldmodeler_soil_profile(lonlat = c(-93, 42), wrt.dir = tdir, 
                                       filename = "temp_soils.soils")
  
  lonlat.mat <- rbind(c(-93, 42), c(-92, 41), c(-91, 40))
  get_worldmodeler_soil_profile(lonlat = lonlat.mat, wrt.dir = tdir, 
                                filename = "mat_temp_soils.soils")
  
  sp1 <- read_apsim_soils("temp_soils.soils", src.dir = tdir)

  plot(sp1[[1]], property = "Carbon")  
  
  sps <- read_apsim_soils("mat_temp_soils.soils", src.dir = tdir)
  
  plot(sps[[2]], property = "Carbon")  
  
}

### World modeler is not currently working (2026-06-22)
if(run.test.worldmodeler && Sys.info()[["user"]] == "fernandomiguez" && FALSE){
  
  am0 <- get_worldmodeler_apsim_met(lonlat = c(-93, 42), dates = c("1990-01-01", "2020-12-31"))
  
  if(inherits(am0[[1]], 'met')){
    plot(am0[[1]], met.var = "rain", cumulative = TRUE, climatology = TRUE)
  }
  
  tdir <- "~/Dropbox/apsimx-other/worldmodeler"

  lonlat.mat <- rbind(c(-93, 42), c(-92, 41), c(-91, 40))
 
  am1 <- get_worldmodeler_apsim_met(lonlat = lonlat.mat, 
                                    dates = c("1990-01-01", "2020-12-31"))
  
  if(inherits(am1[[1]], 'met')){
    plot(am1[[1]], met.var = "rain", cumulative = TRUE, climatology = TRUE)
  } 
  
  if(inherits(am1[[2]], 'met')){
    plot(am1[[2]], met.var = "rain", cumulative = TRUE, climatology = TRUE)
  } 
  
  am2 <- get_worldmodeler_apsim_met(lonlat = lonlat.mat, 
                                    dates = c("1990-01-01", "2020-12-31"),
                                    wrt.dir = tdir,
                                    filenames = c("marshall1.met", "marshall2.met", "marshall3.met"))
  
  sapply(am2, class)
  
  if(inherits(am2[[1]], 'met')){
    plot(am2[[1]], met.var = "rain", cumulative = TRUE, climatology = TRUE)
  } 
}

#' Read a met file into R
#' 
#' This function uses S3 classes and stores the additional information as attributes \cr
#' I use a more strict format than \acronym{APSIM} and reading and writing will not \cr
#' preserve all the details. For example, at this moment comments are lost through \cr
#' the process of read and write unless they are added back in manually. \cr
#' Also, empty lines are ignored so these will be lost as well in the read and write process.
#' 
#' @title Read in an APSIM met file
#' @name read_apsim_met
#' @description Read into R a met file and return an object of class \sQuote{met}
#' @param file path to met file
#' @param src.dir optional source directory
#' @param verbose whether to suppress all messages and warnings
#' @return an object of class \sQuote{met} with attributes
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ames.met <- read_apsim_met("Ames.met", src.dir = extd.dir)
#' ames.met
#' }
#' 

read_apsim_met <- function(file, src.dir = ".", verbose = TRUE){

  if(!grepl("met$",file)) stop("file should have a .met extension")
  
  file.path <- file.path(src.dir,file)

  ## Read the header
  header <- scan(file = file.path, 
                 what = "character", 
                 sep = "\n",
                 blank.lines.skip = FALSE,
                 nlines = 30, 
                 quiet = TRUE)
  ## hdrl is for keeping track of header lines
  hdrl <- 0; skip.lines <- 0 
  ## attrs <- c("name","site","latitude","longitude","tav","amp","clnms","clunits")
  name <- NULL; site <- NULL; latitude <- NULL; longitude <- NULL; 
  tav <- NULL; amp <- NULL; clnms <- NULL; clunits <- NULL; comments <- NULL
  constants <- vector(mode = "list",30); constant.count <- 0; fnd <- FALSE
  comment.lines <- 0
  ## This is as ugly as it gets but so are met files
  for(i in 1:30){
    if(grepl("^!",header[i])){comment.lines <- comment.lines + 1; next}
    if(grepl("[weather.met.weather]",header[i],fixed=TRUE)){name <- header[i];hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("^site",header[i],ignore.case=TRUE)){site <- header[i];hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("^latitude",header[i],ignore.case=TRUE)){latitude <- header[i];hdrl <- hdrl + 1; fnd <- TRUE} 
    if(grepl("^longitude",header[i],ignore.case=TRUE)){longitude <- header[i];hdrl <- hdrl + 1; fnd <- TRUE} 
    if(grepl("^tav",header[i])){tav <- header[i];hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("^amp",header[i])){amp <- header[i];hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("year",header[i]) && grepl("radn",header[i])){clnms <- header[i];hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("()",header[i],fixed=TRUE)){clunits <- header[i];skip.lines <- i;hdrl <- hdrl + 1; fnd <- TRUE}
    if(grepl("=",header[i],fixed=TRUE) && fnd == FALSE){
      constant.count <- constant.count + 1
      constants[constant.count] <- header[i]
      hdrl <- hdrl + 1
    } 
    fnd <- FALSE
  }

  constants <- unlist(constants[1:constant.count])
  
  if(constant.count == 0){
    constants <- NA
  }

  if(verbose){
    cat("Found ",hdrl," header lines \n")
    cat("Found ",comment.lines," comment lines \n")
    cat("Found ",skip.lines," skip lines \n")
    cat("Found ",constant.count,"constants \n")
  }
  
  ## I only check the first 6 column names but there might be more
  clnms <- sub("^\\s+","",clnms)
  clnms.s <- strsplit(clnms,"\\s+")[[1]]
  if(sum(clnms.s %in% c("year","day","radn","maxt","mint","rain")) < 6){
    cat("All column names:",clnms,"\n") 
    warning("column names might be wrong")
  }
  
  clunits <- sub("^\\s+","",clunits)
  clunits.s <- strsplit(clunits,"\\s+")[[1]]
  ## Sounds like there is no point in checking units
  ## As they are a complete mess
    
  met <- utils::read.table(file = file.path, 
                           header = FALSE, 
                           as.is = TRUE,
                           na.strings = c(NA,-99),
                           comment.char = "!", 
                           col.names = clnms.s,
                           skip = skip.lines)
  
  attr(met, "filename") <- file
  attr(met, "site") <- ifelse(is.null(site),NA,site)
  attr(met, "latitude") <- latitude
  attr(met, "longitude") <- ifelse(is.null(longitude),NA,longitude)
  attr(met, "tav") <- tav
  attr(met, "amp") <- amp
  attr(met, "colnames") <- clnms.s
  attr(met, "units") <- clunits.s
  attr(met, "constants") <- constants
  attr(met, "comments") <- ifelse(is.null(comments),NA,comments)
  class(met) <- c("met","data.frame")
  return(met)
}

#' Write a met file to disk. It takes an object of class \sQuote{met}
#' 
#' @title Write an APSIM met file
#' @name write_apsim_met
#' @description Write an object of class \sQuote{met} to disk
#' @param met object of class \sQuote{met}
#' @param wrt.dir directory where the file will be written
#' @param filename optional alternative filename
#' @return does not create an R object, it only writes to disk
#' @details at the moment the read-write cycle will strip comments
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ames.met <- read_apsim_met("Ames.met", src.dir = extd.dir)
#' ames.met
#' tmp.dir <- tempdir()
#' write_apsim_met(ames.met, wrt.dir = tmp.dir, filename = "Ames.met")
#' ## Here I write to a temporary directory, but change this to where
#' ## you want to write to
#' }
#' 

write_apsim_met <- function(met, wrt.dir = NULL, filename = NULL){
  
  if(attr(met, "filename") != "noname.met" && is.null(filename)) filename <- attr(met, "filename")
    
  if(is.null(wrt.dir) && is.null(filename)){
    ## This assumes that the full path is in filename
    file.path <- attr(met, "filename")
  }
  if(!is.null(wrt.dir) && is.null(filename)){
    if(attr(met, "noname.met")){
      stop("Need to supply filename if 'wrt.dir' is not NULL")
    }else{
     file.path <- file.path(wrt.dir, attr(met, "filename")) 
    }
  }
  if(is.null(wrt.dir) && !is.null(filename)){
    stop("Need to supply 'wrt.dir' if filename is not NULL")
  }
  if(!is.null(wrt.dir) && !is.null(filename)){
    file.path <- file.path(wrt.dir, filename)
  }
  if(!is.null(filename)){
    if(!grepl(".met", filename, fixed=TRUE)) stop("filename should end in .met")
  }
  ## Open connection
  con <- file(description = file.path, open = "w")
  ## Write comments if they exist
  if(!is.na(attr(met,"comments")) && length(attr(met,"site")) > 0) 
    writeLines(attr(met,"comments"), con = con)
  ## Start header
  writeLines("[weather.met.weather]", con = con)
  ## Write site if it exists
  if(!is.na(attr(met,"site")) && length(attr(met,"site")) > 0){
    writeLines(attr(met,"site"), con = con)
  }
  if(is.na(attr(met,"latitude")) || length(attr(met,"latitude")) == 0){
    stop("latitude should be present", call. = FALSE)
  }
  writeLines(attr(met,"latitude"), con = con)
  if(!is.na(attr(met,"longitude")) && length(attr(met,"longitude")) > 0){
    writeLines(attr(met,"longitude"), con = con)
  }
  writeLines(attr(met,"tav"), con = con)
  writeLines(attr(met,"amp"), con = con)
  ## Write constants
  if(!is.na(attr(met,"constants")) && length(attr(met,"constants")) > 0){
    for(i in seq_along(attr(met,"constants"))){
      writeLines(attr(met,"constants")[i], con = con)
    }
  }
  writeLines(paste(attr(met,"colnames"), collapse = " "), con = con)
  writeLines(paste(attr(met,"units"), collapse = " "), con = con)
  
  met <- as.data.frame(unclass(met))
  attr(met, "names") <- NULL
  utils::write.table(met, file = con,
                     append = TRUE, quote = FALSE,
                     row.names = FALSE, col.names = FALSE)
  close(con)
}

#' @title Printer-friendly version of a metfile
#' @name print.met
#' @description Print a met file in a friendly way
#' @param x an R object of class \sQuote{met}
#' @param ... additional printing arguments
#' @return It prints to console. Not used to return an R object.
#' @export
#' 
print.met <- function(x,...){
  
  ## print the header and just head
  cat(attr(x, "filename"),"\n")
  if(!is.na(attr(x,"site"))) cat(attr(x, "site"),"\n")
  cat(attr(x, "latitude"),"\n")
  if(!is.na(attr(x,"longitude"))) cat(attr(x, "longitude"),"\n")
  cat(attr(x, "tav"),"\n")
  cat(attr(x, "amp"),"\n")
  cat(attr(x, "colnames"),"\n")
  cat(attr(x, "units"),"\n")
  if(!any(is.na(attr(x,"constants"))) && !is.null(attr(x,"constants"))) cat(attr(x, "constants"), "\n")
  
  print(utils::head(as.data.frame(x),...))
}

#' @title Perform imputation for missing data in a met file
#' @name impute_apsim_met
#' @description Takes in an object of class \sQuote{met} and imputes values
#' @param met object of class \sQuote{met}
#' @param method method for imputation, \sQuote{approx} (\code{\link[stats]{approxfun}}),
#' \sQuote{spline} (\code{\link[stats]{splinefun}}) or \sQuote{mean} (\code{\link{mean}}).
#' @param verbose whether to print missing data to the console, default = FALSE
#' @param ... additional arguments to be passed to imputation method
#' @return an object of class \sQuote{met} with attributes
#' @export
#' 

impute_apsim_met <- function(met, method = c("approx","spline","mean"), verbose = FALSE, ...){
  
  if(!inherits(met, "met")) stop("met should be of class 'met'") 
  
  method <- match.arg(method)
  
  ## Someday I will do this when it is needed
  args <- list(...)
  
  ans <- as.data.frame(met) ### make copy of met object as a data frame
  
  ## If there is a missing value in the first row it won't be imputed
  if(any(is.na(ans[1,]))){
    wn1r <- which(is.na(ans[1,]))
    for(i in seq_along(wn1r)){
      if(all(is.na(ans[1:15, wn1r[i]]))){
        warning("The mean was used to impute the first row as the first 15 values were NAs")
        ans[1, wn1r[i]] <- mean(ans[, wn1r[i]], na.rm = TRUE)
      }else{
        ans[1, wn1r[i]] <- mean(ans[1:15, wn1r[i]], na.rm = TRUE)  
      }
      cat("Imputed first row for:", names(ans)[wn1r[i]], "\n")
    }
    print(as.data.frame(ans[1,]))
  }
  
  ## If there is a missing value in the last row it won't be imputed
  if(any(is.na(ans[nrow(ans),]))){
    wn1r <- which(is.na(ans[nrow(ans),]))
    for(i in seq_along(wn1r)){
      if(all(is.na(ans[(nrow(ans) - 15):nrow(ans), wn1r[i]]))){
        warning("The mean was used to impute the last row as the last 15 values were NAs")
        ans[nrow(ans), wn1r[i]] <- mean(ans[, wn1r[i]], na.rm = TRUE)
      }else{
        ans[nrow(ans), wn1r[i]] <- mean(ans[(nrow(ans) - 15):nrow(ans), wn1r[i]], na.rm = TRUE)  
      }
      cat("Imputed last row for:", names(ans)[wn1r[i]], "\n")
    }
    print(as.data.frame(ans[nrow(ans),]))
  }
  
  ## Which rows have missing data
  missing.vector <- vector(mode = "numeric", length = length(names(ans)))
  
  if(all(sapply(sapply(ans, function(x) which(is.na(x))),length) == 0))
    warning("No missing data found")
  
  missing.rows <- sapply(ans, function(x) which(is.na(x)), simplify = FALSE)

  if(verbose){
    for(i in 1:ncol(ans)){
      tmp.mr <- missing.rows[[i]]
      if(length(tmp.mr) > 0){
        cat("Missing values for", names(ans)[i], "\n")
        print(as.data.frame(ans)[tmp.mr,])
      }
    }
  }
  
  col.classes <- sapply(ans, class) 
  ## I might need to prevent imputation on characters/factors 
  which.col.missing.values <- which(sapply(missing.rows, function(x) length(x)) > 0)

  for(i in which.col.missing.values){
    if(method == "approx"){
      imputed.values <- stats::approx(x = seq_len(nrow(ans)),
                                      y = ans[[i]],
                                      xout = missing.rows[[i]])$y
    }
    if(method == "spline"){
      imputed.values <- stats::spline(x = seq_len(nrow(ans)),
                                      y = met[[i]],
                                      xout = missing.rows[[i]])$y
    }
    if(method == "mean"){
      imputed.values <- mean(ans[[i]], na.rm = TRUE)
    }
    
    ans[missing.rows[[i]],i] <- imputed.values
    
  }
  
  ### Copy attributes to metd
  attr(ans, "filename") <- attr(met, "filename")
  attr(ans, "site") <- attr(met, "site")
  attr(ans, "latitude") <- attr(met, "latitude")
  attr(ans, "longitude") <- attr(met, "longitude")
  attr(ans, "tav") <- attr(met, "tav")
  attr(ans, "amp") <- attr(met, "amp")
  attr(ans, "colnames") <- attr(met, "colnames")
  attr(ans, "units") <- attr(met, "units")
  attr(ans, "constants") <- attr(met, "constants")
  attr(ans, "comments") <- attr(met, "comments")
  class(ans) <- c("met","data.frame")
  return(ans)
}

#' @title Check a met file for possible errors
#' @name check_apsim_met
#' @description Takes in an object of class \sQuote{met} and checks for missing/valid/reasonable values
#' @param met object of class \sQuote{met}
#' @details It will only check for missing values and reasonable (within range) values for:
#'  \sQuote{year}: range (1500 to 3000); \cr
#'  \sQuote{day}: range (1 to 366); \cr 
#'  \sQuote{maxt}: range (-60 to 60) -- units (C); \cr
#'  \sQuote{mint}: range (-60 to 40) -- units (C); \cr
#'  \sQuote{radn}: range (0 to 40) -- units (MJ/m2/day);  \cr
#'  \sQuote{rain}: range (0 to 100) -- units (mm/day)
#' @return does not return anything unless possible errors are found
#' @export
#' 
check_apsim_met <- function(met){
  
  if(!inherits(met, "met")) 
    stop("object should be of class 'met'", call. = FALSE)
  
  if(nrow(met) == 0)
    stop("No rows of data present in this object.", call. = FALSE)
  
  if(is.null(attr(met, 'amp')))
    warning("'amp' is missing")

  if(is.null(attr(met, 'tav')))
    warning("'tav' is missing")
  
  if(!is.null(attr(met, 'amp'))){
    amp.value <- as.numeric(strsplit(attr(met, 'amp'), split = "\\s+")[[1]][3])
    met0 <- amp_apsim_met(met) 
    amp.value.calc <- as.numeric(strsplit(attr(met0, 'amp'), split = "\\s+")[[1]][3])
    if(abs(amp.value - amp.value.calc) > 0.5)
      warning(paste("'amp' value", amp.value, "in met file is different from calculated", amp.value.calc))
  }

  if(!is.null(attr(met, 'tav'))){
    tav.value <- as.numeric(strsplit(attr(met, 'tav'), split = "\\s+")[[1]][3])
    met0 <- tav_apsim_met(met) 
    tav.value.calc <- as.numeric(strsplit(attr(met0, 'tav'), split = "\\s+")[[1]][3])
    if(abs(tav.value - tav.value.calc) > 0.5)
      warning(paste("'tav' value", tav.value, "in file is different from calculated", tav.value.calc))
  }
  
  if(length(colnames(met)) != length(attr(met, "colnames")))
    stop("Length of column names does not match the length of 'colnames' in the attributes", call. = FALSE)
  
  col.names <- attr(met, "colnames")
  
  if(length(col.names) != ncol(met)){
    cat("Names in data.frame:", names(met), "\n")
    cat("Names in attribute column names", col.names, "\n")
    cat("Different names", setdiff(names(met), col.names), "\n")
    warning("Number of columns in data.frame do not match column names")
  } 
  ## check for column names
  if(sum(names(met) %in% col.names) < 6) warning("column names might be wrong")
  ## Detect possible errors with years
  if(any(is.na(met[["year"]]))){
    print(met[is.na(met$year),])
    warning("Missing values found for year")
  }
  if(any(min(met[["year"]], na.rm = TRUE) < 1500)){
    print(met[met$year < 1500,])
    warning("year is less than 1500")
  }
  if(any(max(met[["year"]], na.rm = TRUE) > 3000)){
    print(met[met$year > 3000,])
    warning("year is greater than 3000")
  }
  ## Detect possible errors with day
  if(any(is.na(met[["day"]]))){
    print(met[is.na(met$day),])
    warning("Missing values found for day")
  }
  if(any(min(met[["day"]], na.rm = TRUE) < 1)){
    print(met[met$day < 1,])
    warning("day is less than 1")
  }
  if(any(max(met[["day"]], na.rm = TRUE) > 366)){
    print(met[met$day > 366,])
    warning("day is greater than 366")
  }
  ## Check for date discontinuities
  origin <- as.Date(paste0(met$year[1], "-01-01"))
  first.day <- doy2date(met$day[1], year = met$year[1])
  last.day <- doy2date(met$day[nrow(met)], year = met$year[nrow(met)])
  dates <- seq(from = first.day, to = last.day, by = 1)
  if(nrow(met) < length(dates)){
    warning(paste(length(dates) - nrow(met), "date discontinuities found. Consider using napad_apsim_met."))
  }
  ## Check for last day of leap year
  if(is_leap_year(met$year[nrow(met)]) && met$day[nrow(met)] == 365){
    warning("Last year in the met file is a leap year and it only has 365 days")
  }
  ## Detect possible errors with minimum temperature
  if(any(is.na(met[["mint"]]))){
    print(met[is.na(met$mint),])
    warning("Missing values found for minimum temperature")
  }
  if(any(min(met[["mint"]], na.rm = TRUE) < -60)){
    print(met[met$mint < -60,])
    warning("Minimum temperature is less than -60")
  }
  if(any(max(met[["mint"]], na.rm = TRUE) > 40)){
    print(met[met$mint > 40,])
    warning("Minimum temperature is greater than 40")
  }
  ## Detect possible errors with maximum temperature
  if(any(is.na(met[["maxt"]]))){
    print(met[is.na(met$maxt),])
    warning("Missing values found for maximum temperature")
  }
  if(any(min(met[["maxt"]], na.rm = TRUE) < -60)){
    print(met[met$maxt < -60,])
    warning("Maximum temperature less than -60")
  }
  if(any(max(met[["maxt"]], na.rm = TRUE) > 60)){
    print(met[met$maxt > 60,])
    warning("Maximum temperature is greater than 60")
  }
  ## Make sure that maxt is always higher or equal to mint
  temp.diff <- met[["maxt"]] - met[["mint"]]
  temp.diff <- temp.diff[!is.na(temp.diff)] ## Just look at data which is not NA
  if(any(temp.diff < 0)){
    print(met[which(temp.diff < 0),])
    warning("Minimum temperature is greater than maximum temperature")
  }
  ## Detect possible errors with radiation
  if(any(is.na(met[["radn"]]))){
    print(met[is.na(met$radn),])
    warning("Missing values found for solar radiation")
  }
  if(any(min(met[["radn"]], na.rm = TRUE) < 0)){
    print(met[met$radn < 0,])
    warning("Radiation is negative")
  }
  if(any(max(met[["radn"]], na.rm = TRUE) > 40)){
    print(met[met$radn > 40,])
    warning("Radiation is greater than 40 (MJ/m2/day)")
  }
  ## Detect possible errors with rain
  if(any(is.na(met[["rain"]]))){
    print(met[is.na(met$rain),])
    warning("Missing values found for precipitation")
  }
  if(any(min(met[["rain"]], na.rm = TRUE) < 0)){
    warning("Rain is negative")
  }
  if(any(max(met[["rain"]], na.rm = TRUE) > 100)){
    ## Apparently it is more or less reasonable to question 100mm in 24 hours
    ## https://journals.ametsoc.org/view/journals/hydr/20/12/jhm-d-19-0039_1.xml
    warning("Rain is possibly too high")
  }
  
  if(!is.null(met$vp)){
    if(max(met[["vp"]], na.rm = TRUE) > 100){
      warning("Vapor Pressure units might be worng. It should be in hecto Pascals")
    }
  }
}

#' Fill in with missing data date discontinuities in a met file
#' 
#' @title Pad a met file with NAs when there are date discontinuities
#' @name napad_apsim_met
#' @description It will fill in or \sQuote{pad} a met object with NAs
#' @param met object of class \sQuote{met}
#' @note The purpose of this function is to allow for imputation using \code{\link{impute_apsim_met}}
#' @return It returns an object of class \sQuote{met} with padded NAs.
#' @export

napad_apsim_met <- function(met){
 
  if(!inherits(met, "met")) stop("object should be of class 'met'")
  
  ## First check if there are any discontinuities
  origin <- as.Date(paste0(met$year[1], "-01-01"))
  ## first.day <- doy2date(met$day[1], year = met$year[1])
  last.day <- doy2date(met$day[nrow(met)], year = met$year[nrow(met)])
  dates <- seq(from = origin, to = last.day, by = "day")
  
  if(nrow(met) == length(dates) && !is_leap_year(met$year[nrow(met)])){
    stop("No discontinuities found", call. = FALSE)
  }
  
  fix1 <- FALSE
  
  if(nrow(met) != length(dates)){
    ## Create a data.frame with continuous dates

    namet <- data.frame(date = dates)
    
    ## Create date column for merging
    met <- as.data.frame(met)
    met$date <- as.Date(paste0(met$year, "-", met$day), format = "%Y-%j")

    ans <- merge(namet, met, by = "date", all.x = TRUE)
    ans$year <- as.numeric(format(ans$date, "%Y"))
    ans$day <- as.numeric(format(ans$date, "%j"))
    ans$date <- NULL  
    fix1 <- TRUE
    ans <- as_apsim_met(ans, 
                        filename = attr(met, "filename"),
                        site = attr(met, "site"),
                        latitude = attr(met, "latitude"),
                        tav = attr(met, "tav"),
                        amp = attr(met, "amp"),
                        colnames = attr(met, "colnames"),
                        units = attr(met, "units"),
                        comments = attr(met, "comments"),
                        check = FALSE)
  }  
  
  fix2 <- FALSE
  ## Is the last year a leap year?
  if(is_leap_year(met$year[nrow(met)]) && met$day[nrow(met)] == 365){
      ## This adds a row at the end when it is a leap year and it only has 365 days
    if(fix1) met <- ans
    nms.met <- names(met)
    fill.dat <- as.data.frame(matrix(ncol = length(nms.met)))
    names(fill.dat) <- nms.met
    fill.row <- data.frame(year = met$year[nrow(met)],
                           day = 366,
                           fill.dat[, -c(1:2)])
    ans <- rbind(met, fill.row)
    fix2 <- TRUE
  }
  
  if(!fix1 && !fix2)
    stop("No discontinuities found", call. = FALSE)
  
  attr(ans, "filename") <- attr(met, "filename")
  attr(ans, "site") <- attr(met, "site")
  attr(ans, "latitude") <- attr(met, "latitude")
  attr(ans, "longitude") <- attr(met, "longitude")
  attr(ans, "tav") <- attr(met, "tav")
  attr(ans, "amp") <- attr(met, "amp")
  attr(ans, "colnames") <- attr(met, "colnames")
  attr(ans, "units") <- attr(met, "units")
  attr(ans, "constants") <- attr(met, "constants")
  attr(ans, "comments") <- attr(met, "comments")
  class(ans) <- c("met","data.frame")
  return(ans) 
}

is_leap_year <- function(year){
  if((year %% 4) == 0){
    if((year %% 100) == 0){
      if((year %% 400) == 0){
        ans <- TRUE
      }else{
        ans <- FALSE
      }
    }else{
      ans <- TRUE
    }
  }else{
    ans <- FALSE
  }
  return(ans)
}

#' Simple utility for converting a data frame to an object of class met
#' 
#' @title Conversion from data frame to met object
#' @name as_apsim_met
#' @description It makes minimum assumptions about the data so it is recommended to change defaults
#' @param x object of class \sQuote{data frame}
#' @param filename default \sQuote{noname.met}
#' @param site default \sQuote{nosite}
#' @param latitude default is zero (0)
#' @param longitude default is zero (0)
#' @param tav average temperature (calculated if not supplied)
#' @param amp temperature amplitude (calculated if not supplied)
#' @param colnames default are \dQuote{year}, \dQuote{day}, \dQuote{radn}, 
#'                  \dQuote{maxt}, \dQuote{mint}, \dQuote{rain}
#' @param units default are \dQuote{()}, \dQuote{()}, \dQuote{(MJ/m2/day)},
#'              \dQuote{(oC)}, \dQuote{(oC)}, \dQuote{(mm)}
#' @param constants default is \dQuote{NA}
#' @param comments default is \dQuote{NA}   
#' @param check whether to check the resulting met file using \code{\link{check_apsim_met}}.
#' default is TRUE.  
#' @return it returns an object of class \sQuote{met}.   
#' @export
#' 
as_apsim_met <- function(x,
                         filename = "noname.met",
                         site = "nosite",
                         latitude = 0,
                         longitude = 0,
                         tav = NA,
                         amp = NA,
                         colnames = c("year", "day", "radn", "maxt", "mint", "rain"),
                         units = c("()", "()", "(MJ/m2/day)", "(oC)", "(oC)", "(mm)"),
                         constants = NA,
                         comments = NA,
                         check = TRUE){
  
  if(!inherits(x, "data.frame"))
    stop("Object should be of class data.frame", call. = FALSE)

  if(ncol(x) != 6 && length(colnames) == 6)
    stop("If number of columns is not 6 then provide column names", call. = FALSE)
  
  names(x) <- colnames
  
  attr(x, "filename") <- filename
  attr(x, "site") <- paste("site =", site)
  attr(x, "latitude") <- paste("latitude =", latitude)
  attr(x, "longitude") <- paste("longitude =", longitude)
  
  if(is.na(tav)){
    warning(" 'tav' is NA, use function tav_apsim_met to add it. ") 
  }else{
    attr(x, "tav") <- tav
  }
  
  attr(x, "colnames") <- colnames
  attr(x, "units") <- units
  attr(x, "constants") <- constants
  attr(x, "comments") <- comments
  class(x) <- c("met","data.frame")
  
  if(is.na(amp)){
    warning(" 'amp' is NA, use function amp_apsim_met to add it. ") 
  }else{
    attr(x, "amp") <- amp  
  }
  
  if(check) check_apsim_met(x)
  
  return(x) 
}
  
#' Calculating Thermal Time using a variety of methods.
#' The function will fail if the method is not selected.
#' Also, it does not work if each year does not have at least
#' 365 days.
#' 
#' @title Calculates Thermal Time taking a \sQuote{met} object
#' @name tt_apsim_met
#' @description Calculates Thermal Time using the \sQuote{Classic} formula, 
#' Heat Stress, Crop Heat Unit and other methods
#' @param met object of class \sQuote{met}
#' @param dates when the calculation starts and when it ends. At the moment
#' it needs to be a character vector (e.g. c(\sQuote{01-05}, \sQuote{10-10})). It will
#' use the same dates every year for multiple years.
#' @param method one of \sQuote{Classic_TT}, \sQuote{HeatStress_TT}, \sQuote{ASPIM_TT},
#' \sQuote{CERES_TT} and \sQuote{all}
#' @param x_temp cardinal temperatures (base, optimal and maximum)
#' @param y_tt thermal time accumulation for cardinal temperatures
#' @param base_temp base temperature for Classic TT calculation
#' @param max_temp maximum temperature for Classic TT calculation
#' @param dates.format default is \sQuote{\%d-\%m} which means day and month
#' @return it returns an object of class \sQuote{met} with additional columns
#' \sQuote{Date} and the corresponding TT calculation
#' @export
#' @references  Abendroth, L.J., Miguez, F.E., Castellano, M.J. and Hatfield, J.L. (2019),
#'  Climate Warming Trends in the U.S. Midwest Using Four Thermal Models. 
#'  Agron. J., 111: 3230-3243. (doi:10.2134/agronj2019.02.0118)
#' @examples 
#' \dontrun{
#' require(nasapower)
#' require(ggplot2)
#' 
#' pwr <- get_power_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2015-12-31"))
#' check_apsim_met(pwr)
#' pwr <- impute_apsim_met(pwr)
#' 
#' pwr2 <- tt_apsim_met(pwr, dates = c("01-05", "30-10"), method = c("Classic", "Heat"))
#' 
#' ggplot(data = pwr2, aes(x = Date, y = Classic_TT)) + geom_point()
#' 
#' ggplot(data = pwr2, aes(x = Date, y = HeatStress_TT)) + geom_point()
#' 
#' }

tt_apsim_met <- function(met, dates, 
                         method = c("Classic_TT", "HeatStress_TT", "CropHeatUnit_TT",
                                    "APSIM_TT", "CERES_TT", "all"),
                         x_temp = c(0, 26, 34),
                         y_tt = c(0, 26, 0),
                         base_temp = 0,
                         max_temp = 30,
                         dates.format = c("%d-%m")){

  if(!missing(met) && !inherits(met, "met"))
    stop("Object met should be of class met")

  if(identical(method, c("Classic_TT", "HeatStress_TT", "CropHeatUnit_TT", "APSIM_TT", "CERES_TT", "all")))
    stop("Please select a method. Not all of them are implemented at the moment.", call. = FALSE)
  
  method <- match.arg(method, several.ok = TRUE)
  
  if("all" %in% method) method <- c("Classic_TT", "HeatStress_TT", "CropHeatUnit_TT", "APSIM_TT")
  
  if("CERES_TT" %in% method) stop("not implemented yet.")
  
  if(is.na(as.Date(dates[1], format = dates.format))) stop("first date might be in the wrong format")
  if(is.na(as.Date(dates[2], format = dates.format))) stop("second date might be in the wrong format")
  
  start.date <- as.Date(paste0("01-01", "-", min(met$year)), format = paste0(dates.format, "-%Y"))
  end.date <- as.Date(paste0("31-12", "-", max(met$year)), format = paste0(dates.format, "-%Y"))
  
  tmpd <- data.frame(Dates = seq(from = start.date, 
                                 to = end.date, 
                                 by = "day"), index = 0)
  
  if("Classic_TT" %in% method){
    Classic_TT_vec <- numeric(nrow(met))
  } 
  if("HeatStress_TT" %in% method){
    HeatStress_TT_vec <- numeric(nrow(met))
  } 
  if("CropHeatUnit_TT" %in% method){
    CropHeatUnit_TT_vec <- numeric(nrow(met))
  } 
  # if("APSIM_TT" %in% method) met$APSIM_TT <- 0
  if("APSIM_TT" %in% method) stop("Not implemented yet.", call. = FALSE)
  
  if(nrow(met) != nrow(tmpd))
    warning("Days for each year should be complete")
  
  if(any(table(met$year) < 365))
    stop("Each year should have at least 365 days", call. = FALSE)
  
  ## This first instance will result in calculating TT for every year
  if(inherits(dates, "character")){
    doy1m <- format(as.Date(dates[1], format = dates.format), "%m-%d")
    doynm <- format(as.Date(dates[2], format = dates.format), "%m-%d")
    doy1 <- date2doy(doy1m)
    doyn <- date2doy(doynm)
    if(doy1 > doyn){
      ## We are in the southern hemisphere
      d1 <- as.Date(paste0(doy1m, "-", 2020), format = "%m-%d-%Y")
      dn <- as.Date(paste0(doynm, "-", 2021), format = "%m-%d-%Y")
      doy.seq <- as.numeric(format(seq(from = d1, to = dn, by = "day"), "%j")) 
      ## Writing it in this way will always include 366
      tmpd$index <- ifelse(!is.na(match(met$day, doy.seq)), 1, 0)
    }else{
      doy.seq <- doy1:doyn
      tmpd$index <- ifelse(!is.na(match(met$day, doy.seq)), 1, 0)
    }
  }

  cum.classic.tt <- 0
  cum.heatstress.tt <- 0
  cum.cropheatunit.tt <- 0
  cum.apsim.tt <- 0
  k <- 0

  for(i in 1:nrow(met)){
    
    ## In the Southern hemisphere we start at k = 1
    if(tmpd$index[i] > 0.5 && i == 1) k <- 1
    ## In the Northern hemisphere we should just skip the first days until doy1
    if(tmpd$index[i] < 0.5 && k == 0) next
    
    if(tmpd$index[i] > 0.5 && met$day[i] == doy1) k <- k + 1  
    
    if(tmpd$index[i] > 0.5){
      if("Classic_TT" %in% method){
        mint.m <- max(met$mint[i], base_temp)
        maxt.m <- min(met$maxt[i], max_temp)
        classic.tt <- (maxt.m + mint.m)/2 - base_temp
        classic.tt <- ifelse(classic.tt >= 0, classic.tt, 0)
        cum.classic.tt <- cum.classic.tt + classic.tt
        Classic_TT_vec[i] <- cum.classic.tt
      }
      if("HeatStress_TT" %in% method){
        heatstress.tt <- met$maxt[i] - max_temp
        heatstress.tt <- ifelse(heatstress.tt >= 0, heatstress.tt, 0)
        cum.heatstress.tt <- cum.heatstress.tt + heatstress.tt
        HeatStress_TT_vec[i] <- cum.heatstress.tt
      }
      if("CropHeatUnit_TT" %in% method){
        mint.m <- 1.8 * met$mint[i] - 4.4
        maxt.m <- 3.33 * (met$maxt[i] - 10) - 0.084 * (met$maxt[i] - 10)^2
        cropheatunit.tt <- (maxt.m + mint.m)/2 
        cropheatunit.tt <- ifelse(cropheatunit.tt >= 0, cropheatunit.tt, 0)
        cum.cropheatunit.tt <- cum.cropheatunit.tt + cropheatunit.tt
        CropHeatUnit_TT_vec[i] <- cum.cropheatunit.tt
      }
      if("APSIM_TT" %in% method){
        apsim.tt <- apsim_tt(met$maxt[i], met$mint[i], Tb = base_temp, To = max_temp, cardinal.temps = x_temp, gdd.coord = y_tt)
        cum.apsim.tt <- cum.apsim.tt + apsim.tt
        met$APSIM_TT[i] <- cum.apsim.tt
      }
    }else{
      cum.classic.tt <- 0
      cum.heatstress.tt <- 0
      cum.cropheatunit.tt <- 0
      cum.apsim.tt <- 0
    }
  }
  
  ### Add variables as needed
  met <- add_column_apsim_met(met, tmpd$Dates, "Dates", units = "()")
  
  if("Classic_TT" %in% method){
    met <- add_column_apsim_met(met, Classic_TT_vec, "Classic_TT", units = "(Cd)")
  }
  
  if("HeatStress_TT" %in% method){
    met <- add_column_apsim_met(met, HeatStress_TT_vec, "HeatStress_TT", units = "(Cd)")
  }
  
  if("CropHeatUnit_TT" %in% method){
    met <- add_column_apsim_met(met, CropHeatUnit_TT_vec, "CropHeatUnit_TT", units = "(Cd)")
  }
  
  return(met)
}
  
## APSIM calculation of thermal time
########################################################################################################
########################################################################################################
## Functions to convert temperature to gdd following APSIM's XY interpolation method. 
## There are three options available in this script. Users can modify or add new functions. 
## We used the temp2gdd function in the paper (APSIM-soybean version 7.5).
## We provide two more options as examples (temp3gdd and temp4dd function)
## See supplementary materials, figure S1  for the shape of each function

## function 1 or method 1 (temp2gdd)
temp2gdd <- function(temp, cardinal.temps = c(10,30,40), gdd.coord=c(0,20,0)){
  
  gdd <- numeric(length(temp))
  
  for(i in 1:length(temp)){
    if(temp[i] <= cardinal.temps[2]){
      slp <- c(gdd.coord[2]-gdd.coord[1])/c(cardinal.temps[2]-cardinal.temps[1])    # slope = (20-0)/(30-10)=1
      int <- gdd.coord[1]                                                           # int   = 0
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[1])    
    }else{
      slp <- c(gdd.coord[3]-gdd.coord[2])/c(cardinal.temps[3]-cardinal.temps[2])    # slope = (0-20)/(40-30)=-2
      int <- gdd.coord[2]                                                           # int   = 20
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[2])
    }
  }
  
  gdd <- pmax(gdd,0)
  return(gdd)
}


## function 2 or method 2 (temp3gdd)
temp3gdd <- function(temp, cardinal.temps = c(0,15,30,40), gdd.coord=c(0,5,20,0)){
  
  gdd <- numeric(length(temp))
  
  for(i in 1:length(temp)){
    if(temp[i] <= cardinal.temps[2]){                                                 # temp < 15
      slp <- c(gdd.coord[2]-gdd.coord[1])/c(cardinal.temps[2]-cardinal.temps[1])      # slope = (5-0)/(15-0)=0.333
      int <- gdd.coord[1]                                                             # int = gdd.coord[1] = 0
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[1])                             # if temp = 15, gdd=5
    }
    if(temp[i] > cardinal.temps[2] && temp[i] <= cardinal.temps[3]){                  # temp < 30
      slp <- c(gdd.coord[3]-gdd.coord[2])/c(cardinal.temps[3]-cardinal.temps[2])      # slope = (20-5)/(30-15)=1
      int <- gdd.coord[2]                                                             # int = gdd.coord[2] = 5
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[2])                             # if temp = 30, gdd=20
    }
    if(temp[i] > cardinal.temps[3] && temp[i] <= cardinal.temps[4]){                  # temp < 40
      slp <- c(gdd.coord[4]-gdd.coord[3])/c(cardinal.temps[4]-cardinal.temps[3])      # slope = (0-20)/(40-30)=-2
      int <-  gdd.coord[3]                                                            # int = gd.coord[3]=20
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[3])                             # if temp = 40, gdd=0
      #}else{
      # gdd[i] <- 0
    }
  }
  gdd <- pmax(gdd,0)
  return(gdd)
}


## function 3 or method 3 (temp4gdd)
temp4gdd <- function(temp, cardinal.temps = c(7,28,35,45), gdd.coord=c(0,21,21,0)){
  
  gdd <- numeric(length(temp))
  
  for(i in 1:length(temp)){
    if(temp[i] <= cardinal.temps[2]){                                                 # temp < 28
      slp <- c(gdd.coord[2]-gdd.coord[1])/c(cardinal.temps[2]-cardinal.temps[1])      # slope = (21-0)/(28-7)=1
      int <- gdd.coord[1]                                                             # int = gdd.coord[1] = 0
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[1])                             # if temp = 28, gdd=21
    }
    if(temp[i] > cardinal.temps[2] && temp[i] <= cardinal.temps[3]){                  # temp < 35
      slp <- c(gdd.coord[3]-gdd.coord[2])/c(cardinal.temps[3]-cardinal.temps[2])      # slope = (21-21)/(35-28)=0
      int <- gdd.coord[2]                                                             # int = gdd.coord[2] = 21
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[2])                             # if temp = 35, gdd=21
    }
    if(temp[i] > cardinal.temps[3] && temp[i] <= cardinal.temps[4]){                  # temp < 45
      slp <- c(gdd.coord[4]-gdd.coord[3])/c(cardinal.temps[4]-cardinal.temps[3])      # slope = (0-21)/(45-35)=-2.1
      int <-  gdd.coord[3]                                                            # int = gd.coord[3]=21
      gdd[i] <- int + slp * (temp[i] - cardinal.temps[3])                             # if temp = 45, gdd=0
    }
  }
  gdd <- pmax(gdd,0)
  return(gdd)
}


########################################################################################################
########################################################################################################
## This function calculates daily gdd using the 3-hour interval approach from APSIM
## Min and max daily temperature, Tb, To, and interpolation method are the inputs (see above for options).
## We used the "temp2gdd" interpolation method and APSIM's default Tb and To values (Table 1).   

apsim_tt <- function(maxt, mint, Tb=10, To=30, cardinal.temps, gdd.coord){
  
  h2c <- function(hr){
    ans <- 0.92105 + 0.114 * hr - 0.0703 * (hr^2) + 0.0053*(hr^3)   # apsim equation  
    ans
  } 
  
  method <- "temp2gdd"
  if(missing(cardinal.temps) && missing(gdd.coord)){
    method <- "temp2gdd"
  }else{
    if(length(cardinal.temps) == 3) method <- "temp2gdd"
    if(length(cardinal.temps) == 4) method <- "temp3gdd"
    if(length(cardinal.temps) == 5) 
      stop("Method not implemented yet", call. = FALSE)
  } 

  if(method == "temp2gdd"){
    if(mint < Tb || maxt > To){
      temp1 <- mint + h2c(1)*(maxt - mint)
      temp2 <- mint + h2c(2)*(maxt - mint)
      temp3 <- mint + h2c(3)*(maxt - mint)
      temp4 <- mint + h2c(4)*(maxt - mint)
      temp5 <- mint + h2c(5)*(maxt - mint)
      temp6 <- mint + h2c(6)*(maxt - mint)
      temp7 <- mint + h2c(7)*(maxt - mint)
      temp8 <- mint + h2c(8)*(maxt - mint)
      
      if(missing(cardinal.temps) && missing(gdd.coord)){
        gdd <- mean(temp2gdd(c(temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8)))    
      }else{
        if(length(cardinal.temps) != 3 || length(gdd.coord) != 3)
          stop("Length of cardinal.temps and/or gdd.coord is not equal to 3", call. = FALSE)
        gdd <- mean(temp2gdd(c(temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8),
                             cardinal.temps = cardinal.temps, gdd.coord = gdd.coord))    
      }
      
    }else{
      gdd <- temp2gdd((maxt + mint)*0.5)                                                 
    } 
  }
  
  if(method == "temp3gdd"){
    if(mint < Tb || maxt > To){
      temp1 <- mint + h2c(1)*(maxt - mint)
      temp2 <- mint + h2c(2)*(maxt - mint)
      temp3 <- mint + h2c(3)*(maxt - mint)
      temp4 <- mint + h2c(4)*(maxt - mint)
      temp5 <- mint + h2c(5)*(maxt - mint)
      temp6 <- mint + h2c(6)*(maxt - mint)
      temp7 <- mint + h2c(7)*(maxt - mint)
      temp8 <- mint + h2c(8)*(maxt - mint)
      
      if(missing(cardinal.temps) && missing(gdd.coord)){
        gdd <- mean(temp3gdd(c(temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8)))   
      }else{
        if(length(cardinal.temps) != 4 || length(gdd.coord) != 4)
          stop("Length of cardinal.temps and/or gdd.coord is not equal to 4", call. = FALSE)
        gdd <- mean(temp3gdd(c(temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8),
                             cardinal.temps = cardinal.temps, gdd.coord = gdd.coord))    
      }
    }else{
      gdd <- temp3gdd((maxt + mint)*0.5)                                                 
    }
  }
  
  return(gdd)
}

## Function to include the photoperiod in an APSIM met file
pp_apsim_met <- function(metfile, lat, sun_angle=0){
  
  if(!inherits(metfile, "met"))
    stop("Object 'metfile' should be of class 'met'", call. = FALSE)
  if(missing(lat)){
    lat0 <-  attr(metfile, "latitude") # read latitude from the Measured file
    lat <- suppressWarnings(as.numeric(strsplit(lat0, "=")[[1]][2]))
    if(is.na(lat)){
      lat00 <- strsplit(lat0, "=")[[1]][2]
      lat0 <- suppressWarnings(as.numeric(strsplit(lat00, " ")[[1]]))
      lat <- lat0[!is.na(lat0)]
    }    
  }
  day <-  metfile$day                  # read day from the Met file 
  
  aeqnox <- 79.25              
  pi     <- 3.14159265359
  dg2rdn <- (2.0 * pi) /360.0    
  decsol <- 23.45116 * dg2rdn  
  dy2rdn <- (2.0 * pi) /365.25   
  rdn2hr <- 24.0/(2.0 * pi)      
  
  sun_alt <- sun_angle * dg2rdn
  dec     <- decsol * sin (dy2rdn * (day - aeqnox))
  latrn   <- lat * dg2rdn
  slsd    <- sin(latrn) * sin(dec)
  clcd    <- cos(latrn) * cos(dec)
  altmn   <- asin(pmin(pmax(slsd - clcd, -1.0), 1.0))
  altmx   <- asin(pmin(pmax(slsd + clcd, -1.0), 1.0))
  alt     <- pmin(pmax(sun_alt, altmn), altmx)
  
  coshra1 <- (sin (alt) - slsd) /clcd
  coshra  <- pmin(pmax(coshra1, -1.0), 1.0)
  hrangl  <- acos (coshra)
  PP      <- hrangl * rdn2hr * 2.0          
  ##metfile$photoperiod <- PP
  metfile <- add_column_apsim_met(metfile, PP, "photoperiod", units = "(hour)")
  return(metfile)
}


#' @title Plot method for object of class \sQuote{met} 
#' @name plot.met
#' @description Some plots are similar to APSIM, others are different
#' and more useful in some respects
#' @param x object of class \sQuote{met}
#' @param ... additional arguments. None used at the moment.
#' @param years optional argument to subset years
#' @param met.var optional argument to choose a certain variable. By default,
#' temperature (min and max) is displayed
#' @param plot.type type of plot, default is \sQuote{ts} or time-series. 
#' The options \sQuote{area} and \sQuote{col} are only available when summary = TRUE.
#' Option \sQuote{density} produces a simple plot. Option \sQuote{anomaly} ignores
#' argument cumulative is treated as TRUE regardless.
#' @param cumulative default is FALSE. Especially useful for \sQuote{rain}.
#' @param facet whether to display the years in in different panels (facets). Not implemented yet.
#' @param climatology logical (default FALSE). Whether to display the \sQuote{climatology}
#' which would be the average of the data. 
#' Ideally, there are at least 20 years in the \sQuote{met} object.
#' @param summary whether to plot \sQuote{summary} data. (default FALSE).
#' @param by whether to perform summary by \sQuote{year} or \sQuote{month}.
#' @export
#' @examples 
#' \donttest{
#' ## Read in and plot a met file
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
#' plot(ames, years = 2012:2015)
#' ## Perhaps more informative
#' plot(ames, years = 2012:2015, cumulative = TRUE)
#' ## for rain
#' plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE)
#' plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE, climatology = TRUE)
#' plot(ames, met.var = "rain", years = 2012:2015, plot.type = "anomaly")
#' ## It is possible to add ggplot elements
#' library(ggplot2)
#' p1 <- plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE)
#' p1 + ggtitle("Cumulative rain for 2012-2015")
#' }
#' 
plot.met <- function(x, ..., years, met.var, 
                     plot.type = c("ts", "area", "col", "density", "anomaly"), 
                     cumulative = FALSE,
                     facet = FALSE,
                     climatology = FALSE,
                     summary = FALSE,
                     by = c("year", "month")){
  
  if(!requireNamespace("ggplot2", quietly = TRUE)){
    warning("ggplot2 is required for this plotting function")
    return(NULL)
  }
  
  plot.type <- match.arg(plot.type)
  
  if(plot.type %in% c("area", "col") && !summary)
    stop("plot.type = area and plot.type = col are only available when summary = TRUE", call. = FALSE)
  
  ## Global variables?
  day <- NULL; cum.maxt <- NULL; Years <- NULL; cum.mint <- NULL;
  value <- NULL; temperature <- NULL; maxt <- NULL; mint <- NULL;
  cum.met.var <- NULL; year <- NULL; Year <- NULL; q25 <- NULL;
  q75 <- NULL; tav <- NULL; tav1 <- NULL; tav2 <- NULL; 
  InitialValues <- NULL 
  ## Calculate climatology before subsetting years
  if(climatology || plot.type == "anomaly"){
    maxt.climatology <- stats::aggregate(maxt ~ day, data = x, FUN = mean)
    maxt.climatology.q25 <- stats::aggregate(maxt ~ day, data = x, FUN = stats::quantile, probs = 0.25)
    maxt.climatology.q75 <- stats::aggregate(maxt ~ day, data = x, FUN = stats::quantile, probs = 0.75)
    mint.climatology <- stats::aggregate(mint ~ day, data = x, FUN = mean)
    mint.climatology.q25 <- stats::aggregate(mint ~ day, data = x, FUN = stats::quantile, probs = 0.25)
    mint.climatology.q75 <- stats::aggregate(mint ~ day, data = x, FUN = stats::quantile, probs = 0.75)
    rain.climatology <- stats::aggregate(rain ~ day, data = x, FUN = mean)
    rain.climatology.q25 <- stats::aggregate(rain ~ day, data = x, FUN = stats::quantile, probs = 0.25)
    rain.climatology.q75 <- stats::aggregate(rain ~ day, data = x, FUN = stats::quantile, probs = 0.75)
    radn.climatology <- stats::aggregate(radn ~ day, data = x, FUN = mean)
    radn.climatology.q25 <- stats::aggregate(radn ~ day, data = x, FUN = stats::quantile, probs = 0.25)
    radn.climatology.q75 <- stats::aggregate(radn ~ day, data = x, FUN = stats::quantile, probs = 0.75)
    met.var.climatology <- cbind(maxt.climatology, 
                                 mint.climatology[,"mint", drop = FALSE], 
                                 rain.climatology[,"rain", drop = FALSE], 
                                 radn.climatology[,"radn", drop = FALSE])
    if(any(grepl("vp", names(x), fixed = TRUE))){
      vp.climatology <- stats::aggregate(vp ~ day, data = x, FUN = mean)
      met.var.climatology <- cbind(met.var.climatology, vp.climatology[, "vp", drop = FALSE])
    }
    if(any(grepl("windspeed", names(x), fixed = TRUE))){
      windspeed.climatology <- stats::aggregate(windspeed ~ day, data = x, FUN = mean)
      met.var.climatology <- cbind(met.var.climatology, windspeed.climatology[, "windspeed", drop = FALSE])
    }
    if(any(grepl("Classic_TT", names(x), fixed = TRUE))){
      classic_tt.climatology <- stats::aggregate(Classic_TT ~ day, data = x, FUN = mean)
      met.var.climatology <- cbind(met.var.climatology, classic_tt.climatology[, "Classic_TT", drop = FALSE])
    }
  }

  if(!missing(years)){
    if(max(years) > max(x$year) || min(years) < min(x$year))
      stop("Selected year is not within the range of the data", call. = FALSE)    
  }

  ## Need to make a copy for at least this case
  if(plot.type == "density" && climatology) x2 <- x
  if(plot.type == "anomaly" && summary) x2 <- x

  if(!missing(years)){
    x <- x[x$year %in% years,]
    ## Is this better? x <- subset(x, year %in% years) ## programmatically?
  }
  
  x <- add_column_apsim_met(x, value = as.factor(x$year), name = "Years", units = "()")
  
  if(isFALSE(summary)){
    if(!missing(met.var)){
      if(!met.var %in% names(x)){
        cat("Variables in met:", names(x), "\n")
        stop("met.var was not found in the names of the 'met' object", call. = FALSE)
      }
      if(met.var %in% c("day", "year"))
        stop("year or day cannot be used as a 'met.var'", call. = FALSE)
    }    
  }

  if(isFALSE(summary)){
    if(plot.type == "ts"){
      if(missing(met.var)){
        if(cumulative){
          dat <- NULL
          for(i in seq_along(sort(unique(x$year)))){
            yr <- sort(unique(x$year))[i]
            x.tmp <- x[x$year == yr,]
            x.ag.maxt <- cumsum(x.tmp$maxt)
            x.ag.mint <- cumsum(x.tmp$mint)
            dat <- rbind(dat, data.frame(year = yr, day = 1:nrow(x.tmp), cum.maxt = x.ag.maxt, cum.mint = x.ag.mint))
          }
          
          dat$Years <- as.factor(dat$year)
          
          if(climatology){
            maxt.climatology$cum.maxt <- cumsum(maxt.climatology$maxt)
            maxt.climatology$climatology <- "climatology"
            mint.climatology$cum.mint <- cumsum(mint.climatology$mint)
            mint.climatology$climatology <- "climatology"
            gp1 <- ggplot2::ggplot() + 
                            ggplot2::geom_line(data = dat, ggplot2::aes(day, cum.maxt, color = Years)) +
                            ggplot2::geom_line(data = dat, ggplot2::aes(day, cum.mint, color = Years), linetype = 2) +
                            ggplot2::geom_line(data = maxt.climatology, ggplot2::aes(day, cum.maxt, linetype = climatology), color = "black") + 
                            ggplot2::geom_line(data = mint.climatology, ggplot2::aes(day, cum.mint, linetype = climatology), color = "black", linetype = 2) +
                            ggplot2::scale_linetype_manual(name = NULL, values = 1) + 
                            ggplot2::geom_hline(yintercept = 0, linetype = 3) + 
                            ggplot2::ylab("Cumulative temperature (degree C)") 
          }else{
            dat1 <- data.frame(day = dat$day, temperature = "maxt", value = dat$cum.maxt, Years = dat$Years)
            dat2 <- data.frame(day = dat$day, temperature = "mint", value = dat$cum.mint, Years = dat$Years)
            dat <- rbind(dat1, dat2)
            gp1 <- ggplot2::ggplot(data = dat) + 
              ggplot2::geom_line(ggplot2::aes(day, value, color = Years, linetype = temperature)) +
              ggplot2::geom_line(ggplot2::aes(day, value, color = Years, linetype = temperature)) +
              ggplot2::scale_linetype_manual(name = NULL, values = c(1, 2)) + 
              ggplot2::geom_hline(yintercept = 0, linetype = 3) + 
              ggplot2::ylab("Cumulative temperature (degree C)") 
          }
          print(gp1)        
        }else{
          if(climatology){
            maxt.climatology$climatology <- "climatology"
            mint.climatology$climatology <- "climatology"
            x <- add_column_apsim_met(x, value = as.factor(x$year), name = "Years", units = "()")
            x <- as.data.frame(x)
            gp1 <- ggplot2::ggplot() + 
                            ggplot2::geom_line(data = x, ggplot2::aes(day, maxt, color = Years)) +
                            ggplot2::geom_line(data = x, ggplot2::aes(day, mint, color = Years), linetype = 2) +
                            ggplot2::geom_line(data = maxt.climatology, ggplot2::aes(day, maxt, linetype = climatology), color = "black") +
                            ggplot2::geom_line(data = mint.climatology, ggplot2::aes(day, mint, linetype = climatology), color = "black", linetype = 2) +
                            ggplot2::scale_linetype_manual(name = NULL, values = 1) + 
                            ggplot2::geom_hline(yintercept = 0, linetype = 3) + 
                            ggplot2::ylab("Temperature (degree C)")                        
          }else{
            x <- as.data.frame(x)
            gp1 <- ggplot2::ggplot(data = x) + 
              ggplot2::geom_line(ggplot2::aes(day, maxt, color = Years, linetype = "maxt")) +
              ggplot2::geom_line(ggplot2::aes(day, mint, color = Years, linetype = "mint")) +
              ggplot2::geom_hline(yintercept = 0, linetype = 3) + 
              ggplot2::scale_linetype_manual(name = NULL, values = c(1, 2)) + 
              ggplot2::ylab("Temperature (degree C)")            
          }
          print(gp1)        
        }
      }else{
        if(met.var %in% c("rain", "radn", "vp", "rh", "maxt", "mint", "windspeed", 
                          "Classic_TT", "HeatStress_TT", "CropHeatUnit_TT", "photoperiod")){
          
          met.var.units <- switch(met.var,
                                  rain = "(mm)",
                                  radn = "(MJ/m2)",
                                  rh = "(%)",
                                  maxt = "(degrees C)",
                                  mint = "(degrees C)",
                                  vp = "(hPa)",
                                  windspeed = "(m/s)",
                                  Classic_TT = "(Cd)",
                                  HeatStress_TT = "(Cd)",
                                  CropHeatUnit_TT = "(Cd)",
                                  photoperiod = "(hours)")

          if(climatology){
            met.var.clima <- met.var.climatology[ ,c("day", met.var)]
            met.var.clima$climatology <- "climatology"            
          }

          if(cumulative){
            ylabel <- paste("Cumulative ", met.var, met.var.units)
            dat <- NULL
            for(i in seq_along(sort(unique(x$year)))){
              yr <- sort(unique(x$year))[i]
              x.tmp <- x[x$year == yr,]
              x.ag.met.var <- cumsum(x.tmp[[met.var]])
              dat <- rbind(dat, data.frame(year = yr, day = 1:nrow(x.tmp), cum.met.var = x.ag.met.var))
            }
            
            dat$Years <- as.factor(dat$year)
            
            if(climatology){
              met.var.clima$cum.met.var <- cumsum(met.var.clima[[met.var]])
              
              gp1 <- ggplot2::ggplot() + 
                ggplot2::geom_line(data = dat, ggplot2::aes(day, cum.met.var, color = Years)) +
                ggplot2::geom_line(data = met.var.clima, ggplot2::aes(day, cum.met.var, linetype = climatology), color = "black") +
                ggplot2::scale_linetype_manual(name = NULL, values = 1) + 
                ggplot2::ylab(ylabel)                            
            }else{
              gp1 <- ggplot2::ggplot(data = dat) + 
                ggplot2::geom_line(ggplot2::aes(day, cum.met.var, color = Years)) +
                ggplot2::ylab(ylabel)              
            }
            print(gp1)        
          }else{
            ylabel <- paste(met.var, met.var.units)
            if(climatology){
              x <- as.data.frame(x)
              gp1 <- ggplot2::ggplot() + 
                ggplot2::geom_point(data = x, ggplot2::aes(day, 
                                                 y = eval(parse(text = eval(met.var))),
                                                 color = Years)) +
                ggplot2::geom_line(data = met.var.clima, ggplot2::aes(day, 
                                                 y = eval(parse(text = eval(met.var))),
                                                 linetype = climatology)) +
                ggplot2::scale_linetype_manual(name = NULL, values = 1) + 
                ggplot2::ylab(ylabel)                  
            }else{
              x <- as.data.frame(x)
              gp1 <- ggplot2::ggplot(data = x) + 
                              ggplot2::geom_point(ggplot2::aes(day, 
                                                 y = eval(parse(text = eval(met.var))),
                                                 color = Years)) +
                              ggplot2::ylab(ylabel)              
            }
            print(gp1)        
          }
        }
      }
    }   
    ### Density could work with summary FALSE
    if(plot.type == "density" && isFALSE(climatology) && isFALSE(cumulative)){
      x <- as.data.frame(x)
      if(missing(met.var)){
        gp1 <- ggplot2::ggplot(data = x, 
                        ggplot2::aes(x = maxt, color = Years)) + 
          ggplot2::geom_density() + 
          ggplot2::xlim(c(min(x$maxt) * 0.25, max(x$maxt) * 1.5)) + 
          ggplot2::xlab("Maximum temperature (C)")
        print(gp1)    
      }else{
        gp1 <- ggplot2::ggplot(data = x) + 
          ggplot2::geom_density(ggplot2::aes(x = eval(parse(text = eval(met.var))), color = Years)) +
          ggplot2::xlim(c(min(x[[met.var]]) * 0.25, max(x[[met.var]]) * 1.5)) + 
          ggplot2::xlab(met.var)
        print(gp1)    
      }
    }
    ### If climatology is true
    if(plot.type == "density" && climatology && isFALSE(cumulative)){
      x <- as.data.frame(x)
      if(missing(met.var)){
        gp1 <- ggplot2::ggplot() + 
          ggplot2::geom_density(data = x, ggplot2::aes(x = maxt, color = Years)) + 
          ggplot2::geom_density(ggplot2::aes(x = maxt.climatology$maxt), linewidth = 2) + 
          ggplot2::xlim(c(min(x$maxt) * 0.25, max(x$maxt) * 1.5)) + 
          ggplot2::xlab("Maximum temperature (C)")
        print(gp1)    
      }else{
        gp1 <- ggplot2::ggplot() + 
          ggplot2::geom_density(data = x, ggplot2::aes(x = eval(parse(text = eval(met.var))), color = Years)) + 
          ggplot2::geom_density(ggplot2::aes(x = met.var.climatology[[met.var]]), linewidth = 2) + 
          ggplot2::xlim(c(min(met.var.climatology[[met.var]]) * 0.25, max(met.var.climatology[[met.var]]) * 1.5)) + 
          ggplot2::xlab(met.var)
        print(gp1)    
      }
    }
    
    #### Anomaly ----
    if(plot.type == "anomaly"){
      if(missing(met.var)) met.var <- "maxt"
      
      met.var.units <- switch(met.var,
                              rain = "(mm)",
                              radn = "(MJ/m2)",
                              rh = "(%)",
                              maxt = "(degrees C)",
                              mint = "(degrees C)",
                              vp = "(hPa)",
                              windspeed = "(m/s)",
                              Classic_TT = "(Cd)",
                              HeatStress_TT = "(Cd)",
                              CropHeatUnit_TT = "(Cd)",
                              photoperiod = "(hours)")
      
      xdat <- NULL
      for(i in seq_along(sort(unique(x$year)))){
        yr <- sort(unique(x$year))[i]
        x.tmp <- x[x$year == yr,]
        if(nrow(x.tmp) < 365){
          message(paste("year:", unique(x$year)[i], "was removed as it is incomplete"))
          next ## Skip incomplete years
        } 
        if(nrow(x.tmp) == 366) x.tmp <- x.tmp[1:365,]
        x.dag.maxt <- cumsum(x.tmp$maxt) - cumsum(maxt.climatology[1:365, "maxt"])
        x.dag.mint <- cumsum(x.tmp$mint) - cumsum(mint.climatology[1:365, "mint"])
        x.dag.rain <- cumsum(x.tmp$rain) - cumsum(rain.climatology[1:365, "rain"])
        x.dag.radn <- cumsum(x.tmp$radn) - cumsum(radn.climatology[1:365, "radn"])
        if(met.var == "Classic_TT"){
          x.dag.classic_tt <- x.tmp$Classic_TT - classic_tt.climatology[1:365, "Classic_TT"]  
        }else{
          x.dag.classic_tt <- NA
        }
        xdat <- rbind(xdat, data.frame(year = yr, day = 1:nrow(x.tmp), 
                                       maxt = x.dag.maxt, 
                                       mint = x.dag.mint,
                                       rain = x.dag.rain,
                                       radn = x.dag.radn,
                                       Classic_TT = x.dag.classic_tt))
      }
      xdat$year <- as.factor(xdat$year)
      ## browser()
      xdats <- subset(xdat, select = c("year", "day", met.var))
      names(xdats) <- c("year", "day", "met.var")
      if(isFALSE(climatology)){
        gp1 <- ggplot2::ggplot() + 
          ggplot2::geom_line(data = xdats, ggplot2::aes(x = day, y = met.var, color = year)) +
          ggplot2::ylab(paste("Cumulative anomaly for", met.var, met.var.units)) + 
          ggplot2::geom_hline(yintercept = 0)        
      }else{
        met.var.sd <- aggregate(met.var ~ day, data = xdats, FUN = sd)
        met.var.mean <- aggregate(met.var ~ day, data = xdats, FUN = mean)
        dmat <- data.frame(day = 1:365, q75 = met.var.sd$met.var + met.var.mean$met.var, q25 = met.var.mean$met.var - met.var.sd$met.var)
        gp1 <- ggplot2::ggplot() + 
          ggplot2::geom_ribbon(data = dmat, ggplot2::aes(x = day, ymin = q25, ymax = q75, fill = "25-75"), fill = "deepskyblue1", alpha = 0.2) +
          ggplot2::geom_line(data = xdats, ggplot2::aes(x = day, y = met.var, color = year)) +
          ggplot2::ylab(paste("Cumulative anomaly for", met.var, met.var.units)) + 
          ggplot2::geom_hline(yintercept = 0)
      }
      plot(gp1)
    }
    ### The cumulative does not make much sense
    ### If summary is FALSE at least
    if(plot.type == "density" && cumulative)
      stop("This is plot.type is not available", call. = FALSE)
  }else{
    
    if(missing(met.var)){
      met.var <- c("avg_maxt", "avg_mint")
    }
    
    if(is.null(list(...)$compute.frost)){
      valid.met.vars <- c("high_maxt", "high_mint", "avg_maxt", "avg_mint", "low_maxt", "low_mint", "rain_sum", "radn_sum", "radn_avg", "high_classic_tt", "avg_classic_tt") 
    }else{
      valid.met.vars <- c("high_maxt", "high_mint", "avg_maxt", "avg_mint", "low_maxt", "low_mint", "rain_sum", "radn_sum", "radn_avg", "first_half_frost", "second_half_frost", "frost_free_period", "frost_days", "high_classic_tt", "avg_classic_tt") 
    }
    sel.met.var <- sapply(met.var, function(x) grep(x, valid.met.vars)) 
    if(length(sel.met.var) == 0 || is.list(sel.met.var)){
      cat("Valid met.vars:", valid.met.vars, "\n")
      stop("'met.var' is not a valid meteorological variable", call. = FALSE)
    }else{
      met.var <- valid.met.vars[sel.met.var] 
    }
    
    if(any(met.var %in% c("high_maxt", "high_mint", "avg_maxt", "avg_mint", "low_maxt", "low_mint")))
      ylabs <- "Temperature (degree C)"
    
    if(plot.type != "anomaly"){
      if(any(grepl("rain_sum", met.var)) && length(met.var) > 1){
        cat("Selected met variables:", met.var, "\n")
        stop("If 'rain_sum' is chosen, it should be the only met variable", call. = FALSE)      
      }
    }

    if(any(met.var %in% c("rain_sum")))
      ylabs <- "Rainfall (mm)"
    
    if(any(met.var %in% c("radn_sum", "radn_avg")))
      ylabs <- "Radiation (MJ/m2)"
    
    if(any(grepl("frost", met.var)))
      ylabs <- "Days"
      
    if(plot.type == "density" && climatology){
      stmp2 <- summary(x2, ...) 
      tmp2 <- NULL
      for(i in seq_along(met.var)){
        dat2 <- data.frame(year = stmp2$year, met.var = met.var[i], value = stmp2[[met.var[i]]])
        tmp2 <- rbind(tmp2, dat2)
      }
    }
      
    stmp <- summary(x, ...)      
    tmp <- NULL
    for(i in seq_along(met.var)){
      dat <- data.frame(year = stmp$year, met.var = met.var[i], value = stmp[[met.var[i]]])
      tmp <- rbind(tmp, dat)
    }

    if(plot.type == "ts" && !climatology){
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = year, y = value, color = met.var)) + 
        ggplot2::geom_point() + 
        ggplot2::geom_line() + 
        ggplot2::ylab(ylabs)
      print(gp1)      
    }
    
    if(plot.type == "ts" && climatology){
      y.ints <- aggregate(value ~ met.var, data = tmp, FUN = mean, na.rm = TRUE)$value
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = year, y = value, color = met.var)) + 
        ggplot2::geom_point() + 
        ggplot2::geom_line() +
        ggplot2::geom_hline(yintercept = y.ints, linetype = 2) + 
        ggplot2::ylab(ylabs)
      print(gp1)      
    }
    
    if(plot.type == "area"){
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = year, y = value, fill = met.var)) + 
        ggplot2::geom_area() + 
        ggplot2::ylab(ylabs)
      print(gp1)      
    }
    
    if(plot.type == "col"){
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = year, y = value, fill = met.var)) + 
        ggplot2::geom_col() + 
        ggplot2::ylab(ylabs)
      print(gp1)      
    }
    
    if(!climatology){
      if(missing(years)){
        if(plot.type == "density"){
          gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = value)) + 
            ggplot2::geom_density() + 
            ggplot2::xlim(c(min(tmp$value) * 0.5, max(tmp$value) * 1.5)) + 
            ggplot2::xlab(ylabs)
          print(gp1)      
        }      
      }else{
        if(plot.type == "density"){
          tmp$Year <- as.factor(tmp$year)
          gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = value)) + 
            ggplot2::geom_density() +
            ggplot2::geom_vline(ggplot2::aes(xintercept = value, color = Year)) + 
            ggplot2::xlim(c(min(tmp$value) * 0.5, max(tmp$value) * 1.5)) + 
            ggplot2::xlab(ylabs)
          print(gp1)      
        }
      }      
    }else{
      if(missing(years)){
        if(plot.type == "density"){
          gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = value)) + 
            ggplot2::geom_density() + 
            ggplot2::xlim(c(min(tmp$value) * 0.5, max(tmp$value) * 1.5)) + 
            ggplot2::xlab(ylabs)
          print(gp1)      
        }      
      }else{
        if(plot.type == "density"){
          tmp2$Year <- as.factor(tmp2$year)
          tmp$Year <- as.factor(tmp$year)
          gp1 <- ggplot2::ggplot(data = tmp2, ggplot2::aes(x = value)) + 
            ggplot2::geom_density() +
            ggplot2::geom_vline(data = tmp, ggplot2::aes(xintercept = value, color = Year)) + 
            ggplot2::xlim(c(min(tmp2$value) * 0.5, max(tmp2$value) * 1.5)) + 
            ggplot2::xlab(ylabs)
          print(gp1)      
        }
      }
    }
    
    if(plot.type == "anomaly"){
      if(missing(met.var)) 
        stop("'met.var' is missing with no default", call. = FALSE)
      
      ax2 <- summary(x2, anomaly = met.var)

      if(length(grep("anomaly", names(ax2))) == 1){
        if(missing(years)){
          gav <- grep("anomaly", names(ax2), value = TRUE)
          ax2$tav <- ax2[[gav]]
          gp1 <- ggplot2::ggplot(data = ax2, ggplot2::aes(x = tav, y = 0)) + 
            ggplot2::geom_point() + 
            ggplot2::geom_vline(xintercept = 0) + 
            ggplot2::xlab(paste("Anomaly", met.var, " (%)")) +
            ggplot2::ylab("") + 
            ggplot2::scale_y_continuous(breaks = NULL)
        }else{
          gav <- grep("anomaly", names(ax2), value = TRUE)
          ax2$tav <- ax2[[gav]]
          sax2y <- subset(ax2, year %in% years)
          sax2y$Year <- as.factor(sax2y$year)
          gp1 <- ggplot2::ggplot() + 
            ggplot2::geom_point(data = ax2, ggplot2::aes(x = tav, y = 0)) + 
            ggplot2::geom_point(data = sax2y, ggplot2::aes(x = tav, y = 0, color = Year), size = 4) + 
            ggplot2::geom_vline(xintercept = 0) + 
            ggplot2::xlab(paste("Anomaly", met.var, " (%)")) + 
            ggplot2::ylab("") +
            ggplot2::scale_y_continuous(breaks = NULL)
        }
      }
      
      if(length(grep("anomaly", names(ax2))) > 1){
        
        if(length(grep("anomaly", names(ax2))) > 2){
          cat("Variables:", grep("anomaly", names(ax2), value = TRUE), "\n")
          stop("Select 'met.var' so that there are only two variables", call. = FALSE)
        }
          
        #### Select only the variables needed
        tavs <- grep("anomaly", names(ax2), value = TRUE)
        ax2$tav1 <- ax2[[tavs[1]]]
        ax2$tav2 <- ax2[[tavs[2]]]
        
        if(missing(years)){
          gp1 <- ggplot2::ggplot(data = ax2, ggplot2::aes(x = tav1, y = tav2)) + 
            ggplot2::geom_point() + 
            ggplot2::geom_hline(yintercept = 0) + 
            ggplot2::geom_vline(xintercept = 0) + 
            ggplot2::xlab(paste("Anomaly", gsub("anomaly_", "", tavs[1]), " (%)")) + 
            ggplot2::ylab(paste("Anomaly", gsub("anomaly_", "", tavs[2]), " (%)"))
        }else{
          sax2y <- subset(ax2, year %in% years)
          sax2y$Year <- as.factor(sax2y$year)
          gp1 <- ggplot2::ggplot() + 
            ggplot2::geom_point(data = ax2, ggplot2::aes(x = tav1, y = tav2)) + 
            ggplot2::geom_point(data = sax2y, ggplot2::aes(x = tav1, y = tav2, color = Year), size = 4) +
            ggplot2::geom_hline(yintercept = 0) + 
            ggplot2::geom_vline(xintercept = 0) + 
            ggplot2::xlab(paste("Anomaly", gsub("anomaly_", "", tavs[1]), " (%)")) + 
            ggplot2::ylab(paste("Anomaly", gsub("anomaly_", "", tavs[2]), " (%)"))
        }
      }
      print(gp1)
    }
  }
  invisible(gp1)
}

#' 
#' @title Add a column to an object of class \sQuote{met}
#' @rdname add_column_apsim_met
#' @description The usual way of adding a column to a data frame might
#' not work for an object of class \sQuote{met}, so this method is recommended
#' @param met object of class \sQuote{met}
#' @param value vector of the appropriate length
#' @param name optional name if the vector value is unnamed
#' @param units units for the new column (required)
#' @return an object of class \sQuote{met} with the additional column
#' @export
#' @examples
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
#' 
#' ## The recommended method is
#' val <- abs(rnorm(nrow(ames), 10))
#' ames <- add_column_apsim_met(ames, value = val, name = "vp", units = "(hPa)")
#' 
#' ## This is also possible
#' vp <- data.frame(vp = abs(rnorm(nrow(ames), 10)))
#' attr(vp, "units") <- "(hPa)"
#' ames$vp <- vp$vp
#' 
#' ## or
#' ames[["vp"]] <- vp
#' 
#' ## This is needed to ensure that units and related attributes are also removed
#' ames <- remove_column_apsim_met(ames, "vp")
#' ## However, ames$vp <- NULL will also work
#' }
#' 
#' 
add_column_apsim_met <- function(met, value, name, units){
  
  if(!inherits(met, "met"))
    stop("Object should be of class met.", call. = FALSE)
  
  if(missing(name)){
    name <- names(value)
    if(is.null(name) && is.null(attr(value, 'name'))){
      stop("If 'name' is not provided, 'value' should have a 'name' attribute.", call. = FALSE)
    }
  }
  
  if(missing(units))
    stop("argument 'units' is required for this function", call. = FALSE)
  
  units <- as.character(units)
  
  ## Need to use `[[<-.data.frame` to avoid issues
  oatt <- attributes(met)
  metd <- as.data.frame(unclass(met))
  metd[[name]] <- value
  
  tmp.units <- attr(met, "units")
  new.units <- c(tmp.units, units)   

  attr(metd, "filename") <- oatt$filename
  attr(metd, "site") <- oatt$site
  attr(metd, "latitude") <- oatt$latitude
  attr(metd, "longitude") <- oatt$longitude
  attr(metd, "tav") <- oatt$tav
  attr(metd, "amp") <- oatt$amp
  attr(metd, "colnames") <- names(metd)
  attr(metd, "units") <- new.units
  attr(metd, "constants") <- oatt$constants
  attr(metd, "comments") <- oatt$comments

  met <- structure(metd, class = c("met", "data.frame"))

  return(met)
}

#' @rdname add_column_apsim_met
#' @param x object of class \sQuote{met}
#' @param name name of the variable to be added or modified
#' @param value value for the data.frame. It could be an integer, double or vector of length equal to the number of rows in x.
#' @export
`$<-.met` <- function(x, name, value){
  
  if(is.null(value) && !(name %in% names(x)))
    stop("Trying to remove a column which is not present in object of class 'met'", call. = FALSE)
  
  if(is.null(value) && name %in% names(x)){
    ## The thing here is that I also need to remove units and column names
    wn2r <- which(names(x) == name) ## which name to remove
    x.att <- attributes(x)
    xd <- as.data.frame(x) ## convert to data.frame first
    xd[[name]] <- value
    ## copy attributes
    attr(xd, "filename") <- x.att$filename
    attr(xd, "site") <- x.att$site
    attr(xd, "latitude") <- x.att$latitude
    attr(xd, "longitude") <- x.att$longitude
    attr(xd, "tav") <- x.att$tav
    attr(xd, "amp") <- x.att$amp
    attr(xd, "colnames") <- attr(x.att, "colnames")[-wn2r]
    attr(xd, "units") <- attr(x.att, "units")[-wn2r]
    attr(xd, "constants") <- x.att$constants
    attr(xd, "comments") <- x.att$comments
    x <- structure(xd, class = c("met", "data.frame"))
    return(x)
  }
  
  if(name %in% names(x)){
    ### This means that the variable is already present and we are replacing the 
    ### data
    oatt <- attributes(x)
    xd <- as.data.frame(unclass(x))
    xd[[name]] <- value
    
    attr(xd, "filename") <- oatt$filename
    attr(xd, "site") <- oatt$site
    attr(xd, "latitude") <- oatt$latitude
    attr(xd, "longitude") <- oatt$longitude
    attr(xd, "tav") <- oatt$tav
    attr(xd, "amp") <- oatt$amp
    attr(xd, "colnames") <- names(x)
    attr(xd, "units") <- oatt$units
    attr(xd, "constants") <- oatt$constants
    attr(xd, "comments") <- oatt$comments
    
    x <- structure(xd, class = c("met", "data.frame"))
    return(x)
  }else{
    ### 'value' should have the units attribute regardless
    if(is.null(attr(value, "units"))){
      stop("It is recommended to use function add_column_apsim_met for this operation instead.
         Partly because units are needed", call. = FALSE)    
    }
    
  
    ### If 'value' is a data.frame
    if(inherits(value, 'data.frame')){
      if(dim(value)[2] > 1)
        stop("'value' as data.frame should have just one column.", call. = FALSE)
      nm <- names(value)
      x <- add_column_apsim_met(x, value = value[[1]], name = nm, units = attr(x, 'units'))
      return(x)
    }else{
      ### If 'value' is a list
      if(inherits(value, 'list')){
        if(length(value) > 1)
          stop("'value' as a 'list' should have just one element.", call. = FALSE)
        nm <- names(value)
        if(is.null(nm))
          stop("'value' as a 'list' should be named.", call. = FALSE)
        x <- add_column_apsim_met(x, value = value[[1]], name = nm, units = attr(x, 'units'))
        return(x)
      }else{
        ### If 'value' is atomic for ('logical', 'integer', 'numeric', 'complex', 'character' and 'raw') vector 
        if(is.atomic(value)){
          if(length(value) != nrow(x))
            stop("'value' should be of length equal to number of rows in 'met' file.", call. = FALSE)
          if(is.null(attr(x, 'name')))
            stop("'value' should have an 'name' attribute.", call. = FALSE)
          nm <- attr(value, 'name')
          x <- add_column_apsim_met(x, value = value, name = nm, units = attr(x, 'units'))
          return(x)
        }else{
          if(!inherits(value, 'data.frame') || !inherits(value, 'list') || !is.atomic(value))
            stop("'value' should either be a 'data.frame' or a vector of length equal to the number of rows in the 'met' file", call. = FALSE)
        }
      }
    }
  }
}

#' @rdname add_column_apsim_met
#' @param x object of class \sQuote{met}
#' @param i indices specifying elements to extract or replace. 
#' @param j indices specifying elements to extract or replace. 
#' @param value value for the data.frame. It could be an integer, double or vector of length equal to the number of rows in x.
#' @export
`[<-.met` <- function(x, i, j, value){
  
  if(is.null(attr(value, "units"))){
    stop("It is recommended to use function add_column_apsim_met for this operation instead.
         Partly because units are needed", call. = FALSE)    
  }
  
  oatt <- attributes(x)
  xd <- as.data.frame(unclass(x))
  ### If 'value' is missing this is about extracting data
   if(missing(value)){
     xd <- xd[i, j]
   }else{
     ### If value is not missing, then we are replacing data
     xd[i, j] <- value
   }
  
  attr(xd, "filename") <- oatt$filename
  attr(xd, "site") <- oatt$site
  attr(xd, "latitude") <- oatt$latitude
  attr(xd, "longitude") <- oatt$longitude
  attr(xd, "tav") <- oatt$tav
  attr(xd, "amp") <- oatt$amp
  attr(xd, "colnames") <- names(x)
  attr(xd, "units") <- oatt$units
  attr(xd, "constants") <- oatt$constants
  attr(xd, "comments") <- oatt$comments
  
  x <- structure(xd, class = c("met", "data.frame"))
  
  return(x)
}

#' @rdname add_column_apsim_met
#' @param x object of class \sQuote{met}
#' @param i indices specifying elements to extract or replace. 
#' @param j indices specifying elements to extract or replace. 
#' @param value value for the data.frame. It could be an integer, double or vector of length equal to the number of rows in x.
#' @export
`[[<-.met` <- function(x, i, j, value){
  
  if(is.null(attr(value, "units"))){
    stop("It is recommended to use function add_column_apsim_met for this operation instead.
         Partly because units are needed", call. = FALSE)    
  }else{
    if(!is.null(names(value))){
      return(add_column_apsim_met(x, value = value, units = attr(value, "units")))  
    }else{
      stop("It is recommended to use function add_column_apsim_met for this operation instead.
         Partly because units are needed", call. = FALSE)    
    }
  }
}

#' @rdname add_column_apsim_met
#' @name remove_column_apsim_met
#' @param met object of class \sQuote{met}
#' @param name name of the variable to be removed
#' @return an object of class \sQuote{met} without the variable (column) in \sQuote{name}
#' @export
remove_column_apsim_met <- function(met, name){
  
  if(!inherits(met, "met"))
    stop("Object should be of class met.", call. = FALSE)
  
  if(missing(name)){
      stop("'name' is not provided. It should be the name of the column to remove", call. = FALSE)
  }
  
  if(!name %in% names(met)){
    cat("name:", name, "\n")
    cat("names in met:", names(met), "\n")
    stop("'name' to be removed is not in 'met' object", call. = FALSE)
  }
    
  ### This logic works well because it is in the right order?
  new.units <- attr(met, "units")[-which(names(met) == name)]  
  
  oatt <- attributes(met)
  metd <- as.data.frame(unclass(met))
  metd[[name]] <- NULL
  
  attr(metd, "filename") <- oatt$filename
  attr(metd, "site") <- oatt$site
  attr(metd, "latitude") <- oatt$latitude
  attr(metd, "longitude") <- oatt$longitude
  attr(metd, "tav") <- oatt$tav
  attr(metd, "amp") <- oatt$amp
  attr(metd, "colnames") <- names(metd)
  attr(metd, "units") <- new.units
  attr(metd, "constants") <- oatt$constants
  attr(metd, "comments") <- oatt$comments
  
  met <- structure(metd, class = c("met", "data.frame"))
  
  return(met)
}

#' This function can re-calculate annual mean monthly amplitude
#' for an object of class \sQuote{met}
#' @title Calculates attribute amp for an object of class \sQuote{met}
#' @param met object of class \sQuote{met}
#' @param by.year whether to perform calculations by year (default is TRUE)
#' @return an object of class \sQuote{met} with a recalculation of annual amplitude in mean monthly temperature 
#' @export
amp_apsim_met <- function(met, by.year = TRUE){
  
  if(!inherits(met, "met"))
    stop("Object should be of class 'met", call. = FALSE)

  ## Step 1: create date
  date <- as.Date(paste(met$year, met$day, sep = "-"), format = "%Y-%j") 
  ## Step 2: create month column
  mnth <- as.numeric(format(date, "%m")) 
  met <- add_column_apsim_met(met = met, value = mnth, name = "month", units = "()")

  ## Step 3: This is how APSIM does it
  if(by.year){
    Year <- as.numeric(format(date, "%Y"))   
    met <- add_column_apsim_met(met = met, value = Year, name = "Year", units = "()")  
    amp.vec <- numeric(length(unique(Year)))
    for(i in seq_along(unique(met$Year))){
      tmp <- subset(met, Year == unique(met$Year)[i])
      if(nrow(tmp) < 300){
        warning(paste("Year:", unique(met$Year)[i], "was not used for calculating 'amp' because less than 300 days were present"))
        next
      } 
      mtemp <- (tmp$maxt + tmp$mint) / 2
      tmp <- add_column_apsim_met(met = tmp, value = mtemp, name = "m.temp", units = "(oC)")
      tmp.agg <- aggregate(m.temp ~ month, data = tmp, FUN = mean)
      amp.vec[i] <- round(max(tmp.agg$m.temp) - min(tmp.agg$m.temp), 2) 
    }
    ans <- mean(amp.vec)
  }else{
    ## Alternative simpler method
    mtemp <- (met$maxt + met$mint) / 2
    met <- add_column_apsim_met(met = met, value = mtemp, name = "m.temp", units = "(oC)")
    met.agg <- aggregate(m.temp ~ mnth, data = met, FUN = mean)
    ans <- round(max(met.agg$m.temp) - min(met.agg$m.temp), 2)    
  }

  ## Clean up
  met <- remove_column_apsim_met(met, "month")
  if(by.year){
    met <- remove_column_apsim_met(met, "Year")
  }else{
    met <- remove_column_apsim_met(met, "m.temp")
  }
  
  attr(met, "amp") <- paste("amp =", ans, "!calculated with the apsimx R package:", Sys.time())
  
  return(met)
}

#' This function can re-calculate annual mean temperature
#' for an object of class \sQuote{met}
#' @title Calculates attribute amp for an object of class \sQuote{met}
#' @param met object of class \sQuote{met}
#' @param by.year whether to compute tav for each year and then average (default is TRUE)
#' @param na.rm whether to remove missing values (NAs). Default is TRUE.
#' @return an object of class \sQuote{met} with a recalculation of annual mean temperature amplitude 
#' @export
tav_apsim_met <- function(met, by.year = TRUE, na.rm = TRUE){
  
  if(!inherits(met, "met"))
    stop("Object should be of class 'met", call. = FALSE)
  
  ## Step 1: create date
  date <- as.Date(paste(met$year, met$day, sep = "-"), format = "%Y-%j") 

  ## Step 3: This is how APSIM does it
  if(by.year){
    Year <- as.numeric(format(date, "%Y"))   
    met <- add_column_apsim_met(met = met, value = Year, name = "Year", units = "()")  
    tav.vec <- numeric(length(unique(Year)))
    for(i in seq_along(unique(met$Year))){
      tmp <- subset(met, Year == unique(met$Year)[i])
      mtemp <- mean(colMeans(tmp[, c("maxt", "mint")], na.rm = na.rm), na.rm = na.rm)
      tav.vec[i] <- mtemp
    }
    ans <- mean(tav.vec)
  }else{
    ## Alternative simpler method
      mtemp <- (met$maxt + met$mint) / 2
      met <- add_column_apsim_met(met = met, value = mtemp, name = "m.temp", units = "(oC)")
      met.agg <- aggregate(m.temp ~ mnth, data = met, FUN = mean)
      ans <- round(max(met.agg$m.temp) - min(met.agg$m.temp), 2)    
  }
  
  ## Clean up
  if(by.year){
    met <- remove_column_apsim_met(met, "Year")
  }else{
    met <- remove_column_apsim_met(met, "m.temp")  
  }
  
  attr(met, "tav") <- paste("tav =", ans, "! calculated average annual temperature", Sys.time())
  
  return(met)
}

#' @title Extract or Replace Part of a \sQuote{met} object
#' @param x object of class \sQuote{met}
#' @param i index for rows
#' @param j index for columns
#' @param ... additional arguments
#' @param drop result is coerced to lowest possible dimension if TRUE.
#' @export
`[.met` <- function(x, i, j, ..., drop = TRUE){
  
 orig.met.attr <- attributes(x)
 
 ans <- as.data.frame(unclass(x))[i, j, drop = drop]
 
 attr(ans, "filename") <- orig.met.attr$filename
 attr(ans, "site") <- orig.met.attr$site
 attr(ans, "latitude") <- orig.met.attr$latitude
 attr(ans, "longitude") <- orig.met.attr$longitude
 attr(ans, "tav") <- orig.met.attr$tav
 attr(ans, "amp") <- orig.met.attr$amp
 attr(ans, 'colnames') <- names(ans)

 if(!missing(j)){
   attr(ans, "units") <- orig.met.attr$units[j]
 }else{
   attr(ans, "units") <- orig.met.attr$units
 }
 
 attr(ans, "constants") <- orig.met.attr$constants
 attr(ans, "comments") <- orig.met.attr$comments

 ans <- structure(ans, class = c("met", "data.frame"))
 return(ans)
  
}

#'
#' Simple optimization for APSIM Next Generation
#'
#' * At the moment it is required to provide starting values for the parameters of interest.
#'
#' * It is suggested that you keep a backup of the original file. This function
#' will edit and overwrite the file during the optimization.
#'
#' * When you use the parm.vector.index you cannot edit two separate elements of
#' a vector at the same time. This should be used to target a single element of
#' a vector only. (I can add this feature in the future if it is justified.)
#'
#' * Internally, the optimization is done around the scaled value of the initial parameter
#' values. A value of 1 would correspond to the inital value of the parameter.
#' The \sQuote{lower} and \sQuote{upper} (or \sQuote{ub} and \sQuote{lb}) are also scaled
#' to the initial values of the parameters. So, for example, if your initial value is 20
#' and you provide an upper bound of 5, it means that the actual upper value that you are allowing for is 100.
#'
#' * I have tested other optimizers and packages, but I think these are enough for most purposes. I tried
#' function stats::nlm (but it does not support bounds and it can fail), package 'optimx' is a bit messy and it
#' does not provide sufficient additional functionality. Package 'ucminf' seems like a good alternative, but it
#' did not perform better than the other ones.
#'
#' @title Optimize parameters in an APSIM Next Generation simulation
#' @name optim_apsimx
#' @rdname optim_apsimx
#' @description It is a wrapper for running APSIM-X and optimizing parameters using \code{\link[stats]{optim}}
#' @param file file name to be run (the extension .apsimx is optional)
#' @param src.dir directory containing the .apsimx file to be run (defaults to the current directory)
#' @param parm.paths absolute or relative paths of the coefficients to be optimized.
#'             It is recommended that you use \code{\link{inspect_apsimx}} for this
#' @param data data frame with the observed data. By default is assumes there is a 'Date' column for the index.
#' @param type Type of optimization. For now, \code{\link[stats]{optim}}, and, if available, \code{\link[nloptr]{nloptr}} or
#' \sQuote{mcmc} through \code{\link[BayesianTools]{runMCMC}}. Option \sQuote{ucminf} uses the \code{\link[ucminf]{ucminf}} function.
#' If \sQuote{type} is \sQuote{grid}, then a grid can be passed and no optimization will be performed.
#' @param weights Weighting method or values for computing the residual sum of squares.
#' @param index Index for filtering APSIM output. Typically, \dQuote{Date}, but it can be c(\dQuote{report}, \dQuote{Date}) for
#' multiple simulations
#' @param parm.vector.index Index to optimize a specific element of a parameter vector.  At the moment it is
#' possible to only edit one element at a time. This is because there is a conflict when generating multiple
#' elements in the candidate vector for the same parameter.
#' @param replacement TRUE or FALSE for each parameter. Indicating whether it is part of
#' the \sQuote{replacement} component. Its length should be equal to the length or \sQuote{parm.paths}.
#' @param root root argument for \code{\link{edit_apsimx_replacement}}
#' @param initial.values (required) supply the initial values of the parameters. (Working on fixing this...).
#' If the parameters to be optimized correspond to a single value, then a simple numeric vector can be supplied.
#' If one or more of the parameters represent a vector in APSIM, then the initial values should be passed as a
#' list. At the moment, it is not possible to check if these are appropriate (correct name and length).
#' @param grid grid used when \sQuote{type = grid}. Columns should be parameters and rows different values for
#' those parameters.
#' @param ... additional arguments to be passed to the optimization algorithm. See \code{\link[stats]{optim}}
#' @note When computing the objective function (residual sum-of-squares) different variables are combined.
#' It is common to weight them since they are in different units. If the argument weights is not supplied
#' no weighting is applied. It can be 'mean', 'variance' or a numeric vector of appropriate length.
#' @return object of class \sQuote{optim_apsim}, but really just a list with results from optim and additional information.
#' @export
#' @examples
#' \donttest{
#' ## See the vignette for examples
#' }
#'

optim_apsimx <- function(file, src.dir = ".",
                         parm.paths, data,
                         type = c("optim", "nloptr", "mcmc", "ucminf", "grid"),
                         weights,
                         index = "Date",
                         parm.vector.index,
                         replacement,
                         root,
                         initial.values,
                         grid,
                         ...){

  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }

  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)

  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to run.")
  }

  file <- match.arg(file, file.names)

  file.name.path <- file.path(src.dir, file)

  type <- match.arg(type)

  if(type == "nloptr"){
    if(!requireNamespace("nloptr", quietly = TRUE)){
      warning("The nloptr package is required for this optimization method")
      return(NULL)
    }
  }

  if(type == "mcmc"){
    if(!requireNamespace("BayesianTools", quietly = TRUE)){
      warning("The BayesianTools package is required for this method")
      return(NULL)
    }
  }

  if(type == "ucminf"){
    if(!requireNamespace("ucminf", quietly = TRUE)){
      warning("The ucminf package is required for this method")
      return(NULL)
    }
  }

  ## Data needs to be a data.frame
  if(!inherits(data, "data.frame"))
    stop("Object 'data' should be of class 'data.frame'.", call. = FALSE)
  
  if(all(c("SimulationName", "Date") %in% names(data)) && length(index) == 1){
    stop("'SimulationName' and 'Date' found in  'data' but index length is equal to 1.
         Maybe index should be c('SimulationName', 'Date')?", call. = FALSE)
  }
  
  ## Setting up Date
  if(any(grepl("Date", index))) data$Date <- as.Date(data$Date)
  datami <- data[ ,-which(names(data) %in% index), drop = FALSE]

  ## Setting up weights
  if(missing(weights)){
    weights <- rep(1, ncol(datami))
  }else{
    if(weights == "mean"){
      weights <- abs(1 / apply(datami, 2, mean, na.rm = TRUE))
    }else{
      if(weights == "var"){
        weights <- 1 / apply(datami, 2, var, na.rm = TRUE)
      }else{
        if(length(weights) != ncol(datami))
          stop("Weights not of correct length")
        if(!is.numeric(weights))
          stop("Weights should be numeric")
      }
    }
  }

  if(missing(parm.vector.index)){
    parm.vector.index <- rep(-1, length(parm.paths))
  }else{
    if(length(parm.vector.index) != length(parm.paths))
      stop("parm.vector.index should have length equal to parm.paths")
    if(!is.numeric(parm.vector.index))
      stop("parm.vector.index should be numeric")
  }

  ## Set up replacement
  if(missing(replacement)) replacement <- rep(FALSE, length(parm.paths))

  if(length(replacement) != length(parm.paths))
    stop("Length of replacement vector should be equal to parm.paths", call. = FALSE)

  ## If root is not present
  if(missing(root)) root <- list("Models.Core.Replacements", NA)

  ## Read apsimx file (JSON)
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))

  ## Build the initial values
  iparms <- vector("list", length = length(parm.paths))
  names(iparms) <- parm.paths

#  if(missing(initial.values))
#    stop("Initial values should be supplied. (Working on a fix for this)", call. = FALSE)

  ## How do I retrieve the current value I want to optimize?
  for(i in seq_along(parm.paths)){
    if(missing(initial.values)){
      if(replacement[i]){
        if(!grepl(".Simulations.Replacements.", parm.paths[i])){
          parm.val <- extract_values_apsimx(file = file, src.dir = src.dir,
                                            parm.path = paste0(".Simulations.Replacements.", parm.paths[i]))
        }else{
          parm.val <- extract_values_apsimx(file = file, src.dir = src.dir, parm.path = parm.paths[i])
        }
      }else{
        parm.val <- extract_values_apsimx(file = file, src.dir = src.dir,
                                          parm.path = parm.paths[i])
      }
      if(is.list(parm.val)){
        iparms[[i]] <- as.numeric(unlist(parm.val))
      }else{
        iparms[[i]] <- as.numeric(parm.val)
      }
    }else{
      if(is.list(initial.values)){
        iparms[[i]] <- initial.values[[i]]
      }else{
        iparms[[i]] <- as.numeric(initial.values[i])
      }

    }
  }

  obj_fun <- function(cfs, parm.paths, data, iparms, weights,
                      index, parm.vector.index, replacement, root, gpi){

    ## Need to edit the parameters in the simulation file or replacement
    for(i in seq_along(cfs)){
      ## Edit the specific parameters with the corresponding values
      if(parm.vector.index[i] <= 0){
        par.val <- iparms[[i]] * cfs[i]
      }else{
        pvi <- parm.vector.index[i]
        iparms[[i]][pvi] <- iparms[[i]][pvi] * cfs[i]
        par.val <- iparms[[i]]
      }

      if(replacement[i]){
        pp0 <- strsplit(parm.paths[i], ".", fixed = TRUE)[[1]]
        mpp <- paste0(pp0[-length(pp0)], collapse = ".")
        edit_apsimx_replacement(file = file,
                                src.dir = src.dir,
                                wrt.dir = src.dir,
                                node.string = mpp,
                                overwrite = TRUE,
                                parm = pp0[length(pp0)],
                                value = par.val,
                                root = root,
                                verbose = FALSE)
        }else{
          edit_apsimx(file = file,
                      src.dir = src.dir,
                      wrt.dir = src.dir,
                      node = "Other",
                      parm.path = parm.paths[i],
                      overwrite = TRUE,
                      value = par.val,
                      verbose = FALSE)
        }
    }

    ## Run simulation
    if(missing(gpi)){
      sim <- try(apsimx(file = file, src.dir = src.dir,
                        silent = TRUE, cleanup = TRUE, value = "report"),
                 silent = TRUE)
    }else{
      new.file.name <- paste0(tools::file_path_sans_ext(file), "-", gpi, ".apsimx")
      file.copy(from = file.path(src.dir, file),
                to = file.path(src.dir, new.file.name))
      xrgs <- xargs_apsimx(single.threaded = TRUE, cpu.count = 1L)
      sim <- try(apsimx(file = new.file.name, src.dir = src.dir,
                        silent = TRUE, cleanup = TRUE, xargs = xrgs,
                        value = "report"),
                 silent = TRUE)
    }


    if(inherits(sim, "try-error")) return(NA)

    if(!inherits(sim, "data.frame"))
      stop("the apsimx simulation should return an object of class 'data.frame'. \n
           Check that the report does simplify to a 'data.frame' instead of a 'list'.")

    if("report" %in% names(sim) && index == "Date")
      stop("If you have multiple simulations, you should modify the index argument. \n
           It is possible that 'index = c('report', 'Date')' might work.")

    ## Only keep those columns with corresponding names in the data
    ## and only the dates that match those in 'data'
    if(!all(names(data) %in% names(sim)))
      stop("names in 'data' do not match names in simulation")

    if(length(index) == 1 && index == "Date"){

      ## Apparently 'Date' can be a 'date' in both but if classes do not
      ## match, then it is a problem...
      if(!inherits(data[, index], "Date"))
        warning("Column 'Date' in 'data' is not of class 'Date'")
      if(!inherits(sim[, index], "Date"))
        warning("Column 'Date' in simulations is not of class 'Date'")
      
      sim.s <- subset(sim, sim$Date %in% data[[index]], select = names(data))
      sim.s <- sim.s[order(sim.s[, index[1]]),]
      data <- data[order(data[, index[1]]),]
      
      if(!all(sim.s[[index[1]]] == data[[index[1]]])){
        stop(paste("simulations and data for", index[1], "do not match"))
      }

    }else{

      if(!is.null(data$report)) data$report <- as.factor(data$report)
      if(!is.null(sim$report)) sim$report <- as.factor(sim$report)
      if(!is.null(sim$SimulationName)) sim$SimulationName <- as.factor(sim$SimulationName)
      if(!is.null(data$SimulationName)) data$SimulationName <- as.factor(data$SimulationName)
      sim.s0 <- merge(sim, subset(data, select = index), by = index)
      sim.s <- subset(sim.s0, select = names(data))
      ## However, they need to be in the exact same order
      sim.s <- sim.s[order(sim.s[, index[1]], sim.s[ ,index[2]]),]
      data <- data[order(data[, index[1]], data[, index[2]]),]

      if(!all(sim.s[[index[1]]] == data[[index[1]]]))
          stop(paste("simulations and data for", index[1], "do not match"))

      if(!all(sim.s[[index[2]]] == data[[index[2]]])){
        stop(paste("simulations and data for", index[2], "do not match"))
      }
    }

    if(nrow(sim.s) != nrow(data)){
      cat("Number of rows in data", nrow(data), "\n")
      cat("Number of rows in subset simulation", nrow(sim.s), "\n")
      stop("Number of rows in data does not equal number of rows in simulation")
    }

    if(nrow(sim.s) == 0L) stop("Something went wrong. No rows selected in simulations")
    ## Assuming they are aligned, get rid of the 'Date' column
    sim.s <- sim.s[,-which(names(sim.s) %in% index)]
    data <- data[,-which(names(data) %in% index)]
    ## Now I need to calculate the residual sum of squares
    ## For this to work all variables should be numeric
    if(isFALSE(all(sapply(data, is.numeric)))){
      cat("Column classes for 'data':", sapply(data, class), "\n")
      stop("All columns for the computation of RSS should be of class 'numeric' in 'data'", call. = FALSE)
    }
    if(isFALSE(all(sapply(sim.s, is.numeric)))){
      cat("Column classes for 'simulations':", sapply(data, class), "\n")
      stop("All columns for the computation of RSS should be of class 'numeric' in 'simulations'", call. = FALSE)
    }
    diffs <- as.matrix(data) - as.matrix(sim.s)
    rss <- sum(weights * colSums(diffs^2, na.rm = TRUE))
    return(log(rss))
  }

  ## Pre-optimized RSS
  pre.lrss <- obj_fun(cfs = rep(1, length(parm.paths)),
                      parm.paths = parm.paths,
                      data = data,
                      iparms = iparms,
                      weights = weights,
                      index = index,
                      parm.vector.index = parm.vector.index,
                      replacement = replacement,
                      root = root)
  ## optimization
  if(type == "optim"){
    op <- stats::optim(par = rep(1, length(parm.paths)),
                       fn = obj_fun,
                       parm.paths = parm.paths,
                       data = data,
                       iparms = iparms,
                       weights = weights,
                       index = index,
                       parm.vector.index = parm.vector.index,
                       replacement = replacement,
                       root = root,
                       ...)
  }

  if(type == "ucminf"){

    op <- ucminf::ucminf(par = rep(1, length(parm.paths)),
                         fn = obj_fun,
                         parm.paths = parm.paths,
                         data = data,
                         iparms = iparms,
                         weights = weights,
                         index = index,
                         parm.vector.index = parm.vector.index,
                         replacement = replacement,
                         root = root,
                         ...)
  }

  if(type == "nloptr"){
    op <- nloptr::nloptr(x0 = rep(1, length(parm.paths)),
                         eval_f = obj_fun,
                         parm.paths = parm.paths,
                         data = data,
                         iparms = iparms,
                         weights = weights,
                         index = index,
                         parm.vector.index = parm.vector.index,
                         replacement = replacement,
                         root = root,
                         ...)
    op$par <- op$solution
    op$value <- op$objective
    op$convergence <- op$status
  }

  #### Developing grid ----
  if(type == "grid"){

    if(missing(grid))
      stop("'grid' is required when 'type' = 'grid'", call. = FALSE)

    grid <- as.data.frame(grid)
    oiparms <- iparms ## Original initial parameters

    if(ncol(grid) != length(parm.paths))
      stop("Number of columns in grid should be equal to the number of parameters")

    ## Check that the name in the grid appears somewhere in the parameter path
    for(.ii in seq_along(parm.paths)){
      is.dot.present <- grepl(".", names(grid)[.ii], fixed = TRUE)
      if(is.dot.present){
        ## The first element should match simulation names
        fspe <- strsplit(names(grid)[.ii], ".", fixed = TRUE)[[1]]
        ## First and second parameter elements
        apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
        simulation.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
        ippgn1 <- grepl(fspe[1], parm.paths[.ii], ignore.case = TRUE)
        if(!ippgn1){
          cat("Name in first elelment grid name:", fspe[1], "\n")
          cat("parameter name", parm.paths[.ii], "\n")
          warning("names in first element of grid object name do not match parameter path name")
        }
        ippgn2 <- grepl(fspe[2], parm.paths[.ii], ignore.case = TRUE)
        if(!ippgn2){
          cat("Name in second element grid names:", fspe[2], "\n")
          cat("parameter name", parm.paths[.ii], "\n")
          warning("names in second element of grid object name do not match parameter path name")
        }
      }else{
        ippgn <- grepl(names(grid)[.ii], parm.paths[.ii], ignore.case = TRUE)
        if(!ippgn){
          cat("Name in grid:", names(grid)[.ii], "\n")
          cat("parameter name", parm.paths[.ii], "\n")
          warning("names in grid object do not match parameter path name")
        }
      }
    }

    xargs <- list(...)
    verbose <- FALSE; cores <- 1L
    if(!is.null(xargs$verbose)){
      verbose <- ifelse(xargs$verbose, TRUE, FALSE)
      pb <- utils::txtProgressBar(min = 1, max = nrow(grid), style = 3)
    }

    if(!is.null(xargs$cores)){
      cores <- xargs$cores
      acores <- parallel::detectCores()
      if(cores > acores)
        stop("'cores' argument should be less than the available cores")
      if(cores > 1){
        cl <- parallel::makeCluster(cores)
        ## Unless I rename the file there will be a conflict
        evars1 <- c('file', 'obj_fun', 'parm.paths', 'data', 'iparms', 'weights')
        evars2 <- c('index', 'parm.vector.index', 'replacement', 'root')
        evars3 <- c('grid')
        parallel::clusterExport(cl, c(evars1, evars2, evars3),
                                environment())
        parallel::clusterEvalQ(cl, {library('apsimx')})
      }
    }

    start <- Sys.time()
    if(cores == 1L){
      lrss.vec <- vector("list", length = nrow(grid))
      for(i in seq_len(nrow(grid))){

        iparms <- as.list(grid[i, ])
        lrss <- obj_fun(cfs = rep(1, length(parm.paths)),
                        parm.paths = parm.paths,
                        data = data,
                        iparms = iparms,
                        weights = weights,
                        index = index,
                        parm.vector.index = parm.vector.index,
                        replacement = replacement,
                        root = root)
        lrss.vec[[i]] <- lrss

        if(isTRUE(verbose)){
          utils::setTxtProgressBar(pb, i)
        }
      }
      close(pb)
    }else{
      lrss.vec <- parallel::parLapply(cl,
                                      seq_len(nrow(grid)),
                                      function(i) {
                                        lrss <- try(obj_fun(cfs = rep(1, length(parm.paths)),
                                                            parm.paths = parm.paths,
                                                            data = data,
                                                            iparms = as.list(grid[i, ]),
                                                            weights = weights,
                                                            index = index,
                                                            parm.vector.index = parm.vector.index,
                                                            replacement = replacement,
                                                            root = root,
                                                            gpi = i), silent = TRUE)
                                        if(inherits(lrss, 'try-error')){
                                          lrss <- -1
                                        }
                                        return(lrss)
                                      })

      lrss.vec <- do.call(c, lrss.vec)
      parallel::stopCluster(cl)
    }

    ## It looks like I still need to clean up when running parallel
    if(cores > 1){
      for(i in seq_len(nrow(grid))){
        for(j in c("apsimx", "db", "db-wal", "db-shm")){
          to.delete <- paste0(tools::file_path_sans_ext(file), "-", i, ".", j)
          file.to.delete <- file.path(src.dir, to.delete)
          if(file.exists(file.to.delete)) file.remove(file.to.delete)
        }
      }
    }

    ## Run the model one more time with the best result
    ans.grid <- cbind(grid, lrss = unlist(lrss.vec))
    if(verbose){
      number.parse.fails <- nrow(ans.grid[ans.grid$lrss < 0, ])
      cat("Number of parse fails:", number.parse.fails, "\n")
    }
    if(any(ans.grid$lrss < 0))  ans.grid[ans.grid$lrss < 0, "lrss"] <- NA
    wminlrss <- which.min(ans.grid$lrss)
    ### 'Optimized' parameters are the ratio over the original values
    op.par <- as.vector(unlist(grid[wminlrss,])) / as.vector(unlist(oiparms))
    best.parms <- grid[wminlrss,]
    lrss <- obj_fun(cfs = rep(1, length(parm.paths)),
                    parm.paths = parm.paths,
                    data = data,
                    iparms = best.parms,
                    weights = weights,
                    index = index,
                    parm.vector.index = parm.vector.index,
                    replacement = replacement,
                    root = root)

    op <- list(par = op.par,
               value = min(ans.grid$lrss),
               counts = nrow(grid),
               convergence = NA,
               message = "grid")
  }

  if(type == "mcmc"){
    ## Setting defaults
    datami.sds <- apply(datami, 2, sd)
    mcmc.args <- list(...)
    if(is.null(mcmc.args$lower)) lower <- rep(0, length(iparms) + ncol(datami))
    if(is.null(mcmc.args$upper)) upper <- c(rep(2, length(iparms)), datami.sds * 10)
    if(is.null(mcmc.args$sampler)) sampler <- "DEzs"
    if(is.null(mcmc.args$parallel)) mcmc.args$parallel <- FALSE
    if(is.null(mcmc.args$settings)) stop("runMCMC settings are missing with no default")


    cfs <- c(rep(1, length(iparms)), apply(datami, 2, sd))

    ## Create environment with objects
    assign('.file', file, mcmc.apsimx.env)
    assign('.src.dir', src.dir, mcmc.apsimx.env)
    assign('.parm.paths', parm.paths, mcmc.apsimx.env)
    assign('.data', data, mcmc.apsimx.env)
    assign('.iparms', iparms, mcmc.apsimx.env)
    assign('.index', index, mcmc.apsimx.env)
    assign('.parm.vector.index', parm.vector.index, mcmc.apsimx.env)
    assign('.replacement', replacement, mcmc.apsimx.env)
    assign('.root', root, mcmc.apsimx.env)

    ## Pre-optimized log-likelihood
    pll <- log_lik(cfs)

    cat("Pre-optimized log-likelihood", pll, "\n")

    nms <- c(names(iparms), paste0("sd_", names(datami)))
    bayes.setup <- BayesianTools::createBayesianSetup(log_lik,
                                                      lower = lower,
                                                      upper = upper,
                                                      names = nms,
                                                      parallel = mcmc.args$parallel)

    op.mcmc <- BayesianTools::runMCMC(bayes.setup,
                                      sampler = sampler,
                                      settings = mcmc.args$settings)
    return(op.mcmc)
  }

  if(type != "grid"){
    ans <- structure(list(pre.rss = exp(pre.lrss),
                          post.rss = exp(op$value), ## This is weighted RSS
                          weights = weights,
                          iaux.parms = iparms, op = op, n = nrow(data),
                          res = NA,
                          parm.vector.index = parm.vector.index),
                     class = "optim_apsim")
  }else{
    ans <- structure(list(pre.rss = exp(pre.lrss),
                          post.rss = exp(min(ans.grid$lrss, na.rm = TRUE)), ## This is smallest RSS
                          weights = weights,
                          iaux.parms = oiparms, op = op, n = nrow(data),
                          res = ans.grid,
                          parm.vector.index = parm.vector.index),
                     class = "optim_apsim")
  }

  return(ans)
}


## Log-likelihood
log_lik <- function(.cfs){

  .file <- get('.file', envir = mcmc.apsimx.env)
  .src.dir <- get('.src.dir', envir = mcmc.apsimx.env)
  .parm.paths <- get('.parm.paths', envir = mcmc.apsimx.env)
  .data <- get('.data', envir = mcmc.apsimx.env)
  .iparms <- get('.iparms', envir = mcmc.apsimx.env)
  .index <- get('.index', envir = mcmc.apsimx.env)
  .parm.vector.index <- get('.parm.vector.index', envir = mcmc.apsimx.env)
  .replacement <- get('.replacement', envir = mcmc.apsimx.env)
  .root <- get('.root', envir = mcmc.apsimx.env)

  ## Need to edit the parameters in the simulation file or replacement
  for(i in 1:length(.iparms)){
    ## Edit the specific parameters with the corresponding values
    if(.parm.vector.index[i] <= 0){
      par.val <- .iparms[[i]] * .cfs[i]
    }else{
      pvi <- .parm.vector.index[i]
      .iparms[[i]][pvi] <- .iparms[[i]][pvi] * .cfs[i]
      par.val <- .iparms[[i]]
    }

    if(.replacement[i]){
      pp0 <- strsplit(.parm.paths[i], ".", fixed = TRUE)[[1]]
      mpp <- paste0(pp0[-length(pp0)], collapse = ".")
      edit_apsimx_replacement(file = .file,
                              src.dir = .src.dir,
                              wrt.dir = .src.dir,
                              node.string = mpp,
                              overwrite = TRUE,
                              parm = pp0[length(pp0)],
                              value = par.val,
                              root = .root,
                              verbose = FALSE)
    }else{
      edit_apsimx(file = .file,
                  src.dir = .src.dir,
                  wrt.dir = .src.dir,
                  node = "Other",
                  parm.path = .parm.paths[i],
                  overwrite = TRUE,
                  value = par.val,
                  verbose = FALSE)
    }
  }

  ## Run simulation
  sim <- try(apsimx(file = .file, src.dir = .src.dir,
                    silent = TRUE, cleanup = TRUE, value = "report"),
             silent = TRUE)

  if(inherits(sim, "try-error")) return(NA)

  ## Only keep those columns with corresponding names in the data
  ## and only the dates that match those in 'data'
  if(!all(names(.data) %in% names(sim)))
    stop("names in 'data' do not match names in simulation")

  if(length(.index) == 1){

    sim.s <- subset(sim, sim[,.index, drop = FALSE] %in% .data[[.index]], select = names(.data))
    sim.s <- sim.s[order(sim.s[, .index]),]
    .data <- .data[order(.data[, .index]),]

    if(!all(sim.s[[.index]] == .data[[.index]]))
      stop(paste("simulations and data for", .index, "do not match"))

  }else{

    if(!is.null(.data$report)) .data$report <- as.factor(.data$report)
    if(!is.null(sim$report)) sim$report <- as.factor(sim$report)
    sim.s0 <- merge(sim, subset(.data, select = .index), by = .index)
    sim.s <- subset(sim.s0, select = names(.data))
    ## However, they need to be in the exact same order
    sim.s <- sim.s[order(sim.s[, .index[1]], sim.s[ ,.index[2]]),]
    .data <- .data[order(.data[, .index[1]], .data[, .index[2]]),]

    if(!all(sim.s[[.index[1]]] == .data[[.index[1]]]))
      stop(paste("simulations and data for", .index[1], "do not match"))

    if(!all(sim.s[[.index[2]]] == .data[[.index[2]]]))
      stop(paste("simulations and data for", .index[2], "do not match"))
  }

  if(nrow(sim.s) == 0L){
    cat("number of rows in sim", nrow(sim),"\n")
    cat("number of rows in data", nrow(.data), "\n")
    print(sim)
    print(.data)
    stop("no rows selected in simulations")
  }
  ## Assuming they are aligned, get rid of the 'Date' column
  sim.s <- sim.s[,-which(names(sim.s) == .index)]
  .data <- .data[,-which(names(.data) == .index)]
  ## Now I need to calculate the residual sum of squares
  ## For this to work all variables should be numeric
  diffs <- as.matrix(.data) - as.matrix(sim.s)
  if(ncol(diffs) == 1){
    lls <- stats::dnorm(diffs[,1], sd = .cfs[length(.cfs)], log = TRUE)
    return(sum(lls))
  }else{
    Sigma <- diag(.cfs[(length(.iparms) + 1):length(.cfs)])
    lls <- mvtnorm::dmvnorm(diffs, sigma = Sigma, log = TRUE)
    return(sum(lls))
  }
}

## Create an environment to solve this problem?
#' Create an apsimx environment for MCMC
#'
#' @title Environment to store data for apsimx MCMC
#' @description Environment which stores data for MCMC
#' @return This is an environment, so nothing to return.
#' @export
#'
mcmc.apsimx.env <- new.env(parent = emptyenv())

## Different behaviors of grep_json_list

require(apsimx)
apsimx_options(warn.versions = FALSE)

run.grep.test <- get(".run.local.tests", envir = apsimx.options)

if(run.grep.test){
  
  file <- "Wheat.apsimx"
  src.dir <- auto_detect_apsimx_examples()
  
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
  
  ## This function sort of works, but it is not quite what we want
  ## ans1 <- grep_json_list1("Cultivar", apsimx_json)
  
  ## A better function?
  ans <- grep_json_list("Cultivar", apsimx_json)
  
  ans <- grep_json_list("Hartog", apsimx_json)
  
  ## This does not work
  ## ans <- grep_json_list("Version", apsimx_json)

  ans <- grep_json_list("Sowing", apsimx_json)
  
  ## Testing inspect_apsimx_json
  pp0 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Version")
  
  pp1 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Simulations")
  
  pp2 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Clock") 
  
  ## pp2.1 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Start") ## Need to account for this
  
  pp3 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Weather") 
  
  ## pp3.1 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "FileName") ## Need to account for this
  
  pp4 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "DUL")
  
  pp6 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "SWCON")
  
  ## pp7 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "LL15")
  
  ## pp8 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Sow using") 
  
  pp9 <- inspect_apsimx_json("Wheat.apsimx", src.dir = src.dir, parm = "Hartog") 

}

#' @title Sensitivity Analysis for APSIM Next Generation simulation
#' @name sens_apsim
#' @rdname sens_apsim
#' @description It is a wrapper for running APSIM and evaluating different parameters values
#' @param file file name to be run (with extension .apsim)
#' @param src.dir directory containing the .apsim file to be run (defaults to the current directory)
#' @param crop.file name of auxiliary xml file where parameters are stored. If this is missing, it is 
#'                  assumed that the parameters to be edited are in the main simulation file.
#' @param parm.paths absolute or relative paths of the coefficients to be evaluated. 
#'             It is recommended that you use \code{\link{inspect_apsim}} for this
#' @param parm.vector.index Index to evaluate a specific element of a parameter vector.  At the moment it is
#' possible to only edit one element at a time. This is because there is a conflict when generating multiple
#' elements in the candidate vector for the same parameter.
#' @param xml.parm TRUE or FALSE for each parameter. Indicating whether it is part of 
#' an xml file. Its length should be equal to the length or \sQuote{parm.paths}.
#' @param grid grid of parameter values for the evaluation. It can be a data.frame.
#' @param summary function name to use to summarize the output to be a sinlge row (default is the mean).
#' @param root root argument for \code{\link{edit_apsim}}
#' @param verbose whether to print progress in percent and elapsed time.
#' @param cores number of cores to use for parallel evaluation
#' @param save whether to save intermediate results. By default they will be saved as a
#' \sQuote{csv} file using the name of the apsim file. This will replace \sQuote{apsim} with \sQuote{csv}.
#' It is also possible to provide the file name here (for example: \sQuote{Some_results.csv}).
#' @param ... additional arguments (none used at the moment).
#' @note The summary function is stored as an attribute of the data frame \sQuote{grid.sims}.
#' @return object of class \sQuote{sens_apsim}, but really just a list with results from the evaluations.
#' @export
#' @examples 
#' \donttest{
#' ## See the vignette for examples
#' }
#' 

sens_apsim <- function(file, src.dir = ".", 
                       crop.file,
                       parm.paths,
                       parm.vector.index,
                       xml.parm,
                       grid,
                       summary = c("mean", "max", "var", "sd", "none"),
                       root,
                       verbose = TRUE,
                       cores = 1L,
                       save,
                       ...){
  
  if(missing(file))
    stop("file is missing with no default")
  
  .check_apsim_name(file)
  .check_apsim_name(src.dir)
  
  if(cores > 1L){
    stop("This feature has not been implemented yet for Classic. 
         Please submit an issue in github if you need it")    
  }

  if(src.dir != ".") stop("At the moment it is not possible \n
                          to change the source directory.")
  
  ## This might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsim$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsim files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  summary <- match.arg(summary)
  
  if(missing(parm.vector.index)){
    parm.vector.index <- rep(-1, length(parm.paths))
  }else{
    if(length(parm.vector.index) != length(parm.paths))
      stop("parm.vector.index should have length equal to parm.paths") 
    if(!is.numeric(parm.vector.index))
      stop("parm.vector.index should be numeric")
  }
  
  ## What this does, is pick the crop.file to be edited when it is not missing
  if(!missing(crop.file)){
    aux.file <- crop.file
    if(missing(xml.parm)){
      cfile <- rep(TRUE, length(parm.paths))  
    }else{
      cfile <- xml.parm
    } 
  }else{
    aux.file <- file
    if(missing(xml.parm)){
      cfile <- rep(FALSE, length(parm.paths))
    }else{
      cfile <- xml.parm
    } 
  }

  if(missing(grid))
    stop("grid argument is missing")
  
  grid <- as.data.frame(grid)
  
  if(ncol(grid) != length(parm.paths))
    stop("Number of columns in grid should be equal to the number of parameters")
  
  ## Check that the name in the grid appears somewhere in the parameter path
  for(i in seq_along(parm.paths)){
    ippgn <- grepl(names(grid)[i], parm.paths[i], ignore.case = TRUE)
    if(!ippgn){
      cat("Name in grid:", names(grid)[i], "\n")
      cat("parameter name", parm.paths[i], "\n")
      warning("names in grid object do not match parameter path name")  
    }
  }
  
  col.sim <- NULL
  start <- Sys.time()
  
  for(i in 1:dim(grid)[1]){
    
    ## Need to edit the parameters in the simulation file or replacement
    for(j in seq_along(parm.paths)){
      ## Edit the specific parameters with the corresponding values
      if(parm.vector.index[j] <= 0){
        par.val <- grid[i, j]  
      }else{
        stop("Submit an issue in github if you need this feature", call. = FALSE)
      }
      
      if(cfile[j]){
        ## Here I'm editing an auxiliary file ending in .xml
        edit_apsim_xml(file = aux.file, 
                       src.dir = src.dir,
                       parm.path = parm.paths[j],
                       overwrite = TRUE,
                       value = as.character(par.val),
                       verbose = FALSE)        
      }else{
        ## Here I'm editing the main simulation file .apsim
        edit_apsim(file = file, 
                   node = "Other",
                   src.dir = src.dir,
                   parm.path = parm.paths[j],
                   overwrite = TRUE,
                   value = as.character(par.val),
                   verbose = FALSE)        
      }
    }
    
    ## Run simulation  
    sim <- try(apsim(file = file, src.dir = src.dir,
                      silent = TRUE, cleanup = TRUE, value = "report"),
               silent = TRUE)
    
    if(inherits(sim, "try-error") && i == 1){
      stop("Simulation failed for initial parameter combination")
    }
    
    if(inherits(sim, "try-error") && i > 1){
      mat <- matrix(ncol = ncol(sim[,col.class.numeric]))
      sim.sd <- as.data.frame(mat)
      names(sim.sd) <- nms.sim
      col.sim <- rbind(col.sim, sim.sd)
      next
    } 
    
    ## Extract basic information from sim
    col.class.numeric <- which(sapply(sim, class) == "numeric") ## Which columns are numeric
    nms.sim <- names(sim[, col.class.numeric]) ## Names of the columns
    
    if(summary == "mean"){
      sim.s <- colMeans(sim[, col.class.numeric], na.rm = TRUE)
      sim.sd <- as.data.frame(t(sim.s))
    }
    
    if(summary == "max"){
      sim.s <- apply(sim[, col.class.numeric], 2, max, na.rm = TRUE)
      sim.sd <- as.data.frame(t(sim.s))
    }
    
    if(summary == "var"){
      sim.s <- apply(sim[, col.class.numeric], 2, var, na.rm = TRUE)
      sim.sd <- as.data.frame(t(sim.s))
    }
    
    if(summary == "sd"){
      sim.s <- apply(sim[, col.class.numeric], 2, sd, na.rm = TRUE)
      sim.sd <- as.data.frame(t(sim.s))
    }
    
    if(summary == "none"){
      sim.sd <- cbind(grid[i, ,drop = FALSE], sim, row.names = NULL)
    }
    
    col.sim <- rbind(col.sim, sim.sd)
    
    if(save != FALSE){
      if(isTRUE(save)){
        save.file.name <- paste0(tools::file_path_sans_ext(file), ".csv")
        utils::write.csv(col.sim, 
                         file = save.file.name,
                         row.names = FALSE)
      }else{
        if(!is.character(save))
          stop("'save' argument should be a character", call. = FALSE)
        if(tools::file_ext(save) != "csv")
          stop("'save' only accepts 'csv' as an extension", call. = FALSE)
        save.file.name <- save
        utils::write.csv(col.sim, 
                         file = save.file.name,
                         row.names = FALSE)
      }
    }
    
    if(verbose){
      nrow.grid <- nrow(grid)
      old.prev.div <- 0
      
      if(nrow.grid <= 10){
        dftm <- difftime(Sys.time(), start)
        cat("Progress:", round((i/nrow.grid) * 100), "%. Time elapsed:", dftm, units(dftm)," \n")  
      }else{
        progress.step <- ifelse(nrow.grid <= 20, 10, 5)
        
        prev.div <- round((i/nrow.grid) * 100) %/% progress.step
        
        if(prev.div > old.prev.div){
          dftm <- difftime(Sys.time(), start)
          cat("Progress:", round((i/nrow(grid)) * 100), "%. Time elapsed:", dftm, units(dftm)," \n")  
          old.prev.div <- prev.div
        } 
      }
    }
  }
  
  if(verbose && !isFALSE(save))
    cat("Saved results as:", save.file.name, "\n")
  
  if(summary != "none"){
    cdat <- cbind(grid, col.sim)  
  }else{
    cdat <- col.sim
  }
  
  attr(cdat, "summary") <- summary
  
  ans <- structure(list(grid.sims = cdat, grid = grid, parm.paths = parm.paths), class = "sens_apsim")
  
  return(ans) 
}

#### Testing extract data function
require(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

run.extract.tests <- get(".run.local.tests", envir = apsimx.options)

if(run.extract.tests){
  
  tmp.dir <- tempdir()
  dir(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()
  
  ### Test with all examples
  ex.dir.list <- dir(ex.dir, recursive = FALSE, pattern = "apsimx$")
  
  ## ex.dir.list2 <- ex.dir.list[c(2,4,5,7:11,13,16:20,22:24,28:30,32:35)]
  ## Excluded: AgPasture, BiomassRemovalFromPlant, Chickpea, Factorial, Grapevine
  ##           Graph, Pinus, Rotation, SCRUM, SimpleGrazing, Stock  
  ## Trying node = "Clock" - default
  
  ex.dir.list2 <- c("Barley.apsimx", "Canola.apsimx", "CanolaGrazing.apsimx",
                    "Chicory.apsimx", "ControlledEnvironment.apsimx", "CsvWeather.apsimx",
                    "Eucalyptus.apsimx", "EucalyptusRotation.apsimx", "FodderBeet.apsimx",
                    "Maize.apsimx", "Mungbean.apsimx", "Oats.apsimx",  "OilPalm.apsimx",
                    "Peanut.apsimx", "PlantainForage.apsimx", "Potato.apsimx", "RedClover.apsimx",
                    "Slurp.apsimx", "Sorghum.apsimx", "Soybean.apsimx", "Sugarcane.apsimx",
                    "SWIM.apsimx",  "Wheat.apsimx", "WhiteClover.apsimx") 
  
  for(i in ex.dir.list2){
    if(!file.exists(file.path(tmp.dir, i))) file.copy(from = file.path(ex.dir, i), to = tmp.dir)  
    cat("Simulation:", i)
    (edf <- extract_data_apsimx(i, src.dir = tmp.dir))  
    cat("\n")
  }
  
  ## ex.dir.list3 <- ex.dir.list[c(1, 3, 6, 12, 14, 15, 21, 25, 26, 27, 31)]
  ## Number 27: SimpleGrazing is a mess
  ## ex.dir.list3 <- ex.dir.list[c(1, 3, 6, 12, 14, 15, 21, 25, 26, 31)]
  
  ex.dir.list3 <- c("AgPasture.apsimx", "BiomassRemovalFromPlant.apsimx",
                    "Chickpea.apsimx", "Factorial.apsimx",   
                    "Pinus.apsimx", "Rotation.apsimx",  
                    "SCRUM.apsimx")   
  
  for(i in ex.dir.list3){
    if(!file.exists(file.path(tmp.dir, i))) file.copy(from = file.path(ex.dir, i), to = tmp.dir)  
    cat("Simulation:", i, "\n")
    next
    ## inspect_apsimx(i, src.dir = tmp.dir, node = "Other")
    if(i == "AgPasture.apsimx")  (edf.agp <- extract_data_apsimx(i, src.dir = tmp.dir, root = "AgPastureExample"))  
    if(i == "BiomassRemovalFromPlant.apsimx") (edf.brfp <- extract_data_apsimx(i, src.dir = tmp.dir, root = "SendingDatesFromOpperations"))  
    if(i == "Chickpea.apsimx") (edf.chp <- extract_data_apsimx(i, src.dir = tmp.dir, root = list("Continuous_TOS", "Cont_TOS")))  
    if(i == "Factorial.apsimx") (edf.fct <- extract_data_apsimx(i, src.dir = tmp.dir, root = list("RangeExperiment", "Base1")))  
    if(i == "Pinus.apsimx") (edf.pns <- extract_data_apsimx(i, src.dir = tmp.dir, root = list("Plantation_IxF_Experiment", "Treatment", "Base")))  
    if(i == "Rotation.apsimx") (edf.rot <- extract_data_apsimx(i, src.dir = tmp.dir, root = list("Mar", "Mar")))
    if(i == "Rotation.apsimx") (edf.rot2 <- extract_data_apsimx(i, src.dir = tmp.dir, root = "Mar.Mar"))
    if(i == "SCRUM.apsimx") (edf.scrm <- extract_data_apsimx(i, src.dir = tmp.dir, root = "Crop Comparisions.CropCompBase"))
    cat("\n")
  }
}

### Testing extraction of initialwater and Solutes ----

if(run.extract.tests){
  
  ex.dir <- auto_detect_apsimx_examples()
  
  ex.dir.list <- dir(ex.dir, recursive = FALSE, pattern = "apsimx$")
  ### Only select the simulations which do not have multiple simulations
  
  ex.dir.list4 <- ex.dir.list[c(2, 16, 17, 18, 24, 29, 30, 34)]
  
  ex.dir.list4 <- c("Barley.apsimx", "Maize.apsimx", "Mungbean.apsimx", "Oats.apsimx",     
                    "RedClover.apsimx", "Sorghum.apsimx", "Soybean.apsimx", "Wheat.apsimx")
  
  for(i in ex.dir.list4){
    (edf.iw <- extract_data_apsimx(i, src.dir = ex.dir, node = "Soil",
                                  soil.child = "InitialWater"))
    
    (edf.sol <- extract_data_apsimx(i, src.dir = ex.dir, node = "Soil",
                                   soil.child = "Solute"))  
    
    (edf.sol.NO3 <- extract_data_apsimx(i, src.dir = ex.dir, node = "Soil",
                                       soil.child = "Solute", parm = "NO3"))    
  }
}

### Testing whether operations can be extracted
if(FALSE){
  
  inspect_apsimx("FodderBeet.apsimx", ex.dir, node = "Other", parm = 2)
  
  inspect_apsimx("FodderBeet.apsimx", ex.dir, node = "Other", 
                 parm = list(1, 3, 8, 4))

  tmpp.dir <- "~/Dropbox/apsimx-other-issues/matteolongo/fodderbeet_quadr"
  setwd(tmpp.dir)
  dir()
  inspect_apsimx("fodderbeet_quadr.apsimx", src.dir = tmpp.dir, node = "Other", parm = 3)
  inspect_apsimx("fodderbeet_quadr.apsimx", src.dir = tmpp.dir, node = "Operations", 
                 root = "quadr_0_l_fodderbeet")
  edfo <- extract_data_apsimx("fodderbeet_quadr.apsimx", src.dir = tmpp.dir, node = "Operations", 
                              root = "quadr_0_l_fodderbeet")
}

## Testing the compare_apsim function
require(apsimx)
packageVersion("apsimx")
require(ggplot2)
apsimx_options(warn.versions = FALSE)

run.test.compare.apsimx <- get(".run.local.tests", envir = apsimx.options)

extd.dir <- system.file("extdata", package = "apsimx")

if(run.test.compare.apsimx){
  
  data(obsWheat)
  sim.opt <- read.csv(file.path(extd.dir, "wheat-sim-opt.csv"))
  sim.opt$Date <- as.Date(sim.opt$Date)

  ## Testing compare_apsim
  cap <- compare_apsim(obsWheat, sim.opt, labels = c("obs", "sim"))
  
  compare_apsim(obsWheat, sim.opt, labels = c("obs", "sim"), verbose = TRUE)
  
  plot(cap)
  plot(cap, plot.type = "diff")
  plot(cap, plot.type = "ts")
  
  plot(cap, variable = "AboveGround")
  plot(cap, variable = "AboveGround", plot.type = "diff")
  plot(cap, variable = "AboveGround", plot.type = "resid")
  plot(cap, variable = "AboveGround", plot.type = "ts")
  
  ## Testing the id functionality ----
  plot(cap, variable = "AboveGround", plot.type = "resid", id = 0.05)
  plot(cap, variable = "AboveGround", plot.type = "resid", id = 0.05, 
       id.label = letters[1:10])
  plot(cap, variable = "AboveGround", plot.type = "ts", id = 0.05)
  plot(cap, variable = "AboveGround", plot.type = "ts", id = 0.05, 
       id.label = letters[1:10])
  
  ## Adding the capability when the length of the index is equal to 2
  user.name <- Sys.info()[["user"]]
  
  if(user.name == "femiguez"){
    root.dir <- "C:/Users/femiguez/Dropbox/apsimx-other/pheno_optim"
  }else{
    if(user.name == "fernandomiguez"){
      root.dir <- "~/Dropbox/apsimx-other/pheno_optim/"    
    }else{
      stop("This test should only run in local computers") 
    }
  }
  
  obsPheno <- read.csv(file.path(root.dir, "obsPheno.csv"))
  simPheno <- read.csv(file.path(root.dir, "simPheno.csv"))
  
  obsPheno$Date <- as.Date(obsPheno$Date)
  simPheno$Date <- as.Date(simPheno$Date)
  
  cmp <- compare_apsim(obsPheno, simPheno, index = c("report", "Date"), labels = c("Obs", "Sim"))
  
  plot(cmp)
  plot(cmp, by = "report")
  plot(cmp, by = "report", se = FALSE)
  plot(cmp, by = "report", se = FALSE, facet = TRUE)
  plot(cmp, by = "report", plot.type = "ts", facet = TRUE, se = FALSE)
  
  plot(cmp, by = "report", plot.type = "ts", facet = TRUE, se = FALSE) + 
    ggplot2::theme(legend.position = "top")
  
  ### Testing id feature ----
  plot(cmp, by = "report", se = FALSE, id = 0.05, facet = TRUE,
       id.label = obsPheno$Date)
  
}

if(FALSE){
  
  ### This test is not fully developed
  if(user.name == "femiguez"){
    root.dir <- "C:/Users/femiguez/Dropbox"
  }else{
    root.dir <- "~/Dropbox"
  }
  
  ### Testing inspired by a more complicated dataset?
  sim.wheat <- read_apsim_all(src.dir = file.path(root.dir, "apsimx-other/KeLiu/mcmc_test/mcmc_test"))
  sim.wheat$outfile <- as.factor(sim.wheat$outfile)
  ## obs.wheat <- as.data.frame(readxl::read_excel("~/Dropbox/apsimx-other/KeLiu/mcmc_test/mcmc_test/Obs.xlsx"))
  obs.wheat$outfile <- as.factor(obs.wheat$outfile)
  obs.wheat$Date <- as.Date(obs.wheat$Date)

  cmp1 <- compare_apsim(obs.wheat, sim.wheat, 
                        labels = c("obs", "sim"))
  
  cmp1 <- compare_apsim(obs.wheat, sim.wheat, index = c("outfile", "Date"), 
                        labels = c("obs", "sim"))
  
  plot(cmp1)
  plot(cmp1, by = "outfile")
  plot(cmp1, by = "outfile", se = FALSE)
  plot(cmp1, by = "outfile", plot.type = "ts", facet = TRUE, se = FALSE)
  
}

#' Data source is USDA-NRCS Soil Data Access. See package soilDB for more details
#' 
#' * If a point is requested then an object of class \sQuote{sf} is returned (for mapunit.shp) 
#' with the MUKEY and AREASYMBOL with GEOMETRY type: POINT. 
#' 
#' * If a the request is for a spatial polygon, then an object of class \sQuote{sf} is returned
#' with gid, mukey and area_ac with GEOMETRY type: POLYGON.
#' 
#' @title Retrieve soil profile data and return a (list) with data frames (tables)
#' @description This function does partially what get_ssurgo_soil_profile does, but it 
#' returns a list with tables for mapunit, component, chorizon and mapunit.shp (object of class sf)
#' @name get_ssurgo_tables
#' @param lonlat Longitude and latitude vector (e.g. c(-93, 42))
#' @param shift simple mechanism for creating an area of interest by displacing the point indicated in 
#' lonlat by some amount of distance (e.g. 300 - in meters)
#' @param aoi area of interest, if supplied the lonlat and shift arguments will be ignored. Should be
#' of class \sQuote{sp::SpatialPolygons} or \sQuote{sf}. 
#' @param verbose whether to print messages and warnings to the console default FALSE
#' @param ... additional arguments to be passed to \sQuote{soilDB} functions.
#' @return a list with elements: mapunit, component, chorizon and mapunit.shp
#' @export
#' @examples 
#' \dontrun{
#' require(soilDB)
#' require(sp)
#' require(sf)
#' require(spData)
#' ## retrieve data from lon -93, lat = 42
#' stbls <- get_ssurgo_tables(lonlat = c(-93, 42)) 
#' 
#' stbls2 <- get_ssurgo_tables(lonlat = c(-93, 42), shift = 200)
#' 
#' }

get_ssurgo_tables <- function(lonlat, shift = -1, aoi, verbose = FALSE, ...){
  
  if(!requireNamespace("soilDB", quietly = TRUE)){
    warning("The soilDB package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sp", quietly = TRUE)){
    warning("The sp package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sf", quietly = TRUE)){
    warning("The sf package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("spData", quietly = TRUE)){
    warning("The spData package is required for this function")
    return(NULL)
  }
  
  if(!missing(lonlat) && !missing(aoi))
    stop("Either use 'lonlat' or 'aoi', but not both", call. = FALSE)

  if(missing(aoi)){
    lon <- lonlat[1]
    lat <- lonlat[2]
    
    if(lon < -180 || lon > 180) stop("longitude should be between -180 and 180")
    if(lat < -90 || lat > 90) stop("latitude should be between -90 and 90")
    
    if(shift <= 0){
      spg <- sp::SpatialPoints(cbind(x = lon, y = lat), 
                               proj4string = sp::CRS('+proj=longlat +datum=WGS84'))
    }else{
      shift <- (shift / 111) * 0.001 ## This is now in degrees
      lonlat.mat <- rbind(lonlat, ##root
                          lonlat + c(shift * 0.75, 0), ## x = 1, y = 0
                          lonlat + c(shift * 0.75, shift), ## x = 1, y = 1
                          lonlat + c(0, shift), ## x = 0, y = 1
                          lonlat) ## back to root
      rownames(lonlat.mat) <- NULL
      ## the previous matrix is a rectangle
      pg <- sp::Polygon(lonlat.mat)
      spg <- sp::SpatialPolygons(list(sp::Polygons(list(pg), "s1")),
                                 proj4string = sp::CRS("+proj=longlat +datum=WGS84"))
    }    
  }else{
    
    ## It appears that coercion is not needed
    ## if(inherits(aoi, "sf")) aoi <- sf::as_Spatial(aoi, "Spatial")
      
    if(!inherits(aoi, "SpatialPolygons") && !inherits(aoi, "sf"))
      stop("'aoi' should be of class 'SpatialPolygons' or 'sf'.", call. = TRUE)
    spg <- aoi    
  }

  if(verbose == FALSE){
    res <- suppressWarnings(soilDB::SDA_spatialQuery(spg, what = 'mupolygon', geomIntersection = TRUE))
  }else{
    res <- soilDB::SDA_spatialQuery(spg, what = 'mupolygon', geomIntersection = TRUE)
  }
  
  mu.is <- soilDB::format_SQL_in_statement(res$mukey)  
  sql <- sprintf("mukey IN %s", mu.is)
  if(verbose == FALSE){
    fSDA <- suppressWarnings(suppressMessages(soilDB::fetchSDA(sql, duplicates = TRUE)))
  }else{
    fSDA <- soilDB::fetchSDA(sql, duplicates = TRUE)
  } 
  
  ### Mapunit ### 
  if(verbose == FALSE){
    mapunit <- suppressWarnings(suppressMessages(soilDB::get_mapunit_from_SDA(sql)))
  }else{
    mapunit <- soilDB::get_mapunit_from_SDA(sql) 
  }
  
  ### Component ###
  cmpnt <- fSDA@site
  names(cmpnt) <- gsub("_", ".", names(cmpnt), fixed = TRUE)
  cmpnt$geomdesc <- cmpnt$geompos
  
  ### Getting interpretation information ----
  cointer <- soilDB::get_cointerp_from_SDA(sql,
                                           mrulename = "NCCPI - National Commodity Crop Productivity Index (Ver 3.0)",
                                           duplicates = TRUE)
  
  ### retrieve the state information ###
  ## This is unnecessary, but it works for now
  states <- spData::us_states ## This object contains states and their geometries
  states <- sf::st_transform(states, crs = 3857)
  pspg <- sf::st_transform(sf::st_as_sf(spg), crs = 3857)
  ii <- as.integer(sf::st_intersects(pspg, states))
  cmpnt$state <- states[["NAME"]][[ii]]
  
  ### Chorizon ###
  chrzns <- fSDA@horizons
  names(chrzns) <- gsub("_", ".", names(chrzns), fixed = TRUE)
  ### Things missing from horizons: hzthk.r, partdensity, wsatiated.r, wfifteenbar.r, wtenthbar.r, wthirdbar.r,
  if(sum(grepl("partdensity", names(chrzns))) == 0) chrzns$partdensity <- NA
  if(sum(grepl("hzthk", names(chrzns))) == 0) chrzns$hzthk.r <- NA
  if(sum(grepl("wsatiated", names(chrzns))) == 0) chrzns$wsatiated.r <- NA
  if(sum(grepl("wfifteenbar", names(chrzns))) == 0) chrzns$wfifteenbar.r <- NA
  if(sum(grepl("wthirdbar", names(chrzns))) == 0) chrzns$wthirdbar.r <- NA
  
  if(shift <= 0 && missing(aoi)){
    spg.sf <- sf::st_as_sf(spg)
    spg.sf[["MUKEY"]] <- res$mukey
    spg.sf[["AREASYMBOL"]] <- mapunit$areasymbol
    mapunit.shp <- spg.sf
  }else{
    mapunit.shp <- sf::st_as_sf(res)
  }
  
  ans <- list(mapunit = mapunit, component = cmpnt, 
              chorizon = chrzns, mapunit.shp = mapunit.shp)
  ans
}

## Test soil properties in APSIM-X

require(apsimx)
require(ggplot2)

apsimx_options(warn.versions = FALSE)

run.soil <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()
setwd(tmp.dir)

if(run.soil){
  
  ex.dir <- auto_detect_apsimx_examples()

  file.copy(file.path(ex.dir, "Wheat.apsimx"), tmp.dir)
  
  wheat <- apsimx("Wheat.apsimx")  
  
  inspect_apsimx("Wheat.apsimx", 
                 node = "Soil", soil.child = "Physical")

  duls <- c(0.52, 0.49, 0.488, 0.48, 0.47, 0.457, 0.452) * 1.05
  
  edit_apsimx("Wheat.apsimx", src.dir = ex.dir,
              wrt.dir = tmp.dir, node = "Soil",
              soil.child = "Physical", parm = "DUL",
              value = duls)   

  inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil", soil.child = "Physical")
  
  wheat2 <- apsimx("Wheat-edited.apsimx")

  ggplot() + 
    geom_line(data = wheat, aes(x = Date, y = Wheat.AboveGround.Wt, color = "Original DUL")) + 
    geom_line(data = wheat2, aes(x = Date, y = Wheat.AboveGround.Wt, color = "DUL times 1.05"))
  
}

if(run.soil){
  
  ## For now
  extd.dir <- system.file("extdata", package = "apsimx")
  
  inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
                 node = "Soil", soil.child = "Physical")

  inspect_apsimx("Maize.apsimx", src.dir = extd.dir, 
                 node = "Soil", soil.child = "Physical")
  
  inspect_apsimx("Wheat_soil.apsimx", src.dir = extd.dir, 
                 node = "Soil", soil.child = "Physical")
  
  ## CanI now just edit the Crop specific soil physical?
  edit_apsimx("Wheat_soil.apsimx", src.dir = extd.dir,
              wrt.dir = tmp.dir,
              node = "Soil", soil.child = "Physical",
              parm = "Maize KL", value = rep(0.07, 7))
  
  edit_apsimx("Wheat_soil-edited.apsimx", 
              src.dir = tmp.dir,
              overwrite = TRUE,
              node = "Soil", soil.child = "Physical",
              parm = "Wheat KL", value = rep(0.05, 7))
  
  inspect_apsimx("Wheat_soil-edited.apsimx", src.dir = tmp.dir, 
                 node = "Soil", soil.child = "Physical")
  
}

## Test reading and writing soils in XML format
## Author: Fernando E. Miguez
## Date: 2022-02-16
##
## Required: R package apsimx 2.3.2
require(apsimx)
require(ggplot2)

apsimx_options(warn.versions = FALSE)

run.apsimx.soils <- get(".run.local.tests", envir = apsimx.options)

run.apsimx.soils <- FALSE ## It takes 23 minutes to run the code below

if(run.apsimx.soils){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  
  sls <- read_apsim_soils("Clarion.soils", src.dir = extd.dir)
  
  ## All soils in APSoil
  apsoil.dir <- "~/Dropbox/apsimx-other/APSoil"
  
  ## Reading in all this data takes a while
  start <- Sys.time()
  asls <- read_apsim_soils("APSRU-Australia-soils.soils", src.dir = apsoil.dir)
  end <- Sys.time()
  
}

### Testing fixing a soil profile

run.apsimx.soils.fix <- get(".run.local.tests", envir = apsimx.options)

try.isric <- try(get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, verbose = FALSE), silent = TRUE)

## If the above fails it is very likely that the server is down
if(inherits(try.isric, 'try-error')) run.apsimx.soils.fix <- FALSE
run.apsimx.soils.fix <- FALSE

if(run.apsimx.soils.fix){
  
  tmp.dir <- tempdir()
  setwd(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()
  
  file.copy(file.path(ex.dir, "Wheat.apsimx"), ".")  
  
  lon <- seq(1, 2, length.out = 30)
  lat <- seq(48, 49, length.out = 30)
  
  for(i in seq_along(lon)){
    
    sp <- try(get_isric_soil_profile(lonlat = c(lon[i], lat[i]), fix = TRUE, verbose = FALSE), silent = TRUE)
    if(inherits(sp, "try-error")){
      cat("No soil for this location. Lon:", lon[i], "Lat:", lat[i], "\n")
      next
    }
    
    edit_apsimx_replace_soil_profile("Wheat.apsimx", soil.profile = sp,
                                     edit.tag = paste0("-", i))
    sim1 <- try(apsimx(paste0("Wheat-", i, ".apsimx")), silent = TRUE)
    
    if(inherits(sim1, "try-error"))
      cat("Simulation:", i, "failed\n")
  }
}

## Run a few tests for the examples
require(apsimx)
apsimx_options(warn.versions = FALSE)

run.example.tests <- get(".run.local.tests", envir = apsimx.options)

examples <- c("AgPasture", "Barley", "Canola", "Chickpea", 
           "ControlledEnvironment", "Eucalyptus",
           "EucalyptusRotation",
           "Maize", "Mungbean",
           "Oats", "OilPalm", "Peanut", "Pinus", "Potato",
           "RedClover",
           "Rotation", "Slurp", "Sorghum",
           "Soybean", "Sugarcane", "Wheat", "WhiteClover")

tmp.dir <- tempdir()

setwd(tmp.dir)

if(run.example.tests){
  
  ex.dir <- auto_detect_apsimx_examples()
  
  start <- Sys.time()
  for(i in examples){
    ex.tst <- apsimx_example(i)
    cat("Ran: ", i, "\n")
  }
  end <- Sys.time()
  cat("Total time:", end - start, "\n")
    
  if(.Platform$OS.type == "unix"){
    cat("APSIM-X version:", apsim_version(which = "inuse"),"\n")
  }else{
    cat("APSIM-X version:", as.vector(apsim_version(which = "inuse")[[2]]),"\n")
  }
}

#'
#' This function requires the \CRANpkg{daymetr} package. This function should replace 
#' the \code{\link{get_daymet_apsim_met}} function.
#'
#' @title Get DAYMET data for an APSIM met file
#' @description Uses \code{\link[daymetr]{download_daymet}} from the \CRANpkg{daymetr} package to download data to create an APSIM met file.
#' @name get_daymet2_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param years a numeric vector of years to extract (c(start, end)). For example, f you need 2012 through 2015, use c(2012, 2015).
#' @param wrt.dir write directory (default is the current directory)
#' @param filename file name for writing out to disk
#' @param silent argument passed to \code{\link[daymetr]{download_daymet}}
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk. The variable \sQuote{srad} as downloaded from
#' daymet is average solar radiation, so it is converted to total. 
#' Daily total radiation (MJ/m2/day) can be calculated as 
#' follows: ((srad (W/m2) * dayl (s/day)) / 1,000,000) \cr
#' Vapor Pressure Deficit (vp) should be in hecto Pascals
#' @source The data is retrieved using the \CRANpkg{daymetr} package. For the original
#' source see: https://daymet.ornl.gov/
#' @return It returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @export
#' @examples 
#' \dontrun{
#' require(daymetr)
#' ## I write to a temp directory but replace as needed
#' dmet12 <- get_daymet2_apsim_met(lonlat = c(-93,42), years = 2012)
#' summary(dmet12)
#' ## Check for reasonable ranges 
#' check_apsim_met(dmet12)
#' }
#' 

get_daymet2_apsim_met <- function(lonlat, years, wrt.dir = ".", filename, silent = FALSE){
  
  if(!requireNamespace("daymetr", quietly = TRUE)){
    warning("The daymetr is required for this function")
    return(NULL)
  }
  
  if(length(years) == 1) years <- rep(years, 2)
  if(length(years) > 2) stop("years should be a numeric vector of size = 2", call. = FALSE)
  
  if(missing(filename)) filename <- "noname.met"
  
  if(!grepl(".met", filename, fixed=TRUE)) stop("filename should end in .met")
  
  dmet0 <- daymetr::download_daymet(site = filename,
                                      lat = lonlat[2], lon = lonlat[1],
                                      start = years[1], end = years[2],
                                      path = wrt.dir, silent = silent,
                                      internal = TRUE)

  dmet.year <- dmet0$data$year
  dmet.doy <- dmet0$data$yday
  dmet.dayl <- dmet0$data$dayl..s. 
  dmet.srad <- dmet0$data$srad..W.m.2.
  dmet.srad.total <- dmet.dayl * dmet.srad * 1e-6
  dmet.tmax <- dmet0$data$tmax..deg.c.
  dmet.tmin <- dmet0$data$tmin..deg.c.
  dmet.prcp <- dmet0$data$prcp..mm.day.
  dmet.vp <- dmet0$data$vp..Pa. * 0.01
  dmet.swe <- dmet0$data$swe..kg.m.2.
      
  dmet <- data.frame(dmet.year, dmet.doy, dmet.srad.total, dmet.tmax, dmet.tmin, dmet.prcp, dmet.vp, dmet.swe)    
  
  names(dmet) <- c("year","day","radn","maxt","mint","rain","vp","swe")
  
  units <- c("()","()","(MJ/m2/day)","(oC)","(oC)","(mm)","(hPa)","(kg/m2)")
  
  comments <- paste("!data from DayMet obtained through daymetr R pacakge. retrieved: ",Sys.time())
  
  attr(dmet, "filename") <- filename
  attr(dmet, "site") <- paste("site = ", sub(".met","", filename, fixed = TRUE))
  attr(dmet, "latitude") <- paste("latitude =",lonlat[2])
  attr(dmet, "longitude") <- paste("longitude =",lonlat[1])
  attr(dmet, "tav") <- paste("tav =",mean(colMeans(dmet[,c("maxt","mint")],na.rm=TRUE),na.rm=TRUE))
  attr(dmet, "colnames") <- names(dmet)
  attr(dmet, "units") <- units
  attr(dmet, "comments") <- comments
  ## No constants
  class(dmet) <- c("met", "data.frame")
  
  dmet <- amp_apsim_met(dmet)
  
  if(filename != "noname.met"){
    write_apsim_met(dmet, wrt.dir = wrt.dir, filename = filename)
  }  

  return(invisible(dmet))  
}

#'
#' This function requires the \CRANpkg{daymetr} package. This function should replace 
#' the \code{\link{get_daymet_apsim_met}} function.
#'
#' @title Get DAYMET data for an APSIM met file
#' @description Uses \code{\link[daymetr]{download_daymet}} from the \CRANpkg{daymetr} package to download data to create an APSIM met file.
#' @name get_daymet_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param years a numeric vector of years to extract (c(start, end)). For example, f you need 2012 through 2015, use c(2012, 2015).
#' @param wrt.dir write directory (default is the current directory)
#' @param filename file name for writing out to disk
#' @param silent argument passed to \code{\link[daymetr]{download_daymet}}
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk. The variable \sQuote{srad} as downloaded from
#' daymet is average solar radiation, so it is converted to total. 
#' Daily total radiation (MJ/m2/day) can be calculated as 
#' follows: ((srad (W/m2) * dayl (s/day)) / 1,000,000) \cr
#' Vapor Pressure Deficit (vp) should be in hecto Pascals
#' @source The data is retrieved using the \CRANpkg{daymetr} package. For the original
#' source see: https://daymet.ornl.gov/
#' @return It returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @export
#' @examples 
#' \dontrun{
#' require(daymetr)
#' ## I write to a temp directory but replace as needed
#' dmet12 <- get_daymet_apsim_met(lonlat = c(-93,42), years = 2012)
#' summary(dmet12)
#' ## Check for reasonable ranges 
#' check_apsim_met(dmet12)
#' }
#' 
get_daymet_apsim_met <- get_daymet2_apsim_met

## These are tests that I will run to reproduce the part of the vignette
## That depends on APSIM-X. In this way the vignette can be build independently
## of APSIM-X, but the testing side-effect of the vignette is still present
require(apsimx)
apsimx_options(warn.versions = FALSE)

extd.dir <- system.file("extdata", package = "apsimx")

run.vignette <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.vignette){

  ## Copy 'Maize.apsimx' to 'tmp.dir'
  ex.dir <- auto_detect_apsimx_examples()
  file.copy(from = paste0(ex.dir,"/","Maize.apsimx"),
            to = tmp.dir, copy.mode = FALSE)
  maize <- apsimx("Maize.apsimx", src.dir = tmp.dir)

  ## Running 'Wheat' example
  file.copy(from = paste0(ex.dir,"/","Wheat.apsimx"),
            to = tmp.dir, copy.mode = FALSE)
  wheat <- apsimx("Wheat.apsimx", src.dir = tmp.dir)

  ## Running 'MaizeSoybean' example
  file.copy(from = paste0(extd.dir,"/","Ames.met"),
            to = tmp.dir)
  file.copy(from = paste0(extd.dir,"/","MaizeSoybean.apsimx"),
            to = tmp.dir)
  maize.soy <- apsimx("MaizeSoybean.apsimx", src.dir = tmp.dir)

  ## The code below should only be run from my laptop and only when a new
  ## version of APSIM-X is released
  ## This updates the 'Wheat' and 'Maize' example files in case they have
  ## changed

  aiu <- apsim_version(which = "inuse", verbose = FALSE)

  ## Need to change this for new versions of APSIM-X
  ## apsimx.new.version <- as.numeric(strsplit(aiu, ".", fixed = TRUE)[[1]][4])

  my.laptop <- grepl("fernandomiguez", Sys.info()[["user"]])
  ## This is only for my laptop
  if(my.laptop && FALSE){
    my.extd <- "~/Dropbox/apsimx/inst/extdata"
    file.copy(from = paste0(ex.dir,"/","Maize.apsimx"),
              to = my.extd, copy.mode = FALSE, 
              overwrite = TRUE)
    file.copy(from = paste0(ex.dir,"/","Wheat.apsimx"),
              to = my.extd, copy.mode = FALSE,
              overwrite = TRUE)
  }
}

## Testing the factorial feature for inspect_apsim
require(apsimx)

run.factorial.apsim <- FALSE

extd.dir <- system.file("extdata", package = "apsimx")

tmp.dir <- tempdir()

if(run.factorial.apsim){
  
  ## There are two folders in 'maize-factorial'
  inspect_apsim("maize-factorial.apsim", src.dir = extd.dir,
                node = "Clock", parm = "start_date",
                root = "IA-CC_Canisteo_No-Cover",
                print.path = TRUE)
  
  inspect_apsim("maize-factorial.apsim", src.dir = extd.dir,
                node = "Clock", parm = "start_date",
                root = "IA-CC_Canisteo_Cover",
                print.path = TRUE)
  
  ## Edit them
  pp1 <- inspect_apsim("maize-factorial.apsim", src.dir = extd.dir,
                        node = "Clock", parm = "start_date",
                        root = "IA-CC_Canisteo_No-Cover",
                        print.path = TRUE)
  
  edit_apsim("maize-factorial.apsim", 
             src.dir = extd.dir,
             wrt.dir = tmp.dir,
             node = "Other", 
             parm.path = pp1, 
             value = "01/02/1999")
  
  inspect_apsim("maize-factorial-edited.apsim", 
                src.dir = tmp.dir,
                node = "Clock", parm = "start_date",
                root = "IA-CC_Canisteo_No-Cover",
                print.path = TRUE)
  
  ## Second example
  pp2 <- inspect_apsim("maize-factorial.apsim", src.dir = extd.dir,
                       node = "Clock", parm = "start_date",
                       root = "IA-CC_Canisteo_No-Cover",
                       print.path = TRUE)
 
  edit_apsim("maize-factorial.apsim", 
             src.dir = extd.dir,
             wrt.dir = tmp.dir,
             node = "Other", 
             parm.path = pp2, 
             value = "01/02/2009")
  
  inspect_apsim("maize-factorial-edited.apsim", 
                src.dir = tmp.dir,
                node = "Clock", parm = "start_date",
                root = "IA-CC_Canisteo_No-Cover",
                print.path = TRUE)
  
  
}

## Testing inspect for Classic
require(apsimx)

apsimx_options(warn.versions = FALSE)

## Run inspect tests
## inspect_apsim returns relative paths (an xpath expression), but still a character
## inspect_apsim_xml returns absolute paths 

run.inspect.tests <- get(".run.local.tests", envir = apsimx.options)

if(run.inspect.tests){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  ## Clock and clock path
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Clock")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Clock", 
                      parm = "start", print.path = TRUE)
  if(pp != ".//clock/start_date") stop("Clock path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Weather and weather path
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Weather")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Weather", 
                      parm = "filename", print.path = TRUE)
  if(pp != ".//metfile/filename") stop("Weather path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Soil Metadata
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Metadata")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "Metadata", parm = "LocalName", print.path = TRUE)
  if(pp != ".//Soil/LocalName") stop("Soil Metadata path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Soil Water
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Water")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "Water", parm = c("Barley", "KL"), print.path = TRUE)
  if(pp != ".//Soil/Water/SoilCrop[@name='Barley']/KL") stop("Soil Water KL path is incorrect")
  
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Water",
                parm = "SWCON")
  pp <-   inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Water",
                        parm = "SWCON", print.path = TRUE)
  if(pp != ".//Soil/SoilWater/SWCON") stop("Soil Water SWCON path is incorrect")
  ## In this case there are multiple elements for KL, so the path needs to be different
  ## If we want to identify a single value. There are many KLs, one for each crop, and there 
  ## are 8 crops here
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = ".//SoilCrop[@name='Barley']/KL/double[1]")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "Water", parm = "SummerCona", print.path = TRUE)
  if(pp != ".//Soil/SoilWater/SummerCona") stop("Soil Water SummerCona path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Soil Organic Matter
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "OrganicMatter")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "OrganicMatter", parm = "OC", print.path = TRUE)
  if(pp != ".//Soil/SoilOrganicMatter/OC") stop("Soil Organic Matter OC path is incorrect")
  ## In this case there are multiple values of OC, so we need to select one
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = paste0(pp, "/double[1]"))
  ### Analysis
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Analysis")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "Analysis", parm = "PH", print.path = TRUE)
  if(pp != ".//Soil/Analysis/PH") stop("Soil Analysis PH path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = paste0(pp, "/double[1]"))
  ## Initial Water
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "InitialWater")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "InitialWater", parm = "FractionFull", print.path = TRUE)
  if(pp != ".//Soil/InitialWater/FractionFull") stop("Soil InitialWater FractionFull path is incorrect")
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Sample
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Sample")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil",
                      soil.child = "Sample", parm = "NO3", print.path = TRUE)
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = paste0(pp, "/double[1]"))
  ## Surface Organic Matter
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "SurfaceOrganicMatter")
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "SurfaceOrganicMatter",
                      parm = "type", print.path = TRUE)
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = pp)
  ## Crop or Manager
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",NA)) 
  ## Crop with Parameter, This returns the full absolute path
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",7),
                      print.path = TRUE)
  ## Strangely the full path does not work here, but just "cultivar" does
  inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, parm = "cultivar")
  
}

#'
#' In general, this function is used to inspect one parameter at a time. There are some exceptions. \cr
#'
#' When node equals \sQuote{Other} there are several options. If \sQuote{parm} is not specified the structure
#' of the simulation file will be returned. In this case, the parameter to print is typically just \sQuote{Simulations}.
#' This option is useful when the intention is to show the simulation structure to pick a root presumably. \sQuote{parm}
#' can be set as 0, 1, 2 or 3 for different levels.
#' \sQuote{parm} can also be a list with integers, such as \sQuote{list(1, 2, 3)}. If zero is included, available elements
#  will be displayed. At the moment, the maximum length of this list is 7.
#' If a parameter is specified the function will try to \sQuote{guess} the root elements from the parameter path supplied.
#'
#' @title Inspect an .apsimx (JSON) file
#' @name inspect_apsimx
#' @description inspect a JSON apsimx file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
#' @param file file ending in .apsimx to be inspected (JSON)
#' @param src.dir directory containing the .apsimx file to be inspected; defaults to the current working directory
#' @param node specific node to be inspected either \sQuote{Clock}, \sQuote{Weather},
#' \sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop},
#'  \sQuote{Manager}, \sQuote{Operations} or \sQuote{Other}
#' @param soil.child specific soil component to be inspected. The options vary depending on what is available (see details)
#' @param parm parameter to refine the inspection of the \sQuote{manager} list(\sQuote{parm},\sQuote{position}), use \sQuote{NA} for all the positions. \sQuote{parm} can be a regular expression for partial matching.
#' @param digits number of decimals to print (default 3). Not used now because everything is a character.
#' @param print.path whether to print the path to the specific parameter. Useful to give the later editing. (Also returned as \sQuote{invisible})
#' @param root root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.
#' @details This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
#'          To investigate the available \sQuote{soil.childs} specify \sQuote{Soil} for \sQuote{node} and do not specify the \sQuote{soil.child}.
#' @return prints a table with inspected parameters and values (and \sQuote{parm path} when \sQuote{print.path} = TRUE).
#' @export
#' @examples
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Clock")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Weather")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Metadata")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Physical")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "SoilWater")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Organic")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Chemical")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "InitialWater")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "InitialN")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "SurfaceOrganicMatter")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "MicroClimate")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager")
#' inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Report")
#'
#' ## Examples of using node = "Other"
#' extd.dir <- system.file("extdata", package = "apsimx")
#'
#' ## When parm is not provided
#' inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir, node = "Other")
#' ## When parm = 2
#' inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
#'                node = "Other", parm = 2)
#' ## When parm = 3
#' inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
#'                node = "Other", parm = 3)
#' ## When parm is a path
#' inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
#'                node = "Other", parm = ".Simulations.Simulation")
#' ## When parm is a list with numbers (integers)
#' pp <- inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
#'                      node = "Other", parm = list(1, 1, 5),
#'                      print.path = TRUE)
#' ## Same as above, but with zero prints possible options
#' inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
#'                 node = "Other", parm = list(1, 1, 5, 0))
#'
#' ## It is possible to look into folders using this method
#' inspect_apsimx("maize-manager-folder.apsimx", node = "Other", src.dir = extd.dir,
#'                parm = list("Manager", "Fertiliser", "Amount"))
#'
#'
#'
#' }
#'

inspect_apsimx <- function(file = "", src.dir = ".",
                           node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager","Report", "Operations", "Other"),
                           soil.child = c("Metadata", "Water", "InitialWater",
                                          "Chemical", "Physical", "Analysis", "SoilWater",
                                          "InitialN", "CERESSoilTemperature", "SoilTemperature",
                                          "Sample", "Solute", "NO3", "NH4", "Urea",
                                          "Nutrient", "Organic", "Swim3"),
                           parm = NULL,
                           digits = 3,
                           print.path = FALSE,
                           root){
  #### Beginning of function ----
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }

  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)

  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to inspect.")
  }

  node <- match.arg(node)
  soil.child <- match.arg(soil.child)

  if(soil.child %in% c("Nutrient")) stop("Not implemented yet", call. = FALSE)

  ## This matches the specified file from a list of files
  ## Notice that the .apsimx extension will be added here
  file <- match.arg(file, file.names)

  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))

  find.root <- TRUE
  other.parm.flag <- 1
  if(node == "Other" && is.numeric(parm)) parm <- as.integer(parm)
  parm.path.0 <- paste0(".", apsimx_json$Name) ## Root
  ## I think that everything I might want to look at
  ## is under this Children/Children node
  ## It looks like I need to 'find' the "Models.Core.Simulation" node
  fcsn <- grep("Models.Core.Simulation", apsimx_json$Children, fixed = TRUE)

  ## The code below is confusing. There are some reasons for it.
  ## It is activated if node == "Other" and there is more than on simulation
  ## Or if node == "Other" and parm is either NULL or an integer
  if((node == "Other" && length(fcsn) > 1) || (node == "Other" && (is.null(parm) || is.integer(parm)))){
    parm.path <- parm.path.0
    other.parm.flag <- -1
    find.root <- FALSE
    ## Process 'parm'
    if(is.null(parm) || is.integer(parm)){
      if(is.null(parm)) parm <- 1L
      if(length(parm) > 1)
        stop("'parm' should be of length = 1", call. = FALSE)
      if(parm == 0){
        cat("Level 0:", gsub(".", "", parm.path.0, fixed = TRUE), "\n")
      }
      ## If parm is null there are different levels of display
      if(parm == 1){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(root.names.level.1) - 1)),
                           second_level = root.names.level.1)
        print(knitr::kable(pdat, row.names = TRUE))
      }
      ## If parm == 2
      if(parm == 2){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        second.level.names <- NULL
        third.level.names <- NULL
        for(i in seq_along(root.names.level.1)){
          chld.len <- length(apsimx_json$Children[[i]]$Children)
          rep.times <- ifelse(chld.len == 0, 1, chld.len)
          second.level.names <- c(second.level.names, c(root.names.level.1[i], rep(".", times = rep.times - 1)))
          ## Need to generate third level names
          third.level.names.0 <- sapply(apsimx_json$Children[[i]]$Children, FUN = function(x) x$Name)
          if(length(third.level.names.0) == 0) third.level.names.0 <- "."
          third.level.names.1 <- c(third.level.names, third.level.names.0)
          third.level.names <- unlist(third.level.names.1)
        }
        ### Is this table good enough?
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(second.level.names) - 1)),
                           second_level = second.level.names,
                           third_level = third.level.names)
        print(knitr::kable(pdat, row.names = TRUE))
      }
      ## If parm == 3
      if(parm == 3){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        second.level.names <- NULL
        third.level.names <- NULL
        fourth.level.names <- NULL
        k <- 1
        for(i in seq_along(root.names.level.1)){
          chld.len <- length(apsimx_json$Children[[i]]$Children)
          rep.times <- ifelse(chld.len == 0, 1, chld.len)
          second.level.names <- c(second.level.names, c(root.names.level.1[i], rep(".", times = rep.times - 1)))
          ## Need to generate third level names
          third.level.names.0 <- sapply(apsimx_json$Children[[i]]$Children, FUN = function(x) x$Name)
          if(length(third.level.names.0) == 0) third.level.names.0 <- "."
          for(j in seq_along(third.level.names.0)){
            if(length(third.level.names.0) == 1){
              if(third.level.names.0 == "."){
                third.level.names <- c(third.level.names, ".")
                fourth.level.names <- c(fourth.level.names, ".")
                next
              }
            }
            fourth.level <- apsimx_json$Children[[i]]$Children
            if(length(fourth.level[[j]]) == 0){
              third.level.names <- c(third.level.names, ".")
              fourth.level.names <- c(fourth.level.names, ".")
              next
            }
            if(length(fourth.level[[j]]$Children) == 0){
              second.level.names <- c(second.level.names, rep(".", times = length(names(fourth.level[[j]])) - 1))
              third.level.names <- c(third.level.names, c(third.level.names.0[j], rep(".", times = length(names(fourth.level[[j]])) - 1)))
              fourth.level.names <- c(fourth.level.names, names(fourth.level[[j]]))
            }else{
              second.level.names <- c(second.level.names, rep(".", times = length(names(fourth.level[[j]])) - 1))
              fourth.level.names <- c(fourth.level.names, names(fourth.level[[j]]))
              third.level.names <- c(third.level.names, third.level.names.0[j],
                                     rep(".", times = length(names(fourth.level[[j]])) - 1))
            }
          }
        }
        ### Is this table good enough?
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(fourth.level.names) - 1)),
                           second_level = second.level.names,
                           third_level = third.level.names,
                           fourth_level = fourth.level.names)
        print(knitr::kable(pdat, row.names = TRUE))
      }
      if(parm == 4) stop("Not implemented yet")
    }else{
      ## This gets triggered if there are multiple simulations
      ## and parm is a character. If it is a list it will be
      ## handled by the code in the section at the bottom
      if(!is.list(parm)){
        if(!is.character(parm))
          stop("parm should be a character string when node = 'Other' and is not a number", call. = FALSE)
        if(!grepl(".", parm, fixed = TRUE))
          stop("'parm' needs to be a json path")
        if(parm == ".")
          stop("'parm' needs to be a proper json path")
        pparm <- strsplit(parm, split = ".", fixed = TRUE)[[1]]
        root.name.level.0 <- gsub(".", "", parm.path.0, fixed = TRUE)
        if(pparm[2] != root.name.level.0)
          stop(paste("First parm element does not match:",
                     paste(root.name.level.0, collapse = " ")), call. = FALSE)
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0 || all(is.na(wroot1)))
          stop(paste("Second element of parm did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        ## Need to test if the fourth element of pparm is a node
        nodes <- c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager","Report", "Operations", "Other")
        if(pparm[4] %in% nodes){
          ## This amounts to guessing that root should be of length 1
          root <- list(pparm[3])
        }else{
          ## This amounts to guessing that pparm[4] should be the second element in
          root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                       FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          root2 <- pparm[4]
          wroot2 <- grep(as.character(root2), root.names.level.2)
          if(length(wroot2) == 0 || all(is.na(wroot2)))
            stop(paste("Third element of parm did not match:",
                       paste(root.names.level.2, collapse = " ")), call. = FALSE)
          if(pparm[5] %in% nodes){
            root <- list(pparm[3], pparm[4])
          }else{
            root.names.level.3 <- vapply(apsimx_json$Children[[wroot1]]$Children[[wroot2]]$Children,
                                         FUN = function(x) x$Name,
                                         FUN.VALUE = "character")
            root3 <- pparm[5]
            wroot3 <- grep(as.character(root3), root.names.level.3)
            if(length(wroot3) == 0)
              stop(paste("Fourth element of parm did not match:",
                         paste(root.names.level.3, collapse = " ")), call. = FALSE)
          }
        }
        parm.path <- parm
      }else{
        ### This flag is used to activate the code at the bottom of the function
        ### This is not great code, but I'm trying to get something that works for now
        other.parm.flag <- 1
      }
    }
  }

  if((length(fcsn) > 1 || !missing(root)) && find.root){
    if(missing(root)){
      cat("Simulation structure: \n")
      str_list(apsimx_json)
      stop("more than one simulation found and no root node label has been specified \n select one of the children names above", call. = FALSE)
    }else{
      ### Parse root
      root <- parse_root(root)
      if(length(root) > 3)
        stop("At the moment 3 is the maximum length for root for this function", call. = FALSE)
      if(length(root) == 1){
        nms <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                      FUN.VALUE = "character")
        fcsn <- grep(as.character(root), nms)
        if(length(fcsn) == 0 || length(fcsn) > 1){
          cat("Children:", nms, "\n")
          stop("'root' not found. Choose one of the options above", call. = FALSE)
        }
        parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
        parent.node <- apsimx_json$Children[[fcsn]]$Children
      }else{
        if(length(root) == 2){
          root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
          wcore1 <- grep(as.character(root[1]), root.node.0.names)
          if(length(wcore1) == 0 || length(wcore1) > 1){
            cat("Children:", root.node.0.names, "\n")
            stop("no root node label in position 1 found or root is not unique", call. = FALSE)
          }
          root.node.0 <- apsimx_json$Children[[wcore1]]
          root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
          wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
          if(length(wcore2) == 0 || length(wcore2) > 1)
            stop("no root node label in position 2 found or root is not unique")
          parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children
          parm.path.1 <- paste0(parm.path.0,".",apsimx_json$Children[[wcore1]]$Children[[wcore2]])
          ## Is this only a problem for factorials?
          if(length(parm.path.1) > 1){
            parm.path.1 <- paste(parm.path.0, apsimx_json$Children[[wcore1]]$Name, apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Name, sep = ".")
          }
        }
        if(length(root) == 3){
          root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
          wcore1 <- grep(as.character(root[1]), root.node.0.names)
          if(length(wcore1) == 0 || length(wcore1) > 1)
            stop("no root node label in position 1 found or root is not unique")
          root.node.0 <- apsimx_json$Children[[wcore1]]
          root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
          wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
          if(length(wcore2) == 0 || length(wcore2) > 1)
            stop("no root node label in position 2 found or root is not unique")
          root.node.1 <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]
          root.node.1.child.names <- sapply(root.node.1$Children, function(x) x$Name)
          wcore3 <- grep(as.character(root[3]), root.node.1.child.names)
          if(length(wcore3) == 0 || length(wcore3) > 1)
            stop("no root node label in position 3 found or root is not unique")
          parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children
          parm.path.1 <- paste0(parm.path.0,".",apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]])
          ### Is this a problem for factorials only?
          if(length(parm.path.1) > 1){
            parm.path.1 <- paste(parm.path.0, apsimx_json$Children[[wcore1]]$Name, apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Name,
                                 apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Name, sep = ".")
          }
        }
      }
    }
  }else{
    if(find.root){
      parent.node <- apsimx_json$Children[[fcsn]]$Children
      parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
    }
  }

  if(node == "Clock"){
    wlc <- function(x) grepl("Models.Clock", x$`$type`, ignore.case = TRUE)
    wlcl <- sapply(parent.node, FUN = wlc)
    if(all(wlcl == FALSE)){
      stop("Clock not found")
    }
    clock.node <- as.list(parent.node[wlcl])[[1]]
    start.name <- grep("start", names(clock.node), ignore.case = TRUE, value = TRUE)
    end.name <- grep("end", names(clock.node), ignore.case = TRUE, value = TRUE)
    cat("Start:", clock.node[[start.name]], "\n")
    cat("End:",  clock.node[[end.name]], "\n")
    ## It is possible for the 'start' and 'end' to be called: 'Start' and 'End'
    ## It is also possible for them to be called 'StartDate' and 'EndDate'
    ## I think APSIM-X is in a state of change and eventually this will
    ## stabilize. At the moment 'Maize' and 'Barley' do not agree.
    ## Final name is 'parm.path', but this is 1.2
    parm.path <- paste0(parm.path.1,".",parent.node[wlcl][[1]]$Name)
  }

  ## The previous creates a list
  if(node == "Weather"){
    ## Extract the list which has a component Name == "Weather"
    wlw <- function(x) grepl("Models.Climate.Weather|Models.Weather", x$`$type`)
    wlwl <- sapply(parent.node, FUN = wlw)
    if(all(wlwl == FALSE)){
      stop("Weather not found")
    }
    weather.node <- parent.node[wlwl]
    ## Select the string which has a met file
    gf1 <- function(x) grep(".met$", x, value = TRUE)
    cat("Met file:", as.character(sapply(weather.node, gf1)), "\n")
    parm.path <- paste0(parm.path.1, ".", parent.node[wlwl][[1]]$Name)
  }

  ## From here on there is an important component that lives inside
  ## 'Models.Core.Zone'
  if(find.root){
    wcz <- grepl("Models.Core.Zone", parent.node)

    ## sapply(parent.node, \(x) x$`$type`)
    ## browser()
    ### In case of multiple fields per simulation, 'Models.Core.Zone' does not exist
    ### FIXME: The code below needs more careful thinking about how I extract "Models.Core.Zone"
    ### and what does the [[1]] mean below
    if(sum(wcz) < 0.5){
      wcfz <- grepl("Models.* Models", parent.node)  ## This is for which core field zone
      if(all(wcfz)){
        core.zone.node <- parent.node
        parm.path.2 <- parm.path.1 
      }else{
        stop("Core Zone Simulation not found", call. = FALSE)
      }
    }else{
      ### What does the [[1]] imply here?
      core.zone.node <- parent.node[wcz][[1]]$Children
      parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
    }
    
    if(is.null(core.zone.node))
      stop("Core Zone Simulation not found", call. = FALSE)

  }

  if(node == "Soil"){
    ## Which soils node
    wsn <- grepl("Models.Soils.Soil", core.zone.node)
    if(all(wsn == FALSE)){
      stop("Soil not found")
    }
    soil.node <- core.zone.node[wsn]

    parm.path.2.1 <- paste0(parm.path.2, ".", soil.node[[1]]$Name)

    ## Print some basic soil information
    cat("Soil Type: ", soil.node[[1]]$SoilType, "\n")
    cat("Latitude: ", soil.node[[1]]$Latitude, "\n")
    cat("Longitude: ", soil.node[[1]]$Longitude, "\n")

    if(length(soil.node) != 1) stop("soil.node not equal to one")

    soil.children.names <- sapply(soil.node[[1]]$Children, function(x) x$Name)

    cat("Soil children:", soil.children.names, "\n")

    if(soil.child == "Metadata"){
      parm.path <- parm.path.2.1
      metadata <- NULL
      for(i in names(soil.node[[1]])){
        if(i %in% c("Name","Children","IncludeInDocumentation","Enabled","ReadOnly")) next
        val <- as.character(ifelse(is.null(soil.node[[1]][[i]]),NA,soil.node[[1]][[i]]))
        if(!is.na(val) && nchar(val) > options()$width-30) val <- paste(strtrim(val, options()$width-30),"...")
        metadata <- rbind(metadata, data.frame(parm = i, value = val))
      }

      if(missing(parm)){
        print(knitr::kable(metadata, longtable = FALSE))
      }else{
        if(!(parm %in% metadata[["parm"]])) stop("parm does not match a parameter in metadata")
        print(knitr::kable(metadata[metadata$parm == parm,]))
      }

    }else{
      ## Pick which soil component we want to look at
      ## Which is not 'Metadata"
      ## browser()
      wsiw <- "InitialWater" %in% soil.children.names
      if(soil.child %in% c("Water", "InitialWater")){
        if(soil.child == "InitialWater" && isFALSE(wsiw)){
          wsiw2 <- grep("initial water", soil.children.names, ignore.case = TRUE)
          if(length(wsiw2) > 0){
           soil.child <-  grep("initial water", soil.children.names, ignore.case = TRUE, value = TRUE)
          }else{
            wsiw3 <- grep("^Water", soil.children.names)
            if(length(wsiw3) > 0){
              soil.child <- "Water"
            }else{
              soil.child <- "InitialWater"
            }
          }
        }
      }

      if(soil.child != "Solute"){
        wsc <- which(soil.child == soil.children.names)
      }else{
        wsc <- which(soil.children.names %in% c("NO3", "NH4", "Urea"))
      }

      ### This is a bad idea probably
      if(soil.child == "Water" && length(wsc) == 2) wsc <- wsc[2]

      if(length(wsc) == 0) stop("soil.child likely not present")

      selected.soil.node.child <- soil.node[[1]]$Children[wsc]
    }

      ## For some variables now it is the time to print
      ## The code below is not strictly needed but it is here
      ## in case I need a second level of soil in the future
    first.level.soil <- c("Water", "Physical",
                          "Chemical", "Analysis", "InitialWater", "Initial water", "initial water",
                          "InitialN", "SoilWater", "SoilTemperature",
                          "CERESSoilTemperature", "Organic", "Swim3")
    if(soil.child %in% first.level.soil){
      ## Assuming there is only one 'relevant' level here
      ## This parameter level would be 2.1.1
      parm.path <- paste0(parm.path.2.1, ".", selected.soil.node.child[[1]]$Name)
      enms <- c("IncludeInDocumentation", "Enabled", "ReadOnly", "Children", "Name", "$type")
      cnms <- setdiff(names(selected.soil.node.child[[1]]), enms)
      ## print(names(selected.soil.node.child[[1]]))

      if(soil.child == "Physical")
        cnms <- c(cnms, "Crop LL", "Crop KL", "Crop XF")

      soil.d1 <- NULL
      soil.d2 <- NULL
      soil.d3 <- NULL
      col.nms <- NULL
      d3.col.nms <- NULL

      for(ii in cnms){

        tmp <- selected.soil.node.child[[1]][ii][[1]]

        if(ii %in% c("Crop LL", "Crop KL", "Crop XF")){
          for(j in seq_along(selected.soil.node.child[[1]]$Children)){
            crop.name <- gsub("Soil", "", selected.soil.node.child[[1]]$Children[[j]]$Name)
            if(ii == "Crop LL") tmp <- selected.soil.node.child[[1]]$Children[[j]]$LL
            if(ii == "Crop KL") tmp <- selected.soil.node.child[[1]]$Children[[j]]$KL
            if(ii == "Crop XF") tmp <- selected.soil.node.child[[1]]$Children[[j]]$XF
            d3.col.nms <- c(d3.col.nms, gsub("Crop", crop.name, ii))
            vals <- as.vector(unlist(tmp))
            soil.d3 <- cbind(soil.d3, vals)
          }
        }

        if(length(tmp) == 0) next
        if(length(tmp) == 1){
          soil.d1 <- rbind(soil.d1,
                           data.frame(parm = ii, value = as.character(tmp)))
        }
        if(length(tmp) > 1){
          if(!ii %in% c("Crop LL", "Crop KL", "Crop XF")){
            if(grepl("Metadata", ii)) next
            col.nms <- c(col.nms, ii)
            vals <- as.vector(unlist(tmp))
            if(is.null(vals)) vals <- rep(".", length(tmp))
            soil.d2 <- cbind(soil.d2, vals)
          }
        }
      }

      if(missing(parm)){
        ## Print first set of soil parameters
        if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))
        ## Print second set of soil parameters
        if(!is.null(soil.d2)){
          soil.d2 <- as.data.frame(soil.d2)
          names(soil.d2) <- col.nms
          print(knitr::kable(soil.d2, digits = digits))
        }
        ## Print third set of crop-soil parameters
        if(!is.null(soil.d3)){
          soil.d3 <- as.data.frame(soil.d3)
          names(soil.d3) <- d3.col.nms
          soil.d3 <- subset(soil.d3, select = sort(names(soil.d3)))
          print(knitr::kable(soil.d3, digits = digits))
        }
      }else{
        parm.physical.found <- FALSE
        ## Print first set of soil parameters
        if(!is.null(soil.d1)){
          if(parm %in% names(soil.d1)){
            ##print(knitr::kable(soil.d1[soil.d1$parm == parm,], digits = digits))
            print(knitr::kable(soil.d1[, parm, drop = FALSE], digits = digits))
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d1$parm){
            print(knitr::kable(soil.d1[soil.d1$parm == parm,], digits = digits))
            parm.physical.found <- TRUE
          }
        }
        ## Print second set of soil parameters
        if(!is.null(soil.d2)){
          soil.d2 <- as.data.frame(soil.d2)
          names(soil.d2) <- col.nms
          ## print(knitr::kable(soil.d2[soil.d2$parm == parm,], digits = digits))
          if(parm %in% names(soil.d2)){
            print(knitr::kable(soil.d2[, parm, drop = FALSE], digits = digits))
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d2$parm){
            print(knitr::kable(soil.d2[soil.d2$parm == parm,], digits = digits))
            parm.physical.found <- TRUE
          }
        }
        ## Print third set of crop-soil parameters
        if(!is.null(soil.d3)){
          soil.d3 <- as.data.frame(soil.d3)
          names(soil.d3) <- d3.col.nms
          soil.d3 <- subset(soil.d3, select = sort(names(soil.d3)))
          ##print(knitr::kable(soil.d3[soil.d3$parm == parm,], digits = digits))
          if(parm %in% names(soil.d3)){
            print(knitr::kable(soil.d3[, parm, drop = FALSE], digits = digits))
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d3$parm){
            print(knitr::kable(soil.d3[soil.d3$parm == parm,], digits = digits))
            parm.physical.found <- TRUE
          }
        }
        if(!parm.physical.found)
          stop("Soil physical parameter not found", call. = FALSE)
      }
    }
  }
  #### Solute ----
  second.level.soil <- c("Solute", "NO3", "NH4", "Urea")

  if(soil.child %in% second.level.soil){

    soil.d2 <- NULL
    soil.d3 <- NULL
    enms <- c("IncludeInDocumentation", "Enabled", "ReadOnly", "Children", "Name", "$type")
    ssnc <- selected.soil.node.child

    if(soil.child %in% c("Solute", "NO3", "NH4", "Urea")){

      solutes <- sapply(ssnc, function(x) x$Name)
      parm.path <- paste0(parm.path.2.1, ".", soil.child)

      if(missing(parm)){
        cat("Solutes:", solutes, "\n")
      }else{
        if(soil.child == "Solute"){
          sel.parm <- grep(parm[[1]], solutes)
          if(length(sel.parm) == 0)
            stop("'parm' should be one of: ", paste(solutes, collapse = ", "), call. = FALSE)
          solutes <- parm[[1]]
        }
        if(soil.child %in% c("NO3", "NH4", "Urea")){
          ## In this case 'parm' could be either Thickness, InitialValues
          ## or some of the other ones.
          parm.path <- paste0(parm.path.2.1, ".", soil.child)
        }
      }

      for(j in seq_along(solutes)){
        cat("\nSolute:", solutes[j], "\n")
        ssnc.solute <- ssnc[[j]]
        if(length(unlist(ssnc.solute$Thickness)) != length(unlist(ssnc.solute$InitialValues))){
          cat("Length of Thickness:", length(unlist(ssnc.solute$Thickness)), "\n")
          cat("Length of InitialValues:", length(unlist(ssnc.solute$InitialValues)), "\n")
          stop("Length of 'Thickness' does not match length of 'InitialValues'", call. = FALSE)
        }
        soil.d1 <- data.frame(Thickness = unlist(ssnc.solute$Thickness),
                              InitialValues = unlist(ssnc.solute$InitialValues))
        if(missing(parm)){
          if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))
        }else{
          if(length(parm) == 1){
            if(parm %in% c("Thickness", "InitialValues")){
              soil.d1.s <- soil.d1[, parm, drop = FALSE]
              if(!is.null(soil.d1)) print(knitr::kable(soil.d1.s, digits = digits))
            }else{
              ### Parm is of length equal to 1 and is not "Thickness" or "InitialValues"
              if(parm %in% c("NO3", "NH4", "Urea")){
                if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))
              }
            }
          }
        }

        cnms <- setdiff(names(ssnc.solute), enms)
        for(k in cnms){
          if(k %in% c("Thickness", "InitialValues")) next
          tmp <- ssnc.solute[k][[1]]
          if(is.null(tmp)) tmp <- ""
          soil.d2 <- rbind(soil.d2,
                           data.frame(parm = k, value = as.character(tmp)))
        }
        if(missing(parm)){
          if(!is.null(soil.d2)) print(knitr::kable(soil.d2, digits = digits))
        }else{
          if(length(parm) == 1){
            if(soil.child == "Solute"){
              if(!parm %in% solutes)
                stop("'parm' should be one of:", solutes, call. = FALSE)
              if(!is.null(soil.d2)) print(knitr::kable(soil.d2, digits = digits))
            }else{
              if(!parm %in% c("Thickness", "InitialValues")){
                if(!parm %in% soil.d2$parm)
                  stop("'parm' should be one of:", paste(soil.d2$parm, collapse = " "), call. = FALSE)
                soil.d2.s <- soil.d2[soil.d2$parm == parm, , drop = FALSE]
                if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits))
              }
            }
          }
          if(is.list(parm)){
            if(length(parm) != 2)
              stop("'parm' should be a list of length 2", call. = FALSE)
            if(is.numeric(parm[[2]])){
              if(parm[[2]] < 1 || parm[[2]] > nrow(soil.d2))
                stop("The second elment of 'parm' should be a number between 1 and ", nrow(soil.d2), call. = FALSE)
              soil.d2.s <- soil.d2[parm[[2]], , drop = FALSE]
            }
            if(is.character(parm[[2]])){
              if(!parm[[2]] %in% soil.d2$parm)
                stop("The second element of 'parm' should be one of:", soil.d2$parm, call. = FALSE)
              soil.d2.s <- soil.d2[soil.d2$parm == parm[[2]], , drop = FALSE]
            }
            if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits))
          }
        }
        soil.d2 <- NULL
      }
    }
  }

  if(node == "SurfaceOrganicMatter"){
    ## Which is 'SurfaceOrganicMatter'
    wsomn <- grepl("Models.Surface.SurfaceOrganicMatter", core.zone.node)
    if(all(wsomn == FALSE)){
      stop("SurfaceOrganicMatter not found")
    }
    som.node <- core.zone.node[wsomn][[1]]

    parm.path <- paste0(parm.path.2,".",som.node$Name)
    ## The relevant components might be unpredictable
    ## Will need to find a better method in the future
    names.som.node0 <- names(som.node)
    som.names.table <- c("InitialResidueName", "InitialResidueType", "InitialResidueMass",
                         "InitialStandingFraction", "InitialCPR", "InitialCNR")
    wsomnn <- names.som.node0 %in% som.names.table
    values.som.node0 <- unlist(som.node)
    wsomvn <- names(values.som.node0) %in% som.names.table
    som.d <- data.frame(parm = names(som.node)[wsomnn],
                        value = as.vector(values.som.node0[wsomvn]))

    if(missing(parm)){
      print(knitr::kable(som.d, digits = digits))
    }else{
      if(parm %in% som.d$parm){
        print(knitr::kable(som.d[som.d$parm == parm, ], digits = digits))
      }else{
        stop("Surface OM parameter not found", call. = FALSE)
      }
    }
  }

  if(node == "MicroClimate"){
    ## Which is 'MicroClimate'
    ## This only works if it is under Field (old versions, before 2023-12-10)
    wmcn <- grepl("Models.MicroClimate", core.zone.node)
    mcincz <- TRUE ## MicroClimate in core.zone
    if(all(wmcn == FALSE)){
      wmcnf <- function(x) grepl("Models.MicroClimate", x$`$type`)
      wmcn <- sapply(parent.node, FUN = wmcnf)
      mcincz <- FALSE
      if(all(wmcn == FALSE))
         stop("MicroClimate not found")
    }
    if(mcincz){
      microclimate.node <- core.zone.node[wmcn][[1]]
    }else{
      microclimate.node <- parent.node[wmcn][[1]]
    }

    parm.path <- paste0(parm.path.2,".", microclimate.node$Name)

    microclimate.d <- data.frame(parm = names(microclimate.node)[2:9],
                                 value = as.vector(unlist(microclimate.node)[2:9]))

    if(missing(parm)){
      print(knitr::kable(microclimate.d, digits = digits))
    }else{
      if(parm %in% microclimate.d$parm){
        print(knitr::kable(microclimate.d[microclimate.d$parm == parm, ], digits = digits))
      }else{
        stop("MicroClimate parameter not found", call. = FALSE)
      }
    }
  }

  if(node == "Crop"){
    ## Which is 'Crop'
    wmmn <- grepl("Models.Manager", core.zone.node)
    if(all(wmmn == FALSE)){
      stop("Crop not found")
    }
    manager.node <- core.zone.node[wmmn]

    #cat("Manager Name:",names(manager.node[[1]]),"\n")
    ## Which element has the crop information?
    wcn <- grepl("CultivarName", manager.node)
    crop.node <- manager.node[wcn][[1]]$Parameters

    ## This would be 2.1.1
    parm.path <- paste0(parm.path.2,".",manager.node[wcn][[1]]$Name)

    mat <- matrix(NA, nrow = length(crop.node), ncol = 2,
                  dimnames = list(NULL,c("parm","value")))
    j <- 1
    for(i in 1:length(crop.node)){
      mat[j,1] <- crop.node[[i]]$Key
      mat[j,2] <- crop.node[[i]]$Value
      j <- j + 1
    }

    print(knitr::kable(as.data.frame(mat), digits = digits))
  }

  if(node == "Manager"){
    wmmn <- grepl("Models.Manager", core.zone.node)
    if(all(wmmn == FALSE)){
      stop("Manager not found")
    }
    manager.node <- core.zone.node[wmmn]
    parm.path <- parm.path.2
    ## Print available Manager components
    manager.node.names <- sapply(manager.node, FUN = function(x) x$Name)
    cat("Management Scripts: ", manager.node.names,"\n\n")

    if(!is.null(parm)){
      parm1 <- parm[[1]]
      position <- parm[[2]]
      find.manager <- grep(parm1, manager.node.names, ignore.case = TRUE)
      selected.manager.node <- manager.node.names[find.manager]
      parm.path <- paste0(parm.path.2, ".", selected.manager.node)

      if(is.na(position)){
        ms.params <- manager.node[[find.manager]]$Parameters
        if(length(ms.params) == 0) warning("parameter not found")

        mat <- matrix(NA, ncol=2, nrow = length(ms.params),
                      dimnames = list(NULL, c("parm", "value")))

        if(length(ms.params) > 0){
          for(j in 1:length(ms.params)){
            mat[j,1] <- ms.params[[j]]$Key
            mat[j,2] <- ms.params[[j]]$Value
          }
        }
        cat("Name: ", selected.manager.node, "\n")
        print(knitr::kable(as.data.frame(mat), digits = digits))
        cat("\n")
      }

      if(!is.na(position)){
        ms.params <- manager.node[[find.manager]]$Parameters
        if(length(ms.params) == 0) warning("no parameters found")
        mat <- matrix(NA, ncol=2, nrow = length(position),
                      dimnames = list(NULL, c("parm", "value")))

        k <- 1
        for(j in 1:length(ms.params)){
          if(j == position){
            mat[k,1] <- ms.params[[j]]$Key
            mat[k,2] <- ms.params[[j]]$Value
            k <- k + 1
          }
        }
        cat("Name: ", selected.manager.node,"\n")
        parm2 <- ms.params[[position]]$Key
        cat("Key:", ms.params[[position]]$Key, "\n")
        print(knitr::kable(as.data.frame(mat), digits = digits))
        cat("\n")
      }
    }
  }

  if(node == "Operations"){
    won <- grepl("Models.Operations", core.zone.node)
    if(length(won) == 0)
      stop("Operations node not found", call. = FALSE)
    operations.node <- core.zone.node[won]

    if(length(operations.node) > 1)
      stop("Not ready to handle multiple 'Operations'", call. = FALSE)

    if(is.null(operations.node[[1]]$Operation))
      stop("'Operation' child node not found", call. = FALSE)

    len.op <- length(operations.node[[1]]$Operation)
    op.mat <- matrix(nrow = len.op, ncol = 3,
                     dimnames = list(NULL, c("Date", "Action", "Line")))

    for(i in seq_len(len.op)){
      op.mat[i, ] <- c(operations.node[[1]]$Operation[[i]]$Date,
                       operations.node[[1]]$Operation[[i]]$Action,
                       operations.node[[1]]$Operation[[i]]$Line)
    }

    if(is.null(parm)){
      parm.path <- paste0(parm.path.2, ".Operations")
      print(knitr::kable(as.data.frame(op.mat), digits = digits))
    }else{
      parm.path.3 <- paste0(parm.path.2, ".Operations")
      parm.path <- paste(parm.path.3, parm[[1]], parm[[2]], sep = ".")
      op.mat.dat <- as.data.frame(op.mat)
      op.mat.dat.parm <- op.mat.dat[parm[[1]], parm[[2]], drop = FALSE]
      print(knitr::kable(op.mat.dat.parm, digits = digits))
    }
  }

  if(node == "Report"){
    wrn <- grepl("Models.Report", core.zone.node)
    if(all(wrn == FALSE)){
      stop("Report not found")
    }
    report.node <- core.zone.node[wrn]
    parm.path <- parm.path.2
    ## Print available Manager components
    report.node.names <- sapply(report.node, FUN = function(x) x$Name)

    tmp <- vector("list", length = length(report.node.names))
    for(i in 1:length(report.node)){
      ## Variable Names
      vn <- as.data.frame(unlist(report.node[[i]]$VariableNames))
      names(vn) <- "VariableNames"
      en <- as.data.frame(unlist(report.node[[i]]$EventNames))
      names(en) <- "EventNames"
      tmp[[i]] <- list(vn = vn, en = en)
    }

    if(missing(parm)){
      for(i in 1:length(tmp)){
        cat("Report name:", report.node[[i]]$Name, "\n")
        print(knitr::kable(tmp[[i]]$vn))
        print(knitr::kable(tmp[[i]]$en))
        cat("\n")
      }
      parm.path <- paste0(parm.path.2,".", report.node.names)
    }else{
        if(!is.list(parm)){
          if(length(report.node.names) > 1)
            stop("More than one Report is present. Use a list to choose one.")
          if(!grepl(parm, "VariableNames") && !grepl(parm, "EventNames"))
            stop("parm should contain either VariableNames or EventNames")
          if(parm == "VariableNames") print(knitr::kable(tmp[[i]]$vn))
          if(parm == "EventNames") print(knitr::kable(tmp[[i]]$en))
          parm.path <- paste0(parm.path.2,".", report.node.names)
       }else{
          if(!any(grepl(parm[[1]], report.node.names)))
            stop("the first element of parm should match a report name")
          wr2p <- grep(parm[[1]], report.node.names)
          if(is.na(parm[[2]])){
            print(knitr::kable(tmp[[wr2p]]$vn))
            print(knitr::kable(tmp[[wr2p]]$en))
            parm.path <- paste0(parm.path.2,".", report.node.names[wr2p])
            position <- NA
          }else{
            if(grepl(parm[[2]],"VariableNames")) print(knitr::kable(tmp[[wr2p]]$vn))
            if(grepl(parm[[2]], "EventNames")) print(knitr::kable(tmp[[wr2p]]$en))
            parm.path <- paste0(parm.path.2,".", report.node.names[wr2p])
            parm1 <- grep(parm[[1]], report.node.names, value = TRUE)
            parm2 <- grep(parm[[2]], c("VariableNames", "EventNames"), value = TRUE)
            position <- wr2p
          }
       }
    }
  }

  #### Option for 'Other' character or 'list' ----
  #### I might have to re-factor this code someday.
  #### The list with numbers seems to be quite useful
  #### now for creating paths. Initially, I thought I would
  #### just use it to form 'root' paths, but now it has enough
  #### functionality to inspect and create paths
  #### 'parm' can be:
  ####               1. A character. This can be used to create a json path
  ####.              2. A list: with integers as elements. Easy and simple way of creating paths
  ####               3. A list: with characters. Still developing this...
  if(node == "Other" && other.parm.flag > 0){
    ## I will develop this option assuming parm is a list or a character
    if(is.null(parm))
      stop("'parm' is missing", call. = FALSE)

    if(is.character(parm)){
      if(!grepl(".", parm, fixed = TRUE))
        stop("'parm' needs to be a proper json path")
      parm.path <- parm
      if(parm == ".")
        stop("'parm' path is empty")
      pparm <- strsplit(parm, split = ".", fixed = TRUE)[[1]]
      root.name.level.0 <- gsub(".", "", parm.path.0, fixed = TRUE)
      if(pparm[2] != root.name.level.0)
        stop(paste("First 'parm' element does not match:",
                   paste(root.name.level.0, collapse = " ")), call. = FALSE)
      children.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
      if(length(pparm) == 2){
        cat("Simulation children names:")
        print(children.names)
        cat("\n")
      }
      if(length(pparm) == 3){
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0)
          stop(paste("Second 'parm' element did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        cat(paste(root1, "Names level 2:"))
        print(root.names.level.2)
        cat("\n")
      }
      if(length(pparm) == 4){
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0)
          stop(paste("Second 'parm' element did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        root2 <- pparm[4]
        wroot2 <- grep(as.character(root2), root.names.level.2)
        if(length(wroot2) == 0)
          stop(paste("Third 'parm' element did not match:",
                     paste(root.names.level.2, collapse = " ")), call. = FALSE)
        root.names.level.3 <- vapply(apsimx_json$Children[[wroot1]]$Children[[wroot2]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        cat(paste(root1, root2, "Names level 3:"))
        print(root.names.level.3)
        cat("\n")

      }
      if(length(pparm) > 4) stop("Not implemented yet", call. = FALSE)
    }

    if(is.list(parm)){
      if(length(parm) == 0)
        stop("Length of 'parm' list should be greater than zero", call. = FALSE)
      if(is.numeric(parm[[1]])){
        ### In this case the list will be used to create a parameter path
        if(parm[[1]] != 1)
          stop("First element of list should be equal to 1", call. = FALSE)
        parm.path.0 <- paste0(".", apsimx_json$Name) ## Root
        if(length(parm) == 1)
          stop("Length of 'parm' should be greated than 1", call. = FALSE)
        if(length(parm) >= 2){
          if(!is.numeric(parm[[2]]))
            stop("Second element of 'parm' should be numeric")
          root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.1) == 0)
            stop("'parm' index parm[[2]] not found at this level")
          if(length(parm[[2]]) == 1){
            if(parm[[2]] > 0){
              wlevel1 <- try(apsimx_json$Children[[parm[[2]]]], silent = TRUE)
              if(inherits(wlevel1, "try-error"))
                stop("parm[[2]] did not match an available node", call. = FALSE)
              parm.path <- paste0(parm.path.0, ".", root.names.level.1[parm[[2]]])
            }else{
              parm.path <- parm.path.0
              pdat <- data.frame(second_level = root.names.level.1)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            ### Here I assume that parm[[2]] is a vector
            if(length(parm[[2]]) > length(root.names.level.1))
              stop("Length of parm[[2]] should be less than number of elements in the second level")
            selected.levels <- root.names.level.1[parm[[2]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(second_level = root.names.level.1)
            print(knitr::kable(pdat[parm[[2]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 3){
          root.names.level.2 <- vapply(wlevel1$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.2) == 0){
            ## This means that Children are empty
            ## Need to pass this to the next step
            root.names.level.2 <- names(wlevel1)
          }
          if(length(parm[[3]]) == 1){
            if(parm[[3]] > 0){
              wlevel2 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]], silent = TRUE)
              if(inherits(wlevel2, "try-error"))
                stop("parm[[3]] did not match an available node", call. = FALSE)
              parm.path <- paste0(parm.path, ".", root.names.level.2[parm[[3]]])
            }else{
              pdat <- data.frame(third_level = root.names.level.2)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            ### Here I assume that parm[[3]] is a vector
            if(length(parm[[3]]) > length(root.names.level.2))
              stop("Length of parm[[3]] should be less than number of elements in the third level")
            selected.levels <- root.names.level.2[parm[[3]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(third_level = root.names.level.2)
            print(knitr::kable(pdat[parm[[3]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 4){
          names.level.3 <- FALSE
          root.names.level.3 <- vapply(wlevel2$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.3) == 0){
            root.names.level.3 <- names(wlevel2)
            names.level.3 <- TRUE
          }
          if(length(parm[[4]]) == 1){
            if(parm[[4]] > 0){
              if(names.level.3){
                selected.names <- root.names.level.3[parm[[4]]]
                ##pdat <- data.frame(fourth_level = selected.names)
                wlevel3 <- NULL
                ##print(knitr::kable(pdat, row.names = TRUE))
                parm.path <- paste0(parm.path, ".", selected.names)
              }else{
                wlevel3 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]], silent = TRUE)
                if(inherits(wlevel3, "try-error"))
                  stop("parm[[4]] did not match an available node", call. = FALSE)
                parm.path <- paste0(parm.path, ".", root.names.level.3[parm[[4]]])
              }
            }else{
              pdat <- data.frame(fourth_level = root.names.level.3)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            ### Here I assume that parm[[4]] is a vector
            if(length(parm[[4]]) > length(root.names.level.3))
              stop("Length of parm[[4]] should be less than number of elements in the fourth level")
            selected.levels <- root.names.level.3[parm[[4]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(fourth_level = root.names.level.3)
            print(knitr::kable(pdat[parm[[4]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 5){
          names.level.4 <- FALSE
          ##browser()
          root.names.level.4 <- vapply(wlevel3$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.4) == 0){
            root.names.level.4 <- names(wlevel3)
            names.level.4 <- TRUE
          }
          if(length(parm[[5]]) == 1){
            if(parm[[5]] > 0){
              if(names.level.4){
                selected.names <- root.names.level.4[parm[[5]]]
                ##pdat <- data.frame(fifth_level = selected.names)
                ##print(knitr::kable(pdat, row.names = TRUE))
                parm.path <- paste0(parm.path, ".", selected.names)
              }else{
                wlevel4 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]], silent = TRUE)
                if(inherits(wlevel4, "try-error"))
                  stop("parm[[5]] did not match an available node", call. = FALSE)
                parm.path <- paste0(parm.path, ".", root.names.level.4[parm[[5]]])
              }
            }else{
              pdat <- data.frame(fifth_level = root.names.level.4)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            if(length(parm[[5]]) > length(root.names.level.4))
              stop("Length of parm[[5]] should be less than number of elements in the fifth level")
            selected.levels <- root.names.level.4[parm[[5]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(fifth_level = root.names.level.4)
            print(knitr::kable(pdat[parm[[5]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 6){
          names.level.5 <- FALSE
          root.names.level.5 <- vapply(wlevel4$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.5) == 0){
            root.names.level.5 <- names(wlevel4)
            names.level.5 <- TRUE
          }
          if(length(parm[[6]]) == 1){
            if(parm[[6]] > 0){
              if(names.level.5){
                selected.names <- root.names.level.5[parm[[6]]]
                ##pdat <- data.frame(sixth_level = selected.names)
                ##print(knitr::kable(pdat, row.names = TRUE))
                parm.path <- paste0(parm.path, ".", selected.names)
              }else{
                wlevel5 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]]$Children[[parm[[6]]]], silent = TRUE)
                if(inherits(wlevel5, "try-error"))
                  stop("parm[[6]] did not match an available node", call. = FALSE)
                parm.path <- paste0(parm.path, ".", root.names.level.5[parm[[6]]])
              }
            }else{
              pdat <- data.frame(sixth_level = root.names.level.5)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            ### Here I assume that parm[[6]] is a vector
            if(length(parm[[6]]) > length(root.names.level.5))
              stop("Length of parm[[6]] should be less than number of elements in the sixth level")
            selected.levels <- root.names.level.5[parm[[6]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(sixth_level = root.names.level.5)
            print(knitr::kable(pdat[parm[[6]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 7){

          names.level.6 <- FALSE
          root.names.level.6 <- vapply(wlevel5$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.6) == 0){
            root.names.level.6 <- names(wlevel5)
            names.level.6 <- TRUE
          }
          if(length(parm[[7]]) == 1){
            if(parm[[7]] > 0){
              if(names.level.6){
                selected.names <- root.names.level.6[parm[[7]]]
                ##pdat <- data.frame(sixth_level = selected.names)
                ##print(knitr::kable(pdat, row.names = TRUE))
                parm.path <- paste0(parm.path, ".", selected.names)
              }else{
                wlevel6 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]]$Children[[parm[[6]]]]$Children[[parm[[7]]]], silent = TRUE)
                if(inherits(wlevel6, "try-error"))
                  stop("parm[[7]] did not match an available node", call. = FALSE)
                parm.path <- paste0(parm.path, ".", root.names.level.6[parm[[7]]])
              }
            }else{
              pdat <- data.frame(seventh_level = root.names.level.6)
              print(knitr::kable(pdat, row.names = TRUE))
            }
          }else{
            ### Here I assume that parm[[7]] is a vector
            if(length(parm[[7]]) > length(root.names.level.6))
              stop("Length of parm[[7]] should be less than number of elements in the seventh level")
            selected.levels <- root.names.level.6[parm[[7]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(seventh_level = root.names.level.6)
            print(knitr::kable(pdat[parm[[7]],, drop = FALSE], row.names = TRUE))
          }
        }
        if(length(parm) >= 8){
          stop("Have not developed this yet", call. = FALSE)
        }
      }else{
        ## This is if the parm is just one term
        parm.found <- FALSE
        if(length(parm) == 1){
          tparm1 <- try(inspect_apsimx_json(file = file, src.dir = src.dir, parm = parm[[1]]), silent = TRUE)
          if(!inherits(tparm1, 'try-error')){
            parm.path <- tparm1
            parm.found <- TRUE
          }else{
            stop("Parameter not found")
          }
        }
        #### node = 'Other' and list with characters ----
        if(!parm.found){
          if(length(fcsn) > 1){
            if(missing(root)){
              ## In this case the first element of the list should be the root
              if(length(parm) == 1)
                stop("'parm' should be at least length equal to 2", call. = FALSE)
              ## select one of the simulations
              simulation.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
              wsim <- grep(parm[[1]], simulation.names)
              if(length(wsim) == 0){
                cat("Simulation names:", paste(simulation.names, collpse = " "), "\n")
                stop("First element of 'parm' list does not match Simulation names", call. = FALSE)
              }else{
                fcsn <- wsim
                parm <- parm[2:length(parm)]
                if(parm[[1]] == "")
                  stop("'parm' is empty")
              }
              parent.node <- apsimx_json$Children[[fcsn]]$Children
            }else{
              ### root is present
              if(length(root) == 1){
                nms <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                              FUN.VALUE = "character")
                fcsn <- grep(as.character(root), nms)
                if(length(fcsn) == 0 || length(fcsn) > 1){
                  cat("Children:", nms, "\n")
                  stop("'root' not found. Choose one of the options above", call. = FALSE)
                }
                parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
                parent.node <- apsimx_json$Children[[fcsn]]$Children
              }else{
                stop("Have not developed this yet")
              }
            }
          }else{
            parent.node <- apsimx_json$Children[[fcsn]]$Children
          }
        }

        if(!parm.found){
          wcz <- grepl("Models.Core.Zone", parent.node)
          if(sum(wcz) < 0.5)
            stop("Core Simulation not found", call. = FALSE)
          core.zone.node <- parent.node[wcz][[1]]$Children
          parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
          parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
          tmp <- core.zone.node
          parm.path.2.1 <- parm.path.2
        }

        if(!parm.found){
          if(length(parm) > 1){
            for(i in 1:(length(parm) - 1)){
              nms <- sapply(tmp, function(x) x$Name)
              wcp <- grep(parm[[i]], nms)
              if(length(wcp) == 0){
                cat("Names: ", nms, "\n")
                cat("parm[[i]]", parm[[i]], "\n")
                stop("Parameter not found")
              }
              tmp <- tmp[[wcp]]
              if(!is.null(tmp$Children)) tmp <- tmp$Children
              ## Build the parm.path
              parm.path.2.1 <- paste0(parm.path.2.1, ".", nms[wcp])
            }
          }else{
            ## How do I find parm?
            tmp.names <- sapply(tmp, FUN = function(x) x$Name)
            wparm <- grep(parm[[1]], tmp.names)
            if(length(wparm) == 1){
              tmp <- tmp[[wparm]]
            }else{
              #### Work in progress ----
              for(j in seq_along(tmp)){
                tmp.nms <- names(tmp[[j]])
                for(jj in seq_along(parm)){
                  wtn <- grep(parm[[jj]], tmp.nms)
                  if(length(wtn) > 0){
                    print(tmp[[j]][wtn])
                  }
                }
              }
            }
          }

          if(!is.null(tmp$Parameters)){
            wp <- grep(parm[[length(parm)]], tmp$Parameters)
            tmp2 <- tmp$Parameters[[wp]]
            ## Process parameter path
            parm.path <- paste0(parm.path.2.1, ".", tmp2$Key)
            print(knitr::kable(as.data.frame(tmp2)))
          }else{
            parm.path <- parm.path.2.1
            unpack_node(tmp)
          }
        }
      }
    }
  }

  if(print.path && node != "Other" && node != "Operations"){
    if(!missing(parm)){
      if(length(parm) == 1){
        parm.path <- paste0(parm.path, ".", parm)
      }else{
        if(!is.na(position)){
          parm.path <- paste0(parm.path, ".", parm2)
        }
      }
    }
    cat("Parm path:", parm.path,"\n")
  }else{
    if(print.path) cat("Parm path:", parm.path,"\n")
  }

  invisible(parm.path)
}

#'
#'  This function is a work in progress. There are many instances for which it will not work.
#'
#'  It will probably only find the first instance that matches.
#'
#' @title Inspect an .apsimx or .json (JSON) file
#' @name inspect_apsimx_json
#' @description inspect an .apsimx or .json (JSON) file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
#' @param file file ending in .apsimx or .json to be inspected (JSON)
#' @param src.dir directory containing the .apsimx or .json file to be inspected; defaults to the current working directory
#' @param parm string or regular expression for partial matching. It can be two strings separated by a period to search within a node (child).
#' @param search.depth default is 15. How deep should the algorithm explore the structure of the list.
#' @param print.path whether to print the parameter path (default is FALSE)
#' @param verbose whether to print additional information (mostly used for debugging)
#' @return prints a table with inspected parameters and values (and the path when \sQuote{print.path} = TRUE).
#' @export
#' @examples
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## It seems to work for simple search
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Version")
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Simulations")
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Clock")
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Weather")
#' ## Does return soil components
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "DUL")
#' ## Or cultivar
#' inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Hartog")
#'
#' }

inspect_apsimx_json <- function(file = "", src.dir = ".", parm,
                                search.depth = 15,
                                print.path = FALSE,
                                verbose = FALSE){

  .check_apsim_name(file)
  .check_apsim_name(src.dir)

  if(missing(parm))
    stop("You need to specify the parm argument", call. = FALSE)

  file.names.apsimx <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)
  file.names.json <- dir(path = src.dir, pattern = ".json$", ignore.case = TRUE)

  if(length(file.names.apsimx) == 0 && length(file.names.json) == 0){
    stop("There are no .json or .apsimx files in the specified directory to inspect.")
  }

  apsimx_json <- jsonlite::read_json(file.path(src.dir, file)) ### This is a list

  jsonpath <- "$"

  x <- apsimx_json

  ### This means that parm could be used as a jspath
  if(grepl(".", parm, fixed = TRUE)){
    pparm <- strsplit(parm, ".", fixed = TRUE)[[1]]
    if(pparm[1] == "") pparm <- pparm[2:length(pparm)]
    if(length(pparm) == 2){
      ## Find the 'root' to extract new x
      first.level.names <- sapply(X = x$Children, FUN = function(xlist) xlist$Name)
      wfirst <- grep(pparm[1], first.level.names)
      if(length(wfirst) > 1){
        stop(paste("More than one node found", paste(first.level.names, collapse = " ")), call. = FALSE)
      }else{
        jsonpath <- paste0(jsonpath, ".", x$Name)
        x <- x$Children[[wfirst]]
        parm <- pparm[2]
      }
    }
  }

  ## This handles parameters at the first level
  if(any(grepl(parm, names(x)))){
    wpi <- grep(parm, names(x))
    jsonpath <- paste0(jsonpath, ".", names(x)[wpi])
    print(knitr::kable(as.data.frame(x[wpi])))
    if(print.path) print(jsonpath)
    if(verbose) cat("Level: 0 \n")
    return(invisible(jsonpath))
  }

  jsonpath <- paste0(jsonpath, ".", x$Name)

  for(i in 1:search.depth){

    ## First try to see if parameter is in names
    nms <- names(x)

    if(!is.null(nms) && any(grepl(parm, nms))){
      wpi <- grep(parm, nms)
      jsonpath <- paste0(jsonpath, ".", names(x)[wpi])
      xd0 <- x[wpi]
      xd1 <- jsonlist2dataframe(xd0)

      print(knitr::kable(xd1))
      if(print.path) print(jsonpath)
      if(verbose) cat("Level: 1 \n")
      return(invisible(jsonpath))
    }

    ## It is possible for parm to be in Name
    if(!is.atomic(x)) nm <- x$Name

    if(!is.null(nm)){
      if(grepl(parm, nm)){
        wpn <- grep(parm, nm)
        jsonpath <- paste0(jsonpath, ".", nm)
        xd0 <- x[wpn]
        xd1 <- jsonlist2dataframe(xd0)
        names(xd1) <- nm
        print(knitr::kable(xd1))
        if(print.path) print(jsonpath)
        return(invisible(jsonpath))
      }
    }

    if(any(grepl(parm, x))){

      wgpx <- grep(parm, x)

      if(length(wgpx) > 1){
        cat("Parameter found in more than one postion\n")
        for(i in wgpx){
          cat("Position:", i, "\n")
          print(names(x)[i])
        }
        stop("Parameter is not unique", call. = FALSE)
      }


      nms <- try(sapply(x[[wgpx]], function(x) x$Name), silent = TRUE)

      if(verbose){
        print(i)
        print(wgpx)
        print(jsonpath)
        print(nms)
      }

      if(inherits(nms, "try-error")) nms <- NULL

      if(any(sapply(nms, is.null))) nms <- NULL

      if(!is.null(nms)){
        if(any(grepl(parm, nms))){
          wpn <- grep(parm, nms)
          jsonpath <- paste0(jsonpath, ".", nms[wpn])
          xd0 <- x[[wgpx]][wpn]
          xd1 <- jsonlist2dataframe(xd0)
          print(knitr::kable(xd1))
          if(print.path) print(jsonpath)
          return(invisible(jsonpath))
        }
      }

      wgpx2 <- grep(parm, x[[wgpx]])

      if(length(wgpx2) > 1){
        cat("Parameter found in more than one postion\n")
        for(i in wgpx2){
          cat("Position:", i, "\n")
          print(names(x)[i])
        }
        stop("Parameter is not unique", call. = FALSE)
      }

      if(!is.null(nms)){

        if(!is.null(nms[wgpx2]))
          jsonpath <- paste0(jsonpath, ".", nms[wgpx2])

        if(wgpx2 <= length(x[[wgpx]])){
          x <- x[[wgpx]][[wgpx2]]
        }else{
          if(verbose){
            cat("x length:", length(x), "\n")
            cat("wgpx2", wgpx2, "\n")
            cat("length x[[wgpx]]", length(x[[wgpx]]), "\n")
          }
        }

      }else{

        gjl <- grep_json_list(parm, x)
        gjlm <- strsplit(gjl$positions, ".", fixed = TRUE)[[1]]
        gjlms <- as.numeric(gjlm[2:(length(gjlm) - 1)])

        for(i in gjlms) x <- x[[i]]

        ## Need to bring back the code I wrote before to extract the right stuff
        if(!is.null(x$Key))
          jsonpath <- paste0(jsonpath, ".", x$Key)

        print(knitr::kable(as.data.frame(x)))
        if(print.path) print(jsonpath)
        return(invisible(jsonpath))
      }
    }
  }

  if(i == search.depth) stop("Parameter not found")

  invisible(jsonpath)
}

## Convert a json list to a data.frame
## This needs to handle a variety of cases
## 1. Simple approach (unlist and data.frame return no error)
##    In this case all the list elements have equal length
## 2.
jsonlist2dataframe <- function(x){

  nms0 <- names(x)
  if(is.list(x) && length(x) == 1) x <- x[[1]]

  if(length(unique(sapply(x, length))) == 1){
    dt0 <- try(as.data.frame(unlist(x)), silent = TRUE)
    if(!inherits(dt0, "try-error")){
      if(!is.null(nms0)) names(dt0) <- nms0
      return(dt0)
    }else{
      print(str_list(x))
      stop("Can't convert list to data.frame")
    }
  }

  ## The problem is that some elements might be null
  winn <- !sapply(x, function(x) is.null(x)) ## which is not null
  x0 <- x[winn]
  tmp <- NULL
  for(i in seq_along(x0)){
    nms.x0 <- names(x0)[i]
    if(!names(x0)[i] %in% c("Children", "Code", "Parameters")){
      tmp <- rbind(tmp, as.data.frame(unlist(x0[i])))
    }

    if(names(x0)[i] == "Children"){
      lchildren <- length(x0$Children)
      tmp <- rbind(tmp, paste0("List of length: ", lchildren))
      rnms <- row.names(tmp)
      rnms[i] <- "Children"
      row.names(tmp) <- rnms
    }

    if(names(x0)[i] == "Code"){
      tmp <- rbind(tmp, paste0("C# code..."))
      rnms <- row.names(tmp)
      rnms[i] <- "C# code"
      row.names(tmp) <- rnms
    }

    if(names(x0)[i] == "Parameters"){
      tmp <- rbind(tmp, paste0("Parameters..."))
      rnms <- row.names(tmp)
      rnms[i] <- "Parameters"
      row.names(tmp) <- rnms
    }
  }

  if(!is.null(x0$Name)){
    names(tmp) <- x$Name
  }else{
    if(!is.null(names(x)[winn])) names(tmp) <- names(x)[winn]
  }
  tmp
}


## The idea of this function is that it will return the position where the
## pattern is found and also the node
## The trick is that the previous function does not quite give me what I want
#' @title grep but for json list
#' @name grep_json_list
#' @description recursive grep adapted for a json list
#' @param pattern as in grep
#' @param x object (a list)
#' @param ignore.case as in grep
#' @param search.depth search depth for the list (to prevent endless search)
#' @param how argument passed to \link{rapply}
#' @return It returns a list with the found object, the json path and the positions in the list.
#' @export
grep_json_list <- function(pattern, x, ignore.case = FALSE, search.depth = 10,
                           how = c("unlist", "replace", "list")){

  ## Check first
  hw <- match.arg(how)
  rar <- rapply(x, function(x) grep(pattern, x, ignore.case = ignore.case),
                how = hw)
  # rar2 <- rapply(x, function(x) grep(pattern, names(x), ignore.case = ignore.case))

  # if(length(rar) == 0L && length(rar2) == 0)
  #   stop("pattern not found")
  if(length(rar) == 0L)
    stop("pattern not found")

  positions <- ""
  jsonpath <- ""

  for(i in 1:search.depth){

    if(is.list(x) && length(x) == 1) x <- x[[1]]
    wgnp <- grep(pattern, x, ignore.case = ignore.case)

    if(length(wgnp) < 0.5) break

    if(!is.atomic(x) && !is.null(x$Name))
      jsonpath <- paste0(jsonpath, ".", x$Name)

    if(!is.atomic(x) && length(wgnp) > 0)
      positions <- paste0(positions, ".", wgnp)

    if(length(wgnp) == 1){
      x <- x[[wgnp]]
    }else{
      xn <- list()
      for(i in seq_along(wgnp)){
        xn[[i]] <- x[[i]]
      }
      x <- xn
    }
  }

  return(list(x = x, jsonpath = jsonpath, positions = positions))
}

## This version of grep is not exposed at the moment
grep_json_list1 <- function(pattern, x, ...){

  tmp <- rapply(x, function(x,...) grep(pattern = pattern, x = x, value = TRUE, ...), ...)

  ans <- vector("list", length = length(tmp))

  for(i in seq_along(tmp)){
    ans[[i]] <- tmp[[i]]
  }

  return(ans)
}

#### Parse 'root' ----
parse_root <- function(root){
  ## Add flexibility to what can be passed as 'root'
  ## If root is a json path
  if(!all(sapply(root, is.character)))
    stop("'root' should be a character", call. = FALSE)
  if(length(root) == 1){
    if(grepl(".", root, fixed = TRUE)){
      ## Split the root
      proot <- strsplit(root, ".", fixed = TRUE)[[1]]
      wec <- which(proot == "") ## which empty character
      if(length(wec) > 0){
        root <- proot[-wec]
      }else{
        root <- proot
      }
    }
  }
  root
}

### This function is from ChatGPT
traverse_json <- function(file, path = "$") {
  
  json_obj <- jsonlite::read_json(file)
  if (is.list(json_obj)) {
    for (key in names(json_obj)) {
      traverse_json(json_obj[[key]], paste0(path, ".", key))
    }
  } else {
    print(paste(path, ":", json_obj))
  }
}

#' Function which performs common unit conversions
#' 
#' At the moment possible conversions are:
#' \itemize{
#' 
#' \item \sQuote{g/m2} to \sQuote{kg/ha}
#' \item \sQuote{kg/ha} to \sQuote{g/m2}
#' \item \sQuote{lb} to \sQuote{kg}
#' \item \sQuote{kg} to \sQuote{lb}
#' \item \sQuote{maize bu} to \sQuote{kg}
#' \item \sQuote{kg} to \sQuote{maize bu}
#' \item \sQuote{soy bu} to \sQuote{kg}
#' \item \sQuote{kg} to \sQuote{soy bu}
#' \item \sQuote{maize bu/ac} to \sQuote{kg/ha}
#' \item \sQuote{maize bu/ac} to \sQuote{g/m2}
#' \item \sQuote{kg/ha} to \sQuote{maize bu/ac}
#' \item \sQuote{g/m2} to \sQuote{maize bu/ac}
#' \item \sQuote{soy bu/ac} to \sQuote{kg/ha}
#' \item \sQuote{soy bu/ac} to \sQuote{g/m2}
#' \item \sQuote{kg/ha} to \sQuote{soy bu/ac}
#' \item \sQuote{g/m2} to \sQuote{soy bu/ac}
#' \item \sQuote{mm} to \sQuote{inches}
#' \item \sQuote{inches} to \sQuote{mm}
#' \item \sQuote{lb/ac} to \sQuote{kg/ha}
#' \item \sQuote{kg/ha} to \sQuote{lb/ac}
#' \item \sQuote{lb/ac} to \sQuote{g/m2}
#' \item \sQuote{g/m2} to \sQuote{lb/ac}
#' \item \sQuote{decimal} to \sQuote{degrees}
#' \item \sQuote{degrees} to \sQuote{decimal}
#' \item \sQuote{Fahrenheit} to \sQuote{Celsius}
#' \item \sQuote{Celsius} to \sQuote{Fahrenheit} 
#' }
#' 
#' This is for metric and Imperial conversions
#' Source: https://www.extension.iastate.edu/agdm/wholefarm/html/c6-80.html
#'
#' @title performs common unit conversions
#' @name unit_conv
#' @description This function is slowly getting better. Adding more unit conversions as I need them.
#' @param x input variable
#' @param from original units 
#' @param to target units
#' @param ... additional arguments passed to specific conversions
#' @return value of the input variable with new units
#' @export
#' @examples 
#' \donttest{
#' grain.yield.gm2 <- 600
#' grain.yield.kgha <- unit_conv(grain.yield.gm2, from = "g/m2", to = "kg/ha")
#' grain.yield.kgha
#' ## Converting coordinates
#' require(sp)
#' unit_conv("42d 0' 0\" N", from = "degrees", to = "decimal")
#' unit_conv(42, from = "decimal", to = "degrees") ## EW by default
#' unit_conv(42, from = "decimal", to = "degrees", NS = TRUE)
#' }
#' 

unit_conv <- function(x, from, to, ...){
  
  from.table <- c("g/m2","Mg/ha","kg/m2","kg/ha","lb/ac","kg","lb",
                  "maize bu", "soy bu", "maize bu/ac", "soy bu/ac", "mm", "inches",
                  "lb/ac", "degrees", "decimal", "F", "Fahrenheit", "C", "Celsius")
  to.table <- c("g/m2","Mg/ha","kg/m2","kg/ha","lb/ac","kg","lb",
                "maize bu", "soy bu", "maize bu/ac", "soy bu/ac", "mm", "inches",
                "lb/ac", "degrees", "decimal", "F", "Fahrenheit", "C", "Celsius")
  
  from <- match.arg(from, from.table)
  to <- match.arg(to, to.table)
  
  if(from == to){
    warning("'from' is equal to 'to'. No conversion was performed.")
    return(x)
  }
    
  convs <- FALSE
  ## Create conversion matrix
  
  if(from == "g/m2" && to == "kg/ha"){
    ## From m2 to ha multiply by 1e4
    ## From g to kg divide by 1e3
    ans <- x * 10
    convs <- TRUE
  }
  
  if(from == "kg/ha" && to == "g/m2"){
    ## From ha to m2 divide by 1e4
    ## from kg to grams multiply by 1e3
    ans <- x * 0.1
    convs <- TRUE
  }
  
  if(from == "kg" && to == "lb"){
    ## 1 lb (pound) = 0.454 kg
    ans <- x * 2.205
    convs <- TRUE
  }
  
  if(from == "lb" && to == "kg"){
    ## 1 lb (pound) = 0.454 kg
    ans <- x / 2.205
    convs <- TRUE
  }
  
  if(from == "maize bu" && to == "kg"){
    ## 1 maize bushel = 25.4 kg
    ans <- x * 25.4
    convs <- TRUE
  }
  
  if(from == "kg" && to == "maize bu"){
    ## 1 maize bushel = 25.4 kg
    ans <- x / 25.4
    convs <- TRUE
  }
  
  if(from == "soy bu" && to == "kg"){
    ## 1 soy bushel = 27.22 kg
    ans <- x * 27.22
    convs <- TRUE
  }
  
  if(from == "kg" && to == "soy bu"){
    ## 1 soy bushel = 27.22 kg
    ans <- x / 27.22
    convs <- TRUE
  }
  
  if(from == "maize bu/ac" && to == "kg/ha"){
    ## 1 maize bushel/acre = 56 
    ans <- x * 62.77
    convs <- TRUE
  }
  
  if(from == "kg/ha" && to == "maize bu/ac"){
    ## 1 maize bushel/acre = 56 
    ans <- x / 62.77
    convs <- TRUE
  }
  
  if(from == "g/m2" && to == "maize bu/ac"){
    ## 1 maize bushel/acre = 56 
    ans <- (x * 10) / 62.77
    convs <- TRUE
  }
  
  if(from == "maize bu/ac" && to == "g/m2"){
    ## 1 maize bushel/acre = 56 
    ans <- (x / 10) * 62.77
    convs <- TRUE
  }
  
  if(from == "soy bu/ac" && to == "kg/ha"){
    ## 1 soy bushel/acre = 60 
    ans <- x * 67.25
    convs <- TRUE
  }
  
  if(from == "kg/ha" && to == "soy bu/ac"){
    ## 1 soy bushel/acre = 60 
    ans <- x / 67.25
    convs <- TRUE
  }
  
  if(from == "g/m2" && to == "soy bu/ac"){
    ans <- (x * 10) / 67.25 
    convs <- TRUE
  }
  
  if(from == "soy bu/ac" && to == "g/m2"){
    ans <- (x / 10) * 67.25 
    convs <- TRUE
  }
  
  if(from == "mm" && to == "inches"){
    ## 1 inch = 25.4 mm
    ans <- x / 25.4
    convs <- TRUE
  }
  
  if(from == "inches" && to == "mm"){
    ## 1 inch = 25.4 mm
    ans <- x * 25.4
    convs <- TRUE
  }
  
  if(from == "lb/ac" && to == "kg/ha"){
    ## 1 lb 0.453592 kg, 1 acre 2.47105 hectares
    ans <- x * 0.453592 * 2.47105
    convs <- TRUE
  }
  
  if(from == "kg/ha" && to == "lb/ac"){
    ans <- x / (0.453592 * 2.47105)
    convs <- TRUE
  }
  
  if(from == "lb/ac" && to == "g/m2"){
    ## 1 lb 0.453592 kg, 1 acre 2.47105 hectares
    ans <- (x * 0.453592 * 2.47105) * 0.1
    convs <- TRUE
  }
  
  if(from == "g/m2" && to == "lb/ac"){
    ans <- (x / (0.453592 * 2.47105)) * 10
    convs <- TRUE
  }
  
  if((from == "Fahrenheit" && to == "Celsius") || (from == "F" && to == "C")){
    ans <- (x - 32) * (5/9)
    convs <- TRUE
  }
  
  if((from == "Celsius" && to == "Fahrenheit") || (from == "C" && to == "F")){
    ans <- (x * 9/5) + 32
    convs <- TRUE
  }
  
  if(from == "decimal" && to == "degrees"){
    if(!requireNamespace("sp", quietly = TRUE)){
      warning("sp is required for this option")
      return(NULL)
    }
    if(!is.numeric(x)) stop("'x' should be numeric", call. = FALSE)
    dots <- list(...)
    if(is.null(dots$NS)){
      NS <- FALSE
    }else{
      NS <- dots$NS
    } 
    ans <- sp::dd2dms(x, NS = NS)  
    convs <- TRUE
  }
  
  if(from == "degrees" && to == "decimal"){
    if(!requireNamespace("sp", quietly = TRUE)){
      warning("sp is required for this option")
      return(NULL)
    }
    if(!is.character(x)) stop("'x' should be character", call. = FALSE)
    dots <- list(...)
    if(is.null(dots$chd)){ chd <- "d" }else{ chd <- dots$chd } 
    if(is.null(dots$chm)){ chm <- "'" }else{ chm <- dots$chm }
    if(is.null(dots$chs)){ chs <- "\"" }else{ chs <- dots$chs }
    ans <- as.numeric(sp::char2dms(x, chd = chd, chm = chm, chs = chs))  
    convs <- TRUE
  }
  
  if(isFALSE(convs))
    stop("Conversion not available", call. = FALSE)
  
  return(ans)
}

#' 
#' @title Converts from doy to date
#' @name doy2date
#' @rdname doy2date
#' @description Given a day of the year as julian (1-366) it converts to \sQuote{Date}
#' @param x either an integer 1-366 or a \sQuote{Date}
#' @param year year
#' @param inverse if TRUE it goes from date to day
#' @return an object of class \sQuote{Date} or a numeric if inverse equals TRUE.
#' @export
#' @examples 
#' doy2date(120)

doy2date <- function(x, year = 2001, inverse = FALSE){
  
  if(inherits(x, "character") && inverse == FALSE){
    stop("Did you intend to use it in inverse mode?")
  }
  if(inverse == FALSE){
    if(!is.numeric(x)){
      stop("x should be numeric")
    }else{
      if(any(x < 1)) stop("x should be greater than 1")
      if(any(x > 366)) stop("x should be less than 365")
    }
    
    day1 <- as.Date(paste0(year, "-01-01"))
    dayn <- as.Date(paste0(year, "-12-31"))
    dates <- seq(day1, dayn, by="day")
    doy <- dates[x]
  }else{
    if(!inherits(x, "character")) stop("x should be of 'character' class")
    
    doy <- as.numeric(format(as.Date(paste0(year, "-", x)), "%j"))
  }
  doy
}

#' 
#' @rdname doy2date
#' @description Given a \sQuote{Date} it converts to julian day (1-366) or day of the year
#' @param x either an integer 1-366 or a \sQuote{Date}
#' @param year year
#' @param inverse if TRUE it goes from doy to \sQuote{Date}
#' @return an numeric or an object of class \sQuote{Date} if inverse equals TRUE.
#' @export
#' @examples 
#' date2doy("04-30")
#' 
date2doy <- function(x, year = 2001, inverse = FALSE){
  ans <- doy2date(x, year = year, inverse = !inverse)
  return(ans)
}

#' Auxiliary unexported and \sQuote{hidden} functions
#' Find output file names in an APSIM \sQuote{Classic} file
#' @name .find_ouput_names
#' @description In APSIM \sQuote{Classic} the outputfiles are named internally and this
#' function can retrieve this information
#' @param .file file name
#' @param .src.dir source directory
#'
#' @return output file names
#'
#' @noRd
.find_output_names <- function(.file, .src.dir = "."){

  .apsim_xml <- xml2::read_xml(paste0(.src.dir,"/",.file))

  .find.output <- xml2::xml_find_all(.apsim_xml, ".//outputfile/filename")

  .ans <- xml2::xml_text(.find.output)

  return(.ans)
}

#' Check the name and disallow white spaces
#' These are not allowed as it would not work (easily)
#' at the command line
#' @name .check_apsim_name
#' @description Checks whether an APSIM file has spaces in it
#' @param .file an APSIM file
#' @return It only returns an error if there is a space in the file
#' @noRd
#'
.check_apsim_name <- function(.file){
  ## No spaces are allowed, provide informative error
  if(missing(.file))
    stop("argument .file is missing", call. = FALSE)
  if(grepl("\\s", .file))
    stop("White spaces are not allowed in file names or directories", call. = FALSE)
}

#' This is for when a list has only one element
#' But inside that element there is a children with multiple
#' elements, or at least is not null
#' @name .extract_bad_children
#' @description Possibly extract unnamed children more robustlly
#' @param x a list with (potentially) a \sQuote{Children} object
#' @return it returns a subset of a json list
#' @noRd
#'
.extract_bad_children <- function(x){

  if(!is.list(x)) stop("x should be a list")

  lnms <- names(x)
  nco <- is.null(x$Children)

  if(length(x) == 1 && lnms == 0 && !nco){
    x <- x$Children[[1]]
  }
  return(x)
}

#' This function extracts the APSIM-X year, month version
#' This used to be able to extract the date, but this is not
#' part of the name anymore, so it is not available
#' @name .favd
#' @description Extract the year, month version from an APSIM binary name
#' @param x an APISM binary name such as 'APSIM2020.06.5260.app'
#' @return it returns a string for the APSIM year, month, version
#' @noRd
#'
.favd <- function(x){
  x1 <- gsub("APSIM","\\1", x) ## Remove 'APSIM'
  x2 <- strsplit(x1, ".", fixed = TRUE)[[1]] ## Split by '.'
  x3 <- paste(paste(x2[1:2], collapse="-"), " version:", as.numeric(x2[3]))
  return(x3)
}

#' This function extracts the APSIM-X Version number
#' Find Apsim Version number
#' @name .favn
#' @description Extract the number from an APSIM binary name being used
#' @param x an APISM binary name such as 'APSIM2020.06.5260.app'
#' @return it returns a number for the APSIM version
#' @noRd
#'
.favn <- function(x){
  x1 <- gsub("APSIM","\\1", x) ## Remove 'APSIM'
  x2 <- strsplit(x1, ".", fixed = TRUE)[[1]] ## Split by '.'
  # x3 <- as.Date(paste(x2[1:3], collapse="-"), format = "%Y-%m-%d") ## Convert to 'Date'
  x3 <- as.numeric(x2[3]) + as.numeric(x2[4])
  return(x3)
}

#' This function converts thickness to depth for soil profiles
#' Thickness is assumed to be in mm and the returned depths would be in
#' cm
#' @name .t2d
#' @description Takes a \sQuote{Thickness} column and returns \sQuote{Depth}
#' @param x a soil profile \sQuote{Thickness} column (numeric)
#' @return it returns a column with strings such as \sQuote{0-20}
#' @noRd
#'
.t2d <- function(x){
  x2 <- c(0, x)/10 ## Divide by 10 to go from mm to cm
  ans <- character(length(x))
  csx2 <- cumsum(x2)
  for(i in 2:length(x2)){
    ans[i-1] <- paste0(csx2[i-1], "-", csx2[i])
  }
  ans
}

## Testing GSOD in general and the radiation fill in feature
##
require(apsimx)

## This is for UGANDA
lonlat <- c(33, 1)
dates <- c("2000-01-01", "2019-01-01")

if(FALSE){
  ## Without filling in missing radiation data
  gsd <- get_gsod_apsim_met(lonlat = lonlat, dates = dates)
  ## Test fillin.radn feature
  gsd2 <- get_gsod_apsim_met(lonlat = lonlat, dates = dates, fillin.radn = TRUE)
  
  ## Test padding and imputation
  gsd.napad <- napad_apsim_met(gsd)
  gsd2.napad <- napad_apsim_met(gsd2)
  
  ## This shows that impute does not work on the edges
  gsd2.impt <- impute_apsim_met(gsd2.napad)
}

## Testing the capability of optimizing parameters
require(apsimx)
require(ggplot2)
apsimx_options(warn.versions = FALSE)

## 1. Simulate data from a model with known parameters
## 2. Change the parameter values and some some random "noise"
## 3. Run the optimization algorithm
## 4. Compare optimized parameter values with original ones

extd.dir <- system.file("extdata", package = "apsimx")

## extd.dir <- "~/Dropbox/apsimx/inst/extdata"

## I won't routinely run this
#### First set of tests ----
if(FALSE){
  
  ## One model run takes ~4 seconds
  ## The original values for these were 1.5 for RUE and 90 for BasePhyllochron
  system.time(sim0 <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir))

  pp1 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = extd.dir, 
                                    node = "Wheat", 
                                    node.child = "Leaf",
                                    node.subchild = "Photosynthesis",
                                    node.subsubchild = "RUE",
                                    parm = "FixedValue",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)
  
  pp2 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = extd.dir, 
                                    node = "Wheat", 
                                    node.child = "Cultivars",
                                    node.subchild = "USA",
                                    node.subsubchild = "Yecora",
                                    parm = "BasePhyllochron",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)
  
  ## edit_apsimx_replacement("Wheat-opt-ex.apsimx")
  
  ## Visualize the data
  ggplot(sim0, aes(Date, Wheat.AboveGround.Wt)) + 
    geom_line()
  ggplot(sim0, aes(Date, Wheat.Leaf.LAI)) + 
    geom_line()
  ggplot(sim0, aes(Date, Wheat.Phenology.Zadok.Stage)) + 
    geom_line()

  ## Create 'observed' data
  sample.dates <- seq(from = as.Date("2016-10-01"), to = as.Date("2017-06-01"), length.out = 10)

  set.seed(12345)
  obs0 <- sim0[sim0$Date %in% sample.dates, c("Date","Wheat.Phenology.Zadok.Stage","Wheat.Leaf.LAI","Wheat.AboveGround.Wt")]
  obs0$Wheat.Phenology.Zadok.Stage <- obs0$Wheat.Phenology.Zadok.Stage + rnorm(10, 0, sd = 0.1)
  obs0$Wheat.Leaf.LAI[5:10] <- obs0$Wheat.Leaf.LAI[5:10] + rnorm(6, 0, sd = 0.5)
  obs0$Wheat.AboveGround.Wt[3:10] <- obs0$Wheat.AboveGround.Wt[3:10] + rnorm(8, 0, sd = 3)

  obsWheat <- obs0
  save(obsWheat, file = "obsWheat.rda")

  data(obsWheat)
  ## We will try to optimize only 2 parameters
  ## RUE: default 1.5
  ## Phyllochron for cultivar Yecora Fixed Value 90
  inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                             src.dir = extd.dir, 
                             node = "Wheat", 
                             node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE",
                             parm = "FixedValue",
                             print.path = TRUE,
                             display.available = FALSE, 
                             verbose = FALSE)

  inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                             src.dir = extd.dir, 
                             node = "Wheat", 
                             node.child = "Cultivars",
                             node.subchild = "USA",
                             node.subsubchild = "Yecora",
                             parm = "BasePhyllochron",
                             print.path = TRUE,
                             display.available = FALSE, 
                             verbose = FALSE)
  
  ## Run the model with incorrect parameters
  ## RUE = 1.2
  ## Phyllochron = 120
  tmp <- tempdir()
  setwd(tmp)
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  system.time(sim.b4 <- apsimx("Wheat-opt-ex.apsimx", src.dir = tmp)) ## This takes 2 seconds (Windows 2025-3-6)
  
  ## write.csv(sim.b4, "wheat-sim-b4-opt.csv", row.names = FALSE)

  sim.b4.s <- subset(sim.b4, Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))
  ## phenology
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Zadok.Stage)) +
    geom_line(data = sim.b4.s, aes(x = Date, y = Wheat.Phenology.Zadok.Stage)) + 
    ggtitle("Phenology")
  ## LAI
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
    geom_line(data = sim.b4.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
    ggtitle("LAI")
  ## Biomass
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
    geom_line(data = sim.b4.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")
  
  ## Optimization
  pp1 <- "Wheat.Leaf.Photosynthesis.RUE.FixedValue"
  pp2 <- "Wheat.Cultivars.USA.Yecora.BasePhyllochron"
  
  # edit_apsimx_replacement("Wheat-b4-opt.apsimx",
  #                         src.dir = ".",
  #                         overwrite = TRUE,
  #                         node = "Wheat", node.child = "Cultivars",
  #                         node.subchild = "USA", node.subsubchild = "Yecora",
  #                         parm = "BasePhyllochron", value = 110)
  
  ## This takes about ~7 minutes
  start <- Sys.time()
  wop <- optim_apsimx("Wheat-opt-ex.apsimx", 
                       parm.paths = c(pp1, pp2),
                       data = obsWheat, 
                       weights = "mean",
                       replacement = c(TRUE, TRUE),
                       initial.values = c(1.2, 120))
  end <- Sys.time()
  
  ## This took 8.8 minutes
  start <- Sys.time()
  wop.h <- optim_apsimx("Wheat-opt-ex.apsimx", 
                          src.dir = extd.dir, 
                          parm.paths = c(pp1, pp2),
                          data = obsWheat, 
                          weights = "mean",
                          replacement = c(TRUE, TRUE),
                          initial.values = c(1.2, 120),
                          hessian = TRUE)
  end <- Sys.time()
  
  sim.opt <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir, value = "report")  

  sim.opt.s <- subset(sim.opt, Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))
  
  ## For vignette
  ## write.csv(sim.opt.s, file = "wheat-sim-opt.csv", row.names = FALSE)
  
  ## phenology
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Zadok.Stage)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Phenology.Zadok.Stage)) + 
    ggtitle("Phenology")
  ## LAI
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
    ggtitle("LAI")
  ## Biomass
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")

  ## What about nloptr?
  start <- Sys.time()
  wop.n <- optim_apsimx("Wheat-opt-ex.apsimx", 
                        src.dir = extd.dir, 
                        parm.paths = c(pp1, pp2),
                        data = obsWheat, 
                        weights = "mean",
                        replacement = c(TRUE, TRUE),
                        initial.values = c(1.2, 120),
                        type = "nloptr",
                        opts = list("algorithm" = "NLOPT_LN_NELDERMEAD"))
  end <- Sys.time()
  
  ## Test BayesianTools??? Would need to embbed it inside optim
  ## How long does this take in Windows laptop? 500 iter, 9 chains: 3464 seconds ~ 1 hour
  ## 500 iter, 3 chains: 18 minutes
  ## 5000 iter, 3 chains: 4 hours, but still not good enough
  start <- Sys.time()
  wop.mcmc <- optim_apsimx("Wheat-opt-ex.apsimx", 
                           src.dir = extd.dir, 
                           parm.paths = c(pp1, pp2),
                           data = obsWheat, 
                           weights = "mean",
                           replacement = c(TRUE, TRUE),
                           initial.values = c(1.2, 120),
                           type = "mcmc",
                           parallel = FALSE,
                           settings = list(iterations = 5000))
  end <- Sys.time()
  
}

#### Classic #### 
if(FALSE){

  tmp <- tempdir()
  setwd(tmp)
  
  apsim_options(warn.versions = FALSE)
  
  ## Testing optimization for an APSIM Classic example
  ## Maize
  if(file.exists("Maize.apsim")) file.remove("Maize.apsim")
  if(file.exists("Maize.xml")) file.remove("Maize.xml")
  if(file.exists("Maize.out")) file.remove("Maize.out")
  if(file.exists("Maize.sum")) file.remove("Maize.sum")
  
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Maize-op.apsim"), "./Maize.apsim")
  file.copy("C:/Program Files (x86)/APSIM710-r4207/Model/Maize.xml", ".")

  inspect_apsim("Maize.apsim")
  
  inspect_apsim("Maize.apsim", node = "Crop",
                parm = list("Sow", 3))
  
  edit_apsim("Maize.apsim", node = "Clock",
             parm = "start_date", value = "01/01/1987",
             overwrite = TRUE)
  
  sim0 <- apsim("Maize.apsim")
  
  obsMaize <- data.frame(Date = as.Date(c("1988-02-23", "1989-02-25")),
                         biomass = c(4600, 5000),
                         yield = c(350, 570))
  
  pp1 <- inspect_apsim_xml("Maize.xml", parm = "sc401/rue")
  pp2 <- inspect_apsim_xml("Maize.xml", parm = "sc401/GNmaxCoef")
  
  ## Optimize a single parameter
  op01 <- optim_apsim("Maize.apsim", 
                     crop.file = "Maize.xml",
                     parm.paths = pp2,
                     data = obsMaize,
                     hessian = TRUE,
                     weights = "mean")
  
  
  op1 <- optim_apsim("Maize.apsim", 
                     crop.file = "Maize.xml",
                     parm.paths = c(pp1, pp2),
                     data = obsMaize,
                     weights = "mean")

  file.remove("Maize.xml")
  file.copy("C:/Program Files (x86)/APSIM710-r4207/Model/Maize.xml", ".")
  
  op2 <- optim_apsim("Maize.apsim", 
                     crop.file = "Maize.xml",
                     parm.paths = c(pp1, pp2),
                     data = obsMaize,
                     hessian = TRUE,
                     weights = "mean")
  
  # op2.u <- optim_apsim("Maize.apsim", 
  #                      crop.file = "Maize.xml",
  #                      parm.paths = c(pp1, pp2),
  #                      type = "ucminf",
  #                      data = obsMaize,
  #                      hessian = TRUE,
  #                      weights = "mean")
  
  op2nl <- optim_apsim("Maize.apsim", 
                     crop.file = "Maize.xml",
                     parm.paths = c(pp1, pp2),
                     data = obsMaize,
                     type = "nloptr",
                     weights = "mean",
                     opts = list("algorithm" = "NLOPT_LN_NELDERMEAD"))
  
  sim2 <- apsim("Maize.apsim")
  
  ## Can I also optimize management?
  pp3 <- inspect_apsim("Maize.apsim", node = "Crop",
                       parm = list("Sow", 17),
                       print.path = TRUE)

  pp4 <- inspect_apsim("Maize.apsim", node = "Crop",
                       parm = list("Sow", 28),
                       print.path = TRUE)

  ## The goal of this is to show that you 
  ## can try to optimize parameters which are in the
  ## .apsim and .xml file
  op3 <- optim_apsim("Maize.apsim", 
                     crop.file = "Maize.xml",
                     parm.paths = c(pp2, pp4),
                     xml.parm = c(TRUE, FALSE),
                     data = obsMaize,
                     weights = "mean",
                     hessian = TRUE)

  inspect_apsim("Maize.apsim", node = "Crop",
                parm = list("Sow", 17),
                print.path = TRUE)
  
  inspect_apsim("Maize.apsim", node = "Crop",
                 parm = list("Sow", 28),
                 print.path = TRUE)

  
  if(file.exists("Maize.apsim")) file.remove("Maize.apsim")
  if(file.exists("Maize.xml")) file.remove("Maize.xml")
  if(file.exists("Maize.out")) file.remove("Maize.out")
  if(file.exists("Maize.sum")) file.remove("Maize.sum")
}

## Testing the optimization of a single parameter 
## comparing methods
if(FALSE){
  
  tmp <- tempdir()
  setwd(tmp)
  
  extd.dir <- system.file("extdata", package = "apsimx")
  
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  sim0 <- apsimx("Wheat-opt-ex.apsimx")

  pp1 <- "Wheat.Leaf.Photosynthesis.RUE.FixedValue"
  pp2 <- "Wheat.Cultivars.USA.Yecora.BasePhyllochron"
  
  ## Testing the "unreliable" method
  start <- Sys.time()
  wop.1 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                        parm.paths = pp1,
                        data = obsWheat, 
                        replacement = TRUE,
                        initial.values = 1.2)
  end <- Sys.time() ## It took 3.25 minutes
  
  ## Plus hessian
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  start <- Sys.time()
  wop.h.1 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                          parm.paths = pp1,
                          data = obsWheat, 
                          replacement = TRUE,
                          hessian = TRUE,
                          initial.values = 1.2)
  end <- Sys.time() ## It took 3.4 minutes
 
  ## Erase and try Brent 
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
 
  start <- Sys.time()
  wop.b.1 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                          parm.paths = pp1,
                          data = obsWheat, 
                          method = "Brent",
                          lower = 0.5, upper = 2,
                          replacement = TRUE,
                          initial.values = 1.2)
  end <- Sys.time() ## It took 2.18 minutes 
  ## Brent, naturally, will not provide the best solution if
  ## the solution is outside the interval (lower, upper)
  
  ## Erase and try L-BFGS-B
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  start <- Sys.time()
  wop.bfgs.1 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                          parm.paths = pp1,
                          data = obsWheat, 
                          method = "L-BFGS-B",
                          lower = 0.5,
                          hessian = TRUE,
                          replacement = TRUE,
                          initial.values = 1.2,
                          control = list(trace = 3))
  end <- Sys.time() ## It took 7.02 minutes 
  
  ## Next step is to test whether weighting affects the estimate of the vcov
  ## and how different it is to the Bayesian analysis
  
  ## Erase and try L-BFGS-B
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  start <- Sys.time()
  wop.bfgs.2 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                             parm.paths = pp1,
                             data = obsWheat,
                             weights = "mean",
                             method = "L-BFGS-B",
                             lower = 0.5,
                             hessian = TRUE,
                             replacement = TRUE,
                             initial.values = 1.2,
                             control = list(trace = 3))
  end <- Sys.time() ## It took 4.16 minutes 
  
  ## Testing with two parameters
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  ## Testing with two parameters
  start <- Sys.time()
  wop.2 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                        parm.paths = c(pp1, pp2),
                        data = obsWheat, 
                        replacement = c(TRUE, TRUE),
                        hessian = TRUE,
                        initial.values = c(1.2, 120))
  end <- Sys.time() ## It took 15.7 minutes
  
  ## Testing with two parameters, without the hessian
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  ## Testing with two parameters
  start <- Sys.time()
  wop.nh.2 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                        parm.paths = c(pp1, pp2),
                        data = obsWheat, 
                        replacement = c(TRUE, TRUE),
                        initial.values = c(1.2, 120))
  end <- Sys.time() ## It took 13.97 minutes
  
  file.remove("Wheat-opt-ex.apsimx")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  ## Testing with two parameters
  start <- Sys.time()
  wop.3 <- optim_apsimx("Wheat-opt-ex.apsimx", 
                        parm.paths = c(pp1, pp2),
                        data = obsWheat, 
                        type = "ucminf",
                        weights = "mean",
                        replacement = c(TRUE, TRUE),
                        hessian = TRUE,
                        initial.values = c(1.2, 120))
  end <- Sys.time() ## It took 28.97 minutes and it did not find the right answer
}

#### Testing grid method ----
if(FALSE){
  
  data(obsWheat)
  
  system.time(sim0 <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir)) 
  
  cmp.b4 <- compare_apsim(obsWheat, sim0, labels = c("obs", "sim"), cRSS = TRUE)
  ### pre-optimized log-RSS
  log(cmp.b4$cRSS)
  
  ## Creating a grid for two parameters
  pp1 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = extd.dir, 
                                    node = "Wheat", 
                                    node.child = "Leaf",
                                    node.subchild = "Photosynthesis",
                                    node.subsubchild = "RUE",
                                    parm = "FixedValue",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)
  
  pp2 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = extd.dir, 
                                    node = "Wheat", 
                                    node.child = "Cultivars",
                                    node.subchild = "USA",
                                    node.subsubchild = "Yecora",
                                    parm = "BasePhyllochron",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)

  pgrd <- expand.grid(RUE = c(0.8, 1, 1.2, 1.4, 1.6, 1.8, 2.0), 
                      BasePhyllochron = c(40, 60, 80, 100, 120, 140, 160))  
  
  tmp <- tempdir()
  setwd(tmp)
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), ".")
  
  system.time(sim.b4 <- apsimx("Wheat-opt-ex.apsimx", src.dir = tmp)) ## This takes 4-5 seconds
  xrgs <- xargs_apsimx(single.threaded = TRUE)
  system.time(sim.b4 <- apsimx("Wheat-opt-ex.apsimx", src.dir = tmp, xargs = xrgs)) ## This takes 4-5 seconds
  

  start <- Sys.time()
  wop.grd <- optim_apsimx("Wheat-opt-ex.apsimx",
                          src.dir = tmp,
                          parm.paths = c(pp1, pp2),
                          data = obsWheat, 
                          type = "grid",
                          replacement = c(TRUE, TRUE),
                          initial.values = list(RUE = 1.2, BasePhyllochron = 120),
                          verbose = TRUE,
                          grid = pgrd,
                          cores = 2)
  end <- Sys.time() ## It took 6.1 minutes 
                    ## Took 3.67 minutes using two cores and single-thread (Mac 2017)
  
  ggplot(wop.grd$res, aes(x = RUE, y = lrss)) + 
    facet_wrap(~BasePhyllochron) + 
    geom_point() + 
    geom_line()
  
  pp1 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = ".", 
                                    node = "Wheat", 
                                    node.child = "Leaf",
                                    node.subchild = "Photosynthesis",
                                    node.subsubchild = "RUE",
                                    parm = "FixedValue",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)
 
  pp2 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx", 
                                    src.dir = ".", 
                                    node = "Wheat", 
                                    node.child = "Cultivars",
                                    node.subchild = "USA",
                                    node.subsubchild = "Yecora",
                                    parm = "BasePhyllochron",
                                    print.path = TRUE,
                                    display.available = FALSE, 
                                    verbose = FALSE)
  
 ## Compare to optim
  start <- Sys.time()
  wop.nm <- optim_apsimx("Wheat-opt-ex.apsimx",
                          src.dir = tmp,
                          parm.paths = c(pp1, pp2),
                          data = obsWheat, 
                          replacement = c(TRUE, TRUE),
                          initial.values = list(RUE = 1.8, BasePhyllochron = 120))
  end <- Sys.time() ## It took 5.7 minutes 
  
  ### Maybe we need a fine grid around the true values?
  pgrd3 <- expand.grid(RUE = c(1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9), 
                      BasePhyllochron = c(70, 75, 80, 85, 90, 95, 100))  
  
  start <- Sys.time()
  wop.grd3 <- optim_apsimx("Wheat-opt-ex.apsimx",
                          src.dir = tmp,
                          parm.paths = c(pp1, pp2),
                          data = obsWheat, 
                          type = "grid",
                          replacement = c(TRUE, TRUE),
                          initial.values = list(RUE = 1.5, BasePhyllochron = 90),
                          verbose = TRUE,
                          grid = pgrd3,
                          cores = 3)
  end <- Sys.time() ## It took 6.63 minutes 
  
  ggplot(wop.grd3$res, aes(x = RUE, y = lrss)) + 
    facet_wrap(~BasePhyllochron) + 
    geom_point() + 
    geom_line()

  system.time(sim.a4 <- apsimx("Wheat-opt-ex.apsimx", src.dir = tmp)) ## This takes 4-5 seconds
  wop.grd3  
  cmp <- compare_apsim(obsWheat, sim.a4, labels = c("obs", "sim"), cRSS = TRUE)
}

## Testing the edit family of functions
require(apsimx)
extd.dir <- system.file("extdata", package = "apsimx")

run.test.edit.apsimx.replacement <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.test.edit.apsimx.replacement){
  
  ## Inspect, edit, inspect
  inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                          node = "Soybean", 
                          node.child = "Leaf",
                          parm = "Gsmax350")
  
  edit_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                          wrt.dir = tmp.dir, 
                          node = "Soybean", 
                          node.child = "Leaf",
                          parm = "Gsmax350", value = 0.009, 
                          verbose = FALSE)
  
  inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                          node = "Soybean", 
                          node.child = "Leaf",
                          parm = "Gsmax350")
 
  ## Example for RUE 
  ## Inspect, edit, inspect
  inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                             node = "Soybean", 
                             node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE",
                             parm = "FixedValue")
  
  edit_apsimx_replacement("MaizeSoybean.apsimx", 
                          src.dir = extd.dir, wrt.dir = tmp.dir,
                          node = "Soybean", 
                          node.child = "Leaf",
                          node.subchild = "Photosynthesis",
                          node.subsubchild = "RUE",
                          parm = "FixedValue", value = 1, verbose = FALSE)
  
  inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", src.dir = tmp.dir,
                             node = "Soybean", 
                             node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE",
                             parm = "FixedValue")
  
  #### Looking at Soybean
  inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                             node = "Soybean", 
                             node.child = "Cultivars",
                             node.subchild = "USA",
                             node.subsubchild = "IA2008_MG20",
                             parm = "VegetativePhotoperiod",
                             print.path = TRUE)
}

if(run.test.edit.apsimx.replacement){
  
  inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                             node = "Weather",
                             parm = "FileName",
                             root = "SimulationMaize")

  inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                             node = "Weather",
                             parm = "FileName",
                             root = "SimulationSoybean")

  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                                   node = "field",
                                   node.child = "Soil",
                                   node.subchild = "Physical",
                                   parm = "LL15",
                                   root = "SimulationMaize",
                                   print.path = TRUE)
  
}

## Script to test cores functionality
## Last time running this was with R 4.4.2
## future ‘1.34.0’
## apsimx ‘2.8.253’

library(apsimx)
packageVersion("apsimx")
library(future)
options(apsimx.warn.versions = FALSE)

apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

run.test.two.cores.sns <- FALSE

if(run.test.two.cores.sns){

    tmp.dir <- tempdir()
    setwd(tmp.dir)
    ex.dir <- auto_detect_apsimx_examples()

    file.copy(file.path(ex.dir, "Wheat.apsimx"), ".")

    pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                           node = "Manager", parm = list("ertilise", 1))
    pp1 <- paste0(pp1, ".Amount")

    pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                           node = "Manager", parm = list("Sow using", 10))
    pp2 <- paste0(pp2, ".Population")

    ## The names in the grid should (partially) match the parameter path names
    grd <- expand.grid(Fertilise = c(50, 100, 150), Population = c(100, 200, 300))

    ## This takes 2-3 minutes
    system.time(sns0 <- sens_apsimx(file = "Wheat.apsimx",
                                    src.dir = ".",
                                    parm.paths = c(pp1, pp2),
                                    grid = grd))
    ## date        elapsed
    ## 2022-11-22      125 (2.08 minutes)
    ## 2024-05-05      121 (2.0 minutes) - Mac 2017
    ## 2024-08-05      110 (1.9 minutes) - Mac 2017
    ## 2024-08-19      63.68 (Dell Precision 5680)
    ## 2024-05-05      95.8 (Mac Pro 2021)
    ## 2024-08-12      61.92 (Dell Precision 7865)
    ## 2025-07-21      89.23 (Dell Precision 7865)
    
    ## The two core simulation seems to work when number of simulations are even for n - 1
    system.time(sns1 <- sens_apsimx(file = "Wheat.apsimx",
                                    src.dir = ".",
                                    parm.paths = c(pp1, pp2),
                                    grid = grd,
                                    cores = 2))

    ## Make sure that data.frames are the same

    ## date        user system elapsed
    ## 2022-11-22   292     11    100 (1.67 minutes. It takes 84% of the time compared to 1 core)
    ## 2024-03-06   64      3.3    37.259 (0.52 minutes 57% of the time)
    ## 2024-08-12                  34.87 (Dell Precision 7865)
    ## 2024-08-19                  40.47 (Dell Precision 5680)
    ## 2024-08-19                  18.16 (Dell Precision 5680 - 8 cores)
    ## 2024-05-05                  55.77 (Mac Pro 2021)
    ## 2024-05-05                  27 (Mac Pro 2021 - 8 cores)
    ## 2025-07-21                 51.15 (Dell Precision 7865)

    ## Are the results identical?
    is.it.zero <- sum(colSums(sns0$grid.sims - sns1$grid.sims))
    if(is.it.zero > 0.5)
      stop("Results are not identical when cores = 1 and cores = 2")

    ## Testing having uneven number of simulations to parallelize
    grd2 <- grd[1:8, ]

    system.time(sns2 <- sens_apsimx(file = "Wheat.apsimx",
                                    src.dir = ".",
                                    parm.paths = c(pp1, pp2),
                                    grid = grd2,
                                    cores = 2))

    nrow.sns2 <- nrow(sns2$grid.sims)
    ## Compare results
    is.it.zero <- sum(colSums(sns1$grid.sims[1:8,] - sns2$grid.sims))

    if(is.it.zero > 0.5)
      stop("Results are not identical when cores = 1 and cores = 2")

}

run.test.more.cores.sns <- FALSE

if(run.test.more.cores.sns){

  tmp.dir <- tempdir()
  setwd(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()

  file.copy(file.path(ex.dir, "Wheat.apsimx"), ".")

  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Fertilise", 1))
  pp1 <- paste0(pp1, ".Amount")

  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Sow using", 10))
  pp2 <- paste0(pp2, ".Population")

  ## The names in the grid should (partially) match the parameter path names
  grd <- expand.grid(Fertilise = c(50, 100, 150), Population = c(100, 200, 300))

  system.time(sns3 <- sens_apsimx(file = "Wheat.apsimx",
                                 src.dir = ".",
                                 parm.paths = c(pp1, pp2),
                                 grid = grd,
                                 cores = 3))

  ## Compare results
  is.it.zero <- sum(colSums(sns1$grid.sims - sns3$grid.sims))

  if(is.it.zero > 0.5)
    stop("Results are not identical when cores = 2 and cores = 3")

  grd2 <- grd[1:8, ]

  system.time(sns4 <- sens_apsimx(file = "Wheat.apsimx",
                                  src.dir = ".",
                                  parm.paths = c(pp1, pp2),
                                  grid = grd2,
                                  cores = 4))

  ## Compare results
  is.it.zero <- sum(colSums(sns2$grid.sims - sns4$grid.sims))

  if(is.it.zero > 0.5)
    stop("Results are not identical when cores = 3 and cores = 4 (smaller grid)")

  ## Trying more cores
  system.time(sns5 <- sens_apsimx(file = "Wheat.apsimx",
                                  src.dir = ".",
                                  parm.paths = c(pp1, pp2),
                                  grid = grd,
                                  cores = 5))

  is.it.zero <- sum(colSums(sns1$grid.sims - sns5$grid.sims))

  if(is.it.zero > 0.5)
    stop("Results are not identical when cores = 1 and cores = 5")


}


run.test.even.more.cores.sns <- FALSE

if(run.test.even.more.cores.sns){

  tmp.dir <- tempdir()
  setwd(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()

  file.copy(file.path(ex.dir, "Wheat.apsimx"), ".")

  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Fertilise", 1))
  pp1 <- paste0(pp1, ".Amount")

  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Sow using", 10))
  pp2 <- paste0(pp2, ".Population")

  ## The names in the grid should (partially) match the parameter path names
  grd <- expand.grid(Fertilise = c(50, 100, 150), Population = c(100, 200, 300))

  system.time(sns3 <- sens_apsimx(file = "Wheat.apsimx",
                                 src.dir = ".",
                                 parm.paths = c(pp1, pp2),
                                 grid = grd,
                                 cores = 4))
  ## 2024-08-12 Elapsed time: 21.95 (Dell Precision 7865)
  ## 2024-07-21 Elapsed time: 32.39 (Dell Precision 7865)
  

  system.time(sns8 <- sens_apsimx(file = "Wheat.apsimx",
                                  src.dir = ".",
                                  parm.paths = c(pp1, pp2),
                                  grid = grd,
                                  cores = 8))
  ## 2024-08-12 Elapsed time: 15.91 (Dell Precision 7865)
  ## 2025-07-21 Elapsed time: 22.45 (Dell Precision 7865)
  
  ## Compare results
  is.it.zero <- sum(colSums(sns3$grid.sims - sns8$grid.sims))

  if(is.it.zero > 0.5)
    stop("Results are not identical when cores = 2 and cores = 3 (smaller grid)")

  grd2 <- expand.grid(Fertilise = c(50, 100, 150, 200, 250), Population = c(100, 200, 300, 400, 500))

  ## Trying more cores
  system.time(sns10 <- sens_apsimx(file = "Wheat.apsimx",
                                   src.dir = ".",
                                   parm.paths = c(pp1, pp2),
                                   grid = grd2,
                                   cores = 10))
  ## 2025-07-21 Elapsed time: 46.97 (Dell Precision 7865)
  
}

#### Testing the 'save' feature ----

run.test.save.cores.sns <- FALSE

if(run.test.save.cores.sns){

  tmp.dir <- tempdir()
  setwd(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()

  file.copy(file.path(ex.dir, "Wheat.apsimx"), ".")

  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Fertilise", 1))
  pp1 <- paste0(pp1, ".Amount")

  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = ".",
                        node = "Manager", parm = list("Sow using", 10))
  pp2 <- paste0(pp2, ".Population")

  ## The names in the grid should (partially) match the parameter path names
  grd <- expand.grid(Fertilise = c(50, 100, 150), Population = c(100, 200, 300))

  ## This takes 1 minute
  system.time(sns01 <- sens_apsimx(file = "Wheat.apsimx",
                                  src.dir = ".",
                                  parm.paths = c(pp1, pp2),
                                  grid = grd,
                                  save = TRUE))

  ## This takes 1 minute
  system.time(sns02 <- sens_apsimx(file = "Wheat.apsimx",
                                  src.dir = ".",
                                  parm.paths = c(pp1, pp2),
                                  grid = grd,
                                  save = "wheat_int_results.csv"))

}

## Testing the essence of the ssurgo workflow
require(apsimx)
require(sf)
apsimx_options(warn.versions = FALSE)

extd.dir <- system.file("extdata", package = "apsimx")

run.test.ssurgo.workflow <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.test.ssurgo.workflow){

  ## Read the main files
  chorizon <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_chorizon.csv"))
  component <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_component.csv"))
  mapunit <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_mapunit.csv"))
  mapunit.shp <- sf::st_read(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_Mapunits.shp"), quiet = TRUE)
  
  ## Get tables from SSURGO
  srgtbls <- get_ssurgo_tables(lonlat = c(-93, 42))
  chorizon <- srgtbls$chorizon
  component <- srgtbls$component
  mapunit <- srgtbls$mapunit
  mapunit.shp <- srgtbls$mapunit.shp
  
  sp <- ssurgo2sp(mapunit = mapunit, 
                  component = component, 
                  chorizon = chorizon,
                  mapunit.shp = mapunit.shp,
                  nlayers = 20)
  
  ## For first soil
  metadata <- attributes(sp[[1]])
  metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL;
  
  asp <- apsimx_soil_profile(nlayers = 20,
                            Thickness = sp[[1]]$Thickness * 10,
                            BD = sp[[1]]$BD,
                            AirDry = sp[[1]]$AirDry,
                            LL15 = sp[[1]]$LL15,
                            DUL = sp[[1]]$DUL,
                            SAT = sp[[1]]$SAT,
                            KS = sp[[1]]$KS,
                            Carbon = sp[[1]]$Carbon,
                            crop.LL = sp[[1]]$LL15,
                            soil.bottom = 200,
                            metadata = metadata)
  
  check_apsimx_soil_profile(asp)

  edit_apsimx_replace_soil_profile("WheatRye.apsimx",
                                   src.dir = extd.dir,
                                   wrt.dir = tmp.dir,
                                   soil.profile = asp,
                                   edit.tag = "-sp")
  
  inspect_apsimx("WheatRye-sp.apsimx", src.dir = tmp.dir, node = "Soil")
  
  inspect_apsimx("WheatRye-sp.apsimx", src.dir = tmp.dir, 
                 node = "Soil", soil.child = "Physical")
  
  ## If I want to actually run the model
  file.copy(paste0(extd.dir,"/Ames.met"),tmp.dir)
  
  inspect_apsimx("WheatRye-sp.apsimx", src.dir = tmp.dir, node = "Clock")
  
  sim <- apsimx("WheatRye-sp.apsimx", src.dir = tmp.dir, value = "report")
  
}

if(run.test.ssurgo.workflow){
  
  ## Testing the shift feature
  stbls <- get_ssurgo_tables(lonlat = c(-82.97, 40.739), shift = 300)
  stbls$mapunit$mukey  
  
  sp0 <- ssurgo2sp(mapunit = stbls$mapunit, component = stbls$component,
                   chorizon = stbls$chorizon, mapunit.shp = stbls$mapunit.shp)
  
  stbls.sp <- get_ssurgo_soil_profile(lonlat = c(-82.97, 40.739), shift = 300)
  
  
}

require(apsimx)

run.test.slga.workflow <- get(".run.local.tests", envir = apsimx.options)
#### This takes too long for routine testing
run.test.slga.workflow <- FALSE

if(run.test.slga.workflow){
  
  ## Sample points within Australia
  lonlat <- c(151.8306, -27.4969)
  
  start <- Sys.time()
  slga1 <- get_slga_soil(lonlat = lonlat)
  (Sys.time() - start)
  
  slga1.sp <- get_slga_soil_profile(lonlat = lonlat)
  (Sys.time() - start)
  
  ### Testing the effectiveness of the 'xargs' option
  
  ### Testing the number of layers
  sp4 <- get_slga_soil_profile(lonlat = lonlat, fix = TRUE, xargs = list(nlayers = 5))
  (Sys.time() - start)
  
  if(length(sp4$soil$Depth) != 5){
    stop("Length of sp4$soil$Depth should be equal to 5")
  }
  
  ### Testing the soil.bottom argument
  sp5 <- get_slga_soil_profile(lonlat = lonlat, fix = TRUE, xargs = list(soil.bottom = 210))
  (Sys.time() - start)
  
  if(sum(sp5$soil$Thickness) * 1e-1 != 210){
    stop("Bottom of soil profile shuod be equal to 210")
  }
  
  ### Testing soil bottom and number of layers
  sp6 <- get_slga_soil_profile(lonlat = lonlat, fix = TRUE, xargs = list(nlayers = 7, soil.bottom = 210))
  (Sys.time() - start)
  
  if(sum(sp6$soil$Thickness) * 1e-1 != 210){
    stop("Bottom of soil profile shuod be equal to 210")
  }
  if(length(sp6$soil$Depth) != 7){
    stop("Length of sp6$soil$Depth should be equal to 7")
  }
  
  ### This still works
  sp7 <- get_slga_soil_profile(lonlat = lonlat, fix = TRUE, xargs = list(crops = "Canola"))
  (Sys.time() - start)
  
  if(sp7$crops != "Canola"){
    stop("sp7$crops should be 'Canola'")
  }
}

#' Artificial observed data for Wheat
#'
#' A dataset containing the Date, phenology stage, LAI and above ground biomass for Wheat
#'
#' @title Observed wheat phenology, LAI and biomass
#' @format A data frame with 10 rows and 4 variables:
#' \describe{
#'   \item{Date}{ -date- date starting Oct 1 2016 and ending June 6 2017}
#'   \item{Wheat.Phenology.Stage}{ -numeric- phenology stage of wheat}
#'   \item{Wheat.Leaf.LAI}{ -numeric- Leaf Area Index}
#'   \item{Wheat.AboveGround.Wt}{-numeric- above ground biomass (g/m2)}
#' }
#' @source These are simulated data. For details see the APSIM documentation
"obsWheat"

#' Results from Wheat optimization example
#'
#' @title Wheat example optimization results
#' @format An object of class 'optim_apsim' 
#' \describe{
#'   \item{ wop }{ wheat optimization results }
#' }
#' @source Result of running the examples in Parameter Optimization vignette
"wop"

#' Results from Wheat optimization example plus the Hessian
#'
#' @title Wheat example optimization results plus Hessian
#' @format An object of class 'optim_apsim' 
#' \describe{
#'   \item{ wop.h }{ wheat optimization results plus Hessian}
#' }
#' @source Result of running the examples in Parameter Optimization vignette with the added Hessian
"wop.h"

## Testing downloading soil data from ISRIC
## In repeated testing it seems that speed slows down when multiple queries are requested

require(apsimx)

run.test.isric.workflow <- get(".run.local.tests", envir = apsimx.options)

if(run.test.isric.workflow){
  
  trrry <- try(sp1 <- get_isric_soil_profile(lonlat = c(-93, 42)), silent = TRUE)
  
  if(!inherits(trrry, "try-error")){
    # This takes a few seconds
    system.time(sp1 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE)) ## 1.508 seconds
    system.time(sp2 <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE))  ## 1.858 seconds
    
    plot(sp1)
    
    plot(sp2[[1]])
    
    ## The code below now should add LL15, KL and XF
    sp3 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, xargs = list(crops = "Oilpalm"))
    ## Only get SSURGO tables
    ## stbls <- get_ssurgo_tables(lonlat = c(-93, 42))    
  }
}



if(run.test.isric.workflow){
  
  ### Testing the effectiveness of the 'xargs' option
  
  trrry <- try(sp1 <- get_isric_soil_profile(lonlat = c(-93, 42)), silent = TRUE)
  
  if(!inherits(trrry, "try-error")){
    ### Testing the number of layers
    sp4 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, xargs = list(nlayers = 5))
    
    if(length(sp4$soil$Depth) != 5){
      stop("Length of sp4$soil$Depth should be equal to 5")
    }
    
    ### Testing the soil.bottom argument
    sp5 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, xargs = list(soil.bottom = 210))
    
    if(sum(sp5$soil$Thickness) * 1e-1 != 210){
      stop("Bottom of soil profile shuod be equal to 210")
    }
    
    ### Testing soil bottom and number of layers
    sp6 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, xargs = list(nlayers = 7, soil.bottom = 210))
    
    if(sum(sp6$soil$Thickness) * 1e-1 != 210){
      stop("Bottom of soil profile shuod be equal to 210")
    }
    if(length(sp6$soil$Depth) != 7){
      stop("Length of sp6$soil$Depth should be equal to 7")
    }
    
    ### This still works
    sp7 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, xargs = list(crops = "Canola"))
    
    if(sp7$crops != "Canola"){
      stop("sp7$crops should be 'Canola'")
    }
  }
}

#' Data source is USDA-NRCS Soil Data Access. See package soilDB for more details
#' 
#' @title Retrieve soil profile data and convert it to an object of class \sQuote{soil_profile}
#' @description Generate a synthetic soil profile based on the information in SSURGO database
#' @name get_ssurgo_soil_profile
#' @param lonlat Longitude and latitude vector (e.g. c(-93, 42))
#' @param shift simple mechanism for creating an area of interest by displacing the point indicated in 
#' lonlat by some amount of distance (e.g. 300 - in meters)
#' @param nmapunit number of mapunits to select (see \code{\link{ssurgo2sp}})
#' @param nsoil number of soils to select (see \code{\link{ssurgo2sp}}). If the 
#' number of soils is negative or NA it will fetch all the soils in the mapunit
#' @param xout see \code{\link{ssurgo2sp}}
#' @param soil.bottom see \code{\link{ssurgo2sp}}
#' @param method interpolation method see \code{\link{ssurgo2sp}}
#' @param nlayers number for layer for the new soil profile
#' @param check whether to check for reasonable values using \code{\link{check_apsimx_soil_profile}}.
#' TRUE by default. If \sQuote{fix} is TRUE, it will be applied only after the fix attempt.
#' @param fix whether to fix compatibility between saturation and bulk density (default is FALSE).
#' @param verbose default FALSE. Whether to print messages.
#' @param xargs additional arguments passed to \code{\link{apsimx_soil_profile}} function.
#' @return this function will always return a list. Each element of the list will
#' be an object of class \sQuote{soil_profile}
#' @export
#' @examples 
#' \dontrun{
#' require(soilDB)
#' require(sp)
#' require(sf)
#' require(spData)
#' require(ggplot2)
#' ## Soil inforation for a single point
#' sp <- get_ssurgo_soil_profile(lonlat = c(-93, 42))
#' ## The initial attempt throws warnings, so better to use 'fix'
#' sp <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE)
#' plot(sp[[1]])
#' plot(sp[[1]], property = "water")
#' ## Add initial water
#' iwat <- initialwater_parms(Thickness = sp[[1]]$soil$Thickness,
#'                            InitialValues = sp[[1]]$soil$DUL * 0.8)
#' sp[[1]]$initialwater <- iwat
#' plot(sp[[1]], property = "initialwater")
#' }
#' 
#' 

get_ssurgo_soil_profile <- function(lonlat, shift = -1,
                                    nmapunit = 1, nsoil = 1,
                                    xout = NULL, soil.bottom = 200,
                                    method = c("constant", "linear"),
                                    nlayers = 10,
                                    check = TRUE,
                                    fix = FALSE,
                                    verbose = FALSE,
                                    xargs = NULL){
  
  if(!requireNamespace("soilDB", quietly = TRUE)){
    stop("The soilDB package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sp", quietly = TRUE)){
    stop("The sp package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sf", quietly = TRUE)){
    stop("The sf package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("spData", quietly = TRUE)){
    stop("The spData package is required for this function")
    return(NULL)
  }
  
  if(length(lonlat) != 2 || !is.numeric(lonlat)) 
    stop("lonlat should be a vector with length equal to 2")

  lon <- lonlat[1]
  lat <- lonlat[2]
  
  ## Determine if the location is in the US
  if(requireNamespace("maps", quietly = TRUE)){
    country <- maps::map.where(x = lon, y = lat)
    if(country != "USA" || is.na(country))
      stop("These coordinates do not correspond to a location in the USA. \n Did you specify the coordinates correctly?")
  }
  
  if(shift <= 0){
    spg <- sp::SpatialPoints(cbind(x = lon, y = lat), 
                       proj4string = sp::CRS('+proj=longlat +datum=WGS84'))
  }else{
    shift <- (shift / 111) * 0.001 ## This is now in degrees
    lonlat.mat <- rbind(lonlat, ##root
                        lonlat + c(shift * 0.75, 0), ## x = 1, y = 0
                        lonlat + c(shift * 0.75, shift), ## x = 1, y = 1
                        lonlat + c(0, shift), ## x = 0, y = 1
                        lonlat) ## back to root
    rownames(lonlat.mat) <- NULL
    ## the previous matrix is a rectangle
    pg <- sp::Polygon(lonlat.mat)
    spg <- sp::SpatialPolygons(list(sp::Polygons(list(pg), "s1")),
                               proj4string = sp::CRS("+proj=longlat +datum=WGS84"))
  }
  
  if(verbose == FALSE){
    res <- suppressWarnings(soilDB::SDA_spatialQuery(spg, what = 'mupolygon', geomIntersection = TRUE))
  }else{
    res <- soilDB::SDA_spatialQuery(spg, what = 'mupolygon', geomIntersection = TRUE)  
  }
  
  mu.is <- soilDB::format_SQL_in_statement(res$mukey)
  sql <- sprintf("mukey IN %s", mu.is)
  if(verbose == FALSE){
    fSDA <- suppressWarnings(suppressMessages(soilDB::fetchSDA(sql, duplicates = TRUE)))
  }else{
    fSDA <- soilDB::fetchSDA(sql, duplicates = TRUE)
  } 
  
  ### Mapunit ### -- this might contain the iacornsr
  if(verbose == FALSE){
    mapunit <- suppressWarnings(suppressMessages(soilDB::get_mapunit_from_SDA(sql)))
  }else{
    mapunit <- soilDB::get_mapunit_from_SDA(sql) 
  }
  
  ### Component ###
  cmpnt <- fSDA@site
  names(cmpnt) <- gsub("_", ".", names(cmpnt), fixed = TRUE)
  cmpnt$geomdesc <- cmpnt$geompos
  
  ## Retrieve the state from the areasymbol
  if(shift <= 0 || length(unique(mapunit$areasymbol)) == 1){
    cmpnt$state <- unique(strtrim(mapunit$areasymbol, 2))
  }else{
    cmpnt$state <- NA
    warning("This area includes more than one state. 
            I have not though about how to get the state in this case. Please submit an issue 
            with a reproducible example to https://github.com/femiguez/apsimx/issues")
  }
  
  ### Chorizon ###
  chrzns <- fSDA@horizons
  names(chrzns) <- gsub("_", ".", names(chrzns), fixed = TRUE)
  ### Things missing from horizons: hzthk.r, partdensity, wsatiated.r, wfifteenbar.r, wtenthbar.r, wthirdbar.r,
  if(sum(grepl("partdensity", names(chrzns))) == 0) chrzns$partdensity <- NA
  if(sum(grepl("hzthk", names(chrzns))) == 0) chrzns$hzthk.r <- NA
  if(sum(grepl("wsatiated", names(chrzns))) == 0) chrzns$wsatiated.r <- NA
  if(sum(grepl("wfifteenbar", names(chrzns))) == 0) chrzns$wfifteenbar.r <- NA
  if(sum(grepl("wthirdbar", names(chrzns))) == 0) chrzns$wthirdbar.r <- NA
  
  if(shift <= 0){
    spg.sf <- sf::st_as_sf(spg)
    spg.sf[["MUKEY"]] <- res$mukey
    spg.sf[["AREASYMBOL"]] <- mapunit$areasymbol
    mapunit.shp <- spg.sf
  }else{
    mapunit.shp <- sf::st_as_sf(res)
  }

  sp0 <- ssurgo2sp(mapunit = mapunit, component = cmpnt,
                   chorizon = chrzns, mapunit.shp = mapunit.shp,
                   nmapunit = nmapunit, nsoil = nsoil, xout = xout,
                   soil.bottom = soil.bottom, method = method, nlayers = nlayers,
                   verbose = verbose)
  
  ans <- vector("list", length(sp0))
  
  for(i in seq_along(sp0)){
    metadata <- attributes(sp0[[i]])
    metadata$DataSource <- paste("SSURGO (https://sdmdataaccess.nrcs.usda.gov/) through R package soilDB, R package apsimx function ssurgo2sp. Timestamp",Sys.time())
    metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL;

    if(fix){
      icheck <- check
      check <- FALSE
    } 
    
    if(!is.null(xargs)){
      if(!is.null(xargs$crops)){
        crops <- xargs$crops
      }
    }else{
      crops <- c("Maize", "Soybean", "Wheat")
    }
    
    asp <- apsimx_soil_profile(nlayers = nlayers,
                               Thickness = sp0[[i]]$Thickness * 10,
                               BD = sp0[[i]]$BD,
                               AirDry = sp0[[i]]$AirDry,
                               LL15 = sp0[[i]]$LL15,
                               DUL = sp0[[i]]$DUL,
                               SAT = sp0[[i]]$SAT,
                               KS = sp0[[i]]$KS,
                               Carbon = sp0[[i]]$Carbon,
                               crop.LL = sp0[[i]]$LL15,
                               ParticleSizeClay = sp0[[i]]$ParticleSizeClay,
                               ParticleSizeSilt = sp0[[i]]$ParticleSizeSilt,
                               ParticleSizeSand = sp0[[i]]$ParticleSizeSand,
                               soil.bottom = soil.bottom,
                               metadata = metadata,
                               check = check, 
                               crops = crops)
    
    if(fix){
      asp <- fix_apsimx_soil_profile(asp, verbose = verbose)
      if(icheck)
        check_apsimx_soil_profile(asp)
    } 
    
    ans[[i]] <- asp
  }
  
  return(ans)
}

#' 
#' If the \sQuote{met} files do not have the same number of columns or the same
#' names for the data frame, then an error will be returned. If the years are not
#' the same a message will be returned with the different years and the years 
#' in common. 
#' 
#' @title Compare two or more \sQuote{met} files
#' @name compare_apsim_met
#' @rdname compare_apsim_met
#' @description Helper function which allows for a simple comparison among \sQuote{met} objects
#' @param ... met file objects. Should be of class \sQuote{met}
#' @param met.var meteorological variable to use in the comparison. Either \sQuote{all},
#' \sQuote{radn}, \sQuote{maxt}, \sQuote{mint}, \sQuote{rain}, \sQuote{rh}, 
#' \sQuote{wind_speed} or \sQuote{vp}. 
#' @param labels labels for plotting and identification of \sQuote{met} objects.
#' @param check whether to check \sQuote{met} objects using \sQuote{check_apsim_met}.
#' @param verbose whether to print agreement stats to console (default is FALSE).
#' @note I have only tested this for 2 or 3 objects. The code is set up to be able to 
#' compare more, but I'm not sure that would be all that useful.
#' @export
#' @return object of class \sQuote{met_mrg}, which can be used for further plotting
#' @examples 
#' \dontrun{
#' require(nasapower)
#' ## Specify the location
#' lonlat <- c(-93, 42)
#' ## dates
#' dts <- c("2017-01-01","2017-12-31")
#' ## Get pwr
#' pwr <- get_power_apsim_met(lonlat = lonlat, dates = dts)
#' ## Get data from IEM
#' iem <- get_iem_apsim_met(lonlat = lonlat, dates = dts)
#' ## Compare them
#' cmet <- compare_apsim_met(pwr[,1:6], iem, labels = c("pwr","iem"))
#' ## Visualize radiation
#' plot(cmet, met.var = "radn")
#' plot(cmet, plot.type = "diff")
#' plot(cmet, plot.type = "ts")
#' ## Visualize maxt
#' plot(cmet, met.var = "maxt")
#' plot(cmet, met.var = "maxt", plot.type = "diff")
#' plot(cmet, met.var = "maxt", plot.type = "ts")
#' ## Cumulative rain
#' plot(cmet, met.var = "rain", plot.type = "ts", cumulative = TRUE)
#' }
#' 

compare_apsim_met <- function(..., 
                              met.var = c("all", "radn", "maxt", 
                                          "mint", "rain", "rh", 
                                          "wind_speed", "vp"),
                              labels,
                              check = FALSE,
                              verbose = FALSE){
  
  mets <- list(...)
  
  n.mets <- length(mets)
  
  met.var <- match.arg(met.var)
  
  if(n.mets < 2) stop("you should provide at least two met objects", call. = FALSE)
  
  met1 <- mets[[1]]
  
  if(!inherits(met1, "met"))
    stop("The first object should be of class 'met'", call. = FALSE)
  
  m.nms <- NULL
  if(!missing(labels)){
    m.nms <- labels
    if(length(labels) != n.mets)
      stop(" 'labels' length should be the same as the number of 'met' objects", call. = FALSE)
  } 
  
  if(!inherits(met1, "met")) stop("object should be of class 'met' ", call. = FALSE)
  
  ## Check for any issues
  if(check) check_apsim_met(met1)
  
  ## Create the 'date' for indexing
  nms1 <- names(met1)
  met.mrg <- as.data.frame(met1)
  yr <- as.character(met1$year[1])
  met.mrg$dates <- as.Date(0:c(nrow(met1) - 1), origin = as.Date(paste0(yr, "-01-01")))
  names(met.mrg) <- c(paste0(names(met1), ".1"), "dates")

  for(i in 2:n.mets){
    
    if(!inherits(mets[[i]], "met")){
      stp.mssg <- paste("Object in position:", i, "is of class:", class(mets[[i]]),
                        ". Was expecting an object of class 'met'.")
      stop(stp.mssg, call. = FALSE)
    }
    
    met.i <- as.data.frame(unclass(mets[[i]]))

    if(ncol(met1) != ncol(met.i)) stop("met objects should have the same number of columns", call. = FALSE)
    if(length(setdiff(names(met1), names(met.i))) > 0){
      message(paste("Names for 'met1'", paste(names(met1), collapse = " ")))
      message(paste("Names for 'met.i'", paste(names(met.i), collapse = " ")))
      stop("met objects should have the same column names", call. = FALSE)
    } 
    
    ### Check about years in common
    unique.met1.years <- unique(met1$year)
    unique.met.i.years <- unique(met.i$year)
    if(length(intersect(unique.met1.years, unique.met.i.years)) == 0)
      stop("No years in common between the met files", call. = FALSE)
      
    if(length(setdiff(unique.met.i.years, unique.met1.years))){
      message("Years are different between met files")
      message(paste(c("Years in met1: ", unique.met1.years), collapse = " "))
      message(paste(c("Years in met.i: ", unique.met.i.years), collapse = " "))
      years.in.common <- as.character(intersect(unique.met1.years, unique.met.i.years))
      message(paste("Years in common: ", paste(years.in.common, collapse = " ")))
    }
    
    if(check) check_apsim_met(met.i)
    
    yr <- as.character(met.i$year[1])
    nms0 <- names(met.i)
    met.i$dates <- as.Date(0:c(nrow(met.i) - 1), origin = as.Date(paste0(yr, "-01-01")))
    names(met.i) <- c(paste0(nms0, ".", i), "dates")
    
    nms <- names(met.i)
    
    ## drop the year.i and day.i names
    met.mrg <- merge(met.mrg, met.i, by = "dates")

    if(dim(met.mrg)[[1]] == 0)
      stop("Failed to merge met files")
  }
  
  if(met.var == "all"){
    vrs <- rep(setdiff(names(met1), c("year", "day")), each = n.mets - 1)
    ans <- data.frame(variable = vrs,
                      vs = NA, labels = NA,
                      bias = NA, slope = NA, corr = NA)
    if(missing(labels)) ans$labels <- NULL
  }else{
    ans <- data.frame(variable = rep(met.var, n.mets - 1),
                      vs = NA, labels = NA,
                      bias = NA, slope = NA, corr = NA)
    if(missing(labels)) ans$labels <- NULL
  }
  
  ## Calculate bias for all variables
  if(met.var == "all"){
    met.var.sel <- nms1[!(nms1 %in% c("year", "day", "dates"))]
    gvar.sel <- paste0(met.var.sel, collapse = "|")
    idx.met.mrg <- grep(gvar.sel, names(met.mrg))
    met.mrg.s <- met.mrg[,idx.met.mrg]

    k <- 1  
    z <- 1
    ## Compute Bias matrix
    for(i in met.var.sel){
      if(verbose) cat("Variable: ", i, "\n")
      ans$variable[k] <- i
      tmp <- met.mrg.s[, grep(i, names(met.mrg.s))]
      if(ncol(tmp) < 2) stop("merged selected variables should be at least of length 2", call. = FALSE)

      for(j in 2:ncol(tmp)){
        if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
        ans$vs[k] <- paste(names(tmp)[j - 1], "vs.", names(tmp)[j])
        if(!missing(labels)){
          if(verbose) cat("labels:", labels[j - 1], " vs. ", labels[j], "\n")
          ans$labels[k] <- paste(labels[j - 1], "vs.", labels[j])
        } 
        fm0 <- lm(tmp[, j - 1] ~ tmp[, j])
        if(verbose) cat(" \t Bias: ", coef(fm0)[1], "\n")
        ans$bias[k] <- coef(fm0)[1]
        if(verbose) cat(" \t Slope: ", coef(fm0)[2], "\n")
        ans$slope[k] <- coef(fm0)[2]
        if(verbose) cat(" \t Corr: ", cor(tmp[,j - 1], tmp[, j]), "\n")
        ans$corr[k] <- cor(tmp[,j - 1], tmp[, j])
        if(verbose) cat(" \t RSS: ", deviance(fm0), "\n")
        ans$rss[k] <- deviance(fm0)
        if(verbose) cat(" \t RMSE: ", sigma(fm0), "\n")
        ans$rmse[k] <- sigma(fm0)
        k <- k + 1
      }
    }
  }
  
  if(met.var != "all"){
    ## Just select the appropriate variable
    idx.met.mrg <- grep(met.var, names(met.mrg))
    met.mrg.s <- met.mrg[, idx.met.mrg]
    
    if(verbose) cat("Variable ", met.var, "\n")
    ans$variable[1] <- met.var
    
    tmp <- met.mrg.s
    for(j in 2:ncol(tmp)){
      if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
      ans$vs[j - 1] <- paste(names(tmp)[j - 1], "vs.", names(tmp)[j])
      if(!missing(labels)){
        if(verbose) cat("labels", labels[j - 1], " vs. ", labels[j], "\n")
        ans$labels[j - 1] <- paste(labels[j - 1], "vs.", labels[j])
      }
      fm0 <- lm(tmp[, j - 1] ~ tmp[, j])
      if(verbose) cat(" \t Bias: ", coef(fm0)[1], "\n")
      ans$bias[j - 1] <- coef(fm0)[1]
      if(verbose) cat(" \t Slope: ", coef(fm0)[2], "\n")
      ans$slope[j - 1] <- coef(fm0)[2]
      if(verbose) cat(" \t Corr: ", cor(tmp[,j - 1], tmp[, j]), "\n")
      ans$corr[j - 1] <- cor(tmp[,j - 1], tmp[, j])
      if(verbose) cat(" \t RSS: ", deviance(fm0), "\n")
      ans$rss[j - 1] <- deviance(fm0)
      if(verbose) cat(" \t RMSE: ", sigma(fm0), "\n")
      ans$rmse <- sigma(fm0)
    }
  }

  attr(met.mrg, "met.names") <- m.nms
  attr(met.mrg, "length.mets") <- n.mets  
  met.mrg <- structure(list(met.mrg = met.mrg, index.table = ans),
                       class = "met_mrg")
  invisible(met.mrg)
}

#' print method for met_mrg
#' @rdname compare_apsim_met
#' @description print method for \sQuote{met_mrg}
#' @param x object of class \sQuote{met_mrg}
#' @param ... additional arguments passed to print
#' @param digits digits to print (default is 2)
#' @export
#' @return it prints the index.table data.frame
print.met_mrg <- function(x, ..., digits = 2){
  print(x$index.table, digits = digits)
}

#' Plotting function for weather data
#' @rdname compare_apsim_met
#' @description plotting function for compare_apsim_met, it requires ggplot2
#' @param x object of class \sQuote{met_mrg}
#' @param ... met file objects. Should be of class \sQuote{met}
#' @param plot.type either \sQuote{vs}, \sQuote{diff}, \sQuote{ts} - for time series or \sQuote{density}
#' @param pairs pair of objects to compare, defaults to 1 and 2 but others are possible
#' @param cumulative whether to plot cumulative values (default FALSE)
#' @param met.var meteorological variable to plot 
#' @param id identification (not implemented yet)
#' @param span argument to be passed to \sQuote{geom_smooth}
#' @return it produces a plot
#' @export
#' 
plot.met_mrg <- function(x, ..., plot.type = c("vs", "diff", "ts", "density"),
                         pairs = c(1, 2),
                         cumulative = FALSE,
                         met.var = c("radn", "maxt", "mint", "rain"),
                         id, span = 0.75){

  if(!requireNamespace("ggplot2", quietly = TRUE)){
    warning("ggplot2 is required for this plotting function")
    return(NULL)
  }
  
  x <- x$met.mrg
  
  m.nms <- attr(x, "met.names")
  if(max(pairs) > attr(x, "length.mets")) stop("pairs index larger than length of mets")
  
  x <- as.data.frame(unclass(x))
  
  plot.type <- match.arg(plot.type)
  met.var <- match.arg(met.var)
  
  if(cumulative && plot.type != "ts") 
    stop("cumulative is only available for plot.type = 'ts' ")
  
  if(plot.type == "vs" && met.var != "all" && !cumulative){
    tmp <- x[, grep(met.var, names(x))]
    prs <- paste0(met.var, ".", pairs)
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs[1]))), 
                                  y = eval(parse(text = eval(prs[2]))))) +
                 ggplot2::geom_point() + 
                 ggplot2::xlab(paste(m.nms[pairs[1]], prs[1])) + 
                 ggplot2::ylab(paste(m.nms[pairs[2]], prs[2])) + 
                 ggplot2::geom_smooth(method = "lm") + 
                 ggplot2::geom_abline(intercept = 0, slope = 1, color = "orange")
    
    print(gp1)

  }
  
  if(plot.type == "diff" && met.var != "all" && !cumulative){
    
    prs0 <- paste0(met.var, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    
    ## x Variable is prs[1]
    ## y Variable is prs[2] - prs[1]
    dff <- tmp[,prs0[2]] - tmp[,prs0[1]]
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))), 
                                                    y = dff)) +
      ggplot2::geom_point() + 
      ggplot2::xlab(paste(m.nms[pairs[1]], prs0[1])) + 
      ggplot2::ylab(paste("Difference", prs0[2], "-", prs0[1])) + 
      ggplot2::geom_smooth(method = "lm", ...) + 
      ggplot2::geom_hline(yintercept = 0, color = "orange")
  
    print(gp1)   
  }
  
  if(plot.type == "ts" && met.var != "all" && !cumulative){
    
    prs0 <- paste0(met.var, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    tmp$dates <- x$dates ## Put it back in - kinda dumb 
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = .data[["dates"]], 
                                                    y = eval(parse(text = eval(prs0[1]))),
                                                    color = paste(m.nms[pairs[1]], prs0[1]))) +
                                                    
      ggplot2::geom_point() + 
      ggplot2::geom_smooth(span = span, ...) + 
      ggplot2::geom_point(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                       color = paste(m.nms[pairs[2]], prs0[2]))) + 
      ggplot2::geom_smooth(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                        color = paste(m.nms[pairs[2]], prs0[2])),
                           span = span, ...) + 
      ggplot2::xlab("Date") + 
      ggplot2::ylab(met.var) + 
      ggplot2::theme(legend.title = ggplot2::element_blank())
    
    print(gp1)   
  }
  
  if(plot.type == "ts" && met.var != "all" && cumulative){
    
    prs0 <- paste0(met.var, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    tmp$dates <- x$dates
    tmp$cum_var1 <- cumsum(tmp[, prs0[1]])
    tmp$cum_var2 <- cumsum(tmp[, prs0[2]])
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = .data[["dates"]], 
                                                    y = .data[["cum_var1"]],
                                                    color = paste(m.nms[pairs[1]], prs0[1]))) +
      
      ggplot2::geom_line() + 
      ggplot2::geom_line(ggplot2::aes(y = .data[["cum_var2"]],
                                       color = paste(m.nms[pairs[2]], prs0[2]))) + 
      ggplot2::xlab("Date") + 
      ggplot2::ylab(paste("Cumulative ", met.var)) + 
      ggplot2::theme(legend.title = ggplot2::element_blank())
    
    print(gp1)   
  }
  
  if(plot.type == "density" && met.var != "all" && !cumulative){
    
    prs0 <- paste0(met.var, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))),
                                                    color = paste(m.nms[pairs[1]], prs0[1]))) + 
      ggplot2::geom_density() + 
      ggplot2::geom_density(ggplot2::aes(x = eval(parse(text = eval(prs0[2]))),
                                         color = paste(m.nms[pairs[2]], prs0[2]))) +
      ggplot2::xlab(met.var) + 
      ggplot2::theme(legend.title = ggplot2::element_blank())
    
    print(gp1)
  }
  invisible(gp1)
}

#' The frost free period is computed by first spliting each year (or year interval)
#' in two halves. The first and last frosts in the first and second period are found.
#' For the Northern hemisphere calendar days are used (1-365).
#' For the Southern hemisphere the year is split in two halfs, but the second half of
#' the year is used as the first part of the growing season.
#' If frost is not found a zero is returned.
#'
#' @title Summary for an APSIM met file
#' @name summary.met
#' @description Create a data.frame summarizing an object of class \sQuote{met}
#' @param object object of class \sQuote{met}
#' @param ... optional argument (none used at the moment) 
#' @param years optional argument to subset years
#' @param months optional argument to subset by months. If an integer, it should
#' be between 1 and 12. If a character, it can be in the format, for example, 
#' \sQuote{jan} or \sQuote{Jan}.
#' @param days optional argument to subset by days. It should be an integer
#'  between 1 and 31. 
#' @param julian.days optional argument to subset by julian days. It
#' should be a vector of integers between 1 and 365. Either use \sQuote{days} or
#' \sQuote{julian.days} but not both.
#' @param by by default the summary is performed by \sQuote{year}. This allows
#' the use of \sQuote{month}. 
#' @param compute.frost logical (default FALSE). Whether to compute
#' frost statistics.
#' @param frost.temperature value to use for the calculation of the frost 
#' period (default is zero).
#' @param anomaly whether to compute the anomaly. Default is FALSE. 
#' It could be TRUE (for all variables) or a character vector for a specific set of variables.
#' @param anomaly.units what units to use for anomaly calculations. Default is \sQuote{percent}.
#' Other options are \sQuote{standard.devaition} and \sQuote{absolute} - this means the difference, 
#' which will be in the original units.
#' @param check logical (default FALSE). Whether to \sQuote{check} the \sQuote{met} object.
#' @param verbose whether to print additional infomation to the console
#' @param na.rm whether to remove missing values. Passed to \sQuote{aggregate}
#' @param digits digits for rounding (default is 2).
#' @return an object of class \sQuote{data.frame} with attributes
#' @export
#' @examples 
#' 
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
#' 
#' summary(ames, years = 2014:2016)
#' 
summary.met <- function(object, ..., years, months, days, julian.days,
                        by = c("year", "month"),
                        compute.frost = FALSE,
                        frost.temperature = 0, 
                        anomaly, 
                        anomaly.units = c("percent", "standard.deviation", "absolute"),
                        check = FALSE, verbose = FALSE, 
                        na.rm = FALSE, digits = 2){

  if(check) check_apsim_met(object)
  
  x <- object
  
  if(!missing(days) && !missing(julian.days))
    stop("Either use days or julian.days but not both", call. = TRUE)

  by <- match.arg(by)
  anomaly.units <- match.arg(anomaly.units)
  
  ## Summarize information by year
  if(!missing(years)){
    #### Error message if years outside the range
    year.range <- range(x$year)
    if(min(years) < year.range[1])
      stop("Smallest value in 'years' is not within the range of years in the 'met' object", call. = FALSE)
    if(max(years) > year.range[2])
      stop("Largest value in 'years' is not within the range of years in the 'met' object", call. = FALSE)
    x <- x[x$year %in% years,]
  } 
  
  if(!missing(months)){
    if(length(months) == 1) months <- as.integer(months)
    if(!inherits(months, "integer") && !inherits(months, "character"))
      stop("months should be either an integer or a character", call. = FALSE)
    ## Select months that fit the criteria
    date.range <- seq(as.Date("2012-01-01"), as.Date("2012-12-31"), by = "day")
    dat <- data.frame(month = as.numeric(format(date.range, "%m")),
                      Month = format(date.range, "%b"),
                      day = as.numeric(format(date.range, "%j")))
    if(inherits(months, "integer")){
      if(any(months < 1) || any(months > 12))
        stop("months should be between 1 and 12", call. = FALSE)
      wch.months <- which(dat$month %in% months)
      x <- x[x$day %in% dat[wch.months, "day"],]
    }
    if(inherits(months, "character")){
      ## Months might be in upper of lower case
      Months <- format(as.Date(paste0(1, months, 2012), "%d%b%Y"), "%b")
      wch.months <- which(dat$Month %in% Months)
      x <- x[x$day %in% dat[wch.months, "day"],]
    }
  }else{
    months <- "1:12"
  }
  
  if(!missing(days)){
    if(!inherits(days, "integer"))
      stop("days should be of class integer", call. = FALSE)
    if(any(days < 1) || any(days > 31))
      stop("days should be between 1 and 31")
    ## Select days that fit the criteria
    date.range <- seq(as.Date("2012-01-01"), as.Date("2012-12-31"), by = "day")
    dat <- data.frame(month = as.numeric(format(date.range, "%m")),
                      Month = format(date.range, "%b"),
                      Day = as.numeric(format(date.range, "%d")),
                      day = as.numeric(format(date.range, "%j")))
    if(is.numeric(days)){
      wch.days <- which(dat$Day %in% days)
      x <- x[x$day %in% dat[wch.days, "day"],]
    }
  }else{
    days <- 1:31
  }
  
  if(!missing(julian.days)){
    x <- x[x$day %in% julian.days, ]
    days <- range(julian.days)
  }
  
  n.years <- length(unique(x$year))
  
  ## If I want to both compute frost and calculate Classic_TT
  nrws <- ifelse(by == "year", n.years, 12) 
    
  if(compute.frost && "Classic_TT" %in% names(x)){
    ans <- matrix(nrow = nrws, ncol = 18)  
  }
  if(!compute.frost && "Classic_TT" %in% names(x)){
    ans <- matrix(nrow = nrws, ncol = 14) 
  }
  if(compute.frost && isFALSE("Classic_TT" %in% names(x))){
    ans <- matrix(nrow = nrws, ncol = 16) 
  }
  if(!compute.frost && !"Classic_TT" %in% names(x)){
    ans <- matrix(nrow = nrws, ncol = 12) 
  }

  if(by == "year"){
    ans[,1] <- sort(unique(x$year))
    x <- add_column_apsim_met(x, value = as.factor(x$year), name = "year", units = "()")
  }else{
    ans[,1] <- 1:12
    mnth <- format(as.Date(as.character(paste(x$year, x$day, sep = "-")), format = "%Y-%j"), "%m")
    x <- add_column_apsim_met(x, value = mnth, name = "month", units = "()")
  }
  
  if(verbose){
    cat("First year:", x$year[1], "\n")
    cat("Last year:", x$year[nrow(x)], "\n")
    cat("Total number of years:", length(unique(x$year)), "\n")    
  }

  ## Store high maxt and mint temperature by year
  if(all(is.na(x$maxt))){
    ans[,4] <- NA
    ans[,6] <- NA
    ans[,8] <- NA
  }else{
    if(by == "year"){
      ans[,4] <- round(stats::aggregate(maxt ~ year, data = x, FUN = max, na.rm = na.rm)$maxt, digits) 
      ans[,6] <- round(stats::aggregate(maxt ~ year, data = x, FUN = mean, na.rm = na.rm)$maxt, digits)
      ans[,8] <- round(stats::aggregate(maxt ~ year, data = x, FUN = min, na.rm = na.rm)$maxt, digits)      
    }
    if(by == "month"){
      ans[,4] <- round(stats::aggregate(maxt ~ month, data = x, FUN = max, na.rm = na.rm)$maxt, digits) 
      ans[,6] <- round(stats::aggregate(maxt ~ month, data = x, FUN = mean, na.rm = na.rm)$maxt, digits)
      ans[,8] <- round(stats::aggregate(maxt ~ month, data = x, FUN = min, na.rm = na.rm)$maxt, digits)      
    }
  }
  
  if(all(is.na(x$mint))){
    ans[,5] <- NA
    ans[,7] <- NA
    ans[,9] <- NA
  }else{
    if(by == "year"){
      ans[,5] <- round(stats::aggregate(mint ~ year, data = x, FUN = max, na.rm = na.rm)$mint, digits)
      ans[,7] <- round(stats::aggregate(mint ~ year, data = x, FUN = mean, na.rm = na.rm)$mint, digits)
      ans[,9] <- round(stats::aggregate(mint ~ year, data = x, FUN = min, na.rm = na.rm)$mint, digits)  
    }
    if(by == "month"){
      ans[,5] <- round(stats::aggregate(mint ~ month, data = x, FUN = max, na.rm = na.rm)$mint, digits)
      ans[,7] <- round(stats::aggregate(mint ~ month, data = x, FUN = mean, na.rm = na.rm)$mint, digits)
      ans[,9] <- round(stats::aggregate(mint ~ month, data = x, FUN = min, na.rm = na.rm)$mint, digits)  
    }
  
  }
  
  ## Total precipitation
  if(all(is.na(x$rain))){
    ans[,10] <- NA
  }else{
    if(by == "year"){
      ans[,10] <- round(stats::aggregate(rain ~ year, data = x, FUN = sum, na.rm = na.rm)$rain, digits) 
    }
    if(by == "month"){
      ans[,10] <- round(stats::aggregate(rain ~ month, data = x, FUN = sum, na.rm = na.rm)$rain, digits) 
    }
  }
  ## Total and mean radiation
  if(all(is.na(x$radn))){
    ans[,11] <- NA
    ans[,12] <- NA
  }else{
    if(by == "year"){
      ans[,11] <- round(stats::aggregate(radn ~ year, data = x, FUN = sum, na.rm = na.rm)$radn, digits)
      ans[,12] <- round(stats::aggregate(radn ~ year, data = x, FUN = mean, na.rm = na.rm)$radn, digits)
    }
    if(by == "month"){
      ans[,11] <- round(stats::aggregate(radn ~ month, data = x, FUN = sum, na.rm = na.rm)$radn, digits)
      ans[,12] <- round(stats::aggregate(radn ~ month, data = x, FUN = mean, na.rm = na.rm)$radn, digits)
    }
  }
  
  if(by == "month" && compute.frost)
    stop("'by' = 'month' and 'compute.frost' are not available yet", call. = FALSE)
  
  ## How do I compute the length of the growing season
  if(compute.frost){
    lat0 <- strsplit(attr(x, "latitude"), "=")[[1]][2]
    lat <- as.numeric(strsplit(lat0, "(", fixed = TRUE)[[1]][1])

    ans[,13] <- rep(0, length(unique(x$year)))
    ans[,14] <- rep(0, length(unique(x$year)))
    ## This should work regardless of the subset, but I haven't tested
    if(lat >= 0){
      ## Northern hemisphere
      ## Last frost in the spring
      length.days.by.year <- stats::aggregate(day ~ year, data = x, FUN = length)$day
      half.point <- floor(mean(length.days.by.year)/2)
      x.first <- x[x$day < half.point, ]
      if(length(unique(x.first$year)) != length(unique(x$year))){
        stop("At least one year has incomplete days. Spring frost cannot be computed.", call. = FALSE)
      }
      frosts <- which(x.first$mint < frost.temperature)
      if(length(frosts) != 0){
        x.spring.frosts <- x.first[frosts,]
        last.spring.frost <- stats::aggregate(day ~ year, data = x.spring.frosts, FUN = max)        
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg1 <- merge(zero.days, last.spring.frost, all.x = TRUE, by = "year")
        mrg1[is.na(mrg1$day.y), "day.y"] <- 0
        ans[,13] <- mrg1[["day.y"]]
      }
      ## First frost in the fall
      x.last <- x[x$day > half.point, ]
      if(length(unique(x.last$year)) != length(unique(x$year))){
        stop("At least one year has incomplete days. Fall frost cannot be computed.", call. = FALSE)
      }
      frosts <- which(x.last$mint < frost.temperature)
      if(length(frosts) != 0){
        x.fall.frosts <- x.last[frosts,]
        first.fall.frost <- stats::aggregate(day ~ year, data = x.fall.frosts, FUN = min)        
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg2 <- merge(zero.days, first.fall.frost, all.x = TRUE, by = "year")
        mrg2[is.na(mrg2$day.y), "day.y"] <- 0
        ans[,14] <- mrg2[["day.y"]]
      }
      ## Frost days
      tmp0 <- x[x$mint < 0,]
      if(nrow(tmp0) > 0){
        all.frost.days <- stats::aggregate(day ~ year, data = tmp0, FUN = length)
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg3 <- merge(zero.days, all.frost.days, all.x = TRUE, by = "year")
        mrg3[is.na(mrg3$day.y), "day.y"] <- 0
        ans[,16] <- mrg3[["day.y"]]        
      }else{
        ans[,16] <- rep(0, length(unique(x$years)))
      }
      ## Frost-free period
      if(sum(ans[,13]) == 0 && sum(ans[,14]) == 0){
        ans[,15] <- stats::aggregate(day ~ year, data = x, FUN = length)$day 
      }else{
        if(all(ans[,13] > 0) && all(ans[,14] > 0)){
          ans[,15] <- ans[,14] - ans[,13]
        }else{
          ## Need to compute this by year
          yrs <- sort(unique(x$year))
          for(j in seq_along(yrs)){
            ## If first half has a zero
            if(ans[j, 13] == 0 && ans[j, 14] != 0){
              tmp <- x.fall.frosts[x.fall.frosts$year == yrs[j],]
              if(nrow(tmp) == 0){
                ans[j, 15] <- length.days.by.year[j]
              }else{
                last.fall.frost <- max(tmp$day)
                fall.frost.days <- last.fall.frost - ans[j, 14]
                ans[j, 15] <-  fall.frost.days                
              }
            }
            ## If second half has a zero
            if(ans[j, 14] == 0 && ans[j, 13] != 0){
              tmp <- x.spring.frosts[x.spring.frosts$year == yrs[j],]
              ##print(tmp)
              if(nrow(tmp) == 0){
                ans[j, 15] <- length.days.by.year[j]
              }else{
                first.spring.frost <- min(tmp$day)
                spring.frost.days <- ans[j, 13] - first.spring.frost
                ans[j, 15] <- spring.frost.days               
              }
            }
            ## Both are zero
            if(ans[j, 14] == 0 && ans[j, 13] == 0){
              ans[j, 15] <- length.days.by.year[j]
            }
            ## Both are not zero
            if(ans[j, 14] != 0 && ans[j, 13] != 0){
              ans[j, 15] <- ans[j,14] - ans[j,13]
            }
          }
        } 
      }
    }
    
    if(lat < 0){
      ## Southern hemisphere
      ## Last frost in the fall
      length.days.by.year <- stats::aggregate(day ~ year, data = x, FUN = length)$day
      half.point <- floor(mean(length.days.by.year)/2)
      x.first <- x[x$day > half.point, ]
      if(length(unique(x.first$year)) != length(unique(x$year))){
        stop("At least one year has incomplete days. Spring frost cannot be computed.", call. = FALSE)
      }
      frosts <- which(x.first$mint < frost.temperature)
      if(length(frosts) != 0){
        x.spring.frosts <- x.first[frosts,]
        last.spring.frost <- stats::aggregate(day ~ year, data = x.spring.frosts, FUN = max)        
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg1 <- merge(zero.days, last.spring.frost, all.x = TRUE, by = "year")
        mrg1[is.na(mrg1$day.y), "day.y"] <- 0
        ans[,13] <- mrg1[["day.y"]]
      }
      ## First frost in the fall
      x.last <- x[x$day < half.point, ]
      if(length(unique(x.last$year)) != length(unique(x$year))){
        stop("At least one year has incomplete days. Fall frost cannot be computed.", call. = FALSE)
      }
      frosts <- which(x.last$mint < frost.temperature)
      if(length(frosts) != 0){
        x.fall.frosts <- x.last[frosts,]
        first.fall.frost <- stats::aggregate(day ~ year, data = x.fall.frosts, FUN = min)        
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg2 <- merge(zero.days, first.fall.frost, all.x = TRUE, by = "year")
        mrg2[is.na(mrg2$day.y), "day.y"] <- 0
        ans[,14] <- mrg2[["day.y"]]
      }
      ## Frost days
      tmp0 <- x[x$mint < 0,]
      if(nrow(tmp0) > 0){
        all.frost.days <- stats::aggregate(day ~ year, data = tmp0, FUN = length)
        zero.days <- data.frame(year = sort(unique(x$year)), day = 0)
        mrg3 <- merge(zero.days, all.frost.days, all.x = TRUE, by = "year")
        mrg3[is.na(mrg3$day.y), "day.y"] <- 0
        ans[,16] <- mrg3[["day.y"]]        
      }else{
        ans[,16] <- rep(0, length(unique(x$year)))
      }
      ## Frost-free period
      if(sum(ans[,13]) == 0 && sum(ans[,14]) == 0){
        ans[,15] <- stats::aggregate(day ~ year, data = x, FUN = length)$day 
      }else{
        if(all(ans[,13] > 0) && all(ans[,14] > 0)){
          ##last.day <- ifelse(is_leap_year(sort(unique(x$year))), 366, 365)
          ans[,15] <- (length.days.by.year - ans[,13]) + ans[,14]
        }else{
          ## Need to compute this by year
          yrs <- sort(unique(x$year))
          for(j in seq_along(yrs)){
            ## If first half has a zero
            if(ans[j, 13] == 0 && ans[j, 14] != 0){
              tmp <- x.fall.frosts[x.fall.frosts$year == yrs[j],]
              if(nrow(tmp) == 0){
                ans[j, 15] <- length.days.by.year[j]
              }else{
                last.fall.frost <- max(tmp$day)
                fall.frost.days <- ans[j, 14] - last.fall.frost
                ans[j, 15] <-  fall.frost.days                
              }
            }
            ## If second half has a zero
            if(ans[j, 14] == 0 && ans[j, 13] != 0){
              tmp <- x.spring.frosts[x.spring.frosts$year == yrs[j],]
              ##print(tmp)
              if(nrow(tmp) == 0){
                ans[j, 15] <- length.days.by.year[j]
              }else{
                first.spring.frost <- min(tmp$day)
                spring.frost.days <- ans[j, 13] - first.spring.frost
                ans[j, 15] <- length.days.by.year[j] - spring.frost.days               
              }
            }
            ## Both are zero
            if(ans[j, 14] == 0 && ans[j, 13] == 0){
              ans[j, 15] <- length.days.by.year[j]
            }
            ## Both are not zero
            if(ans[j, 14] != 0 && ans[j, 13] != 0){
              ans[j, 15] <- (length.days.by.year[j] - ans[j,13]) + ans[j,14]
            }
          }
        } 
      }
    }
  }
  
  if("Classic_TT" %in% names(x)){
    if(isFALSE(compute.frost)){
      if(all(is.na(x$Classic_TT))){
        ans[,13] <- NA
        ans[,14] <- NA
      }else{
        if(by == "year"){
          ans[,13] <- round(stats::aggregate(Classic_TT ~ year, data = x, FUN = max, na.rm = na.rm)$Classic_TT, digits)
          ans[,14] <- round(stats::aggregate(Classic_TT ~ year, data = x, FUN = mean, na.rm = na.rm)$Classic_TT, digits)
        }
        if(by == "month"){
          ans[,13] <- round(stats::aggregate(Classic_TT ~ month, data = x, FUN = max, na.rm = na.rm)$Classic_TT, digits)
          ans[,14] <- round(stats::aggregate(Classic_TT ~ month, data = x, FUN = mean, na.rm = na.rm)$Classic_TT, digits)
        }

      }      
    }else{
      if(all(is.na(x$Classic_TT))){
        ans[,17] <- NA
        ans[,18] <- NA
      }else{
        if(by == "year"){
          ans[,17] <- round(stats::aggregate(Classic_TT ~ year, data = x, FUN = max, na.rm = na.rm)$Classic_TT, digits)
          ans[,18] <- round(stats::aggregate(Classic_TT ~ year, data = x, FUN = mean, na.rm = na.rm)$Classic_TT, digits)
        }
        if(by == "month"){
          ans[,17] <- round(stats::aggregate(Classic_TT ~ month, data = x, FUN = max, na.rm = na.rm)$Classic_TT, digits)
          ans[,18] <- round(stats::aggregate(Classic_TT ~ month, data = x, FUN = mean, na.rm = na.rm)$Classic_TT, digits)
        }
      } 
    }
  }

  if(compute.frost && isFALSE("Classic_TT" %in% names(x))){
    colnames(ans) <- c("year", "months", "days", ## 1, 2, 3
                       "high_maxt", "high_mint", ## 4 and 5
                       "avg_maxt", "avg_mint", ## 6 and 7
                       "low_maxt", "low_mint", ## 8 and 9
                       "rain_sum", "radn_sum", "radn_avg", ## 10, 11, 12
                       "first_half_frost", "second_half_frost", ## 13, 14
                       "frost_free_period", "frost_days") ## 15, 16    
  }
  
  if(isFALSE(compute.frost) && isFALSE("Classic_TT" %in% names(x))){
    colnames(ans) <- c("year", "months", "days", ## 1, 2, 3
                       "high_maxt", "high_mint", ## 4 and 5
                       "avg_maxt", "avg_mint", ## 6 and 7
                       "low_maxt", "low_mint", ## 8 and 9
                       "rain_sum", "radn_sum", "radn_avg") ## 10, 11, 12
  }
  
  if(isFALSE(compute.frost) && isTRUE("Classic_TT" %in% names(x))){
    colnames(ans) <- c("year", "months", "days", ## 1, 2, 3
                       "high_maxt", "high_mint", ## 4 and 5
                       "avg_maxt", "avg_mint", ## 6 and 7
                       "low_maxt", "low_mint", ## 8 and 9
                       "rain_sum", "radn_sum", "radn_avg", ## 10, 11, 12
                       "high_classic_tt", "avg_classic_tt") ## 13, 14 
  }
  
  if(isTRUE(compute.frost) && isTRUE("Classic_TT" %in% names(x))){
    colnames(ans) <- c("year", "months", "days", ## 1, 2, 3
                       "high_maxt", "high_mint", ## 4 and 5
                       "avg_maxt", "avg_mint", ## 6 and 7
                       "low_maxt", "low_mint", ## 8 and 9
                       "rain_sum", "radn_sum", "radn_avg", ## 10, 11, 12
                       "first_half_frost", "second_half_frost", ## 13, 14
                       "frost_free_period", "frost_days", ## 15, 16
                       "high_classic_tt", "avg_classic_tt") ## 17, 18 
  }
  
  if(by == "month"){
    clnms <- colnames(ans)
    colnames(ans) <- c("month", clnms[-1])
  }
  
  #### Calculate anomalies ----
  if(!missing(anomaly)){

    vars.to.anomaly <- setdiff(colnames(ans), c("year", "months", "month", "days"))
    #### This computes anomalies for all variables
    if(!isTRUE(anomaly)){
      v2a <- intersect(anomaly, vars.to.anomaly)
      if(length(v2a) == 0){
        v2a <- lapply(anomaly, function(x) grep(x, vars.to.anomaly, value = TRUE))
        if(length(v2a) == 0){
          stop("anomaly variable does not match an existing variable")  
        }else{
         vars.to.anomaly <- unlist(v2a) 
        }
      }else{
        vars.to.anomaly <- anomaly        
      }
    }

    for(i in vars.to.anomaly){
      tmp.var.to.anomaly <- as.data.frame(ans)[[i]]
      mean.i.var.to.anomaly <- mean(tmp.var.to.anomaly, na.rm = TRUE)
      sd.i.var.to.anomaly <- sd(tmp.var.to.anomaly, na.rm = TRUE)
      
      if(anomaly.units == "percent"){
        ### The units here are in % and the equation is (x / x_mean) - 1
        anmly.change <- ((tmp.var.to.anomaly / mean.i.var.to.anomaly) - 1) * 100        
      }
      if(anomaly.units == "absolute"){
        anmly.change <- tmp.var.to.anomaly - mean.i.var.to.anomaly        
      }
      if(anomaly.units == "standard.deviation"){
        anmly.change <- (tmp.var.to.anomaly - mean.i.var.to.anomaly)/sd.i.var.to.anomaly        
      }

      name.i.var <- paste0("anomaly_", i)
      tmpd <- data.frame(anomaly = anmly.change)
      names(tmpd) <- name.i.var
      ans <- cbind(ans, tmpd)
    }
  }
  
  ansd <- as.data.frame(ans)
  if(inherits(months, "integer")){
    if(grepl(":", deparse(months), fixed = TRUE)){
      ansd$months <- rep(paste(range(months), collapse = ":"), n.years)    
    }else{
      ansd$months <- rep(paste(months, collapse = ","), n.years)    
    }
  }else{
    ansd$months <- rep(paste(months, collapse = ","), n.years)  
  }
  
  ansd$days <- rep(deparse(days), n.years)
  
  if(by == "month"){
    if(missing(years)){
      yrs <- paste(range(x$year), collapse = ":")    
    }else{
      yrs <- paste(range(years), collapse = ":") 
    }
    ansd <- data.frame(month = ansd$month, years = yrs, ansd[,2:ncol(ansd)])
  }

  attr(ansd, "anomaly.units") <- anomaly.units
  
  return(ansd)
}

#' @title Read in a soils (XML) file into a list of \sQuote{soil_profile} objects
#' @name read_apsim_soils
#' @description APSIM soils can be stored as XML files (soils) and reading them in
#' converts them into a list of individual objects of class \sQuote{soil_profile}
#' @param file name of the file (the extension sohuld be .soils)
#' @param src.dir directory containing the .soils file (defaults to the current directory)
#' @param verbose whether to print additional information about the progress of reading 
#' the individual soils in.
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' sls <- read_apsim_soils("Clarion.soils", src.dir = extd.dir)
#' 
#' }

read_apsim_soils <- function(file, src.dir = ".", verbose = TRUE){
  
  src.dir <- normalizePath(src.dir)
  
  file.names <- dir(path = src.dir, pattern=".soils$", ignore.case=TRUE)
  
  if(length(file.names)==0){
    stop("There are no .soils files in the specified directory to edit.")
  }
  
  file <- match.arg(file, file.names)
  
  soils <- xml2::read_xml(file.path(src.dir, file))

  # if(!missing(root)){
  #   soils <- xml2::xml_find_first(soils, xpath = root)
  # }
  ## Find all soils
  soils.names <- xml2::xml_find_all(soils, ".//Soil")
  
  soils.nms <- xml2::xml_attr(soils.names, "name")
  
  ans.soils <- vector("list", length = length(soils.names))
  
  ## There are potentially many soils
  for(i in seq_along(soils.names)){

    if(verbose){
      cat("Soil:", i, "\n")
      cat(xml2::xml_attr(soils.names[i], "name"), "\n")
      cat("Progress (%):", i/length(soils.names) * 100, "\n")
    }    
    
    soil.path <- xml2::xml_path(soils.names[i]) ## Path to the current soil
    soil <- xml2::xml_find_first(soils, soil.path) ## Extract current soil
    ## This would need to extract the current soil
    ## Retrieve metadata
    ## Doing this manually because some need to converted to numeric and others don't
    metadata <- list()
    metadata$RecordNumber <- as.numeric(xml2::xml_text(xml2::xml_find_first(soil, ".//RecordNumber")))
    metadata$ASCOrder <- xml2::xml_text(xml2::xml_find_first(soil, ".//ASCOrder"))
    metadata$ASCSubOrder <- xml2::xml_text(xml2::xml_find_first(soil, ".//ASCSubOrder"))
    metadata$SoilType <- xml2::xml_text(xml2::xml_find_first(soil, ".//SoilType"))
    metadata$LocalName <- xml2::xml_text(xml2::xml_find_first(soil, ".//LocalName"))
    metadata$Site <- xml2::xml_text(xml2::xml_find_first(soil, ".//Site"))
    metadata$NearestTown <- xml2::xml_text(xml2::xml_find_first(soil, ".//NearestTown"))
    metadata$Region <- xml2::xml_text(xml2::xml_find_first(soil, ".//Region"))
    metadata$State <- xml2::xml_text(xml2::xml_find_first(soil, ".//State"))
    metadata$Country <- xml2::xml_text(xml2::xml_find_first(soil, ".//Country"))
    metadata$NaturalVegetation <- xml2::xml_text(xml2::xml_find_first(soil, ".//NaturalVegetation"))
    metadata$ApsoilNumber <- as.numeric(xml2::xml_text(xml2::xml_find_first(soil, ".//ApoilNumber")))
    metadata$Latitude <- as.numeric(xml2::xml_text(xml2::xml_find_first(soil, ".//Latitude")))
    metadata$Longitude <- as.numeric(xml2::xml_text(xml2::xml_find_first(soil, ".//Longitude")))
    metadata$LocationAccuracy <- xml2::xml_text(xml2::xml_find_first(soil, ".//LocationAccuracy"))
    metadata$YearOfSampling <- as.numeric(xml2::xml_text(xml2::xml_find_first(soil, ".//YearOfSampling")))
    metadata$DataSource <- xml2::xml_text(xml2::xml_find_first(soil, ".//DataSource"))
    metadata$Comments <- xml2::xml_text(xml2::xml_find_first(soil, ".//Comments"))
    
    ## Get the soil Water
    properties <- c("Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS", "BDMetadata", "AirDryMetadata", "LL15Metadata",
                    "DULMetadata", "SATMetadata")
    
    soil.soil.water <- vector("list", length = length(properties)) 
    names(soil.soil.water) <- properties
      
    for(j in seq_along(properties)){
      prprty.node <- xml2::xml_find_first(soil, paste0(".//Water/", properties[j]))
      if(grepl("Metadata", properties[j])){
        soil.soil.water[[j]] <- xml2::xml_text(xml2::xml_children(prprty.node))  
      }else{
        soil.soil.water[[j]] <- xml2::xml_double(xml2::xml_children(prprty.node))    
      }
    }
    
    wssw <- as.vector(which(sapply(soil.soil.water, length) > 0))
    soil.soil.water.d0 <- as.data.frame(soil.soil.water[wssw])
    soil.soil.water.metadata <- soil.soil.water.d0[, grepl("Metadata", names(soil.soil.water.d0))]
    soil.soil.water.d <- soil.soil.water.d0[, !grepl("Metadata", names(soil.soil.water.d0))]
    
    ## Get the soil Crop
    soil.soilcrops <- xml2::xml_attr(xml2::xml_find_all(soil, ".//SoilCrop"), "name")
    
    soil.soilcrops.thickness <- xml2::xml_double(xml2::xml_children(xml2::xml_find_first(soil, ".//SoilCrop/Thickness")))
      
    soilcrop.properties <- c("LL", "KL", "XF", "LLMetadata")
    
    soil.soilcrops.list <- vector("list", length = length(soil.soilcrops))
    names(soil.soilcrops.list) <- soil.soilcrops
    soilcrop.properties.list <- vector("list", length = length(soilcrop.properties))
    names(soilcrop.properties.list) <- soilcrop.properties
      
    for(j in seq_along(soil.soilcrops)){
      for(k in seq_along(soilcrop.properties)){
        soil.prprty.node <- xml2::xml_find_first(soil, paste0(".//SoilCrop[", j, "]/", soilcrop.properties[k]))
        if(grepl("Metadata", soilcrop.properties[k])){  
          soilcrop.properties.list[[k]] <- xml2::xml_text(xml2::xml_children(soil.prprty.node))  
        }else{
          tmp <- xml2::xml_double(xml2::xml_children(soil.prprty.node))  
          if(length(tmp) == 0){
            tmp <- rep(0, length(soilcrop.properties.list[[1]]))
          }
          soilcrop.properties.list[[k]] <- tmp
        }
      }
      soil.soilcrops.list[[j]] <- soilcrop.properties.list
    }
    
    ## Remove metadata if empty
    for(k in seq_along(soil.soilcrops.list)){
      tmp.crop <- soil.soilcrops.list[k]
      not.metadata <- !grepl("Metadata", sapply(tmp.crop, names))
      if(length(tmp.crop[[1]][[which(not.metadata == 0)]]) < 1){
        tmp.list <- vector("list", length = 1)
        names(tmp.list) <- names(tmp.crop)
        tmp.list[[1]] <- tmp.crop[[1]][not.metadata]
        soil.soilcrops.list[k] <- tmp.list         
      }
    }
    soil.soilcrops.d0 <- data.frame(Thickness = soil.soilcrops.thickness, as.data.frame(soil.soilcrops.list))
    soil.soilcrops.metadata <- soil.soilcrops.d0[, grepl("Metadata", names(soil.soilcrops.d0))]
    soil.soilcrops.d <- soil.soilcrops.d0[, !grepl("Metadata", names(soil.soilcrops.d0))]
    
    ## Get SoilWat parameters
    soilwat.node <- xml2::xml_find_all(soil, ".//SoilWater")   
    
    ## Get SWCON first
    swcon <- xml2::xml_double(xml2::xml_children(xml2::xml_find_first(soilwat.node, ".//SWCON")))
    soilwat.thickness <- xml2::xml_double(xml2::xml_children(xml2::xml_find_first(soilwat.node, ".//Thickness")))
    
    if(any(is.na(swcon))){
      swcon[is.na(swcon)] <- mean(swcon, na.rm = TRUE)
    }
    
    if(any(is.nan(swcon))){
      swcon[is.nan(swcon)] <- mean(swcon, na.rm = TRUE)
    }
    
    if(any(swcon > 1)){
      swcon[swcon > 1] <- mean(swcon)
    }
    
    soilwat <- soilwat_parms(Thickness = soilwat.thickness, SWCON = swcon)

    soilwat.var.names <- c("SummerCona", "SummerU", "SummerDate", "WinterCona", "WinterU", "WinterDate",
                           "DiffusConst", "DiffusSlope", "Salb", "CN2Bare", "CNRed", "CNCov", "Slope",
                           "DischargeWidth", "CatchmentArea", "MaxPond")
    
    for(j in seq_along(soilwat.var.names)){
      tmp <- xml2::xml_text(xml2::xml_find_first(soilwat.node, soilwat.var.names[j]))
      if(is.na(tmp)) next
      if(tmp == "NaN") next
      if(soilwat.var.names[j] %in% c("SummerDate", "WinterDate")){
        soilwat[[soilwat.var.names[j]]] <- tmp
      }else{
        soilwat[[soilwat.var.names[j]]] <- as.numeric(tmp)
      }
    }
    
    ## Section SoilOrganicMatter
    som.node <- xml2::xml_find_all(soil, ".//SoilOrganicMatter")   
    
    som.vars1 <- c("Thickness", "OC", "FBiom", "FInert", "SoilCN")
    som.vars2 <- c("RootCN", "RootWt", "EnrACoeff", "EnrBCoeff", "OCUnits")
    
    som.lst1 <- vector("list", length = length(som.vars1)) 
    som.lst2 <- vector("list", length = length(som.vars2)) 
    names(som.lst1) <- som.vars1
    names(som.lst2) <- som.vars2
    
    for(j in seq_along(som.vars1)){
      if(som.vars1[j] != "SoilCN"){
        tmp <- xml2::xml_text(xml2::xml_children(xml2::xml_find_first(som.node, som.vars1[j])))  
      }else{
        tmp <- xml2::xml_text(xml2::xml_find_first(som.node, som.vars1[j]))  
      }
      if(som.vars1[j] == "OCMetadata"){
        som.lst1[[som.vars1[j]]] <- tmp
      }else{
        if(som.vars1[j] == "SoilCN"){
          som.lst1[[som.vars1[j]]] <- as.numeric(rep(tmp, nrow(soil.soil.water.d))) 
        }else{
          if(length(tmp) == 0){
            tmp <- rep(0, length(som.vars1[[1]]))
          }
          som.lst1[[som.vars1[j]]] <- as.numeric(tmp)  
        }
      }
    }
    
    for(j in seq_along(som.vars2)){
      tmp <- xml2::xml_text(xml2::xml_find_first(som.node, som.vars2[j]))
      if(som.vars2[j] == "OCUnits"){
        som.lst2[[som.vars2[[j]]]] <- tmp
      }else{
        som.lst2[[som.vars2[j]]] <- as.numeric(tmp)
      }
    }

    ### Section fixing SoilCN
    if(length(som.lst1$SoilCN) == 0){
      som.lst1$SoilCN <- rep(0, length(som.lst1$Thickness))
    }
    
    if(length(som.lst1$SoilCN) == 1){
      som.lst1$SoilCN <- rep(som.lst1$SoilCN, length(som.lst1$Thickness))
    }
    
    if(length(som.lst1$Thickness) != length(som.lst1$SoilCN)){
      som.lst1$SoilCN <- rep(som.lst1$SoilCN[1], length(som.lst1$Thickness))
    }
    
    ### Section fixing FBiom
    if(length(som.lst1$FBiom) == 0){
      som.lst1$FBiom <- rep(0, length(som.lst1$Thickness))
    }
    
    if(length(som.lst1$FBiom) == 1){
      som.lst1$FBiom <- rep(som.lst1$FBiom, length(som.lst1$Thickness))
    }
    
    if(length(som.lst1$Thickness) != length(som.lst1$FBiom)){
      som.lst1$FBiom <- rep(som.lst1$FBiom[1], length(som.lst1$Thickness))
    }
    
    som.d1 <- as.data.frame(som.lst1)
    som.d2 <- soilorganicmatter_parms(RootCN = som.lst2$RootCN, RootWt = som.lst2$RootWt, 
                                      EnrACoeff = som.lst2$EnrACoeff, EnrBCoeff = som.lst2$EnrBCoeff,
                                      OCUnits = som.lst2$OCUnits)
    ## Analysis
    analysis.node <- xml2::xml_find_all(soil, ".//Analysis")       

    analysis.vars1 <- c("Thickness", "NO3N", "NH4N", "PH", "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand")
    analysis.vars2 <- c("PHMetadata", "ParticleSizeMetadata")
    analysis.vars3 <- c("PHUnits", "BoronUnits")
    
    analysis.list1 <- vector("list", length = length(analysis.vars1)) 
    names(analysis.list1) <- analysis.vars1
    analysis.list2 <- vector("list", length = length(analysis.vars2)) 
    names(analysis.list2) <- analysis.vars2
    analysis.list3 <- vector("list", length = length(analysis.vars3)) 
    names(analysis.list3) <- analysis.vars3
    
    for(j in seq_along(analysis.vars1)){
      tmp <- xml2::xml_text(xml2::xml_children(xml2::xml_find_first(analysis.node, analysis.vars1[j])))  
      if(j == 1) nlayers <- length(tmp)
      if(length(tmp) == 0){
        analysis.list1[[j]] <- rep(NA, nlayers)
      }else{
        analysis.list1[[j]] <- as.numeric(tmp)    
      }
    }
    
    for(j in seq_along(analysis.vars2)){
      tmp <- xml2::xml_text(xml2::xml_children(xml2::xml_find_first(analysis.node, analysis.vars2[j])))  
      analysis.list2[[j]] <- tmp  
    }
    
    for(j in seq_along(analysis.vars3)){
      tmp <- xml2::xml_text(xml2::xml_children(xml2::xml_find_first(analysis.node, analysis.vars3[j])))  
      analysis.list3[[j]] <- tmp
    }
    
    analysis.d1 <- as.data.frame(analysis.list1)
    analysis.d1 <- analysis.d1[,apply(analysis.d1[1,], 2, function(x) !is.na(x))]
    
    if(length(soil.soilcrops) > 0){
      if(sum(soil.soil.water.d$Thickness - soil.soilcrops.d$Thickness) > 0)
        stop("Thickness of soil.water does not match Thickness of soilcrops")
      which.soilcrops.thickness <- which(names(soil.soilcrops.d) == "Thickness")
      soil0.soil0 <- cbind(soil.soil.water.d, soil.soilcrops.d[,-which.soilcrops.thickness])      
    }else{
      soil0.soil0 <- soil.soil.water.d
    }

    if(length(soil0.soil0$Thickness) != length(som.d1$Thickness)){
      message("Thickness of soil.water/soilcrops does not match Thickness of soilorganicmatter")
      message("Adding/removing rows as necessary")
      nrow.diff <- nrow(soil0.soil0) - nrow(som.d1)
      if(nrow(som.d1) < nrow(soil0.soil0)){
        for(i in 1:nrow.diff){
          som.d1 <- rbind(som.d1, som.d1[nrow(som.d1), , drop = FALSE])  
        }
      }else{
        som.d1 <- som.d1[1:length(soil0.soil0$Thickness),]
      }
    }
      
    which.som.thickness <- which(names(som.d1) == "Thickness")
    soil0.soil1 <- cbind(soil0.soil0, som.d1[,-which.som.thickness])
    
    if(length(soil0.soil1$Thickness) != length(analysis.d1$Thickness)){
      message("Thickness of soil.water/soilcrops does not match Thickness of analysis")
      message("Adding/removing rows as necessary")
      if(nrow(analysis.d1) < nrow(soil0.soil0)){
        nrow.diff <- nrow(soil0.soil0) - nrow(analysis.d1)
        for(i in 1:nrow.diff){
          analysis.d1 <- rbind(analysis.d1, analysis.d1[nrow(analysis.d1), , drop = FALSE])  
        }
      }else{
        analysis.d1 <- analysis.d1[1:length(soil0.soil0$Thickness),]
      }
    }
      
    which.analysis.thickness <- which(names(analysis.d1) == "Thickness")
    soil0.soil <- cbind(soil0.soil1, analysis.d1[,-which.analysis.thickness])
    
    ## Add 'Depth' to soil0.soil
    Depth <- .t2d(soil0.soil$Thickness)
    soil0.soil <- data.frame(Depth = Depth, soil0.soil)
    soil0.soil$Carbon <- soil0.soil$OC
    
    soil0 <- list(soil = soil0.soil, soilwat = soilwat, soilorganicmatter = som.d2, metadata = metadata)
    class(soil0) <- "soil_profile"
    ans.soils[[i]] <- soil0
  }
  
  names(ans.soils) <- soils.nms
  
  return(ans.soils)
}


write_apsim_soils <- function(x, wrt.dir = "."){
  
  if(!is.list(x)){
    stop("object 'x' should be a list", call. = FALSE)
  }
  
  apsimx.version <- packageVersion("apsimx")
  ## Write the header for the soil
  soil.xml.doc <- xml2::read_xml(paste0("<folder version=\"", apsimx.version,"\" creator=\"apsimx r-package\" name=\"Soils\"> </folder>"))
  
  for(i in 1:length(x)){
    if(is.null(names(x)[i])){
      soil.name <- x[[i]]$metadata$SoilType
    }else{
      soil.name <- names(x)[i]
    }
    soil.name.header <- xml2::read_xml(paste0("<Soil name = \"", soil.name, "\"> </Soil>"))
    xml2::xml_add_child(soil.xml.doc, soil.name.header)
    
    ## Add metadata
    for(j in seq_along(x[[i]]$metadata)){
      mp <- names(x[[i]]$metadata[j]) ## metadata property
      mv <- x[[i]]$metadata[[j]] ## metadata value
      metd <- xml2::read_xml(paste0("<", mp, ">", mv, "</", mp, ">"))
      curr.soil <- xml2::xml_child(soil.xml.doc, search = i)
      xml2::xml_add_child(curr.soil, metd)  
    }
    
    double.vars <- c("Thickness", "BD", "AirDry", "LL15", "DUL", "SAT")
    metadata.vars <- c("BDMetadata", "AirDryMetadata", "LL15Metadata", "DULMetadata", "SATMetadata")
    ## Add soil water
    water.node <- xml2::read_xml("<Water></Water>")
    xml2::xml_add_child(curr.soil, water.node)
    for(j in seq_along(names(x[[i]]$soil))){
      scn <- names(x[[i]]$soil)[j] ## soil column name
      if(scn == "Depth") next
      scv <- x[[i]]$soil[[scn]] ## soil column value
      soil.water.node <- xml2::read_xml(paste0("<", scn, "> </", scn, ">"))
      if(scn %in% double.vars){
        for(k in seq_along(scv)){
          dbl.vr <- xml2::read_xml(paste0("<double>", scv[k], "</double>")) ## double variable
          xml2::xml_add_child(soil.water.node, dbl.vr)
        }
      }
      ## If metadata, I don't know what to do
      if(scn %in% metadata.vars){
        for(k in seq_along(scv)){
          mtd.vr <- xml2::read_xml(paste0("<string>", scv[k], "</string>")) ## string variable
          xml2::xml_add_child(soil.water.node, mtd.vr)
        }
      }
      
      ## Add soil crop
      curr.water <- xml2::xml_child(curr.soil, ".//Water")
      xml2::xml_add_child(curr.water, soil.water.node)
    }
    ## Add soil carbon
    
    
  }
  ## Write to file
  ## This is ugly at the moment, but do not know how to fix it
  xml2::write_xml(curr.soil, file = "current_soil_whitespace.xml",
                  options = "format_whitespace")
}

## Testing the 'edit_apsim_xml' function
require(apsimx)

## I guess I can run this non-locally
extd.dir <- system.file("extdata", package = "apsimx")

tmp.dir <- tempdir()
 
values <- paste(rep(1.7, 12), collapse = " ")
 
edit_apsim_xml("Maize75.xml", 
                src.dir = extd.dir,
                wrt.dir = tmp.dir, 
                parm.path = ".//Model/rue",
                value = values)

pp1 <- edit_apsim_xml("Maize75.xml", 
                      src.dir = extd.dir,
                      wrt.dir = tmp.dir, 
                      parm.path = ".//Model/rue",
                      value = values,
                      verbose = FALSE)

pp2 <- edit_apsim_xml("Maize75.xml", 
                      src.dir = extd.dir,
                      wrt.dir = tmp.dir, 
                      parm.path = ".//rue",
                      value = values,
                      verbose = FALSE)

## Testing read_apsim_met
require(apsimx)
require(nasapower)
apsimx_options(warn.versions = FALSE)

## setwd(tempdir())

run.test.apsim.met <- get(".run.local.tests", envir = apsimx.options)

if(run.test.apsim.met){
  ex.dir <- auto_detect_apsimx_examples()
  extd.dir <- system.file("extdata", package = "apsimx")

  lmf <- list.files(paste0(ex.dir,"/WeatherFiles"))

  lmf2 <- lmf[lmf %in% c("Dalby.met","Goond.met","Ingham.met","lincoln.met","Popondetta.met",
                         "VCS_Ruakura.met","WaggaWagga.met")]
}

if(run.test.apsim.met){
  for(i in lmf2){
    tmp <- read_apsim_met(file = i, src.dir = paste0(ex.dir,"/WeatherFiles"), verbose = FALSE)
    cat("Read:",i,"\n")
  }
}

## Test multiple imputation function

run.test.impute.apsim.met <- get(".run.local.tests", envir = apsimx.options)

## One trick to only test this locally?
username <- Sys.info()[["user"]]

## I need to change this test so that it downloads data instead of distributing 
## nasapower met data with the package
if(run.test.impute.apsim.met && username == "fernandomiguez"){
  
  ## This will not write a file to disk
  pwr <- get_power_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
  ## Note that missing data is coded as -99
  summary(pwr)
  ## Check for reasonable ranges 
  check_apsim_met(pwr)
  ## Impute using linear interpolation
  missing.input <- NA
  attr(missing.input, "units") <- "MJ/day" 
  pwr[1, "radn"] <- missing.input
  pwr[nrow(pwr), "radn"] <- missing.input
  check_apsim_met(pwr)
  pwr.imptd <- impute_apsim_met(pwr, verbose = TRUE)
  summary(pwr.imptd)
  check_apsim_met(pwr.imptd)
}

#'
#' @title Edit APSIM-X file with a replaced soil profile
#' @name edit_apsimx_replace_soil_profile
#' @description Edits an APSIM-X simulation by replacing the soil profile
#' @param file name of the .apsimx file to be edited
#' @param src.dir source directory
#' @param wrt.dir writing directory
#' @param soil.profile a soil profile object with class \sQuote{soil_profile}
#' @param edit.tag default edit tag \sQuote{-edited}
#' @param overwrite default FALSE
#' @param verbose default TRUE and it will print messages to console
#' @param root supply the node position in the case of multiple simulations such as factorials.
#' @return writes a file to disk with the supplied soil profile
#' @details This function is designed to batch replace the whole soil in an APSIM simulation file. 
#' @note There is no such thing as a default soil, carefully build the profile for each simulation.
#' @export
#' @examples 
#' \donttest{
#' sp <- apsimx_soil_profile()
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' ## I write to a temp directory but replace as needed
#' tmp.dir <- tempdir()
#' 
#' edit_apsimx_replace_soil_profile("Maize.apsimx", soil.profile = sp, 
#'                                   src.dir = extd.dir, wrt.dir = tmp.dir)
#' inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir,
#'                  node = "Soil")
#'  }
#'

edit_apsimx_replace_soil_profile <-  function(file = "", src.dir = ".",
                                              wrt.dir = NULL,
                                              soil.profile = NULL,
                                              edit.tag = "-edited",
                                              overwrite = FALSE,
                                              verbose = TRUE,
                                              root = NULL){
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(src.dir)
  } 
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
  
  if(length(file.names)==0){
    stop("There are no .apsimx files in the specified directory to edit.")
  }
  
  file <- match.arg(file, file.names)
  
  if(missing(soil.profile)) stop("soil profile is missing")
  
  if(!inherits(soil.profile, "soil_profile"))
    stop("Object soil.profile should be of class 'soil_profile'", call. = FALSE)
  
  ## Parse apsimx file (JSON)
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
  
  ## Where is the 'Core' simulation?
  parent.node <- apsimx_json$Children
  wcore <- grep("Models.Core.Simulation", parent.node)
  
  if(length(wcore) > 1 || !is.null(root)){
    if(missing(root)){
      cat("Simulation structure: \n")
      str_list(apsimx_json)
      stop("more than one simulation found and no root node label has been specified \n select one of the children names above")   
    }else{
      ### Parse root
      root <- parse_root(root)
      if(length(root) > 3)
        stop("At the moment 3 is the maximum length for root", call. = TRUE)
      
      if(length(root) == 1){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root[1]), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children
      }
      if(length(root) == 2){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root[1]), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label in position 1 found or root is not unique")
        root.node.0 <- apsimx_json$Children[[wcore1]]
        root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)  
        wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
        if(length(wcore2) == 0 || length(wcore2) > 1)
          stop("no root node label in position 2 found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children        
      }
      if(length(root) == 3){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root[1]), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label in position 1 found or root is not unique")
        root.node.0 <- apsimx_json$Children[[wcore1]]
        root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
        wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
        if(length(wcore2) == 0 || length(wcore2) > 1)
          stop("no root node label in position 2 found or root is not unique")
        root.node.1 <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]
        root.node.1.child.names <- sapply(root.node.1$Children, function(x) x$Name)  
        wcore3 <- grep(as.character(root[3]), root.node.1.child.names)
        if(length(wcore3) == 0 || length(wcore3) > 1)
          stop("no root node label in position 3 found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children        
      }
    }
  }else{
    parent.node <- apsimx_json$Children[[wcore]]$Children  
  }

  ## Extract 'Core' simulation
  wcz <- grepl("Models.Core.Zone", parent.node)
  core.zone.node <- parent.node[wcz][[1]]$Children
  
  ## Extract soil
  wsn <- grepl("Models.Soils.Soil", core.zone.node)
  soil.node <- core.zone.node[wsn]
  
  soil.node0 <- soil.node[[1]]$Children ## Is the 1 safe here because there is only one soil?
  
  ## First edit: soil 'Physical'
  ## Depth, Thickness, ParticleSizeClay,
  ## BD, AirDry, LL15, DUL, SAT, KS
  wspn <- grepl("Models.Soils.Physical", soil.node0)
  soil.physical.node <- soil.node0[wspn][[1]]
  for(i in c("Depth", "Thickness", "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand", "BD", "AirDry", "LL15", "DUL", "SAT", "KS")){
    ## Format the variable
    if(is.null(soil.profile$soil[[i]])) next
    tmp <- as.vector(soil.profile$soil[[i]], mode = "list")
    ## Replace the variable
    soil.physical.node[[i]] <- tmp 
  }
  ## Preliminary setup of soil
  if(length(soil.profile$crops) > length(soil.physical.node$Children)){
    for(i in seq_along(soil.profile$crops)){
      soil.physical.node$Children[[i]] <- soil.physical.node$Children[[1]]
      soil.physical.node$Children[[i]]$Name <- paste0(soil.profile$crops[i], "Soil")
    }
  }
  ## Crop parameters
  for(i in 1:length(soil.profile$crops)){
    crop.vars.names <- grep("KL|XF|LL$", names(soil.profile$soil), value = TRUE)
    for(j in crop.vars.names){
      tmp <- as.vector(soil.profile$soil[[j]], mode = "list")
      strpcrop <- strsplit(j, ".", fixed = TRUE)[[1]][2]
      soil.physical.node$Children[[i]][[strpcrop]] <- tmp
    }
  }
  soil.node0[wspn][[1]] <- soil.physical.node
  soil.node[[1]]$Children <- soil.node0
  
  ## Next edit the soil organic component
  wsomn <- grepl("Models.Soils.Organic", soil.node0)
  soil.om.node <- soil.node0[wsomn][[1]]
  
  for(i in c("Depth", "Thickness", "Carbon", "SoilCNRatio", "FBiom", "FInert", "FOM")){
    ## Format the variable
    tmp <- as.vector(soil.profile$soil[[i]], mode = "list")
    ## Replace the variable
    soil.om.node[[i]] <- tmp 
  }
  soil.node0[wsomn][[1]] <- soil.om.node
  soil.node[[1]]$Children <- soil.node0
  
  ## Next edit the Chemical component
  wschn <- grepl("Models.Soils.Chemical", soil.node0)
  soil.chemical.node <- soil.node0[wschn][[1]]
  
  for(i in c("Depth", "Thickness", "PH")){
    ## Format the variable
    tmp <- as.vector(soil.profile$soil[[i]], mode = "list")
    ## Replace the variable
    soil.chemical.node[[i]] <- tmp 
  }
  soil.node0[wschn][[1]] <- soil.chemical.node
  soil.node[[1]]$Children <- soil.node0
  
  ## Next edit the Solute component, this is not present in older versions of APSIM
  wssoln <- grepl("Models.Soils.Solute", soil.node0)
  if(sum(wssoln) > 0){
    soil.solute.nodes <- soil.node0[wssoln]
    for(i in seq_along(soil.solute.nodes)){
      soil.solute.node <- soil.solute.nodes[[i]]
      vname <- soil.solute.node$Name
      vname.in.soil <- grep(vname, names(soil.profile$soil), value = TRUE)
      if(length(vname.in.soil) > 0){
        tmp <- as.vector(soil.profile$soil[[vname.in.soil]], mode = "list") ## This now works because these are initial values?
        soil.solute.node$InitialValues <- tmp  
        soil.solute.node$Thickness <- soil.profile$soil$Thickness
        soil.node0[wssoln][[i]] <- soil.solute.node
      }
    }
    soil.node[[1]]$Children <- soil.node0    
  }

  ## Edit metadata
  if(!is.null(soil.profile$metadata)){
    soil.node.names <- names(soil.node[[1]])
    skp.nms <- c("$type", "Name", "Children", "IncludeInDocumentation", "Enabled", "ReadOnly")
    for(i in soil.node.names){
      if(i %in% skp.nms) next
      if(i %in% names(soil.profile$metadata)){
        soil.node[[1]][[i]] <- soil.profile$metadata[[i]]
      }else{
        if(i %in% c("RecordNumber","ApsoilNumber")){
          soil.node[[1]][[i]] <- 0 
        }else{
          soil.node[[1]][[i]] <- NA  
        }
      }
    }
  }
  
  ## Use metadata to edit the name of the soil
  if(!is.null(soil.profile$metadata$SoilType)){
    soil.node[[1]]$Name <- soil.profile$metadata$SoilType
  }
  
  ## Edit soilWat if present
  if(inherits(soil.profile$soilwat, "soilwat_parms")){
    
    wsswn <- grepl("Models.WaterModel", soil.node0) 
    soil.soilwat.node <- soil.node0[wsswn][[1]]
    for(i in names(soil.profile$soilwat)){
      if(any(is.na(soil.profile$soilwat[[i]]))){
        next
      }else{
          soil.soilwat.node[[i]] <- soil.profile$soilwat[[i]]  
      } 
    }
    soil.node0[wsswn][[1]] <- soil.soilwat.node
    soil.node[[1]]$Children <- soil.node0
  }
  
  ## Edit SWIM if present
  if(inherits(soil.profile$swim, "swim_parms")){
    
    wsswmn <- grepl("Models.Soils.Swim3", soil.node0) 
    ### This is an error instead of a warning because
    ### if 'swim' is provided, then it is likely that 
    ### editing is needed
    if(length(wsswmn) == 0)
      stop("Could not find Swim3 model in simulation", call. = FALSE)
    ## Which soil swim node?
    soil.swim3.node <- soil.node0[wsswmn][[1]]
    for(i in names(soil.profile$swim)){
      if(any(is.na(soil.profile$swim[[i]]))){
        next
      }else{
        soil.swim3.node[[i]] <- soil.profile$swim[[i]]  
      } 
    }
    soil.node0[wsswmn][[1]] <- soil.swim3.node
    soil.node[[1]]$Children <- soil.node0
  }
  
  ## Edit InitialWater if present
  if(inherits(soil.profile$initialwater, "initialwater_parms")){
    
    wsiswn <- grepl("Models.Soils.Water", soil.node0) 
    soil.initialwater.node <- soil.node0[wsiswn][[1]]
    for(i in names(soil.profile$initialwater)){
      if(any(is.na(soil.profile$initialwater[[i]]))){
        next
      }else{
        soil.initialwater.node[[i]] <- soil.profile$initialwater[[i]]  
      } 
    }
    soil.node0[wsiswn][[1]] <- soil.initialwater.node
    soil.node[[1]]$Children <- soil.node0
  }
  
  ## Edit Solutes if present
  if(inherits(soil.profile$solutes, "solutes_parms")){
    
    wssoln <- grepl("Models.Soils.Solute", soil.node0) 
    soil.solute.nodes <- soil.node0[wssoln]
    for(i in seq_along(soil.solute.nodes)){
      soil.solute.node <- soil.solute.nodes[[i]]
      vname <- soil.solute.node$Name
      vname.in.solutes <- grep(vname, soil.profile$solutes$Solutes, value = TRUE)
      if(length(vname.in.solutes) > 0){
        for(j in seq_along(soil.profile$solutes)){
          if(any(is.na(soil.profile$solutes[[j]]))){
            next
          }else{
              sspntc <- names(soil.profile$solutes)[[j]] ## Soil Solute Parameter Name to Change
              ## Is the parameter above present in soil.solute.node?
              soil.solute.node.names <- names(soil.solute.node)
              if(!sspntc %in% soil.solute.node.names)
                stop("soil solute parameter to change is not in soil.solute.node")
              soil.solute.node[sspntc] <- soil.profile$solutes[[j]]    
          } 
        }        
      }
      soil.solute.nodes[[i]] <- soil.solute.node
    }
    soil.node0[wssoln] <- soil.solute.nodes
    soil.node[[1]]$Children <- soil.node0
  }
  
  if(is.null(root)){
    ## Replace the soil
    ## 1. soil.node to core.zone.node
    core.zone.node[wsn] <- soil.node
    ## 2. core.one.node to parent.node
    parent.node[wcz][[1]]$Children <- core.zone.node
    ## parent.node to core
    apsimx_json$Children[[wcore]]$Children <- parent.node    
  }else{
    core.zone.node[wsn] <- soil.node
    parent.node[wcz][[1]]$Children <- core.zone.node
    if(length(root) == 1){
      apsimx_json$Children[[wcore1]]$Children <- parent.node 
    }
    if(length(root) == 2){
      apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children <- parent.node 
    }
    if(length(root) == 3){
      apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children <- parent.node 
    }
  }

  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir, "/",
                      tools::file_path_sans_ext(file),
                      edit.tag,".apsimx")
  }else{
    wr.path <- paste0(wrt.dir, "/", file)
  }
  
  jsonlite::write_json(apsimx_json, path = wr.path, 
                       pretty = TRUE, digits = NA, 
                       auto_unbox = TRUE, null = "null",
                       na = "null")
  
  if(verbose){
    cat("Created: ", wr.path, "\n")
  }
}

#' Run an APSIM-X Simulation
#' 
#' A valid apsimx file can be run from within R. The main goal is to make running APSIM-X
#' simple, especially for large scale simulations or parameter optimization
#' 
#' @title Run an APSIM-X simulation
#' @name apsimx
#' @description Run apsimx from R. It uses \code{\link{system}} (unix) or \code{\link{shell}} (windows) and it attempts to be platform independent.
#' @param file file name to be run (the extension .apsimx is optional)
#' @param src.dir directory containing the .apsimx file to be run (defaults to the current directory)
#' @param silent whether to print messages from apsim simulation. This is passed to argument \sQuote{ignore.stdout} for
#' function \code{\link{system}}.
#' @param intern This is passed to argument \sQuote{intern} for function \code{\link{system}}.
#' @param value how much output to return: \cr
#'              option \sQuote{report} returns only the \sQuote{main} report component; \cr
#'              option \sQuote{all} returns all components of the simulation; \cr
#'              option \sQuote{none} does not create a data.frame but it generates the databases; \cr
#'              option \sQuote{user-defined} should be the name of a specific table
#' @param cleanup logical. Whether to delete the .db file generated by APSIM-X. Default is FALSE
#' @param simplify whether to return a single data frame when multiple reports are present. If FALSE it will return a list.
#' @param xargs extra arguments to be passed to the APSIM-X run. Use function xargs_apsimx.
#' @return a data frame with the \sQuote{Report} from the APSIM-X simulation. The return value depends on the argument \sQuote{value} above.
#' @export
#' @examples 
#' \donttest{
#' ## See function 'apsimx_example' and vignette 'apsimx'
#' }
#'

apsimx <- function(file = "", src.dir = ".",
                   silent = FALSE, 
                   intern = FALSE,
                   value = "report",
                   cleanup = FALSE,
                   simplify = TRUE,
                   xargs){
  
  if(file == "") stop("need to specify file name")
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
     .check_apsim_name(file)
     .check_apsim_name(normalizePath(src.dir))
  }
  
  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to run.")
  }
  
  if(get("system.intern", envir = apsimx::apsimx.options)){
    intern <- TRUE ## This global option overrides the function argument
  }
  
  file <- match.arg(file, file.names)
  
  file.name.path <- file.path(src.dir, file)
  
  ada <- auto_detect_apsimx()
  
  ## Grabs the global variables
  ## Is there a difference between:
  ## 1) apsimx::apsimx.options$dotnet
  ## 2) get("dotnet", apsimx::apsimx.options)
  ## 3) get("dotnet", apsimx.options)
  ## ??
  dotnet.flag <- get("dotnet", envir = apsimx::apsimx.options)
  mono.flag <- get("mono", envir = apsimx::apsimx.options)
  
  if(!missing(xargs)){
    dotnet.flag <- xargs$dotnet
    mono.flag <- xargs$mono
    if(!is.na(xargs$exe.path))
      ada <- xargs$exe.path
  } 

  if(.Platform$OS.type == "unix"){
    if(dotnet.flag){
      run.strng <- paste("dotnet", ada, file.name.path)
    }
    if(mono.flag){
      mono <- system("which mono", intern = TRUE)
      run.strng <- paste(mono, ada, file.name.path) 
    }
    if(isFALSE(dotnet.flag) && isFALSE(mono.flag)){
      run.strng <- paste(ada, file.name.path) 
    }
    ## Run APSIM-X on the command line
    if(!missing(xargs)) run.strng <- paste(run.strng, xargs$xargs.string)
    
    if(get("allow.path.spaces", envir = apsimx::apsimx.options)){
      ## As written here, this will not work with dotnet or mono
      ## Just so I do not forget. The problem with spaces can be
      ## overcome for file paths but I do not think it can be 
      ## solved for the program command name
      run.strng <- paste0(ada, 
                          " ", 
                          shQuote(normalizePath(file.name.path)))
    }

    res <- system(command = run.strng, ignore.stdout = silent, intern = intern)
  }
  
  if(.Platform$OS.type == "windows"){
    if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
      run.strng <- paste0(ada, " ", file.name.path)
    }else{
      run.strng <- paste0(ada, 
                          " ", 
                          shQuote(normalizePath(file.name.path)))
    }
    if(!missing(xargs)) run.strng <- paste(run.strng, xargs$xargs.string)
    shell(cmd = run.strng, translate = TRUE, intern = intern)
  }

  if(value != "none"){
    ans <- read_apsimx(file = sub("apsimx$", "db", file), 
                       src.dir = src.dir, value = value, simplify = simplify)
  }else{
    if(value == "none" && !silent){
      cat("APSIM created .db files, but nothing is returned \n")
    }
  }
  
  if(cleanup){
    ## Default is not to cleanup
    if(value == "none") stop("do not clean up if you choose value = 'none' ")
    ## Delete the apsim-generated sql database 
    file.remove(paste0(src.dir, "/", sub("apsimx", "db", file)))
    ## Other files created by APSIM
    fn1 <- paste0(src.dir, "/", sub("apsimx", "db-wal", file))
    fn2 <- paste0(src.dir, "/", sub("apsimx", "db-shm", file))
    if(file.exists(fn1)) file.remove(fn1)
    if(file.exists(fn2)) file.remove(fn2)
  }
  
  if(value != "none")
    return(ans)
}

#' Extra arguments for running APSIM-X
#' @title Provide extra arguments for APSIM-X
#' @name xargs_apsimx
#' @description This provides additinoal command line arguments when running the model
#' @param verbose Write detailed messages to stdout when a simulation starts/finishes.
#' @param csv Export all reports to .csv files.
#' @param merge.db.files Merge multiple .db files into a single .db file.
#' @param list.simulations List simulation names without running them.
#' @param list.referenced.filenames List all files that are referenced by an .apsimx file(s).
#' @param single.threaded  Run all simulations sequentially on a single thread.
#' @param cpu.count (Default: -1) Maximum number of threads/processes to spawn for running simulations.
#' @param simulation.names Only run simulations if their names match this regular expression.
#' @param dotnet Logical. There is a global option for this argument, but this will override it. This 
#' can be useful if the goal is to compare an old version of Next Gen (before Sept 2021) with a more
#' recent version in the same script. This might be needed if you have your own compiled version of APSIM Next Gen.
#' @param mono Logical. Should be set to TRUE if running a version of APSIM Next Gen from Aug 2021 or older on Mac or Linux.
#' @param exe.path executable path. This can be useful for having both a global option through \sQuote{apsimx.options} and
#' a local option that will override that. This option will take precedence.
#' @return it returns a character vector with the extra arguments.
#' @export

xargs_apsimx <- function(verbose = FALSE, csv = FALSE, merge.db.files = FALSE, list.simulations = FALSE,
                         list.referenced.filenames = FALSE, single.threaded = FALSE, cpu.count = -1L,
                         simulation.names = FALSE, dotnet = FALSE, mono = FALSE,
                         exe.path = NA){

  if(dotnet && mono)
    stop("either dotnet or mono should be TRUE, but not both", call. = TRUE)
  
  verbose <- ifelse(verbose, " --verbose", "")
  csv <- ifelse(csv, " --csv", "")
  merge.db.files <- ifelse(merge.db.files, " --merge-db-files", "")
  list.simulations <- ifelse(list.simulations, " --list-simulations", "")
  list.referenced.filenames <- ifelse(list.referenced.filenames, " --list-referenced-filenames", "")
  single.threaded <- ifelse(single.threaded, " --single-threaded", "")
  if(cpu.count == 0L) stop("cpu.count cannot be zero")
  if(!is.integer(cpu.count)) stop("cpu.count should be an integer. For example, cpu.count=2L for two cores")
  cpu.count <- ifelse(cpu.count > 0, paste0(" --cpu-count=", cpu.count), "")
  simulation.names <- ifelse(simulation.names, " --simulation-names", "")
  
  ## I'm guessing single threaded is not compatible with cpu.count
 
  xargs.string <- paste0(verbose, csv, merge.db.files, list.simulations, list.referenced.filenames, single.threaded, cpu.count, simulation.names)  
  ## Remove beggining or trailing spaces
  xargs.string <- gsub("^\\s+|\\s+$", "", xargs.string)
  xargs.string
  ans <- list(xargs.string = xargs.string, dotnet = dotnet, mono = mono, exe.path = exe.path)
}

## This is an internal function so I won't export/document it
#' @noRd
auto_detect_apsimx <- function(){

  if(.Platform$OS.type == "unix"){

    if(grepl("Darwin", Sys.info()[["sysname"]])){
      laf <- list.files("/Applications/")
      find.apsim <- grep("APSIM", laf)
      ## This deals with the fact that APSIM-X might not be present but perhaps a
      ## custom version is available
      if(length(find.apsim) == 0){
        ## I only throw a warning because maybe the user has a custom version of APSIM-X only
        if(!is.na(get("exe.path", envir = apsimx::apsimx.options)) && get("warn.find.apsimx", envir = apsimx::apsimx.options)){
          warning("APSIM-X not found, but a custom one is present")
        }else{
          if(is.na(get("exe.path", envir = apsimx::apsimx.options)))
            stop("APSIM-X not found and no 'exe.path' exists.")
        }
      }
      ## If only one version of APSIM-X is present
      ## APSIM executable
      st1 <- "/Applications/"
      st3 <- "/Contents/Resources/bin/Models.exe"   
      if(get("dotnet", envir = apsimx::apsimx.options))  st3 <- gsub("exe$", "dll", st3) 
      if(isFALSE(get("mono", envir = apsimx::apsimx.options)) && isFALSE(get("dotnet", envir = apsimx::apsimx.options))) 
        st3 <- "/Contents/Resources/bin/Models"

      if(length(find.apsim) == 1){
        apsimx.name <- laf[find.apsim]
        apsimx_dir <- paste0(st1, apsimx.name, st3)
        ## I could use 'file.path' instead, but it this is not a 'file'
        ## so it could be confusing
      }
      ## If there is more than one version of APSIM-X
      if(length(find.apsim) > 1){
        ## Originally I was sorting by #issue number but this
        ## does not give you the latest version
        len.fa <- length(find.apsim)
        ## This extracts the date from the APSIM name but I 
        ## only need this for debugging in case there is a problem
        fa.dt <- as.numeric(sapply(laf[find.apsim], .favn))
        newest.version.number <- max(fa.dt)
        newest.version <- grep(newest.version.number, laf[find.apsim], value = TRUE)
        if(get("warn.versions", envir = apsimx::apsimx.options) &&
           is.na(get("exe.path", envir = apsimx::apsimx.options))){
          options.warn.versions <- getOption("apsimx.warn.versions")
          if(is.null(options.warn.versions)){
            warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
          }else{
            if(isTRUE(options.warn.versions)){
              warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
            }
          }
        }
        apsimx.name <- newest.version
        apsimx_dir <- paste0(st1, apsimx.name, st3)
      }
    }
  
    if(grepl("Linux", Sys.info()[["sysname"]])){
      
      apsimx.versions <- NULL
      find.apsim <- grep("apsim", list.files("/usr/local/lib"))
      ## What if length equals zero?
      if(length(find.apsim) == 0){
        ## I only throw a warning because maybe the user has a custom version of APSIM-X only
        if(!is.na(get("exe.path", envir = apsimx::apsimx.options)) && get("warn.find.apsimx", envir = apsimx::apsimx.options)){
          warning("APSIM-X not found, but a custom one is present")
        }else{
          if(is.na(get("exe.path", envir = apsimx::apsimx.options)))
            stop("APSIM-X not found and no 'exe.path' exists.")  
        }
      }
      ## APSIM executable
      st1 <- "/usr/local/lib/apsim/"
      st3 <- "/bin/Models.exe" 
      if(get("dotnet", envir = apsimx::apsimx.options)) st3 <- gsub("exe$", "dll", st3) 
      if(isFALSE(get("mono", envir = apsimx::apsimx.options)) && isFALSE(get("dotnet", envir = apsimx::apsimx.options))) 
        st3 <- "/bin/Models"
      
      if(length(find.apsim) == 1){
        apsimx.versions <- list.files("/usr/local/lib/apsim")
        apsimx.name <- apsimx.versions
        apsimx_dir <- paste0(st1, apsimx.name, st3)
      }
      
      ## Note: Apparently Debian does not tolerate multiple 
      ## versions of APSIM-X installed, date 2019-12-12
      if(length(apsimx.versions) > 1){
        len.fa <- length(find.apsim)
        newest.version <- apsimx.versions[find.apsim]
        if(get("warn.versions", envir = apsimx::apsimx.options)){
          options.warn.versions <- getOption("apsimx.warn.versions")
          if(is.null(options.warn.versions)){
            warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
          }else{
            if(isTRUE(options.warn.versions)){
              warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
            }
          }
        }
        apsimx.name <- newest.version
        apsimx_dir <- paste0(st1, apsimx.name, st3)
      }
    }
  }
  
  if(.Platform$OS.type == "windows"){
    st1 <- "C:/PROGRA~1/"
    laf <- list.files(st1)
    find.apsim <- grep("APSIM", laf)
    
    ## What if length equals zero?
    if(length(find.apsim) == 0 && is.na(get("exe.path", envir = apsimx::apsimx.options))){
        ## Try using the registry only if APSIM path has not been set manually
        if(get("warn.find.apsimx", envir = apsimx::apsimx.options)) warning("Searching the Windows registry for APSIM-X")
        ## HCR hive is for HKEY_CLASSES_ROOT, HLM is for HKEY_LOCAL_MACHINE and HCU is for HKEY_CURRENT_USER
        regcmd <- try(utils::readRegistry("APSIMXFile\\shell\\open\\command", "HCR")[[1]], silent = TRUE)
        if(inherits(regcmd, "try-error")) regcmd <- try(utils::readRegistry("APSIMXFile\\shell\\open\\command", "HCU")[[1]], silent = TRUE)
        if(inherits(regcmd, "try-error")) regcmd <- try(utils::readRegistry("APSIMXFile\\shell\\open\\command", "HLM")[[1]], silent = TRUE)
        if(inherits(regcmd, "try-error")) stop("Could not find APSIM-X in the Windows Registry")
        regcmd2 <- gsub("\\\\", "/", strsplit(regcmd, "\"")[[1]][2])
        apsimx_dir <- gsub("ApsimNG", "Models", regcmd2)
        if(length(apsimx_dir) == 0) stop("APSIM-X not found and no 'exe.path' exists.")
        if(grepl("\\s", apsimx_dir)) stop("Found a space in the path. Please provide the path manually to APSIM-X using exe.path in apsimx_options.")
        return(apsimx_dir)        
    }
    
    st3 <- "/bin/Models.exe" 
    if(get("dotnet", envir = apsimx::apsimx.options)) st3 <- gsub("exe$", "dll", st3) 
    
    if(length(find.apsim) == 1){
      apsimx.versions <- laf[find.apsim]
      apsimx.name <- apsimx.versions
      apsimx_dir <- paste0(st1, apsimx.name, st3)
    }
    
    if(length(find.apsim) > 1){
      apsimx.versions <- laf[find.apsim]
      newest.version <- apsimx.versions[length(find.apsim)]
      if(get("warn.versions", envir = apsimx::apsimx.options)){
        options.warn.versions <- getOption("apsimx.warn.versions")
        if(is.null(options.warn.versions)){
          warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
        }else{
          if(isTRUE(options.warn.versions)){
            warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))            
          }
        }
      }
      apsimx.name <- newest.version
      apsimx_dir <- paste0(st1, apsimx.name, st3)
    }
  }
  
  if(!is.na(get("exe.path", envir = apsimx::apsimx.options))){
    ## Windows paths might contain white spaces which are
    ## problematic when running them at the command line
    if(grepl("\\s", get("exe.path", envir = apsimx::apsimx.options)))
      stop("White spaces are not allowed in APSIM-X executable path")
    apsimx_dir <- get("exe.path", envir = apsimx::apsimx.options)
  }
  return(apsimx_dir)
}

#' Auto detect where apsimx examples are located 
#' 
#' @title Auto detect where apsimx examples are located
#' @name auto_detect_apsimx_examples
#' @description simple function to detect where APSIM-X examples are located
#' @return will create a directory (character string) pointing to APSIM-X distributed examples
#' @export
#' @examples 
#' \dontrun{
#' ex.dir <- auto_detect_apsimx_examples()
#' }
#' 

auto_detect_apsimx_examples <- function(){
  
  if(.Platform$OS.type == "unix"){
    apsim.name <- NULL # Why did I create this NULL variable?
    if(grepl("Darwin", Sys.info()[["sysname"]])){
      ## If APSIM-X is installed it will be in /Applications/
      ## look into Applications folder
      laf <- list.files("/Applications/")
      find.apsim <- grep("APSIM",laf)
      
      if(length(find.apsim) == 0) stop("APSIM-X examples not found")
      
      if(length(find.apsim) > 1){
        fa.dt <- as.numeric(sapply(laf[find.apsim], .favn))
        newest.version.number <- max(fa.dt)
        newest.version <- grep(newest.version.number, laf[find.apsim], value = TRUE)
        if(get("warn.versions", envir = apsimx::apsimx.options) && is.na(get("exe.path", envir = apsimx::apsimx.options))){
          warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))
        }
        apsimx.name <- newest.version
      }else{
        apsimx.name <- laf[find.apsim]
      }
      st1 <- "/Applications/"
      st3 <- "/Contents/Resources/Examples" 
      apsimx_ex_dir <- paste0(st1, apsimx.name, st3)
    }
  
    if(grepl("Linux", Sys.info()[["sysname"]])){
      st1 <- "/usr/local/lib/apsim/"
      apsimx.versions <- list.files(st1) 
      ## In Ubuntu it looks like you can only have one version 
      ## of APSIM-X installed
      if(length(apsimx.versions) > 1){
        len.fa <- length(find.apsim)
        newest.version <- apsimx.versions[find.apsim]
        if(get("warn.versions", envir = apsimx::apsimx.options) && is.na(get("exe.path", envir = apsimx::apsimx.options))){
          warning(paste("Multiple versions of APSIM-X installed. \n
                    Choosing the newest one:", newest.version))
        }
        apsimx.name <- newest.version
      }else{
        apsimx.name <- apsimx.versions
      }
      apsimx_ex_dir <- paste0(st1, apsimx.name, "/Examples")
    }
  }
  
  if(.Platform$OS.type == "windows"){
    ## Need to test this change
      adax <- auto_detect_apsimx()
      ## APSIM path to examples
      # st3 <- "/Examples" 
      # apsimx_ex_dir <- paste0(st1, "/", apsimx.name, st3)
      apsimx_ex_dir <- gsub("bin/Models.*", "Examples", adax)
  }
  
  if(!is.na(get("examples.path", envir = apsimx::apsimx.options))){
    ## I dislike white spaces in paths!
    ## I'm looking at you Windows!
    if(grepl("\\s", get("examples.path", envir = apsimx::apsimx.options)))
      stop("White spaces are not allowed in examples.path")
    apsimx_ex_dir <- get("examples.path", envir = apsimx::apsimx.options)
  }
  return(apsimx_ex_dir)
}

#'
#' @title Access Example APSIM-X Simulations
#' @name apsimx_example
#' @description simple function to run some of the built-in APSIM-X examples
#' @param example run an example from built-in APSIM-X. Options are all of the ones included with the APSIM-X distribution, except \sQuote{Graph}.
#' @param silent whether to print standard output from the APSIM-X execution
#' @note This function creates a new column \sQuote{Date} which is in the R \sQuote{Date} format which is convenient for graphics.
#' @details This function creates a temporary copy of the example file distributed with APSIM-X to avoid writing a .db file 
#'          to the directory where the \sQuote{Examples} are located. It is not a good practice and there is no guarantee that 
#'          the user has read/write permissions in that directory.
#' @return It returns a data frame
#' @export
#' @examples 
#' \dontrun{
#' wheat <- apsimx_example("Wheat")
#' maize <- apsimx_example("Maize")
#' barley <- apsimx_example("Barley")
#' ## The 'Date' column is created by this function, based on apsim output.
#' require(ggplot2)
#' ggplot(data = wheat , aes(x = Date, y = Yield)) + 
#'   geom_point()
#' }
#' 

apsimx_example <- function(example = "Wheat", silent = FALSE){

  ## Write to a temp directory only
  tmp.dir <- tempdir()
  ## Run a limited set of examples
  ## Now the only one missing is Graph, which I assume is about
  ## graphics and not that relevant to apsimx
  ## Several examples are not supported because they do not use
  ## relative paths for the weather file
  ## Examples which do not run: Chicory
  ex.ch <- c("AgPasture", "Barley", "Canola", "Chickpea", 
               "ControlledEnvironment", "Eucalyptus",
               "EucalyptusRotation",
               "Maize", "Mungbean",
               "Oats", "OilPalm", "Peanut", "Pinus", "Potato",
               "RedClover",
               "Rotation", "Slurp", "Sorghum",
               "Soybean", "Sugarcane", "Wheat", "WhiteClover")

  example <- match.arg(example, choices = ex.ch)
  
  ex.dir <- auto_detect_apsimx_examples()
  ex <- file.path(ex.dir, paste0(example, ".apsimx"))
  
  if(!file.exists(ex)) stop(paste0("cannot find example file ", example))
  ## Make a temporary copy of the file to the current directory
  ## Do not transfer permissions?
  file.copy(from = ex, to = tmp.dir, copy.mode = FALSE)
  
  sim <- apsimx(paste0(example, ".apsimx"), src.dir = tmp.dir, intern = TRUE,
                value = "report", simplify = FALSE)

  ## OS independent cleanup (risky?)
  ## Should I also try to remove .db-shm and .db-whl (or whatever that is?)
  file.remove(paste0(tmp.dir, "/", example, ".db"))
  file.remove(paste0(tmp.dir, "/", example, ".apsimx"))

  return(sim)
}

#' Read APSIM-X generated .db files
#' 
#' @title Read APSIM-X generated .db files
#' @name read_apsimx
#' @description read SQLite databases created by APSIM-X runs. One file at a time.
#' @param file file name
#' @param src.dir source directory where file is located
#' @param value either \sQuote{report}, \sQuote{all} (list) or user-defined for a specific report
#' @param simplify if TRUE will attempt to simplify multiple reports into a single data.frame. 
#' If FALSE it will return a list.
#' @note if there is one single report it will return a data.frame. 
#' If there are multiple reports, it will attempt to merge them into a data frame. 
#' If not possible it will return a list with names corresponding to the 
#' table report names. It is also possible to select a specific report from several
#' available by selecting \sQuote{value = ReportName}, where \sQuote{ReportName} is the name
#' of the specific report that should be returned.
#' If you select \sQuote{all} it will return all the components in the data base also as a list.
#' @return normally it returns a data frame, but it depends on the argument \sQuote{value} above
#' @seealso \code{\link{read_apsimx_all}}
#' @export
#' 

read_apsimx <- function(file = "", src.dir = ".", value = "report", simplify = TRUE){
  
  if(file == "") stop("need to specify file name")
  
  file.names <- dir(path = src.dir, pattern=".db$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .db files in the specified directory to read.")
  }
  
  file.name.path <- file.path(src.dir, file)
  
  con <- DBI::dbConnect(RSQLite::SQLite(), file.name.path, flags = RSQLite::SQLITE_RO)
  ## create data frame for each table
  ## Find table names first
  table.names <- RSQLite::dbListTables(con)
  other.tables <- grep("^_", table.names, value = TRUE)
  report.names <- setdiff(table.names, other.tables)
  ### Are there other potential tables starting with "_"?
  
  ## I guess I always expect to find a table, but not always... better to catch it here
  if(length(report.names) < 1)
    stop("No report tables found")    
    
  if(length(report.names) == 1L){
    tbl0 <- DBI::dbGetQuery(con, paste("SELECT * FROM ", report.names))  
    
    if(nrow(tbl0) == 0)
      warning("Report table has no data")
    
    if(any(grepl("Clock.Today", names(tbl0)))){
      if(nrow(tbl0) > 0){
        tbl0$Date <- try(as.Date(sapply(tbl0$Clock.Today, function(x) strsplit(x, " ")[[1]][1])), silent = TRUE)  
      }
    }
    
    ## Maybe the user created a 'Date' column in APSIM, so it is useful if I convert it to 'Date' class
    if(any(grepl("Date", names(tbl0), ignore.case = TRUE))){
      if(nrow(tbl0) > 0){
        try.date <- try(as.Date(tbl0$Date), silent = TRUE)
        if(!inherits(try.date, 'try-error')){
          tbl0$Date <- try.date           
        }
      }
    }
    
    ## Maybe include SimulationNames column here?
    if(all(!grepl("SimulationName", names(tbl0)))){
      tbl0$SimulationName <- NA
      stn <- grep("Simulation", other.tables, value = TRUE) ## stn stands for simulation table name
      SimulationNamesTable <- DBI::dbGetQuery(con, paste("SELECT * FROM ", stn))
      for(j in seq_along(SimulationNamesTable$ID)){
        tbl0[tbl0$SimulationID == j, "SimulationName"] <- SimulationNamesTable$Name[j]
      }
    } 
  }
    
  if(length(report.names) > 1L && value %in% c("report", "all")){

    if(simplify){
      lst0 <- NULL
      for(i in seq_along(report.names)){
        tbl0 <- DBI::dbGetQuery(con, paste("SELECT * FROM ", report.names[i]))
        
        if(nrow(tbl0) == 0)
          warning(paste("Report", report.names[i]), "has no data")
        
        if(any(grepl("Clock.Today", names(tbl0)))){
          if(nrow(tbl0) > 0){
            tbl0$Date <- try(as.Date(sapply(tbl0$Clock.Today, function(x) strsplit(x, " ")[[1]][1])), silent = TRUE)  
          }
        }
        
        ## Maybe include SimulationNames column here?
        if(all(!grepl("SimulationName", names(tbl0)))){
          tbl0$SimulationName <- NA
          stn <- grep("Simulation", other.tables, value = TRUE) ## stn stands for simulation table name
          SimulationNamesTable <- DBI::dbGetQuery(con, paste("SELECT * FROM ", stn))
          for(j in seq_along(SimulationNamesTable$ID)){
            tbl0[tbl0$SimulationID == j, "SimulationName"] <- SimulationNamesTable$Name[j]
          }
        }
        
        dat0 <- data.frame(report = report.names[i], tbl0)
        lst0 <- try(rbind(lst0, dat0), silent = TRUE)
        if(inherits(lst0, "try-error")){
          stop("Could not simplify reports into a single data.frame \n
             Choose simplify = FALSE or modify your reports.")
        }
      }
    }else{
      lst0 <- vector("list", length = length(report.names))
      for(i in seq_along(report.names)){
        tbl0 <- DBI::dbGetQuery(con, paste("SELECT * FROM ", report.names[i]))
        if(any(grepl("Clock.Today", names(tbl0)))){
          tbl0$Date <- try(as.Date(sapply(tbl0$Clock.Today, function(x) strsplit(x, " ")[[1]][1])), silent = TRUE)
        }
        
        ## Maybe include SimulationNames column here?
        if(all(!grepl("SimulationName", names(tbl0)))){
          tbl0$SimulationName <- NA
          stn <- grep("Simulation", other.tables, value = TRUE) ## stn stands for simulation table name
          SimulationNamesTable <- DBI::dbGetQuery(con, paste("SELECT * FROM ", stn))
          for(j in seq_along(SimulationNamesTable$ID)){
            tbl0[tbl0$SimulationID == j, "SimulationName"] <- SimulationNamesTable$Name[j]
          }
        }
        lst0[[i]] <- tbl0   
      }
      names(lst0) <- report.names ## Name the lists with report names
    }
  }
  
  if(!value %in% c("report", "all") && length(report.names) > 1L){
    if(!value %in% report.names){
      cat("Available table names: ", report.names ,"\n")
      stop("user defined report name is not in the list of available tables",
           call. = FALSE)
    }
    tbl0 <- DBI::dbGetQuery(con, paste("SELECT * FROM ", value))
    if(any(grepl("Clock.Today", names(tbl0)))){
      tbl0$Date <- try(as.Date(sapply(tbl0$Clock.Today, function(x) strsplit(x, " ")[[1]][1])), silent = TRUE)
    }
    
    ## Maybe include SimulationNames column here?
    if(all(!grepl("SimulationName", names(tbl0)))){
      tbl0$SimulationName <- NA
      stn <- grep("Simulation", other.tables, value = TRUE) ## stn stands for simulation table name
      SimulationNamesTable <- DBI::dbGetQuery(con, paste("SELECT * FROM ", stn))
      for(j in seq_along(SimulationNamesTable$ID)){
        tbl0[tbl0$SimulationID == j, "SimulationName"] <- SimulationNamesTable$Name[j]
      }
    }
  }
    
  if(value == "all"){
    other.tables.list <- vector("list", length = length(other.tables))
    for(i in seq_along(other.tables)){
      other.tables.list[[i]] <- DBI::dbGetQuery(con, paste("SELECT * FROM ", other.tables[i]))
    }
    names(other.tables.list) <- gsub("_", "", other.tables, fixed = TRUE)
  }
  ## Disconnect
  DBI::dbDisconnect(con)
  
  ## Return a list if there is only one report, whatever the name and value == "all"
  if(value == "all" && length(report.names) == 1L){
    lst1 <- list(Report = tbl0)
    ans <- do.call(c, list(lst1, other.tables.list))
  }
  if(value == "all" && length(report.names) > 1L){
    ans <- do.call(c, list(lst0, other.tables.list))
  }
  if(value == "report" && length(report.names) > 1L){
    ans <- lst0
  }
  ## Return data.frame if report and length 1 or user defined
  if((value == "report" && length(report.names) == 1L) || (!value %in% c("report", "all"))){
    ans <- tbl0
  }
    
  return(ans)
}

#' Read all APSIM-X generated .db files in a directory
#' 
#' @title Read all APSIM-X generated .db files in a directory
#' @name read_apsimx_all
#' @description Like \code{\link{read_apsimx}}, but it reads all .db files in a directory. 
#' @param src.dir source directory where files are located
#' @param value either \sQuote{report} or \sQuote{all} (only \sQuote{report} implemented at the moment)
#' @note Warning: very simple function at the moment, not optimized for memory or speed.
#' @return it returns a data frame or a list if \sQuote{value} equals \sQuote{all}.
#' @export
#' 

read_apsimx_all <- function(src.dir = ".", value = "report"){
  
  ## This is super memorey hungry and not efficient at all, but it might work 
  ## for now
  
  file.names <- dir(path = src.dir, pattern=".db$", ignore.case=TRUE)
  
  ans <- NULL
  
  for(i in file.names){
    
    tmp <- read_apsimx(file.names[i], value = value)
    tmp.d <- data.frame(file.name = file.names[i], tmp)
    ans <- rbind(ans, tmp)
    
  }
  return(ans)
}

#### check_apsimx ----

#' @title Partial checking of an apsimx file for possible issues.
#' @name check_apsimx
#' @param file file ending in .apsimx to be edited (JSON)
#' @param src.dir directory containing the .apsimx file to be checked; defaults to the current working directory
#' @param node either \sQuote{all}, \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}
#' @param soil.child specific soil component to be checked.
#' @param check.apsim.met whether to check the \sQuote{met} file. Default is FALSE.
#' @param root supply the node position in the case of multiple simulations such as factorials.
#' @param verbose whether to print information
#' @return It does not return an object, but it prints messages useful for diagnosing issues.
#' @export
#' @examples
#' \donttest{
#' 
#' ## Check file distributed with the package
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' check_apsimx("Wheat.apsimx", src.dir = extd.dir)
#' ## This throws warnings but it should not produce errors
#' }

check_apsimx <- function(file = "", src.dir = ".",
                         node = c("all", "Clock", "Weather", "Soil"),
                         soil.child = c("all", "Physical", "InitialWater", "SoilWater", "Solute", "Organic"),
                         check.apsim.met = FALSE,
                         root = NULL,
                         verbose = TRUE){
  
  if(file == "") stop("need to specify file name")
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }
  
  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  if(verbose){
    fjson <- jsonlite::read_json(file.path(src.dir, file))
    cat("File APSIM version:", fjson$Version, "\n")
  }

  node <- match.arg(node)
  soil.child <- match.arg(soil.child)
  
  ### Checking the Clock
  if(node %in% c("all", "Clock")){
    if(verbose) cat("Checking the Clock \n")
    dates <- try(extract_data_apsimx(file = file, src.dir = src.dir,
                                 node = "Clock", root = root), silent = TRUE)
    if(inherits(dates, 'try-error') || nrow(dates) == 0){
      warning("Clock not found", immediate. = TRUE)
    }else{
      try.start <- try(as.Date(dates[[1]], tryFormats = c("%Y-%m-%dT%H:%M:%S")), silent = TRUE)
      if(inherits(try.start, "try-error"))
        warning(names(dates)[1], " : ", try.start, immediate. = TRUE)
      try.end <- try(as.Date(dates[[2]], tryFormats = c("%Y-%m-%dT%H:%M:%S")), silent = TRUE)
      if(inherits(try.end, "try-error"))
        warning(names(dates)[2], " : ", try.end, immediate. = TRUE)      
    }
  }
  
  ### Checking the Weather
  if(node %in% c("all", "Weather")){
    if(verbose) cat("Checking the Weather (met) \n")
    met.name <- try(extract_data_apsimx(file = file, src.dir = src.dir,
                                    node = "Weather",
                                    root = root), silent = TRUE)
    if(inherits(met.name, 'try-error')){
      message("Weather (met) file path: ", met.name)
      warning("Weather (met) not found", immediate. = TRUE)  
    }else{
      if(grepl("%root%", met.name)){
        ## This means that the weather file is from an example 
        ex.dir <- auto_detect_apsimx_examples()
        mfn0 <- strsplit(met.name[[1]], "[\\/]", fixed = FALSE)[[1]]
        mfn0.length <- length(mfn0)
        mfn <- mfn0[mfn0.length]
        mwf <- mfn0[mfn0.length - 1]
        mfe <- file.exists(file.path(ex.dir, mwf, mfn))
      }else{
        mfe <- file.exists(file.path(src.dir, met.name[[1]]))
      }
      if(!mfe){
        message("Weather (met) file path: ", met.name)
        warning("Could not find 'met' file", immediate. = TRUE)
      }else{
        if(check.apsim.met){
          ### Annoying, but I need to split the name
          mfn <- basename(normalizePath(met.name[[1]])) ## met file name
          mfsd <- dirname(normalizePath(met.name[[1]]))
          met <- read_apsim_met(file = mfn, src.dir = mfsd)
          check_apsim_met(met)
        }
      }
    }
  }

  if(node %in% c("all", "Soil")){
    
    if(soil.child %in% c("all", "Physical")){
      
      if(verbose) cat("Checking the soil physical properties \n")
      ### First: extract soil physical
      soil.physical <- try(extract_data_apsimx(file = file, src.dir = src.dir, 
                                           node = "Soil", 
                                           soil.child = "Physical",
                                           root = root), silent = TRUE)
      if(inherits(soil.physical, 'try-error')){
        warning("Soil 'Physical' component not found", immediate. = TRUE)
      }else{
        ### Check the layers
        soil.physical.layers <- soil.physical[[1]]
        if(verbose) cat("Soil has: ", nrow(soil.physical.layers), "layers \n")
        
        for(i in seq_len(nrow(soil.physical.layers))){
          if(verbose) cat("Checking Physical layer:", i, "\n")
          ### Checking Thickness
          thickness.value <- as.numeric(soil.physical.layers[["Thickness"]][i])
          if(thickness.value <= 0)
            message("Thickness for layer: ", i, " is less than or equal to zero. Value: ", thickness.value)
          ### Checking Bulk Density
          bd.value <- as.numeric(soil.physical.layers[["BD"]][i])
          if(bd.value <= 0)
            message("Bulk Density for layer: ", i, " is less than or equal to zero. Value: ", bd.value)
          if(bd.value > 3)
            message("Bulk Density for layer: ", i, " is greater than 3. Value: ", bd.value)
          ### Checking AirDry
          AirDry.value <- as.numeric(soil.physical.layers[["AirDry"]][i])
          if(AirDry.value <= 0)
            message("AirDry for layer: ", i, " is less than or equal to zero. Value: ", AirDry.value)
          if(AirDry.value > 1)
            message("AirDry for layer: ", i, " is greater than 1. Value: ", AirDry.value)
          ### Checking LL15
          LL15.value <- as.numeric(soil.physical.layers[["LL15"]][i])
          if(LL15.value <= 0)
            message("LL15 for layer: ", i, " is less than or equal to zero. Value: ", LL15.value)
          if(LL15.value > 1)
            message("LL15 for layer: ", i, " is greater than 1. Value: ", LL15.value)
          if(AirDry.value > LL15.value) ## AirDry value should be lower than the LL15 value, always right?
            message("The AirDry value is greater than the LL15 value for layer: ", i, ". AirDry value: ", AirDry.value, ". LL15 value:", LL15.value)
          ### Checking DUL
          DUL.value <- as.numeric(soil.physical.layers[["DUL"]][i])
          if(DUL.value <= 0)
            message("DUL for layer: ", i, " is less than or equal to zero. Value: ", DUL.value)
          if(DUL.value > 1)
            message("DUL for layer: ", i, " is greater than 1. Value: ", DUL.value)
          if(DUL.value < LL15.value) ## DUL value should be greater than the LL15 value, always right?
            message("The DUL value is lower than the LL15 value for layer: ", i, ". DUL value: ", DUL.value, ". LL15 value:", LL15.value)
          ### Checking SAT
          SAT.value <- as.numeric(soil.physical.layers[["SAT"]][i])
          if(SAT.value <= 0)
            message("SAT for layer: ", i, " is less than or equal to zero. Value: ", SAT.value)
          if(SAT.value > 1)
            message("SAT for layer: ", i, " is greater than 1. Value: ", SAT.value)
          if(SAT.value < DUL.value) ## SAT value should be greater than the DUL value, always right?
            message("The SAT value is lower than the DUL value for layer: ", i, ". SAT value: ", SAT.value, ". DUL value:", DUL.value)
        }        
      }
    }
    
    if(soil.child %in% c("all", "InitialWater")){

      if(verbose) cat("Checking the soil initial water \n")
      ### First: extract soil InitialWater
      soil.initialwater <- try(extract_data_apsimx(file = file, src.dir = src.dir, 
                                               node = "Soil", 
                                               soil.child = "InitialWater",
                                               root = root), silent = TRUE)
      if(inherits(soil.initialwater, 'try-error')){
        warning("Soil 'InitialWater' not found", immediate. = TRUE)
      }else{
        soil.initialwater.initialvalues <- soil.initialwater$second
        if(is.null(soil.initialwater.initialvalues)){
          warning("'soil.initialwater.initialvalues' is null", immediate. = TRUE)
        }else{
          ## Check number of layers
          if(soil.child %in% "all"){
            ## Are the number of layers the same as for 'Physical'?
            soil.physical.layers.length <- nrow(soil.physical.layers)
            soil.initialwater.initialvalues.length <- nrow(soil.initialwater.initialvalues)
            if(soil.physical.layers.length != soil.initialwater.initialvalues.length){
              cat("Number of layers in soil physical:", soil.physical.layers.length, "\n")
              cat("Number of layers in initial water values:", soil.initialwater.initialvalues.length, "\n")
              message("Number of layers in physical does match the ones in InitialWater - InitialValues")
            }
            ## Checking that InitialWater is between LL15 and SAT (It can be greater than DUL?)
            if(soil.physical.layers.length == soil.initialwater.initialvalues.length){
              for(i in seq_len(soil.initialwater.initialvalues.length)){
                soil.initialwater.layer.thickness <- soil.initialwater.initialvalues$Thickness[i]
                soil.physical.layer.thickness <- soil.physical.layers$Thickness[i]
                if(soil.physical.layer.thickness != soil.initialwater.layer.thickness){
                  cat("Soil physical Thickness for layer:", i, " is ", soil.physical.layer.thickness, "\n")
                  cat("Soil InitialWater Thickness:", soil.initialwater.layer.thickness, "\n")
                  message("Soil Physical Thickness does not match InitialWater")
                }
                soil.initialwater.layer.values <- soil.initialwater.initialvalues$InitialValues[i]
                soil.physical.layers.LL15 <- soil.physical.layers$LL15[i]
                soil.physical.layers.SAT <- soil.physical.layers$SAT[i]
                if(soil.initialwater.layer.values < soil.physical.layers.LL15){
                  message("Soil InitialWater InitialValues for layer ", i, " is lower than LL15")
                }
                if(soil.initialwater.layer.values > soil.physical.layers.SAT){
                  message("Soil InitialWater InitialValues for layer ", i, " is greater than SAT")
                }
              }
            }
          }        
        }        
      }
    } ## This closes the if(soil.child %in% c("all", "InitialWater"))
    
    ### Checking SoilWater
    if(soil.child %in% c("all", "SoilWater")){
      
      if(verbose) cat("Checking the soil water \n")
      ### First: extract soil InitialWater
      soil.soilwater <- try(extract_data_apsimx(file = file, src.dir = src.dir, 
                                               node = "Soil", 
                                               soil.child = "SoilWater",
                                               root = root), silent = TRUE)
      if(inherits(soil.soilwater, 'try-error')){
        warning("Soil 'SoilWater' not found.", immediate. = TRUE)
      }else{
        soil.soilwater.second <- soil.soilwater$second
        if(is.null(soil.soilwater.second))
          warning("'soil.soilwater.second' is null", immediate. = TRUE)
        ## Check number of layers
        if(soil.child %in% "all"){
          ## Are the number of layers the same as for 'Physical'?
          soil.soilwater.second.length <- nrow(soil.soilwater.second)
          for(i in seq_len(soil.soilwater.second.length)){
            ### Checking that Thickness is grater than zero
            if(verbose) cat("Checking SoilWater layer:", i, "\n")
            if(soil.soilwater.second$Thickness[i] < 0)
              message("SoilWater Thickness for layer: ", i, " is less than zero")
            if(soil.soilwater.second$SWCON[i] < 0)
              message("SoilWater SWCON for layer: ", i, " is less than zero")
            if(soil.soilwater.second$SWCON[i] > 1)
              message("SoilWater SWCON for layer: ", i, " is greater than one")
          }
        }        
      }
    }
    
    if(soil.child %in% c("all", "Solute")){
      if(verbose) cat("Checking the soil Solutes \n")
      ### First: extract soil Solutes
      soil.solutes <- try(extract_data_apsimx(file = file, src.dir = src.dir, 
                                               node = "Soil", 
                                               soil.child = "Solute",
                                               root = root), silent = TRUE)
      if(inherits(soil.solutes, 'try-error')){
        warning("Soil 'Solutes' not found", immediate. = TRUE)
      }else{
        if(verbose) cat("Solutes: ", names(soil.solutes), "\n")
        for(i in seq_along(soil.solutes)){
          soil.solute.initialvalues <- soil.solutes[[i]][[1]]
          soil.solute.initialvalues.length <- nrow(soil.solute.initialvalues)
          if(soil.physical.layers.length != soil.solute.initialvalues.length){
            cat("Number of layers in soil physical:", soil.physical.layers.length, "\n")
            cat("Number of layers in solute values:", soil.solute.initialvalues.length, "\n")
            message("Number of layers in physical does match the ones in Solute - InitialValues")
          }
          for(j in seq_len(soil.solute.initialvalues.length)){
            soil.solute.layer.thickness <- soil.solute.initialvalues$Thickness[i]
            soil.physical.layer.thickness <- soil.physical.layers$Thickness[i]
            if(soil.physical.layer.thickness != soil.solute.layer.thickness){
              cat("Soil physical Thickness for layer:", i, " is ", soil.physical.layer.thickness, "\n")
              cat("Soil solute Thickness:", soil.solute.layer.thickness, "\n")
              message("Soil Physical Thickness does not match Solute")
            }
            soil.solute.layer.values <- soil.solute.initialvalues$InitialValues[i]
            if(!is.numeric(soil.solute.layer.values)){
              warning("Soil Solute InitialValues for layer ", i, " is not numeric", immediate. = TRUE)
            }
            if(soil.solute.layer.values < 0){
              warning("Soil Solute InitialValues for layer ", i, " is less than zero", immediate. = TRUE)
            }
          }
        }        
      }
    }
    
    if(soil.child %in% c("all", "Organic")){
      if(verbose) cat("Checking the soil Organic \n")
      ### First: extract soil Organic
      soil.organic <- try(extract_data_apsimx(file = file, src.dir = src.dir, 
                                          node = "Soil", 
                                          soil.child = "Organic",
                                          root = root), silent = TRUE)
      if(inherits(soil.organic, 'try-error')){
        warning("Soil 'Organic' not found", immediate. = TRUE)
      }else{
        soil.organic.layers <- soil.organic[[2]]
        ### Only checking the second component for now
        if(verbose) cat("Soil Organic has: ", nrow(soil.organic.layers), "layers \n")
        
        for(i in seq_len(nrow(soil.organic.layers))){
          if(verbose) cat("Checking Organic layer:", i, "\n")
          thickness.value <- as.numeric(soil.organic.layers[["Thickness"]][i])
          if(thickness.value <= 0)
            message("Thickness for layer: ", i, " is less than or equal to zero. Value: ", thickness.value)         
          ### Checking Carbon
          carbon.value <- as.numeric(soil.organic.layers[["Carbon"]][i])
          if(carbon.value <= 0)
            message("Carbon for layer: ", i, " is less than or equal to zero. Value: ", carbon.value)
          if(carbon.value > 100)
            message("Carbon for layer: ", i, " is greater than 100. Value: ", carbon.value)
          ### Checking SoilCNRatio
          soilcnratio.value <- as.numeric(soil.organic.layers[["SoilCNRatio"]][i])
          if(soilcnratio.value <= 0)
            message("SoilCNRatio for layer: ", i, " is less than or equal to zero. Value: ", soilcnratio.value)
          if(soilcnratio.value > 100)
            message("SoilCNRatio for layer: ", i, " is greater than 100. Value: ", soilcnratio.value)
          ### Checking FBiom
          FBiom.value <- as.numeric(soil.organic.layers[["FBiom"]][i])
          if(FBiom.value <= 0)
            message("FBiom for layer: ", i, " is less than or equal to zero. Value: ", FBiom.value)
          if(FBiom.value > 1)
            message("FBiom for layer: ", i, " is greater than 1. Value: ", FBiom.value)
          ### Checking FInert
          FInert.value <- as.numeric(soil.organic.layers[["FInert"]][i])
          if(FInert.value <= 0)
            message("FInert for layer: ", i, " is less than or equal to zero. Value: ", FInert.value)
          if(FInert.value > 1)
            message("FInert for layer: ", i, " is greater than 1. Value: ", FInert.value)
          ### Checking that FBiom + FInert are not greater than 1
          if(FBiom.value + FInert.value > 1){
            message("FBiom + FInert for layer: ", i, " is greater than 1. FBiom Value: ", FBiom.value, " FInert Value: ", FInert.value)
          }
          ### Checking FOM
          FOM.value <- as.numeric(soil.organic.layers[["FOM"]][i])
          if(FOM.value < 0)
            message("FOM for layer: ", i, " is less than zero. Value: ", FOM.value)
          if(FOM.value > 1e5)
            message("FOM for layer: ", i, " is greater than 100,000. Value: ", FOM.value)
        }
      }
    }
  }
}


#' Set apsimx options
#' 
#' @title Setting some options for the package
#' @name apsimx_options
#' @description Set the path to the APSIM-X executable, examples and warning suppression. 
#' @param exe.path path to apsim executable. White spaces are not allowed.
#' @param dotnet logical indicating if APSIM should be run through the dotnet command
#' @param mono logical indicating if the mono command should be used when running APSIM. This is for versions 
#' for Mac/Linux older than Sept 2021. 
#' @param examples.path path to apsim examples
#' @param warn.versions logical. warning if multiple versions of APSIM-X are detected.
#' @param warn.find.apsimx logical. By default a warning will be thrown if APSIM-X is not found. 
#' If \sQuote{exe.path} is \sQuote{NA} an error will be thrown instead.
#' @param allow.path.spaces logical. By default spaces are not allowed in paths or in the run command.
#' @param system.intern logical. Default is FALSE. \sQuote{intern} argument passed to \code{\link{system}}.
#' @note It is possible that APSIM-X is installed in some alternative location other than the 
#'       defaults ones. Guessing this can be difficult and then the auto_detect functions might
#'       fail. Also, if multiple versions of APSIM-X are installed apsimx will choose the newest
#'       one but it will issue a warning. Suppress the warning by setting warn.versions = FLASE.
#' @return as a side effect it modifies the \sQuote{apsimx.options} environment.
#' @export
#' @examples 
#'\donttest{
#' names(apsimx.options)
#' apsimx_options(exe.path = "some-new-path-to-executable")
#' apsimx.options$exe.path
#' }

apsimx_options <- function(exe.path = NA, dotnet = FALSE, mono = FALSE, examples.path = NA, 
                           warn.versions = TRUE, warn.find.apsimx = TRUE, allow.path.spaces = FALSE,
                           system.intern = FALSE){
  
  if(dotnet && mono)
    stop("either dotnet or mono should be TRUE, but not both", call. = TRUE)
  
  assign('exe.path', exe.path, apsimx.options)
  assign('dotnet', dotnet, apsimx.options)
  assign('mono', mono, apsimx.options)
  assign('examples.path', examples.path, apsimx.options)
  assign('warn.versions', warn.versions, apsimx.options)
  assign('warn.find.apsimx', warn.find.apsimx, apsimx.options)
  assign('allow.path.spaces', allow.path.spaces, apsimx.options)
  assign('system.intern', system.intern, apsimx.options)
}

#' Environment which stores APSIM-X options
#' 
#' @title Environment which stores APSIM-X options
#' @name apsimx.options
#' @description Environment which can store the path to the executable, warning settings and 
#'              where examples are located.
#'              Creating an environment avoids the use of global variables or other similar practices
#'              which would have possible undesriable consequences. 
#' @return This is an environment, not a function, so nothing is returned.
#' @export
#' @examples 
#' \donttest{
#' names(apsimx.options)
#' apsimx_options(exe.path = "some-new-path-to-executable")
#' apsimx.options$exe.path
#' }
#' 

apsimx.options <- new.env(parent = emptyenv())
assign('exe.path', NA, apsimx.options)
assign('dotnet', FALSE, apsimx.options)
assign('mono', FALSE, apsimx.options)
assign('examples.path', NA, apsimx.options)
assign('warn.versions', TRUE, apsimx.options)
assign('warn.find.apsimx', TRUE, apsimx.options)
assign('allow.path.spaces', FALSE, apsimx.options)
assign('system.intern', FALSE, apsimx.options)
assign('.run.local.tests', FALSE, apsimx.options)

## I'm planning to use '.run.local.tests' for running tests
## which do not require an APSIM install

## Import packages needed for apsimx to work correctly
#' @import DBI jsonlite knitr RSQLite xml2 
#' @importFrom utils head read.table write.table packageVersion
#' @importFrom tools file_path_sans_ext file_ext
#' @importFrom stats aggregate anova coef cor cov2cor deviance lm optim pnorm qt quantile var sd setNames sigma terms reformulate
NULL

utils::globalVariables(".data")

#' Source: https://isric.org/ \cr
#' Details: https://docs.isric.org/globaldata/soilgrids/SoilGrids_faqs.html \cr
#' 
#' Pedotransfer functions: Saxton and Rawls, 2006. Soil Water Characteristic Estimates by Texture and Organic Matter for Hydrologic Solutions.
#' Soil Sci. Soc. Am. J. 70:1569–1578. \cr
#' 
#' TODO: need to look into how this is done in APSIM NG
#' https://github.com/APSIMInitiative/ApsimX/pull/3994/files \cr
#' 
#' NOTE: Eric Zurcher provided help by sending me an R file originally written by
#' Andrew Moore. It provides a bit of context for how some of the decisions
#' were made for constructing the synthetic soil profiles in APSIM. (email from june 3 2021).
#' 
#' @title Generate a synthetic APSIM soil profile from the ISRIC soil database
#' @description Retrieves soil data from the ISRIC global database and converts it to an APSIM soil_profile object
#' @name get_isric_soil_profile
#' @param lonlat Longitude and latitude vector (e.g. c(-93, 42)).
#' @param statistic default is the mean
#' @param soil.profile a soil profile to fill in in case the default one is not appropriate
#' @param find.location.name default is TRUE. Use either maps package or photon API to find Country/State.
#' If you are running this function many times it might be better to set this to FALSE.
#' @param fix whether to fix compatibility between saturation and bulk density (default is FALSE).
#' @param verbose argument passed to the fix function.
#' @param check whether to check the soil profile (default is TRUE)
#' @param physical whether soil physical properties are obtained from the data base or through \sQuote{SR}, Saxton and Rawls pedotransfer functions.
#' @param xargs additional arguments passed to \code{\link{apsimx_soil_profile}} or \sQuote{apsimx:::approx_soil_variable} function. At the moment these are:
#' \sQuote{soil.bottom}, \sQuote{crops}, and \sQuote{nlayers} for the first function and \sQuote{method} for the second function. 
#' @return it generates an object of class \sQuote{soil_profile}.
#' @details Variable which are directly retrieved and a simple unit conversion is performed: \cr
#' * Bulk density - bdod \cr
#' * Carbon - soc \cr
#' * Clay - clay \cr
#' * Sand - sand \cr
#' * PH - phh2o \cr
#' * Nitrogen - nitrogen \cr
#' Variables which are optionally estimated using pedotransfer functions: \cr
#' LL15, DUL, SAT, KS, AirDry \cr
#' TO-DO: \cr
#' What do I do with nitrogen? \cr
#' Can I use CEC? \cr
#' How can I have a guess at FBiom and Finert? \cr
#' FBiom does not depend on any soil property at the moment, should it? \cr
#' @seealso \code{\link{apsimx_soil_profile}}, \code{\link{edit_apsim_replace_soil_profile}}, \code{\link{edit_apsimx_replace_soil_profile}}.
#' @export
#' @author Fernando E. Miguez, Eric Zurcher (CSIRO) and Andrew Moore (CSIRO)
#' @examples 
#' \dontrun{
#' ## Only run this if rest.isric.org is working
#' rest.isric.on <- suppressWarnings(try(readLines("http://rest.isric.org", 
#' n = 1, warn = FALSE), silent = TRUE))
#'
#' ## Get soil profile properties for a single point
#' if(!inherits(rest.isric.on, "try-error")){
#'   sp1 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, verbose = FALSE)
#'   ## Visualize
#'   plot(sp1)
#'   plot(sp1, property = "water")
#'  }
#' }

get_isric_soil_profile <- function(lonlat, 
                                   statistic = c("mean", "Q0.5"),
                                   soil.profile,
                                   find.location.name = TRUE,
                                   fix = FALSE,
                                   verbose = TRUE,
                                   check = TRUE,
                                   physical = c("default", "SR"),
                                   xargs = NULL){

  statistic <- match.arg(statistic)
  physical <- match.arg(physical)
  
  #### Create extent step ####
  lon <- as.numeric(lonlat[1])
  lat <- as.numeric(lonlat[2])
  
  if(lon < -180 || lon > 180) stop("longitude should be between -180 and 180")
  if(lat < -90 || lat > 90) stop("latitude should be between -90 and 90")
  
##  rest0 <- "https://rest.soilgrids.org/soilgrids/v2.0/properties/query?lon="
  rest0 <- "https://rest.isric.org/soilgrids/v2.0/properties/query?lon="
  rest1 <- paste0(rest0, lon, "&lat=", lat)
  rest.properties <- paste("&property=bdod", 
                           "property=soc",
                           "property=phh2o",
                           "property=clay", 
                           "property=sand", 
                           "property=nitrogen",
                           "property=cec", 
                           "property=wv0010",
                           "property=wv0033",
                           "property=wv1500", sep = "&")
  rest.depths <- paste("&depth=0-5cm", "depth=0-30cm", "depth=5-15cm", 
                       "depth=15-30cm", "depth=30-60cm", "depth=60-100cm", "depth=100-200cm", sep = "&")
  rest.statistic <- paste("&value", statistic, sep = "=")
  rest.query <- paste0(rest1, rest.properties, rest.depths, rest.statistic)
  rest.data <- jsonlite::fromJSON(rest.query)
  
  #### Process query
  sp.nms <- rest.data$properties$layers[["name"]]
  
  if(!identical(sp.nms, c("bdod", "cec", "clay", "nitrogen", "phh2o", "sand", "soc", "wv0010", "wv0033", "wv1500"))){
    cat("Found these properties", sp.nms, "\n")
    cat("Expected these properties", c("bdod", "cec", "clay", "nitrogen", "phh2o", "sand", "soc", "wv0010", "wv0033", "wv1500"), "\n")
    stop("soil properties names do not match")
  }
    
  bdod <- rest.data$properties$layers[1,3][[1]][,3]
  cec <- rest.data$properties$layers[2,3][[1]][,3]
  clay <- rest.data$properties$layers[3,3][[1]][,3]
  nitrogen <- rest.data$properties$layers[4,3][[1]][,3]
  phh2o <- rest.data$properties$layers[5,3][[1]][,3]
  sand <- rest.data$properties$layers[6,3][[1]][,3]
  soc <- rest.data$properties$layers[7,3][[1]][,3]
  wv0010 <- rest.data$properties$layers[8,3][[1]][,3]
  wv0033 <- rest.data$properties$layers[9,3][[1]][,3]
  wv1500 <- rest.data$properties$layers[10,3][[1]][,3]
  
  if(any(is.na(soc))) stop("No soil data available for this location. Did you specify the coordinates correctly?")

  ## These are the default thicknesses in ISRIC
  o.thcknss <- c(50, 100, 150, 300, 400, 1000) ## in mm
  thcknss <- c(50, 100, 150, 300, 400, 1000) ## in mm
  
  ## Some variables can be passed to apsimx:::approx_soil_variable
  soil.bottom <- 200
  method <- "constant"
  nlayers <- 6
  crps <- c("Maize", "Soybean", "Wheat")
  if(!is.null(xargs)){
    ### Soil bottom
    if(!is.null(xargs$soil.bottom) && is.null(xargs$nlayers)){
      soil.bottom <- xargs$soil.bottom
      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
    }
    ### Method
    if(!is.null(xargs$method)){
      method <- xargs$method
    }
    ### Number of layers
    if(!is.null(xargs$nlayers) && is.null(xargs$soil.bottom)){
      nlayers <- xargs$nlayers
      ### Need to redefine 'Thickness'
      thcknss <- apsimx_soil_profile(nlayers = nlayers)$soil$Thickness
    }
    ### Do I need this?
    if(!is.null(xargs$nlayers) && !is.null(xargs$soil.bottom)){
      nlayers <- xargs$nlayers
      soil.bottom <- xargs$soil.bottom
      ### Need to redefine 'Thickness'
      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
    }
    ### Crops
    if(!is.null(xargs$crops)){
      crps <- xargs$crops
    }
  }

  ## Create the empty soil profile
  if(missing(soil.profile)){
    if(is.null(xargs)){
      new.soil <- FALSE
    }else{
      new.soil <- TRUE      
    }
    soil_profile <- apsimx_soil_profile(nlayers = nlayers, Thickness = thcknss, soil.bottom = soil.bottom, crops = crps) 
    soil_profile$soil$ParticleSizeClay <- NA
    soil_profile$soil$ParticleSizeSilt <- NA
    soil_profile$soil$ParticleSizeSand <- NA
    soil_profile$soil$CEC <- NA  
    soil_profile$soil$Nitrogen <- NA
    soil_profile$soil$DUL <- NA
    soil_profile$soil$LL15 <- NA
    soil_profile$soil$SAT <- NA
    
  }else{
    ## stop("This is not fully implemented yet. Submit a github issue if you need it.", call. = FALSE)
    soil_profile <- soil.profile
    new.soil <- TRUE
  }
  
  ### For some of the conversions see: https://www.isric.org/explore/soilgrids/faq-soilgrids
  if(new.soil){
    sp.xout <- cumsum(soil_profile$soil$Thickness)
    
    ### Will this take care of both number of layers and soil.bottom?
    if(length(cumsum(thcknss)) != length(bdod[[1]])){
      bdod.dat <- stats::approx(data.frame(cumsum(o.thcknss), bdod[[1]] * 1e-2), xout = sp.xout)
      soc.dat <- stats::approx(data.frame(cumsum(o.thcknss), soc[[1]] * 1e-2), xout = sp.xout)
      phh2o.dat <- stats::approx(data.frame(cumsum(o.thcknss), phh2o[[1]] * 1e-1), xout = sp.xout)
      clay.dat <- stats::approx(data.frame(cumsum(o.thcknss), clay[[1]] * 1e-1), xout = sp.xout)
      sand.dat <- stats::approx(data.frame(cumsum(o.thcknss), sand[[1]] * 1e-1), xout = sp.xout)
      nitrogen.dat <- stats::approx(data.frame(cumsum(o.thcknss), nitrogen[[1]]), xout = sp.xout)
      cec.dat <- stats::approx(data.frame(cumsum(o.thcknss), cec[[1]]), xout = sp.xout)
      wv0010.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv0010[[1]]), xout = sp.xout)
      wv0033.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv0033[[1]]), xout = sp.xout)
      wv1500.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv1500[[1]]), xout = sp.xout)
    }else{
      bdod.dat <- data.frame(x = cumsum(thcknss), y = bdod[[1]] * 1e-2)
      soc.dat <- data.frame(x = cumsum(thcknss), y = soc[[1]] * 1e-2)
      phh2o.dat <- data.frame(x = cumsum(thcknss), y = phh2o[[1]] * 1e-1)
      clay.dat <- data.frame(x = cumsum(thcknss), y = clay[[1]] * 1e-1)
      sand.dat <- data.frame(x = cumsum(thcknss), y = sand[[1]] * 1e-1)
      nitrogen.dat <- data.frame(x = cumsum(thcknss), y = nitrogen[[1]])
      cec.dat <- data.frame(x = cumsum(thcknss), y = cec[[1]])
      wv0010.dat <- data.frame(x = cumsum(thcknss), y = wv0010[[1]])
      wv0033.dat <- data.frame(x = cumsum(thcknss), y = wv0033[[1]])
      wv1500.dat <- data.frame(x = cumsum(thcknss), y = wv1500[[1]])
    }

    soil_profile$soil$BD <- approx_soil_variable(bdod.dat, 
                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$Carbon <- approx_soil_variable(soc.dat, 
                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$PH <- approx_soil_variable(phh2o.dat, 
                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$ParticleSizeClay <- approx_soil_variable(clay.dat, 
                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$ParticleSizeSand <- approx_soil_variable(sand.dat, 
                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$Nitrogen <- approx_soil_variable(nitrogen.dat, 
                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
    soil_profile$soil$CEC <- approx_soil_variable(cec.dat, 
                                                       xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
    soil_profile$soil$SAT <- approx_soil_variable(wv0010.dat, 
                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3
    soil_profile$soil$DUL <- approx_soil_variable(wv0033.dat, 
                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3  
    soil_profile$soil$LL15 <- approx_soil_variable(wv1500.dat, 
                                                  xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3   
  }else{
    soil_profile$soil$BD <- bdod[[1]] * 1e-2
    soil_profile$soil$Carbon <- soc[[1]] * 1e-2
    soil_profile$soil$PH <- phh2o[[1]] * 1e-1  
    soil_profile$soil$ParticleSizeClay <- clay[[1]] * 1e-1
    soil_profile$soil$ParticleSizeSand <- sand[[1]] * 1e-1
    soil_profile$soil$Nitrogen <- nitrogen[[1]]
    soil_profile$soil$CEC <- cec[[1]]
    soil_profile$soil$SAT <- wv0010[[1]] * 1e-3
    soil_profile$soil$DUL <- wv0033[[1]] * 1e-3
    soil_profile$soil$LL15 <- wv1500[[1]] * 1e-3
  }

  soil_profile$soil$ParticleSizeSilt <- 100 - (soil_profile$soil$ParticleSizeSand + soil_profile$soil$ParticleSizeClay)
  ## Populating DUL and LL. These are equations from Table 1 in Saxton and Rawls 2006
  if(physical == "SR"){
    soil_profile$soil$DUL <- sr_dul(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    soil_profile$soil$LL15 <- sr_ll(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    DUL_S <- sr_dul_s(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
    soil_profile$soil$SAT <- sr_sat(soil_profile$soil$ParticleSizeSand, soil_profile$soil$DUL, DUL_S)    
  }

  ## Comparing this to the previous calculation
  soil_profile$soil$KS <- sr_ks(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
  
  soil_profile$soil$AirDry <- soil_profile$soil$LL15
  soil_profile$soil$AirDry[1] <- soil_profile$soil$LL15[1] * 0.5 ## AirDry is half of LL for the first layer
  
  for(i in soil_profile$crops){
    soil_profile$soil[[paste0(i,".LL")]] <- soil_profile$soil$LL15 ## Without better information  
  }
  
  #### Passing parameters from soilwat
  ## The soil texture class will be based on the first layer only
  txt_clss <- texture_class(soil_profile$soil$ParticleSizeClay[1] * 1e-2, soil_profile$soil$ParticleSizeSilt[1] * 1e-2)
  t2sp <- texture2soilParms(txt_clss)
  
  if(missing(soil.profile)){
    soil_profile$soilwat <- soilwat_parms(Salb = t2sp$Albedo, CN2Bare = t2sp$CN2, 
                                          SWCON = rep(t2sp$SWCON, nrow(soil_profile$soil)),
                                          Thickness = thcknss)    
  }else{
    soil_profile$soilwat <- soilwat_parms(Salb = t2sp$Albedo, CN2Bare = t2sp$CN2, 
                                          SWCON = rep(t2sp$SWCON, length(soil_profile$soil$Thickness)),
                                          Thickness = soil_profile$soil$Thickness)    
  }
  
  ### Passing the initial soil water?
  if(new.soil){
    ini.wat <- (soil_profile$soil$DUL + soil_profile$soil$LL15) / 2
    isw <- initialwater_parms(Thickness = soil_profile$soil$Thickness, 
                              InitialValues = ini.wat)
  }else{
    ini.wat <- (soil_profile$soil$DUL + soil_profile$soil$LL15) / 2
    isw <- initialwater_parms(Thickness = thcknss, InitialValues = ini.wat)    
  }

  soil_profile$initialwater <- isw

  if(find.location.name){
    if(requireNamespace("maps", quietly = TRUE)){
      country <- maps::map.where(x = lon, y = lat)
      if(country == "USA"){
        state <- toupper(maps::map.where(database = "county", x = lon, y = lat)) 
      }else{
        url <- paste0("https://photon.komoot.io/reverse?lon=", lon, "&lat=", lat)
        fgeo <- jsonlite::fromJSON(url)
        state <- fgeo$feature$properties$state
      }
    }else{
      url <- paste0("https://photon.komoot.io/reverse?lon=", lon, "&lat=", lat)
      fgeo <- jsonlite::fromJSON(url)
      state <- fgeo$feature$properties$state
      country <- fgeo$features$properties$country
    }    
  }else{
    state <- ""
    country <- ""
  }

  #### Attributes ####
  alist <- list()
  alist$SoilType <- paste("SoilType = ", txt_clss)
  alist$State <- state
  alist$Country <- country
  alist$Longitude <- lon
  alist$Latitude <- lat
  alist$DataSource <- paste("Original source is www.isric.org. See: https://www.isric.org/explore/soilgrids/faq-soilgrids ",Sys.time())
  alist$Comments <- paste("resolution = 250m",
                          "- taxonomic classification name =", txt_clss,
                          "- drainage class =", NA, 
                          "- elevation =", NA,
                          "- slope =", NA,
                          "- geomdesc =", NA)
  
  soil_profile$metadata <- alist

  if(fix) soil_profile <- fix_apsimx_soil_profile(soil_profile, verbose = verbose)
  
  if(check) check_apsimx_soil_profile(soil_profile)
  
  return(soil_profile)  
}


#### Pedo Transfer equations (Saxton and Rawls) ####

## Field Capacity or DUL
## According to Saxton and Rawls the inputs are in percent
## However, to match the values in Table 3 I need to convert
## clay and sand to proportion, BUT OM stays in percent!
sr_dul <- function(clay, sand, om){
  p2p <- 1e-2
  clay <- clay * p2p
  sand <- sand * p2p
  om <- om 
  ans0 <- -0.251 * sand + 0.195 * clay + 0.011 * om +
    0.006 * (sand * om) - 0.027 * (clay * om) + 0.452 * (sand * clay) + 0.299
  ans <- ans0 + (1.283 * ans0^2 - 0.374 * ans0 - 0.015)
  ans
}

sr_dul_s <- function(clay, sand, om){
  p2p <- 1e-2
  clay <- clay * p2p
  sand <- sand * p2p
  om <- om 
  ans0 <- 0.278 * sand + clay * 0.034 + om * 0.022 +
    -0.018 * sand * om - 0.027 * clay * om + 
    -0.584 * sand * clay + 0.078
  ans <- ans0 + (0.636 * ans0 - 0.107)
  ans
}

sr_sat <- function(sand, sr_dul, sr_dul_s){
  sand <- sand * 1e-2
  ans <- sr_dul + sr_dul_s - 0.097 * sand + 0.043
  ans
}

sr_ll <- function(clay, sand, om){
  p2p <- 1e-2 ## percent to proportion
  clay <- clay * p2p
  sand <- sand * p2p
  om <- om
  ans0 <- -0.024 * sand + 0.487 * clay + 0.006 * om + 
    0.005 * (sand * om) - 0.013 * (clay * om) + 0.068 * (sand * clay) + 0.031
  ans <- ans0 + (0.14 * ans0 - 0.02)
  ans
} 

### Saxton and Rawls KS
### This version was developed to estimate KS in mm/h
### and this is for 'gravel-free' soil
### Multiplying by 24 converts hours to a full day
### but I'm not sure this will work. APSIM is not too
### sensitive to this soil property as long as it is not
### too low
sr_ks <- function(clay, sand, om, units = c("mm/day", "mm/h")){
  
  units <- match.arg(units)
  dul <- sr_dul(clay, sand, om)
  dul_s <- sr_dul_s(clay, sand, om)
  ll15 <- sr_ll(clay, sand, om)
  sat <- sr_sat(sand, dul, dul_s)
  B <- (log(1500) - log(33))/(log(dul) - log(ll15))
  Lambda <- 1/B
  conv <- ifelse(units == "mm/day", 24, 1)
  ans <- 1930 * (sat - dul)^(3 - Lambda) * conv
  
  if(any(is.na(ans))){
    stop("The function resulted in an NA")
  }
  ans
}

## Texture to other parameters
texture2soilParms <- function(texture.class = "NO DATA"){

  if(length(texture.class) > 1)
    stop("This function only takes arguments of length equal to 1", call. = FALSE)
  
  textureClasses <- c("clay", "silty clay", "sandy clay", "clay loam", "silty clay loam", "sandy clay loam", "loam", "silty loam", "sandy loam", "silt", "loamy sand", "sand", "NO DATA")  
  Albedo <- c(0.12, 0.12, 0.13, 0.13, 0.12, 0.13, 0.13, 0.14, 0.13, 0.13, 0.16, 0.19, 0.13)
  CN2 <- c(73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 68.0, 73.0, 68.0, 68.0, 73.0)
  SWCON <- c(0.25, 0.3, 0.3, 0.4, 0.5, 0.5, 0.5, 0.5, 0.6, 0.5, 0.6, 0.75, 0.5)
  
  wtc <- which(textureClasses == texture.class)
  ans <- list(textureClasses = textureClasses[wtc], Albedo = Albedo[wtc], CN2 = CN2[wtc], SWCON = SWCON[wtc])
  ans
}

## Potentially useful function provided by Eric Zurcher
## written by Andrew Moore
## I think the values should be in the 0-1 range
#==========================================================================
# Texture class mapping function
#==========================================================================

# Re-express the PSD in terms of the International system, using an equation from Minasny et al. (2001)

intl_clay_propn <- function( usda_clay, usda_silt ) { 
  return( usda_clay) 
}

intl_silt_propn <- function( usda_clay, usda_silt ) { 
  return( max( 0.0, -0.0041 - 0.127*usda_clay + 0.553*usda_silt + 0.17*usda_clay^2 - 0.19*usda_silt^2 + 0.59*usda_clay*usda_silt ) ) 
}

intl_sand_propn <- function( usda_clay, usda_silt ) { 
  return( 1.0 - intl_clay_propn( usda_clay, usda_silt ) - intl_silt_propn( usda_clay, usda_silt ) )
}  

# Texture triangle as equations

texture_class <- function (usda_clay, usda_silt ) {
  
  if(length(usda_clay) > 1 || length(usda_silt) > 1)
    stop("This function only accepts numeric vectors of length equal to 1", call. = FALSE)
  
  if(usda_clay < 0 || usda_clay > 1) stop("usda_clay should be between 0 and 1")
  if(usda_silt < 0 || usda_silt > 1) stop("usda_silt should be between 0 and 1")
  
  intl_clay <- intl_clay_propn( usda_clay, usda_silt )
  intl_silt <- intl_silt_propn( usda_clay, usda_silt )
  intl_sand <- 1.0 - intl_clay - intl_silt
  
  if ((intl_sand < 0.75 - intl_clay) && (intl_clay >= 0.40)) {
    class <- "silty clay"
  } else if ((intl_sand < 0.75 - intl_clay) && (intl_clay >= 0.26)) {
    class <- "silty clay loam"
  } else if (intl_sand < 0.75 - intl_clay) {
    class <- "silty loam"
  } else if ((intl_clay >= 0.40 + (0.305-0.40)/(0.635-0.35) * (intl_sand-0.35)) && (intl_clay < 0.50 + (0.305-0.50)/(0.635-0.50) * (intl_sand - 0.50))) {
    class <- "clay"
  } else if (intl_clay >= 0.26 + (0.305-0.26)/(0.635-0.74) * (intl_sand-0.74)) {
    class <- "sandy clay"
  } else if ((intl_clay >= 0.26 + (0.17-0.26)/(0.83-0.49) * (intl_sand-0.49)) && (intl_clay < 0.10 + (0.305-0.10)/(0.635-0.775) * (intl_sand - 0.775))) {
    class <- "clay loam"
  } else if (intl_clay >= 0.26 + (0.17-0.26)/(0.83-0.49) * (intl_sand-0.49)) {
    class <- "sandy clay loam"
  } else if ((intl_clay >= 0.10 + (0.12-0.10)/(0.63-0.775) * (intl_sand-0.775)) && (intl_clay < 0.10 + (0.305-0.10)/(0.635-0.775) * (intl_sand - 0.775))) {
    class <- "loam"
  } else if (intl_clay >= 0.10 + (0.12-0.10)/(0.63-0.775) * (intl_sand-0.775)) {
    class <- "sandy loam"
  } else if (intl_clay < 0.00 + (0.08-0.00)/(0.88-0.93) * (intl_sand-0.93)) {
    class <- "loamy sand"
  } else {
    class <- "sand"
  }  
  
  return( class )
}

### Saxton and Rawls (Table 3 in the paper)
sr_table <- function(clay, sand, om){
  
  if(missing(clay)){
    texture.classes <- c("sand", "loamy sand", "sandy loam", "loam", "silty loam", "silt", "sandy clay loam", "clay loam", "silty clay loam", "silty clay", "sandy clay", "clay")
  }else{
    
    if(length(clay) != length(sand))
      stop("length of 'clay' should be the same as length of 'sand'")
    
    if(length(clay) != length(om))
      stop("length of 'clay' should be the same as length of 'om'")
    
    if(any(clay < 0)) stop("Clay should be greater than zero")
    if(any(clay > 100)) stop("Clay should be less than 100")
    if(any(sand < 0)) stop("Sand should be greater than zero")
    if(any(sand > 100)) stop("Sand should be less than 100")
    
  }
  
  ans <- data.frame(texture.class = NA, sand = NA, clay = NA,
                    om = NA, ll15 = NA, dul = NA, sat = NA, 
                    paw = NA, ks = NA)
  
  if(missing(clay)){
    clay <- c(5, 5, 10, 20, 15, 5, 25, 35, 35, 45, 40, 50)
    sand <- c(88, 80, 65, 40, 20, 10, 60, 30, 10, 10, 50, 25)
    om <- rep(2.5, length(clay))
    for(i in seq_along(texture.classes)){
      ans[i, "texture.class"] <- texture.classes[i]
      ans[i, "sand"] <- sand[i]
      ans[i, "clay"] <- clay[i]
      ans[i, "om"] <- om[i]
      ll15 <- sr_ll(clay[i], sand[i], om[i])
      ans[i, "ll15"] <- ll15
      dul <- sr_dul(clay[i], sand[i], om[i])
      ans[i, "dul"] <- dul
      dul_s <- sr_dul_s(clay[i], sand[i], om[i])
      ans[i, "sat"] <- sr_sat(sand[i], dul, dul_s)
      ans[i, "paw"] <- dul - ll15 
      ans[i, "ks"] <- sr_ks(clay[i], sand[i], om[i], units = "mm/h")
    }    
  }else{
    for(i in seq_along(clay)){
      clay.plus.sand <- clay[i] + sand[i]
      if(clay.plus.sand >= 100){
        warning("Clay plus sand cannot be greater or equal to 100")
      }
      silt <- 100 - (clay[i] + sand[i])
      ans[i, "texture.class"] <- texture_class(clay[i], silt)
      ans[i, "sand"] <- sand[i]
      ans[i, "clay"] <- clay[i]
      ans[i, "om"] <- om[i]
      ll15 <- sr_ll(clay[i], sand[i], om[i])
      ans[i, "ll15"] <- ll15
      dul <- sr_dul(clay[i], sand[i], om[i])
      ans[i, "dul"] <- dul
      dul_s <- sr_dul_s(clay[i], sand[i], om[i])
      ans[i, "sat"] <- sr_sat(sand[i], dul, dul_s)
      ans[i, "paw"] <- dul - ll15 
      ans[i, "ks"] <- sr_ks(clay[i], sand[i], om[i], units = "mm/h")
    }
  }
 return(ans) 
}

## Testing replacements examples with APSIM-X
## Run a few tests for the examples
require(apsimx)
apsimx_options(warn.versions = FALSE)

run.replacements.tests <- get(".run.local.tests", envir = apsimx.options)

extd.dir <- system.file("extdata", package = "apsimx")

replacements <- c("MaizeSoybean.apsimx","WheatRye.apsimx","Soybean.apsimx")

tmp.dir <- tempdir()

setwd(tmp.dir)

if(run.replacements.tests){
  start <- Sys.time()
  for(i in replacements){
    rep.tst <- apsimx(i, src.dir = extd.dir, cleanup = TRUE)
    cat("Ran: ", i, "\n")
  }
  end <- Sys.time()
  cat("Total time:", end - start, "\n")
  if(Sys.info()[["sysname"]] != "Windows"){
    cat("APSIM-X version:", apsim_version(which = "inuse", verbose = FALSE)[1],"\n")  
  }else{
    aiu <- apsim_version(which = "inuse", verbose = FALSE)[2]
    cat("APSIM-X version:", as.character(aiu[1,1]),"\n")
  }
}

#' 
#' There are different metrics with different interpretations produced by this function.
#' Many of them take one object (assuming that is the observed data) and another object
#' (assuming that is the simulated or predicted) object. Some of these metrics
#' are a result from the regression.
#' 
#' - metrics that do not rely on regression (obs vs. pred): bias, mean bias, rss, rmse,
#' corr, concorr, mod.eff
#' 
#' - metrics that do depend on a regression model (Reg): intercept, slope, rsigma, R2. In this case,
#' a simple linear regression is first adjusted to the two variables and the parameters and
#' statistics are derived from that model. 
#' 
#' \sQuote{rsigma} is the regression residual standard deviation obtained by using the function
#' \code{\link[stats]{sigma}} to the regression object.
#' 
#' @title Compare two or more apsim output objects
#' @name compare_apsim
#' @rdname compare_apsim
#' @description Function which allows for a simple comparison between APSIM output objects
#' @param ... data frames with APSIM output or observed data. 
#' @param variable specific variable to compare. By default all common ones are compared.
#' @param index index for merging objects. Default is \sQuote{Date}
#' @param by factor for splitting the comparison, such as a treatment effect (not implemented yet).
#' @param labels labels for plotting and identification of objects.
#' @param cRSS compute (weighted) combined residual sum of squares using some or all variables
#' @param weights optional weights for computing the (weighted) combined sum of squares
#' @param verbose whether to print indexes to console (default is FALSE).
#' @note \sQuote{Con Corr} is the concordance correlation coefficient (https://en.wikipedia.org/wiki/Concordance_correlation_coefficient);
#' \sQuote{ME} is the model efficiency (https://en.wikipedia.org/wiki/Nash-Sutcliffe_model_efficiency_coefficient)
#' @export
#' @return object of class \sQuote{out_mrg}, which can be used for further plotting
#' @examples 
#' \donttest{
#' ## Directory with files
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## Comparing observed and simulated for Wheat
#' data(obsWheat)
#' sim.opt <- read.csv(file.path(extd.dir, "wheat-sim-opt.csv"))
#' sim.opt$Date <- as.Date(sim.opt$Date)
#'
#' cap <- compare_apsim(obsWheat, sim.opt, labels = c("obs", "sim"))
#'
#' plot(cap)
#' plot(cap, plot.type = "diff")
#' plot(cap, plot.type = "resid")
#' plot(cap, plot.type = "ts")
#'
#' plot(cap, variable = "AboveGround")
#' plot(cap, variable = "AboveGround", plot.type = "diff")
#' plot(cap, variable = "AboveGround", plot.type = "ts")
#' 
#' ## Selecting a variable
#' cap <- compare_apsim(obsWheat, sim.opt, variable = "AboveGround", 
#'                      labels = c("obs", "sim"))
#'                      
#' ## Using id
#' plot(cap, variable = "AboveGround", id = 0.05)
#'                      
#' }
#' 

compare_apsim <- function(..., 
                          variable,
                          index = "Date",
                          by,
                          labels, 
                          cRSS = FALSE,
                          weights,
                          verbose = FALSE){
  
  outs <- list(...)
  
  n.outs <- length(outs)
  
  if(n.outs < 2) stop("you should provide at least two data frames")
  
  if(!missing(by))
    stop("Not implemented yet")
  
  out1 <- as.data.frame(outs[[1]])
  
  o.nms <- NULL
  if(!missing(labels)){
    o.nms <- labels
    if(length(labels) != n.outs)
      stop(" 'labels' lenght should be the same as the number of 'output' objects", call. = FALSE)
  } 
  
  if(!inherits(out1, "data.frame")) stop("object should be of class 'data.frame' ", call. = FALSE)
  
  if(!missing(variable)){
    if(length(variable) > 1) 
      stop("Only one variable can be selected", call. = FALSE)    
  }

  ## Process out1
  nms1 <- names(out1)
  
  if(!all(index %in% nms1)) 
    stop("index not found in first data.frame")
  
  if(!all(index %in% names(outs[[2]]))) 
    stop("index not found in second data.frame")
  
  nms1.i <- intersect(nms1, names(outs[[2]])) ## Variables in common

  if(all(c("report", "Date") %in% nms1) && length(index) == 1){
    stop("'report' and 'Date' found in first data frame but index length is equal to 1.
         Maybe index should be c('report', 'Date')?", call. = FALSE)
  }
  
  if(all(c("outfile", "Date") %in% nms1) && length(index) == 1){
    stop("'outfile' and 'Date' found in first data frame but index length is equal to 1.
         Maybe index should be c('outfile', 'Date')?", call. = FALSE)
  }
  
  if(all(c("SimulationName", "Date") %in% nms1) && length(index) == 1){
    stop("'SimulationName' and 'Date' found in first data frame but index length is equal to 1.
         Maybe index should be c('SimulationName', 'Date')?", call. = FALSE)
  }

  if(length(index) == 1){
    if(length(nms1.i) < 2) 
      stop("No names in common between the data.frames")    
  }else{
    if(length(nms1.i) < 3) 
      stop("No names in common between the data.frames")    
  }

  ## The line below drops irrelevant columns and brings index to the first column
  ##out1 <- subset(out1, select = c(index, nms1.i[-which(nms1 %in% index)]))
  out1 <- subset(out1, select = c(index, setdiff(nms1.i, index)))

  ### Drop character or factor variables which are not in index
  which.var.not.numeric <- sapply(subset(out1, select = setdiff(nms1.i, index)), is.numeric) 
  ### Remove non-numeric variables and throw warning if verbose
  if(any(isFALSE(which.var.not.numeric))){
    out1 <- out1[, -c(which(which.var.not.numeric == 0) + length(index))]
    if(verbose) warning("Removed non-numeric non-index variables from first data.frame")
  }
  
  new.nms1 <- paste0(names(out1), ".1") ## This simply adds a 1
  if(length(index) == 1){
    out1.new.names <- gsub(paste0(index, ".1"), index, new.nms1) ## Rename Date.1 to Date  
  }else{
    out1.new.names <- gsub(paste0(index[1], ".1"), index[1], new.nms1) ## Rename report.1 to report
    out1.new.names <- gsub(paste0(index[2], ".1"), index[2], out1.new.names) ## Rename Date.1 to Date
  }
  
  out.mrg <- stats::setNames(out1, out1.new.names) 
  
  for(i in 2:n.outs){
    
    ## This selects only those columns present in the first object
    out.i <- subset(outs[[i]], 
                    select = c(index, nms1.i[-which(nms1.i %in% index)])) 
    nms.i <- names(out.i)
    new.nms.i <- paste0(nms.i, ".", i)
    if(length(index) == 1){
      names(out.i) <- gsub(paste0(index, ".", i), index, new.nms.i)  
    }else{
      names.out.i <- gsub(paste0(index[1], ".", i), index[1], new.nms.i)  
      names(out.i) <- gsub(paste0(index[2], ".", i), index[2], names.out.i)  
    }
    
    out.mrg <- merge(out.mrg, out.i, by = index)
    
    if(nrow(out.mrg) < 1){
      stop("No data in common between data frames")
    }
  }

  if(length(index) == 1){
    ans.out.length <- ifelse(missing(variable), length(nms1.i) - 1, 1)  
  }else{
    ans.out.length <- ifelse(missing(variable), length(nms1.i) - 2, 1)  
  }
  
  ans <- data.frame(variable = rep(NA, ans.out.length), vs = NA, labels = NA, bias = NA, mean.bias = NA,
                    rss = NA, rmse = NA, concorr = NA, mod.eff = NA, corr = NA,
                    intercept = NA, slope = NA, rsigma = NA, r2 = NA)
  k <- 1
  ## Calculate statistics for all variables
  resid.matrix <- NULL
  crss <- NA
  
  if(missing(variable)){
    var.sel <- setdiff(nms1.i, index) ## All variables except index
    gvar.sel <- paste0(var.sel, collapse = "|")
    idx.out.mrg <- grep(gvar.sel, names(out.mrg))
    out.mrg.s <- out.mrg[, idx.out.mrg]
    ## Compute Bias matrix
    for(i in var.sel){
      if(verbose) cat("Variable: ", i, "\n")
      tmp <- out.mrg.s[, grep(i, names(out.mrg.s))]
      if(ncol(tmp) < 2) stop("merged selected variables should be at least of length 2", call. = FALSE)
      if(ncol(tmp) > 2){
        ### This means that the variable matched more than one target in the dataframes
        ### I need an exact match
        wvm1 <- which(paste0(i, ".1") == names(out.mrg.s))
        wvm2 <- which(paste0(i, ".2") == names(out.mrg.s))
        tmp <- out.mrg.s[, c(wvm1, wvm2)]
      }
      ans$variable[k] <- i
   
      for(j in 2:ncol(tmp)){
        if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
        ans$vs[k] <- c(paste(names(tmp)[j - 1], "vs.", names(tmp)[j]))
        if(!missing(labels)){
          if(verbose) cat("labels:", labels[j - 1], " vs. ", labels[j], "\n")
          ans$labels[k] <- paste(labels[j - 1], " vs. ", labels[j])
        }
        fm0 <- lm(tmp[, j] ~ tmp[, j - 1], na.action = "na.exclude")
        ### Store residuals
        resid.matrix <- cbind(resid.matrix, stats::residuals(fm0, type = "pearson"))
        #### First model-free statistics
        if(verbose) cat("Model-free statistics \n")
        if(verbose) cat(" \t Bias (sum of residuals): ", sum(tmp[, j - 1] - tmp[, j]), "\n")
        ans$bias[k] <- sum(tmp[, j - 1] - tmp[, j])
        if(verbose) cat(" \t Bias (mean of residuals): ", mean(tmp[, j - 1] - tmp[, j]), "\n")
        ans$mean.bias[k] <- mean(tmp[, j - 1] - tmp[, j])
        if(verbose) cat(" \t RSS: ", rss(tmp[,j - 1], tmp[, j]), "\n")
        ans$rss[k] <- rss(tmp[,j - 1], tmp[, j])
        if(verbose) cat(" \t RMSE: ", rmse(tmp[,j - 1], tmp[, j]), "\n")
        ans$rmse[k] <- rmse(tmp[,j - 1], tmp[, j])
        if(verbose) cat(" \t Corr: ", cor(tmp[, j - 1], tmp[, j]), "\n")
        ans$corr[k] <- cor(tmp[, j - 1], tmp[, j])
        if(verbose) cat(" \t Con. Corr: ", con_cor(tmp[, j - 1], tmp[, j]), "\n")
        ans$concorr[k] <- con_cor(tmp[, j - 1], tmp[, j])
        if(verbose) cat(" \t ME: ", mod_eff(tmp[, j - 1], tmp[, j]), "\n")
        ans$mod.eff[k] <- mod_eff(tmp[, j - 1], tmp[, j])
        #### Regression-based statistics
        if(verbose) cat("Regression-based statistics \n")
        if(verbose) cat(" \t Reg. Intercept: ", coef(fm0)[1], "\n")
        ans$intercept[k] <- coef(fm0)[1]
        if(verbose) cat(" \t Reg. Slope: ", coef(fm0)[2], "\n")
        ans$slope[k] <- coef(fm0)[2]
        if(verbose) cat(" \t Reg. RSS: ", deviance(fm0), "\n")
        ans$rss[k] <- deviance(fm0)
        if(verbose) cat(" \t Reg. sigma: ", sigma(fm0), "\n")
        ans$rsigma[k] <- sigma(fm0)
        if(verbose) cat(" \t Reg. R^2: ", summary(fm0)$r.squared, "\n")
        ans$r2[k] <- summary(fm0)$r.squared
      }
      k <- k + 1
    }
    #### Compute cRSS (combined Residual Sum of Squares) ----
    if(cRSS){
      sndout1 <- grep(".1", names(out.mrg), fixed = TRUE, value = TRUE)
      sndout2 <- grep(".2", names(out.mrg), fixed = TRUE, value = TRUE)
      dout1 <- subset(out.mrg, select = sndout1)
      dout2 <- subset(out.mrg, select = sndout2)
      diffs <- as.matrix(dout1) - as.matrix(dout2)
      if(missing(weights)) weights <- rep(1, ncol(diffs))
      crss <- sum(weights * colSums(diffs^2, na.rm = TRUE)) 
    }
    ### Need to assign names to residual matrix
    colnames(resid.matrix) <- var.sel
  }
  
  if(!missing(variable)){
    ## Just select the appropriate variable
    idx.out.mrg <- grep(variable, names(out.mrg))  
    out.mrg.s <- out.mrg[, idx.out.mrg]
    
    if(verbose) cat("Variable:", variable, "\n")
    ans$variable[k] <- i
    tmp <- out.mrg.s
    for(j in 2:ncol(tmp)){
      if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
      ans$vs[k] <- c(paste(names(tmp)[j - 1], "vs.", names(tmp)[j]))
      if(!missing(labels)){
        if(verbose) cat("labels", labels[j - 1], " vs. ", labels[j], "\n")
        ans$labels[k] <- paste(labels[j - 1], " vs. ", labels[j])
      }
      fm0 <- lm(tmp[, j - 1] ~ tmp[, j])
      ### Store residuals
      resid.matrix <- cbind(resid.matrix, stats::residuals(fm0, type = "pearson"))
      #### First model-free statistics
      if(verbose) cat("Model-free statistics \n")
      if(verbose) cat(" \t Bias (sum of residuals): ", sum(tmp[, j - 1] - tmp[, j]), "\n")
      ans$bias[k] <- sum(tmp[, j - 1] - tmp[, j])
      if(verbose) cat(" \t Bias (mean of residuals): ", mean(tmp[, j - 1] - tmp[, j]), "\n")
      ans$mean.bias[k] <- mean(tmp[, j - 1] - tmp[, j])
      if(verbose) cat(" \t RSS: ", rss(tmp[,j - 1], tmp[, j]), "\n")
      ans$rss[k] <- rss(tmp[,j - 1], tmp[, j])
      if(verbose) cat(" \t RMSE: ", rmse(tmp[,j - 1], tmp[, j]), "\n")
      ans$rmse[k] <- rmse(tmp[,j - 1], tmp[, j])
      if(verbose) cat(" \t Corr: ", cor(tmp[,j - 1], tmp[, j]), "\n")
      ans$corr[k] <- cor(tmp[, j - 1], tmp[, j])
      if(verbose) cat(" \t Con. Corr: ", con_cor(tmp[, j - 1], tmp[, j]), "\n")
      ans$concorr[k] <- con_cor(tmp[, j - 1], tmp[, j])
      if(verbose) cat(" \t ME: ", mod_eff(tmp[, j-1], tmp[, j]), "\n")
      ans$mod.eff[k] <- mod_eff(tmp[, j - 1], tmp[, j])
      #### Regression-based statistics
      if(verbose) cat("Regression-based statistics \n")
      if(verbose) cat(" \t Reg. Intercept: ", coef(fm0)[1], "\n")
      ans$intercept[k] <- coef(fm0)[1]
      if(verbose) cat(" \t Reg. Slope: ", coef(fm0)[2], "\n")
      ans$slope[k] <- coef(fm0)[2]
      if(verbose) cat(" \t Reg. RSS: ", deviance(fm0), "\n")
      ans$rss[k] <- deviance(fm0)
      if(verbose) cat(" \t Reg. sigma: ", sigma(fm0), "\n")
      ans$rsigma[k] <- sigma(fm0)
      if(verbose) cat(" \t Reg. R^2: ", summary(fm0)$r.squared, "\n")
      ans$r2[k] <- summary(fm0)$r.squared
    }
    clnm <- gsub(".1", "", grep(variable, names(out.mrg), value = TRUE)[1], fixed = TRUE) 
    colnames(resid.matrix) <- clnm
  }

  attr(out.mrg, "out.names") <- o.nms
  attr(out.mrg, "length.outs") <- n.outs
  attr(out.mrg, "index") <- index
  out.mrg <- structure(list(out.mrg = out.mrg, index.table = ans, cRSS = crss, resid.matrix = resid.matrix),
                       class = "out_mrg")
  invisible(out.mrg)
}

#' print method for out_mrg
#' @rdname compare_apsim
#' @description print method for \sQuote{out_mrg}
#' @param x object of class \sQuote{out_mrg}
#' @param ... additional arguments passed to print
#' @param digits digits to print (default is 2)
#' @export
#' @return it prints the index.table data.frame
print.out_mrg <- function(x, ..., digits = 2){
  print(x$index.table, digits = digits)
}

#' The by and facet arguments are only available for plot.type vs and ts for now
#' 
#' Plotting function for observed and simulated data
#' @rdname compare_apsim
#' @description plotting function for compare_apsim, it requires ggplot2
#' @param x object of class \sQuote{out_mrg}
#' @param ... data frames with APSIM output or observed data. 
#' @param plot.type either \sQuote{vs}, \sQuote{diff}, \sQuote{ts} - for time series or \sQuote{density}
#' @param pairs pair of objects to compare, defaults to 1 and 2 but others are possible
#' @param cumulative whether to plot cumulative values (default FALSE)
#' @param variable variable to plot 
#' @param id identification. Useful for finding extreme values. If this values is equal to 
#' 1 and no id.label is provided all observations are labeled by the row number. If it is 
#' less than one points are labeled if their probability is equal or less than the id value.
#' For example, a value of 0.05 will label values that have a probability of 0.05 (or less)
#' under a normal distribution.
#' @param id.label optional label for the id
#' @param by variable in \sQuote{index} used for plotting
#' @param facet whether to facet or use color for the by variable (default is FALSE, meaning \sQuote{color})
#' @param span argument passed to \sQuote{geom_smooth}
#' @param dodge.width optional argument to control the \sQuote{dodge} for the \sQuote{id.label}
#' @return it produces a plot
#' @export
#' 
plot.out_mrg <- function(x, ..., plot.type = c("vs", "diff", "resid", "ts", "density"),
                         pairs = c(1, 2),
                         cumulative = FALSE, ## Might not make sense for these type of graphs...
                         variable,
                         id,
                         id.label,
                         by,
                         facet = FALSE,
                         span = 0.75,
                         dodge.width = NULL){
  
  if(!requireNamespace("ggplot2", quietly = TRUE)){
    warning("ggplot2 is required for this plotting function")
    return(NULL)
  }
  
  x.resid <- x$resid.matrix
  colnames(x.resid) <- paste0("residuals.", colnames(x.resid))
  x <- x$out.mrg
  
  o.nms <- attr(x, "out.names")
  if(max(pairs) > attr(x, "length.outs")) stop("pairs index larger than length of outs")
  
  index <- attr(x, "index") ## This is normally 'Date'
  x <- as.data.frame(unclass(x))

  if(!missing(by) && length(index) == 1)
    stop("'by' is only available when index length is equal to 2.")
  
  if(missing(variable)){
    vnms <- setdiff(names(x), index)[1]
    variable <- gsub(".1", "", vnms, fixed = TRUE)
  }else{
    variable <- gsub(".1", "", grep(variable, names(x), value = TRUE)[1], fixed = TRUE)
  } 
  
  if(is.na(variable) || length(variable) == 0) stop("variable not found")
  
  ### If I'm guaranteed to have a variable then I need to subset residuals
  x.resid <- x.resid[, grep(variable, colnames(x.resid), value = TRUE), drop = FALSE]
  
  plot.type <- match.arg(plot.type)
  
  if(cumulative && plot.type != "ts") 
    stop("cumulative is only available for plot.type = 'ts' ")
  
  ### id and label ----
  ### If id and label are missing
  if(missing(id) && missing(id.label)){
    id.lbs <- rep("", nrow(x))
    if(is.null(dodge.width)) dodge.width <- 0
  }
    
  if(missing(id) && !missing(id.label))
    stop("'id' should be specified if 'id.label' is not missing", call. = FALSE)
  
  ### I should be able to add the label to all plots and not need the
  ### duplication
  prs0 <- paste0(variable, ".", pairs)
  prs <- paste0(prs0, collapse = "|")
  tmp <- x[, grep(prs, names(x))]
  ## Add residuals
  tmpr <- cbind(tmp, x.resid)
  if(!missing(id)){
    if(is.null(dodge.width)){
      dodge.width <- diff(range(tmp[[prs0[1]]])) * (1/10)  
    }
    if(missing(id.label) && id == 1){
      id.lbs <- seq_len(nrow(tmp))
    }
    if(missing(id.label) && id < 1){
      ### Probability of residuals under the normal distribution
      p.vls <- stats::pnorm(-abs(x.resid))
      w.pvls <- !c(p.vls <= id)
      id.lbs <- seq_len(nrow(tmp))
      id.lbs[w.pvls] <- ""
    }
    if(!missing(id.label) && id == 1){
      if(length(id.label) != nrow(tmp))
        stop("length of id.label whould be equal to the number of observations", call. = FALSE)
      id.lbs <- id.label
    }
    if(!missing(id.label) && id < 1){
      if(length(id.label) != nrow(tmp))
        stop("length of id.label should be equal to the number of observations", call. = FALSE)
      id.lbs <- id.label
      p.vls <- stats::pnorm(-abs(x.resid))
      w.pvls <- !c(p.vls <= id)
      id.lbs[w.pvls] <- ""
    }    
  }

  #### plot.type = "vs" ----
  if(plot.type == "vs" && !cumulative && (length(index) == 1 || missing(by))){
    tmp <- x[, grep(variable, names(x))]
    prs <- paste0(variable, ".", pairs)
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs[1]))), 
                                                    y = eval(parse(text = eval(prs[2]))))) +
      ggplot2::geom_point() + 
      ggplot2::xlab(paste(o.nms[pairs[1]], prs[1])) + 
      ggplot2::ylab(paste(o.nms[pairs[2]], prs[2])) + 
      ggplot2::geom_smooth(method = "lm", ...) + 
      ggplot2::geom_abline(intercept = 0, slope = 1, color = "orange") + 
      ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                         position = ggplot2::position_dodge2(dodge.width))
    
    print(gp1)
    
  }
  
  if(plot.type == "vs" && !cumulative && length(index) == 2 && !missing(by)){
    if(!facet){
      tmp <- x[, c(by, grep(variable, names(x), value = TRUE))]
      prs <- paste0(variable, ".", pairs)
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs[1]))), 
                                                      y = eval(parse(text = eval(prs[2]))),
                                                      color = eval(parse(text = eval(by))))) +
        ggplot2::geom_point() + 
        ggplot2::xlab(paste(o.nms[pairs[1]], prs[1])) + 
        ggplot2::ylab(paste(o.nms[pairs[2]], prs[2])) + 
        ggplot2::geom_smooth(method = "lm", ...) + 
        ggplot2::geom_abline(intercept = 0, slope = 1, color = "orange") + 
        ggplot2::guides(color=ggplot2::guide_legend(title=by)) + 
        ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                           position = ggplot2::position_dodge2(dodge.width))
      
      print(gp1)      
    }else{
      tmp <- x[, c(by, grep(variable, names(x), value = TRUE))]
      prs <- paste0(variable, ".", pairs)
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs[1]))), 
                                                      y = eval(parse(text = eval(prs[2]))))) +
        ggplot2::facet_wrap(~eval(parse(text = eval(by)))) + 
        ggplot2::geom_point() + 
        ggplot2::xlab(paste(o.nms[pairs[1]], prs[1])) + 
        ggplot2::ylab(paste(o.nms[pairs[2]], prs[2])) + 
        ggplot2::geom_smooth(method = "lm", ...) + 
        ggplot2::geom_abline(intercept = 0, slope = 1, color = "orange") + 
        ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                           position = ggplot2::position_dodge2(dodge.width))
      
      print(gp1)
    }
  }
  
  #### plot.type = "diff" ----
  if(plot.type == "diff" && !cumulative){
    
    prs0 <- paste0(variable, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    
    ## x Variable is prs[1]
    ## y Variable is prs[2] - prs[1]
    dff <- tmp[,prs0[2]] - tmp[,prs0[1]]
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))), 
                                                    y = dff)) +
      ggplot2::geom_point() + 
      ggplot2::xlab(paste(o.nms[pairs[1]], prs0[1])) + 
      ggplot2::ylab(paste("Difference", prs0[2], "-", prs0[1])) + 
      ggplot2::geom_smooth(method = "lm", ...) + 
      ggplot2::geom_hline(yintercept = 0, color = "orange") + 
      ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                         position = ggplot2::position_dodge2(dodge.width))
    
    print(gp1)   
  }

  #### plot.type = "resid" ----
  if(plot.type == "resid" && !cumulative){
    
    if(is.null(dodge.width)) dodge.width <- 0
    
    resid.name <- grep("residuals", names(tmpr), value = TRUE)
    
    gp1 <- ggplot2::ggplot(data = tmpr, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))), 
                                                     y = eval(parse(text = eval(resid.name))))) +
      ggplot2::geom_point() + 
      ggplot2::xlab(paste(o.nms[pairs[1]], prs0[1])) + 
      ggplot2::ylab(paste("Pearson ", resid.name)) + 
      ggplot2::geom_smooth(method = "lm", ...) + 
      ggplot2::geom_hline(yintercept = 0, color = "orange") + 
      ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                           position = ggplot2::position_dodge2(dodge.width))
    print(gp1)   
  }
  
    
  if(plot.type == "ts" && !cumulative && (length(index) == 1 || missing(by))){
  
    prs0 <- paste0(variable, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    if(length(index) == 1){
      tmp[[index]] <- x[[index]] ## Put it back in - kinda dumb   
    }else{
      if(index[2] == 'Date'){
        tmp[[index[2]]] <- x[[index[2]]] ## Put it back in - kinda dumb   
      }else{
        stop("I have not implemented this yet", call. = FALSE)
      }
    }
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = .data[[index[1]]], 
                                                    y = eval(parse(text = eval(prs0[1]))),
                                                    color = paste(o.nms[pairs[1]], prs0[1]))) +
      
      ggplot2::geom_point() + 
      ggplot2::geom_smooth(span = span, ...) + 
      ggplot2::geom_point(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                       color = paste(o.nms[pairs[2]], prs0[2]))) + 
      ggplot2::geom_smooth(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                        color = paste(o.nms[pairs[2]], prs0[2])),
                           span = span, ...) + 
      ggplot2::xlab(index) + 
      ggplot2::ylab(variable) + 
      ggplot2::guides(color=ggplot2::guide_legend(title="")) + 
      ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                         position = ggplot2::position_dodge2(dodge.width))
    
    print(gp1)   
  }

  
  #### plot.type = "ts" ----
  if(plot.type == "ts" && !cumulative && length(index) == 2 && !missing(by)){
    
    if(facet){
      prs0 <- paste0(variable, ".", pairs)
      prs <- paste0(prs0, collapse = "|")
      tmp <- x[, c(index, grep(prs, names(x), value = TRUE))]

      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = .data[[index[2]]], 
                                                      y = eval(parse(text = eval(prs0[1]))),
                                                      color = paste(o.nms[pairs[1]], prs0[1]))) +
        ggplot2::facet_wrap(~eval(parse(text = eval(by)))) + 
        ggplot2::geom_point() + 
        ggplot2::geom_smooth(span = span, ...) + 
        ggplot2::geom_point(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                         color = paste(o.nms[pairs[2]], prs0[2]))) + 
        ggplot2::geom_smooth(ggplot2::aes(y = eval(parse(text = eval(prs0[2]))),
                                          color = paste(o.nms[pairs[2]], prs0[2])),
                             span = span, ...) + 
        ggplot2::xlab(index[2]) + 
        ggplot2::ylab(variable) + 
        ggplot2::guides(color=ggplot2::guide_legend(title="")) + 
        ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                           position = ggplot2::position_dodge2(dodge.width))
      
      print(gp1)         
    }else{
      stop("Set 'facet' to TRUE for this type of graph", call. = FALSE)
    }

  }
    
  if(plot.type == "ts" && cumulative){
    
    if(length(index) != 1)
      stop("I have only implemented this when 'index' length is equal to 1")
    
    prs0 <- paste0(variable, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    tmp[[index]] <- x[[index]]
    tmp$cum_var1 <- cumsum(tmp[, prs0[1]])
    tmp$cum_var2 <- cumsum(tmp[, prs0[2]])
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = .data[[index]], 
                                                    y = .data[["cum_var1"]],
                                                    color = paste(o.nms[pairs[1]], prs0[1]))) +
      
      ggplot2::geom_line() + 
      ggplot2::geom_line(ggplot2::aes(y = .data[["cum_var2"]],
                                      color = paste(o.nms[pairs[2]], prs0[2]))) + 
      ggplot2::xlab(index) + 
      ggplot2::ylab(paste("Cumulative ", variable)) + 
      ggplot2::theme(legend.title = ggplot2::element_blank()) + 
      ggplot2::geom_text(ggplot2::aes(label = id.lbs),
                         position = ggplot2::position_dodge2(dodge.width))
    
    print(gp1)   
  }
  
  if(plot.type == "density"){
    
    prs0 <- paste0(variable, ".", pairs)
    prs <- paste0(prs0, collapse = "|")
    tmp <- x[, grep(prs, names(x))]
    
    gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))),
                                                    color = paste(o.nms[pairs[1]], prs0[1]))) + 
      ggplot2::geom_density() + 
      ggplot2::geom_density(ggplot2::aes(x = eval(parse(text = eval(prs0[2]))),
                                         color = paste(o.nms[pairs[2]], prs0[2]))) +
      ggplot2::xlab(variable) + 
      ggplot2::theme(legend.title = ggplot2::element_blank())
    
    print(gp1)
  }
  invisible(gp1)
}

## Model efficiency
## https://en.wikipedia.org/wiki/Nash%E2%80%93Sutcliffe_model_efficiency_coefficient
mod_eff <- function(x, y){
  sse <- sum((x - y)^2)
  ssx <- sum((x - mean(x))^2)
  ans <- 1 - sse / ssx
  ans
}

## Concordance correlation
## https://en.wikipedia.org/wiki/Concordance_correlation_coefficient
con_cor <- function(x, y){
  num <- 2 * cor(x, y) * sd(x) * sd(y)
  den <- var(x) + var(y) + (mean(x) - mean(y))^2
  ans <- num/den
  ans
}

## RMSE
## https://en.wikipedia.org/wiki/Root_mean_square_deviation
rmse <- function(x, y){
  n <- length(x)
  ssd <- sum((x - y)^2)
  ans <- sqrt((1/n) * ssd)
  ans
}

## RSS
## https://en.wikipedia.org/wiki/Errors_and_residuals
rss <- function(x, y){
  ans <- sum((x - y)^2)
  ans
}

#' 
#'  Some of the conversions use pedotrasnfer equations from Saxton and Rawls.
#'  Soil Water Characteristic Estimates by Texture and Organic Matter for Hydrologic Solutions.
#'  Soil Sci. Soc. Am. J. 70:1569–1578 (2006). 
#' 
#' @title Take in \acronym{SSURGO} csv files and create a soil profile
#' @name ssurgo2sp
#' @description Utility function to convert \acronym{SSURGO} data to soil profile
#' @param mapunit mapunit \acronym{SSURGO} file
#' @param component component \acronym{SSURGO} file
#' @param chorizon chorizon \acronym{SSURGO} file
#' @param mapunit.shp mapunit shapefile for creating metadata
#' @param nmapunit number of mapunits to select
#' @param nsoil number of soil components (within a mapunit) to consider
#' @param xout vector for interpolation and extrapolation
#' @param soil.bottom bottom of the soil profile
#' @param method method used for interpolation (see \code{\link{approx}})
#' @param nlayers number of soil layers to generate
#' @param carbon.to.om conversion factor from carbon to organic matter (default is 1.724). This is known as the \sQuote{von Bemmelen factor}. 
#' @param pedotransfer.method \sQuote{Saxton-Rawls}. Only option for now.
#' @param verbose whether to print details of the process
#' @return a list with soil profile matrices with length equal to nsoil
#' @details Download the data from \acronym{SSURGO} using the \sQuote{FedData} package \cr
#' This will generate csv files \sQuote{chorizon}, \sQuote{component} and \sQuote{mapunit}, \cr
#' but also many other files which are not needed for creating a soil profile.
#' @export
#' @examples 
#' \donttest{
#' require(ggplot2)
#' require(sf)
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' chorizon <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_chorizon.csv"))
#' component <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_component.csv"))
#' mapunit <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_mapunit.csv"))
#' mapunit.shp <- st_read(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_Mapunits.shp"), quiet = TRUE)
#' 
#' ## Using default 'constant' method
#' sp.c <- ssurgo2sp(mapunit = mapunit, 
#'                  component = component, 
#'                  chorizon = chorizon, 
#'                  mapunit.shp = mapunit.shp)
#' 
#' sp.c <- sp.c[[1]]
#' 
#' ggplot(data = sp.c, aes(y = -Depth, x = Carbon)) + 
#' geom_point() + 
#'   geom_path() + 
#'   ylab("Soil Depth (cm)") + xlab("Organic Matter (percent)") +
#'   ggtitle("method = constant")
#'   
#'   
#' ## Using 'linear' method
#' sp.l <- ssurgo2sp(mapunit = mapunit, 
#'                  component = component, 
#'                  chorizon = chorizon, 
#'                  mapunit.shp = mapunit.shp,
#'                  method = "linear")
#'                  
#' sp.l <- sp.l[[1]]
#'                  
#' ggplot(data = sp.l, aes(y = -Depth, x = Carbon)) + 
#' geom_point() + 
#'   geom_path() + 
#'   ylab("Soil Depth (cm)") + xlab("Organic Matter (percent)") +
#'   ggtitle("Method linear")
#' }
#' 
#' \dontrun{
#' ## Method using get_ssurgo_tables
#' 
#' require(soilDB)
#' require(sp)
#' require(sf)
#' require(spData)
#' ## retrieve data from lon -93, lat = 42
#' stbls <- get_ssurgo_tables(lonlat = c(-93, 42))
#' 
#' sp2.c <- ssurgo2sp(mapunit = stbls$mapunit, 
#'                  component = stbls$component, 
#'                  chorizon = stbls$chorizon, 
#'                  mapunit.shp = stbls$mapunit.shp)  
#' names(sp2.c)
#' 
#' metadata <- attributes(sp2.c[[1]])
#' metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL
#' 
#' ## Convert to an APSIM soil profile
#' asp2.c <- apsimx_soil_profile(nlayers = 10,
#'                               Thickness = sp2.c[[1]]$Thickness * 10,
#'                               BD = sp2.c[[1]]$BD,
#'                               AirDry = sp2.c[[1]]$AirDry,
#'                               LL15 = sp2.c[[1]]$LL15,
#'                               DUL = sp2.c[[1]]$DUL,
#'                               SAT = sp2.c[[1]]$SAT,
#'                               KS = sp2.c[[1]]$KS,
#'                               Carbon = sp2.c[[1]]$Carbon,
#'                               PH = sp2.c[[1]]$PH,
#'                               ParticleSizeClay = sp2.c[[1]]$ParticleSizeClay,
#'                               ParticleSizeSilt = sp2.c[[1]]$ParticleSizeSilt,
#'                               ParticleSizeSand = sp2.c[[1]]$ParticleSizeSand,
#'                               metadata = metadata)
#'                               
#' plot(asp2.c)
#' plot(asp2.c, property = "water")
#' 
#' }

ssurgo2sp <- function(mapunit = NULL, component = NULL, 
                      chorizon = NULL, mapunit.shp = NULL,
                      nmapunit = 1, nsoil = 1,
                      xout = NULL, soil.bottom = 200,
                      method = c("constant","linear"),
                      nlayers = 10,
                      carbon.to.om = 1.724,
                      pedotransfer.method = c("Saxton-Rawls"),
                      verbose = FALSE){
  
  if(!requireNamespace("sf", quietly = TRUE)){
    warning("sf package is required for this function")
    return(NULL)
  }
  
  DUL.method <- "ssurgo"
  LL15.method <- "ssurgo"
  SAT.method <- "ssurgo"
  KS.method <- "ssurgo"
  ## Select important variables from mapunit
  mapunit2 <- subset(mapunit, 
                     select = c("musym", "muname", "muacres", "farmlndcl", 
                                "lkey", "mukey"))
  ## Calculate acres percent and number of mapunits to consider
  ## In decreasing order of number of acres
  mapunit2$muacres.percent <- mapunit2$muacres/sum(mapunit2$muacres) * 100
  mapunit2 <- mapunit2[order(mapunit2$muacres, decreasing = TRUE),]
  if(nmapunit < 1 || is.na(nmapunit)){
    mapunit3 <- mapunit2
  }else{
    if(nmapunit > nrow(mapunit2))
      stop(paste("There are", nrow(mapunit2), "mapunits available and ", nmapunit, "are requested"), call. = FALSE)
    mapunit3 <- mapunit2[seq_len(nmapunit),]    
  }

  ## Process component
  
  component2 <- subset(component, 
                       select = c("compname", "comppct.r",
                                  "slope.r", "drainagecl",
                                  "elev.r", "taxsubgrp", 
                                  "taxpartsize", "taxclname", "geomdesc",
                                  "mukey", "cokey"))
  
  if(any(grepl("state", names(component)))) component2$state <- component$state
    
  component5 <- NULL
  for(i in mapunit3$mukey){
    component3 <- component2[component2$mukey == i,]
    component4 <- component3[order(component3$comppct.r, decreasing = TRUE),]
    ## This calculates the percent of the total acres in the area of interest
    component4$acres.proportion <- mapunit3[mapunit3$mukey == i,]$muacres.percent/100 * component4$comppct.r/100
    component4$compname.mukey <- as.factor(component4$compname):as.factor(component4$mukey)
    
    if(inherits(mapunit.shp, "sf")){
      lonlat <- colMeans(sf::st_coordinates(mapunit.shp["MUKEY" == i]))  
      mapunit.shp.d <- as.data.frame(mapunit.shp)
      if(any(grepl("AREASYMBOL", names(mapunit.shp.d)))){
        component4$state <- rep(unique(strtrim(as.character(mapunit.shp.d[mapunit.shp.d$MUKEY == i,"AREASYMBOL"]),2)),nrow(component4))
      }
    }else{
      stop("mapunit.shp should be of class sf")
    }

    component4$longitude <- lonlat[1]
    component4$latitude <- lonlat[2]
    if(nsoil < 1 || is.na(nsoil)){
      component5 <- rbind(component5, component4)      
    }else{
      if(nsoil > nrow(component4)){
        stop(paste("There are", nrow(component4), "components (soils) available and", nsoil, "are requested"), call. = FALSE)
      }
      component5 <- rbind(component5, component4[seq_len(nsoil),])      
    }
  }
  
  ## Process chorizon
  chorizon2 <- chorizon[chorizon$cokey %in% component5$cokey,]
  if(nrow(chorizon2) < 1){
    stop("component horizon does not match component cokey")
  }
  chorizon3 <- subset(chorizon2, 
                      select = c("hzname", "hzdept.r", "hzdepb.r", 
                                 "hzthk.r", "sandtotal.r", "silttotal.r",
                                 "claytotal.r", "om.r", "partdensity", 
                                 "ksat.r", "awc.r", 
                                 "wthirdbar.r", "wfifteenbar.r",
                                 "wsatiated.r", "ph1to1h2o.r", "caco3.r",
                                 "cokey", "chkey"))

  ### This takes the middle point! Important to remember!  
  chorizon3$hzdepa.r <- (chorizon3$hzdept.r + chorizon3$hzdepb.r) / 2
  
  ## Rename to match APSIM
  chorizon3$Depth <- chorizon3$hzdepa.r
  
  if(any(is.na(chorizon3$hzthk.r))){
    chorizon3$Thickness <- chorizon3$hzdepb.r - chorizon3$hzdept.r
  }else{
    chorizon3$Thickness <- chorizon3$hzthk.r  
  }
  
  ## Use Saxton and Rawls to generate these based on texture
  if(any(is.na(chorizon3$wthirdbar.r))){
    if(verbose) message("'wthirdbar.r' is missing. Using 'SR' to estimate it.")
    DUL.method <- "Saxton-Rawls"
    chorizon3$DUL <- sr_dul(chorizon3$claytotal.r, chorizon3$sandtotal.r, chorizon3$om.r)
  }else{
    chorizon3$DUL <- chorizon3$wthirdbar.r * 1e-2 ## convert to fraction  
  }
  
  if(any(is.na(chorizon3$wfifteenbar.r))){
    if(verbose) message("'wfifteenbar.r' is missing. Using 'SR' to estimate it.")
    LL15.method <- "Saxton-Rawls"
    chorizon3$LL15 <- sr_ll(chorizon3$claytotal.r, chorizon3$sandtotal.r, chorizon3$om.r)
  }else{
    chorizon3$LL15 <- chorizon3$wfifteenbar.r * 1e-2 ## convert to fraction  
  }
  
  if(any(is.na(chorizon3$wsatiated.r))){
    if(verbose) message("'wsatiated.r' is missing. Using 'SR' to estimate it.")
    SAT.method <- "Saxton-Rawls"
    DUL_S <- sr_dul_s(chorizon3$claytotal.r, chorizon3$sandtotal.r, chorizon3$om.r)
    chorizon3$SAT <- sr_sat(chorizon3$sandtotal.r, chorizon3$DUL, DUL_S)
  }else{
    chorizon3$SAT <- chorizon3$wsatiated.r * 1e-2 ## convert to fraction  
  }
  
  ## Convert micro meters per second to mm/day
  if(any(is.na(chorizon3$ksat.r))){
    if(verbose) message("'ksat.r' is missing. Need to estimate it?")
  }else{
    chorizon3$KS <- chorizon3$ksat.r * 1e-6 * (60 * 60 * 24) * 1e3 
  }
  
  if(all(is.na(chorizon3$ph1to1h2o.r))){
    if(verbose) message("'ph1to1h2o.r' is missing. Need to estimate it?")    
  }else{
    if(any(is.na(chorizon$ph1to1h2o.r))){
      if(verbose) message("'ph1to1h2o.r' is partially missing. Need to estimate it?")    
    }
    chorizon3$PH <- chorizon3$ph1to1h2o.r
  }
  
  if(all(is.na(chorizon3$caco3.r))){
    if(verbose) message("'caco3.r' is missing. Need to estimate it?")    
  }else{
    if(any(is.na(chorizon$caco3.r))){
      if(verbose) message("'caco3.r' is partially missing. Need to estimate it?")    
    }
    chorizon3$CACO3 <- chorizon3$caco3.r
  }
  
  ## From Saxton and Rawls
  chorizon3$BD <- (1 - chorizon3$SAT) * ifelse(is.na(chorizon3$partdensity), 2.65, chorizon3$partdensity)
  
  ## Convert to fraction
  chorizon3$AirDry <- chorizon3$LL15 * ifelse(chorizon3$hzdept.r == 0, 0.5, 1)
  
  ## Soil Carbon
  ## SOM contains approximately 58% C; therefore, a factor of
  ## 1.72 can be used to convert OC to SOM.
  ## This is known as the 'von Bemmelen' factor, but it is known
  ## that it is not constant. A value of 2 might be better.
  ## Douglas W. Pribyl,
  ## A critical review of the conventional SOC to SOM conversion factor,
  ## Geoderma, Volume 156, Issues 3–4, 2010, Pages 75-83, ISSN 0016-7061,
  ## https://doi.org/10.1016/j.geoderma.2010.02.003.
  if(all(is.na(chorizon3$om.r))){
    if(verbose) message("'om.r' is missing. Need to estimate it?")    
  }else{
    if(any(is.na(chorizon$om.r))){
      if(verbose) message("'om.r' is partially missing. Need to estimate it?")    
    }
    chorizon3$Carbon <- chorizon3$om.r * (1/carbon.to.om)
  }
  
  ### Adding texture
  if(any(is.na(chorizon3$ParticleSizeClay))){
    if(verbose) message("'ParticleSizeClay' is missing. Need to estimate it?")    
  }
  if(any(is.na(chorizon3$ParticleSizeSilt))){
    if(verbose) message("'ParticleSizeSilt' is missing. Need to estimate it?")    
  }
  if(any(is.na(chorizon3$ParticleSizeSand))){
    if(verbose) message("'ParticleSizeSand' is missing. Need to estimate it?")    
  }

  chorizon3$ParticleSizeClay <- chorizon3$claytotal.r
  chorizon3$ParticleSizeSilt <- chorizon3$silttotal.r
  chorizon3$ParticleSizeSand <- chorizon3$sandtotal.r
  
  ## Soil names
  soil.names <- component5$compname.mukey
  soil.list <- vector(mode = "list", length = length(soil.names))
  names(soil.list) <- soil.names
  
  ### Should I add carbonates here? Are they available sometimes?
  vars <- c("Depth", "Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS", 
            "Carbon", "PH", "CACO3", "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand")
  
  for(sz in 1:length(soil.names)){
    
    one.soil <- chorizon3[chorizon3$cokey == component5$cokey[sz],]
    
    if(is.na(soil.bottom)) soil.bottom <- max(one.soil$hzdepb.r)
    
    if(nrow(one.soil) < 1){
      stop("There is no soil horizon for this component")
    }
  
    soil.mat <- matrix(nrow = nlayers, ncol = length(vars))
  
    nlayers0 <- nlayers
    nlayers1 <- nlayers + 1
  
    for(i in 2:length(vars)){

      if(is.na(match(vars[i], names(one.soil)))){
        message(paste(names(one.soil), collapse = " "))
        stop(paste("soil variable:", vars[i], "not found in soil table"))
      }
      tmp <- one.soil[,c("Depth",vars[i])]
    
      nlayers <- ifelse(vars[i] == "Thickness", nlayers1, nlayers0)
    
      if(verbose) cat("Processing variable:", vars[i], "\n")
      
      sva <- try(approx_soil_variable(tmp, 
                                      xout = xout, 
                                      soil.bottom = soil.bottom,
                                      method = method, 
                                      nlayers = nlayers), silent = TRUE)
      
      if(inherits(sva, "try-error")){
        print(one.soil)
        print(tmp)
        stop("interpolation did not work. Possibly due to missing values.")
      }

      if(vars[i] == "Thickness"){
        thck <- numeric(nlayers0)
        for(j in 2:nlayers1) thck[j-1] <- sva$x[j] - sva$x[j-1] 
          soil.mat[,i] <- thck
      }else{
        soil.mat[,i] <- sva$y
      }
    }
    soil.mat[,1] <- sva$x
    soil.d <- as.data.frame(soil.mat)

    ## Attributes will be modeled after APSIM metadata
    attr(soil.d, which = "SoilType") <- as.character(soil.names[sz])
    attr(soil.d, which = "State") <- component5$state[sz]
    attr(soil.d, which = "Country") <- "USA"
    attr(soil.d, which = "Longitude") <- component5$longitude[1]
    attr(soil.d, which = "Latitude") <- component5$latitude[1]
    attr(soil.d, which = "DataSource") <- paste("R package apsimx function ssurgo2sp. Timestamp", Sys.time())
    attr(soil.d, which = "Comments") <- paste("cokey =", component5$cokey[sz],
                                              "- acres percent =", component5$acres.proportion[sz] * 100,
                                              "- component percent =", component5$comppct.r[sz],
                                              "- taxonomic classification name =", as.character(component5$taxclname)[sz],
                                              "- drainage class =", as.character(component5$drainagecl)[sz],
                                              "- elevation =", component5$elev.r[sz],
                                              "- slope =", component5$slope.r[sz],
                                              "- geomdesc =", as.character(component5$geomdesc)[sz],
                                              "- DUL.method =", DUL.method,
                                              "- LL15.method =", LL15.method,
                                              "- SAT.method =", SAT.method,
                                              "- KS.method =", KS.method)
    names(soil.d) <- vars
    ## Store in list
    soil.list[[sz]] <- soil.d
  }

  return(soil.list)
}

#' Interpolate a soil variable using 'approx' and return a data.frame
#' @name approx_soil_variable
#' @description interpolate a soil variable over a range of values
#' @param x input vector
#' @param xout output vector (see stats::approx)
#' @param soil.bottom bottom of the soil layer in cm
#' @param method either 'constant' or 'linear' as used in 'stats::approx'
#' @param nlayers number of soil layers
#' @return interpolated numeric vector as a data frame.
#' @noRd
#' 
approx_soil_variable <- function(x, xout = NULL, soil.bottom = 200, 
                                 method = c("constant", "linear"),
                                 nlayers = 10){
  
  xd <- as.data.frame(x)
  method <- match.arg(method)
  if(is.null(xout)) xout <- seq(0, soil.bottom, length.out = nlayers)
  
  ans <- stats::approx(x = xd[[1]], y = xd[[2]], method = method, xout = xout, rule = 2)
  
  ansd <- data.frame(x = ans$x, y = ans$y)
  return(ansd)
}

### NOTE: This test passed on Jun 22 2026 with package version 
library(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE, warn.find.apsimx = FALSE)

run.apsimx.path.spaces <- get(".run.local.tests", envir = apsimx.options)

if(run.apsimx.path.spaces){
  
  #### Allow all examples to be run with spaces
  tmp.dir <- tempdir()
  ex.dir <- auto_detect_apsimx_examples()
  dir.create(file.path(tmp.dir, "examples from apsim"))
  
  examples <- c("AgPasture", "Barley", "Canola", "Chickpea", 
                "ControlledEnvironment", "Eucalyptus",
                "EucalyptusRotation",
                "Maize", "Mungbean",
                "Oats", "OilPalm", "Peanut", "Pinus", "Potato",
                "RedClover",
                "Rotation", "Sorghum",
                "Soybean", "Sugarcane", "Wheat", "WhiteClover")
  
  for(i in examples){
    if(!file.exists(file.path(tmp.dir, "examples from apsim", paste0(i, ".apsimx"))))
      file.copy(from = file.path(ex.dir, paste0(i, ".apsimx")), 
                to = file.path(tmp.dir, "examples from apsim", paste0(i, ".apsimx")))
  }
  ## dir(include.dirs = TRUE, recursive = TRUE)
  for(i in examples){
    file.to.run <- paste0(i, ".apsimx")
    src.dir <- file.path(tmp.dir, "examples from apsim")
    cat("Running:", file.to.run)
    atmp <- apsimx(file = file.to.run, src.dir = src.dir, simplify = FALSE)
    cat("... Done ... \n")
  }
}

#'
#' The original data can be obtained from: (mesonet.agron.iastate.edu/iemre)
#'
#' @title Get weather data from Iowa Environmental Mesonet Reanalysis
#' @description Retrieves weather data from Iowa Environmental Mesonet Reanalysis into an APSIM met file
#' @name get_iemre_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param dates date ranges
#' @param wrt.dir write directory
#' @param filename file name for writing out to disk
#' @param fillin.radn whether to fill in radiation data using the nasapower pacakge. Default is FALSE.
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk.
#' @note Multi-year query is not supported for this product. 
#' @return returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @export
#' @examples 
#' \dontrun{
#' ## This will not write a file to disk
#' iemre <- get_iemre_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
#' ## Note that solar radiation is not available, but can be filled in
#' ## using the nasapower package
#' iemre2 <- get_iemre_apsim_met(lonlat = c(-93,42), 
#'                              dates = c("2012-01-01","2012-12-31"), 
#'                              fillin.radn = TRUE)
#' summary(iemre)
#' summary(iemre2)
#' 
#' ## Still it is important to check this object
#' check_apsim_met(iemre2)
#' }
#' 

get_iemre_apsim_met <- function(lonlat, dates, wrt.dir=".", filename=NULL,
                                fillin.radn = FALSE){
  
  if(missing(filename)) filename <- "noname.met"
  
  if(!grepl("met$", filename)) stop("filename should end in .met")
  
  str0 <- "https://mesonet.agron.iastate.edu/iemre/multiday/"
  
  ## Build date
  day1 <- as.character(as.Date(dates[1]))
  dayn <- as.character(as.Date(dates[2]))
  
  yr1 <- as.numeric(format(as.Date(dates[1]), "%Y"))
  yr2 <- as.numeric(format(as.Date(dates[2]), "%Y"))
  
  if(yr2 > yr1) stop("Multi-year queries are not supported at the moment. See the source.")
  
  ## Longitude and latitude
  lon <- as.numeric(lonlat[1])
  lat <- as.numeric(lonlat[2])
  
  str1 <- paste0(str0, day1, "/", dayn, "/", lat, "/", lon, "/json")
  
  iem.json <- jsonlite::fromJSON(str1)
  
  iem.dat <- as.data.frame(iem.json[[1]])
  
  iem.dat$year <- as.numeric(format(as.Date(iem.dat$date),"%Y"))
  
  iem.dat$radn <- NA
  
  ## Dates sequence
  dates.seq <- seq(from = as.Date(dates[1]), to = as.Date(dates[2]), by = "day")
  
  iem.dat$day <- as.numeric(format(dates.seq, "%j"))
  
  iem.dat$daily_high_c <- (iem.dat$daily_high_f - 32) * (5/9)
  iem.dat$daily_low_c <- (iem.dat$daily_low_f - 32) * (5/9)
  iem.dat$daily_precip_mm <- iem.dat$daily_precip_in * 25.4 
  
  iem.dat2 <- subset(iem.dat, select = c("year","day",
                                         "radn",
                                         "daily_high_c",
                                         "daily_low_c",
                                         "daily_precip_mm"))
  
  if(fillin.radn){
    if(!requireNamespace("nasapower", quietly = TRUE)){
      warning("The nasapower package is required for this function")
      return(NULL)
    }
    pwr <- get_power_apsim_met(lonlat = lonlat, 
                               dates = as.Date(c(paste0(yr1, "-01-01"), paste0(yr2, "-12-31"))))
    ##pwr$date <- as.Date(c(1:nrow(pwr)-1), origin = paste0(yr1,"-01-01"))
    pwr <- add_column_apsim_met(pwr, 
                                value = as.Date(c(1:nrow(pwr)-1), origin = paste0(yr1,"-01-01")),
                                name = "date", units = "()")
    pwr <- subset(pwr, date >= as.Date(day1) & date <= as.Date(dayn))
    
    if(nrow(iem.dat2) != nrow(pwr))
      stop("Something went wrong. Number of rows do not match.")
    
    iem.dat2$radn <- pwr$radn
  }
  
  names(iem.dat2) <- c("year","day","radn","maxt","mint","rain")
  units <- c("()","()","(MJ/m2/day)","(oC)","(oC)","(mm)")
  
  comments <- paste("!data from IEM Reanalysis. retrieved: ", Sys.time())
  
  attr(iem.dat2, "filename") <- filename
  attr(iem.dat2, "site") <- paste("site = ", sub(".met","", filename, fixed = TRUE))
  attr(iem.dat2, "latitude") <- paste("latitude =", lonlat[2])
  attr(iem.dat2, "longitude") <- paste("longitude =", lonlat[1])
  attr(iem.dat2, "tav") <- paste("tav =",mean(colMeans(iem.dat2[,c("maxt","mint")], na.rm=TRUE), na.rm=TRUE))
  attr(iem.dat2, "colnames") <- names(iem.dat2)
  attr(iem.dat2, "units") <- units
  attr(iem.dat2, "comments") <- comments
  ## No constants
  class(iem.dat2) <- c("met", "data.frame")
  
  iem.dat2 <- amp_apsim_met(iem.dat2)
  
  if(filename != "noname.met"){
    write_apsim_met(iem.dat2, wrt.dir = wrt.dir, filename = filename)
  }
  return(invisible(iem.dat2))
}


#'
#' The original data can be obtained from: (mesonet.agron.iastate.edu/request/coop)
#'
#' @title Get weather data from Iowa Environmental Ag Weather Stations
#' @description Retrieves weather data from Iowa Environmental Mesonet (AgWeather) into an APSIM met file
#' @name get_iem_apsim_met
#' @param lonlat Longitude and latitude vector (optional)
#' @param dates date ranges
#' @param wrt.dir write directory
#' @param state state which you choose climate data from
#' @param station station which you choose cliamte data from
#' @param filename file name for writing out to disk
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk. For this function either provide the longitude 
#' and latitude or the state and station, but not both. In fact, \sQuote{state} and
#' \sQuote{station} will be ignored if \sQuote{lonlat} is supplied.
#' @return returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @export
#' @examples 
#' \dontrun{
#' ## This will not write a file to disk
#' iem.met <- get_iem_apsim_met(state = "IA", 
#'                              station = "IA0200", 
#'                              dates = c("2012-01-01","2012-12-31"))
#' 
#' summary(iem.met)
#' 
#' ## Alternatively, coordinates can be used
#' ## This should be equivalent to the previous request
#' iem.met2 <- get_iem_apsim_met(lonlat = c(-93.77, 42.02), 
#'                               dates = c("2012-01-01","2012-12-31"))
#'
#' summary(iem.met2)
#' }
#' 

get_iem_apsim_met <- function(lonlat, dates, wrt.dir = ".", 
                              state, station, filename){
  
  if(missing(filename)) filename <- "noname.met"
  
  if(!grepl("met$", filename)) stop("filename should end in .met")
  
  str0 <- "http://mesonet.agron.iastate.edu/cgi-bin/request/coop.py?network="
  
  ## Build date
  day1 <- as.character(as.Date(dates[1]))
  dayn <- as.character(as.Date(dates[2]))
  yr1 <- format(as.Date(dates[1]), "%Y")
  yr2 <- format(as.Date(dates[2]), "%Y")
  mnth1 <- format(as.Date(dates[1]), "%m")
  mnth2 <- format(as.Date(dates[2]), "%m")
  ## dy1 <- format(as.Date(dates[1]), "%j") The format changed for the API (April 2026)
  ## dy2 <- format(as.Date(dates[2]), "%j")
  dy1 <- format(as.Date(dates[1]), "%d")
  dy2 <- format(as.Date(dates[2]), "%d")
  
  ## Longitude and latitude or State and Station
  stts <- datasets::state.abb
  
  if(!missing(lonlat)){
    lon <- as.numeric(lonlat[1])
    lat <- as.numeric(lonlat[2])
    
    if(!requireNamespace("sf",quietly = TRUE)){
      warning("The sf is required for this function")
      return(NULL)
    }
    
    if(!requireNamespace("spData",quietly = TRUE)){
      warning("The spData is required for this function")
      return(NULL)
    }
    
    ## Need to find the state based on lonlat
    pts <- sf::st_as_sf(data.frame(lon = lon, lat = lat), coords = 1:2, crs = 4326)
    
    states <- spData::us_states ## This object contains states and their geometries
    states <- sf::st_transform(states, crs = 3857)
    pts <- sf::st_transform(pts, crs = 3857)
    ii <- as.integer(sf::st_intersects(pts, states))
    
    stt0 <- states[["NAME"]][[ii]]
    state <- datasets::state.abb[which(stt0 == datasets::state.name)]
    stt.climate <- paste0(state, "CLIMATE")
    
    ## Now find the station
    ftrs <- jsonlite::fromJSON(paste0("http://mesonet.agron.iastate.edu/geojson/network.php?network=",stt.climate))$features
    
    ## Subset stations which include the dates request
    yr1 <- as.numeric(format(as.Date(dates[1]), "%Y"))
    yr2 <- as.numeric(format(as.Date(dates[2]), "%Y"))
    
    ## This chunk of code selects stations which have data in the desired range
    start.year <- sapply(ftrs$properties$time_domain, function(x) as.numeric(gsub("(", "", strsplit(x, "-")[[1]][1], fixed = TRUE)))
    end.year0 <- sapply(ftrs$properties$time_domain, function(x) gsub("(", "", strsplit(x, "-")[[1]][2], fixed = TRUE))
    end.year <- ifelse(end.year0 == "Now)", 
                       as.numeric(format(Sys.Date(), "%Y")), 
                       suppressWarnings(as.numeric(gsub(")", "", end.year0, fixed = TRUE))))
    
    wch.yr <- which(start.year <= yr1 & end.year >= yr2)
    ftrs <- ftrs[wch.yr,]
    ## end of time span selection
    
    geo.station <- ftrs$geometry$coordinates
    geo.station.lon <- sapply(geo.station, FUN = function(x) x[[1]])
    geo.station.lat <- sapply(geo.station, FUN = function(x) x[[2]])
    
    station.coords <- sf::st_as_sf(data.frame(lon = geo.station.lon, 
                                              lat = geo.station.lat,
                                              station = ftrs$id), 
                                   coords = c("lon","lat"), 
                                   crs = sf::st_crs(4326))
    station.coords <- sf::st_transform(station.coords, crs = 3857)
    near.station.index <- sf::st_nearest_feature(pts, station.coords)
    station <- ftrs[near.station.index, "id"]
    
    ## Compute distance between input point and station
    pt.to.station.distance <- sf::st_distance(pts, station.coords[near.station.index, ])
    
  }else{
  
    ## In this case, state and station should be given
    ## Get the state
    
    if(missing(state)) stop("state is missing with no default")
    
    if(!(state %in% stts)) stop("state must be incorrect")
    stt.climate <- paste0(state, "CLIMATE")
  
    ## Get the station
    ftrs <- jsonlite::fromJSON(paste0("http://mesonet.agron.iastate.edu/geojson/network.php?network=",stt.climate))$features
    if(!(station %in% ftrs$id)){
      cat("Available stations", ftrs$id, "\n")
      stop("station must be incorrect")
    } 
  }
   
  ## Build string
  str1 <- paste0(str0, stt.climate, "&stations=", station)
  str2 <- paste0(str1, "&year1=", yr1, "&month1=", mnth1, "&day1=", dy1,
                       "&year2=", yr2, "&month2=", mnth2, "&day2=", dy2)
  str3 <- paste0(str2, "&vars%5B%5D=apsim&what=view&delim=comma&gis=no")
  
  ## Retrieve data
  iem0 <- readLines(str3)
  write(iem0, file = file.path(wrt.dir, filename))
  iem.dat <- read_apsim_met(filename, src.dir = wrt.dir, verbose = FALSE)
  
  attr(iem.dat, "filename") <- paste0(state, "-", station, ".met")
  attr(iem.dat, "site") <- paste0("site = ", state, "-", station)
  
  if(!missing(lonlat)){
    attr(iem.dat, "longitude") <- paste("longitude =", lonlat[1], "(DECIMAL DEGREES)")
    ## Add distance as a comment
    attr(iem.dat, "comment") <- paste("Distance from station to input point (meters):", round(pt.to.station.distance))
  }
  
  if(missing(lonlat)){
    stt <- ftrs[ftrs$id == station, ]
    attr(iem.dat, "longitude") <- paste("longitude =", stt$geometry$coordinates[[1]][1], "(DECIMAL DEGREES)")
    attr(iem.dat, "latitude") <- paste("latitude =", stt$geometry$coordinates[[1]][2], "(DECIMAL DEGREES)")
  }
  
  if(nrow(iem.dat) == 0)
    warning("No weather data for this location and dates combination")
  
  if(filename == "noname.met"){
    unlink(file.path(wrt.dir, filename))
    return(iem.dat)
  }else{
    return(invisible(iem.dat))
  } 
}

## Testing initialwater with a variety of files
require(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

run.apsimx.initialwater <- get(".run.local.tests", envir = apsimx.options)

#### Testing InitialWater ----
if(run.apsimx.initialwater){
  
  tmp.dir <- tempdir()
  ex.dir <- auto_detect_apsimx_examples()
  ex.dir.list <- dir(ex.dir, pattern = "apsimx$")
  
  #### I will skip files that require 'root'
  ##ex.dir.list2 <- ex.dir.list[c(2, 4, 16:19, 23, 24, 29, 30, 34)]
  ex.dir.list3 <- c("Barley.apsimx", "Canola.apsimx", "Maize.apsimx", "Mungbean.apsimx", 
                     "Oats.apsimx", "OilPalm.apsimx", "Potato.apsimx", "RedClover.apsimx",
                     "Sorghum.apsimx", "Soybean.apsimx", "Wheat.apsimx")
  for(.i in ex.dir.list3){
    if(.i == "Slurp.apsimx") stop("This is bananas", call. = FALSE)
    cat("Simulation:", .i, "\n")
    #### 1) Inspect ----
    cat("Inspecting...\n")
    inspect_apsimx(.i, ex.dir, node = "Soil", soil.child = "InitialWater")
    pp <- inspect_apsimx(.i, ex.dir, node = "Soil", soil.child = "InitialWater", print.path = TRUE)
    #### 2) Extract ----
    cat("Extracting...\n")
    edf.iw <- extract_data_apsimx(.i, ex.dir, node = "Soil", soil.child = "InitialWater")
    #### 3) Edit ----
    cat("Editing...\n")
    if(!file.exists(file.path(tmp.dir, .i))) file.copy(from = file.path(ex.dir, .i), to = tmp.dir)
    if(!is.null(edf.iw$second) && nrow(edf.iw$second) > 0){
      initialvalues <- rep(0.3, nrow(edf.iw$second))
      edit_apsimx(.i, tmp.dir, node = "Soil", 
                  soil.child = "InitialWater",
                  parm = "InitialValues",
                  value = initialvalues)      
      #### 4) Check ----
      cat("Checking...\n")
      edited.file <- paste0(tools::file_path_sans_ext(.i), "-edited.apsimx")
      inspect_apsimx(edited.file, tmp.dir, node = "Soil", soil.child = "InitialWater")
      check_apsimx(edited.file, tmp.dir)
    }
    if(file.exists(file.path(tmp.dir, edited.file))) file.remove(file.path(tmp.dir, edited.file))
  }
}

#' Test whether an .apsimx file is XML or json
#' 
#' @title Test file format for .apsimx files
#' @name apsimx_filetype
#' @param file file ending in .apsimx to be tested
#' @param src.dir directory containing the .apsimx file to be tested; defaults to the current working directory
#' @return \sQuote{xml}, \sQuote{json} or \sQuote{unknown}
#' @note Minimal function which reads only the first line in a file and tries to guess whether it is
#'       an \sQuote{xml} or \sQuote{json} file type.
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' apsimx_filetype("Wheat.apsimx", src.dir = extd.dir) 
#' }
#' 

apsimx_filetype <- function(file = "", src.dir = "."){
  
  
  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
  file.names.c <- dir(path = src.dir, pattern=".apsim$", ignore.case=TRUE)
  ## print(file.names.c)
  # file.names <- c(file.names.x, file.names.c)
  
  if(length(file.names)==0){
    stop("There are no .apsimx files in the specified directory to test.")
  }
  
  file <- match.arg(file, file.names, several.ok=FALSE)
  
  rfl1 <- utils::read.table(file = file.path(src.dir, file), nrows = 1)[1,1]
  ## The next line will work if all XML files contain xml in the first line
  if(length(grep("xml", as.character(rfl1))) != 0){
    ans <- "xml"
  }else{
    if(length(grep("{", as.character(rfl1), fixed = TRUE)) == 0){
      ans <- "unkown"
    }else{
      ans <- "json"
    }
  }
  return(ans)
}

#' Data source is USDA-NRCS Soil Data Access. See package soilDB for more details
#' 
#' @title Retrieve soil data and convert it to a grid object of class \sQuote{soil_profile_grid}
#' @description Generate a synthetic soil profile grid based on the Web Coverage Service from soilDB package
#' @name get_wcs_soil_profile_grid
#' @param aoi area of interest compatible with \sQuote{mukey.wcs}
#' @param db soil database to use (default is gSSURGO)
#' @param soil.bottom see \code{\link{ssurgo2sp}}
#' @param method interpolation method see \code{\link{ssurgo2sp}}
#' @param nlayers number for layer for the new soil profile
#' @param check whether to check for reasonable values using \code{\link{check_apsimx_soil_profile}}.
#' TRUE by default. If \sQuote{fix} is TRUE, it will be applied only after the fix attempt.
#' @param fix whether to fix compatibility between saturation and bulk density (default is FALSE).
#' @param verbose default FALSE. Whether to print messages.
#' @param xargs additional arguments passed to \code{\link{apsimx_soil_profile}} function.
#' @return this function will always return a list. Each element of the list will
#' be an object of class \sQuote{soil_profile_grid}
# @export
#' @noRd
#' @examples 
#' \dontrun{
#' require(soilDB)
#' ## Example in progress
#' }
#' 
#' 

get_wcs_soil_profile_grid <- function(aoi, 
                                      db = "gSSURGO"){
                                      # soil.bottom = 200,
                                      # method = c("constant", "linear"),
                                      # nlayers = 10,
                                      # check = TRUE,
                                      # fix = FALSE,
                                      # verbose = FALSE,
                                      # xargs = NULL){
  
  if(!requireNamespace("soilDB", quietly = TRUE)){
    stop("The soilDB package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sp", quietly = TRUE)){
    stop("The sp package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("sf", quietly = TRUE)){
    stop("The sf package is required for this function")
    return(NULL)
  }
  
  if(!requireNamespace("spData", quietly = TRUE)){
    stop("The spData package is required for this function")
    return(NULL)
  }
  
  ## Determine if the location is in the US
  # if(requireNamespace("maps", quietly = TRUE)){
  #   country <- maps::map.where(x = lon, y = lat)
  #   if(country != "USA" || is.na(country))
  #     stop("These coordinates do not correspond to a location in the USA. \n Did you specify the coordinates correctly?")
  # }
  
  #### Getting the mukey
  mukeys <- soilDB::mukey.wcs(aoi = aoi, 
                              db = db,
                              quiet = TRUE)
  
  ### Variables I need for ssurgo2sp
  ### Texture: sandtotal_r, silttotal_r, claytotal_r
  ### Hydrology: ksat_r, wtenthbar_r, wthirdbar_r, wfifteenbar_r, wsatiated_r
  ### Bulkdensity: 
  ### Organic matter: om_r
  # prpty <- c(sandtotal_r, silttotal_r, claytotal_r)
  # sda.properties <- soilDB::get_SDA_property(property = )
  # 
  # ### Mapunit ### -- this might contain the iacornsr
  # if(verbose == FALSE){
  #   mapunit <- suppressWarnings(suppressMessages(soilDB::get_mapunit_from_SDA(sql)))
  # }else{
  #   mapunit <- soilDB::get_mapunit_from_SDA(sql) 
  # }
  # 
  # ### Component ###
  # cmpnt <- fSDA@site
  # names(cmpnt) <- gsub("_", ".", names(cmpnt), fixed = TRUE)
  # cmpnt$geomdesc <- cmpnt$geompos
  # 
  # ## Retrieve the state from the areasymbol
  # if(shift <= 0 || length(unique(mapunit$areasymbol)) == 1){
  #   cmpnt$state <- unique(strtrim(mapunit$areasymbol, 2))
  # }else{
  #   cmpnt$state <- NA
  #   warning("This area includes more than one state. 
  #           I have not though about how to get the state in this case. Please submit an issue 
  #           with a reproducible example to https://github.com/femiguez/apsimx/issues")
  # }
  # 
  # ### Chorizon ###
  # chrzns <- fSDA@horizons
  # names(chrzns) <- gsub("_", ".", names(chrzns), fixed = TRUE)
  # ### Things missing from horizons: hzthk.r, partdensity, wsatiated.r, wfifteenbar.r, wtenthbar.r, wthirdbar.r,
  # if(sum(grepl("partdensity", names(chrzns))) == 0) chrzns$partdensity <- NA
  # if(sum(grepl("hzthk", names(chrzns))) == 0) chrzns$hzthk.r <- NA
  # if(sum(grepl("wsatiated", names(chrzns))) == 0) chrzns$wsatiated.r <- NA
  # if(sum(grepl("wfifteenbar", names(chrzns))) == 0) chrzns$wfifteenbar.r <- NA
  # if(sum(grepl("wthirdbar", names(chrzns))) == 0) chrzns$wthirdbar.r <- NA
  # 
  # if(shift <= 0){
  #   spg.sf <- sf::st_as_sf(spg)
  #   spg.sf[["MUKEY"]] <- res$mukey
  #   spg.sf[["AREASYMBOL"]] <- mapunit$areasymbol
  #   mapunit.shp <- spg.sf
  # }else{
  #   mapunit.shp <- sf::st_as_sf(res)
  # }
  # 
  # sp0 <- ssurgo2sp(mapunit = mapunit, component = cmpnt,
  #                  chorizon = chrzns, mapunit.shp = mapunit.shp,
  #                  nmapunit = nmapunit, nsoil = nsoil, xout = xout,
  #                  soil.bottom = soil.bottom, method = method, nlayers = nlayers,
  #                  verbose = verbose)
  # 
  # ans <- vector("list", length(sp0))
  # 
  # for(i in seq_along(sp0)){
  #   metadata <- attributes(sp0[[i]])
  #   metadata$DataSource <- paste("SSURGO (https://sdmdataaccess.nrcs.usda.gov/) through R package soilDB, R package apsimx function ssurgo2sp. Timestamp",Sys.time())
  #   metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL;
  #   
  #   if(fix){
  #     icheck <- check
  #     check <- FALSE
  #   } 
  #   
  #   if(!is.null(xargs)){
  #     if(!is.null(xargs$crops)){
  #       crops <- xargs$crops
  #     }
  #   }else{
  #     crops <- c("Maize", "Soybean", "Wheat")
  #   }
  #   
  #   asp <- apsimx_soil_profile(nlayers = nlayers,
  #                              Thickness = sp0[[i]]$Thickness * 10,
  #                              BD = sp0[[i]]$BD,
  #                              AirDry = sp0[[i]]$AirDry,
  #                              LL15 = sp0[[i]]$LL15,
  #                              DUL = sp0[[i]]$DUL,
  #                              SAT = sp0[[i]]$SAT,
  #                              KS = sp0[[i]]$KS,
  #                              Carbon = sp0[[i]]$Carbon,
  #                              crop.LL = sp0[[i]]$LL15,
  #                              ParticleSizeClay = sp0[[i]]$ParticleSizeClay,
  #                              ParticleSizeSilt = sp0[[i]]$ParticleSizeSilt,
  #                              ParticleSizeSand = sp0[[i]]$ParticleSizeSand,
  #                              soil.bottom = soil.bottom,
  #                              metadata = metadata,
  #                              check = check, 
  #                              crops = crops)
  #   
  #   if(fix){
  #     asp <- fix_apsimx_soil_profile(asp, verbose = verbose)
  #     if(icheck)
  #       check_apsimx_soil_profile(asp)
  #   } 
  #   
  #   ans[[i]] <- asp
  # }
  # 
  # return(ans)
}

#' 
#' @title Viewing an APSIM-X file interactively
#' @name view_apsimx
#' @description Generate an interactive viewer for an APSIM-X file
#' @param file 	a file ending in .apsimx to be inspected (JSON)
#' @param src.dir directory containing the .apsimx file to be inspected; defaults to the current working directory
#' @param viewer either \dQuote{json} or \dQuote{react}.
#' @param ... additional arguments passed to either \sQuote{jsonedit} or \sQuote{reactjson}.
#' These are functions in package \CRANpkg{listviewer}.
#' @return a display with the APSIM file structure.
#' @note I do not know how to edit an APSIM-X file using this method yet.
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## View the structure of the APSIM-X simulation file
#' view_apsimx("Wheat.apsimx", src.dir = extd.dir)
#' }
#' 

view_apsimx <- function(file, src.dir, viewer = c("json","react"), ...){
  
  if(!requireNamespace("listviewer", quietly = TRUE)){
    warning("The listviewer package is required for this function")
    return(NULL)
  }
  
  if(missing(file)) stop("need to specify file name")
  
  if(isFALSE(get("allow.path.spaces", apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }
  
  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  if(missing(src.dir)) src.dir <- "."

  viewer <- match.arg(viewer)
  
  file.name.path <- file.path(src.dir, file)
  
  apsimx_json <- jsonlite::read_json(file.name.path)
  
  if(viewer == "json"){
    ans <- listviewer::jsonedit(listdata = apsimx_json, ...) 
    return(ans)
  }
  
  if(viewer == "react"){
    
    if(!requireNamespace("reactR", quietly = TRUE)){
      warning("The reactR package is required for this function")
      return(NULL)
    }
    
    ans <- listviewer::reactjson(listdata = apsimx_json, ...)  
    return(ans)
  }
}


#' 
#' @title Viewing an APSIM Classic file interactively
#' @name view_apsim
#' @description Generate an interactive viewer for an APSIM file
#' @param file 	a file ending in .apsim to be inspected (XML)
#' @param src.dir directory containing the .apsim file to be inspected; defaults to the current working directory
#' @param viewer either \dQuote{json} or \dQuote{react}.
#' @param ... additional arguments passed to either \sQuote{jsonedit} or \sQuote{reactjson}.
#' These are functions in package \CRANpkg{listviewer}.
#' @return a display with the APSIM file structure.
#' @note I do not know how to edit an APSIM file using this method yet.
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## View the structure of the APSIM-X simulation file
#' view_apsim("Millet.apsim", src.dir = extd.dir)
#' }
#' 

view_apsim <- function(file, src.dir, viewer = c("json","react"), ...){
  
  if(!requireNamespace("listviewer", quietly = TRUE)){
    warning("The listviewer package is required for this function")
    return(NULL)
  }
  
  if(missing(file)) stop("need to specify file name")
  
  .check_apsim_name(file)
  
  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsim$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsim files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  if(missing(src.dir)) src.dir <- "."
  
  viewer <- match.arg(viewer)
  
  file.name.path <- file.path(src.dir, file)
  
  apsim_xml <- xml2::as_list(xml2::read_xml(file.name.path))
  
  if(viewer == "json"){
    ans <- listviewer::jsonedit(listdata = apsim_xml, ...) 
    return(ans)
  }
  
  if(viewer == "react"){
    
    if(!requireNamespace("reactR", quietly = TRUE)){
      warning("The reactR package is required for this function")
      return(NULL)
    }
    
    ans <- listviewer::reactjson(listdata = apsim_xml, ...)  
    return(ans)
  }
}

#' @title Obtain a synthetic APSIM soil profile from the World Modeler database
#' @description Retrieves soil data from the World Modeler global database and (optionally) saves it to a soils file
#' @name get_worldmodeler_soil_profile
#' @param lonlat Longitude and latitude vector (e.g. c(-93, 42)) or matrix.
#' @param soil.name optional soil name
#' @param wrt.dir optional directory where to save a file with \sQuote{soils} extension.
#' If missing it will be written to a temporary directory.
#' @param filename optional name to be used when saving the file
#' @param verbose verbose argument passed to \sQuote{read_apsim_soils}
#' @return it returns a list with objects of class \sQuote{soil_profile}. If
#' \sQuote{filename} is specified it also creates a file with extension \sQuote{soils}, 
#' which can be read using function \code{\link{read_apsim_soils}}.
#' @author Brian Collins (University of Southern Queensland) and Fernando Miguez
#' @export
#' @examples 
#' \dontrun{
#' ## Get soil profile properties for a single point
#'   sp1 <- get_worldmodeler_soil_profile(lonlat = c(-93, 42))
#'   
#'   if(inherits(sp1[[1]], 'soil_profile')){
#'     plot(sp1[[1]], property = "Carbon")
#'   }
#' }

get_worldmodeler_soil_profile <- function(lonlat, soil.name, wrt.dir, filename, verbose = FALSE){
  
  ## Create temporary directory 
  if(missing(wrt.dir)){
    wrt.dir <- tempdir()    
  }else{
    wrt.dir <- normalizePath(wrt.dir)
  }
  
  cleanup <- FALSE
  
  if(missing(filename)){
    filename <- "wm_temp_soils.soils"
    cleanup <- TRUE
  }else{
    if(tools::file_ext(filename) != "soils")
      stop("'filename' should have a '.soils' extention", call. = FALSE)
  }

  if(inherits(lonlat, "numeric")){
    lonlat <- matrix(lonlat, nrow = 1)
  }
  
  if(missing(soil.name)){
    soil.name <- paste("SoilName", seq_len(nrow(lonlat)), sep = "_")
  }
  
  ## Lonlat will be a matrix from now on
  soil.node <- xml2::read_xml('<soils version="37"></soils>')
  
  for(i in seq_len(nrow(lonlat))){
    x <- try(xml2::read_xml(paste0('https://worldmodel.csiro.au/apsimsoil?lon=', lonlat[i, 1], '&lat=',lonlat[i, 2])), silent = TRUE)
    if(!inherits(x, 'try-error')){
      xs <- xml2::xml_find_all(x, 'Soil')[[1]]
      xml2::xml_set_attr(xs, 'name', soil.name[i])
      xml2::xml_add_child(soil.node, xs, .copy = TRUE)      
    }else{
      warning(paste("Could not get a soil for row", i), immediate. = TRUE)
    }
  }
  
  xml2::write_html(soil.node, file = file.path(wrt.dir, filename), options = c("format", "no_declaration")) 
  res <- read_apsim_soils(filename, src.dir = wrt.dir, verbose = verbose)
  
  if(cleanup) file.remove(file.path(wrt.dir, filename)) 
  
  return(res)
}

#### Get WM APSIM met ----
#' @title Obtain a weather APSIM met from the World Modeler database
#' @description Retrieves met data from the World Modeler global database and (optionally) saves it to a file
#' @name get_worldmodeler_apsim_met
#' @param lonlat Longitude and latitude vector (e.g. c(-93, 42)) or matrix.
#' @param dates date range (see example for format)
#' @param wrt.dir optional directory where to save a file with \sQuote{met} extension.
#' If missing it will be written to a temporary directory.
#' @param filenames optional name(s) to be used when saving the file. It should be equal to the
#' number of rows of the input matrix.
#' @param check whether to check the met file
#' @param verbose argument passed to read_apsim_met
#' @return it creates a list with objects of class \sQuote{met}. 
#' If it fails, then the objects will be of class \sQuote{try-error}.
#' @export
#' @examples 
#' \dontrun{
#' ## Get soil profile properties for a single point
#'   am1 <- get_worldmodeler_apsim_met(lonlat = c(-93, 42), 
#'                                     dates = c("2010-01-01", "2013-12-31"))
#'   if(inherits(am1, 'met')){
#'      plot(am1[[1]], met.var = "rain", cumulative = TRUE)
#'   }
#'  }
#' 

get_worldmodeler_apsim_met <- function(lonlat, dates, wrt.dir, filenames, check = FALSE, verbose = FALSE){
  
  ## Create temporary directory 
  if(missing(wrt.dir)){
    wrt.dir <- tempdir()    
  }else{
    wrt.dir <- normalizePath(wrt.dir)
  }
  
  if(inherits(lonlat, "numeric")){
    lonlat <- matrix(lonlat, nrow = 1)
  }

  cleanup <- FALSE
  
  if(missing(filenames)){
    filenames <- paste0("wm_apsim_met", seq_len(nrow(lonlat)), ".met")
    cleanup <- TRUE
  }else{
    for(i in seq_along(filenames)){
      if(tools::file_ext(filenames[i]) != "met")
        stop("'filenames' should have a '.met' extention", call. = FALSE)      
    }
  } 
 
  if(length(filenames) != nrow(lonlat))
    stop("'filenames' length should be equal to the number of rows in the input matrix", call. = FALSE)
  
  res <- vector('list', length = nrow(lonlat))
  for(i in seq_len(nrow(lonlat))){
    url1 <- 'https://worldmodel.csiro.au/gclimate?lat='
    url2 <- paste0(lonlat[i, 2], '&lon=', lonlat[i, 1], '&')
    url3 <- 'format=apsim&start='
    url4 <- gsub("-", "", dates[1], fixed = TRUE)
    url5 <- '&stop='
    url6 <- gsub("-", "", dates[2], fixed = TRUE)
    x <- try(readLines(paste0(url1, url2, url3, url4, url5, url6)), silent = TRUE)
    if(!inherits(x, 'try-error')){
      writeLines(x, file.path(wrt.dir, filenames[i]))
      res[[i]] <- read_apsim_met(filenames[i], src.dir = wrt.dir, verbose = verbose)
      if(check){
        cat("Checking lonlat", lonlat[i, ], "\n")
        check_apsim_met(res[[i]])
      }      
    }else{
      if(verbose)
        warning(paste("Could not get met file for row", i), immediate. = TRUE)
     res[[i]] <- x 
    }
  }  
  
  ## If filenames is missing I will cleanup
  if(cleanup){
    for(i in seq_len(nrow(lonlat))){
      file.remove(file.path(wrt.dir, filenames[i]))  
    }
  } 
  
  return(res)
}

#' Edit an APSIM (Classic) Simulation auxiliary xml file
#' 
#' This function allows editing of an APSIM (Classic) simulation xml file.
#' 
#' The variables specified by \code{parm} within the .apsim file specified by \code{file} 
#' in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
#' is a list that has the same number of elements as the length of the vector \code{parm}.  The current
#' .xml file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
#' \sQuote{file} \emph{-edited.xml} will be created.  If (verbose = TRUE) then the name
#'  of the written file is returned. The function is similar to the edit_sim_file functin in the \sQuote{apsimr}
#'  package, but with the difference that here the xml2 package is used instead.
#'  
#' @name edit_apsim_xml
#' @param file file ending in .xml to be edited
#' @param src.dir directory containing the .xml file to be edited; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param parm.path parameter path to be edited (see example)
#' @param value new values for the parameter to be edited 
#' @param overwrite logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise
#' @param edit.tag if the file is edited a different tag from the default \sQuote{-edited} can be used.
#' @param verbose whether to print information about successful edit
#' @return (when verbose=TRUE) complete file path to edited .xml file is returned as a character string.
#' As a side effect this function creates a new XML file.
#' @note This function cannot check whether replacement is of the correct length. Also, there is
#' an inspect equivalent. It is more flexible than \sQuote{edit_apsim} and (perhaps) similar to 
#' \sQuote{apsimr::edit_sim_file}.
#' @export
#' @examples 
#' \donttest{
#' ## This example changes the RUE values
#' 
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' values <- paste(rep(1.7, 12), collapse = " ")
#' 
#' ## Writing to a temp directory, but replace as needed
#' tmp.dir <- tempdir()
#' 
#' edit_apsim_xml("Maize75.xml", 
#'                src.dir = extd.dir,
#'                wrt.dir = tmp.dir,
#'                parm.path = ".//Model/rue",
#'                value = values)
#' }

edit_apsim_xml <- function(file, src.dir = ".", wrt.dir = NULL,
                           parm.path=NULL, 
                           value=NULL, 
                           overwrite = FALSE,
                           edit.tag = "-edited",
                           verbose = TRUE){
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern=".xml$",ignore.case=TRUE)
  
  if(length(file.names)==0)
    stop("There are no .xml files in the specified directory to edit.")
  
  if(!identical(length(value),length(parm.path))) 
    stop("length of parm.path should be equal to length of value")
  
  file <- match.arg(file, file.names)
  
  ## Parse apsim file (XML)
  apsim_xml <- xml2::read_xml(paste0(src.dir,"/",file))
  
  xml.parm.path.out <- character(length(parm.path))
  
  for(i in seq_along(parm.path)){

    xml.path <- xml2::xml_find_all(apsim_xml, parm.path[i])
    
    if(is.na(xml.path)) stop("parameter not found")
    
    if(length(xml.path) > 1) stop("path is not unique")
    
    if(length(xml.path) != length(value[i])){
      stop("length of parm.path should be equal to length of value")
    }
    
    xml.parm.path.out[i] <- xml2::xml_path(xml.path)
    
    xml2::xml_set_text(xml.path, value[i])
    
  }

  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir,"/",
                      tools::file_path_sans_ext(file),
                      edit.tag,".xml")
  }else{
    wr.path <- paste0(wrt.dir,"/",file)
  }
  xml2::write_xml(apsim_xml, file = wr.path)
  
  if(verbose){
    cat("Edited",parm.path, "\n")
    cat("Paramter path:", xml.parm.path.out, "\n")
    cat("New values ",value, "\n")
    cat("Created ",wr.path,"\n")
  }
  invisible(xml.parm.path.out)
}

## Testing read_apsim_all function
require(apsimx)

run.examples <- FALSE

extd.dir <- system.file("extdata", package = "apsimx")

maize.out <- read_apsim("Maize.out", src.dir = extd.dir)

millet.out <- read_apsim("Millet.out", src.dir = extd.dir)

## Running all examples
os <- .Platform$OS.type

if(os == "windows" && run.examples){
  
  apsim_options(warn.versions = FALSE)
  ex.classic.dir <- apsimx:::auto_detect_apsim_examples()
  apsim.examples <- dir(ex.classic.dir, pattern = "apsim$")
  
  tmp.dir <- tempdir()
  setwd(tmp.dir)
  
  for(i in apsim.examples){
    if(grepl("\\s", i)) next
    file.copy(file.path(ex.classic.dir, i), to = file.path(tmp.dir, i))
    cat("Example:", i)
    sim <- try(apsim(file = i, simplify = FALSE), silent = TRUE)
    if(inherits(sim, "try-error")){
      cat(" Failed. \n")
      next
    }else{
      cat("; object class:", class(sim), " ")
      cat("; Done. \n")  
    }
  }
}

## The purpose of this test file is to consider the inspecting
## and editing capabilities of:
##
## 1. inspect_apsimx_json
## 2. edit_apsimx (node == "Other")
## 3. edit_apsimx_json
##
## I need to consider both files with a .apsimx and .json extension
require(apsimx)
apsimx_options(warn.versions = FALSE)

run.edit.apsimx.json <- get(".run.local.tests", envir = apsimx.options)

if(run.edit.apsimx.json){
  
  tmp.dir <- tempdir()
  ## Testing with an .apsimx file first
  ex.dir <- auto_detect_apsimx_examples()
  pp <- inspect_apsimx_json("Barley.apsimx", src.dir = ex.dir, parm = "Version$")
  ## Try editing
  apsimx:::edit_apsimx_json("Barley.apsimx", src.dir = ex.dir, wrt.dir = tmp.dir,
                   parm.path = pp[1], value = 200)
  inspect_apsimx_json("Barley-edited.apsimx", src.dir = tmp.dir, parm = "Version")
  
}

#' Edit an APSIM (Classic) Simulation
#' 
#' This function allows editing of an APSIM (Classic) simulation file.
#' 
#' The variables specified by \code{parm} within the .apsim file specified by \code{file} 
#' in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
#' is a list that has the same number of elements as the length of the vector \code{parm}.  The current
#' .apsim file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
#' \sQuote{file} \emph{-edited.apsim} will be created.  If (verbose = TRUE) then the name
#'  of the written file is returned. 
#'  
#'  When node equals Outputfile, the editing allows to add variables, but not to remove them at the moment.
#' 
#' @name edit_apsim
#' @param file file ending in .apsim to be edited
#' @param src.dir directory containing the .apsim file to be edited; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param node either \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}, 
#' \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other} 
#' @param soil.child specific soil component to be edited
#' @param manager.child specific manager component to be edited (not implemented yet)
#' @param parm parameter to be edited
#' @param value new values for the parameter to be edited 
#' @param overwrite logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise
#' @param edit.tag if the file is edited a different tag from the default \sQuote{-edited} can be used.
#' @param parm.path path to the attribute to edit when node is \sQuote{Other}
#' @param root supply the node postion in the case of multiple simulations such as factorials.
#' @param verbose whether to print information about successful edit
#' @param check.length check whether vectors are of the correct length
#' @return (when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
#' As a side effect this function creates a new (XML) .apsimx file.
#' @note The components that can be edited are restricted becuase this is better in preventing
#' errors of editing unintended parts of the file. The disadvantage is that there is less flexibility
#' compared to the similar function in the 'apsimr' package. 
#' @export
#' @examples 
#' \donttest{
#' ## This example will read one of the examples distributed with APSIM
#' ## but write to a temporary directory
#' 
#' tmp.dir <- tempdir()
#' 
#' extd.dir <- system.file("extdata", package = "apsimx")
#' edit_apsim("Millet", src.dir = extd.dir, wrt.dir = tmp.dir, 
#'            node = "Clock",
#'            parm = "start_date", value = "01/02/1940")
#' 
#' ## Editing all of the KL values for Millet
#' pp.KL <- inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, 
#'                   parm = "SoilCrop[8]/KL")
#'                  
#' kls <- seq(0.08, 0.2, length.out = 11)
#' 
#' edit_apsim("Millet.apsim", 
#'            src.dir = extd.dir,
#'            wrt.dir = tmp.dir,
#'            node = "Other", 
#'            parm.path = pp.KL,
#'            value = kls)
#'            
#' ## Check that it was properly edited
#' 
#' inspect_apsim("Millet-edited.apsim", 
#'               src.dir = tmp.dir,
#'               node = "Soil",
#'               soil.child = "Water",
#'               parm = "KL")
#' 
#'               
#' }
#' 

edit_apsim <- function(file, src.dir = ".", wrt.dir = NULL,
                       node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter",
                                "MicroClimate", "Crop", "Manager", "Outputfile", "Other"),
                       soil.child = c("Metadata", "Water", "Physical", "OrganicMatter", "Chemical",
                                      "Analysis", "InitialWater", "Sample", "SWIM"),
                       manager.child = NULL,
                       parm = NULL, value = NULL, 
                       overwrite = FALSE,
                       edit.tag = "-edited",
                       parm.path = NULL,
                       root,
                       verbose = TRUE,
                       check.length = TRUE){
  
  if(is.null(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern=".apsim$", ignore.case=TRUE)
  
  if(identical(length(file.names),0))
    stop("There are no .apsim files in the specified directory to edit.")
  
  node <- match.arg(node)
  soil.child <- match.arg(soil.child)
  
  ## For now we just edit one file at a time
  file <- match.arg(file, file.names)
  
  ## Parse apsim file (XML)
  apsim_xml <- xml2::read_xml(file.path(src.dir, file))
  
  ## This is my attempt at picking the right node in a factorial
  if(!missing(root)){
    apsim_xml00 <- apsim_xml ## Make a copy to do the right insert later
    apsim_xml0 <- xml2::xml_find_all(apsim_xml, ".//simulation")
    sim.names <- unlist(xml2::xml_attrs(apsim_xml0))
    wsim <- grep(root, sim.names)
    apsim_xml <- apsim_xml0[[wsim]] ## This replaces the node
    parm.path.root <- xml2::xml_path(apsim_xml)
    apsim_xml_root_node <- xml2::xml_find_first(apsim_xml00, parm.path.root)
  }
  
  ## Edit the 'Clock'
  if(node == "Clock"){
    parm.choices <- c("start_date", "end_date")
    parm <- match.arg(parm, choices = parm.choices, several.ok = TRUE)
    j <- 1
    for(i in parm){
      parm.path <- paste0(".//clock","/", i)
      startend.node <- xml2::xml_find_first(apsim_xml, parm.path)
      ## apsim requires %Y-%m-%d
      xml2::xml_set_text(startend.node, as.character(value[j]))
      j <- j + 1
    }
  }
  
  ## Editing the weather file name only
  if(node == "Weather"){
    if(missing(parm) || parm == "filename"){
      parm.path <- ".//metfile/filename"
      weather.filename.node <- xml2::xml_find_first(apsim_xml, parm.path)
      if(length(grep(".met$", value)) == 0) 
        stop("value should be a .met file")
      xml2::xml_set_text(weather.filename.node, value)
    }else{
      stop("parm not appropriate for node = Weather")
    }
  }
  
  ## Editing the 'Soil' component
  if(node == "Soil"){
    
    if(soil.child == "Metadata"){
      ## Not sure if there is minimum set of required parameters
      parm.path <- paste0(".//Soil/", parm)
      soil.metadata.node <- xml2::xml_find_first(apsim_xml, parm.path)
      xml2::xml_set_text(soil.metadata.node, as.character(value))
    }
    
    if(soil.child == "Water" || soil.child == "Physical"){
      ## First set of parameters are crop specific
      if(parm %in% c("LL", "KL", "XF")){
        parm.path <- paste0(".//Soil/Water/SoilCrop", "/", parm)
      }
      ## Second set of parameters are soil specific
      if(parm %in% c("Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS", "SWCON", "MWCON", "KLAT")){
        parm.path <- paste0(".//Soil/Water", "/", parm)
      }
      
      if(parm %in% c("SWCON", "MWCON", "KLAT")){
        parm.path <- paste0(".//Soil/SoilWater", "/", parm)
      }
      
      ## Soil Water
      soil.water.parms <- c("SummerCona", "SummerU", "SummerDate",
                            "WinterCona", "WinterU", "WinterDate",
                            "DiffusConst","DiffusSlope", "Salb",
                            "CN2Bare", "CNRed", "CNCov")
      
      if(parm %in% soil.water.parms){
        ## These are of length 1
        parm.path <- paste0(".//Soil/Water", "/", parm)
        soil.water.node <- xml2::xml_find_first(apsim_xml, parm.path)
        ## Error checking
        if(check.length){
          if(length(value) != length(soil.water.node))
            stop("value vector of incorrect length")
        }
        xml2::xml_set_text(soil.water.node, as.character(value))
      }else{
        soil.water.node <- xml2::xml_find_first(apsim_xml, parm.path)
        if(check.length){
          if(length(value) != length(xml2::xml_children(soil.water.node))){
            print(parm.path)
            cat("Length of value:", length(value), "\n")
            cat("Length of node: ", length(xml2::xml_children(soil.water.node)), "\n")
            stop("value vector of incorrect length")            
          }
            
        }
        ## With this code it is still possible to provide an incorrect parameter
        ## Not sure...
        len.child.soil.water.node <- length(xml2::xml_children(soil.water.node))
        ## If value is larger I grow the children to match lengths
        if(length(value) > len.child.soil.water.node){
            for(i in seq_len(length(value) - len.child.soil.water.node)){
              xml2::xml_add_child(soil.water.node, xml2::xml_children(soil.water.node)[[len.child.soil.water.node]])
            }
        }
        if(length(value) < len.child.soil.water.node){
          cat("length of value is shorter than length of soil water node.\n")
          stop("Don't really know how to do this yet")
        }
        xml2::xml_set_text(xml2::xml_children(soil.water.node), as.character(value))
      }
    }

    if(soil.child == "SWIM"){
      
      parm.path.0 <- ".//Soil/Swim"
      ## This first set are single values
      swim.parms1 <- c("Salb", "CN2Bare", "CNRed", "CNCov", "KDul",
                       "PSIDul", "VC", "DTmin", "DTmax", "MaxWaterIncrement",
                       "SpaceWeightingFactor", "SoluteSpaceWeightingFactor",
                       "Diagnostics")

      swim.parms2 <- c("DrainDepth", "DrainSpacing", "DrainRadius", "Klat", "ImpermDepth")
      
      if(parm %in% swim.parms1){
        parm.path <- paste0(".//Soil/Swim", "/", parm)
      }
      if(parm == "WaterTableDepth"){
        parm.path <- ".//Soil/Swim/SwimWaterTable/WaterTableDepth"
      }
      if(parm %in% swim.parms2){
        parm.path <- paste0(".//Soil/Swim/SwimSubsurfaceDrain", "/", parm)
      }
      soil.swim.node <- xml2::xml_find_first(apsim_xml, parm.path)
      xml2::xml_set_text(soil.swim.node, as.character(value))
    }
        
    if(soil.child == "Nitrogen"){
      stop("not implemented yet")
      parm.choices <- c("fom_type", "fract_carb", "fract_cell", "fract_lign")
      parm <- match.arg(parm, choices = parm.choices)
      
      ## for all of these the length should be 6
      if(length(value) != 6) stop("value should be of length=6")
      
      parm.path <- paste0(".//Soil/SoilNitrogen", "/", parm)
      soil.nitrogen.node <- xml2::xml_find_first(apsim_xml, parm.path)
      xml2::xml_set_text(xml2::xml_children(soil.nitrogen.node), as.character(value))
    }
    
    if(soil.child == "OrganicMatter"){
      ## State what are organic matter possible parameters
      parm.ch1 <- c("RootCN", "RootWt", "SoilCN", "EnrACoeff", "EnrBCoeff")
      parm.ch2 <- c("Thickness", "OC", "FBiom", "FInert")
      
      parm.path <- paste0(".//Soil/SoilOrganicMatter", "/", parm)
      
      if(parm %in% parm.ch1){
        if(length(value) != 1) stop("value should be of length = 1")
        soil.OM.node <- xml2::xml_find_first(apsim_xml, parm.path)
        xml2::xml_set_text(soil.OM.node, as.character(value))
      }
      
      if(parm %in% parm.ch2){
        soil.OM.node <- xml2::xml_find_first(apsim_xml, parm.path)
        
        if(check.length){
          if(length(value) != length(xml2::xml_children(soil.OM.node)))
            stop("value vector of incorrect length")
        }
        
        len.child.soil.OM.node <- length(xml2::xml_children(soil.OM.node))
        ## If value is larger I grow the children to match lengths
        if(length(value) > len.child.soil.OM.node){
          for(i in seq_len(length(value) - len.child.soil.OM.node)){
            xml2::xml_add_child(soil.OM.node, xml2::xml_children(soil.OM.node)[[len.child.soil.OM.node]])
          }
        }
        if(length(value) < len.child.soil.OM.node){
          cat("length of value is shorter than length of soil water node. \n At the moment I think I can grow XML nodes but not shrink them.\n")
          stop("Don't really know how to do this yet.")
        }
        xml2::xml_set_text(xml2::xml_children(soil.OM.node), as.character(value))
      }
    }
    
    if(soil.child == "Analysis"){
      ## State what are possible analysis parameters
      parm.ch <- c("Thickness", "PH", "EC")
      
      parm <- match.arg(parm, choices = parm.ch)
      
      parm.path <- paste0(".//Soil/Analysis", "/", parm)
      
      soil.Analysis.node <- xml2::xml_find_first(apsim_xml, parm.path)
      
      if(check.length){
        if(length(value) != length(xml2::xml_children(soil.Analysis.node)))
          stop("value vector of incorrect length")
      }
      
      len.child.soil.Analysis.node <- length(xml2::xml_children(soil.Analysis.node))
      ## If value is larger I grow the children to match lengths
      if(length(value) > len.child.soil.Analysis.node){
        for(i in seq_len(length(value) - len.child.soil.Analysis.node)){
          xml2::xml_add_child(soil.Analysis.node, xml2::xml_children(soil.Analysis.node)[[len.child.soil.Analysis.node]])
        }
      }
      if(length(value) < len.child.soil.Analysis.node){
        cat("length of value is shorter than length of soil water node. \n At the moment I think I can grow XML nodes but not shrink them.\n")
        stop("Don't really know how to do this yet.")
      }
      
      xml2::xml_set_text(xml2::xml_children(soil.Analysis.node), as.character(value))
    }
    
    if(soil.child == "InitialWater"){
      ## State what are possible InitialWater parameters
      parm.ch <- c("PercentMethod", "FractionFull", "DepthWetSoil")
      
      parm <- match.arg(parm, choices = parm.ch)
      
      parm.path <- paste0(".//Soil/InitialWater", "/", parm)
      
      soil.InitialWater.node <- xml2::xml_find_first(apsim_xml, parm.path)
      
      if(check.length){
        if(length(value) != length(soil.InitialWater.node))
          stop("value vector of incorrect length")
      }
      
      len.child.soil.InitialWater.node <- length(xml2::xml_children(soil.InitialWater.node))
      ## If value is larger I grow the children to match lengths
      if(length(value) > len.child.soil.InitialWater.node){
        for(i in seq_len(length(value) - len.child.soil.InitialWater.node)){
          xml2::xml_add_child(soil.InitialWater.node, xml2::xml_children(soil.InitialWater.node)[[len.child.soil.InitialWater.node]])
        }
      }
      if(length(value) < len.child.soil.InitialWater.node){
        cat("length of value is shorter than length of soil water node. \n At the moment I think I can grow XML nodes but not shrink them.\n")
        stop("Don't really know how to do this yet.")
      }
      
      xml2::xml_set_text(soil.InitialWater.node, as.character(value))
    }
    
    if(soil.child == "Sample"){
      ## State what are possible InitialWater parameters
      parm.ch <- c("Thickness", "NO3", "NH4", "SW", "OC", "EC", "CL", "ESP", "PH")
      
      parm <- match.arg(parm, choices = parm.ch)
      
      parm.path <- paste0(".//Soil/Sample", "/", parm)
      
      soil.Sample.node <- xml2::xml_find_first(apsim_xml, parm.path)
      
      if(check.length){
        if(length(value) != length(soil.Sample.node))
          stop("value vector of incorrect length")
      }
      
      len.child.soil.Sample.node <- length(xml2::xml_children(soil.Sample.node))
      ## If value is larger I grow the children to match lengths
      if(length(value) > len.child.soil.Sample.node){
        for(i in seq_len(length(value) - len.child.soil.Sample.node)){
          xml2::xml_add_child(soil.Sample.node, xml2::xml_children(soil.Sample.node)[[len.child.soil.Sample.node]])
        }
      }
      if(length(value) < len.child.soil.Sample.node){
        cat("length of value is shorter than length of soil water node. \n At the moment I think I can grow XML nodes but not shrink them.\n")
        stop("Don't really know how to do this yet.")
      }
      
      xml2::xml_set_text(xml2::xml_children(soil.Sample.node), as.character(value))
    }
    ## For now changing other components of the 'Soil' might make little sense
  }
  
  if(node == "SurfaceOrganicMatter"){
      ## State what are possible Pools parameters
      parm.ch <- c("PoolName", "ResidueType", "Mass", "CNRatio",
                   "CPRatio", "StandingFraction", "type", "mass",
                   "cnr", "standing_fraction")
      
      parm <- match.arg(parm, choices = parm.ch)
      
      parm.path <- paste0(".//surfaceom/", parm)
      
      soil.Pools.Pool.node <- xml2::xml_find_first(apsim_xml, parm.path)
      
      if(check.length){
        if(length(value) != length(xml2::xml_text(soil.Pools.Pool.node)))
          stop("value vector of incorrect length")
      }
      
      len.child.soil.Pools.Pool.node <- length(xml2::xml_children(soil.Pools.Pool.node))
      ## If value is larger I grow the children to match lengths
      if(length(value) > len.child.soil.Pools.Pool.node){
        for(i in seq_len(length(value) - len.child.soil.Pools.Pool.node)){
          xml2::xml_add_child(soil.Pools.Pool.node, xml2::xml_children(soil.Pools.Pool.node)[[len.child.soil.Pools.Pool.node]])
        }
      }
      if(length(value) < len.child.soil.Pools.Pool.node){
        cat("length of value is shorter than length of soil water node. \n At the moment I think I can grow XML nodes but not shrink them.\n")
        stop("Don't really know how to do this yet.")
      }
      
      xml2::xml_set_text(soil.Pools.Pool.node, as.character(value))
  }
  
  if(node == "Crop" || node == "Manager"){
    
    if(!missing(manager.child)){
      
      manager.node <- xml2::xml_find_all(apsim_xml, ".//manager")
      manager.node.names <- xml2::xml_attr(manager.node, "name")
      wmnn <- grep(manager.child, manager.node.names, ignore.case = TRUE)
      
      if(length(wmnn) == 0L) stop("manager child not found")
      
      select.manager.node <- manager.node[[wmnn]]
      
      ## If parameter is missing I should print available options, maybe...
      parm.select.manager.node <- xml2::xml_find_first(select.manager.node, paste0(".//",parm))
      
      xml2::xml_set_text(parm.select.manager.node, as.character(value))
      
    }else{
      crop.node <- xml2::xml_find_all(apsim_xml, parm)
      
      if(length(crop.node) == 0) stop("parm not found")
      
      if(check.length){
        if(length(value) != length(crop.node))
            stop("value vector of incorrect length")
      }
      
      xml2::xml_set_text(crop.node, as.character(value))
      
    }
  }
  
  if(node == "Outputfile"){
    outputfile.node <- xml2::xml_find_all(apsim_xml, ".//outputfile")
    outputfile.parms <- c("filename", "title", "variables", "events")
    
    if(missing(parm)){
      cat("Parameter choices:", outputfile.parms, "\n")
      stop("Parameter is missing. Choose one from the options above.", call. = FALSE)
    }
    
    parm <- match.arg(parm, outputfile.parms)
    
    if(parm == "filename" && !grepl("out$", value))
      stop("value should have '.out' extension when parm = 'filename'", call. = FALSE)
    
    if(parm == "filename" || parm == "title" || parm == "events"){
      output.node <- xml2::xml_find_first(outputfile.node, parm)
      xml2::xml_set_text(output.node, as.character(value))
    }
    
    if(parm == "variables"){
      variables.node <- xml2::xml_find_first(outputfile.node, parm)
      len.variables.node <- length(xml2::xml_children(variables.node))
      if(substr(value[1], 1, 1) == "-"){
        stop("Have not implemented this yet.")
        ## Here I remove a variable        
      }else{
        for(i in seq_along(value)){
          xml2::xml_add_child(variables.node, xml2::xml_children(variables.node)[[len.variables.node]])
          last.child <- xml2::xml_children(variables.node)[[len.variables.node+i]]
          xml2::xml_set_text(last.child, value[i])          
        }
      }
    }
  }
  
  if(node == "Other"){
    other.node <- xml2::xml_find_first(apsim_xml, parm.path)
    
    if(length(other.node) == 0) stop("other node parameter not found")
    
    length.other.node0 <- xml2::xml_length(other.node)
    length.other.node1 <- length(xml2::xml_text(other.node))
    length.other.node <- max(c(length.other.node0, length.other.node1))
    
    if(length(value) != length.other.node)
      stop("Length of value should be equal to the vector you are trying to edit")
    
    if(length.other.node == 1){
      xml2::xml_set_text(other.node, as.character(value))  
    }else{
      for(i in seq_len(length.other.node)){
        xml2::xml_set_text(xml2::xml_children(other.node)[i], as.character(value[i]))  
      }
    }
  }
  
  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir, "/",
                      tools::file_path_sans_ext(file),
                      edit.tag, ".apsim")
  }else{
    wr.path <- paste0(wrt.dir, "/", file)
  }
  
  ## If I use the root method need to use this workaround
  if(!missing(root)){
    # apsim_xml00[[wsim]] <- apsim_xml
    # apsim_xml <- apsim_xml00
    ## Alternative
    ## Find root node
    xml2::xml_replace(apsim_xml_root_node, apsim_xml)
    xml2::write_xml(apsim_xml00, file = wr.path) 
  }else{
    xml2::write_xml(apsim_xml, file = wr.path)    
  }
  
  # cat("Class apsim_xml", class(apsim_xml), "\n")
  # cat("wr.path: ", wr.path, "\n")
  # cat("Class wr.path:", class(wr.path), "\n")
  
  if(verbose){
    cat("Edited", parm.path, "\n")
    cat("Edited parameter", parm, "\n")
    cat("New values ", value, "\n")
    cat("Created ", wr.path, "\n")
  }
}

#'
#' This function does not print anything (compared to inspect_apsimx). The purpose is to return data
#' contained in the APSIM simulation as a data.frame. It will return a \sQuote{list} when a data frame
#' does not naturally accommodate the result. For example, the complete manager node does not naturally
#' fit into a data frame structure. In some cases, multiple data frames are returned as part of lists.
#'
#' @title Extract data from an .apsimx (JSON) file
#' @name extract_data_apsimx
#' @description Extract data from a JSON apsimx file.
#' @param file file ending in .apsimx to be inspected (JSON)
#' @param src.dir directory containing the .apsimx file to be inspected; defaults to the current working directory
#' @param node specific node to be used either \sQuote{Clock}, \sQuote{Weather},
#' \sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop},
#'  \sQuote{Manager}, \sQuote{Operations} or \sQuote{Other}
#' @param soil.child specific soil component to be inspected. The options vary depending on what is available (see \link{inspect_apsimx})
#' @param parm parameter to refine the extraction of the \sQuote{manager} list(\sQuote{parm},\sQuote{position}), use \sQuote{NA} for all the positions. \sQuote{parm} can be a regular expression for partial matching.
#' @param digits number of decimals to print (default 3). Not used now because everything is a character.
#' @param root root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.
#' @details Have not written this section yet
#' @return a \link{data.frame} or a \link{list}. It does not return a path.
#' @export
#' @examples
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Clock"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Weather"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "Metadata"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "Physical"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "SoilWater"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "Organic"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "Chemical"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "InitialWater"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
#' soil.child = "InitialN"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "SurfaceOrganicMatter"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "MicroClimate"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager"))
#' (edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Report"))
#' }
#'

extract_data_apsimx <- function(file = "", src.dir = ".",
                                node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager","Report", "Operations", "Other"),
                                soil.child = c("Metadata", "Water", "InitialWater",
                                          "Chemical", "Physical", "Analysis", "SoilWater",
                                          "InitialN", "CERESSoilTemperature", "Sample",
                                          "Solute", "NO3", "NH4", "Urea",
                                          "Nutrient", "Organic", "Swim3"),
                                parm = NULL,
                                digits = 3,
                                root = NULL){
  #### Beginning of function ----
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }

  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)

  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to inspect.")
  }

  node <- match.arg(node)
  soil.child <- match.arg(soil.child)

  if(soil.child %in% c("Nutrient")) stop("Not implemented yet", call. = FALSE)

  ## This matches the specified file from a list of files
  ## Notice that the .apsimx extension will be added here
  file <- match.arg(file, file.names)

  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))

  find.root <- TRUE
  other.parm.flag <- 1
  if(node == "Other" && is.numeric(parm)) parm <- as.integer(parm)
  parm.path.0 <- paste0(".", apsimx_json$Name) ## Root
  ## I think that everything I might want to look at
  ## is under this Children/Children node
  ## It looks like I need to 'find' the "Models.Core.Simulation" node
  fcsn <- grep("Models.Core.Simulation", apsimx_json$Children, fixed = TRUE)

  ret_df <- NULL ## This is the returned data frame
  ret_lst <- NULL
  ## When node == "Other" root should always be missing (not sure)
  if((node == "Other" && length(fcsn) > 1) || (node == "Other" && (is.null(parm) || is.integer(parm)))){
    parm.path <- parm.path.0
    other.parm.flag <- -1
    find.root <- FALSE
    ## Process 'parm'
    if(is.null(parm) || is.integer(parm)){
      if(is.null(parm)) parm <- 1L
      if(length(parm) > 1)
        stop("'parm' should be of length = 1", call. = FALSE)
      if(parm == 0){
        ret_df <- data.frame(Level_0 = gsub(".", "", parm.path.0, fixed = TRUE))
      }
      ## If parm is null there are different levels of display
      if(parm == 1){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(root.names.level.1) - 1)),
                           second_level = root.names.level.1)
        ret_df <- pdat
      }
      ## If parm == 2
      if(parm == 2){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        second.level.names <- NULL
        third.level.names <- NULL
        for(i in seq_along(root.names.level.1)){
          chld.len <- length(apsimx_json$Children[[i]]$Children)
          rep.times <- ifelse(chld.len == 0, 1, chld.len)
          second.level.names <- c(second.level.names, c(root.names.level.1[i], rep(".", times = rep.times - 1)))
          ## Need to generate third level names
          third.level.names.0 <- sapply(apsimx_json$Children[[i]]$Children, FUN = function(x) x$Name)
          if(length(third.level.names.0) == 0) third.level.names.0 <- "."
          third.level.names.1 <- c(third.level.names, third.level.names.0)
          third.level.names <- unlist(third.level.names.1)
        }
        ### Is this table good enough?
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(second.level.names) - 1)),
                           second_level = second.level.names,
                           third_level = third.level.names)
        ret_df <- pdat
      }
      ## If parm == 3
      if(parm == 3){
        first.column.name <- gsub(".", "", parm.path.0, fixed = TRUE)
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        second.level.names <- NULL
        third.level.names <- NULL
        fourth.level.names <- NULL
        k <- 1
        for(i in seq_along(root.names.level.1)){
          chld.len <- length(apsimx_json$Children[[i]]$Children)
          rep.times <- ifelse(chld.len == 0, 1, chld.len)
          second.level.names <- c(second.level.names, c(root.names.level.1[i], rep(".", times = rep.times - 1)))
          ## Need to generate third level names
          third.level.names.0 <- sapply(apsimx_json$Children[[i]]$Children, FUN = function(x) x$Name)
          if(length(third.level.names.0) == 0) third.level.names.0 <- "."
          for(j in seq_along(third.level.names.0)){
            if(length(third.level.names.0) == 1){
              if(third.level.names.0 == "."){
                third.level.names <- c(third.level.names, ".")
                fourth.level.names <- c(fourth.level.names, ".")
                next
              }
            }
            fourth.level <- apsimx_json$Children[[i]]$Children
            if(length(fourth.level[[j]]) == 0){
              third.level.names <- c(third.level.names, ".")
              fourth.level.names <- c(fourth.level.names, ".")
              next
            }
            if(length(fourth.level[[j]]$Children) == 0){
              second.level.names <- c(second.level.names, rep(".", times = length(names(fourth.level[[j]])) - 1))
              third.level.names <- c(third.level.names, c(third.level.names.0[j], rep(".", times = length(names(fourth.level[[j]])) - 1)))
              fourth.level.names <- c(fourth.level.names, names(fourth.level[[j]]))
            }else{
              second.level.names <- c(second.level.names, rep(".", times = length(names(fourth.level[[j]])) - 1))
              fourth.level.names <- c(fourth.level.names, names(fourth.level[[j]]))
              third.level.names <- c(third.level.names, third.level.names.0[j],
                                     rep(".", times = length(names(fourth.level[[j]])) - 1))
            }
          }
        }
        ### Is this table good enough?
        pdat <- data.frame(first_level = c(first.column.name, rep(".", length(fourth.level.names) - 1)),
                           second_level = second.level.names,
                           third_level = third.level.names,
                           fourth_level = fourth.level.names)
        ret_df <- pdat
      }
      if(parm == 4) stop("Not implemented yet")
    }else{
      ## This gets triggered if there are multiple simulations
      ## and parm is a character. If it is a list it will be
      ## handled by the code in the section at the bottom
      if(!is.list(parm)){
        if(!is.character(parm))
          stop("parm should be a character string when node = 'Other' and is not a number", call. = FALSE)
        if(!grepl(".", parm, fixed = TRUE))
          stop("'parm' needs to be a json path")
        if(parm == ".")
          stop("'parm' needs to be a proper json path")
        pparm <- strsplit(parm, split = ".", fixed = TRUE)[[1]]
        root.name.level.0 <- gsub(".", "", parm.path.0, fixed = TRUE)
        if(pparm[2] != root.name.level.0)
          stop(paste("First parm element does not match:",
                     paste(root.name.level.0, collapse = " ")), call. = FALSE)
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0 || all(is.na(wroot1)))
          stop(paste("Second element of parm did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        ## Need to test if the fourth element of pparm is a node
        nodes <- c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager","Report", "Operations", "Other")
        if(pparm[4] %in% nodes){
          ## This amounts to guessing that root should be of length 1
          root <- list(pparm[3])
        }else{
          ## This amounts to guessing that pparm[4] should be the second element in
          root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                       FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          root2 <- pparm[4]
          wroot2 <- grep(as.character(root2), root.names.level.2)
          if(length(wroot2) == 0 || all(is.na(wroot2)))
            stop(paste("Third element of parm did not match:",
                       paste(root.names.level.2, collapse = " ")), call. = FALSE)
          if(pparm[5] %in% nodes){
            root <- list(pparm[3], pparm[4])
          }else{
            root.names.level.3 <- vapply(apsimx_json$Children[[wroot1]]$Children[[wroot2]]$Children,
                                         FUN = function(x) x$Name,
                                         FUN.VALUE = "character")
            root3 <- pparm[5]
            wroot3 <- grep(as.character(root3), root.names.level.3)
            if(length(wroot3) == 0)
              stop(paste("Fourth element of parm did not match:",
                         paste(root.names.level.3, collapse = " ")), call. = FALSE)
          }
        }
        parm.path <- parm
      }else{
        other.parm.flag <- 1
      }
    }
  }

  if((length(fcsn) > 1 || !is.null(root)) && find.root){
    if(is.null(root)){
      cat("Simulation structure: \n")
      str_list(apsimx_json)
      stop("more than one simulation found and no root node label has been specified \n select one of the children names above", call. = FALSE)
    }else{
      ## Parse root
      root <- parse_root(root)
      if(length(root) > 3)
        stop("At the moment 3 is the maximum length for root for this function", call. = FALSE)
      if(length(root) == 1){
        nms <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                      FUN.VALUE = "character")
        fcsn <- grep(as.character(root), nms)
        if(length(fcsn) == 0 || length(fcsn) > 1){
          cat("Children:", nms, "\n")
          stop("'root' not found. Choose one of the options above", call. = FALSE)
        }
        parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
        parent.node <- apsimx_json$Children[[fcsn]]$Children
      }else{
        if(length(root) == 2){
          root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
          wcore1 <- grep(as.character(root[1]), root.node.0.names)
          if(length(wcore1) == 0 || length(wcore1) > 1){
            cat("Children:", root.node.0.names, "\n")
            stop("no root node label in position 1 found or root is not unique", call. = FALSE)
          }
          root.node.0 <- apsimx_json$Children[[wcore1]]
          root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
          wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
          if(length(wcore2) == 0 || length(wcore2) > 1)
            stop("no root node label in position 2 found or root is not unique")
          parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children
          parm.path.1 <- paste0(parm.path.0,".",apsimx_json$Children[[wcore1]]$Children[[wcore2]])
          ## Is this only a problem for factorials?
          if(length(parm.path.1) > 1){
            parm.path.1 <- paste(parm.path.0, apsimx_json$Children[[wcore1]]$Name, apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Name, sep = ".")
          }
        }
        if(length(root) == 3){
          root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
          wcore1 <- grep(as.character(root[1]), root.node.0.names)
          if(length(wcore1) == 0 || length(wcore1) > 1)
            stop("no root node label in position 1 found or root is not unique")
          root.node.0 <- apsimx_json$Children[[wcore1]]
          root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
          wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
          if(length(wcore2) == 0 || length(wcore2) > 1)
            stop("no root node label in position 2 found or root is not unique")
          root.node.1 <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]
          root.node.1.child.names <- sapply(root.node.1$Children, function(x) x$Name)
          wcore3 <- grep(as.character(root[3]), root.node.1.child.names)
          if(length(wcore3) == 0 || length(wcore3) > 1)
            stop("no root node label in position 3 found or root is not unique")
          parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children
          parm.path.1 <- paste0(parm.path.0,".",apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]])
          ### Is this a problem for factorials only?
          if(length(parm.path.1) > 1){
            parm.path.1 <- paste(parm.path.0, apsimx_json$Children[[wcore1]]$Name, apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Name,
                                 apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Name, sep = ".")
          }
        }
      }
    }
  }else{
    if(find.root){
      parent.node <- apsimx_json$Children[[fcsn]]$Children
      parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
    }
  }

  if(node == "Clock"){
    wlc <- function(x) grepl("Models.Clock", x$`$type`, ignore.case = TRUE)
    wlcl <- sapply(parent.node, FUN = wlc)
    if(all(wlcl == FALSE)){
      stop("Clock not found")
    }
    clock.node <- as.list(parent.node[wlcl])[[1]]
    start.name <- grep("start", names(clock.node), ignore.case = TRUE, value = TRUE)
    end.name <- grep("end", names(clock.node), ignore.case = TRUE, value = TRUE)
    ret_df <- data.frame(Start = clock.node[[start.name]],
                      End = clock.node[[end.name]])
    ## It is possible for the 'start' and 'end' to be called: 'Start' and 'End'
    ## It is also possible for them to be called 'StartDate' and 'EndDate'
    ## I think APSIM-X is in a state of change and eventually this will
    ## stabilize. At the moment 'Maize' and 'Barley' do not agree.
    ## Final name is 'parm.path', but this is 1.2
    ## parm.path <- paste0(parm.path.1,".",parent.node[wlcl][[1]]$Name)
  }

  ## The previous creates a list
  if(node == "Weather"){
    ## Extract the list which has a component Name == "Weather"
    wlw <- function(x) grepl("Models.Climate.Weather|Models.Weather", x$`$type`)
    wlwl <- sapply(parent.node, FUN = wlw)
    if(all(wlwl == FALSE)){
      stop("Weather not found")
    }
    weather.node <- parent.node[wlwl]
    ## Select the string which has a met file
    gf1 <- function(x) grep(".met$", x, value = TRUE)
    ret_df <- data.frame(Met_file = as.character(sapply(weather.node, gf1)))
    ## parm.path <- paste0(parm.path.1, ".", parent.node[wlwl][[1]]$Name)
  }

  ## From here on there is an important component that lives inside
  ## 'Models.Core.Zone'
  if(find.root){
    wcz <- grepl("Models.Core.Zone", parent.node)
    if(sum(wcz) < 0.5)
      stop("Core Zone Simulation not found", call. = FALSE)
    core.zone.node <- parent.node[wcz][[1]]$Children

    parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
  }

  if(node == "Soil"){
    #### Soil node ----
    wsn <- grepl("Models.Soils.Soil", core.zone.node)
    if(all(wsn == FALSE)){
      stop("Soil not found")
    }
    soil.node <- core.zone.node[wsn]

   parm.path.2.1 <- paste0(parm.path.2, ".", soil.node[[1]]$Name)

   ## Print some basic soil information
   ret_df1 <- data.frame(Soil_Type = soil.node[[1]]$SoilType,
                      Latitude = soil.node[[1]]$Latitude,
                      Longitude = soil.node[[1]]$Longitude)

   if(length(soil.node) != 1) stop("soil.node not equal to one")

   soil.children.names <- sapply(soil.node[[1]]$Children, function(x) x$Name)

   ## cat("Soil children:", soil.children.names, "\n")

    if(soil.child == "Metadata"){
      parm.path <- parm.path.2.1
      metadata <- NULL
      for(i in names(soil.node[[1]])){
        if(i %in% c("Name","Children","IncludeInDocumentation","Enabled","ReadOnly")) next
        val <- as.character(ifelse(is.null(soil.node[[1]][[i]]),NA,soil.node[[1]][[i]]))
        if(!is.na(val) && nchar(val) > options()$width-30) val <- paste(strtrim(val, options()$width-30),"...")
        metadata <- rbind(metadata, data.frame(parm = i, value = val))
      }

      if(missing(parm)){
        ret_lst <- list(soil = ret_df1, metadata = metadata)
      }else{
        if(!(parm %in% metadata[["parm"]])) stop("parm does not match a parameter in metadata")
        ret_df <- metadata[metadata$parm == parm,]
      }

    }else{
      ## Pick which soil component we want to look at
      ## Which is not 'Metadata"
      ## If "InitialWater" is present then use it
      wsiw <- "InitialWater" %in% soil.children.names
      if(soil.child %in% c("Water", "InitialWater")){
        if(soil.child == "InitialWater" && isFALSE(wsiw)){
          wsiw2 <- grep("initial water", soil.children.names, ignore.case = TRUE)
          if(length(wsiw2) > 0){
            soil.child <-  grep("initial water", soil.children.names, ignore.case = TRUE, value = TRUE)
          }else{
            wsiw3 <- grep("^Water", soil.children.names)
            if(length(wsiw3) > 0){
              soil.child <- "Water"
            }else{
              soil.child <- "InitialWater"
            }
          }
        }
      }

      if(soil.child != "Solute"){
        wsc <- which(soil.child == soil.children.names)
      }else{
        wsc <- which(soil.children.names %in% c("NO3", "NH4", "Urea"))
      }

      if(soil.child == "Water" && length(wsc) == 2) wsc <- wsc[2]

      if(length(wsc) == 0) stop("soil.child likely not present")

      selected.soil.node.child <- soil.node[[1]]$Children[wsc]
    }
    ## if(soil.child == "InitialWater") browser()
    ## For some variables now it is the time to print
    ## The code below is not strictly needed but it is here
    ## in case I need a second level of soil in the future
    first.level.soil <- c("Water", "Physical",
                          "Chemical", "Analysis", "InitialWater", "Initial water", "initial water",
                          "InitialN", "SoilWater", "Analysis",
                          "CERESSoilTemperature", "Organic", "Swim3")
    if(soil.child %in% first.level.soil){
      ## Assuming there is only one 'relevant' level here
      ## This parameter level would be 2.1.1
      parm.path <- paste0(parm.path.2.1, ".", selected.soil.node.child[[1]]$Name)
      enms <- c("IncludeInDocumentation", "Enabled", "ReadOnly", "Children", "Name", "$type")
      cnms <- setdiff(names(selected.soil.node.child[[1]]), enms)
      ## print(names(selected.soil.node.child[[1]]))

      if(soil.child == "Physical")
        cnms <- c(cnms, "Crop LL", "Crop KL", "Crop XF")

      soil.d1 <- NULL
      soil.d2 <- NULL
      soil.d3 <- NULL
      col.nms <- NULL
      d3.col.nms <- NULL

      for(ii in cnms){

        tmp <- selected.soil.node.child[[1]][ii][[1]]

        if(ii %in% c("Crop LL", "Crop KL", "Crop XF")){
          for(j in seq_along(selected.soil.node.child[[1]]$Children)){
            crop.name <- gsub("Soil", "", selected.soil.node.child[[1]]$Children[[j]]$Name)
            if(ii == "Crop LL") tmp <- selected.soil.node.child[[1]]$Children[[j]]$LL
            if(ii == "Crop KL") tmp <- selected.soil.node.child[[1]]$Children[[j]]$KL
            if(ii == "Crop XF") tmp <- selected.soil.node.child[[1]]$Children[[j]]$XF
            d3.col.nms <- c(d3.col.nms, gsub("Crop", crop.name, ii))
            vals <- as.vector(unlist(tmp))
            soil.d3 <- cbind(soil.d3, vals)
          }
        }

        if(length(tmp) == 0) next
        if(length(tmp) == 1){
          soil.d1 <- rbind(soil.d1,
                           data.frame(parm = ii, value = as.character(tmp)))
        }
        if(length(tmp) > 1){
          if(!ii %in% c("Crop LL", "Crop KL", "Crop XF")){
            if(grepl("Metadata", ii)) next
            col.nms <- c(col.nms, ii)
            vals <- as.vector(unlist(tmp))
            if(is.null(vals)) vals <- rep(".", length(tmp))
            soil.d2 <- cbind(soil.d2, vals)
          }
        }
      }

      if(missing(parm)){
        ## Print first set of soil parameters
        if(!is.null(soil.d1)) ret_df <- soil.d1
        ## Print second set of soil parameters
        if(!is.null(soil.d2)){
          soil.d2 <- as.data.frame(soil.d2)
          if(length(names(soil.d2)) != length(col.nms)){
            cat("I encountered a problem \n")
            cat("Names for the data.frame are:", col.nms, "\n")
            cat("The data frame has:", ncol(soil.d2), " columns \n")
            print(utils::head(soil.d2))
            stop("Encountered an error", call. = FALSE)
          }
          names(soil.d2) <- col.nms
          if(is.null(soil.d1)){
            ret_df <- soil.d2
          }else{
            ret_lst <- list(first = soil.d1, second = soil.d2)
          }
        }
        ## Print third set of crop-soil parameters
        if(!is.null(soil.d3)){
          soil.d3 <- as.data.frame(soil.d3)
          names(soil.d3) <- d3.col.nms
          soil.d3 <- subset(soil.d3, select = sort(names(soil.d3)))
          if(!is.null(soil.d1) && !is.null(soil.d2)){
           ret_lst <- list(first = soil.d1, soil.layers = soil.d2, crop = soil.d3)
          }else{
            if(is.null(soil.d1) && !is.null(soil.d2)){
              ret_df <- list(soil.layers = soil.d2, crop = soil.d3)
            }else{
              ret_df <- soil.d3
            }
          }
        }
      }else{
        parm.physical.found <- FALSE
        ## Print first set of soil parameters
        if(!is.null(soil.d1)){
          if(parm %in% names(soil.d1)){
            ##print(knitr::kable(soil.d1[soil.d1$parm == parm,], digits = digits))
            ret_df <- soil.d1[, parm, drop = FALSE]
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d1$parm){
            ret_df <- soil.d1[soil.d1$parm == parm,]
            parm.physical.found <- TRUE
          }
        }
        ## Print second set of soil parameters
        if(!is.null(soil.d2)){
          soil.d2 <- as.data.frame(soil.d2)
          names(soil.d2) <- col.nms
          ## print(knitr::kable(soil.d2[soil.d2$parm == parm,], digits = digits))
          if(parm %in% names(soil.d2)){
            ret_df <- soil.d2[, parm, drop = FALSE]
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d2$parm){
            ret_df <- soil.d2[soil.d2$parm == parm,]
            parm.physical.found <- TRUE
          }
        }
        ## Print third set of crop-soil parameters
        if(!is.null(soil.d3)){
          soil.d3 <- as.data.frame(soil.d3)
          names(soil.d3) <- d3.col.nms
          soil.d3 <- subset(soil.d3, select = sort(names(soil.d3)))
          ##print(knitr::kable(soil.d3[soil.d3$parm == parm,], digits = digits))
          if(parm %in% names(soil.d3)){
            ret_df <- soil.d3[, parm, drop = FALSE]
            parm.physical.found <- TRUE
          }
          if(parm %in% soil.d3$parm){
            ret_df <- soil.d3[soil.d3$parm == parm,]
            parm.physical.found <- TRUE
          }
        }
        if(!parm.physical.found)
          stop("Soil physical parameter not found", call. = FALSE)
      }
    }
  }

  second.level.soil <- c("Solute", "NO3", "NH4", "Urea")

  if(soil.child %in% second.level.soil){

    soil.d2 <- NULL
    soil.d3 <- NULL
    enms <- c("IncludeInDocumentation", "Enabled", "ReadOnly", "Children", "Name", "$type")
    ssnc <- selected.soil.node.child

    if(soil.child == "Solute"){

      solutes <- sapply(ssnc, function(x) x$Name)

      parm.path <- paste0(parm.path.2.1, ".", "Solute")

      if(missing(parm)){
        ## cat("Solutes:", solutes, "\n")
        ## Do I need to do anything here?
      }else{
        ret_lst <- vector('list', length = length(solutes))
        names(ret_lst) <- solutes
        sel.parm <- grep(parm[[1]], solutes)
        if(length(sel.parm) == 0)
          stop("'parm' should be one of: ", paste(solutes, collapse = ", "), call. = FALSE)
        solutes <- parm[[1]]
      }

      ret_lst <- vector('list', length = length(solutes))
      names(ret_lst) <- solutes

      for(j in seq_along(solutes)){
        ## cat("\nSolute:", solutes[j], "\n")
        ssnc.solute <- ssnc[[j]]
        if(length(unlist(ssnc.solute$Thickness)) != length(unlist(ssnc.solute$InitialValues))){
          cat("Length of Thickness:", length(unlist(ssnc.solute$Thickness)), "\n")
          cat("Length of InitialValues:", length(unlist(ssnc.solute$InitialValues)), "\n")
          stop("Length of 'Thickness' does not match length of 'InitialValues'", call. = FALSE)
        }
        soil.d1 <- data.frame(Thickness = unlist(ssnc.solute$Thickness),
                              InitialValues = unlist(ssnc.solute$InitialValues))
        if(!is.null(soil.d1)){
          if(is.null(parm)){
            ret_lst[[j]][["first"]] <- soil.d1
          }else{
            if(length(parm) == 1){
              if(parm %in% c("Thickness", "InitialValues")){
                ret_lst[[j]][["first"]] <- soil.d1[, parm, drop = FALSE]
              }else{
                ret_lst[[j]][["first"]] <- soil.d1
              }
            }
          }
        }

        cnms <- setdiff(names(ssnc.solute), enms)
        for(k in cnms){
          if(k %in% c("Thickness", "InitialValues")) next
          tmp <- ssnc.solute[k][[1]]
          if(is.null(tmp)) tmp <- ""
          soil.d2 <- rbind(soil.d2,
                           data.frame(parm = k, value = as.character(tmp)))
        }
        if(missing(parm) || length(parm) == 1){
          if(!is.null(soil.d2)){
            ret_lst[[j]][["second"]] <- soil.d2
          }
        }else{
          if(is.list(parm) && is.numeric(parm[[2]])){
            soil.d2.s <- soil.d2[parm[[2]], , drop = FALSE]
            if(!is.null(soil.d2)){
              ret_lst[[j]][["second"]] <- soil.d2.s
            }
          }
        }
        soil.d2 <- NULL
      }
    }
  }

  if(node == "SurfaceOrganicMatter"){
    ## Which is 'SurfaceOrganicMatter'
    wsomn <- grepl("Models.Surface.SurfaceOrganicMatter", core.zone.node)
    if(all(wsomn == FALSE)){
      stop("SurfaceOrganicMatter not found")
    }
    som.node <- core.zone.node[wsomn][[1]]

    ## parm.path <- paste0(parm.path.2,".",som.node$Name)
    ## The relevant components might be unpredictable
    ## Will need to find a better method in the future
    names.som.node0 <- names(som.node)
    som.names.table <- c("InitialResidueName", "InitialResidueType", "InitialResidueMass",
                         "InitialStandingFraction", "InitialCPR", "InitialCNR")
    wsomnn <- names.som.node0 %in% som.names.table
    values.som.node0 <- unlist(som.node)
    wsomvn <- names(values.som.node0) %in% som.names.table
    som.d <- data.frame(parm = names(som.node)[wsomnn],
                        value = as.vector(values.som.node0[wsomvn]))

    if(missing(parm)){
      ret_df <- som.d
    }else{
      if(parm %in% som.d$parm){
        ret_df <- som.d[som.d$parm == parm, ]
      }else{
        stop("Surface OM parameter not found", call. = FALSE)
      }
    }
  }

  if(node == "MicroClimate"){
    ## Which is 'MicroClimate'
    ## This only works if it is under Field (old versions, before 2023-12-10)
    wmcn <- grepl("Models.MicroClimate", core.zone.node)
    mcincz <- TRUE ## MicroClimate in core.zone
    if(all(wmcn == FALSE)){
      wmcnf <- function(x) grepl("Models.MicroClimate", x$`$type`)
      wmcn <- sapply(parent.node, FUN = wmcnf)
      mcincz <- FALSE
      if(all(wmcn == FALSE))
        stop("MicroClimate not found")
    }
    if(mcincz){
      microclimate.node <- core.zone.node[wmcn][[1]]
    }else{
      microclimate.node <- parent.node[wmcn][[1]]
    }

    parm.path <- paste0(parm.path.2,".", microclimate.node$Name)

    microclimate.d <- data.frame(parm = names(microclimate.node)[2:9],
                                 value = as.vector(unlist(microclimate.node)[2:9]))

    if(missing(parm)){
      ret_df <- microclimate.d
    }else{
      if(parm %in% microclimate.d$parm){
        ret_df <- microclimate.d[microclimate.d$parm == parm, ]
      }else{
        stop("MicroClimate parameter not found", call. = FALSE)
      }
    }
  }

  if(node == "Crop"){
    ## Which is 'Crop'
    wmmn <- grepl("Models.Manager", core.zone.node)
    if(all(wmmn == FALSE)){
      stop("Crop not found")
    }
    manager.node <- core.zone.node[wmmn]

    #cat("Manager Name:",names(manager.node[[1]]),"\n")
    ## Which element has the crop information?
    wcn <- grepl("CultivarName", manager.node)
    crop.node <- manager.node[wcn][[1]]$Parameters

    ## This would be 2.1.1
    parm.path <- paste0(parm.path.2,".",manager.node[wcn][[1]]$Name)

    mat <- matrix(NA, nrow = length(crop.node), ncol = 2,
                  dimnames = list(NULL,c("parm","value")))
    j <- 1
    for(i in 1:length(crop.node)){
      mat[j,1] <- crop.node[[i]]$Key
      mat[j,2] <- crop.node[[i]]$Value
      j <- j + 1
    }

    ret_df <- as.data.frame(mat)
  }

  if(node == "Manager"){
    wmmn <- grepl("Models.Manager", core.zone.node)
    if(all(wmmn == FALSE)){
      stop("Manager not found")
    }
    manager.node <- core.zone.node[wmmn]
    parm.path <- parm.path.2
    ## Print available Manager components
    manager.node.names <- sapply(manager.node, FUN = function(x) x$Name)
    ## cat("Management Scripts: ", manager.node.names,"\n")

    if(!is.null(parm)){
      parm1 <- parm[[1]]
      position <- parm[[2]]
      find.manager <- grep(parm1, manager.node.names, ignore.case = TRUE)
      selected.manager.node <- manager.node.names[find.manager]
      parm.path <- paste0(parm.path.2, ".", selected.manager.node)

      if(is.na(position)){
        ms.params <- manager.node[[find.manager]]$Parameters
        if(length(ms.params) == 0) warning("parameter not found")

        mat <- matrix(NA, ncol=2, nrow = length(ms.params),
                      dimnames = list(NULL, c("parm", "value")))

        if(length(ms.params) > 0){
          for(j in 1:length(ms.params)){
            mat[j,1] <- ms.params[[j]]$Key
            mat[j,2] <- ms.params[[j]]$Value
          }
        }
        ## cat("Name: ", selected.manager.node, "\n")
        ret_df <- as.data.frame(mat)
        ## cat("\n")
      }

      if(!is.na(position)){
        ms.params <- manager.node[[find.manager]]$Parameters
        if(length(ms.params) == 0) warning("no parameters found")
        mat <- matrix(NA, ncol=2, nrow = length(position),
                      dimnames = list(NULL, c("parm", "value")))

        k <- 1
        for(j in 1:length(ms.params)){
          if(j == position){
            mat[k,1] <- ms.params[[j]]$Key
            mat[k,2] <- ms.params[[j]]$Value
            k <- k + 1
          }
        }
        ## cat("Name: ", selected.manager.node,"\n")
        parm2 <- ms.params[[position]]$Key
        ## cat("Key:", ms.params[[position]]$Key, "\n")
        ret_df <- as.data.frame(mat)
        ## cat("\n")
      }
    }else{
      ret_lst <- vector("list", length = length(manager.node))
      for(i in seq_along(manager.node)){
        ret_lst[[i]] <- manager.node[[i]]
        names(ret_lst) <- manager.node.names
      }
    }
  }

  if(node == "Operations"){
    won <- grepl("Models.Operations", core.zone.node)
    if(length(won) == 0)
      stop("Operations node not found", call. = FALSE)
    operations.node <- core.zone.node[won]

    if(length(operations.node) > 1)
      stop("Not ready to handle multiple 'Operations'", call. = FALSE)

    if(is.null(operations.node[[1]]$Operation))
      stop("'Operation' child node not found", call. = FALSE)

    len.op <- length(operations.node[[1]]$Operation)
    op.mat <- matrix(nrow = len.op, ncol = 3,
                     dimnames = list(NULL, c("Date", "Action", "Line")))

    for(i in seq_len(len.op)){
      op.mat[i, ] <- c(operations.node[[1]]$Operation[[i]]$Date,
                       operations.node[[1]]$Operation[[i]]$Action,
                       operations.node[[1]]$Operation[[i]]$Line)
    }

    if(is.null(parm)){
      parm.path <- paste0(parm.path.2, ".Operations")
      ret_df <- as.data.frame(op.mat)
    }else{
      parm.path.3 <- paste0(parm.path.2, ".Operations")
      parm.path <- paste(parm.path.3, parm[[1]], parm[[2]], sep = ".")
      op.mat.dat <- as.data.frame(op.mat)
      op.mat.dat.parm <- op.mat.dat[parm[[1]], parm[[2]], drop = FALSE]
      ret_df <- op.mat.dat.parm
    }
  }

  if(node == "Report"){
    wrn <- grepl("Models.Report", core.zone.node)
    if(all(wrn == FALSE)){
      stop("Report not found")
    }
    report.node <- core.zone.node[wrn]
    parm.path <- parm.path.2
    ## Print available Manager components
    report.node.names <- sapply(report.node, FUN = function(x) x$Name)

    tmp <- vector("list", length = length(report.node.names))
    for(i in 1:length(report.node)){
      ## Variable Names
      vn <- as.data.frame(unlist(report.node[[i]]$VariableNames))
      names(vn) <- "VariableNames"
      en <- as.data.frame(unlist(report.node[[i]]$EventNames))
      names(en) <- "EventNames"
      tmp[[i]] <- list(vn = vn, en = en)
    }

    if(missing(parm)){
      ret_lst <- vector("list", length = length(tmp))
      for(i in 1:length(tmp)){
        cat("Report name:", report.node[[i]]$Name, "\n")
        ret_lst[[i]] <- list(VariableNames = tmp[[i]]$vn, EventNames = tmp[[i]]$en)
        cat("\n")
      }
      parm.path <- paste0(parm.path.2,".", report.node.names)
    }else{
      if(!is.list(parm)){
        if(length(report.node.names) > 1)
          stop("More than one Report is present. Use a list to choose one.")
        if(!grepl(parm, "VariableNames") && !grepl(parm, "EventNames"))
          stop("parm should contain either VariableNames or EventNames")
        if(parm == "VariableNames") ret_df <- as.data.frame(tmp[[i]]$vn)
        if(parm == "EventNames") ret_df <- as.data.frame(tmp[[i]]$en)
        parm.path <- paste0(parm.path.2,".", report.node.names)
      }else{
        if(!any(grepl(parm[[1]], report.node.names)))
          stop("the first element of parm should match a report name")
        wr2p <- grep(parm[[1]], report.node.names)
        if(is.na(parm[[2]])){
          print(knitr::kable(tmp[[wr2p]]$vn))
          print(knitr::kable(tmp[[wr2p]]$en))
          parm.path <- paste0(parm.path.2,".", report.node.names[wr2p])
          position <- NA
        }else{
          if(grepl(parm[[2]],"VariableNames")) print(knitr::kable(tmp[[wr2p]]$vn))
          if(grepl(parm[[2]], "EventNames")) print(knitr::kable(tmp[[wr2p]]$en))
          parm.path <- paste0(parm.path.2,".", report.node.names[wr2p])
          parm1 <- grep(parm[[1]], report.node.names, value = TRUE)
          parm2 <- grep(parm[[2]], c("VariableNames", "EventNames"), value = TRUE)
          position <- wr2p
        }
      }
    }
  }

  #### Option for 'Other' character or 'list' ----
  if(node == "Other" && other.parm.flag > 0){
    ## I will develop this option assuming parm is a list or a character
    if(is.null(parm))
      stop("'parm' is missing", call. = FALSE)

    if(is.character(parm)){
      if(!grepl(".", parm, fixed = TRUE))
        stop("'parm' needs to be a proper json path")
      parm.path <- parm
      if(parm == ".")
        stop("'parm' path is empty")
      pparm <- strsplit(parm, split = ".", fixed = TRUE)[[1]]
      root.name.level.0 <- gsub(".", "", parm.path.0, fixed = TRUE)
      if(pparm[2] != root.name.level.0)
        stop(paste("First 'parm' element does not match:",
                   paste(root.name.level.0, collapse = " ")), call. = FALSE)
      children.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
      if(length(pparm) == 2){
        cat("Simulation children names:")
        print(children.names)
        cat("\n")
      }
      if(length(pparm) == 3){
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0)
          stop(paste("Second 'parm' element did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        cat(paste(root1, "Names level 2:"))
        print(root.names.level.2)
        cat("\n")
      }
      if(length(pparm) == 4){
        root1 <- pparm[3]
        ## Guess if 'root' is contained in the first level of names
        root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        wroot1 <- grep(as.character(root1), root.names.level.1)
        if(length(wroot1) == 0)
          stop(paste("Second 'parm' element did not match:",
                     paste(root.names.level.1, collapse = " ")), call. = FALSE)
        root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        root2 <- pparm[4]
        wroot2 <- grep(as.character(root2), root.names.level.2)
        if(length(wroot2) == 0)
          stop(paste("Third 'parm' element did not match:",
                     paste(root.names.level.2, collapse = " ")), call. = FALSE)
        root.names.level.3 <- vapply(apsimx_json$Children[[wroot1]]$Children[[wroot2]]$Children,
                                     FUN = function(x) x$Name,
                                     FUN.VALUE = "character")
        cat(paste(root1, root2, "Names level 3:"))
        print(root.names.level.3)
        cat("\n")

      }
      if(length(pparm) > 4) stop("Not implemented yet", call. = FALSE)
    }

    if(is.list(parm)){
      if(length(parm) == 0)
        stop("Length of 'parm' list should be greater than zero", call. = FALSE)
      if(is.numeric(parm[[1]])){
        ### In this case the list will be used to create a parameter path
        if(parm[[1]] != 1)
          stop("First element of list should be equal to 1", call. = FALSE)
        parm.path.0 <- paste0(".", apsimx_json$Name) ## Root
        if(length(parm) >= 2){
          if(!is.numeric(parm[[2]]))
            stop("Second element of 'parm' should be numeric")
          root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.1) == 0)
            stop("'parm' index parm[[2]] not found at this level")
          if(length(parm[[2]]) == 1){
            if(parm[[2]] > 0){
              wlevel1 <- apsimx_json$Children[[parm[[2]]]]
              parm.path <- paste0(parm.path.0, ".", root.names.level.1[parm[[2]]])
            }else{
              parm.path <- parm.path.0
              pdat <- data.frame(second_level = root.names.level.1)
              ret_df <- pdat
            }
          }else{
            ### Here I assume that parm[[2]] is a vector
            if(length(parm[[2]]) > length(root.names.level.1))
              stop("Length of parm[[2]] should be less than number of elements in the second level")
            selected.levels <- root.names.level.1[parm[[2]]]
            parm.path <- paste0(parm.path, ".", selected.levels)
            pdat <- data.frame(second_level = root.names.level.1)
            ret_df <- pdat[parm[[2]],, drop = FALSE]
          }
        }
        if(length(parm) >= 3){
          root.names.level.2 <- vapply(wlevel1$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.2) == 0){
            ## This means that Children are empty
            ## Need to pass this to the next step
            root.names.level.2 <- names(wlevel1)
          }
          if(length(parm[[3]]) == 1){
            if(parm[[3]] > 0){
              wlevel2 <- apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]
              parm.path <- paste0(parm.path, ".", root.names.level.2[parm[[3]]])
            }else{
              pdat <- data.frame(third_level = root.names.level.2)
              ret_df <- pdat
            }
          }else{
            ### Here I assume that parm[[3]] is a vector
            if(length(parm[[3]]) > length(root.names.level.2))
              stop("Length of parm[[3]] should be less than number of elements in the third level")
            selected.levels <- root.names.level.2[parm[[3]]]
            parm.path <- paste0(parm.path.0, ".", selected.levels)
            pdat <- data.frame(third_level = root.names.level.2)
            ret_df <- pdat[parm[[3]],, drop = FALSE]
          }
        }
        if(length(parm) >= 4){
          root.names.level.3 <- vapply(wlevel2$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.3) == 0)
            root.names.level.3 <- names(wlevel2)
          if(length(parm[[4]]) == 1){
            if(parm[[4]] > 0){
              wlevel3 <- apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]
              parm.path <- paste0(parm.path, ".", root.names.level.3[parm[[4]]])
            }else{
              pdat <- data.frame(fourth_level = root.names.level.3)
              ret_df <- pdat
            }
          }else{
            ### Here I assume that parm[[4]] is a vector
            if(length(parm[[4]]) > length(root.names.level.3))
              stop("Length of parm[[4]] should be less than number of elements in the fourth level")
            selected.levels <- root.names.level.3[parm[[4]]]
            parm.path <- paste0(parm.path.0, ".", selected.levels)
            pdat <- data.frame(fourth_level = root.names.level.3)
            ret_df <- pdat[parm[[4]],, drop = FALSE]
          }
        }
        if(length(parm) >= 5){
          root.names.level.4 <- vapply(wlevel3$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(root.names.level.4) == 0)
            root.names.level.4 <- names(wlevel3)
          if(length(parm[[5]]) > 1)
            stop("Have not implemented this yet")
          if(parm[[5]] > 0){
            wlevel4 <- apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]]
            parm.path <- paste0(parm.path, ".", root.names.level.4[parm[[5]]])
          }else{
            pdat <- data.frame(fifth_level = root.names.level.4)
            ret_df <- pdat
          }
        }
        if(length(parm) >= 6){
          root.names.level.5 <- vapply(wlevel4$Children, FUN = function(x) x$Name,
                                       FUN.VALUE = "character")
          if(length(parm[[6]]) > 1)
            stop("Have not implemented this yet")
          if(parm[[6]] > 0){
            wlevel5 <- apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]]$Children[[parm[[6]]]]
            parm.path <- paste0(parm.path, ".", root.names.level.5[parm[[6]]])
          }else{
            pdat <- data.frame(sixth_level = root.names.level.5)
            ret_df <- pdat
          }
        }
        if(length(parm) >= 7)
          stop("Have not developed this yet", call. = FALSE)
      }else{
        # if(length(parm) == 1){
        #   tmpr <- grep_json_list(parm, apsimx_json$Children)
        #   print(tmpr)
        # }
        warning("This is a work in progress")
        #### node = 'Other' and list with characters ----
        if(length(fcsn) > 1){
          if(missing(root)){
            ## In this case the first element of the list should be the root
            if(length(parm) == 1)
              stop("'parm' should be at least length equal to 2", call. = FALSE)
            ## select one of the simulations
            simulation.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
            wsim <- grep(parm[[1]], simulation.names)
            if(length(wsim) == 0){
              cat("Simulation names:", paste(simulation.names, collpse = " "), "\n")
              stop("First element of 'parm' list does not match Simulation names", call. = FALSE)
            }else{
              fcsn <- wsim
              parm <- parm[2:length(parm)]
              if(parm[[1]] == "")
                stop("'parm' is empty")
            }
            parent.node <- apsimx_json$Children[[fcsn]]$Children
          }else{
            ### root is present
            if(length(root) == 1){
              nms <- vapply(apsimx_json$Children, FUN = function(x) x$Name,
                            FUN.VALUE = "character")
              fcsn <- grep(as.character(root), nms)
              if(length(fcsn) == 0 || length(fcsn) > 1){
                cat("Children:", nms, "\n")
                stop("'root' not found. Choose one of the options above", call. = FALSE)
              }
              parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
              parent.node <- apsimx_json$Children[[fcsn]]$Children
            }else{
              stop("Have not developed this yet")
            }
          }
        }else{
          parent.node <- apsimx_json$Children[[fcsn]]$Children
        }
        wcz <- grepl("Models.Core.Zone", parent.node)
        if(sum(wcz) < 0.5)
          stop("Core Simulation not found", call. = FALSE)
        core.zone.node <- parent.node[wcz][[1]]$Children
        parm.path.1 <- paste0(parm.path.0, ".", apsimx_json$Children[[fcsn]]$Name)
        parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
        tmp <- core.zone.node
        parm.path.2.1 <- parm.path.2

        if(length(parm) > 1){
          for(i in 1:(length(parm) - 1)){
            nms <- sapply(tmp, function(x) x$Name)
            print(nms)
            wcp <- grep(parm[[i]], nms)
            if(length(wcp) == 0){
              cat("Names: ", nms, "\n")
              cat("parm[[i]]", parm[[i]], "\n")
              stop("Parameter not found")
            }
            tmp <- tmp[[wcp]]
            if(!is.null(tmp$Children)) tmp <- tmp$Children
            ## Build the parm.path
            parm.path.2.1 <- paste0(parm.path.2.1, ".", nms[wcp])
          }
        }else{
          ## How do I find parm?
          tmp.names <- sapply(tmp, FUN = function(x) x$Name)
          wparm <- grep(parm[[1]], tmp.names)
          if(length(wparm) == 1){
            tmp <- tmp[[wparm]]
          }else{
            #### Work in progress ----
            for(j in seq_along(tmp)){
              tmp.nms <- names(tmp[[j]])
              for(jj in seq_along(parm)){
                wtn <- grep(parm[[jj]], tmp.nms)
                if(length(wtn) > 0){
                  print(tmp[[j]][wtn])
                }
              }
            }
          }
        }

        if(!is.null(tmp$Parameters)){
          wp <- grep(parm[[length(parm)]], tmp$Parameters)
          tmp2 <- tmp$Parameters[[wp]]
          ## Process parameter path
          parm.path <- paste0(parm.path.2.1, ".", tmp2$Key)
          print(knitr::kable(as.data.frame(tmp2)))
        }else{
          parm.path <- parm.path.2.1
          unpack_node(tmp)
        }
      }
    }
  }

  # if(print.path && node != "Other" && node != "Operations"){
  #   if(!missing(parm)){
  #     if(length(parm) == 1){
  #       parm.path <- paste0(parm.path, ".", parm)
  #     }else{
  #       if(!is.na(position)){
  #         parm.path <- paste0(parm.path, ".", parm2)
  #       }
  #     }
  #   }
  #   cat("Parm path:", parm.path,"\n")
  # }else{
  #   if(print.path) cat("Parm path:", parm.path,"\n")
  # }
  #### Return ----
  if(!is.null(ret_lst)){
    ret <- ret_lst
  }else{
    ret <- ret_df
  }
  return(ret)
}

#'
#' Extract initial values from a parameter path
#' @title Extract values from a parameter path
#' @name extract_values_apsimx
#' @param file file name to be run (the extension .apsimx is optional)
#' @param src.dir directory containing the .apsimx file to be run (defaults to the current directory)
#' @param parm.path parameter path either use inspect_apsimx or see example below
#' @return a vector with extracted parameter values from an APSIM file.
#' @export
#' @examples
#' \donttest{
#' ## Find examples
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## Extract parameter path
#' pp <- inspect_apsimx("Maize.apsimx", src.dir = extd.dir,
#'                      node = "Manager", parm = list("Fert", 1))
#' ppa <- paste0(pp, ".Amount")
#' ## Extract value
#' extract_values_apsimx("Maize.apsimx", src.dir = extd.dir, parm.path = ppa)
#' }
extract_values_apsimx <- function(file, src.dir, parm.path){

  .check_apsim_name(file)

  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)

  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to inspect.")
  }

  file <- match.arg(file, file.names)

  apsimx_json <- jsonlite::read_json(paste0(src.dir, "/", file))

  upp <- strsplit(parm.path, ".", fixed = TRUE)[[1]]
  upp.lngth <- length(upp)
  if(upp.lngth < 5) stop("Parameter path too short?")
  if(upp.lngth > 10) stop("Cannot handle this yet")
  ## upp[2] is typically "Simulations"
  if(apsimx_json$Name != upp[2])
    stop("Simulation root name does not match")
  wl3 <- which(upp[3] == sapply(apsimx_json$Children, function(x) x$Name))
  ## At this level I select among simulation children
  ## upp[3] is typically "Simulation"
  n3 <- apsimx_json$Children[[wl3]]
  ## Look for the first reasonable parameter
  wl4 <- which(upp[4] == sapply(n3$Children, function(x) x$Name))
  ## This is super dumb but I do not know how to do it otherwise
  ## Length is equal to 5
  if(upp.lngth == 5){
    if(upp[5] %in% names(n3$Children[[wl4]])){
      value <- n3$Children[[wl4]][[upp[5]]]
    }else{
      wl5 <- which(upp[5] == sapply(n3$Children[[wl4]]$Children, function(x) x$Name))
      if(length(wl5) == 0) stop("Parameter not found at level 5")
      value <- n3$Children[[wl4]]$Children[[wl5]][[upp[5]]]
    }
  }
  ## Length is equal to 6
  if(upp.lngth == 6){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
    if(upp[6] %in% names(n4$Children[[wl5]])){
      value <- n4$Children[[wl5]][[upp[6]]]
    }else{
      if("Parameters" %in% names(n4$Children[[wl5]])){
        wp <- grep(upp[6], n4$Children[[wl5]]$Parameters)
        if(length(wp) == 0) stop("Could not find parameter")
        value <- n4$Children[[wl5]]$Parameters[[wp]]$Value
      }else{
        wl6 <- which(upp[6] == sapply(n4$Children[[wl5]]$Children, function(x) x$Name))
        if(length(wl6) == 0) stop("Could not find parameter")
        value <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[6]]]
      }
    }
  }
  if(upp.lngth == 7){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
    n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
    wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
    value <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[7]]]
    if(is.null(value)){
      n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
      if("Command" %in% names(n6)){
        gpv <- grep(upp[7], n6$Command, value = TRUE)
        value <- as.numeric(strsplit(gpv, "=")[[1]][2])
      }
    }
  }
  if(upp.lngth == 8){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
    n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
    wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
    n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
    wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
    value <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]][[upp[8]]]
    if(is.null(value)){
      n7 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]
      if("Command" %in% names(n7)){
        gpv <- grep(upp[8], n7$Command, value = TRUE)
        value <- as.numeric(strsplit(gpv, "=")[[1]][2])
      }
    }
  }
  if(upp.lngth == 9){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
    n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
    wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
    n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
    wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
    n7 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]
    wl8 <- which(upp[8] == sapply(n7$Children, function(x) x$Name))
    value <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]][[upp[9]]]
    if(is.null(value)){
      n8 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]
      if("Command" %in% names(n8)){
        gpv <- grep(upp[9], n8$Command, value = TRUE)
        value <- as.numeric(strsplit(gpv, "=")[[1]][2])
      }
    }
  }
  if(upp.lngth == 10){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
    n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
    wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
    n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
    wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
    n7 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]
    wl8 <- which(upp[8] == sapply(n7$Children, function(x) x$Name))
    n8 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]
    wl9 <- which(upp[9] == sapply(n8$Children, function(x) x$Name))
    value <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]$Children[[wl9]][[upp[10]]]
    if(is.null(value)){
      n9 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]$Children[[wl9]]
      if("Command" %in% names(n9)){
        gpv <- grep(upp[10], n9$Command, value = TRUE)
        value <- as.numeric(strsplit(gpv, "=")[[1]][2])
      }
    }
  }
  return(value)
}

## Run a few tests for the examples
require(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

## Run inspect tests

run.inspect.tests <- get(".run.local.tests", envir = apsimx.options)

if(run.inspect.tests) ex.dir <- auto_detect_apsimx_examples()

if(run.inspect.tests){
  
  ## Barley
  i <- "Barley.apsimx"
  inspect_apsimx(i, src.dir = ex.dir, node = "Clock")        
  inspect_apsimx(i, src.dir = ex.dir, node = "Weather")        
  inspect_apsimx(i, src.dir = ex.dir, node = "MicroClimate")  
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Physical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater", parm = "SummerDate")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater", parm = "SummerCona")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Organic")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "InitialWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Chemical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute", parm = "NO3")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute", parm = list("NO3", 2)) 
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute", parm = list("NO3", "D0"))
  inspect_apsimx(i, src.dir = ex.dir, node = "SurfaceOrganicMatter")
  inspect_apsimx(i, src.dir = ex.dir, node = "Crop")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Fertilise", NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 1))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 2))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 3))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 4))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 5))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Sow on a fixed date", 6))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report")
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "VariableNames")
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "EventNames")

  ## Maize
  i <- "Maize.apsimx"
  inspect_apsimx(i, src.dir = ex.dir, node = "Clock")        
  inspect_apsimx(i, src.dir = ex.dir, node = "Weather") 
  inspect_apsimx(i, src.dir = ex.dir, node = "MicroClimate")  
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Physical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Organic")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Chemical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute", parm = "NH4")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute", parm = list("NH4", "DepthConstant"))
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "InitialWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "SurfaceOrganicMatter")
  inspect_apsimx(i, src.dir = ex.dir, node = "Crop")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('Fertilise', NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "VariableNames")
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "EventNames")
  
  ## Oats
  i <- "Oats"
  inspect_apsimx(i, src.dir = ex.dir, node = "Clock")        
  inspect_apsimx(i, src.dir = ex.dir, node = "Weather")   
  inspect_apsimx(i, src.dir = ex.dir, node = "MicroClimate")  
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Water")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater", parm = "SummerDate")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Organic")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Chemical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Solute")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "NO3")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "NO3", parm = "InitialValues")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "NO3", parm = "D0")
  inspect_apsimx(i, src.dir = ex.dir, node = "SurfaceOrganicMatter")
  inspect_apsimx(i, src.dir = ex.dir, node = "Crop")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('Fert', NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 1))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 2))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 3))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 4))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 5))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 6))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 7))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 8))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list('variable', 9))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "VariableNames")
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = "EventNames")
  
  ## OilPalm is good for testing Report
  i <- "OilPalm.apsimx"
  inspect_apsimx(i, src.dir = ex.dir, node = "Clock")        
  inspect_apsimx(i, src.dir = ex.dir, node = "Weather")    
  inspect_apsimx(i, src.dir = ex.dir, node = "MicroClimate")  
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Physical")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "SoilWater", parm = "SummerDate")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Organic")
  inspect_apsimx(i, src.dir = ex.dir, node = "Soil", soil.child = "Chemical")
  inspect_apsimx(i, src.dir = ex.dir, node = "SurfaceOrganicMatter")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager")
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Palm Management", NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Manager", parm = list("Palm Management", 3))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report")
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = list("Annual", NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = list("Monthly", NA))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = list("Annual", "Variable"))
  inspect_apsimx(i, src.dir = ex.dir, node = "Report", parm = list("Monthly", "Event"))
  
  
}

run.inspect.print.path.tests <- get(".run.local.tests", envir = apsimx.options)

if(run.inspect.print.path.tests){
  
  i <- "Barley.apsimx"
  #### Clock ####
  pp <- inspect_apsimx(i, src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Clock") 
    stop("Error in inspect_apsimx, Barley, Clock, print.path")
  pp <- inspect_apsimx(i, src.dir = ex.dir, parm = "Start", print.path = TRUE)
  if(pp != ".Simulations.Simulation.Clock.Start") 
    stop("Error in inspect_apsimx, Barley, Start, print.path")
  #### Weather ####
  pp <- inspect_apsimx(i, node = "Weather", src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Weather") 
    stop("Error in inspect_apsimx, Barley, Weather, print.path")
  #### Soil - Metadata ####
  pp <- inspect_apsimx(i, node = "Soil", src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil") 
    stop("Error in inspect_apsimx, Barley, Soil, Metadata, print.path")
  pp <- inspect_apsimx(i, node = "Soil", src.dir = ex.dir, 
                       parm = "Latitude", print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Latitude") ## I think this currently works as expected... ??
    stop("Error in inspect_apsimx, Barley, Soil, Metadata, Latitude, print.path")
  #### Soil - Physical ####
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Physical",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Physical") 
    stop("Error in inspect_apsimx, Barley, Soil, Physical, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Physical",
                       parm = "DUL", src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Physical.DUL") 
    stop("Error in inspect_apsimx, Barley, Soil, Physical, DUL, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Physical",
                       parm = "Barley XF", src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Physical.Barley XF") 
    stop("Error in inspect_apsimx, Barley, Soil, Physical, Barley XF, print.path")
  #### Soil - SoilWater ####
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "SoilWater",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.SoilWater") 
    stop("Error in inspect_apsimx, Barley, Soil, SoilWater, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "SoilWater", parm = "Salb",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.SoilWater.Salb") 
    stop("Error in inspect_apsimx, Barley, Soil, SoilWater, SWCON, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "SoilWater", parm = "SWCON",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.SoilWater.SWCON") 
    stop("Error in inspect_apsimx, Barley, Soil, SoilWater, SWCON, print.path")
  #### Soil - Chemical ####
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Chemical",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Chemical") 
    stop("Error in inspect_apsimx, Barley, Soil, Chemical, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Chemical", parm = "PH",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Chemical.PH") 
    stop("Error in inspect_apsimx, Barley, Soil, Chemical, PH, print.path")
  #### Soil - InitialWater ####
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "InitialWater",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Water") 
    stop("Error in inspect_apsimx, Barley, Soil, InitialWater, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "InitialWater", parm = "RelativeTo",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Water.RelativeTo") 
    stop("Error in inspect_apsimx, Barley, Soil, InitialWater, FractionFull, print.path")
  #### Soil - Organic ####
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Organic",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Organic") 
    stop("Error in inspect_apsimx, Barley, Soil, Organic, print.path")
  pp <- inspect_apsimx(i, node = "Soil", soil.child = "Organic", parm = "Carbon",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Soil.Organic.Carbon") 
    stop("Error in inspect_apsimx, Barley, Soil, Organic, Carbon, print.path")
  #### Soil - InitialN ####
  ## Note (2023-12-10) Need to change this to NO3, NH3, Urea
  # pp <- inspect_apsimx(i, node = "Soil", soil.child = "InitialN",
  #                      src.dir = ex.dir, print.path = TRUE)
  # if(pp != ".Simulations.Simulation.Field.Soil.InitialN") 
  #   stop("Error in inspect_apsimx, Barley, Soil, InitialN, print.path")
  #### Soil - CERESSoilTemperature and Nutrient ####
  ## There is not much here to test or implement at the moment

  #### Manager ####
  pp <- inspect_apsimx(i, node = "Manager", src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field") 
    stop("Error in inspect_apsimx, Barley, Manager, print.path")
  pp <- inspect_apsimx(i, node = "Manager", parm = list("Fertilise", NA),
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Fertilise at sowing") 
    stop("Error in inspect_apsimx, Barley, Manager, Fertilise at sowing, print.path")
  pp <- inspect_apsimx(i, node = "Manager", parm = list("Fertilise", 3),
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.Fertilise at sowing.Amount") 
    stop("Error in inspect_apsimx, Barley, Manager, Fertilise at sowing, Amount, print.path")
  pp <- inspect_apsimx(i, node = "Surface", 
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.SurfaceOrganicMatter") 
    stop("Error in inspect_apsimx, Barley, SurfaceOrganicMatter, print.path")
  pp <- inspect_apsimx(i, node = "Surface", parm = "InitialResidueMass",
                       src.dir = ex.dir, print.path = TRUE)
  if(pp != ".Simulations.Simulation.Field.SurfaceOrganicMatter.InitialResidueMass") 
    stop("Error in inspect_apsimx, Barley, SurfaceOrganicMatter, InitialResidueMass print.path")
  pp <- inspect_apsimx(i, node = "MicroClimate", 
                       src.dir = ex.dir, print.path = TRUE)
  pp <- inspect_apsimx(i, node = "MicroClimate", parm = "SoilHeatFluxFraction",
                       src.dir = ex.dir, print.path = TRUE)
}


inspect.replacement.test <- get(".run.local.tests", envir = apsimx.options)

if(inspect.replacement.test){
  
  ## Define the highest level
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 1),
                             node = "Base", node.child = "Clock")
  ## inspect_apsimx version
  inspect_apsimx("Factorial.apsimx", src.dir = ex.dir,
                 root = c("Experiment", "Base"),
                 node = "Clock")
  
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 1),
                             node = "Base", node.child = "Weather")
  
  ## inspect_apsimx version
  inspect_apsimx("Factorial.apsimx", src.dir = ex.dir,
                 root = c("Experiment", "Base"),
                 node = "Weather")
  
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 1),
                             node = "Base", node.child = "Field",
                             node.subchild = "Soil",
                             display.available = TRUE)
  
  ## inspect_apsimx version
  inspect_apsimx("Factorial.apsimx", src.dir = ex.dir,
                 root = c("Experiment", "Base"),
                 node = "Soil")
  
  ## If we want to look at the second Factorial 'Experiment'
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 2),
                             node = "Base", node.child = "Field",
                             node.subchild = "Soil",
                             display.available = TRUE)
  
  ## Look at Water
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 2),
                             node = "Base", node.child = "Field",
                             node.subchild = "Soil",
                             node.subsubchild = "Water",
                             display.available = TRUE)
  
  ## For a specific parameter
  inspect_apsimx_replacement("Factorial", src.dir = ex.dir, 
                             root = list("Experiment", 2),
                             node = "Base", node.child = "Field",
                             node.subchild = "Soil",
                             node.subsubchild = "Water",
                             parm = "Depth")
}

inspect.replacement.test2 <- get(".run.local.tests", envir = apsimx.options)

extd.dir <- system.file("extdata", package = "apsimx")

if(inspect.replacement.test2){
  
  ## Testing inspect_apsimx on MaizeSoybean.apsimx
  pp <- inspect_apsimx("MaizeSoybean.apsimx", src.dir = extd.dir,
                       node = "Weather",
                       root = "SimulationSoybean",
                       parm = "FileName",
                       print.path = TRUE)
  
  if(pp != ".Simulations.SimulationSoybean.Weather.FileName")
    stop("pp does not match for inspect_apsimx MaizeSoybean.apsimx root = SimulationSoybean", call. = FALSE)
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                                   root = "SimulationSoybean", 
                                   node = "Weather", parm = "FileName", print.path = TRUE)
  
  if(pp != ".Simulations.SimulationSoybean.Weather.FileName")
    stop("pp does not match for inspect_apsimx_replacement MaizeSoybean.apsimx root = SimulationSoybean", call. = FALSE)
  
  ## Looking at values within Leaf
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             display.available = TRUE)
  
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             display.available = TRUE)
  
  ## We can look at parameters at this level
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             parm = "Albedo")
                             
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             parm = "Gsmax350")
                             
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             parm = "R50")
  
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE")
  
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             node = "Soybean",
                             node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE",
                             parm = "FixedValue")

  ## Soybean Cultivars
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   node.child = "Cultivars",
                                   node.subchild = "USA",
                                   node.subsubchild = "PioneerP22T61_MG22",
                                   parm = "Vegetative.Target.FixedValue",
                                   print.path = TRUE)
  
  if(pp != ".Simulations.Replacements.Soybean.Cultivars.USA.PioneerP22T61_MG22.Vegetative.Target.FixedValue") 
    stop("Error in inspect_apsimx_replacement, MaizeSoybean, PioneerP22T61_MG22, Vegetative.Target.FixedValue, print.path")

  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   node.child = "Cultivars",
                                   node.subchild = "USA",
                                   node.subsubchild = "PioneerP22T61_MG22",
                                   parm = "EarlyFlowering",
                                   print.path = TRUE)
  
  if(pp != ".Simulations.Replacements.Soybean.Cultivars.USA.PioneerP22T61_MG22.EarlyFlowering") 
    stop("Error in inspect_apsimx_replacement, MaizeSoybean, PioneerP22T61_MG22, EarlyFlowering, print.path")
  
  ## Testing the root
  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             root = "SimulationSoybean",
                             node = "Weather",
                             parm = "FileName")

  inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                             src.dir = extd.dir, 
                             root = "SimulationMaize",
                             node = "Weather",
                             parm = "FileName")
  
  ## WheatRye version
  inspect_apsimx_replacement("WheatRye.apsimx", src.dir = extd.dir,
                             node = "Wheat", node.child = "Cultivars",
                             node.subchild = "USA", node.subsubchild = "Yecora")
  
  ## Now with parameter
  inspect_apsimx_replacement("WheatRye.apsimx", src.dir = extd.dir,
                             node = "Wheat", node.child = "Cultivars",
                             node.subchild = "USA", node.subsubchild = "Yecora",
                             parm = "MinimumLeafNumber")
  ## This only works for the first one
  inspect_apsimx_replacement("WheatRye.apsimx", src.dir = extd.dir,
                             node = "Wheat", node.child = "Cultivars",
                             node.subchild = "USA", node.subsubchild = "Yecora",
                             parm = "Vrn")
                             
}
 

inspect.replacement.test.parm.path <- get(".run.local.tests", envir = apsimx.options)

if(inspect.replacement.test.parm.path){
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   print.path = TRUE)
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   print.path = TRUE)
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   node.child = "Leaf",
                                   print.path = TRUE)
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   node.child = "Leaf",
                                   parm = "Albedo",
                                   print.path = TRUE)
  
  pp <- inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                                   src.dir = extd.dir, 
                                   node = "Soybean",
                                   node.child = "Leaf",
                                   parm = "Gsmax350",
                                   print.path = TRUE)
  
  ### Testing replacement with new simulations (Dec 2023)
  ### Not normal testing as these files are not distributed with APSIM
  ### In the future, I will create some and distribute with the package
  
  if(FALSE){
    
    csm.dir <- "~/Desktop/csm/lab_05"
    
    pp1 <- inspect_apsimx_replacement("MaizeSoybean_replacement.apsimx",
                                      src.dir = csm.dir,
                                      node = "Maize",
                                      node.child = "Leaf",
                                      node.subchild = "Photosynthesis",
                                      node.subsubchild = "RUE",
                                      parm = "FixedValue",
                                      display.available = FALSE,
                                      print.path = TRUE)
    
    ### Need to look at whether I can edit this
  }
}

#### Test inspect Other ----
inspect.factorial.test.parm.path <- get(".run.local.tests", envir = apsimx.options)

if(inspect.factorial.test.parm.path){
  
  tmp.dir <- tempdir()
  dir(tmp.dir)
  ex.dir <- auto_detect_apsimx_examples()
  
  ### Test with all examples
  ex.dir.list <- dir(ex.dir, recursive = FALSE, pattern = "apsimx$")

  ## Trying node = "Other"
  for(i in ex.dir.list){
    if(!file.exists(file.path(tmp.dir, i))) file.copy(from = file.path(ex.dir, i), to = tmp.dir)  
    cat("Simulation:", i, "\n")
    inspect_apsimx(i, src.dir = tmp.dir, node = "Other")  
    cat("\n")
  }
  
  for(i in ex.dir.list){
    cat("Simulation:", i, "\n")
    inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = 2)  
  }
  
  for(i in ex.dir.list){
    ## file.copy(from = file.path(ex.dir, i), to = tmp.dir)  
    cat("Simulation:", i, "\n")
    inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = 3)  
  }
  
  ## Next need to test if inspect_apsimx works well when parm is a string
  ## pp <- inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(".Simulations.AgPastureExample.Field"))
  ## Would like to implement list(1, 2, 3)
  i <- "AgPasture.apsimx"
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3), print.path = TRUE)
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 0))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 1:3))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, c(3, 5)))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, c(3, 5)), print.path = TRUE)
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 0))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 5, 0))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 7, 7, 0))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 7, 1, 3), print.path = TRUE)
  pp <- inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = ".Simulations.AgPasture.Field", print.path = TRUE)
  i <- "WhiteClover.apsimx"
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = 3)
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 2, 5, 6, 0))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = ".Simulations.Simulation.Field")
  ## Developing/Testing list/grep
  ## This should work for SimpleGrazing and it doesn't at the moment
  i <- "AgPasture.apsimx"
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 7, 8, 0)) 
  (pp <- inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3, 7, 8, 17), print.path = TRUE))
  rootp <- inspect_apsimx(i, src.dir = tmp.dir, node = "Other", parm = list(1, 3))
  rut <- strsplit(rootp, ".", fixed = TRUE)[[1]][3]
  # inspect_apsimx(i, src.dir = tmp.dir, node = "Other", root = rut,
  #               parm = list("SimpleGrazingFrequencyString"))
  inspect_apsimx(i, src.dir = tmp.dir, node = "Other",
                 parm = list("AgPastureExample.SimpleGrazingFrequencyString"))
  ## These examples below do not work well (yet)
  ##inspect_apsimx(i, src.dir = tmp.dir, node = "Other", 
  ##              parm = list("AgPastureExample", "SimpleGrazingFrequencyString")) 
   
  ## inspect_apsimx(i, src.dir = tmp.dir, root = "AgPastureExample", node = "Other") 
  ##inspect_apsimx(i, src.dir = tmp.dir, root = "AgPastureExample", 
  ##                node = "Other", parm = list("SimpleGrazingFrequencyString", "SimpleGrazingResidual"))
  
  ## This works now, but not for all parameters, not sure why
  inspect_apsimx_json(i, src.dir = tmp.dir, parm = "AgPastureExample.SimpleGrazingFrequencyString")
  inspect_apsimx_json(i, src.dir = tmp.dir, parm = "AgPastureExample.SimpleMinGrazable")
  inspect_apsimx_json(i, src.dir = tmp.dir, parm = "AgPastureExample.GrazingRotationType")
  
  
  ## Inspect Replacement version
  inspect_apsimx_replacement(i, src.dir = tmp.dir,
                             node = "Forages",
                             root = list("AgPastureExample", 2),
                             display.available = TRUE)
  
  pp <- inspect_apsimx_replacement(i, src.dir = tmp.dir,
                                   root = list("AgPastureExample", 2),
                                   node = "Field",
                                   node.child = "SimpleGrazing",
                                   parm = "SimpleGrazingFrequencyString",
                                   display.available = TRUE,
                                   verbose = FALSE)
  
  inspect_apsimx("Factorial.apsimx", src.dir = tmp.dir,
                 root = list("RangeExperiment", "Base1"),
                 node = "Soil",
                 soil.child = "Physical",
                 print.path = TRUE)
  
  inspect_apsimx_replacement("Factorial.apsimx", src.dir = tmp.dir,
                             node = "Base1",
                             root = list("RangeExperiment"),
                             display.available = TRUE,
                             print.path = TRUE)

  inspect_apsimx_json("Factorial.apsimx", src.dir = tmp.dir,
                      parm = "Permutation",
                      print.path = TRUE)
  
### AgPasture has changed and these do not work anymore
  # inspect_apsimx_json("AgPasture.apsimx", src.dir = tmp.dir,
  #                parm = "AgPastureExample.CO2EffectScaleFactor",
  #                print.path = TRUE)
  # 
  # inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir,
  #                root = list("CO2xBaseTemperature", "CO2xTb"),
  #                print.path = TRUE)
  # 
  # inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir,
  #                root = list("PastureByWaterAndNitrogen", "Base"),
  #                print.path = TRUE)
}

#### Test Solute ----
if(run.inspect.tests){
  
  ### It is also possible to inspect 'Solute' or individual components in 
  ### other ways (see the initial 'inspect' tests at the beginning of this script)
  ex.dir <- auto_detect_apsimx_examples()
  dir(ex.dir)
  test.files <- c("Maize.apsimx", "Wheat.apsimx", "Soybean.apsimx", "Oats.apsimx")
  
  for(i in test.files){
    
    cat("Test file:", i, "\n")
    inspect_apsimx(i, src.dir = ex.dir,
                   node = "Soil",
                   soil.child = "Solute")
    
    inspect_apsimx(i, src.dir = ex.dir,
                   node = "Soil",
                   soil.child = "Solute",
                   print.path = TRUE)
    
    inspect_apsimx(i, src.dir = ex.dir,
                   node = "Soil",
                   soil.child = "Solute",
                   parm = "NO3",
                   print.path = TRUE)
    
    inspect_apsimx(i, src.dir = ex.dir,
                   node = "Soil",
                   soil.child = "Solute",
                   parm = "NH4",
                   print.path = TRUE)
    
    inspect_apsimx(i, src.dir = ex.dir,
                   node = "Soil",
                   soil.child = "Solute",
                   parm = "Urea",
                   print.path = TRUE)
    
  }
}

### Testing the spanking new inspect_apsimx_json
require(apsimx)

run.inspect.apsimx.json <- FALSE

if(run.inspect.apsimx.json){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "^Version")
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Version")
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "ApsimVersion")
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Clock")
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Soil") 
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Soil") 
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Dekalb") 
  
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "10-jan") 

  ## These parameters do not return a single path
  ## Ideally, I would be able to either return multiple paths or use
  ## a jsonpath for the search instead of a single keyword
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Water") 
  inspect_apsimx_json("Maize.apsimx", src.dir = extd.dir, parm = "Cultivar") 
}

#' Run an APSIM (7.x) \sQuote{Classic} Simulation
#' 
#' A valid apsim file can be run from within R. The main goal is to make running APSIM-X
#' simple, especially for large scale simulations or parameter optimization
#' 
#' @title Run an APSIM (7.x) \sQuote{Classic} simulation
#' @name apsim
#' @description Run apsim from R. It's for Windows only. It uses \sQuote{shell}.
#' @param file file name to be run (the extension .apsim is optional)
#' @param src.dir directory containing the .apsim file to be run (defaults to the current directory)
#' @param silent whether to print messages from apsim simulation
#' @param value how much output to return: \cr
#'              option \sQuote{report} returns only the \sQuote{main} report component;
#'              option \sQuote{all} returns all components of the simulation; \cr
#'              option \sQuote{none} runs simulation but does not return a data frame; \cr
#'              option \sQuote{user-defined} should be the name of a specific output file.
#' @param cleanup logical. Whether to delete the .out and .sum files generated by APSIM. Default is FALSE.
#' @param simplify whether to return a single data frame when multiple simulations are present. If FALSE it will return a list.
#' @return This function returns a data frame with APSIM output, but it depends on the argument \sQuote{value} above.
#' @export
#' @examples 
#' \donttest{
#' ## See function 'apsim_example' 
#' }
#'

apsim <- function(file = "", src.dir = ".",
                  silent = FALSE, 
                  value = "report",
                  cleanup = FALSE,
                  simplify = TRUE){
  
  if(.Platform$OS.type != "windows"){
    stop("This is only for windows. Use apsimx instead.")
  }
    
  if(file == "") stop("need to specify file name")
  
  ## This checks that there are no spaces in the path
  ## this would create a problem when running things at the command line
  .check_apsim_name(.file = file)
  .check_apsim_name(.file = src.dir)
  
  if(src.dir != ".") stop("In APSIM Classic you can only run a file from the current directory.")
  
  ## Extra checking, not sure if it will be triggered
  file.names <- dir(path = src.dir, pattern = ".apsim$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsim files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names, several.ok = FALSE)
  
  file.name.path <- file.path(src.dir, file)
  
  ## Can you run in APSIM 'Classic' from any directory or only from the current one?
  ## I'm assuming only from the current one
  # if(src.dir != "."){
  #   file.copy(file.name.path, ".")
  # }
  
  ada <- auto_detect_apsim()
  run.strng <- paste0(ada, " ", src.dir, "/", file) ## This is a command not a file.path
  shell(cmd = run.strng, translate = TRUE, intern = TRUE)
  
  ## It turns out that the name of the .out file is not as simple
  ## as the name of the input file
  output.names <- .find_output_names(.file = file, .src.dir = src.dir)
  
  ## With the current implementation the source directory will
  ## always be the current one
  if(value != "none"){
    if(value == "report" || value == "all"){
      if(length(output.names) == 1){
        ans <- read_apsim(file = output.names, src.dir = src.dir, 
                          value = value, silent = silent)
      }else{
        ## This will only work when output files have the same columns
        ## If simplify is TRUE
        ans <- read_apsim_all(filenames = output.names, 
                              src.dir = src.dir, value = "report",
                              simplify = simplify, silent = silent)
      }      
    }else{
      if(length(value) != 1)
        stop("'value' should be a string of length one.")
      output.name <- grep(value, output.names, value = TRUE) 
      if(length(output.name) == 0){
        cat("Available output names": output.names, "\n")
        stop(paste(value, "does not match any available output names"), call. = FALSE)
      }
      ## This is singular, just one output.name
      ans <- read_apsim(file = output.name, src.dir = src.dir, 
                            value = "report", silent = silent)
    }
  }else{
    if(value == "none" && !silent){
      cat("APSIM created .out files, but nothing is returned \n")
    }
  }
  
  if(cleanup){
    ## Default is not to cleanup
    if(value == "none") stop("do not clean up if you choose value = 'none' ")
    ## Delete the apsim-generated out file
    for(i in seq_along(output.names)){
        file.remove(output.names[i])
        file.remove(sub("out$","sum", output.names[i]))
      }
  }
  
  if(value != "none")
    return(ans)
}

## Local function used to detect APSIM Classic install
#' @noRd
auto_detect_apsim <- function(){

  if(.Platform$OS.type != "windows"){
    stop("This is only for windows. Use auto_detect_apsimx instead.")
  }
  
  ## Internal function to split APSIM name
  fev <- function(x) as.numeric(strsplit(x, "r", fixed = TRUE)[[1]][2])
  fmv <- function(x){
    tmp <- strsplit(x, "-", fixed = TRUE)[[1]][1]
    ans <- as.numeric(strsplit(tmp, "(M|m)")[[1]][2])
    ans
  } 
  ## I need to deal with the fact that there might be multiple versions
  ## of APSIM installed
  
  st1 <- "C:/PROGRA~2/"
  laf <- list.files(st1)
  find.apsim <- grep("APSIM", laf, ignore.case = TRUE)
  
  if(length(find.apsim) == 0 && is.na(apsimx::apsim.options$exe.path)){
    ## Try the registry approach only if there is no 'exe.path'
    ## HCR hive is for HKEY_CLASSES_ROOT, HLM is for HKEY_LOCAL_MACHINE and HCU is for HKEY_CURRENT_USER
    regcmd <- try(utils::readRegistry("APSIMFile\\shell\\open\\command", "HCR")[[1]], silent = TRUE)
    if(inherits(regcmd, "try-error")) regcmd <- try(utils::readRegistry("APSIMFile\\shell\\open\\command", "HCU")[[1]], silent = TRUE)
    if(inherits(regcmd, "try-error")) regcmd <- try(utils::readRegistry("APSIMFile\\shell\\open\\command", "HLM")[[1]], silent = TRUE)
    if(inherits(regcmd, "try-error")) stop("Could not find APSIM Classic in the Windows Registry")
    regcmd2 <- gsub("\\\\", "/", strsplit(regcmd, "\"")[[1]][2])
    apsim_dir <- gsub("UI", "Models", regcmd2)
    if(length(apsim_dir) == 0) stop("APSIM Classic was not found and no 'exe.path' exists.")
    if(grepl("\\s", apsim_dir)) stop("Found a space in the path. Please provide the path manually to APSIM using exe.path in apsim_options.")
    return(apsim_dir)
  } 

  if(length(find.apsim) == 1){
    apsim.name <- laf[find.apsim]
  }
  
  if(length(find.apsim) > 1){
    apsim.versions <- laf[find.apsim]
    max.main.version <- max(sapply(apsim.versions, fmv))
    if(any(is.na(max.main.version))){
      max.main.version <- max.main.version[!is.na(max.main.version)]
    }
    if(length(max.main.version) == 0){
      stop("It is likely that APSIM Next Gen was found when looking for APSIM Classic. Please set the path manually.", call. = FALSE) 
    }
    which.main.versions <- grep(max.main.version, apsim.versions)
    versions <- sapply(apsim.versions[which.main.versions], fev)
    newest.version <- apsim.versions[which.max(versions)]
    if(apsimx::apsim.options$warn.versions &&
       is.na(apsimx::apsim.options$exe.path)){
      warning(paste("Multiple versions of APSIM installed. \n
                    Choosing the newest one:", newest.version))
      }
    ## apsim.name <- grep(newest.version, apsim.versions, value = TRUE)
    apsim.name <- newest.version
  }
  
  ## APSIM executable
  st3 <- "/Model/Apsim.exe" 
  if(is.na(get("exe.path", envir = apsimx::apsim.options))){
    if(length(apsim.name) >= 1){
      apsim_dir <- paste0(st1, apsim.name, st3)  
    }else{
      stop("APSIM not found. Please try setting the path manually through 'apsim_options'", call. = FALSE)
    }
  }
  
  if(!is.na(get("exe.path", envir = apsimx::apsim.options))){
    ## Windows paths can contain white spaces which are
    ## problematic when running them at the command line
    ## I will simply not allow white spaces
    if(grepl("\\s", get("exe.path", envir = apsimx::apsim.options)))
      stop("White spaces are not allowed in APSIM Classic exe.path")
    apsim_dir <- get("exe.path", envir = apsimx::apsim.options)
  }
  return(apsim_dir)
}

#' Auto detect where APSIM (7.x) \sQuote{Classic} examples are located 
#' 
#' @title Auto detect where apsim examples are located
#' @name auto_detect_apsim_examples
#' @description simple function to detect where APSIM \sQuote{Classic} examples are located
#' @return will create a directory pointing to APSIM \sQuote{Classic} distributed examples
#' @export
#' @examples 
#' \dontrun{
#' ex.dir <- auto_detect_apsim_examples()
#' }
#' 

auto_detect_apsim_examples <- function(){
  
  if(.Platform$OS.type != "windows"){
    stop("This is only for windows. Use auto_detect_apsimx_examples instead.")
  }
  
  ## Internal function to split APSIM name
  fev <- function(x) as.numeric(strsplit(x, "r", fixed = TRUE)[[1]][2])
  
  st1 <- "C:/PROGRA~2"
  laf <- list.files(st1)
  find.apsim <- grep("APSIM",laf, ignore.case = TRUE)
  
  if(length(find.apsim) == 0) stop("APSIM 'Classic' not found")
  
  apsim.versions <- laf[find.apsim]
  if(length(apsim.versions) > 1){
      versions <- sapply(apsim.versions, fev)
      newest.version <- sort(versions, decreasing = TRUE)[1]
      if(get("warn.versions", envir = apsimx::apsim.options)){
        warning(paste("Multiple versions of APSIM installed. \n
                      Choosing the newest one:",newest.version))
      }
      apsim.name <- grep(newest.version, apsim.versions, value = TRUE)
    }else{
      apsim.name <- apsim.versions
    }
    ## APSIM path to examples
    st3 <- "/Examples" 
    apsim_ex_dir <- paste0(st1, "/", apsim.name,st3)
  
  if(!is.na(get("examples.path", envir = apsimx::apsim.options))){
    ## Not sure if I need shQuote here
    if(grepl("\\s", get("examples.path", envir = apsimx::apsim.options)))
      stop("White spaces are not allowed in APSIM Classic examples.path")
    apsim_ex_dir <- get("examples.path", envir = apsimx::apsim.options)
  }
  return(apsim_ex_dir)
}

#'
#' @title Access Example APSIM Simulations
#' @name apsim_example
#' @description simple function to run some of the built-in APSIM examples
#' @param example run an example from built-in APSIM. Options are all of the ones included with the APSIM distribution, except \sQuote{Graph}.
#' @param silent whether to print standard output from the APSIM execution
#' @param tmp.dir temporary directory where to write files
#' @note This function creates a new column \sQuote{Date} which is in the R \sQuote{Date} format which is convenient for graphics.
#' @details This function creates a temporary copy of the example file distributed with APSIM to avoid writing a .out file 
#'          to the directory where the \sQuote{Examples} are located. It is not a good practice and there is no guarantee that 
#'          the user has read/write permissions in that directory.
#' @return This function returns a data frame with APSIM output 
#' @export
#' @examples 
#' \dontrun{
#' ## Only run these if you have APSIM 'Classic' installed (Windows only)
#' millet <- apsim_example("Millet")
#' potato <- apsim_example("Potato")
#' sugar <- apsim_example("Sugar")
#' ## The 'Date' column is created by this function, based on apsim output.
#' require(ggplot2)
#' ggplot(data = millet , aes(x = Date, y = millet_biomass)) + 
#'   geom_line()
#' }
#' 

apsim_example <- function(example = "Millet", silent = FALSE, tmp.dir = NULL){

  if(.Platform$OS.type != "windows"){
    stop("This is only for windows. Use apsimx_example instead.")
  }
  ## Write to a temp dir only
  if(missing(tmp.dir)) tmp.dir <- "."
  ## Run a limited set of examples
  ## Now the only one missing is Graph, which I assume is about
  ## graphics and not that relevant to apsim
  ## Examples not supported: Several
  ex.ch <- c("agpasture", "Canopy", "Centro", "Millet", "Potato", "Sugar")
  
  example <- match.arg(example, choices = ex.ch)
  
  ada <- auto_detect_apsim()
  ex.dir <- auto_detect_apsim_examples()
  ex <- file.path(ex.dir, paste0(example, ".apsim"))
  
  if(!file.exists(ex)) stop("cannot find example files")
  ## Make a temporary copy of the file to the current directory
  ## Do not transfer permissions?
  file.copy(from = ex, to = tmp.dir, copy.mode = FALSE)
  
  run.strng <- paste0(ada, " ", paste0(tmp.dir, "/", example, ".apsim"))
  shell(cmd = run.strng, translate = TRUE, intern = TRUE)
  
  ## Create database connection
  ## I don't need to specify the directory as it should be the current one
  ## I do need to find out the output name
  out.name <- .find_output_names(paste0(example,".apsim"), .src.dir = tmp.dir)
  if(length(out.name) == 1){
    ans <- read_apsim(out.name, value = "report")
  }
  if(length(out.name) > 1){
    stop("not ready to handle this yet")
  }
  ## OS independent cleanup (risky?)
  for(i in out.name){
    file.remove(paste0(tmp.dir, "/", i))
    file.remove(paste0(tmp.dir, "/", strsplit(i, ".", fixed=TRUE)[[1]][1], ".sum"))
  }
  file.remove(paste0(tmp.dir, "/", example, ".apsim"))
  ## Return data frame
  return(ans)
}

#' Read APSIM generated .out files
#' 
#' @title Read APSIM generated .out files
#' @name read_apsim
#' @description read \sQuote{output} databases created by APSIM runs (.out and .sim). One file at a time.
#' @param file file name
#' @param src.dir source directory where file is located
#' @param value either \sQuote{report} (data.frame), \sQuote{user-defined} or \sQuote{all} (list)
#' @param date.format format for adding \sQuote{Date} column 
#' @param silent whether to issue warnings or suppress them
#' @return This function returns a data frame with APSIM output or a list if value equals \sQuote{all}
#' @seealso \code{\link{read_apsim_all}}
#' @export
#' @examples 
#' \dontrun{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' maize.out <- read_apsim("Maize", src.dir = extd.dir, value = "report")
#' millet.out <- read_apsim("Millet", src.dir = extd.dir, value = "report")
#' }
#' 

read_apsim <- function(file = "", src.dir = ".",
                       value = c("report", "all"),
                       date.format = "%d/%m/%Y",
                       silent = FALSE){
  
  if(file == "") stop("need to specify file name")
  
  file.names <- dir(path = src.dir, pattern = ".out$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .out files in the specified directory to read.")
  }
  
  value <- match.arg(value)
  
  if(length(grep(".out$", file)) != 0){
    ## I assume the extention was included
    ## Only use the name from here 
    ## This strips the extension
    file <- tools::file_path_sans_ext(file)
  }
  
  file.name.path <- paste0(src.dir, "/", file, ".out")
  
  ## How many rows to skip might be title plus one
  skip.out.lines <- -1
  rdlns <- readLines(con = file.name.path, n = 6)
  for(i in 1:5){
    if(grepl("^Title =", rdlns[i])){
      skip.out.lines <- i
    }    
  }
  
  if(skip.out.lines < 0)
    stop("Was expecting to find a line with 'Title' in the output, but nothing was found.
         Is the output empty?", call. = FALSE)

  ## Read output file
  hdr <- as.character(sapply(as.vector(read.table(file = file.name.path, 
                                                  header = FALSE,
                                                  sep = "", 
                                                  nrows = 1, 
                                                  skip = skip.out.lines)[1,]), 
                             FUN = function(x) x[[1]]))

  out.file <- read.table(file = file.name.path, header = FALSE, sep = "", skip = c(skip.out.lines + 2))
  
  if(length(hdr) != dim(out.file)[2]){
    cat("length header", length(hdr), " number of columns", dim(out.file)[2], "\n")
    stop("header names are not equal to number of columns")
  }
  
  names(out.file) <- hdr
  ## Read summary file, but fail more or less gracefully
  file.name.summary <- paste0(src.dir, "/", file, ".sum")
  sum.file <- suppressWarnings(try(readLines(con = file.name.summary), silent = TRUE))
  if(inherits(sum.file, "try-error")){
    lf.sum <- list.files(path = src.dir, pattern = "sum$")
    if(length(lf.sum) == 0){
      if(!silent) warning("Could not find summary file.")  
    }
    if(length(lf.sum) == 1){
      ## This means that this is likely the correct file
      sum.file <- try(readLines(con = lf.sum), silent = TRUE)    
      if(inherits(sum.file, "try-error"))
        if(!silent) warning("Could not read summary file.")
    }
    if(length(lf.sum) > 1)
      if(!silent) warning("Multiple 'sum' files. Don't know which one is the correct one.")
  }
  
  if(any(grepl("Date", hdr, ignore.case = TRUE))){
    wcid <- grep("Date", hdr, ignore.case = TRUE) ## The short name stands for 'which column is date'
    ## If there is more than one, I will select the first one
    if(length(wcid) > 1){
      wcid <- wcid[1]
      if(!silent) warning("More than one column with 'Date'. Picking the first one.")
    }
      
    try.date <- try(as.Date(out.file[,wcid], format = date.format), silent=TRUE)
    
    if(inherits(try.date, "try-error")){
      warning("Could not create Date column")
    }else{
      out.file$Date <- try.date   
    }
  }
  ## Return list
  if(value == "all"){
    ans <- list(Report = out.file, Summary = sum.file)
  }
  ## Return data.frame
  if(value == "report"){
    ans <- out.file
  }
  return(ans)
}

#' Read all APSIM generated .out files in a directory
#' 
#' @title Read all APSIM generated .out files in a directory
#' @name read_apsim_all
#' @description Like \code{\link{read_apsim}}, but it can read many .out files in a directory. 
#' It will read all of them unless these are filtered using a regular expression as an argument
#' to \sQuote{value}.
#' @param filenames names of files to be read
#' @param src.dir source directory where files are located
#' @param value either \sQuote{report}, \sQuote{user-defined} or \sQuote{all} (not implemented at the moment)
#' @param date.format format for adding \sQuote{Date} column 
#' @param simplify whether to return a single data frame or a list. 
#' @param silent whether to issue warnings or suppress them
#' @return returns a data frame or a list depending on the argument \sQuote{simplify} above.
#' @note Warning: very simple function at the moment, not optimized for memory or speed.
#' @export
#' 

read_apsim_all <- function(filenames, src.dir = ".", 
                           value = "report",
                           date.format = "%d/%m/%Y", 
                           simplify = TRUE,
                           silent = FALSE){
  
  ## This is memory hungry and not efficient at all, but it might work 
  ## for now

  if(simplify && value == "all") stop("Cannot simplify when value = all")
  
  file.names <- dir(path = src.dir, pattern=".out$", ignore.case = TRUE)
  
  if(!missing(filenames)){
   file.names <- filenames
  }
  
  col.names <- matrix(nrow = length(file.names), ncol = 1000)
  col.num <- numeric(length(file.names))
  
  if(value != "report" && value != "all"){
    ## Here the argument value is defined by the user
    outfiles <- grep(value, file.names, value = TRUE)
    
    if(length(outfiles) == 0)
      stop("When 'value' is not 'all' or 'report' it should match at least one of the outfile names.", call. = FALSE)
  
    if(length(outfiles) == 1){
      ans <- read_apsim(outfiles, src.dir = src.dir, value = "report", date.format = date.format, silent = silent)  
    }else{
      if(simplify){
        ans <- NULL
        for(i in outfiles){
          tmp <- read_apsim(i, src.dir = src.dir, value = value, date.format = date.format, silent = silent)
          tmp.d <- data.frame(outfile = i, tmp)
          ans <- try(rbind(ans, tmp.d), silent = TRUE)
          if(inherits(ans, "try-error")){
            stop("Could not simplify output files into a single data.frame \n
             Choose simplify = FALSE.", call. = FALSE)
          }
        }
      }else{
        ans <- vector("list", length = length(outfiles))
        names(ans) <- outfiles
        for(i in outfiles){
          ans[[i]] <- read_apsim(i, value = value, date.format = date.format, silent = silent)
        }    
      }      
    }  
  }else{
    if(simplify){
      ans <- NULL
      for(i in file.names){
        tmp <- read_apsim(i, src.dir = src.dir, value = value, date.format = date.format, silent = silent)
        tmp.d <- data.frame(outfile = i, tmp)
        ans <- try(rbind(ans, tmp.d), silent = TRUE)
        if(inherits(ans, "try-error")){
          stop("Could not simplify output files into a single data.frame \n
             Choose simplify = FALSE.", call. = FALSE)
        }
      }
    }else{
      ans <- vector("list", length = length(file.names))
      names(ans) <- file.names
      for(i in file.names){
        ans[[i]] <- read_apsim(i, value = value, date.format = date.format, silent = silent)
      }    
    }    
  }
  
  return(ans)
}

#' Set apsim options
#' 
#' @title Setting some options specific to APSIM (7.x) \sQuote{Classic}
#' @name apsim_options
#' @description Set the path to the APSIM executable, examples and warning suppression. 
#' @param exe.path path to apsim executable
#' @param examples.path path to apsim examples
#' @param warn.versions logical. warning if multiple versions of APSIM are detected.
#' @note It is possible that APSIM 7.x \sQuote{Classic} is installed in some alternative location other than the 
#'       defaults ones. Guessing this can be difficult and then the auto_detect functions might
#'       fail. Also, if multiple versions of APSIM are installed apsim will choose the newest
#'       one but it will issue a warning. Suppress the warning by setting warn.versions = FLASE.
#' @return It modifies the \sQuote{apsim.options} environment as a side effect.
#' @export
#' @examples 
#'\dontrun{
#' names(apsim.options)
#' apsim_options(exe.path = "some-new-path-to-executable")
#' apsim.options$exe.path
#' }

apsim_options <- function(exe.path = NA, examples.path = NA, warn.versions = TRUE){
  assign('exe.path', exe.path, apsim.options)
  assign('examples.path', examples.path, apsim.options)
  assign('warn.versions', warn.versions, apsim.options)
}

#' Environment which stores APSIM options
#' 
#' @title Environment which stores APSIM options
#' @name apsim.options
#' @description Environment which can store the path to the executable and where examples are located.
#'              Creating an environment avoids the use of global variables or other similar practices
#'              which would have possible undesriable consequences. 
#' @return This is an environment, so nothing to return.
#' @export
#' @examples 
#' \dontrun{
#' names(apsim.options)
#' apsim_options(exe.path = "some-new-path-to-executable")
#' apsim.options$exe.path
#' }
#' 

apsim.options <- new.env(parent = emptyenv())
assign('exe.path', NA, apsim.options)
assign('examples.path', NA, apsim.options)
assign('warn.versions', TRUE, apsim.options)

#' Display available APSIM \sQuote{Classic} and APSIM-X versions
#' 
#' @name apsim_version
#' @param which either \sQuote{all} or \sQuote{inuse}
#' @param version either \sQuote{number} for a compact display or \sQuote{full}
#' @param verbose whether to print the information to standard output
#' @return a data frame (all) or a vector (inuse) with APSIM-X and/or APSIM versions
#' @export
#' @examples 
#' \dontrun{
#' ## Check which APSIM versions are available
#' ava <- apsim_version(verbose = TRUE)
#' 
#' ## How to get APSIM 'help'
#' ## apc <- apsimx:::auto_detect_apsimx()
#' ## system(paste(apc, "--help"))
#' }
#' 

apsim_version <- function(which = c("all", "inuse"), 
                          version = c("number", "full"),
                          verbose = TRUE){
  
  ## Internal function
  fevc <- function(x) as.numeric(sub("r","",strsplit(x, "-", fixed = TRUE)[[1]][2]))
  
  which <- match.arg(which)
  version <- match.arg(version)
  
  ## Return (all): a table with APSIM and APSIM-X available versions
  ## Return (inuse): the number for the version in use
  
  ## This it the response if we choose all, now I need to populate it
  tmp.dat <- data.frame(APSIM = c("Classic", "Next Generation"))
  
  ## Let's do it by operating system
  if(verbose) cat("OS:", Sys.info()[["sysname"]], "\n")
  
  ## For unix
  if(.Platform$OS.type == "unix"){
    
    if(grepl("Darwin", Sys.info()[["sysname"]])){
        laf <- list.files("/Applications/")
        find.apsim <- grep("APSIM", laf, ignore.case = TRUE)
        if(length(find.apsim) == 0) warning("APSIM-X not found")
        apsimx.versions <- laf[find.apsim]
        
        if(length(find.apsim) > 0){ 
            tmp.mat <- matrix(NA, nrow = 2, ncol = length(find.apsim))
            tmp.mat[2, seq_along(find.apsim)] <- rev(apsimx.versions)
            if(version == "number"){
              tmp.mat[2, ] <- sapply(tmp.mat[2, ], FUN = function(x) strsplit(x, split = ".", fixed = TRUE)[[1]][3])
            }
            ans <- data.frame(tmp.dat, as.data.frame(tmp.mat))
            names(ans) <- c("APSIM", paste0("Version.",seq_along(find.apsim)))
        }
    }
    
    if(grepl("Linux", Sys.info()[["sysname"]])){
        laf <- list.files("/usr/local/lib")
        find.apsim <- grep("apsim", laf, ignore.case = TRUE)
        if(length(find.apsim) == 0) warning("APSIM-X not found")
        apsimx.version <- paste0("/usr/local/lib/apsim/", list.files("/usr/local/lib/apsim"))
        ## Apparently only one version can be present at a time on Debian
        tmp.mat <- matrix(NA, nrow = 2, ncol = length(find.apsim))
        tmp.mat[2, length(find.apsim)] <- list.files("/usr/local/lib/apsim")
        if(version == "number"){
          tmp.mat[2, ] <- sapply(tmp.mat[2, ], FUN = function(x) strsplit(x, split = ".", fixed = TRUE)[[1]][3])
        }
        ans <- data.frame(tmp.dat, as.data.frame(tmp.mat))
        names(ans) <- c("APSIM","Version")
    }
  }
  
  if(.Platform$OS.type == "windows"){
    ## Here I need to look for both APSIM and APSIM-X
    ## First APSIM-X
    st1 <- "C:/PROGRA~1/"
    lafx <- list.files(st1)
    find.apsimx <- grep("APSIM",lafx)
    
    ## Now look for APSIM
    st2 <- "C:/PROGRA~2/"
    laf <- list.files(st2)
    find.apsim <- grep("APSIM",laf, ignore.case = TRUE)
    
    if(length(find.apsim) == 0 && length(find.apsimx) == 0) 
      warning("Could not find APSIM or APSIM-X")
    
    ncols <- max(c(length(find.apsim),length(find.apsimx)))
    
    if(length(find.apsimx) > 0){
      apsimx.versions <- lafx[find.apsimx]
      ## I want to order apsimx.version from newest to oldest
      ## The line below is not necessary now
      ## apsimx.versions <- apsimx.versions[order(sapply(apsimx.versions, fev), decreasing = TRUE)]
      tmp.matx <- matrix(NA, nrow = 1, ncol = ncols)
      tmp.matx[1,seq_along(find.apsimx)] <- rev(apsimx.versions)
    }else{
      tmp.matx <- matrix(NA, nrow = 1, ncol = ncols) 
    }
    
    if(version == "number"){
      tmp.matx[1, ] <- sapply(tmp.matx[1, ], FUN = function(x) strsplit(x, split = ".", fixed = TRUE)[[1]][3])
    }

    ## This is for APSIM 'Classic' and it should work
    if(length(find.apsim) > 0){
      apsim.versions <- laf[find.apsim]
      apsim.versions <- apsim.versions[order(sapply(apsim.versions, fevc), decreasing = TRUE)]
      tmp.matc <- matrix(NA, nrow = 1, ncol = ncols)
      tmp.matc[1,seq_along(find.apsim)] <- apsim.versions
    }else{
      tmp.matc <- matrix(NA, nrow = 1, ncol = ncols) 
    }

    ans <- data.frame(tmp.dat, as.data.frame(rbind(tmp.matc, tmp.matx)))
    names(ans) <- c("APSIM", paste0("Version.", 1:ncols))
  }
  
  if(which == "all"){
    if(verbose)
      print(knitr::kable(ans))
  }
  
  if(which == "inuse"){
    ## For Unix
    if(.Platform$OS.type == "unix"){
      ## If there is only one APSIM present
        if(length(find.apsim) == 1){
            ## Darwin is different from Linux
            if(grepl("Darwin", Sys.info()[["sysname"]])) ans <- laf[find.apsim]
            if(grepl("Linux", Sys.info()[["sysname"]])) ans <- paste0("apsim", list.files("/usr/local/lib/apsim"))
        }
      ## If there are multiple APSIMs present
      if(length(find.apsim) > 1){
        len.fa <- length(find.apsim)
        fa.dt <- sapply(laf[find.apsim],.favd, simplify = FALSE)[[len.fa]]
        if(verbose) cat("APSIM-X version date:", as.character(fa.dt))
        ## This next line picks the last one
        newest.version <- laf[find.apsim][len.fa]
        ans <- newest.version
      }
      ## If the one in use is not any of the above such as a Custom build
      if(!is.na(get("exe.path", envir = apsimx::apsimx.options))){
        ans <- get("exe.path", envir = apsimx::apsimx.options)
      }
    }
    ## For Windows
    if(.Platform$OS.type == "windows"){
      ## If everything fails NAs will be returned
      ansc <- NA
      ansx <- NA
      ## If there is only one APSIM 'Classic'
      if(length(find.apsim) == 1){
        ansc <- laf[find.apsim]
      }
      ## If there are multiple APSIMs present
      if(length(find.apsim) > 1){
        apsim.versions <- sapply(laf[find.apsim],fevc)
        newest.version <- sort(apsim.versions, decreasing = TRUE)[1]
        ansc <- names(newest.version)
      }
      ## It is possible that the one in use is not the latest one
      if(!is.na(get("exe.path", envir = apsimx::apsim.options))){
        ansc <- get("exe.path", envir = apsimx::apsim.options)
      }
      ## For APSIM-X
      if(length(find.apsimx) == 1){
        ansx <- lafx[find.apsimx]
      }
      if(length(find.apsimx) > 1){
        apsimx.versions <- lafx[find.apsimx]
        len.fa <- length(find.apsimx)
        newest.version <- lafx[find.apsimx][len.fa]
        ansx <- newest.version
      }
      ## It is possible that the one in use is not the latest one
      if(!is.na(get("exe.path", envir = apsimx::apsimx.options))){
        ansx <- get("exe.path", envir = apsimx::apsimx.options)
      }
      
      ans <- data.frame(Classic = ansc, NextGeneration = ansx)
    }
    if(verbose) print(knitr::kable(ans))
  }
  
  invisible(ans)
}


### This function retrieves model versions from the github page
apsimx_current_version <- function(model = "Wheat"){
  model <- get_apsimx_json(model)
  return(model$Version)
}

# apsim_version_inuse <- function(verbose = TRUE){
# 
#   ## For Unix
#   if(.Platform$OS.type == "unix"){
#     ## If there is only one APSIM present
#     if(length(find.apsim) == 1){
#       ## Darwin is different from Linux
#       if(grepl("Darwin", Sys.info()[["sysname"]])){
#         laf <- list.files("/Applications/")
#         ans <- laf[find.apsim]
#       } 
#       if(grepl("Linux", Sys.info()[["sysname"]])) ans <- paste0("apsim", list.files("/usr/local/lib/apsim"))
#     }
#     ## If there are multiple APSIMs present
#     if(length(find.apsim) > 1){
#       len.fa <- length(find.apsim)
#       fa.dt <- sapply(laf[find.apsim],.favd, simplify = FALSE)[[len.fa]]
#       if(verbose) cat("APSIM-X version date:", as.character(fa.dt))
#       ## This next line picks the last one
#       newest.version <- laf[find.apsim][len.fa]
#       ans <- newest.version
#     }
#     ## If the one in use is not any of the above such as a Custom build
#     if(!is.na(get("exe.path", envir = apsimx::apsimx.options))){
#       ans <- get("exe.path", envir = apsimx::apsimx.options)
#     }
#   }
#   ## For Windows
#   if(.Platform$OS.type == "windows"){
#     ## If everything fails NAs will be returned
#     ansc <- NA
#     ansx <- NA
#     ## If there is only one APSIM 'Classic'
#     if(length(find.apsim) == 1){
#       ansc <- laf[find.apsim]
#     }
#     ## If there are multiple APSIMs present
#     if(length(find.apsim) > 1){
#       apsim.versions <- sapply(laf[find.apsim],fevc)
#       newest.version <- sort(apsim.versions, decreasing = TRUE)[1]
#       ansc <- names(newest.version)
#     }
#     ## It is possible that the one in use is not the latest one
#     if(!is.na(get("exe.path", envir = apsimx::apsim.options))){
#       ansc <- get("exe.path", envir = apsimx::apsim.options)
#     }
#     ## For APSIM-X
#     if(length(find.apsimx) == 1){
#       ansx <- lafx[find.apsimx]
#     }
#     if(length(find.apsimx) > 1){
#       apsimx.versions <- lafx[find.apsimx]
#       len.fa <- length(find.apsimx)
#       newest.version <- lafx[find.apsimx][len.fa]
#       ansx <- newest.version
#     }
#     ## It is possible that the one in use is not the latest one
#     if(!is.na(get("exe.path", envir = apsimx::apsimx.options))){
#       ansx <- get("exe.path", envir = apsimx::apsimx.options)
#     }
#     ans <- data.frame(Classic = ansc, NextGeneration = ansx)
#   }
#   invisible(ans)
# }

## This is my way of doing this now
require(apsimx)
apsimx_options(warn.versions = FALSE)

extd.dir <- system.file("extdata", package = "apsimx")

run.test.edit.apsimx.batch <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.test.edit.apsimx.batch){

  file.copy(paste0(extd.dir, "/", "Wheat.apsimx"), tmp.dir)

  inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, node = "SurfaceOrganicMatter")

  edit1time <- system.time(edit_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                                       wrt.dir = tmp.dir,
                                       node = "SurfaceOrganicMatter", 
                                       parm = "InitialResidueMass", value = 600))

  inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, node = "SurfaceOrganicMatter")

  ## Using the 'batch' method
  ## Let's say I want to edit 'SurfaceOrganicMatter'
  ## InitialResidueMass from 500 to 650
  ## Understanding the structure of JSON files
  parms <- list(".Simulations.Simulation.Field.SurfaceOrganicMatter.InitialResidueMass" = 650)
  edit2time <- system.time(edit_apsimx_batch("Wheat.apsimx", src.dir = tmp.dir, parms = parms))
  inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, node = "SurfaceOrganicMatter")
  
  ## What is the difference in 'elapsed' real time?
  edit2time[[3]]/edit1time[[3]]
} 

#' Soil Profiles
#'
#' Real soils might have discontinuities, but for APSIM it might be beneficial to be able to create 
#' a soil profile with an arbitrary number of layers and have flexibility in the 
#' distribution of soil physical and chemical properties. Steps:
#'  
#'  1. \code{\link{apsimx_soil_profile}} is a function which can create a soil matrix with many layers \cr
#'  2. It allows for creating a smooth distribution for Physical (or Water), Chemical, InitialWater, Analysis, InitialN, Organic or SoilOrganicMatter \cr
#'  3. The distribution can be specified with the \sQuote{a} and \sQuote{c} parameter of an exponential decay function, using a list. E.g. DUL = list(0.35, 0, -0.1).
#'  This means that the top value for DUL will be 0.35 and it will decay with a rate of -0.1. \cr
#'  4. If an increase and then a decay is needed the Ricker function can be used. See \sQuote{SSricker} in the \sQuote{nlraa} package. \cr
#'  
#' @title Create APSIM-X Soil Profiles
#' @name apsimx_soil_profile
#' @rdname apsimx_soil_profile
#' @description Generates a soil profile that can then replace the existing one in an \sQuote{.apsimx} or \sQuote{.apsim} simulation file
#' @param nlayers Number of soil layers (default = 10)
#' @param Depth specific depths for each soil layer (cm)
#' @param Thickness thickness for each soil layer (mm)
#' @param BD bulk density for each soil layer (g/cc) -- \sQuote{cc} is cubic cm
#' @param AirDry air dry for each soil layer (mm/mm)
#' @param LL15 lower limit (15 bar) for each soil layer (mm/mm)
#' @param DUL drainage upper limit (0.33 bar) for each soil layer (mm/mm)
#' @param SAT saturation (0 bar) for each soil layer (mm/mm)
#' @param KS saturated hydraulic conductivity (mm/day)
#' @param crop.LL lower limit for a specific crop
#' @param crop.KL root ability to extract water for a specific crop
#' @param crop.XF soil root exploration for a specific crop
#' @param Carbon organic carbon (percent)
#' @param SoilCNRatio organic carbon C:N ratio
#' @param FOM fresh organic matter (kg/ha)
#' @param FOM.CN fresh organic matter C:N ratio
#' @param FBiom Fraction of microbial biomass (0-1)
#' @param FInert Fraction of inert carbon (0-1)
#' @param NO3N nitrate nitrogen (Chemical) (ppm)
#' @param NH4N ammonium nitrogen (Chemical) (ppm)
#' @param CACO3 carbonates (chemical) (ppm). Not used by APSIM currently.
#' @param PH soil pH
#' @param ParticleSizeClay particle size clay (in percent)
#' @param ParticleSizeSilt particle size silt (in percent)
#' @param ParticleSizeSand particle size sand (in percent)
#' @param soil.bottom bottom of the soil profile (cm)
#' @param water.table water table level (not used at the moment) (cm)
#' @param soil.type might use it in the future for auto filling missing information
#' @param crops name of crops being grown
#' @param metadata list with soil metadata. For possible parameters and values see an example of \code{\link{inspect_apsimx}} with soil.child = \dQuote{Metadata}.
#' @param soilwat optional \sQuote{list} of class \sQuote{soilwat_parms}
#' @param swim optional \sQuote{list} of class \sQuote{swim_parms}
#' @param initialwater optional \sQuote{list} of class \sQuote{initialsoilwater_parms}
#' @param solutes optional \sQuote{list} of class \sQuote{solutes_parms}
#' @param soilorganicmatter optional \sQuote{list} of class \sQuote{soilorganicmatter_parms}
#' @param dist.parms parameter values for creating a profile. If a == 0 and b == 0 then \cr
#' a constant value of 1 is used. If a == 0 and b != 0, then an exponential decay is used. \cr
#' If a != 0 and b != 0 then the equation is \code{a*soil.layer*exp(-b*soil.layer)}.  
#' @param check whether to check for reasonable values using \code{\link{check_apsimx_soil_profile}}
#' @return a soil profile with class \sQuote{soil_profile} with elements \sQuote{soil}, \sQuote{crops}, \sQuote{metadata},
#' \sQuote{soilwat} and \sQuote{swim}.
#' @export
#' @examples 
#' \donttest{
#'  sp <- apsimx_soil_profile()
#'  require(ggplot2)
#'  plot(sp)
#'  }
#'

apsimx_soil_profile <-  function(nlayers = 10, 
                                 Depth = NULL, 
                                 Thickness = NULL, 
                                 BD = NULL, 
                                 AirDry = NULL,
                                 LL15 = NULL,
                                 DUL = NULL,
                                 SAT = NULL,
                                 KS = NULL,
                                 crop.LL = NULL,
                                 crop.KL = NULL,
                                 crop.XF = NULL,
                                 Carbon = NULL,  
                                 SoilCNRatio = NULL,
                                 FOM = NULL,  
                                 FOM.CN=NULL,
                                 FBiom = NULL, 
                                 FInert = NULL,  
                                 NO3N = NULL,
                                 NH4N = NULL,
                                 CACO3 = NULL,
                                 PH = NULL,
                                 ParticleSizeClay = NULL,
                                 ParticleSizeSilt = NULL,
                                 ParticleSizeSand = NULL,
                                 soil.bottom = 150,
                                 water.table = 200, 
                                 soil.type = 0,
                                 crops = c("Maize","Soybean","Wheat"),
                                 metadata = NULL,
                                 soilwat = NA,
                                 swim = NA,
                                 initialwater = NA,
                                 solutes = NA,
                                 soilorganicmatter = NA,
                                 dist.parms = list(a = 0, b = 0.2),
                                 check = TRUE){

  if(!is.null(Depth) & !is.null(Thickness)){
    stop("Only specify Depth OR Thickness")
  }
  
  ## 1. and 2. Depth and Thickness
  if(missing(Depth) && missing(Thickness)){
    depth.0 <- round(seq(from = 0, to = soil.bottom, length.out = nlayers + 1))
      
    Depth <- character(nlayers)
    Thickness <- numeric(nlayers)
      
    for(i in 1:nlayers){
      Depth[i] <- paste0(depth.0[i],"-",depth.0[i+1])  
      Thickness[i] <- (depth.0[i + 1] - depth.0[i]) * 10
    }
  }
  ## If only Thickness is missing
  if(missing(Thickness) && !missing(Depth)){
    Thickness <- numeric(nlayers)
    for(i in 1:length(Depth)){
      tmp <- strsplit(Depth[i],"-")[[1]]
      Thickness[i] <- (tmp[2] - tmp[1]) * 10
    }
  }
  
  ## If only Depth is missing
  if(!missing(Thickness) && missing(Depth)){
    Depth <- numeric(nlayers)
    thcks <- cumsum(c(0,Thickness))/10
    for(i in 1:length(Thickness)){
      Depth[i] <- paste0(thcks[i],"-",thcks[i + 1])
    }
  }
  
  ## 3. Bulk density
  ## 1.1 is a default value of Bulk Density
  if(missing(BD)) BD <- 1.1 * soil_variable_profile(nlayers, 
                                                    a = dist.parms$a,
                                                    b = -0.05)
  if(is.list(BD)){
    if(length(BD) != 3) stop("BD list should be of length 3")
    ## First element will be the max value of BD 
    BD.max <- BD[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    BD <- BD.max * soil_variable_profile(nlayers, a = BD[[2]], b = BD[[3]])
  }
  
  ## 4. AirDry with default value 0.10
  if(missing(AirDry)) AirDry <- 0.10 * soil_variable_profile(nlayers, 
                                                             a = dist.parms$a,
                                                             b = dist.parms$b)
  if(is.list(AirDry)){
    if(length(AirDry) != 3) stop("AirDry list should be of length 3")
    ## First element will be the max value of AirDry 
    AirDry.max <- AirDry[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    AirDry <- AirDry.max * soil_variable_profile(nlayers, a = AirDry[[2]], b = AirDry[[3]])
  }
   
  ## 4. LL15 with default value of 0.15
  if(missing(LL15)) LL15 <- 0.15 * soil_variable_profile(nlayers, 
                                                             a = dist.parms$a,
                                                             b = dist.parms$b)
  if(is.list(LL15)){
    if(length(LL15) != 3) stop("LL15 list should be of length 3")
    ## First element will be the max value of LL15
    LL15.max <- LL15[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    LL15 <- LL15.max * soil_variable_profile(nlayers, a = LL15[[2]], b = LL15[[3]])
  }
  
  ## 5. DUL with default value of 0.25
  if(missing(DUL)) DUL <- 0.25 * soil_variable_profile(nlayers, 
                                                       a = dist.parms$a,
                                                       b = dist.parms$b)
  if(is.list(DUL)){
    if(length(DUL) != 3) stop("DUL list should be of length 3")
      ## First element will be the top value of DUL 
      DUL.max <- DUL[[1]]
      ## second element will be the a parameter 
      ## third element will be the b parameter 
      DUL <- DUL.max * soil_variable_profile(nlayers, a = DUL[[2]], b = DUL[[3]])
  }
    
  ## 6. SAT with default 0.45
  if(missing(SAT)) SAT <- 0.45 * soil_variable_profile(nlayers, 
                                                       a = dist.parms$a,
                                                       b = dist.parms$b)
  if(is.list(SAT)){
    if(length(SAT) != 3) stop("SAT list should be of length 3")
     ## First element will be the max value of SAT 
     SAT.max <- SAT[[1]]
     ## second element will be the a parameter 
     ## third element will be the b parameter 
     SAT <- SAT.max * soil_variable_profile(nlayers, a = SAT[[2]], b = SAT[[3]])
  }
    
  ## 7. KS with default value 100
  if(missing(KS)) KS <- 100 * soil_variable_profile(nlayers, 
                                                    a = dist.parms$a,
                                                    b = dist.parms$b)
  if(is.list(KS)){
    if(length(KS) != 3) stop("KS list should be of length 3")
    ## First element will be the top value of KS
    KS.max <- KS[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    KS <- KS.max * soil_variable_profile(nlayers, a = KS[[2]], b = KS[[3]])
  }
   
  ## 8. crop.LL with default value 0.15
  if(missing(crop.LL)) crop.LL <- 0.15 * soil_variable_profile(nlayers, 
                                                      a = dist.parms$a,
                                                      b = dist.parms$b)
  if(is.list(crop.LL)){
    if(length(crop.LL) != 3) stop("crop.LL list should be of length 3")
    ## First element will be the top value of crop.LL
    crop.LL.max <- crop.LL[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    crop.LL <- crop.LL.max * soil_variable_profile(nlayers, a = crop.LL[[2]], b = crop.LL[[3]])
  }
  
  ## 9. crop.KL with default value 0.06
  if(missing(crop.KL)) crop.KL <- 0.06 * soil_variable_profile(nlayers, 
                                                               a = dist.parms$a,
                                                               b = dist.parms$b)
  if(is.list(crop.KL)){
    if(length(crop.KL) != 3) stop("crop.KL list should be of length 3")
    ## First element will be the top value of crop.KL
    crop.KL.max <- crop.KL[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    crop.KL <- crop.KL.max * soil_variable_profile(nlayers, a = crop.KL[[2]], b = crop.KL[[3]])
  }
  
  ## 10. crop.XF with default value 1
  if(missing(crop.XF)) crop.XF <- 1 * soil_variable_profile(nlayers, 
                                                               a = dist.parms$a,
                                                               b = 0)
  if(is.list(crop.XF)){
    if(length(crop.XF) != 3) stop("crop.XF list should be of length 3")
    ## First element will be the top value of crop.XF
    crop.XF.max <- crop.XF[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    crop.XF <- crop.XF.max * soil_variable_profile(nlayers, a = crop.XF[[2]], b = crop.XF[[3]])
  }
  
  ## 11. Organic Carbon
  if(missing(Carbon)) Carbon <- 1.2 * soil_variable_profile(nlayers, 
                                                a = dist.parms$a,
                                                b = dist.parms$b)
  if(is.list(Carbon)){
    if(length(Carbon) != 3) stop("Carbon list should be of length 3")
    ## First element will be the top value of OC
    Carbon.max <- Carbon[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    Carbon <- Carbon.max * soil_variable_profile(nlayers, a = Carbon[[2]], b = Carbon[[3]])
  }
    
  ## 12. Organic Carbon C:N ratio
  if(missing(SoilCNRatio)) SoilCNRatio <- 12 * soil_variable_profile(nlayers, 
                                                      a = dist.parms$a,
                                                      b = 0)
  if(is.list(SoilCNRatio)){ 
    if(length(SoilCNRatio) != 3) stop("SoilCNRatio  list should be of length 3")
    ## First element will be the top value of SoilCNRatio 
    SoilCNRatio.max <- SoilCNRatio[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    SoilCNRatio <- SoilCNRatio.max * soil_variable_profile(nlayers, a = SoilCNRatio[[2]], b = SoilCNRatio[[3]])
  }
    
  ## 13. Fresh Organic Matter (kg/ha)
  if(missing(FOM)) FOM <- 150 * soil_variable_profile(nlayers, 
                                                  a = dist.parms$a,
                                                  b = dist.parms$b)
  if(is.list(FOM)){ 
    if(length(FOM) != 3) stop("FOM list should be of length 3")
    ## First element will be the top value of FOM
    FOM.max <- FOM[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    FOM <- FOM.max * soil_variable_profile(nlayers, a = FOM[[2]], b = FOM[[3]])
  }
    
  ## 14. Fresh Organic Matter C:N ratio
  if(missing(FOM.CN)) FOM.CN <- 40 * soil_variable_profile(nlayers, 
                                                        a = dist.parms$a,
                                                        b = 0)
  if(is.list(FOM.CN)){ 
    if(length(FOM.CN) != 3) stop("FOM.CN list should be of length 3")
    ## First element will be the top value of FOM.CN
    FOM.CN.max <- FOM.CN[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    FOM.CN <- FOM.CN.max * soil_variable_profile(nlayers, a = FOM.CN[[2]], b = FOM.CN[[3]])
  }
    
  ## 15. Fraction of microbial biomass with default 0.04
  if(missing(FBiom)) FBiom <- 0.04 * soil_variable_profile(nlayers, 
                                                        a = dist.parms$a,
                                                        b = dist.parms$b)
  if(is.list(FBiom)){ 
    if(length(FBiom) != 3) stop("FBiom list should be of length 3")
    ## First element will be the top value of FBiom
    FBiom.max <- FBiom[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    FBiom <- FBiom.max * soil_variable_profile(nlayers, a = FBiom[[2]], b = FBiom[[3]])
  }
    
  ## 16. Fraction of inert soil organic carbon
  if(missing(FInert)) FInert <- 0.8 * soil_variable_profile(nlayers, 
                                                      a = dist.parms$a,
                                                      b = -0.01)
  if(is.list(FInert)){ 
    if(length(FInert) != 3) stop("FInert list should be of length 3")
    ## First element will be the top value of FInert
    FInert.max <- FInert[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    FInert <- FInert.max * soil_variable_profile(nlayers, a = FInert[[2]], b = FInert[[3]])
  }
  
  ## 17. Chemical soil nitrate-N
  if(missing(NO3N)) NO3N <- 0.5 * soil_variable_profile(nlayers, 
                                                            a = dist.parms$a,
                                                            b = 0.01)
  if(is.list(NO3N)){ 
    if(length(NO3N) != 3) stop("NO3N list should be of length 3")
    ## First element will be the top value of NO3N
    NO3N.max <- NO3N[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    NO3N <- NO3N.max * soil_variable_profile(nlayers, a = NO3N[[2]], b = NO3N[[3]])
  }
  
  ## 18. Chemical soil ammonium-N
  if(missing(NH4N)) NH4N <- 0.05 * soil_variable_profile(nlayers, 
                                                        a = dist.parms$a,
                                                        b = 0.01)
  if(is.list(NH4N)){ 
    if(length(NH4N) != 3) stop("NH4N list should be of length 3")
    ## First element will be the top value of NH4N
    NH4N.max <- NH4N[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    NH4N <- NH4N.max * soil_variable_profile(nlayers, a = NH4N[[2]], b = NH4N[[3]])
  }
  
  ## 18b. Calcium Carbonates (CaCO3)
  if(missing(CACO3)) CACO3 <- 0.05 * soil_variable_profile(nlayers, 
                                                           a = dist.parms$a,
                                                           b = 0.01)
  if(is.list(CACO3)){ 
    if(length(CACO3) != 3) stop("CACO3 list should be of length 3")
    ## First element will be the top value of CACO3
    CACO3.max <- CACO3[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    CACO3 <- CACO3.max * soil_variable_profile(nlayers, a = CACO3[[2]], b = CACO3[[3]])
  }
   
  ## 19. Chemical soil PH
  if(missing(PH)) PH <- 6.5 * soil_variable_profile(nlayers, 
                                                         a = dist.parms$a,
                                                         b = 0)
  if(is.list(PH)){ 
    if(length(PH) != 3) stop("PH list should be of length 3")
    ## First element will be the top value of PH
    PH.max <- PH[[1]]
    ## second element will be the a parameter 
    ## third element will be the b parameter 
    PH <- PH.max * soil_variable_profile(nlayers, a = PH[[2]], b = PH[[3]])
  }
  
  ## These are some values for a sandy clay loam
  if(missing(ParticleSizeClay)){
    ParticleSizeClay <- rep(25, nlayers)
  }else{
    if(length(ParticleSizeClay) != nlayers)
      stop("ParticleSizeClay length should be equal to the number of layers", call. = FALSE)
  }
  
  if(missing(ParticleSizeSilt)){
    ParticleSizeSilt <- rep(15, nlayers)
  }else{
    if(length(ParticleSizeSilt) != nlayers)
      stop("ParticleSizeSilt length should be equal to the number of layers", call. = FALSE)
  }
  
  if(missing(ParticleSizeSand)){
    ParticleSizeSand <- rep(60, nlayers)
  }else{
    if(length(ParticleSizeSand) != nlayers)
      stop("ParticleSizeSand length should be equal to the number of layers", call. = FALSE)
  }
  
  ## Build the crop soil
  crop.soil <- as.data.frame(replicate(length(crops), do.call(cbind, list(crop.KL, crop.LL, crop.XF))))
  names(crop.soil) <- as.vector(sapply(crops, function(x) paste0(x, c(".KL", ".LL", ".XF"))))
    
  soil <- data.frame(Depth=Depth, Thickness=Thickness, BD=BD, 
                     AirDry=AirDry, LL15=LL15, DUL=DUL, SAT=SAT, 
                     KS=KS, Carbon=Carbon, 
                     SoilCNRatio=SoilCNRatio, FOM=FOM, 
                     FOM.CN=FOM.CN, FBiom=FBiom, FInert=FInert,
                     NO3N=NO3N, NH4N=NH4N, CACO3 = CACO3, PH=PH, 
                     ParticleSizeClay = ParticleSizeClay,
                     ParticleSizeSilt = ParticleSizeSilt,
                     ParticleSizeSand = ParticleSizeSand)
  
    names(soil) <- c("Depth","Thickness", "BD", "AirDry","LL15",
                     "DUL","SAT","KS", "Carbon","SoilCNRatio", "FOM",
                     "FOM.CN","FBiom","FInert",
                     "NO3N","NH4N", "CACO3", "PH", "ParticleSizeClay", "ParticleSizeSilt",
                     "ParticleSizeSand")
    
    soil <- cbind(soil, crop.soil)
    
    ans <- list(soil=soil, crops = crops, metadata = metadata, soilwat = soilwat, swim = swim, initialwater = initialwater, solutes = solutes, soilorganicmatter = soilorganicmatter)
    class(ans) <- "soil_profile"
    
    ## Check for reasonable values
    if(check) check_apsimx_soil_profile(ans)
    
    return(ans)
  }

#' This function creates a profile for a given soil variable
#' with flexibility about the given shape: \cr
#' 1. constant \cr
#' 2. exponential decay \cr
#' 3. ricker \cr
#' @name soil_variable_profile
#' @description create a profile given a number of layers
#' @param nlayers number of soil layers
#' @param a parameter in a function
#' @param b parameter in a function
#' @return It returns a numeric vector
#' @noRd
#' 
soil_variable_profile <- function(nlayers, a = 0.5, b = 0.5){
  
  if(a < 0) stop("a parameter cannot be negative")
  ## The shape will be based on the
  ## Ricker function
  ## Y = a * X * exp(-b * X)
  ## https://en.wikipedia.org/wiki/Ricker_model
  ## We'll see if it is flexible enough
  ## If not the specific proportions should be passed
  ## If a = 0, then I will use the exponential decay
  if(a > 0 & b != 0){
    tmp <- a * 1:nlayers * exp(-b * 1:nlayers)
    ans <- tmp/max(tmp)
  }
  
  if(a == 0 & b != 0){
    ans <- exp(-b * 1:nlayers) / exp(-b)
  }
  
  if(a == 0 & b == 0){
    ans <- rep(1, nlayers)
  }
  ans
}

#' @rdname apsimx_soil_profile
#' @description plotting function for a soil profile, it requires \sQuote{ggplot2}
#' @param x object of class \sQuote{soil_profile}.
#' @param ... additional plotting arguments (none use at the moment).
#' @param property \dQuote{all} for plotting all soil properties, \dQuote{water} for just SAT, DUL and LL15
#' @return it produces a plot
#' @export 
plot.soil_profile <- function(x,..., property = c("all", "water", "initialwater", "BD",
                                              "AirDry", "LL15", "DUL", "SAT",
                                              "KS", "Carbon", "SoilCNRatio", 
                                              "FOM", "FOM.CN", "FBiom",
                                              "FInert", "NO3N", "NH4N", "CACO3", "PH",
                                              "ParticleSizeClay", "ParticleSizeSilt",
                                              "ParticleSizeSand", "texture")){
  ## Test for existence of ggplot2
  if(!requireNamespace("ggplot2", quietly = TRUE)){
    warning("ggplot2 is required for this plotting function")
    return(NULL)
  }
  ## Really dumb... but for now...
  dist <- NA; soil.depths <- NA; soil.depth.bottom <- NA; SAT <- NA
  LL15 <- NA; DUL <- NA; AirDry <- NA; InitialValues <- NULL
  texture <- NULL; values <- NULL
  xsoil <- x$soil
  ## Add soil bottom depth
  xsoil$soil.depth.bottom <- sapply(as.character(xsoil$Depth), FUN = function(x) as.numeric(strsplit(x,"-")[[1]][2]))
  
  crops.property <- as.vector(sapply(x$crops, function(x) paste0(x, c(".XF", ".KL", ".LL"))))
  
  property.in.crops <- try(match.arg(property, choices = crops.property, several.ok = TRUE), silent = TRUE)
  
  properties <- c("all", "water", "initialwater", "BD",
                  "AirDry", "LL15", "DUL", "SAT",
                  "KS", "Carbon", "SoilCNRatio", 
                  "FOM", "FOM.CN", "FBiom",
                  "FInert", "NO3N", "NH4N", "CACO3", "PH",
                  "ParticleSizeClay", "ParticleSizeSilt",
                  "ParticleSizeSand", "texture", "ParticleSize")
    
  if(inherits(property.in.crops, "try-error")){
    property <- try(match.arg(property), silent = TRUE) 
    if(inherits(property, "try-error")){
      cat("property should be one of:", c(properties, crops.property), "\n")
      stop("property does not match on of the possible properties", call. = FALSE)
    }
  }else{
    property <- try(match.arg(property, choices = crops.property, several.ok = TRUE), silent = TRUE) 
    if(inherits(property, "try-error")){
      cat("Available crop properties", crops.property, "\n")
      stop("Crop property is not in the possible list of crop properties", call. = FALSE)      
    }
  }

  if(property != "all" && !(property %in% c("water", "initialwater", "texture", "ParticleSize"))){
    
    tmp <- xsoil[,c(property, "Depth","soil.depth.bottom")]
    gp <- ggplot2::ggplot() + 
                   ggplot2::geom_point(ggplot2::aes(x = tmp[,1], y = -tmp[,3])) + 
                   ggplot2::geom_path(ggplot2::aes(x = tmp[,1], y = -tmp[,3])) +
                   ggplot2::xlab(property) + 
                   ggplot2::ylab("Soil Depth (cm)") 
    print(gp)
  }
  
  if(property == "all"){
    
    tmp <- xsoil
    ## Check if property is missing
    tmp.nms0 <- names(tmp)
    tmp.nms <- setdiff(tmp.nms0, c("Depth", "Thickness"))

    dat0 <- NULL
    for(i in tmp.nms){
      tmp1 <- data.frame(var = i, dist = tmp[,i])
      dat0 <- rbind(dat0, tmp1)
    }
    ## This looks dumb, but I'd rather not need a new package for such a simple task
    # bd <- data.frame(var = "BD", dist = tmp[,"BD"]) # 1
    # ad <- data.frame(var = "AirDry", dist = tmp[,"AirDry"]) # 2
    # ll <- data.frame(var = "LL15", dist = tmp[,"LL15"]) # 3
    # dul <- data.frame(var = "DUL", dist = tmp[,"DUL"]) # 4
    # sat <- data.frame(var = "SAT", dist = tmp[,"SAT"]) # 5
    # ks <- data.frame(var = "KS", dist = tmp[,"KS"]) # 6
    # carbon <- data.frame(var = "Carbon", dist = tmp[,"Carbon"]) # 7
    # soilcn <- data.frame(var = "SoilCNRatio", dist = tmp[,"SoilCNRatio"]) # 8
    # fom <- data.frame(var = "FOM", dist = tmp[,"FOM"]) # 9
    # fom.cn <- data.frame(var = "FOM.CN", dist = tmp[,"FOM.CN"]) # 10
    # fbiom <- data.frame(var = "FBiom", dist = tmp[,"FBiom"]) # 11
    # finert <- data.frame(var = "FInert", dist = tmp[,"FInert"]) # 12
    # no3n <- data.frame(var = "NO3N", dist = tmp[,"NO3N"]) # 13
    # nh4n <- data.frame(var = "NH4N", dist = tmp[,"NH4N"]) # 14
    # ph <- data.frame(var = "PH", dist = tmp[,"PH"]) # 15
    
    ## The code below is not needed
    crops.xf <- NULL
    crops.kl <- NULL
    crops.ll <- NULL
    num.crops <- length(x$crops)
    num.crops.dats <- num.crops * 3
    for(i in x$crops){
      crops.xf <- rbind(crops.xf, data.frame(var = paste0(i, ".XF"), dist = tmp[,paste0(i, ".XF")]))
      crops.kl <- rbind(crops.kl, data.frame(var = paste0(i, ".KL"), dist = tmp[,paste0(i, ".KL")])) 
      crops.ll <- rbind(crops.ll, data.frame(var = paste0(i, ".LL"), dist = tmp[,paste0(i, ".LL")])) 
    }

    soil.depth.bottoms <- rep(xsoil$soil.depth.bottom, length(tmp.nms))
    
    # dat0 <- rbind(bd, ad, ll, dul, sat, ks, carbon,
    #               soilcn, fom, fom.cn, fbiom, finert, no3n, nh4n, ph,
    #               crops.xf, crops.kl, crops.ll)
    
    dat <- data.frame(dat0, soil.depths = soil.depth.bottoms)
    dat$soil.depth.bottom <- NULL
    
    gp <- ggplot2::ggplot(data = dat, ggplot2::aes(x = dist, y = -soil.depths)) +
      ggplot2::ylab("Soil Depth (cm)") + ggplot2::xlab("") + 
      ggplot2::facet_wrap(~var, scales = "free") +
      ggplot2::geom_path()
    print(gp)
  }
  
  if(property == "water"){
    
    tmp <- xsoil
    gp <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = -soil.depth.bottom, y = SAT)) +
              ggplot2::xlab("Soil Depth (cm)") + 
              ggplot2::ylab("proportion") + 
              ggplot2::geom_line() +
              ggplot2::geom_line(ggplot2::aes(x = -soil.depth.bottom, y = AirDry), color = "red") + 
              ggplot2::ggtitle("Soil water (AirDry, LL15, DUL, SAT)") +
              ggplot2::geom_ribbon(ggplot2::aes(ymin = LL15, ymax = DUL), 
                                   color = "blue",
                                   fill = "deepskyblue1") + 
              ggplot2::coord_flip()  
    print(gp)
  }
  
  if(property == "texture" || property == "ParticleSize"){
    
    tmp <- utils::stack(xsoil, select = c("ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand"))
    tmp$soil.depth.bottom <- xsoil$soil.depth.bottom
    tmp$texture <- factor(gsub("ParticleSize", "", tmp$ind), levels = c("Clay", "Silt", "Sand"))
    gp <- ggplot2::ggplot(data = tmp) +
      ggplot2::geom_area(ggplot2::aes(x = -soil.depth.bottom, y = values, fill = texture)) +
      ggplot2::ggtitle("Particle Size (Clay, Silt and Sand)") +
      ggplot2::labs(x = "Soil Depth (cm)", y = "Percent (%)", color = "Legend") + 
      ggplot2::scale_fill_manual(name = "Particle Size", 
                                 breaks = c("Clay", "Silt", "Sand"),
                                 values = c(Clay = "brown", Silt = "darkgoldenrod", Sand = "burlywood")) + 
      ggplot2::coord_flip()  
    print(gp)
  }
  
  if(property == "initialwater"){

    iwat <- x$initialwater
    if(!inherits(x$initialwater, "initialwater_parms")){
      stop("Could not find initial water", call. = FALSE)
    }else{
      if(any(is.na(iwat$Thickness))) stop("Thickness is missing in initial water", call. = FALSE)
      if(any(is.na(iwat$InitialWater))) stop("InitialWater is missing in initial water", call. = FALSE)
    }
    
    iwatd <- data.frame(Thickness = iwat$Thickness, InitialValues = iwat$InitialValues)
    iwatd$Depth <- .t2d(iwat$Thickness)
    iwatd$soil.depth.bottom <- sapply(as.character(iwatd$Depth), FUN = function(x) as.numeric(strsplit(x,"-")[[1]][2]))

    ### Need to insert soil.depth.bottom
    tmp <- xsoil
    gp <- ggplot2::ggplot() +
      ggplot2::geom_line(data = tmp, ggplot2::aes(x = -soil.depth.bottom, y = SAT, color = "SAT")) +
      ggplot2::geom_line(data = tmp, ggplot2::aes(x = -soil.depth.bottom, y = AirDry), color = "red") + 
      ggplot2::ggtitle("Soil water (AirDry, LL15, InitialWater, DUL, SAT)") +
      ggplot2::geom_ribbon(data = tmp, ggplot2::aes(x = -soil.depth.bottom, ymin = LL15, ymax = DUL), 
                           color = "blue",
                           fill = "deepskyblue1") + 
      ggplot2::geom_line(data = iwatd, ggplot2::aes(x = -soil.depth.bottom, y = InitialValues, color = "InitialWater")) + 
      ggplot2::labs(x = "Soil Depth (cm)", y = "proportion", color = "Legend") + 
      ggplot2::scale_color_manual(name = "Water", 
                                  breaks = c("SAT", "InitialWater"),
                                  values = c(SAT = "black", InitialWater = "purple")) + 
      ggplot2::coord_flip()
    print(gp)
  }
  
  invisible(gp)
}

#' Check an apsimx soil profile
#' 
#' The value of soil particle density (2.65 g/cm3) is hard coded in APSIM. 
#' https://en.wikipedia.org/wiki/Bulk_density
#' 
#' @rdname apsimx_soil_profile
#' @description checking an apsimx soil profile for reasonable values
#' @param x object of class \sQuote{soil_profile} or the \sQuote{soil} 
#' component within an object of class \sQuote{soil_profile}.
#' @param particle.density default value for soil particle density (2.65 g/cm3)
#' @return It does not produce output unless potential issues are found. Only warnings
#' are produced and it returns an object of class \sQuote{soil_profile}.
#' @export 
#' 

check_apsimx_soil_profile <- function(x, particle.density = 2.65){
  
  if(!inherits(x, "soil_profile"))
    stop("object should be of class 'soil_profile'", call. = FALSE)
  
  if(inherits(x, "soil_profile")){
    soil <- x$soil
  }else{
    soil <- x
  } 
  
  crop.vars <- as.vector(sapply(x$crops, function(x) paste0(x, c(".KL", ".LL", ".XF"))))
  
  vars <- c("Depth", "Thickness", "BD", "AirDry", "LL15",
            "DUL", "SAT", "KS",
            "Carbon", "SoilCNRatio", "FOM", "FOM.CN", "FBiom", "FInert",
            "NO3N", "NH4N", "CACO3", "PH", crop.vars)
  
  soil.names <- names(soil)
  
  if(length(soil.names %in% vars) < length(vars)){
    mtch.nms <- match(soil.names, vars)
    mssn.nms <- vars[-mtch.nms]
    warning(paste("One or more variables missing:", mssn.nms))
  }

  ## Depth cannot be easily checked, need to think harder about this one
  ## Thickness
  if(min(soil$Thickness) <= 0) warning("Thickness is zero or negative")
  ## Bulk Density (BD)
  if(min(soil$BD) <= 0) warning("BD (Bulk Density) cannot be zero or negative")
  if(max(soil$BD) >= 3) warning("BD (Bulk Density) value is too high")
  ## AirDry
  if(min(soil$AirDry) <= 0) warning("AirDry is zero or negative")
  if(max(soil$AirDry) > 1) warning("AirDry is too high")
  ## LL15
  if(min(soil$LL15) <= 0) warning("LL15 is zero or negative")
  if(max(soil$LL15) > 1) warning("LL15 is too high")
  ## DUL
  if(min(soil$DUL) <= 0) warning("DUL is zero or negative")
  if(max(soil$DUL) > 1) warning("DUL is too high")
  ## SAT
  if(min(soil$SAT) <= 0) warning("SAT is zero or negative")
  if(max(soil$SAT) > 1) warning("SAT is too high")
  ## KS 
  if(min(soil$KS) <= 0) warning("KS is zero or negative")
  ## crop.soil
  for(i in seq_along(crop.vars)){
    if(is.null(soil[[crop.vars[i]]])){
      warning(paste(crop.vars[i], "is not present"))
      next
    } 
    if(min(soil[[crop.vars[i]]]) < 0) warning(paste(crop.vars[i], "is negative"))
    if(max(soil[[crop.vars[i]]]) > 1) warning(paste(crop.vars[i], "is greater than 1"))
  }
  ## Carbon
  if(min(soil$Carbon) <= 0) warning("Carbon is zero or negative")
  ## SoilCNRatio
  if(min(soil$SoilCNRatio) <= 0) warning("SoilCNRatio is zero or negative")
  ## FOM
  if(min(soil$FOM) <= 0) warning("FOM is zero or negative")
  ## FOM.CN
  if(min(soil$FOM.CN) <= 0) warning("FOM.CN is zero or negative")
  ## FBiom
  if(min(soil$FBiom) <= 0) warning("FBiom is zero or negative")
  if(max(soil$FBiom) > 1) warning("FBiom is too high")
  ## FInert
  if(min(soil$FInert) <= 0) warning("FInert is zero or negative")
  if(max(soil$FInert) > 1) warning("FInert is too high")
  ## NO3N
  if(min(soil$NO3N) <= 0) warning("NO3N is zero or negative")
  ## NH4N
  if(min(soil$NH4N) <= 0) warning("NH4N is zero or negative")
  ## CACO3
  if(min(soil$CACO3) <= 0) warning("CACO3 is zero or negative")
  ## PH
  if(min(soil$PH) <= 0) warning("PH is zero or negative")
  if(max(soil$PH) > 14) warning("PH is too high")
  
  ## Checking texture if it exists
  if(!is.null(soil$Clay)){
    if(any(soil$Clay > 100)) warning("Clay cannot be greater than 100")
    if(any(soil$Clay < 0)) warning("Clay is negative")
  }
  
  if(!is.null(soil$Silt)){
    if(any(soil$Silt > 100)) warning("Silt cannot be greater than 100")
    if(any(soil$Silt < 0)) warning("Silt is negative")
  }
  
  if(!is.null(soil$Sand)){
    if(any(soil$Sand > 100)) warning("Sand cannot be greater than 100")
    if(any(soil$Sand < 0)) warning("Sand is negative")
  }
  
  SATminusDUL <- soil$SAT - soil$DUL
  DULminusLL <- soil$DUL - soil$LL
  DULminuscrop.LL <- soil$DUL - soil$crop.LL
  SATminusLL <- soil$SAT - soil$LL
  LLminuscrop.LL <- soil$LL - soil$crop.LL
  LLminusAirDry <- soil$LL - soil$AirDry
  AirDryminuscrop.LL <- soil$AirDry - soil$crop.LL
  
  if(any(SATminusDUL <= 0))
    warning("DUL cannot be greater than SAT")
  
  if(any(DULminusLL <= 0))
    warning("LL cannot be greater than DUL")
  
  if(any(DULminuscrop.LL <= 0))
    warning("crop.LL cannot be greater than DUL")

  if(any(SATminusLL <= 0))
    warning("LL cannot be greater than SAT")
  
  if(any(LLminuscrop.LL < 0))
    warning("crop.LL cannot be lower than soil LL")
  
  if(any(LLminusAirDry < 0))
    warning("AirDry cannot be greater than LL")
  
  if(any(AirDryminuscrop.LL < 0))
    warning("crop.LL cannot be lower than AirDry")
  
  ## This check is from APSIM Next Gen Models/Soils/Soil.cs line 250
  ## Compute max BD for each layer
  spd <- particle.density
  max_bd <- (1 - soil$SAT) * spd
  bd_diff <- max_bd - soil$BD

  for(j in seq_along(max_bd)){
    if(bd_diff[j] <= 0){
      warning(paste("Saturation of:", soil$SAT[j], "in layer:", j, "is above acceptable value of:", 1 - soil$BD[j] / spd, ".",
                    "You must adjust bulk density to below", (1 - soil$SAT[j]) * spd, "OR saturation to below", 1 - soil$BD[j] / 2.65))      
    }
  }
  
  ## Check for initial water
  if(inherits(soil$initialwater, "initialwater_parms")){
    ## Initial Water can't be greater than DUL?
    if(!is.na(soil$initialwater$InitialValues)){
      if(length(soil$initialwater$InitialValues) != length(soil$DUL))
        warning("Number of layers in initialwater$InitialValues is different from number of layers in soil$DUL")
      for(j in seq_along(soil$initialwater$InitialValues)){
        iwat <- soil$initialwater$InitialValues[j] - soil$DUL[j]
        if(iwat <= 0){
          warning(paste("Initial Water in layer:", j, "is greater than DUL"))
        }
      }      
    }
  }
  
  ## Check for Solutes
  if(inherits(soil$solutes, "solutes_parms")){
    ## Initial Water can't be greater than DUL?
    for(j in seq_len(soil$solutes$Solutes)){
      if(!is.na(soil$solutes[[j]]$InitialValues)){
        if(length(soil$solutes[[j]]$InitialValues) != length(soil$solutes[[j]]$Thickness))
          warning(paste("Number of layers in solutes", soil$solutes$Solutes[j], "$InitialValues is different from number of Thickness layers"))
        if(!all(sapply(soil$solutes[[j]]$InitialValues, is.numeric)))
          warning("Some IntialValues are not numeric")
      }      
    }
  }
  
  return(invisible(x))
}

#'
#' @title Fixing a soil profile
#' @name fix_apsimx_soil_profile
#' @param x object of class \sQuote{soil_profile}
#' @param soil.var whether to change SAT or BD. At the moment it only changes SAT.
#' @param particle.density soil particle density 
#' @param verbose whether to print the changes made to the soil profile
#' @noRd
#' @author Fernando Miguez with input from Julien Morel
#' @examples
#' \donttest{
#' sp <- get_isric_soil_profile(lonlat = c(1, 48))
#' ## This produces a warning
#' sp1 <- apsimx:::fix_apsimx_soil_profile(sp)
#' check_apsimx_soil_profile(sp1)
#' }
fix_apsimx_soil_profile <- function(x, soil.var = c("SAT", "BD"), particle.density = 2.65, verbose = TRUE){
  
  if(!inherits(x, "soil_profile"))
    stop("object should be of class 'soil_profile'", call. = FALSE)
  ## Heuristics for fixing soil profiles
  
  soil.var <- match.arg(soil.var)
  
  ## Bulk density and saturation issue
  max_bd <- (1 - x$soil$SAT) * 2.65
  bd_diff <- max_bd - x$soil$BD
  
  for(j in seq_along(max_bd)){
    if(bd_diff[j] <= 0){
      x$soil$SAT[j] <- 1 - x$soil$BD[j] / 2.65 - 0.001
      if(verbose && soil.var == "SAT"){
        cat("Saturation of:", x$soil$SAT[j], "in layer:", j, "was above acceptable value of:", 1 - x$soil$BD[j] / 2.65, ".",
            "It was adjusted to:", 1 - x$soil$BD[j] / 2.65 - 0.001, "\n")              
      }
      ## Fixing LL and air dry issue
      if(x$soil$LL15[j] < x$soil$AirDry[j]){
        if(verbose){
          cat("LL15 cannot be lower than AirDry in layer:", j,".\n",
              "It was adjusted to the value of AirDry.\n")
        }
        x$soil$LL15[j] <- x$soil$AirDry[j]
      }
    }
  }
  
  ## Trying to fix the initialwater issue
  if(inherits(x$initialwater, "initialwater_parms")){
      if(!any(is.na(x$initialwater$InitialValues))){
        for(j in seq_along(x$initialwater$InitialValues)){
          iwat <- x$initialwater$InitialValues[j] - x$soil$DUL[j]
          if(length(iwat < 0) > 1){
            cat("Class of iwat", class(iwat), "\n")
            cat("InitialValues:", x$initialwater$InitialValues[j] , "\n")
            cat("DUL:", x$soil$DUL[j] , "\n")
            cat("length of InitialWater:", length(iwat < 0), "layer", j, "\n")
            stop("iwat length greater than one")
          }
          if(any(iwat > 0)){
            ### Changed to greater than zero per github issue #208 by LucioLourido
            x$initialwater$InitialValues[j] <- x$soil$DUL[j] * 0.9  
            if(verbose){
              cat("InitialWater cannot be greater than DUL in layer:", j,".\n",
                  "It was adjusted to the value of", x$soil$DUL[j] * 0.9, ".\n")
            }
          }
        }
      }
  }
  return(x)
}


#' 
#' @title Compare two or more soil profiles
#' @name compare_apsim_soil_profile
#' @rdname compare_apsim_soil_profile
#' @description Helper function which allows for a simple comparison among soil_profile objects
#' @param ... soil_profile objects. Should be of class \sQuote{soil_profile}
#' @param soil.var soil variable to use in the comparison. Either \sQuote{all},
#' or several others such as \SQuote{BD}, \sQuote{DUL} or \sQuote{Carbon}. 
#' @param property same as soil.var 
#' @param labels labels for plotting and identification of \sQuote{soil_profile} objects.
#' @param merge.wide whether to attempt to merge soils in \sQuote{wide} format.
#' @param check whether to check \sQuote{soil_profile} objects using \sQuote{check_apsimx_soil_profile}.
#' @param verbose whether to print agreement values (default is FALSE).
#' @note I have only tested this for 2 or 3 objects. The code is set up to be able to 
#' compare more, but I'm not sure that would be all that useful.
#' @export
#' @return object of class \sQuote{soil_profile_mrg}, which can be used for further plotting
#' @examples 
#' \dontrun{
#' require(soilDB)
#' require(sp)
#' require(sf)
#' require(spData)
#' # Get two soil profiles
#' sp1 <- get_ssurgo_soil_profile(lonlat = c(-93, 42))
#' sp2 <- get_ssurgo_soil_profile(lonlat = c(-92, 41))
#' # Compare them
#' cmp <- compare_apsim_soil_profile(sp1[[1]], sp2[[1]], labels = c("sp1", "sp2"))
#' # Plot the variables
#' plot(cmp)
#' }
#' 

compare_apsim_soil_profile <- function(..., 
                                       soil.var = c("all", "Thickness", 
                                          "BD", "AirDry", "LL15", 
                                          "DUL", "SAT", "KS", "Carbon", "SoilCNRatio",
                                          "FOM", "FOM.CN", "FBiom", "FInert", "NO3N",
                                          "NH4N", "CACO3", "PH", "ParticleSizeClay", 
                                          "ParticleSizeSilt", "ParticleSizeSand"),
                                      property,
                                      labels,
                                      merge.wide = TRUE,
                                      check = FALSE,
                                      verbose = FALSE){
  
  soils <- list(...)
  
  n.soils <- length(soils)
  
  soil.var <- match.arg(soil.var)
  
  if(!missing(property)) soil.var <- property
  
  if(!missing(property) && soil.var == "all")
    warning("Either use property or soil.var but not both. soil.var will be ignored.")
  
  if(n.soils < 2) stop("you should provide at least two soil_profiles", call. = FALSE)
  
  #### Long merging of soils. ----
  ## It is not required that they have the same number of layers 
  soils.soil.names <- lapply(soils, FUN = function(x) names(x$soil))
  soils.soil.common.names <- Reduce(intersect, soils.soil.names)
  
  if(is.null(soils.soil.common.names))
    stop("No common column names between 'soil' data.frames", call. = FALSE)
  
  if(missing(labels)){
    soil.long.labels <- unlist(lapply(soils, FUN = function(x) x$metadata$SoilType))
  }else{
    soil.long.labels <- labels
  }
  
  if(length(soil.long.labels) != length(soils))
    stop("Length of labels should be equal to the number of soil profiles", call. = FALSE)
  
  soils.long <- NULL
  for(ii in seq_along(soils)){
    tmp.long <- subset(soils[[ii]]$soil, select = soils.soil.common.names)
    ## This depth will be used for plotting
    tmp.long$depth <- -(cumsum(tmp.long$Thickness) * 1e-1)
    soils.long <- rbind(soils.long, data.frame(soil = soil.long.labels[ii], tmp.long))
  }

  soil1 <- soils[[1]]
  
  m.nms <- NULL
  if(!missing(labels)){
    m.nms <- labels
    if(length(labels) != n.soils)
      stop(" 'labels' length should be the same as the number of 'soil_profile' objects", call. = FALSE)
  } 
  
  if(!inherits(soil1, "soil_profile")) stop("object should be of class 'soil_profile' ", call. = FALSE)
  
  ## Check for any issues
  if(check) check_apsimx_soil_profile(soil1$soil)
  
  ## Should have the same number of layers
  soil.mrg <- NA; ans <- NA
  if(merge.wide){
    soil.mrg <- soil1$soil
    names(soil.mrg) <- paste0(names(soil.mrg), ".1")  
    soil1 <- soil1$soil
    nms1 <- names(soil1)    
  }

  if(merge.wide){
    for(i in 2:n.soils){
      
      if(check) check_apsimx_soil_profile(soils[[i]])
      
      soil.i <- soils[[i]]$soil
      
      if(nrow(soil1) != nrow(soil.i)){
        message("Soil profiles should have the same number of rows")
        message("Not merging soils in wide format")
        stop("Try setting 'merge.wide' argument to FALSE", call. = FALSE)
      } 
      
      if(ncol(soil1) != ncol(soil.i) || length(setdiff(names(soil1), names(soil.i))) > 0){
        warning("Number of columns is not the same for the soil profiles. Selecting the ones in common.")
        common.names <- intersect(names(soil1), names(soil.i))
        soil1 <- subset(soil1, select = common.names)
        soil.i <- subset(soil.i, select = common.names)
      }
      
      names(soil1) <- nms1
      names(soil.i) <- paste0(names(soil.i), ".", i)
      ## drop the year.i and day.i names
      soil.mrg <- cbind(soil.mrg, soil.i)
    }
    
    if(soil.var == "all"){
      if(n.soils == 2){
        ans <- data.frame(variable = setdiff(names(soil1), c("Depth")),
                          vs = NA, labels = NA,
                          bias = NA, slope = NA, corr = NA)
        if(missing(labels)) ans$labels <- NULL        
      }else{
        ans <- data.frame(variable = rep(setdiff(names(soil1), c("Depth")), each = n.soils - 1),
                          vs = NA, labels = NA,
                          bias = NA, slope = NA, corr = NA)
        if(missing(labels)) ans$labels <- NULL
      }
    }else{
      ans <- data.frame(variable = soil.var,
                        vs = NA, labels = NA,
                        bias = NA, slope = NA, corr = NA)
      if(missing(labels)) ans$labels <- NULL
    }
    
    ## Calculate bias for all variables
    if(soil.var == "all"){
      soil.var.sel <- nms1[!(nms1 %in% c("Depth"))]
      gvar.sel <- paste0(soil.var.sel, collapse = "|")
      idx.soil.mrg <- grep(gvar.sel, names(soil.mrg))
      soil.mrg.s <- soil.mrg[,idx.soil.mrg]

      k <- 1 
      ## Compute Bias matrix
      for(i in soil.var.sel){
        if(verbose) cat("Variable ", i, "\n")
        ans$variable[k] <- i
        
        if(grepl("FOM", i)){
          if(i == "FOM"){
            tmp <- soil.mrg.s[, grep("FOM.[1-9]", names(soil.mrg.s))]    
          }
          if(i == "FOM.CN"){
            tmp <- soil.mrg.s[, grep("FOM.CN.[1-9]", names(soil.mrg.s))]   
          }          
        }else{
          tmp <- soil.mrg.s[, grep(i, names(soil.mrg.s)), drop = FALSE]  
        }
        
        if(ncol(tmp) < 2){
          stop("merged selected variables should be at least of length 2", call. = FALSE)
        } 
        
        for(j in 2:ncol(tmp)){ ## number of columns is equal to number of soils

          if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
          ans$vs[k] <- paste(names(tmp)[j - 1], "vs.", names(tmp)[j])
          if(!missing(labels)){
            if(verbose) cat("labels", labels[j - 1], " vs. ", labels[j], "\n")
            ans$labels[k] <- paste(labels[j - 1], "vs.", labels[j])
          } 
          if(abs(sum(tmp[, j - 1] - tmp[, j])) < 0.0001){
            if(verbose) cat(paste("Variable", i, "appears identical \n"))
            ans$bias[k] <- NA
            ans$slope[k] <- NA
            ans$corr[k] <- NA
            ans$rss[k] <- NA
            ans$rmse[k] <- NA
            k <- k + 1
            next
          }
          
          fm0 <- lm(tmp[, j - 1] ~ tmp[, j])
          if(verbose) cat(" \t Bias: ", coef(fm0)[1], "\n")
          ans$bias[k] <- coef(fm0)[1]
          if(verbose) cat(" \t Slope: ", coef(fm0)[2], "\n")
          ans$slope[k] <- coef(fm0)[2]
          if(verbose) cat(" \t Corr: ", cor(tmp[,j - 1], tmp[, j]), "\n")
          ans$corr[k] <- cor(tmp[,j - 1], tmp[, j])
          if(verbose) cat(" \t RSS: ", deviance(fm0), "\n")
          ans$rss[k] <- deviance(fm0)
          if(verbose) cat(" \t RMSE: ", sigma(fm0), "\n")
          ans$rmse[k] <- sigma(fm0)
          k <- k + 1
        }
      }
    }
    
    if(soil.var != "all"){
      ## Just select the appropriate variable
      idx.soil.mrg <- grep(soil.var, names(soil.mrg))
      soil.mrg.s <- soil.mrg[,idx.soil.mrg]
      
      if(verbose) cat("Variable ", soil.var, "\n")
      ans$variable[1] <- soil.var
      
      tmp <- soil.mrg.s
      for(j in 2:ncol(tmp)){
        if(verbose) cat(names(tmp)[j - 1], " vs. ", names(tmp)[j], "\n")
        ans$vs[1] <- paste(names(tmp)[j - 1], "vs.", names(tmp)[j])
        if(!missing(labels)){
          if(verbose) cat("labels", labels[j - 1], " vs. ", labels[j], "\n")
          ans$labels[1] <- paste(labels[j - 1], "vs.", labels[j])
        }
        fm0 <- lm(tmp[, j - 1] ~ tmp[, j])
        if(verbose) cat(" \t Bias: ", coef(fm0)[1], "\n")
        ans$bias[1] <- coef(fm0)[1]
        if(verbose) cat(" \t Slope: ", coef(fm0)[2], "\n")
        ans$slope[1] <- coef(fm0)[2]
        if(verbose) cat(" \t Corr: ", cor(tmp[,j - 1], tmp[, j]), "\n")
        ans$corr[1] <- suppressWarnings(cor(tmp[,j - 1], tmp[, j]))
        if(verbose) cat(" \t RSS: ", deviance(fm0), "\n")
        ans$rss[1] <- deviance(fm0)
        if(verbose) cat(" \t RMSE: ", sigma(fm0), "\n")
        ans$rmse <- sigma(fm0)
      }
    }    
  }

  attr(soil.mrg, "soil.names") <- m.nms
  attr(soil.mrg, "length.soils") <- n.soils  
  attr(soil.mrg, "merge.wide") <- merge.wide
  soil.mrg <- structure(list(soil.mrg = soil.mrg, index.table = ans, soils.long = soils.long),
                       class = "soil_profile_mrg")
  invisible(soil.mrg)
}

#' print method for soil_profile_mrg
#' Only variables which are not identical will be printed
#' @rdname compare_apsim_soil_profile
#' @description print method for \sQuote{soil_profile_mrg}
#' @param x object of class \sQuote{soil_profile_mrg}
#' @param ... additional arguments passed to print
#' @param format either \sQuote{wide} or \sQuote{long}, which depends on the merging.
#' @param digits number of digits to print (default is 2)
#' @return a table with indexes for the soil profiles
#' @export
print.soil_profile_mrg <- function(x, ..., format = c("wide", "long"), digits = 2){
  
  format <- match.arg(format)
  
  if(attr(x$soil.mrg, "merge.wide")){
    print(x$index.table[!is.na(x$index.table$bias),], digits = digits)  
  }
  
  if(format == "long" || !attr(x$soil.mrg, "merge.wide")){
   cat("Common names in 'long' format", names(x$soils.long), "\n") 
   cat("Dimmensions in 'long' format", dim(x$soils.long), "\n")
   print(head(x$soils.long))
  }
}

#' Plotting function for comparing soil profiles
#' @rdname compare_apsim_soil_profile
#' @description plotting function for compare_apsim_soil_profile, it requires ggplot2
#' @param x object of class \sQuote{soil_profile_mrg}
#' @param ... \sQuote{soil_profile} objects. Should be of class \sQuote{soil_profile}
#' @param plot.type either \sQuote{depth}, \sQuote{vs}, \sQuote{diff} or \sQuote{density}
#' @param pairs pair of objects to compare, defaults to 1 and 2 but others are possible
#' @param soil.var soil variable to plot 
#' @param property same as soil.var 
#' @param span argument to be passed to \sQuote{geom_smooth}
#' @return it produces a plot
#' @export
#' 
plot.soil_profile_mrg <- function(x, ..., plot.type = c("depth", "vs", "diff", "density"),
                         pairs = NULL,
                         soil.var = c("all", "Thickness", 
                                      "BD", "AirDry", "LL15", 
                                      "DUL", "SAT", "KS", "Carbon", "SoilCNRatio",
                                      "FOM", "FOM.CN", "FBiom", "FInert", "NO3N",
                                      "NH4N", "CACO3", "PH", "ParticleSizeClay", 
                                      "ParticleSizeSilt", "ParticleSizeSand"),
                         property,
                         span = 0.75){
  
  if(!requireNamespace("ggplot2", quietly = TRUE)){
    warning("ggplot2 is required for this plotting function")
    return(NULL)
  }

  soil.vars <- c("all", "Thickness", 
               "BD", "AirDry", "LL15", 
               "DUL", "SAT", "KS", "Carbon", "SoilCNRatio",
               "FOM", "FOM.CN", "FBiom", "FInert", "NO3N",
               "NH4N", "CACO3", "PH", "ParticleSizeClay", 
               "ParticleSizeSilt", "ParticleSizeSand")
  
  plot.type <- match.arg(plot.type)
  soil.var <- match.arg(soil.var)
  
  if(!missing(property)){
    ### If property is not in the list of 'soil.var' throw an error
    msp <- property %in% soil.vars
    if(isFALSE(msp)){
      stop("'property' does not match one of the soil variables", call. = FALSE)
    }else{
      soil.var <- soil.vars[match(property, soil.vars)]
    }
  } 
  
  if(!missing(property) && soil.var == "all")
    warning("Either use property or soil.var but not both. soil.var will be ignored.")
  
  if(plot.type != "depth" && soil.var == "all")
    stop("Please select a soil variable for this type of plot", call. = FALSE)
  
  if(is.null(pairs) && attr(x$soil.mrg, "length.soils") == 2){
    pairs <- c(1, 2)
  }
  
  if(attr(x$soil.mrg, "merge.wide")){
    ### All this code only makes sense for 'merge.wide'
    x <- x$soil.mrg  
    
    value <- NULL; depth <- NULL; soil <- NULL; variable <- NULL
    
    m.nms <- attr(x, "soil.names")
    if(!is.null(pairs)){
      if(max(pairs) > attr(x, "length.soils")) stop("pairs index larger than length of soils")  
    }
    
    if(soil.var == "all" || (soil.var != "all" && is.null(pairs))){
      num.vars <- length(grep(".1", names(x), fixed = TRUE))
      num.soils <- attr(x, "length.soils")
      soil.labels <- attr(x, "soil.names")
      tmp <- NULL
      for(i in seq_len(num.soils)){
        wch.col <- grep(paste0(".", i), names(x), fixed = TRUE)
        if(length(wch.col) != num.vars)
          stop("Could not merge soil profiles", call. = FALSE)
        tmp0 <- x[,wch.col]
        names(tmp0) <- gsub(paste0(".", i), "", names(tmp0), fixed = TRUE)
        tmp1 <- data.frame(soil = soil.labels[i], tmp0)
        ## Insert depth variable
        tmp1$depth[1] <- tmp1$Thickness[1] / 2
        tmp1$cum.thickness <- cumsum(tmp1$Thickness)
        for(i in 2:nrow(tmp1)){
          tmp1$depth[i] <- tmp1$cum.thickness[i - 1] + tmp1$Thickness[i] / 2
        }      
        tmp1$Depth <- NULL
        tmp2 <- NULL
        vars <- setdiff(names(tmp1), c("soil", "Thickness"))
        for(j in seq_along(vars)){
          tmp2 <- rbind(tmp2, data.frame(soil = tmp1[["soil"]], depth = tmp1$depth, 
                                         variable = vars[j], value = tmp1[[vars[j]]]))
        }
        tmp <- rbind(tmp, tmp2)
      }
      tmp.o <- tmp[order(tmp$soil, tmp$variable, tmp$depth),]
      tmp3 <- tmp.o[!tmp.o$variable %in% c("depth", "cum.thickness"),]

      if(soil.var == "all"){
        gp1 <- ggplot2::ggplot(data = tmp3, ggplot2::aes(x = value, y = depth * 0.1, color = soil)) + 
          ggplot2::facet_wrap(~ variable, scales = "free") + 
          ggplot2::geom_point() + 
          ggplot2::geom_path() + 
          ggplot2::scale_y_reverse() + 
          ggplot2::ylab("Depth (cm)")         
      }else{
        tmp4 <- subset(tmp3, variable == soil.var)
        gp1 <- ggplot2::ggplot(data = tmp4, ggplot2::aes(x = value, y = depth * 0.1, color = soil)) + 
          ggplot2::geom_point() + 
          ggplot2::geom_path() + 
          ggplot2::scale_y_reverse() + 
          ggplot2::ylab("Depth (cm)") +
          ggplot2::xlab(soil.var)
      }

      print(gp1)
    }
    
    if(plot.type == "vs" && soil.var != "all"){
      if(is.null(pairs))
        stop("argument 'pairs' should not be 'NULL' for this plot")
      tmp <- x[, grep(soil.var, names(x))]
      prs <- paste0(soil.var, ".", pairs)
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs[1]))), 
                                                      y = eval(parse(text = eval(prs[2]))))) +
        ggplot2::geom_point() + 
        ggplot2::xlab(paste(m.nms[pairs[1]], prs[1])) + 
        ggplot2::ylab(paste(m.nms[pairs[2]], prs[2])) + 
        ggplot2::geom_smooth(method = "lm") + 
        ggplot2::geom_abline(intercept = 0, slope = 1, color = "orange")
      
      print(gp1)
    }
    
    if(plot.type == "diff" && soil.var != "all"){
      if(is.null(pairs))
        stop("argument 'pairs' should not be 'NULL' for this plot")
      prs0 <- paste0(soil.var, ".", pairs)
      prs <- paste0(prs0, collapse = "|")
      tmp <- x[, grep(prs, names(x))]
      
      ## x Variable is prs[1]
      ## y Variable is prs[2] - prs[1]
      dff <- tmp[,prs0[2]] - tmp[,prs0[1]]
      
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))), 
                                                      y = dff)) +
        ggplot2::geom_point() + 
        ggplot2::xlab(paste(m.nms[pairs[1]], prs0[1])) + 
        ggplot2::ylab(paste("Difference", prs0[2], "-", prs0[1])) + 
        ggplot2::geom_smooth(method = "lm", ...) + 
        ggplot2::geom_hline(yintercept = 0, color = "orange")
      
      print(gp1)   
    }
    
    if(plot.type == "depth" && soil.var != "all" && !is.null(pairs)){
      
      prs0 <- paste0(soil.var, ".", pairs)
      prs <- paste0(prs0, collapse = "|")
      x$depth[1] <- x$Thickness.1[1] / 2
      x$cum.thickness <- cumsum(x$Thickness.1)
      for(i in 2:nrow(x)){
        x$depth[i] <- x$cum.thickness[i - 1] + x$Thickness.1[i] / 2
      } 
      tmp <- x[, grep(prs, names(x))]
      tmp$Depth <- x$depth * 0.1
      
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(y = .data[["Depth"]], 
                                                      x = eval(parse(text = eval(prs0[1]))),
                                                      color = paste(m.nms[pairs[1]], prs0[1]))) +
        
        ggplot2::geom_point() + 
        ggplot2::geom_path() + 
        ggplot2::geom_point(ggplot2::aes(x = eval(parse(text = eval(prs0[2]))),
                                         color = paste(m.nms[pairs[2]], prs0[2]))) + 
        ggplot2::geom_path(ggplot2::aes(x = eval(parse(text = eval(prs0[2]))),
                                        color = paste(m.nms[pairs[2]], prs0[2]))) + 
        ggplot2::ylab("Depth (cm)") + 
        ggplot2::xlab(soil.var) + 
        ggplot2::scale_y_reverse() + 
        ggplot2::theme(legend.title = ggplot2::element_blank())
      
      print(gp1)   
    }
    

    if(plot.type == "density" && soil.var != "all"){
      if(is.null(pairs))
        stop("argument 'pairs' should not be 'NULL' for this plot")
      prs0 <- paste0(soil.var, ".", pairs)
      prs <- paste0(prs0, collapse = "|")
      tmp <- x[, grep(prs, names(x))]
      
      gp1 <- ggplot2::ggplot(data = tmp, ggplot2::aes(x = eval(parse(text = eval(prs0[1]))),
                                                      color = paste(m.nms[pairs[1]], prs0[1]))) + 
        ggplot2::geom_density() + 
        ggplot2::geom_density(ggplot2::aes(x = eval(parse(text = eval(prs0[2]))),
                                           color = paste(m.nms[pairs[2]], prs0[2]))) +
        ggplot2::xlab(soil.var) + 
        ggplot2::theme(legend.title = ggplot2::element_blank())
      
      print(gp1)
    }
    
  }else{
    x <- x$soils.long
    
    ### First option is plot.type = "depth" and soil.var == "all"
    if(plot.type == "depth" && soil.var == "all"){
      
      v2e <- setdiff(names(x), c("soil", "depth"))
      x2 <- subset(x, select = v2e)
      cdat <- NULL
      for(i in seq_len(dim(x2)[2])){
        if(names(x2)[i] == "Depth") next
        tmp <- data.frame(soil = x$soil, depth = x$depth, 
                          variable = names(x2)[i], value = x2[[i]])
        cdat <- rbind(cdat, tmp)
      }
      gp1 <- ggplot2::ggplot(data = cdat, ggplot2::aes(x = value, y = depth, color = soil)) + 
        ggplot2::facet_wrap(~ variable, scales = "free") + 
        ggplot2::geom_point() + 
        ggplot2::geom_path() + 
        ggplot2::ylab("Depth (cm)") 
      print(gp1)
    }
      
    ### Second option is plot.type = "depth" and soil.var != "all"
    if(plot.type == "depth" && soil.var != "all"){
      fsvn <- grep(soil.var, names(x)) ## Find soil.var in data.frame names
      if(length(fsvn) == 0)
        stop("'soil.var' not found in long soil data frame")
      gp1 <- ggplot2::ggplot(data = x, ggplot2::aes(x = depth, y = .data[[soil.var]], color = soil)) + 
        ggplot2::geom_point() + 
        ggplot2::geom_line() + 
        ggplot2::ylab(soil.var) + 
        ggplot2::coord_flip()
      print(gp1)
    }
  }

  invisible(gp1)
}

#' Function to calculate carbon stocks. The output units depend on the choice of area.
#' If \sQuote{m2} is used, then the output units will be \sQuote{kg/m2}. If the \sQuote{area}
#' is \sQuote{ha}, then the output units will be \sQuote{Mg/ha}.
#' 
#' Note that the bulk density (which is needed in the calculation) is
#' available as part of the \sQuote{soil_profile} object.
#' 
#' @title Calculate soil carbon stocks
#' @description Calculation of carbon stocks based on an object of class \sQuote{soil_profile}
#' @name carbon_stocks
#' @param x object of class \sQuote{soil_profile}
#' @param depth soil depth (in meters). If missing then the whole soil profile is used.
#' @param area either \sQuote{m2} meter squared or \sQuote{ha}.
#' @param method interpolation method. Either \sQuote{linear} or \sQuote{constant}.
#' @param value whether to return a \sQuote{numeric} or an object of class \sQuote{carbon.stocks}.
#' @param ... additional arguments passed to internal functions (none used at the moment).
#' @return returns a value with attribute \sQuote{units} and \sQuote{depth}
#' @export
#' @examples 
#' \dontrun{
#' sp <- apsimx_soil_profile()
#' carbon_stocks(sp)
#' carbon_stocks(sp, depth = 0.1)
#' carbon_stocks(sp, depth = 0.2)
#' carbon_stocks(sp, depth = 0.3)
#' carbon_stocks(sp, depth = 0.4)
#' }

carbon_stocks <- function(x, 
                          depth = NULL, 
                          area = c("m2", "ha"), 
                          method = c("linear", "constant"),
                          value = c("numeric", "carbon.stocks"), ...){
  
  if(!inherits(x, "soil_profile")){
    stop("This function is intended to be used with an object of class 'soil_profile'", call. = FALSE)
  }
  
  value <- match.arg(value)
  
  bottom <- sum(x$soil$Thickness) * 1e-3 ## Thickness is in mm, so after conversion this is in meters
  
  if(!is.null(depth)){
    if(depth <= 0) stop("'depth' should be a positive number", call. = FALSE)
    if(depth > bottom) stop("'depth' should be a lower number than the bottom of the soil profile ", call. = FALSE)
    if(depth > 10){
      warning("'depth' should be in meters and the value entered is larger than 10. Is this correct?")
    }
  }
  
  area <- match.arg(area)
  method <- match.arg(method)
  
  ## Compute carbon for the whole profile
  if(is.null(depth)){
    weights <- x$soil$Thickness / sum(x$soil$Thickness)
    ### Total volume is equal to 'bottom' (m^3)
    ## Original BD (from APSIM) is reported in g/cc, which needs to be multiplied by 1e3 to get kg/m^3
    ## Carbon is in percent so it needs to be divided by 100 to get it as a proportion.
    weighted.carbon <- sum(x$soil$Carbon * 1e-2 * weights * x$soil$BD * 1e3) 
    total.carbon <- weighted.carbon * bottom
    depth <- bottom
  }else{
    ## If depth only includes the first layer
    if(depth <= x$soil$Thickness[1] * 1e-3){
      first.layer.carbon <- x$soil$Carbon[1] * 1e-2 * x$soil$BD[1] * 1e3
      total.carbon <- first.layer.carbon * depth
    }else{
      total.carbon <- 0
      cum.thick <- cumsum(x$soil$Thickness) * 1e-3 ## Cumulative thickness in meters
      for(i in 1:nrow(x$soil)){
        ## If the desired depth is greater than the current depth
        ## then add the soil carbon as it is
        if(depth >= cum.thick[i]){ 
          layer.carbon <- x$soil$Carbon[i] * 1e-2 * (x$soil$BD[i] * 1e3) * (x$soil$Thickness[i] * 1e-3)
          total.carbon <- total.carbon + layer.carbon
        }else{
         ## In this case, we need to interpolate 
          crbn <- x$soil$Carbon
          bds <- x$soil$BD
          dat <- data.frame(depth = cum.thick, carbon = crbn, bd = bds)
          tmp.c <- stats::approx(dat$depth, y = dat$carbon, xout = depth, method = method)
          tmp.bd <- stats::approx(dat$depth, y = dat$bd, xout = depth, method = method)  
          layer.carbon <- tmp.c$y * 1e-2 * (tmp.bd$y * 1e3) * (depth - cum.thick[i - 1])
          total.carbon <- total.carbon + layer.carbon
          break
        }
      }
    }
  }

  if(area == "ha"){
    ans <- total.carbon * 1e4 * 1e-3 ## 1e4 converts from m2 to ha. 1e3 converts from kg to Mg
    attr(ans, "units") <- "Mg/ha"
  }else{
    ans <- total.carbon
    attr(ans, "units") <- "kg/m2"
  }
  attr(ans, "depth (m)") <- depth 
  
  if(value == "carbon.stocks"){
    ans <- structure(list(value = ans, depth = depth, 
                          area = area, method = method, 
                          soil.profile = x))
  }

  return(ans)
}


#' @title Plot carbon stocks (function in progress)
#' @description Plotting of carbon stocks based on an object of class \sQuote{soil_profile}
#' @name plot.carbon.stocks
#' @param x object of class \sQuote{soil_profile}
#' @param ... optional argument, not used at the moment
#' @return returns a graph
#' @export
plot.carbon.stocks <- function(x, ...){

  depth <- NULL; midpoint.depth <- NULL; Carbon <- NULL;
  
  sp <- x$soil.profile$soil
  sp$depth <- -cumsum(sp$Thickness) * 1e-1
  ### Calculate midpoint depth
  sp$midpoint.depth <- 0
  sp$midpoint.depth[1] <- sp$depth[1] / 2
  for(i in 2:dim(sp)[1]){
     sp$midpoint.depth[i] <- (sp$depth[i - 1] + sp$depth[i]) / 2
  }

  first.depth <- sp$depth[1]
  first.carbon <- sp$Carbon[1]

  gp1 <- ggplot2::ggplot(data = sp) +
           ggplot2::geom_vline(mapping = ggplot2::aes(xintercept = depth),
                               linetype = 3) +
           ggplot2::geom_point(ggplot2::aes(x = midpoint.depth, y = Carbon)) +
           ggplot2::geom_step(ggplot2::aes(x = depth, y = Carbon)) +
           ggplot2::annotate("segment", x = 0, xend = first.depth, y = first.carbon, yend = first.carbon) +
           ggplot2::coord_flip() +
           ggplot2::labs(y = "Carbon (%)", x = "depth (cm)")

  invisible(gp1)
}

#' Function to calculate available water content. The output units depend on the choice of area.
#' If \sQuote{m} is used, then the output units will be \sQuote{mm}. If the \sQuote{area} is \sQuote{m2},
#' then the output units will be in \sQuote{m3}. If the \sQuote{area} is \sQuote{ha}, then the output units will be \sQuote{kg/ha}.
#' 
#' @title Calculate available water content
#' @description Calculation of available water content based on an object of class \sQuote{soil_profile}
#' @name available_water_content
#' @param x object of class \sQuote{soil_profile}
#' @param depth soil depth (in meters). If missing then the whole soil profile is used.
#' @param area either \sQuote{m} meter, \sQuote{m2} meter squared or \sQuote{ha}.
#' @param method interpolation method. Either \sQuote{linear} or \sQuote{constant}.
#' @param weights optional weights
#' @param ... additional arguments passed to internal functions (none used at the moment).
#' @return returns a value with attribute \sQuote{units} and \sQuote{depth}
#' @export
#' @examples 
#' \dontrun{
#' sp <- apsimx_soil_profile()
#' available_water_content(sp)
#' }

available_water_content <- function(x, 
                                    depth, 
                                    area = c("m", "m2", "ha"), 
                                    method = c("linear", "constant"), 
                                    weights, ...){
  
  if(!inherits(x, "soil_profile")){
    stop("This function is intended to be used with an object of class 'soil_profile'", call. = FALSE)
  }
  
  bottom <- sum(x$soil$Thickness) * 1e-3 ## Thickness is in mm, so after conversion this is in meters
  
  if(!missing(depth)){
    if(depth <= 0) stop("'depth' should be a positive number", call. = FALSE)
    if(depth > bottom) stop("'depth' should be a lower number than the bottom of the soil profile ", call. = FALSE)
    if(depth > 10){
      warning("'depth' should be in meters and the value entered is larger than 10. Is this correct?")
    }
  }
  
  area <- match.arg(area)
  method <- match.arg(method)
  
  ## Compute carbon for the whole profile
  if(missing(depth)){
    layer.depth <- x$soil$Thickness ## Thickness in mm
    layer.awc <- x$soil$DUL - x$soil$LL15
    total.awc <- sum(layer.depth * layer.awc)
    depth <- bottom
  }else{
    ## If depth only includes the first layer
    if(depth <= x$soil$Thickness[1] * 1e-3){
      first.layer.awc <- x$soil$DUL[1] - x$soil$LL15[1]
      total.awc <- first.layer.awc * (depth * 1e3) ## Depth is in meters, this answer is in mm
    }else{
      total.awc <- 0
      cum.thick <- cumsum(x$soil$Thickness) * 1e-3 ## Cumulative thickness in meters
      for(i in 1:nrow(x$soil)){
        ## If the desired depth is greater than the current depth
        ## then add the available water content as it is
        if(depth >= cum.thick[i]){ 
          layer.awc <- (x$soil$DUL[i] - x$soil$LL15[i]) * x$soil$Thickness[i] ## in mm
          total.awc <- total.awc + layer.awc
        }else{
          ## In this case, we need to interpolate 
          awc <- x$soil$DUL - x$soil$LL15
          dat <- data.frame(depth = cum.thick, awc = awc)
          tmp.awc <- stats::approx(dat$depth, y = dat$awc, xout = depth, method = method)
          layer.awc <- tmp.awc$y * (depth - cum.thick[i - 1]) * 1e3 ## This is in mm
          total.awc <- total.awc + layer.awc
          break
        }
      }
    }
  }

  if(area == "m"){
    ans <- total.awc 
    attr(ans, "units") <- "mm"
  }
  
  if(area == "m2"){
    ans <- total.awc * 1e-3 ## 1e-3 converts from mm to m3
    attr(ans, "units") <- "m3"
  }
  
  if(area == "ha"){
    ans <- total.awc * 1e4 ## 1 mm is = 1 kg/m2, 1 ha = 10000m2
    attr(ans, "units") <- "kg/ha"
  }
  
  attr(ans, "depth (m)") <- depth 
  return(ans)
}

#### Function to generate crop specific soil properties
#### These are:
####           LL (Lower Limit)
####           KL (coefficients that controls crop water uptake)  
####           XF (Root exploration factor)

#' @title Generate Crop-Soil coefficients
#' @name crop_soil_coefs
#' @rdname crop_soil_coefs
#' @description Generates a crop-soil coefficients data frama 
#' @param soil.profile optional soil profile object
#' @param crop a valid crop. Default is \sQuote{Wheat}
#' @param method method to be passed to \code{\link{approx}} function
#' @param check whether to check the soil.profile object
#' @return a data frame with columns: \sQuote{Thickness}, \sQuote{crop.LL}, \sQuote{crop.KL}, and \sQuote{crop.XF}
#' @noRd

crop_soil_coefs <- function(soil.profile = NULL, crop = "Wheat", method = "linear", check = TRUE){
  
  if(crop == "Pigeon Pea") crop <- "Pigeon.Pea"
  
  #### Original thickness
  o.thickness <- c(150, 150, 150, 150, 200, 200, 200, 200, 300, 300)  
  
  if(!is.null(soil.profile)){
    if(!inherits(soil.profile, 'soil_profile')){
      stop("object 'soil.profile' should be of class 'soil.profile'", call. = FALSE)
    }
  
    if(check)
      check_apsimx_soil_profile(soil.profile)
    
    thickness <- soil.profile$soil$Thickness
  }else{
    message("When 'soil.profile' is missing LL is a multiplier")
    thickness <- o.thickness
  }
  
  #### There are 28 crops returned by WorldModeler but only
  #### 9 groups when it comes to LL15
  #### 
  #### This is good for Rice, Wheat, Teff, Maize, Soybean, 
  #### Cassava, Sunflower, Canola, Peanut, Oats, Chickpea,
  #### Mungbean, Pigeon Pea, Lupin, Lentil, Triticale,
  group1.ll15.to.crop.ll <- c(1, 1, 1, 0.9656489, 0.9148148, 0.8642857, 0.8191126, 0.7792208, 0.7317073, 0.7079646)
  #### This is for Sugarcane, OilPalm, Plantain, 
  group2.ll15.to.crop.ll <- c(1, 1, 1, 0.9806202, 0.95, 0.9166667, 0.8888889, 0.8602151, 0.8247423, 0.7894737)
  #### This is for Sorghum, Millet
  group3.ll15.to.crop.ll <- c(1, 1, 1, 0.9768340, 0.9356061, 0.8962963, 0.8602151, 0.8247423, 0.7843137, 0.7430341)
  #### This is for Barley
  group4.ll15.to.crop.ll <- c(1, 1, 1, 0.9693487, 0.9250936, 0.88, 0.8391608, 0.8, 0.7570978, 0.7142857)
  #### This is for Cotton
  group5.ll15.to.crop.ll <- c(1, 1, 1, 1, 0.9959677, 0.9641434, 0.9160305, 0.8727273, 0.8247423, 0.7717042)
  #### This is for Fababean
  group6.ll15.to.crop.ll <- c(1, 1, 1, 1, 0.9840637, 0.9272031, 0.8727273, 0.8219178, 0.7667732, 0.7164179)
  #### This is for Lucerne (alfalfa)
  group7.ll15.to.crop.ll <- c(1, 1, 1, 1, 1, 1, 1, 0.9756098, 0.9266409, 0.8727273)
  #### This is for Eucalyptus
  group8.ll15.to.crop.ll <- c(1, 1, 1, 1, 0.9919679, 0.9877551, 0.9836066, 0.9795918, 0.9716599, 0.9638554)
  #### This is for Gliricidia
  group9.ll15.to.crop.ll <- c(1, 1, 1, 0.9960630, 0.9840637, 0.9758065, 0.9677419, 0.9561753, 0.9448819, 0.9338521)
  
  #### Values for KL
  #### Group 1: Rice, Wheat, Teff, Maize, Soybean, Cassava,
  #### Canola, Oats, Cowpea, Lupin, Lentil
  group1.kl <- c(0.060, 0.060, 0.060, 0.054, 0.047, 0.038, 0.028, 0.019, 0.007, 0.000)
  #### Group 2: Sugarcane, OilPalm, Plantain
  group2.kl <- c(0.060, 0.060, 0.060, 0.057, 0.052, 0.047, 0.042, 0.036, 0.029, 0.021)
  #### Group 3: Sunflower
  group3.kl <- c(0.100, 0.093, 0.084, 0.074, 0.063, 0.049, 0.035, 0.022, 0.005, 0.000)
  #### Group 4: Sorghum, Millet
  group4.kl <- c(0.070, 0.070, 0.070, 0.065, 0.058, 0.050, 0.042, 0.034, 0.023, 0.011)
  #### Group 5: Peanut, Pigeon Pea
  group5.kl <- c(0.060, 0.060, 0.060, 0.060, 0.053, 0.042, 0.030, 0.018, 0.004, 0.000)
  #### Group 6: Barley
  group6.kl <- c(0.070, 0.070, 0.070, 0.064, 0.056, 0.047, 0.037, 0.027, 0.015, 0.002)
  #### Group 7: Cotton
  group7.kl <- c(0.100, 0.100, 0.100, 0.100, 0.100, 0.091, 0.078, 0.065, 0.048, 0.028)
  #### Group 8: Chickpea
  group8.kl <- c(0.060, 0.060, 0.060, 0.060, 0.054, 0.043, 0.032, 0.022, 0.008, 0.000)
  #### Group 9: Mungbean
  group9.kl <- c(0.060, 0.060, 0.060, 0.057, 0.046, 0.032, 0.018, 0.004, 0.000, 0.000)
  #### Group 10: Fababean
  group10.kl <- c(0.080, 0.080, 0.080, 0.080, 0.077, 0.065, 0.052, 0.038, 0.021, 0.003)
  #### Group 11: Lucerne
  group11.kl <- c(0.100, 0.100, 0.100, 0.100, 0.100, 0.100, 0.100, 0.094, 0.080, 0.064)
  #### Group 12: Triticale
  group12.kl <- c(0.070, 0.070, 0.065, 0.055, 0.042, 0.028, 0.013, 0.001, 0.000, 0.000)
  #### Group 13: Eucalyptus
  group13.kl <- c(0.080, 0.080, 0.080, 0.079, 0.079, 0.078, 0.077, 0.076, 0.075, 0.073)
  #### Group 14: Gliricidia
  group14.kl <- c(0.080, 0.080, 0.080, 0.079, 0.077, 0.075, 0.073, 0.071, 0.069, 0.066)

  #### Values for XF
  #### Group 1: Rice, Wheat, Teff, Maize, Soybean, Cassava, Sunflower,
  #### Canola, Peanut, Oats, Chickpea, Pigeon Pea, Cowpea, Lupin,
  #### Lentil
  group1.xf <- c(1.000, 1.000, 0.952, 0.920, 0.889, 0.860, 0.844, 0.841, 0.836, 0.000)
  #### Group 2: Sugarcane, OilPalm, Plantain, Sorghum, Millet,
  #### Barley, Cotton, Fababean, Lucerne, Eucalyptus, Gliricidia
  group2.xf <- c(1.000, 1.000, 0.952, 0.920, 0.889, 0.860, 0.844, 0.841, 0.836, 0.831)
  #### Group 3: Mungbean, Triticale
  group3.xf <- c(1.000, 1.000, 0.952, 0.920, 0.889, 0.860, 0.844, 0.841, 0.000, 0.000)
  
  res <- data.frame(Thickness = thickness, LL = rep(NA, length(thickness)), KL = NA, XF = NA)  

  crop.group1 <- c("Rice", "Wheat", "Teff", "Maize", "Soybean",
                   "Cassava", "Canola", "Oats", "Cowpea", "Lupin", "Lentil")
  
  crop.group2 <- c("Sugarcane", "OilPalm", "Plantain")
  crop.group3 <- c("Sunflower")
  crop.group4 <- c("Sorghum", "Millet")
  crop.group5 <- c("Barley")
  crop.group6 <- c("Cotton")
  crop.group7 <- c("Fababean")
  crop.group8 <- c("Lucerne")
  crop.group9 <- c("Triticale")
  crop.group10 <- c("Peanut", "Pigeon.Pea")
  crop.group11 <- c("Chickpea")
  crop.group12 <- c("Mungbean")
  crop.group13 <- c("Eucalyptus")
  crop.group14 <- c("Gliricidia")
  
  all.crops <- c(crop.group1, crop.group2,
                       crop.group3, crop.group4, crop.group5,
                       crop.group6, crop.group7, crop.group8,
                       crop.group9, crop.group10, crop.group11,
                       crop.group12, crop.group13, crop.group14)
  
  if(isFALSE(crop %in% all.crops)){
    cat("'crop' should be one of:", all.crops, "\n")
    stop("'crop' did not match an existing crop")
  }

  if (crop %in% crop.group1) {
    LL <- group1.ll15.to.crop.ll
    KL <- group1.kl
    XF <- group1.xf
  }
  if (crop %in% crop.group2) {
    LL <- group2.ll15.to.crop.ll
    KL <- group2.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group3) {
    ## Sunflower
    LL <- group1.ll15.to.crop.ll
    KL <- group3.kl
    XF <- group1.xf
  }
  if (crop %in% crop.group4) {
    ## Sorghum and Millet
    LL <- group3.ll15.to.crop.ll
    KL <- group4.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group5) {
    ## Barley
    LL <- group4.ll15.to.crop.ll
    KL <- group6.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group6) {
    ## Cotton
    LL <- group5.ll15.to.crop.ll
    KL <- group7.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group7) {
    ### Fababean
    LL <- group6.ll15.to.crop.ll
    KL <- group10.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group8) {
    ### Lucerne
    LL <- group7.ll15.to.crop.ll
    KL <- group11.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group9) {
    ### Triticale
    LL <- group1.ll15.to.crop.ll
    KL <- group12.kl
    XF <- group3.xf
  }
  if (crop %in% crop.group10) {
    ### Peanut and Pigeon.Pea
    LL <- group1.ll15.to.crop.ll
    KL <- group5.kl
    XF <- group1.xf
  }
  if (crop %in% crop.group11) {
    ### Chickpea
    LL <- group1.ll15.to.crop.ll
    KL <- group8.kl
    XF <- group1.xf
  }
  if (crop %in% crop.group12) {
    ### Mungbean
    LL <- group1.ll15.to.crop.ll
    KL <- group9.kl
    XF <- group3.xf
  }
  if (crop %in% crop.group13) {
    ### Eucalyptus
    LL <- group8.ll15.to.crop.ll
    KL <- group13.kl
    XF <- group2.xf
  }
  if (crop %in% crop.group14) {
    ### Gliciridia
    LL <- group9.ll15.to.crop.ll
    KL <- group14.kl
    XF <- group2.xf
  }
  
  if(is.null(soil.profile)){
    res$LL <- LL
    res$KL <- KL
    res$XF <- XF
  }else{
    cum.thickness <- cumsum(soil.profile$soil$Thickness)
    cum.o.thickness <- cumsum(o.thickness)
    crop.LL.multiplier <- stats::approx(cum.o.thickness, LL, xout = cum.thickness, method = method, rule = 2)
    res$LL <- crop.LL.multiplier$y * soil.profile$soil$LL15
    res$KL <- stats::approx(cum.o.thickness, KL, xout = cum.thickness, method = method, rule = 2)$y
    res$XF <- stats::approx(cum.o.thickness, XF, xout = cum.thickness, method = method, rule = 2)$y
  }
    
  names(res) <- c("Thickness", paste(crop, c("LL", "KL", "XF"), sep = "."))    
  
  return(res)
}

## Testing the function for extracting values from an arbitrary .apsimx file
## using extract_value_apsimx
require(apsimx)

run.extract.value.apsimx <- FALSE

if(run.extract.value.apsimx){
  
  ## Testing with the Barley.apsimx example
  ex.dir <- auto_detect_apsimx_examples()
  
  pp1 <- inspect_apsimx("Barley.apsimx", src.dir = ex.dir)
  ## Extract value of Start
  extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                        parm.path = paste0(pp1, ".Start"))
  ## Extract value of End
  extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                        parm.path = paste0(pp1, ".End"))
  ## Weather
  pp2 <- inspect_apsimx("Barley.apsimx", src.dir = ex.dir, 
                        node = "Weather")
  ## Extract weather file
  extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                        parm.path = paste0(pp2, ".FileName"))
  ## Manager
  pp3 <- inspect_apsimx("Barley.apsimx", src.dir = ex.dir, 
                        node = "Manager", parm = list("Sowing", 1))
  ## Extract fertilizer amount
  extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                        parm.path = paste0(pp3, ".Amount"))
  ## Row Spacing
  pp4 <- inspect_apsimx("Barley.apsimx", src.dir = ex.dir, 
                        node = "Manager", parm = list("fixed", 5))
  ## Extract the row spacing
  ex.rs <- extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                                 parm.path = paste0(pp4, ".RowSpacing"))
  
  ## What about vectors?
  pp5 <- inspect_apsimx("Barley.apsimx", src.dir = ex.dir, node = "Soil",
                        soil.child = "Physical", parm = "DUL")
  ## Extract DUL
  ex.dul <- extract_values_apsimx("Barley.apsimx", src.dir = ex.dir,
                                  parm.path = paste0(pp5, ".DUL"))
  
  ## On the replacement example
  extd.dir <- system.file("extdata", package = "apsimx")
  pp6 <- inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                                    node = "Soybean",
                                    node.child = "Leaf", node.subchild = "ExtinctionCoeff",
                                    node.subsubchild = "XYPairs", parm = "Y")
  ## Extract it
  ext.exc <- extract_values_apsimx("MaizeSoybean.apsimx", src.dir = extd.dir,
                                   parm.path = pp6)
  ## Try RUE
  pp7 <- inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                                    node = "Soybean",
                                    node.child = "Leaf", node.subchild = "Photosynthesis", 
                                    node.subsubchild = "RUE", parm = "FixedValue")
  ext.rue <- extract_values_apsimx("MaizeSoybean.apsimx", src.dir = extd.dir, parm.path = pp7)
}

#'
#' @title Edit APSIM \sQuote{Classic} file with a replaced soil profile
#' @name edit_apsim_replace_soil_profile
#' @description Edits an APSIM Classic simulation by replacing the soil profile
#' @param file name of the .apsim file to be edited
#' @param src.dir source directory
#' @param wrt.dir writing directory
#' @param soil.profile a soil profile object with class \sQuote{soil_profile}
#' @param swim list with SWIM specific parameters
#' @param soilwat list with SoilWat specific parameters
#' @param initialwater list with InitialWater specific parameters
#' @param edit.tag default edit tag \sQuote{-edited}
#' @param overwrite default FALSE
#' @param verbose default TRUE. Will print messages indicating what was done.
#' @param root supply the node postion in the case of multiple simulations such as factorials.
#' @return writes an APSIM file to disk with the supplied soil profile
#' @details This function is designed to batch replace the whole soil in an APSIM simulation. 
#' @note There is no such thing as a default soil, carefully build the profile for each simulation.
#' This function replaces values and it can grow an XML node, but it cannot edit a property
#' which is not present in the original file.
#' @export
#' @examples 
#' \donttest{
#' sp <- apsimx_soil_profile(nlayers = 20,
#'                           crops = c("Barley", "Chickpea", "Lucerne", 
#'                           "Maize", "Perennial Grass", "Sorghum", 
#'                           "Wheat", "Millet"))
#' 
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' ## Writing to a temp directory
#' tmp.dir <- tempdir()
#' edit_apsim_replace_soil_profile("Millet.apsim", soil.profile = sp, 
#'                                 edit.tag = "-newsoil",
#'                                 src.dir = extd.dir, 
#'                                 wrt.dir = tmp.dir)
#'                                 
#' inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir,
#'               node = "Soil", soil.child = "Water")
#'  }
#'

edit_apsim_replace_soil_profile <-  function(file = "", src.dir = ".",
                                             wrt.dir = NULL,
                                             soil.profile = NULL,
                                             swim = NULL,
                                             soilwat = NULL,
                                             initialwater = NULL,
                                             edit.tag = "-edited",
                                             overwrite = FALSE,
                                             verbose = TRUE,
                                             root){
  
  .check_apsim_name(file)
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern=".apsim$", ignore.case=TRUE)
  
  if(length(file.names)==0){
    stop("There are no .apsim files in the specified directory to edit.")
  }
  
  file <- match.arg(file, file.names)
  
  if(missing(soil.profile)) stop("soil profile is missing")
  
  ## I might have to replace things one by one
  soil.water.parms <- c("Thickness","SAT","DUL","KS","LL15","BD","AirDry")
  
  ## This is the first editing that occurs and it it creates a temporary file
  ## with a '-tmp' edit tag
  k <- 1
  for(i in soil.water.parms){
    if(k == 1){
      edit_apsim(file = file, src.dir = src.dir, wrt.dir = ".", 
                 node = "Soil", soil.child = "Water", 
                 edit.tag = "-tmp",
                 parm = i, value = soil.profile$soil[[i]],
                 check.length = FALSE,
                 verbose = verbose,
                 root = root)
    }else{
      new.file.path <- paste0(tools::file_path_sans_ext(file),"-tmp.apsim")

      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".", 
                 node = "Soil", soil.child = "Water", 
                 overwrite = TRUE,
                 parm = i, value = soil.profile$soil[[i]],
                 check.length = FALSE,
                 verbose = verbose,
                 root = root)
    }
    k <- k + 1
  }
  
  ## Edit soil water parameters
  if(!is.null(soilwat) || !all(is.na(soil.profile$soilwat))){
    if(!is.null(soilwat)){
      if(!inherits(soilwat, "soilwat_parms")) 
        stop("object should be of class 'soilwat_parms'")
    }

    for(i in seq_along(soilwat)){
      prm.vl <- soilwat[[i]]
      if(is.na(prm.vl)) next
      
      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".",
                 node = "Soil", soil.child = "Water", 
                 overwrite = TRUE, parm = names(soilwat[i]),
                 value = prm.vl, verbose = verbose,
                 root = root)
    }
  }
  
  if(!is.null(swim) || !all(is.na(soil.profile$swim))){
    if(!is.null(swim)){
      if(!inherits(swim, "swim_parms")) 
        stop("object should be of class 'swim_parms'")  
    }
    
    for(i in seq_along(swim)){
      prm.vl <- swim[[i]]
      if(is.na(prm.vl)) next
      
      if(grepl("^Swim", names(swim[i]))){
        prm.nm <- strsplit(names(swim[i]),"_")[[1]][2]
      }else{
        prm.nm <- names(swim[i])
      }
      
      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".",
                 node = "Soil", soil.child = "SWIM", 
                 overwrite = TRUE, parm = prm.nm,
                 value = prm.vl, verbose = verbose,
                 root = root)
    }
  }
  
  ## Edit Initial Water
  if(!is.null(initialwater) || !all(is.na(soil.profile$initialwater))){
    if(!is.null(initialwater)){
      if(!inherits(initialwater, "initialwater_parms")) 
        stop("object should be of class 'initialwater_parms'")      
    }

    for(i in seq_along(initialwater)){
      prm.vl <- initialwater[[i]]
      if(is.na(prm.vl)) next

      prm.nm <- names(initialwater[i])
      
      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".",
                 node = "Soil", soil.child = "InitialWater", 
                 overwrite = TRUE, parm = prm.nm,
                 value = prm.vl, verbose = verbose,
                 root = root)
    }
  }
  
  ## Edit metadata
  if(!is.null(soil.profile$metadata)){
    
    for(i in seq_along(soil.profile$metadata)){
      prm.vl <- soil.profile$metadata[[i]]
      if(is.na(prm.vl)) next
      
      prm.nm <- names(soil.profile$metadata[i])
    
      edit_apsim(file = new.file.path, 
                 src.dir = ".", wrt.dir = ".",
                 node = "Soil", soil.child = "Metadata", 
                 overwrite = TRUE, verbose = verbose,
                 parm = prm.nm,
                 value = prm.vl,
                 root = root)
    }
  }
  
  ## Edit Carbon (OC), FBiom and FInert
  for(i in c("Thickness","Carbon","FBiom","FInert")){
    if(i == "Carbon"){
      ii <- "OC"
    }else{
      ii <- i
    }
    edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".", 
             node = "Soil", soil.child = "OrganicMatter", 
             overwrite = TRUE,
             parm = ii, value = soil.profile$soil[[i]],
             check.length = FALSE,
             verbose = verbose,
             root = root)
  }
  
  ## Edit Analysis PH
  for(i in c("Thickness","PH","EC")){
    if(i == "EC"){
      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".", 
                 node = "Soil", soil.child = "Analysis", 
                 overwrite = TRUE,
                 parm = i, value = rep(0, nrow(soil.profile$soil)),
                 check.length = FALSE,
                 verbose = verbose,
                 root = root)
    }else{
      edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".", 
                 node = "Soil", soil.child = "Analysis", 
                 overwrite = TRUE,
                 parm = i, value = soil.profile$soil[[i]],
                 check.length = FALSE,
                 verbose = verbose,
                 root = root)
    }
  }
  
  ## Edit Sample (NO3, NH4)
  for(i in c("Thickness", "NO3N", "NH4N")){
    if(i == "Thickness") ii <- "Thickness"
    if(i == "NO3N") ii <- "NO3"
    if(i == "NH4N") ii <- "NH4"

    edit_apsim(file = new.file.path, src.dir = ".", wrt.dir = ".", 
             node = "Soil", soil.child = "Sample", 
             overwrite = TRUE,
             parm = ii, value = soil.profile$soil[[i]],
             check.length = FALSE,
             verbose = verbose,
             root = root)
  }
  
  ## Parse apsim file (XML), but the -tmp one
  apsim_xml <- xml2::read_xml(new.file.path)
  
  ## Try removing the temp file
  ## Do not clean up at this time
  unlink(new.file.path)
  
  ## Print names of crops present in the original file
  if(missing(root)){
    crop.names <- xml2::xml_attr(xml2::xml_find_all(apsim_xml, ".//Soil/Water/SoilCrop"), "name")  
  }else{
    apsim_xml0 <- xml2::xml_find_all(apsim_xml, ".//simulation")
    sim.names <- unlist(xml2::xml_attrs(apsim_xml0))
    wsim <- grep(root, sim.names)
    root_node_ptr <- apsim_xml0[[wsim]]
    root_node_path <- xml2::xml_path(root_node_ptr)
    root_node <- xml2::xml_find_first(apsim_xml, root_node_path)
    crop.names <- xml2::xml_attr(xml2::xml_find_all(root_node, ".//Soil/Water/SoilCrop"), "name")  
  }
  
  if(verbose) cat("Crops in the original file", crop.names, "\n")

  if(!isTRUE(all.equal(sort(soil.profile$crops), sort(crop.names)))){
      cat("Name of crops in soil profile", soil.profile$crops,"\n")
      cat("Name of crops in APSIM file", crop.names,"\n")
      stop("Names of crops are not the same")
      ##cat("Names of crops are not the same")
  }
  
  soil.crops.node <- xml2::xml_find_all(apsim_xml, ".//Soil/Water/SoilCrop")
  
  for(crop in soil.profile$crops){
    crop.index <- which(crop.names == crop)
    for(j in c("Thickness", "XF", "LL", "KL")){
      
      if(missing(root)){
        crop.specific.node <- xml2::xml_find_first(xml2::xml_find_all(apsim_xml, ".//Soil/Water/SoilCrop")[[crop.index]], paste0("./", j))  
      }else{
        crop.specific.node <- xml2::xml_find_first(xml2::xml_find_all(root_node, ".//Soil/Water/SoilCrop")[[crop.index]], paste0("./", j))  
      }
      
      len.child.crop.specific.node <- length(xml2::xml_children(crop.specific.node))
      rows.soil.profile <- nrow(soil.profile$soil)
      ## If value is larger I grow the children to match lengths
      if(rows.soil.profile > len.child.crop.specific.node){
        for(i in seq_len(rows.soil.profile - len.child.crop.specific.node)){
          xml2::xml_add_child(crop.specific.node, xml2::xml_children(crop.specific.node)[[len.child.crop.specific.node]])
        }
      }
      if(rows.soil.profile < len.child.crop.specific.node){
        cat("Number of rows of soil profile:", rows.soil.profile,"\n")
        cat("Number of layers in apsim file:", len.child.crop.specific.node,"\n")
        cat("length of 'value' is shorter than length of crop specific node. \n At the moment I think I can grow XML nodes but not shrink them.")
        stop("Sorry. Don't really know how to do this yet.")
      }
      
      if(j != "Thickness"){
        jj <- paste0(crop, ".", j)
      }else{
        jj <- j
      }
      # print(jj)
      # print(as.character(soil.profile$soil[[jj]]))
      xml2::xml_set_text(xml2::xml_children(crop.specific.node), as.character(soil.profile$soil[[jj]]))
    }
  }

  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir,"/",
                      tools::file_path_sans_ext(file),
                      edit.tag,".apsim")
  }else{
    wr.path <- file.path(wrt.dir, file)
  }
  xml2::write_xml(apsim_xml, file = wr.path)
  
  if(verbose){
    cat("Created ", wr.path,"\n")
  }
}


#' 
#' @title Helper function to supply SoilWat parameters
#' @name soilwat_parms
#' @description Creates a list with specific components for the SoilWat model
#' @param SummerCona see APSIM documentation
#' @param SummerU see APSIM documentation
#' @param SummerDate see APSIM documentation
#' @param WinterCona see APSIM documentation
#' @param WinterU see APSIM documentation
#' @param WinterDate see APSIM documentation
#' @param DiffusConst see APSIM documentation
#' @param DiffusSlope see APSIM documentation
#' @param Salb soil albedo (see APSIM documentation)
#' @param CN2Bare see APSIM documentation
#' @param CNRed see APSIM documentation
#' @param CNCov see APSIM documentation
#' @param Slope see APSIM documentation
#' @param DischargeWidth see APSIM documentation
#' @param CatchmentArea see APSIM documentation
#' @param MaxPond see APSIM documentation
#' @param SWCON see APSIM documentation
#' @param Thickness provide the corresponding thickness layer
#' @return a \sQuote{list} with class \sQuote{soilwat_parms}
#' @details current documentation for APSIM 7.10 \url{https://www.apsim.info/documentation/model-documentation/soil-modules-documentation/soilwat/}
#' @export
#' 

soilwat_parms <- function(SummerCona = NA, SummerU = NA, SummerDate = NA,
                          WinterCona = NA, WinterU = NA, WinterDate = NA,
                          DiffusConst = NA, DiffusSlope = NA, Salb = NA,
                          CN2Bare = NA, CNRed = NA, CNCov = NA, Slope = NA,
                          DischargeWidth = NA, CatchmentArea = NA, MaxPond = NA,
                          SWCON = NA, Thickness = NA){
  
  ## Could incorporate error checking in the future
  
  sw.lst <- list(SummerCona = SummerCona, SummerU = SummerU, SummerDate = SummerDate,
              WinterCona = WinterCona, WinterU = WinterU, WinterDate = WinterDate,
              DiffusConst = DiffusConst, DiffusSlope = DiffusSlope, Salb = Salb,
              CN2Bare = CN2Bare, CNRed = CNRed, CNCov = CNCov, Slope = Slope,
              DischargeWidth = DischargeWidth, CatchmentArea = CatchmentArea, MaxPond = MaxPond, 
              SWCON = SWCON, Thickness = Thickness)
  
  if(!is.na(SWCON[1])){
    if(any(SWCON < 0) || any(SWCON > 1)) stop("SWCON should be between 0 and 1", call. = FALSE)  
  }
  
  if(length(SWCON) != length(Thickness))
    stop("length of SWCON should be the same as length of Thickness", call. = FALSE)
  
  if(!is.character(SummerCona) && !is.na(SummerCona)) stop("SummerCona should be a date as a character day-month. Ex: 1-Nov", call. = FALSE)
  if(!is.character(WinterCona) && !is.na(WinterCona)) stop("WinterCona should be a date as a character day-month. Ex: 1-Apr", call. = FALSE)
  
  ans <- structure(sw.lst, class = c("soilwat_parms","list"))
  ans
}

#'
#' @title Helper function to supply SWIM parameters
#' @name swim_parms
#' @description Creates a list with specific components for the SWIM model
#' @param Salb see APSIM documentation
#' @param CN2Bare see APSIM documentation
#' @param CNRed see APSIM documentation
#' @param CNCov see APSIM documentation
#' @param KDul see APSIM documentation
#' @param PSIDul see APSIM documentation
#' @param VC see APSIM documentation
#' @param DTmin see APSIM documentation
#' @param DTmax see APSIM documentation
#' @param MaxWaterIncrement see APSIM documentation
#' @param SpaceWeightingFactor see APSIM documentation
#' @param SoluteSpaceWeightingFactor see APSIM documentation
#' @param Diagnostics see APSIM documentation
#' @param SwimWaterTable_WaterTableDepth see APSIM documentation
#' @param SwimSubsurfaceDrain_DrainDepth see APSIM documentation
#' @param SwimSubsurfaceDrain_DrainSpacing see APSIM documentation
#' @param SwimSubsurfaceDrain_DrainRadius see APSIM documentation
#' @param SwimSubsurfaceDrain_Klat see APSIM documentation
#' @param SwimSubsurfaceDrain_ImpermDepth see APSIM documentation
#' @return a \sQuote{list} with class \sQuote{swim_parms}
#' @details current documentation for APSIM 7.10 \url{https://www.apsim.info/documentation/model-documentation/soil-modules-documentation/swim3/}
#' @export
#' 

swim_parms <- function(Salb = NA, CN2Bare = NA, CNRed = NA,
                       CNCov = NA, KDul = NA, PSIDul = NA,
                       VC = NA, DTmin = NA, DTmax = NA,
                       MaxWaterIncrement = NA, SpaceWeightingFactor = NA, 
                       SoluteSpaceWeightingFactor = NA, Diagnostics = NA,
                       SwimWaterTable_WaterTableDepth = NA, 
                       SwimSubsurfaceDrain_DrainDepth = NA,
                       SwimSubsurfaceDrain_DrainSpacing = NA,
                       SwimSubsurfaceDrain_DrainRadius = NA,
                       SwimSubsurfaceDrain_Klat = NA, 
                       SwimSubsurfaceDrain_ImpermDepth = NA){
  
  ## Could incorporate error checking in the future
  
  swim.lst <- list(Salb = Salb, CN2Bare = CN2Bare, CNRed = CNRed,
                 CNCov = CNCov, KDul = KDul, PSIDul = PSIDul,
                 VC = VC, DTmin = DTmin, DTmax = DTmax,
                 MaxWaterIncrement = MaxWaterIncrement, 
                 SpaceWeightingFactor = SpaceWeightingFactor, 
                 SoluteSpaceWeightingFactor = SoluteSpaceWeightingFactor, 
                 Diagnostics = Diagnostics,
                 SwimWaterTable_WaterTableDepth = SwimWaterTable_WaterTableDepth, 
                 SwimSubsurfaceDrain_DrainDepth = SwimSubsurfaceDrain_DrainDepth,
                 SwimSubsurfaceDrain_DrainSpacing = SwimSubsurfaceDrain_DrainSpacing,
                 SwimSubsurfaceDrain_DrainRadius = SwimSubsurfaceDrain_DrainRadius,
                 SwimSubsurfaceDrain_Klat = SwimSubsurfaceDrain_Klat, 
                 SwimSubsurfaceDrain_ImpermDepth = SwimSubsurfaceDrain_ImpermDepth)
  
  ans <- structure(swim.lst, class = c("swim_parms","list"))
  ans
}

#'
#' @title Helper function to supply additional Soil Organic Matter parameters
#' @name soilorganicmatter_parms
#' @description Creates a list with specific components for the Soil Organic Matter module
#' @param RootCN Root Carbon:Nitrogen ratio (see APSIM documentation)
#' @param RootWt Root weight (see APSIM documentation)
#' @param EnrACoeff (see APSIM documentation)
#' @param EnrBCoeff (see APSIM documentation)
#' @param OCUnits Organic Carbon Units
#' @export

soilorganicmatter_parms <- function(RootCN = NA, RootWt = NA, EnrACoeff = NA, EnrBCoeff = NA, OCUnits = NA){
  
  som.list <- list(RootCN = RootCN, RootWt = RootWt, EnrACoeff = EnrACoeff, EnrBCoeff = EnrBCoeff, OCUnits = OCUnits)
  
  if(!is.na(RootCN) && RootCN <= 0) warning("RootCN should be a value greater than zero") 
  if(!is.na(RootWt) && RootWt <= 0) warning("RootWt should be a value greater than zero")
  if(!is.na(EnrACoeff) && EnrACoeff <= 0) warning("EnrACoeff should be a value greater than zero")
  if(!is.na(EnrBCoeff) && EnrBCoeff <= 0) warning("EnrBCoeff should be a value greater than zero")
  
  ans <- structure(som.list, class = c("soilorganicmatter_parms","list"))
  ans
  
}

#'
#' @title Helper function to supply additional Initial Soil Water parameters
#' @name initialwater_parms
#' @description Creates a list with specific components for the Initial Soil Water module
#' @param Depth depth for soil layers (see APSIM documentation)
#' @param Thickness soil thickness for layers (either enter Depth or Thickness, but not both)
#' @param InitialValues initial values of soil water
#' @param InitialPAWmm Initial Plant Available Water in mm
#' @param PercentFull Percent full (0 - 100)
#' @param RelativeTo usually LL15
#' @param FilledFromTop either true or false
#' @param DepthWetSoil depth of wet soil in mm
#' @export

initialwater_parms <- function(Depth = NA, Thickness = NA, InitialValues = NA, InitialPAWmm = NA, 
                               PercentFull = NA, RelativeTo = NA, FilledFromTop = NA, DepthWetSoil = NA){
  
  initialwater.list <- list(Depth = Depth, Thickness = Thickness, InitialValues = InitialValues, 
                            InitialPAWmm = InitialPAWmm, PercentFull = PercentFull, 
                            FilledFromTop = FilledFromTop, RelativeTo = RelativeTo, 
                            DepthWetSoil = DepthWetSoil)
  
  if(!is.na(InitialPAWmm) && InitialPAWmm <= 0) warning("InitialPAWmm should be a value greater than zero") 
  if(!is.na(PercentFull) && PercentFull <= 0) warning("PercentFull should be a value greater than zero")
  if(!is.na(PercentFull) && PercentFull > 100) warning("PercentFull should be a value of less than 100")
  if(!is.na(DepthWetSoil) && DepthWetSoil <= 0) warning("DepthWetSoil should be a value greater than zero")
  if(!any(is.na(Thickness)) && any(Thickness < 0)) warning("Thickness values should be greater than zero")

  ans <- structure(initialwater.list, class = c("initialwater_parms","list"))
  ans
  
}


#'
#' @title Helper function to supply additional Solute parameters
#' @name solutes_parms
#' @description Creates a list with specific components for the Solutes module
#' @param Depth depth for soil layers (see APSIM documentation)
#' @param Thickness soil thickness for layers (either enter Depth or Thickness, but not both). 
#' Thickness will be recycled if more than one Solute is passed.
#' @param Solutes Solutes supplied (for now this can be one or more of: \sQuote{NO3}, \sQuote{NH4} or \sQuote{Urea})
#' @param InitialValues initial values of solutes
#' @param InitialValuesUnits passed to Solutes
#' @param WaterTableConcentration passed to Solutes
#' @param D0 passed to Solutes
#' @param Exco passed to Solutes
#' @param FIP passed to Solutes
#' @param DepthConstant passed to Solutes
#' @param MaxDepthSoluteAccessible passed to Solutes
#' @param RunoffEffectivenessAtMovingSolute passed to Solutes
#' @param MaxEffectiveRunoff passed to Solutes
#' @export

solutes_parms <- function(Depth = NA, Thickness = NA, Solutes = NA,
                          InitialValues = NA, InitialValuesUnits = NA,
                          WaterTableConcentration = NA, D0 = NA,
                          Exco = NA, FIP = NA, DepthConstant = NA,
                          MaxDepthSoluteAccessible = NA, RunoffEffectivenessAtMovingSolute = NA,
                          MaxEffectiveRunoff = NA){
  
  solutes.list <- list(Depth = Depth, Thickness = Thickness, Solutes = Solutes,
                       InitialValues = InitialValues, 
                       InitialValuesUnits = InitialValuesUnits, WaterTableConcentration = WaterTableConcentration,
                       D0 = D0, Exco = Exco, FIP = FIP, DepthConstant = DepthConstant, MaxDepthSoluteAccessible = MaxDepthSoluteAccessible,
                       RunoffEffectivenessAtMovingSolute = RunoffEffectivenessAtMovingSolute, MaxEffectiveRunoff)
  
  if(!Solutes %in% c("NO3", "NH4", "Urea"))
    warning("'Solutes' should in one or more of: 'NO3', 'NH4', 'Urea'")
  
  if(length(solutes.list$Solutes) > 1){
    if(!is.list(solutes.list$InitialValues))
      warning("If 'Solutes' is a vector of length > 1, 'InitialValues' should be a 'list'")    
    if(length(solutes.list$Solutes) != length(solutes.list$InitialValues))
      warning("Length of 'Solutes' should match the length of the 'InitialValues' list")
  }

  if(!any(is.na(Thickness)) && any(Thickness < 0)) warning("Thickness values should be greater than zero")
  
  ans <- structure(solutes.list, class = c("solutes_parms", "list"))
  ans
  
}

### Testing SR KS pedotransfer function

library(apsimx)
packageVersion("apsimx")
### Need versions 2.8.215 at least
library(ggplot2)

run.test.ks <- get(".run.local.tests", envir = apsimx.options)

### Motivating example from Ke Liu
### Australia is between 112 and 154 longitude
###          and between -10 and -44 latitude
if(run.test.ks){

  clay <- seq(1, 100, by = 5)
  sand <- seq(1, 100, by = 5)
  om <- seq(0, 15, by = 2)
  
  grd0 <- expand.grid(clay = clay, sand = sand, om = om)
  grd0$total <- grd0$clay + grd0$sand
  grd <- subset(grd0, total <= 100)
  dim(grd)
  
  esub <- NULL
  for(i in 1:nrow(grd)){
    tst <- try(apsimx:::sr_ks(grd$clay[i], grd$sand[i], grd$om[i]), silent = TRUE)
    if(inherits(tst, 'try-error')){
      esub <- c(esub, i)
    }
  }
  
  grd <- grd[-esub, ]
  
  grd$ks <- apsimx:::sr_ks(grd$clay, grd$sand, grd$om)
  summary(grd$ks)

  ### Extreme outlier, when clay = 1, sand = 81 and om = 6
  na.omit(grd[grd$ks > 13000, ])
  
  summary(grd[is.na(grd$ks), ])
  ## The function seems to work ok as long as clay is greater than 1
  summary(grd[grd$clay > 1,])
  
  grd2 <- subset(grd, clay > 1)
  
  ggplot(data = grd2, aes(x = clay, y = ks, color = sand)) + 
    facet_wrap(~om) + 
    geom_point()

}

if(FALSE){

  soil.grid <- matrix(c(145, 144, -35, -34), ncol = 2)
  sls.wm <- get_worldmodeler_soil_profile(lonlat = soil.grid)
  s.wm <- get_worldmodeler_soil_profile(lonlat = c(145.375, -35.625))
  plot(s.wm[[1]], property = "Carbon")
  plot(s.wm[[1]], property = "water")
  
  isl <- get_isric_soil_profile(lonlat = c(145.375, -35.625))
  isl$soil$KS
  
  plot(isl, property = "Carbon")
  plot(isl, property = "water")
  plot(isl, property = "KS")
  
  isl$soil[, c("KS", "ParticleSizeClay", "ParticleSizeSand", "Carbon")]
  isl$soil$KS
  isl.clay <- isl$soil$ParticleSizeClay
  isl.sand <- isl$soil$ParticleSizeSand
  isl.om <- isl$soil$Carbon * 2
  
  longs <- seq(114, 153, by = 5)
  isric.tmps <- vector('list', length = length(longs))
  for(i in seq_along(longs)){
    cat("Getting soil for longitude:", longs[i], "\n")
    isric.tmps[[i]] <- get_isric_soil_profile(lonlat = c(longs[i], -24.625))  
  }
  
  plot(isric.tmps[[1]], property = "KS")
  plot(isric.tmps[[1]], property = "KS")
  
  sapply(isric.tmps, FUN = \(x) range(x$soil$KS))
  
  apsimx:::sr_ks(isl.clay, isl.sand, isl.om)
  
  clay <- seq(1, 100, by = 5)
  sand <- seq(1, 100, by = 5)
  om <- seq(0, 15, by = 2)
  
  grd <- expand.grid(clay = clay, sand = sand, om = om)
  dim(grd)
  grd$ks <- apsimx:::sr_ks(grd$clay, grd$sand, grd$om)
  summary(ans)
  
  hist(grd$ks)
  
  na.omit(grd[grd$ks < 1, ])
  
  ggplot(data = grd, aes(x = clay, y = ks)) + 
    facet_wrap(~ om) + 
    geom_point()
  
  ggplot(data = grd, aes(x = sand, y = ks)) + 
    facet_wrap(~ om) + 
    geom_point()
  
  slg <- get_slga_soil_profile(lonlat = c(145.375, -35.625), fix = TRUE)
  slg$soil$KS
  
  plot(slg, property = "Carbon")
  plot(slg, property = "water")
  plot(slg, property = "KS")
}

## Testing the calculation of thermal time in APSIM
require(apsimx)
require(ggplot2)
apsimx_options(warn.versions = FALSE)

run.test.apsim.met.tt <- FALSE

if(run.test.apsim.met.tt){
  
  tmp.dir <- tempdir()
  
  ex.dir <- auto_detect_apsimx_examples()
  
  file.copy(file.path(ex.dir, "Maize.apsimx"), tmp.dir)
  
  inspect_apsimx("Maize.apsimx", tmp.dir, node = "Report")
  
  edit_apsimx("Maize.apsimx", src.dir = tmp.dir, 
              node = "Report", parm = "EventNames",
              value = "[Clock].EndOfDay", overwrite = TRUE)

  edit_apsimx("Maize.apsimx", src.dir = tmp.dir, 
              node = "Report", parm = "VariableNames",
              value = "[Maize].Phenology.AccumulatedTT", overwrite = TRUE)
  
  inspect_apsimx("Maize.apsimx", src.dir = tmp.dir, node = "Manager",
                 parm = list("Sow using", NA))
  
  inspect_apsimx("Maize.apsimx", src.dir = tmp.dir, node = "Clock")
  
  sim0 <- apsimx("Maize.apsimx", src.dir = tmp.dir)
 
  ggplot(data = sim0, aes(x = Date, y = Maize.Phenology.AccumulatedTT)) + 
    geom_line()
  
  ## Bring metfile here
  inspect_apsimx("Maize.apsimx", src.dir = tmp.dir, node = "Weather")
  
  file.copy(file.path(ex.dir, "WeatherFiles/AU_Dalby.met"), tmp.dir)
  
  dalby <- read_apsim_met("AU_Dalby.met", src.dir = tmp.dir)
  
  ## What is the emergence date for 1990
  sim0[sim0$Maize.Phenology.CurrentStageName == "Emergence",]
  sim0[sim0$Maize.Phenology.CurrentStageName == "HarvestRipe",]
  
  dalby.tt <- tt_apsim_met(dalby, dates = c("07-01", "28-05"), 
                           method = "Classic",
                           x_temp = c(10, 20, 30, 40), y_tt = c(0, 10, 20, 0))
  
  sim0$year <- as.numeric(format(sim0$Date, "%Y"))
  sim1991 <- subset(sim0, year == 1991)
 
  dalby.tt.1991 <- subset(dalby.tt, year == 1991)
  
  ggplot() + 
    geom_line(data = sim1991, aes(x = Date, y = Maize.Phenology.AccumulatedTT, color = "APSIM")) + 
    geom_line(data = dalby.tt.1991, aes(x = Date, y = APSIM_TT, color = "apsimx"))
  
  
  file.remove(file.path(tmp.dir, "Maize.apsimx"))
  
  
}

## Test for napad and impute
if(FALSE){
  
  dts <- c("1990-01-01", "2020-12-31")
  iem <- get_iem_apsim_met(lonlat = c(-93.77, 42), dates = dts)
  pwr <- get_power_apsim_met(lonlat = c(-93.77, 42), dates = dts)
  
  nrow(iem); nrow(pwr)

  dmt <- get_daymet2_apsim_met(lonlat = c(-93.77, 42), years = c(1990, 2020))    
  nrow(iem); nrow(pwr); nrow(dmt)
  
  check_apsim_met(iem)
  check_apsim_met(pwr)
  check_apsim_met(dmt)

  ## Fix IEM
  iem[iem$radn < 0, "radn"] <- NA
  ## The problem is that met radn has a lot of NA's
  iem2 <- impute_apsim_met(iem)
  check_apsim_met(pwr)
  
  dmt2 <- napad_apsim_met(dmt)
  dmt3 <- impute_apsim_met(dmt2)
  check_apsim_met(dmt3)
}

#' 
#' @title Inspect an .apsim (XML) file
#' @name inspect_apsim
#' @description inspect an XML apsim file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
#' @param file file ending in .apsim (Classic) to be inspected (XML)
#' @param src.dir directory containing the .apsim file to be inspected; defaults to the current working directory
#' @param node either \sQuote{Weather}, \sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, 
#' \sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other}
#' @param soil.child specific soil component to be inspected
#' @param parm parameter to inspect when node = \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other}
#' @param digits number of decimals to print (default 3)
#' @param print.path whether to print the parameter path (default = FALSE)
#' @param root root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.
#' @details This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsim file.
#'          For \sQuote{Crop}, \sQuote{Manager} and \sQuote{Other}, \sQuote{parm} should be indicated with a first element to look for and a second with the relative position in case there are
#'          multiple results.
#' @note When multiple folders are present as it is the case when there are factorials. Inspect will find
#'       the instance in the first folder unless 'root' is supplied. By providing the name of the folder to root
#'       (or a regular expression), the appropriate node can be selected. In this case the printed path will
#'       be absolute instead of relative.
#' @return It returns the parameter path (when print.path equals TRUE) and table with inspected parameters and values
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' ## Testing using 'Millet'
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Clock")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Weather") 
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Metadata")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "OrganicMatter")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Analysis")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "InitialWater")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Sample")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "SurfaceOrganicMatter")
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",NA)) 
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",7))
#' 
#' ## when soil.child = "Water" there are potentially many crops to chose from 
#' ## This selects LL, KL and XF for Barley
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
#'               soil.child = "Water", parm = "Barley")
#' ## This selects LL for all the crops
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
#'               soil.child = "Water", parm = "LL")
#' ## To print the parm.path the selection needs to be unique
#' ## but still there will be multiple soil layers
#' ## 'parm' can be a list or a character vector of length equal to two
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
#'               soil.child = "Water", parm = list("Barley", "LL"),
#'               print.path = TRUE)
#' 
#' ## Inspect outputfile
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Outputfile",
#'               parm = "filename")
#'  
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Outputfile",
#'               parm = "variables")
#' 
#' ## Testing with maize-soybean-rotation.apsim
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Clock")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Weather")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil",
#'               soil.child = "Metadata")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
#'                soil.child = "OrganicMatter")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
#'                soil.child = "Analysis")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
#'                soil.child = "InitialWater")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
#'                soil.child = "Sample")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
#'                node = "SurfaceOrganicMatter")
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Crop")
#' ## This has many options and a complex structure
#' ## It is possible to select unique managements, but not non-unique ones
#' ## The first element in parm can be a regular expression
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
#'                node = "Manager", parm = list("rotat",NA))
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
#'                node = "Manager", 
#'                parm = list("sow on a fixed date - maize",NA))
#' ## Select an individual row by position
#' inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
#'               node = "Manager", 
#'               parm = list("sow on a fixed date - maize",7))
#'               
#' ## Illustrating the 'print.path' feature.
#' inspect_apsim("Millet.apsim", src.dir = extd.dir, 
#'                node = "Soil", soil.child = "Water", 
#'                parm = "DUL", print.path = TRUE)
#' ## But the path can also be returned as a string
#' ## Which is useful for later editing
#' pp <-  inspect_apsim("Millet.apsim", src.dir = extd.dir, 
#'                node = "Soil", soil.child = "Water", 
#'                parm = "DUL", print.path = TRUE)
#'                
#' ## Inspecting a factorial
#' ## (or simply a simulation with multiple folders)
#' ## No cover
#' inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
#'                root = "IA-CC_Canisteo_No-Cover")
#'                
#' ## Cover
#' inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
#'                root = "IA-CC_Canisteo_Cover")
#' }
#' 

inspect_apsim <- function(file = "", src.dir = ".", 
                          node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "Crop", "Manager", "Outputfile", "Other"),
                          soil.child = c("Metadata", "Water", "OrganicMatter", "Nitrogen", "Analysis", "InitialWater", "Sample", "SWIM"),
                          parm = NULL,
                          digits = 3,
                          print.path = FALSE,
                          root){
  
  file.names <- dir(path = src.dir, pattern=".apsim$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsim files in the specified directory to inspect.")
  }
  
  node <- match.arg(node)
  soil.child <- match.arg(soil.child)
  
  if(print.path && missing(parm) && node != "Weather") 
    stop("parm should be specified when print.path is TRUE")
  
  ## This matches the specified file from a list of files
  ## Notice that the .apsimx extension will be added here
  file <- match.arg(file, file.names)
  
  ## Read the file
  apsim_xml <- xml2::read_xml(file.path(src.dir, file))
  
  ## This is my attempt at picking the right node in a factorial
  if(!missing(root)){
    apsim_xml0 <- xml2::xml_find_all(apsim_xml, ".//simulation")
    sim.names <- unlist(xml2::xml_attrs(apsim_xml0))
    wsim <- grep(root, sim.names)
    if(length(wsim) == 0)
      stop(c("'root' simulation not found. It should be one of: \n", 
             paste0(sim.names, "\n")),
           call. = FALSE)
    apsim_xml <- apsim_xml0[[wsim]]
    parm.path.root <- xml2::xml_path(apsim_xml)
  }
  
  ## parm.path.0 is the 'root' path
  parm.path <- NULL
  parm.path.0 <- NULL
  parm.path.1 <- NULL

  if(node == "Clock"){
    parm.path.0 <- ".//clock"
    parms <- c("start_date", "end_date")
    for(i in parms){
      clock.node <- xml2::xml_find_first(apsim_xml, paste0(parm.path.0, "/", i))
      cat(i, ":", xml2::xml_text(clock.node), "\n")
    }
    if(!missing(parm)){
      parm <- grep(parm, parms, value = TRUE)
      parm.path.1 <- paste0(parm.path.0, "/", parm)
    }
  }
  
  if(node == "Weather"){
    parm.path.0 <- ".//metfile/filename" 
    weather.filename.node <- xml2::xml_find_first(apsim_xml, parm.path.0)
    cat("Met file:", (xml2::xml_text(weather.filename.node)), "\n")
    if(!missing(parm)){
      if(parm != "filename") warning("parm should be 'filename' when 'node = Weather'")
    }
    parm.path.1 <- parm.path.0
  }
  
  ## Extracting soil Depths
  ## May be I should move this function to 'apsim_internals.R'
  ## t2d is "thickness" to "depth"
  t2d <- function(x){
    x2 <- c(0, x)/10 ## Divide by 10 to go from mm to cm
    ans <- character(length(x))
    csx2 <- cumsum(x2)
    for(i in 2:length(x2)){
        ans[i-1] <- paste0(csx2[i-1], "-", csx2[i]) 
    }
    ans
  }
  
  if(node == "Soil"){
    ## Soil depths for naming columns
    ## It seems that Depth is not explicitly exposed
    ## But Thickness is
    thickness.path <- ".//Thickness"
    soil.thickness <- as.numeric(xml2::xml_text(xml2::xml_children(xml2::xml_find_first(apsim_xml, thickness.path))))
    soil.depths <- t2d(soil.thickness)
    ## Determine the number of soil layers
    number.soil.layers <- length(soil.thickness)
    
    ## Print soil type, latitude and longitude
    cat("Soil Type: ", xml2::xml_text(xml2::xml_find_first(apsim_xml, ".//Soil/SoilType")),"\n")
    cat("Latitude: ", xml2::xml_text(xml2::xml_find_first(apsim_xml, ".//Soil/Latitude")),"\n")
    cat("Longitude: ", xml2::xml_text(xml2::xml_find_first(apsim_xml, ".//Soil/Longitude")),"\n")
    
    if(soil.child == "Metadata"){
      metadata.parms <- c("SoilType", "LocalName", "Site", "NearestTown", "Region", "State",
                          "Country", "ApsoilNumber", "Latitude", "Longitude", "YearOfSampling",
                          "DataSource", "Comments", "NaturalVegetation")
      met.dat <- data.frame(parm = metadata.parms, value = NA)
      j <- 1
      for(i in metadata.parms){
        parm.path.0 <- ".//Soil"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.metadata.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        if(length(soil.metadata.node) > 0){
          met.dat[j,1] <- i
          met.dat[j,2] <- strtrim(xml2::xml_text(soil.metadata.node), options()$width - 35)
          j <- j + 1
        }
      }
      
      if(missing(parm)){
        print(knitr::kable(stats::na.omit(met.dat)))
      }else{
        parm.path.1 <- paste0(parm.path.0, "/", parm)
        met.dat.nmd <- stats::na.omit(met.dat)
        met.dat.nmd.p <- met.dat.nmd[met.dat.nmd$parm == parm, ]
        print(knitr::kable(met.dat.nmd.p))
      } 
    }
    
    if(soil.child == "Water"){
      ## Crop specific
      crop.parms <- c("LL", "KL", "XF")
      
      ## How many crops are present?
      number.of.crops <- length(xml2::xml_attrs(xml2::xml_find_all(apsim_xml, ".//Soil/Water/SoilCrop")))
      crop.names <- as.character(unlist(xml2::xml_attrs(xml2::xml_find_all(apsim_xml, ".//Soil/Water/SoilCrop"))))
      crop.names.parms <- c(sapply(crop.names, function(x) paste(x, crop.parms)))
      
      val.mat <- matrix(NA, 
                        ncol = c(length(crop.parms) * number.of.crops)+1,
                        nrow = number.soil.layers)
      
      crop.d <- data.frame(val.mat)
      crop.d[,1] <- soil.depths
      names(crop.d) <- c("Depth", crop.names.parms)
      j <- 2
      for(i in crop.names){
        for(k in crop.parms){
          parm.path.0 <- ".//Soil/Water/SoilCrop"
          parm.path.1 <- paste0(parm.path.0, "[@name='", i, "']", "/", k)
          soil.water.crop.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
          crop.d[,j] <- xml2::xml_double(xml2::xml_children(soil.water.crop.node))
          j <- j + 1          
        }
      }
      
      soil.parms <- c("Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS")
      
      val.mat <- matrix(NA, 
                        ncol = length(soil.parms),
                        nrow = number.soil.layers)
      
      soil.d <- data.frame(val.mat)
      names(soil.d) <- soil.parms
      
      j <- 1
      for(i in soil.parms){
        parm.path.0 <- ".//Soil/Water"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.water.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        soil.d[,j] <- xml2::xml_double(xml2::xml_children(soil.water.node))
        j <- j + 1
      }

      soil.water.parms <- c("SummerCona", "SummerU", "SummerDate",
                            "WinterCona", "WinterU", "WinterDate",
                            "DiffusConst","DiffusSlope", "Salb",
                            "CN2Bare","CNRed","CNCov")
      
      soil.water.xparms <- c("Slope", "Discharge", "CatchmentArea", "MaximumPond")
      
      soil.water.parms.soilwat <- c("Thickness", "SWCON", "MWCON", "KLAT")
      
      soilwat.present <- FALSE
      ## Soil Water, but only when it is present because of SWIM        
      if(length(xml2::xml_find_all(apsim_xml, ".//Soil/SoilWater")) > 0){
        
        soilwat.present <- TRUE
        soil.water.d <- data.frame(soil.water = soil.water.parms,
                                   value = NA)
        j <- 1
        for(i in soil.water.parms){
          parm.path.0 <- ".//Soil/SoilWater"
          parm.path.1 <- paste0(parm.path.0, "/", i)
          soil.water.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
          soil.water.d[j,"value"] <- xml2::xml_text(soil.water.node)
          j <- j + 1
        }

        soil.thickness.sw <- as.numeric(xml2::xml_text(xml2::xml_children(xml2::xml_find_first(apsim_xml, ".//Soil/SoilWater/Thickness"))))
        
        soil.water.soilwat.d <- data.frame(Thickness = rep(NA, length(soil.thickness.sw)), 
                                           SWCON = NA, MWCON = NA, KLAT = NA)
        
        for(i in soil.water.parms.soilwat){
          parm.path.0 <- ".//Soil/SoilWater"
          parm.path.1 <- paste0(parm.path.0, "/", i)
          soil.water.node.soilwat <- xml2::xml_find_first(apsim_xml, parm.path.1)
          if(length(soil.water.node.soilwat) == 0) next
          soil.water.soilwat.d[[i]] <- xml2::xml_double(xml2::xml_children(soil.water.node.soilwat))
        }
      }

      if(missing(parm)){
        print(knitr::kable(cbind(crop.d,soil.d), digits = digits))  
        if(soilwat.present) print(knitr::kable(soil.water.d, digits = digits))
        if(soilwat.present) print(knitr::kable(soil.water.soilwat.d, digits = digits))
      }else{
        found.parm <- FALSE
        ## parm is either a list or a string and it is in crop.parms or crop.names
        if(length(parm) == 1 && (parm %in% crop.parms || parm %in% crop.names)){
          sparm <- grep(parm, crop.names.parms, value = TRUE, ignore.case = TRUE)
          print(knitr::kable(subset(crop.d, select = sparm), digits = digits))
          if(print.path) stop("parameter should be unique if print.path = TRUE")
          found.parm <- TRUE
        }
        if(length(parm) == 2){
          if(is.list(parm)){
            parm.path.0 <- ".//Soil/Water/SoilCrop"
            parm.path.1 <- paste0(parm.path.0, "[@name='", parm[[1]], "']", "/", parm[[2]])
            sparm <- paste(parm[[1]], parm[[2]])
            if(inherits(try(subset(crop.d, select = sparm), silent = TRUE), "try-error")) stop("parm not found")
            print(knitr::kable(subset(crop.d, select = sparm), digits = digits))
            found.parm <- TRUE
          }
          if(is.character(parm)){
            parm.path.0 <- ".//Soil/Water/SoilCrop"
            parm.path.1 <- paste0(parm.path.0, "[@name='", parm[1], "']", "/", parm[2])
            sparm <- paste(parm[1], parm[2])
            if(inherits(try(subset(crop.d, select = sparm), silent = TRUE), "try-error")) stop("parm not found")
            print(knitr::kable(subset(crop.d, select = sparm), digits = digits))
            found.parm <- TRUE
          }
        }
        ## parm is in 'soil.parms'
        if(length(parm) == 1 && parm %in% soil.parms){
          parm.path.0 <- ".//Soil/Water"
          parm.path.1 <- paste0(parm.path.0, "/", parm)
          print(knitr::kable(subset(soil.d, select = parm), digits = digits))  
          found.parm <- TRUE
        }
        ## parm is in 'soil.water.parms'
        if(length(parm) == 1 && parm %in% soil.water.parms){
          parm.path.0 <- ".//Soil/SoilWater"
          parm.path.1 <- paste0(parm.path.0, "/", parm)
          print(knitr::kable(soil.water.d[soil.water.d$soil.water == parm,], digits = digits)) 
          found.parm <- TRUE
        }
        ## parm is in 'soil.water.parms.soilwat'
        if(length(parm) == 1 && parm %in% soil.water.parms.soilwat){
          parm.path.0 <- ".//Soil/SoilWater"
          parm.path.1 <- paste0(parm.path.0, "/", parm)
          print(knitr::kable(subset(soil.water.soilwat.d, select = parm), digits = digits)) 
          found.parm <- TRUE
        }
        if(!found.parm) stop("parm not found")
      }
    }
    
    if(soil.child == "SWIM"){
      parm.path.0 <- ".//Soil/Swim"
      ## This first set are single values
      swim.parms1 <- c("Salb","CN2Bare","CNRed","CNCov","KDul",
                      "PSIDul","VC","DTmin","DTmax","MaxWaterIncrement",
                      "SpaceWeightingFactor","SoluteSpaceWeightingFactor",
                      "Diagnostics")
      
      swim.parms1.d <- data.frame(parm = swim.parms1, value = NA)
      
      j <- 1
      for(i in swim.parms1){
        parm.path.1 <- paste0(parm.path.0, "/", i)
        swim.parm.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        swim.parms1.d[j,"value"] <- xml2::xml_text(swim.parm.node)
        j <- j + 1
      }
      
      if(missing(parm)) print(knitr::kable(swim.parms1.d))
      
      ## Water table and Subsurface drain
      swim.parms2 <- c("SwimWaterTable/WaterTableDepth",
                       "SwimSubsurfaceDrain/DrainDepth",
                       "SwimSubsurfaceDrain/DrainSpacing",
                       "SwimSubsurfaceDrain/DrainRadius",
                       "SwimSubsurfaceDrain/Klat",
                       "SwimSubsurfaceDrain/ImpermDepth")
      
      swim.parms2.d <- data.frame(parm = gsub("/"," : ",swim.parms2), value = NA)
      
      j <- 1
      for(i in swim.parms2){
        parm.path.1.1 <- paste0(parm.path.0, "/", i)
        swim.parm.node <- xml2::xml_find_first(apsim_xml, parm.path.1.1)
        swim.parms2.d[j,"value"] <- xml2::xml_text(swim.parm.node)
        j <- j + 1
      }
      if(missing(parm)) print(knitr::kable(swim.parms2.d))
      
      if(!missing(parm) && parm %in% swim.parms2){
        if(grepl("WaterTableDepth", parm, fixed=TRUE)){
          parm.path.0 <- paste0(parm.path.0, "/", "SwimWaterTable")
        }else{
          parm.path.0 <- paste0(parm.path.0, "/", "SwimSubsurfaceDrain")
        }
      }
      
      if(!missing(parm)){
        if(parm %in% swim.parms1){
          print(knitr::kable(swim.parms1.d[swim.parms1.d$parm == parm,]))
        }
        if(parm %in% swim.parms2){
          mparm <- gsub("/", " : ", parm)
          print(knitr::kable(swim.parms2.d[swim.parms2.d$parm == mparm,]))
        }
        if(!(parm %in% c(swim.parms1, swim.parms2))) warning("parameter not found in the SWIM module")
      }
    }
    
    if(soil.child == "Nitrogen"){
      stop("not implemented yet")
      nitrogen.parms <- c("fom_type", "fract_carb", "fract_cell", "fract_lign")
      
      val.mat <- matrix(NA, ncol = length(nitrogen.parms),
                        nrow = 6, 
                        dimnames = list(NULL,nitrogen.parms))
      
      nitro.d <- data.frame(val.mat)
      k <- 1
      for(i in nitrogen.parms){
        parm.path <- paste0(".//Soil/SoilNitrogen", "/", i)
        soil.nitrogen.node <- xml2::xml_find_first(apsim_xml, parm.path)
        nitro.d[,k] <- xml2::xml_text(xml2::xml_children(soil.nitrogen.node))
        k <- k + 1
      }
      print(knitr::kable(nitro.d, digits = digits))
    }
    
    if(soil.child == "OrganicMatter"){
      ## State what are organic matter possible parameters
      ## Will keep these ones hard coded
      som.parms1 <- c("RootCN", "RootWt", "SoilCN", "EnrACoeff", "EnrBCoeff")
      
      som.d1 <- data.frame(parm = som.parms1, value = NA)
      
      for(i in som.parms1){
        parm.path.0 <- ".//Soil/SoilOrganicMatter"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.som1.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        som.d1[som.d1$parm == i,2] <- xml2::xml_text(soil.som1.node)
      }
      
      if(missing(parm)) print(knitr::kable(som.d1, digits = digits))
      
      som.parms2 <- c("Thickness", "OC", "FBiom", "FInert")
      
      val.mat <- matrix(NA, 
                        ncol = (length(som.parms2)+1),
                        nrow = number.soil.layers)
      
      val.mat[,1] <- soil.depths
      som.d2 <- as.data.frame(val.mat)
      names(som.d2) <- c("Depth", som.parms2)
      
      j <- 2
      for(i in som.parms2){
        parm.path.0 <- ".//Soil/SoilOrganicMatter"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.som2.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        som.d2[,j] <- xml2::xml_text(xml2::xml_children(soil.som2.node))
        j <- j + 1
      }
      if(missing(parm)) print(knitr::kable(som.d2, digits = digits))
      
      if(!missing(parm) && parm %in% c(som.parms1, som.parms2)){
        parm.path.1 <- paste0(".//Soil/SoilOrganicMatter", "/", parm)
        if(parm %in% som.parms1){
          print(knitr::kable(som.d1[som.d1$parm == parm,], digits = digits))
        }
        if(parm %in% som.parms2){
          print(knitr::kable(subset(som.d2, select = parm), digits = digits))  
        }
      }
    }
    
    if(soil.child == "Analysis"){
      ## I will keep this one hard coded because it is simple
      ## There are many potential elements but can't imagine wanting to edit them
      analysis.parms <- c("Thickness", "PH", "EC")
      
      val.mat <- matrix(NA, 
                        ncol = (length(analysis.parms)+1),
                        nrow = number.soil.layers)
      
      analysis.d <- as.data.frame(val.mat)
      analysis.d[,1] <- soil.depths
      names(analysis.d) <- c("Depth", analysis.parms)
      
      j <- 2
      for(i in analysis.parms){
        parm.path.0 <- ".//Soil/Analysis"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.analysis.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        analysis.d[,j] <- xml2::xml_text(xml2::xml_children(soil.analysis.node))
        j <- j + 1
      }
      if(missing(parm)) print(knitr::kable(analysis.d, digits = digits))
      
      if(!missing(parm) && parm %in% analysis.parms){
        parm.path.1 <- paste0(".//Soil/Analysis", "/", parm)
        print(knitr::kable(subset(analysis.d, select = parm), digits = digits))
      }
    }
    
    if(soil.child == "InitialWater"){
      initialwater.parms <- c("PercentMethod", "FractionFull", "DepthWetSoil")
      
      initial.water.d <- data.frame(parm = initialwater.parms, value = NA)
      
      for(i in initialwater.parms){
        parm.path.0 <- ".//Soil/InitialWater"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.InitialWater.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        initial.water.d[initial.water.d$parm == i,2] <- xml2::xml_text(soil.InitialWater.node)
      }
      if(missing(parm)) print(knitr::kable(initial.water.d, digits = digits))
      
      if(!missing(parm) && parm %in% initialwater.parms){
        parm.path.1 <- paste0(".//Soil/InitialWater", "/", parm)
        print(knitr::kable(initial.water.d[initial.water.d$parm == parm, ], digits = digits))
      }
    }
    
    if(soil.child == "Sample"){
      sample.parms <- c("Thickness", "NO3", "NH4", "SW", "OC", "EC", "CL", "ESP", "PH")
      
      val.mat <- matrix(NA, 
                        ncol = (length(sample.parms)+1),
                        nrow = number.soil.layers)
      
      sample.d <- data.frame(val.mat)
      sample.d[,1] <- soil.depths
      names(sample.d) <- c("Depth", sample.parms)
      
      j <- 2
      for(i in sample.parms){
        parm.path.0 <- ".//Soil/Sample"
        parm.path.1 <- paste0(parm.path.0, "/", i)
        soil.sample.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
        sample.d[,j] <- xml2::xml_text(xml2::xml_children(soil.sample.node))
        j <- j + 1
      }
      if(missing(parm)) print(knitr::kable(sample.d, digits = digits))
      
      if(!missing(parm) && parm %in% sample.parms){
        parm.path.1 <- paste0(".//Soil/Sample", "/", parm)
        print(knitr::kable(subset(sample.d, select = parm), digits = digits))
      }
    }
  }
  
  if(node == "SurfaceOrganicMatter"){
    pools.path <- ".//surfaceom"
    pools.parms <- xml2::xml_name(xml2::xml_children(xml2::xml_find_first(apsim_xml, pools.path)))
      
    pools.d <- data.frame(parm = pools.parms, value = NA)
      
    for(i in pools.parms){
      parm.path.1 <- paste0(pools.path, "/", i)
      soil.pools.node <- xml2::xml_find_first(apsim_xml, parm.path.1)
      pools.d[pools.d$parm == i, 2] <- xml2::xml_text(soil.pools.node)
    }
    
    if(missing(parm)) print(knitr::kable(pools.d, digits = digits))
    
    if(!missing(parm) && parm %in% pools.parms){
      parm.path.0 <- ".//surfaceom"
      parm.path.1 <- paste0(".//surfaceom", "/", parm)
      print(knitr::kable(pools.d[pools.d$parm == parm,], digits = digits))
    }
  }

  if(node == "Crop" | node == "Manager"){
    ## It seems that for nodes Crop and Manager, the names can be arbitrary which
    ## makes this hard parsing complicated
    cat("Crop Type: ",xml2::xml_text(xml2::xml_find_first(apsim_xml, ".//manager/ui/crop")),"\n")
    ## Make sure sowing rule is present
    xfa.manager <- as.vector(unlist(xml2::xml_attrs(xml2::xml_find_all(apsim_xml, ".//manager"))))
    cat("Management Scripts:", xfa.manager, "\n", sep = "\n")
    
    if(missing(parm)) parm.path.0 <- ".//manager"
    
    if(!missing(parm)){
      parm1 <- parm[[1]]
      position <- parm[[2]]
    
      all.manager.names <- xml2::xml_attrs(xml2::xml_find_all(apsim_xml, ".//manager"))
    
      find.parm <- grep(parm1, all.manager.names, ignore.case = TRUE)
      
      if(length(find.parm) == 0)
        stop(paste(parm1, " not found"))
      if(length(find.parm) > 1) stop("parm selection should be unique")
      
      cat("Selected manager: ", all.manager.names[[find.parm]], "\n")

      crop <- xml2::xml_find_all(apsim_xml, paste0(".//manager"))[find.parm]
      crop2 <- xml2::xml_find_first(crop, "ui")
      
      if(!is.na(position)){
        parm.path.0 <- xml2::xml_path(xml2::xml_children(crop2)[[position]])
      }else{
        parm.path.0 <- ".//manager"
      }
      
      descr <- sapply(xml2::xml_attrs(xml2::xml_children(crop2)), function(x) x[["description"]])
      vals <- xml2::xml_text(xml2::xml_children(crop2))
    
      if(is.na(position)){
        position <- 1:length(vals)
      }
      crop.d <- data.frame(parm = descr, value = vals)[position,]
      print(knitr::kable(crop.d, digits = digits))
    }
  }
  
  if(node == "Outputfile"){
    parm.path.0 <- ".//outputfile"
    outputfile.node <- xml2::xml_find_all(apsim_xml, ".//outputfile")
    outputfile.parms <- c("filename", "title", "variables", "events")
    
    if(missing(parm)){
      parm <- "filename"
    }else{
      parm <- match.arg(parm, outputfile.parms)
    } 
    
    parm.path.1 <- paste0(parm.path.0, "/", parm)
    
    if(parm == "filename" || parm == "title"){
      output.node <- xml2::xml_find_first(outputfile.node, parm)
      output.name <- xml2::xml_text(output.node)
      tmpdat <- data.frame(tmp = output.name)
    }
    
    if(parm == "variables" || parm == "events"){
      variables.node <- xml2::xml_find_all(outputfile.node, parm)
      variables.names <- xml2::xml_text(xml2::xml_children(variables.node))
      tmpdat <- data.frame(tmp = variables.names)
    }
    
    names(tmpdat) <- parm
    print(knitr::kable(tmpdat))
  }
  
  if(node == 'Other'){
    if(missing(parm)){
      stop("parm should be specified when node = 'Other'")
    }
    
    other.d <- NULL
    other <- xml2::xml_find_all(apsim_xml, parm)
    for(i in other){
      ms.attr <- xml2::xml_children(other)
      ms.nm <- xml2::xml_name(ms.attr)
      ms.vl <- xml2::xml_text(ms.attr)
      tmp <- data.frame(parm = ms.nm, value = ms.vl)
      other.d <- rbind(other.d, tmp)
    }
    print(knitr::kable(other.d))
    
    parm.path.0 <- xml_path(other)
    if(length(parm.path.0) > 1) stop("figure out why parm.path is greater than 1 for 'Other'")
  }
  
  if(!node %in% c("Crop","Manager") && !is.null(parm)){
    parm.path <- xml2::xml_path(xml2::xml_find_first(apsim_xml,paste0(parm.path.0,"/",parm)))
  }
    
  if(print.path){
    if(is.null(parm.path.0)) stop("root parm path not found")
    
    if(!missing(root)){
      parm.path.0 <- paste0(parm.path.root, gsub("./", "", parm.path.0, fixed = TRUE))
      parm.path.1 <- paste0(parm.path.root, gsub("./", "", parm.path.1, fixed = TRUE))
    }
    
    parm.path <- parm.path.0
    if(!is.null(parm.path.1)){
      parm.path <- parm.path.1
    }
    cat("Parm path:", parm.path, "\n")
  }
  invisible(parm.path)
}


#'
#' @title Inspect an APSIM Classic auxiliary (XML) file
#' @name inspect_apsim_xml
#' @description inspect an auxiliary XML apsim file. 
#' @param file file ending in .xml to be inspected.
#' @param src.dir directory containing the .xml file to be inspected; defaults to the current working directory
#' @param parm parameter to inspect.
#' @param verbose Whether to print to standard output
#' @param print.path Whether to print the parameter path
#' @note the behavior has changed from previous verions (earlier than 1.977). Before,
#' if more than match was found it would return an error. Now it returns a list with all
#' possible matches. This can be useful when trying to find a parameter.
#' @return it returns an absolute parameter path(s)
#' @export
#' @examples  
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' 
#' inspect_apsim_xml("Maize75.xml", src.dir = extd.dir,
#'                   parm = "leaf_no_rate_change")
#'                   
#' pp <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir,
#'                         parm = "leaf_no_rate_change",
#'                         verbose = FALSE, 
#'                         print.path = FALSE)
#' }

inspect_apsim_xml <- function(file = "", 
                              src.dir = ".", 
                              parm,
                              verbose = TRUE,
                              print.path = TRUE){
  
  file.names.xml <- dir(path = src.dir, pattern = ".xml$", ignore.case = TRUE)
  file.names.apsim <- dir(path = src.dir, pattern = ".apsim$", ignore.case = TRUE)
  file.names <- c(file.names.xml, file.names.apsim)
  
  if(length(file.names) == 0){
    stop("There are no .xml or .apsim files in the specified directory to inspect.")
  }
  
  file <- match.arg(file, file.names)
  
  ## Read the file
  apsim_xml <- xml2::read_xml(paste0(src.dir, "/", file))
  
  find.all <- xml2::xml_find_all(apsim_xml, paste0(".//", parm))
  
  if(length(find.all) > 1){
    parm.path <- character(length = length(find.all))
    for(i in seq_along(find.all)){
      if(verbose){
        cat("attrs:", xml2::xml_attrs(find.all[[i]], "*"), "\n")
        length.xml.node <- xml2::xml_length(find.all[[i]])
        if(length.xml.node == 1){
          cat("text:", xml2::xml_text(find.all[[i]]), "\n")    
        }else{
          cat("text:", xml2::xml_text(xml2::xml_children(find.all[[i]])), "\n")    
        }        
      }
      parm.path[i] <- xml2::xml_path(find.all[i])
    }
    if(print.path) cat("path:", unlist(parm.path), "\n")
  } 
    
  if(length(find.all) == 1){
    xml_node <- xml2::xml_find_first(apsim_xml, paste0(".//", parm))
    parm.path <- xml2::xml_path(xml_node)
    if(verbose){
      cat("attrs:", xml2::xml_attrs(xml_node, "*"), "\n")
      length.xml.node <- xml2::xml_length(xml_node)
      if(length.xml.node <= 1){
        cat("text:", xml2::xml_text(xml_node), "\n")    
      }else{
        cat("text:", xml2::xml_text(xml2::xml_children(xml_node)), "\n")    
      }
    }
    if(print.path) cat("path:", parm.path, "\n")
  }
  invisible(parm.path)
}

#' view APSIM XML file
#' @title View an APSIM Classic auxiliary (XML) file
#' @name view_apsim_xml
#' @description view an auxilliary XML apsim file. 
#' @param file file ending in .xml to be viewed.
#' @param src.dir directory containing the .xml file to be viewed; defaults to the current working directory
#' @param viewer either \dQuote{json} or \dQuote{react}.
#' @param ... additional arguments passed to either \sQuote{jsonedit} or \sQuote{reactjson}.
#' @return It does not return an object but it produces a tree display of the APSIM file.
#' @export
#' @examples  
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' view_apsim_xml("Maize75.xml", src.dir = extd.dir)
#' }
#' 
view_apsim_xml <- function(file, src.dir, viewer = c("json","react"), ...){
  
  if(!requireNamespace("listviewer", quietly = TRUE)){
    warning("The listviewer package is required for this function")
    return(NULL)
  }
  
  if(missing(file)) stop("need to specify file name")
  
  .check_apsim_name(file)
  
  ## The might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".xml$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .xml files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  if(missing(src.dir)) src.dir <- "."
  
  viewer <- match.arg(viewer)
  
  file.name.path <- file.path(src.dir, file)
  
  apsim_xml <- xml2::read_xml(file.name.path)
  apsim_lst <- jsonlite::toJSON(xml2::as_list(apsim_xml), auto_unbox = TRUE)
  
  names(apsim_lst) <- NULL
  
  if(viewer == "json"){
    ans <- listviewer::jsonedit(listdata = apsim_lst, ...) 
    return(ans)
  }
  
  if(viewer == "react"){
    
    if(!requireNamespace("reactR", quietly = TRUE)){
      warning("The reactR package is required for this function")
      return(NULL)
    }
    
    ans <- listviewer::reactjson(listdata = apsim_lst, ...)  
    return(ans)
  }
}

## Write a test for running APSIM only under Windows
require(apsimx)
apsim_options(warn.versions = FALSE)

## This needs windows and APSIM Classic
run.classic.examples <- grepl("windows", Sys.info()[["sysname"]], ignore.case = TRUE) && get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()
setwd(tmp.dir)

if(run.classic.examples){
  
  ade <- auto_detect_apsim_examples()
  
  ex <- list.files(path = ade, pattern = ".apsim$")
  ## Will only run a few
  ex.to.run <- c("Canopy","Centro","Millet","Potato","Sugar")
  
  for(i in ex.to.run){
    tmp <- apsim_example(i)
    cat("Ran (apsim_example):",i,"\n")
  }
  
  ## Test examples individually
  ## Note: this will not work unless tmp.dir is the current directory
  ## because of a bug in APSIM
  for(i in ex.to.run){
    file.copy(paste0(ade,"/",i,".apsim"), tmp.dir)
    tmp <- apsim(paste0(i,".apsim"), cleanup = TRUE)
    file.remove(paste0(tmp.dir,"/",i,".apsim"))
    cat("Ran (apsim):",i,"\n")
  }
}

### Testing the performance of the compare_apsim_soil_profile function
### It should be more robust

require(apsimx)
require(ggplot2)

apsimx_options(warn.versions = FALSE)

run.compare.soils <- get(".run.local.tests", envir = apsimx.options)

if(run.compare.soils){
  sp1 <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE)
  sp2 <- get_ssurgo_soil_profile(lonlat = c(-92, 41), fix = TRUE)
  # Individual plots
  plot(sp1[[1]], property = "water")
  plot(sp2[[1]], property = "water")
  plot(sp1[[1]], property = "texture")
  plot(sp2[[1]], property = "texture")
  plot(sp1[[1]], property = "Carbon")
  plot(sp2[[1]], property = "Carbon")
  # Compare them
  cmp0 <- compare_apsim_soil_profile(sp1[[1]], sp2[[1]], labels = c("sp1", "sp2"))
  # Plot the variables
  plot(cmp0)
  plot(cmp0, property = "DUL")
  plot(cmp0, property = "Carbon")
  plot(cmp0, property = "BD")
  plot(cmp0, property = "ParticleSizeClay")
  plot(cmp0, property = "ParticleSizeSilt")
  plot(cmp0, property = "ParticleSizeSand")
}

### Compare soils with different rows
if(run.compare.soils){
  
  trrry <- try(sp1 <- get_isric_soil_profile(lonlat = c(-93, 42)), silent = TRUE)
  
  if(!inherits(trrry, 'try-error') && FALSE){
    ### SSURGO
    sp1 <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE)
    class(sp1[[1]])
    ### SoilGrids
    sp2 <- get_isric_soil_profile(lonlat = c(-92, 42), fix = TRUE)
    class(sp2)
    ### WorldModeler
    sp3 <- get_worldmodeler_soil_profile(lonlat = c(-92, 42))
    class(sp3[[1]])
    # Compare them
    cmp <- compare_apsim_soil_profile(sp1[[1]], sp2, sp3[[1]], merge.wide = FALSE,
                                      labels = c("ssurgo", "SoilGrids", "WorldModeler"))
    
    # Plot the variables
    plot(cmp)
    plot(cmp, soil.var = "DUL")
    plot(cmp, soil.var = "Carbon")
    plot(cmp, soil.var = "LL15")
  }else{
    ### SSURGO
    sp1 <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE)
    class(sp1[[1]])
    ### WorldModeler
    sp3 <- get_worldmodeler_soil_profile(lonlat = c(-92, 42))
    class(sp3[[1]])
    # Compare them
    cmp <- compare_apsim_soil_profile(sp1[[1]], sp3[[1]], merge.wide = FALSE,
                                      labels = c("ssurgo", "WorldModeler"))
    
    # Plot the variables
    plot(cmp)
    plot(cmp, soil.var = "DUL")
    plot(cmp, soil.var = "Carbon")
    plot(cmp, soil.var = "LL15")
  }
}

## Testing the check_apsimx function with a variety of files
require(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

run.apsimx.check <- get(".run.local.tests", envir = apsimx.options)

if(run.apsimx.check){

  ex.dir <- auto_detect_apsimx_examples()
  ex.dir.list <- dir(ex.dir, pattern = "apsimx$")

  for(.j in seq_along(ex.dir.list)){
    
    file.to.check <- ex.dir.list[.j]
    if(file.to.check == "Slurp.apsimx") next
    if(file.to.check == "Graph.apsimx") next
    
    if(.j %in% c(2, 4, 5, 16, 17, 18, 24, 29, 30, 34)){
      cat("Simulation check:", file.to.check, "\n")
      check_apsimx(file.to.check, ex.dir)
    }
    
    if(.j %in% c(10, 11)){
      ## Simulation 10 is eucalyptus
      cat("Simulation to inspect:", file.to.check, "\n")
      capo <- capture.output(inspect_apsimx(file.to.check, ex.dir, node = "Other", parm = 3))
    }
    
    if(.j %in% c(6, 7, 8, 9)){
      ## Simulation 6 is chickpea
      ## Simulation 7 is chicory
      ## Simulation 8 is controlledenvironment
      ## Simulation 9 is csvweather
      cat("Simulation check:", file.to.check, "\n")
      capo <- capture.output(inspect_apsimx(file.to.check, ex.dir, node = "Other", parm = 3))
      if(file.to.check == "Chickpea.apsimx"){
        inspect_apsimx(file.to.check, ex.dir, root = "Continuous_TOS.Cont_TOS")
        inspect_apsimx(file.to.check, ex.dir, node = "Soil",
                       soil.child = "Metadata",
                       root = "Continuous_TOS.Cont_TOS") 
        inspect_apsimx(file.to.check, ex.dir, node = "Soil",
                       soil.child = "Physical",
                       root = "Continuous_TOS.Cont_TOS") 
        check_apsimx(file.to.check, ex.dir, root = "Continuous_TOS.Cont_TOS")        
      }
      if(file.to.check %in% c("Chicory.apsimx", "ControlledEnvironment.apsimx", "CsvWeather.apsimx")){
        inspect_apsimx(file.to.check, ex.dir, root = "Simulation")
        inspect_apsimx(file.to.check, ex.dir, node = "Soil",
                       soil.child = "InitialWater",
                       root = "Simulation")
        check_apsimx(file.to.check, ex.dir, root = "Simulation")        
      }
    }

    if(.j %in% c(1, 3)){
      if(file.to.check == "BiomassRemovalFromPlant.apsimx"){
        check_apsimx(file.to.check, ex.dir, root = "UsingFixedDates")  
        check_apsimx(file.to.check, ex.dir, root = "SendingDatesFromOpperations")
      }
      if(file.to.check == "AgPasture.apsimx"){
        pps <- inspect_apsimx(file.to.check, src.dir = ex.dir, node = "Other", parm = list(1, 3:7), print.path = TRUE)
        for(.jj in pps){
          rut <- strsplit(.jj, ".", fixed = TRUE)[[1]][3]
          if(rut == "Harvested Pasture by Production Year") next
          if(rut == "PastureByWaterAndNitrogen") next
          if(rut == "CO2xBaseTemperature") next
          if(rut == "CO2xHeatOnset ") next
          check_apsimx(file.to.check, ex.dir, root = rut)
        }
      }
    }
  }
}

#### Check files in extd.dir

if(run.apsimx.check){
  
  ### Files that make check throw an error
  ### 1. MaizeSoybean.apsimx
  extd.dir <- system.file("extdata", package = "apsimx")
  
  apsimx.files.to.check <- dir(extd.dir, pattern = "apsimx")
  
  # for(.ii in seq_along(apsimx.files.to.check)){
  #   ### Some of these files need 'root'
  #   if(apsimx.files.to.check[[.ii]] == "Factorial.apsimx"){
  #     for(.jj in )
  #   }
  #   check_apsimx(apsimx.files.to.check[[.ii]], extd.dir)  
  # }
}

#' Get APSIM-X Model Replacement from github
#' 
#' @title fetches the json file for a specific model from APSIMX github
#' @name get_apsimx_json
#' @description Retrieves the json replacement file for a specific model
#' @param model a model (e.g. \sQuote{Wheat} or \sQuote{Maize})
#' @param wrt.dir directory to save the JSON file (default is the current directory)
#' @param cleanup whether to delete the JSON file
#' @return a list read through the jsonlite package
#' @seealso \code{\link{insert_replacement_node}}
#' @export
#' @examples 
#' \donttest{
#' tmp.dir <- tempdir()
#' ### try to download without generating an error
#' wheat <- try(get_apsimx_json(model = "Wheat", wrt.dir = tmp.dir), silent = TRUE)
#' }
#' 

get_apsimx_json <- function(model = "Wheat", wrt.dir = ".", cleanup = FALSE){
  ### Construct the string
  st0 <- "https://raw.githubusercontent.com/APSIMInitiative/ApsimX/master/Models/Resources"
  str <- paste0(st0, "/", model, ".json")
  dst <- file.path(wrt.dir, paste0(model, ".json"))
  utils::download.file(url = str, destfile = dst)

  ans <- jsonlite::read_json(dst)
  
  if(cleanup) unlink(dst)
  
  invisible(ans)
}

#' @title Inserts a replacement node in a simple apsimx simulation file
#' @name insert_replacement_node
#' @description Inserts a replacement node in a simple apsimx simulation file
#' @param file file ending in .apsimx to be edited (JSON)
#' @param src.dir directory containing the .apsimx file to be edited; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param rep.node replacement node as obtained by the \code{\link{get_apsimx_json}} function
#' @param edit.tag if the file is edited a different tag from the default \sQuote{-edited} can be used.
#' @param overwrite logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise
#' @param verbose whether to print information about successful edit
#' @return it does not return an R object but it writes an apsimx file to disk
#' @export
#' @examples
#' \dontrun{
#' ## It is not trivial to produce a reproducible example
#' ## because the model and file versions need to align.
#' ## The steps are:
#' ## 1. Get model: 
#' ##    wheat <- get_apsimx_json(model = "Wheat", wrt.dir = tmp.dir)
#' ## 2. Create file that matches current model version
#' ## 3. Edit the file by inserting the 'replacements' node
#' ##    insert_replacement_node("Wheat.apsimx", rep.node = wheat)
#'                         
#' }
#' 

insert_replacement_node <- function(file, src.dir, wrt.dir, rep.node, 
                                    edit.tag = "-edited", 
                                    overwrite = FALSE,
                                    verbose = FALSE){
  
  .check_apsim_name(file)
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to edit.")
  }
  
  file <- match.arg(file, file.names)
  
  if(apsimx_filetype(file = file, src.dir = src.dir) != "json")
    stop("This function only edits JSON files")
  
  ## I should check that 'rep.node' is a proper 'replacements' node
  ## but I'm not sure how to do that, because there are many different types
  
  ## Parse apsimx file (JSON)
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
  
  ## Replacement version and APSIMX version need to match
  replacement.node.version <- rep.node$Version
  apsimx.file.version <- apsimx_json$Version
  
  if(replacement.node.version != apsimx.file.version)
    stop("Replacement node version and apsimx version should match")

  wcore <- grep("Core.Simulation", apsimx_json$Children)

  if(verbose){
    cat("Simulation(s) is/are in node(s)", wcore, "\n")
    cat("Adding node", rep.node$Children[[1]]$Name, "\n")
  }
  
  ## Test if Replacements folder is already present
  simulation.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
  wreplace <- grep("Replacements", simulation.names)
  
  if(length(wreplace) == 0){
    replacements.folder <- vector("list", length = 1)
    rep.node.list <- vector("list", length = 1)
    rep.node.list[[1]] <- rep.node$Children[[1]]
    
    replacements.folder.elements <- list(`$type` = "Models.Core.Folder, Models",
                                         ShowInDocs = FALSE,
                                         GraphsPerPage = 6,
                                         Name = "Replacements",
                                         ResourceName = NULL,
                                         Children = rep.node.list,
                                         Enabled = TRUE,
                                         ReadOnly = FALSE)
    
    replacements.folder[[1]] <- replacements.folder.elements
    
    apsimx_json$Children <- append(apsimx_json$Children, replacements.folder, after = 0)    
  }else{
    if(verbose)
      cat("Replacement folder already present. Adding model")
    replacements.folder <- apsimx_json$Children[[wreplace]]
    rep.node.list <- vector("list", length = 1)
    rep.node.list[[1]] <- rep.node$Children[[1]]
    replacements.folder$Children <- append(replacements.folder$Children, rep.node.list)
    apsimx_json$Children[[wreplace]] <- replacements.folder
  }

  
  ## Write to file
  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir, "/",
                      tools::file_path_sans_ext(file),
                      edit.tag, ".apsimx")
  }else{
    wr.path <- paste0(wrt.dir, "/", file)
  }
  
  jsonlite::write_json(apsimx_json, path = wr.path, 
                       pretty = TRUE, digits = NA, 
                       auto_unbox = TRUE, null = "null")
  
  if(verbose){
    cat("Created: ", wr.path,"\n")
  }
}

## Testing whether 

require(apsimx)
apsimx_options(warn.versions = FALSE)

# apsimx_options(warn.versions = FALSE,
#                exe.path = "/Applications/APSIM2022.6.7044.0.app/Contents/Resources/bin/Models",
#                examples.path = "/Applications/APSIM2022.6.7044.0.app/Contents/Resources/Examples")

run.insert <- get(".run.local.tests", envir = apsimx.options)

#### This does not work at the moment
run.insert <- FALSE

tmp.dir <- tempdir()
extd.dir <- system.file("extdata", package = "apsimx")

if(run.insert){
  
  ex.dir <- auto_detect_apsimx_examples()
  
  ## tmp.dir <- "~/Desktop/csm"
  #### Test for wheat ####
  wheat <- get_apsimx_json(model = "Wheat", wrt.dir = tmp.dir)
  
  inspect_apsimx_json("Wheat.json", src.dir = tmp.dir,
                      parm = "Version")
  
  file.copy(file.path(ex.dir, "Wheat.apsimx"), tmp.dir)  
  
  inspect_apsimx("Wheat_7382.apsimx", src.dir = tmp.dir, node = "Other", parm = 2)
  
  inspect_apsimx_json("Wheat_7382.apsimx", src.dir = tmp.dir,
                      parm = "Version")
  
  ## Versions need to match, but they don't right now even with the latest
  
  insert_replacement_node("Wheat_7382.apsimx", 
                          src.dir = tmp.dir, 
                          wrt.dir = tmp.dir,
                          rep.node = wheat)
  
  inspect_apsimx("Wheat_7382-edited.apsimx", src.dir = tmp.dir, node = "Other", parm = 2)

  #### Test for maize ####
  maize <- get_apsimx_json(model = "Maize", wrt.dir = tmp.dir)  
  insert_replacement_node("Wheat_7382-edited.apsimx", 
                          src.dir = tmp.dir,
                          wrt.dir = tmp.dir,
                          rep.node = maize,
                          edit.tag = "-maize")

  #### Test for soybean ####
  soybean <- get_apsimx_json(model = "Soybean", wrt.dir = tmp.dir)
  
  file.copy(file.path(ex.dir, "Soybean.apsimx"), tmp.dir)  
  
  insert_replacement_node("Soybean.apsimx", 
                          src.dir = extd.dir, 
                          wrt.dir = tmp.dir,
                          rep.node = soybean)
  
  sim2 <- apsimx("Soybean-edited.apsimx", src.dir = tmp.dir)
  
}

#'
#' This function requires the \CRANpkg{chirps} package, version 0.1.4.
#'
#' @title Get CHIRPS data for an APSIM met file
#' @description Uses \code{\link[chirps]{get_chirps}} from the \CRANpkg{chirps} package to download data to create an APSIM met file.
#' @name get_chirps_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param dates date ranges
#' @param wrt.dir write directory
#' @param filename file name for writing out to disk
#' @param fillin.radn whether to fill in radiation data using the nasapower pacakge. Default is TRUE.
#' @param silent defaut is FALSE. Changing it will not do anything at the moment. A future feature.
#' @return returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @details If the filename is not provided it will not write the file to disk,
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk.
#' @export
#' @examples
#' \dontrun{
#' require(chirps)
#' ## This will not write a file to disk
#' chrp <- get_chirps_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
#' }
#'

get_chirps_apsim_met <- function(lonlat, dates, wrt.dir = ".", filename = NULL,
                                 fillin.radn = TRUE, silent = FALSE){

  if(!requireNamespace("chirps", quietly = TRUE)){
    warning("The chirps package is required for this function")
    return(NULL)
  }

  if(missing(filename)) filename <- "noname.met"

  if(!grepl(".met", filename, fixed = TRUE)) stop("filename should end in .met")

  objct <- data.frame(lon = lonlat[1], lat = lonlat[2])

  chrp <- chirps::get_chirps(object = objct, dates = dates, server = "ClimateSERV") ## This is only precipitation

  pwr <- get_power_apsim_met(lonlat = lonlat, dates = dates)

  pwr$rain <- chrp$chirps
  pwr[["rh"]] <- NULL
  pwr[["windspeed"]] <- NULL
  
  attr(pwr, "units") <- c("()", "()", "(MJ/m2/day)", "(oC)", "(oC)", "(mm)")
  attr(pwr, "colnames") <- c("year", "day", "radn", "maxt", "mint", "rain")

  chrp <- pwr

  if(filename != "noname.met"){
    write_apsim_met(chrp, wrt.dir = wrt.dir, filename = filename)
  }
  return(invisible(chrp))
}

#'
#' It is possible to provide a list of soil profiles for replacement in the simulations. In this
#' case, the parameter path can be simply \sQuote{soil.profile} or \sQuote{soil_profile} if there
#' is one single simulation. It can also be the path to \sQuote{Soil}. In this case, the path should
#' be something such as \sQuote{Simulations.SimulationName.Soil}. \sQuote{SimulationName} should
#' be replaced with the appropriate string.
#'
#' In the grid, the column with name \sQuote{soil.profile} should contain integers that will be used to
#' pick from the list of provided soil profiles. In this case it is possible to re-use them.
#' For example, the values could be 1, 2, 3, etc. to select the corresponding soil profiles from
#' the \sQuote{soil.profiles} list.
#'
#' If the \sQuote{cores} argument is greater than 1, then the package \CRANpkg{future} is required.
#' It will first search for a future plan under options and if nothing is found it will chose an OS-appropriate plan
#' (\sQuote{multisession}) and it uses the chosen number of cores for execution. Errors, messages and warnings are normally 
#' suppressed during parallel execution, so it is important to ensure that the simulations are constructed properly.
#' In testing, cloud services (Box, Dropbox, etc.) do not work well as they seem to interfere with the syncing
#' of apsim database files. It is suggested that they are turned off when running simulations.
#' 
#' In version 2.8.0 and earlier the original file was changed after it was edited by the function.
#' In newer versions, the function creates a backup file and then restores it after the code is
#' executed, if there are no errors.
#'
#' Suggested reading on the topic of sensitivity analysis:
#'
#' Pianosa et al (2016). Sensitivity analysis of environmental models: A systematic review with practical workflow.
#' \doi{10.1016/j.envsoft.2016.02.008}
#'
#' Saltelli et al. . Global Sensitivity Analysis.
#'
#' @title Sensitivity Analysis for APSIM Next Generation simulation
#' @name sens_apsimx
#' @rdname sens_apsimx
#' @description It is a wrapper for running APSIM-X and evaluating different parameters values
#' @param file file name to be run (the extension .apsimx is optional)
#' @param src.dir directory containing the .apsimx file to be run (defaults to the current directory)
#' @param parm.paths absolute or relative paths of the coefficients to be evaluated.
#'             It is recommended that you use \code{\link{inspect_apsimx}} for this
#' @param convert (logical) This argument is needed if there is a need to pass a vector instead of a single value.
#' The vector can be passed as a character string (separated by spaces) and it will be converted to a
#' numeric vector. It should be either TRUE or FALSE for each parameter.
#' @param replacement TRUE or FALSE for each parameter. Indicating whether it is part of
#' the \sQuote{replacement} component. Its length should be equal to the length or \sQuote{parm.paths}.
#' @param grid grid of parameter values for the evaluation. It can be a data.frame.
#' @param soil.profiles list with soil profiles for replacement (see details.)
#' @param summary function name to use to summarize the output to be a sinlge row (default is the mean).
#' @param root root argument for \code{\link{edit_apsimx_replacement}}
#' @param verbose whether to print progress in percent and elapsed time.
#' @param cores number of cores to use for parallel evaluation
#' @param save whether to save intermediate results. By default they will be saved as a
#' \sQuote{csv} file using the name of the apsim file. This will replace \sQuote{apsimx} with \sQuote{csv}.
#' It is also possible to provide the file name here (for example: \sQuote{Some_results.csv}).
#' @param ... additional arguments.
#' @note The summary function is stored as an attribute of the data frame \sQuote{grid.sims}.
#' @return object of class \sQuote{sens_apsim}, but really just a list with results from the evaluations.
#' @export
#' @examples
#' \donttest{
#' ## See the vignette for examples
#' }
#'

sens_apsimx <- function(file, src.dir = ".",
                        parm.paths,
                        convert,
                        replacement,
                        grid,
                        soil.profiles = NULL,
                        summary = c("mean", "max", "var", "sd", "none"),
                        root,
                        verbose = TRUE,
                        cores = 1L,
                        save = FALSE,
                        ...){

  if(missing(file))
    stop("file is missing with no default", call. = FALSE)

  if(isFALSE(get("allow.path.spaces", apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }

  ### Make a backup copy of the file, since it will be modified later
  file.backup.name <- paste0(tools::file_path_sans_ext(file), "-backup.apsimx")
  file.copy(from = file.path(src.dir, file), 
            to = file.path(src.dir, file.backup.name))
  if(verbose > 1L)
    message("Created backup copy of original file")
  
  if(cores > 1L){

    if(!requireNamespace("future", quietly = TRUE)){
      warning("The 'future' package is required for this option")
      return(NULL)
    }

    if(cores > (nrow(grid) - 1))
       stop("'cores' should be an integer smaller than the number of simulations minus one", call. = FALSE)
    ### detect.cores <- parallel::detectCores()
    ### Suggestion by Henrik Bengtsson to use parallely instead of parallel (github issue #191)
    ### Also, the code below might be unnecessary (or redundant) - I will keep it for now
    detect.cores <- parallelly::availableCores()
    if(cores > detect.cores)
      stop("'cores' argument should not be higher than the number of available cores", call. = FALSE)

    ## This code was contributed by Henrik Bengtsson
    cplan <- future::plan() ## This is the current plan
    if (inherits(cplan, "sequential")) {
      oplan <- future::plan(strategy = future::multisession, workers = cores)
    } else {
      oplan <- future::plan(cplan, workers = cores) 
    }
    on.exit(future::plan(oplan), add = TRUE) 
    
  }

  ## This might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)

  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to run.", call. = FALSE)
  }

  file <- match.arg(file, file.names)

  summary <- match.arg(summary)

  if(missing(convert)){
    convert <- rep(FALSE, length(parm.paths))
  }else{
    if(length(convert) != length(parm.paths))
      stop("convert should have length equal to parm.paths", call. = FALSE)
    if(!is.logical(convert))
      stop("convert should be logical", call. = FALSE)
  }

  if(missing(replacement)) replacement <- rep(FALSE, length(parm.paths))

  if(missing(grid))
    stop("grid argument is missing", call. = FALSE)

  grid <- as.data.frame(grid)

  if(ncol(grid) != length(parm.paths))
      stop("Number of columns in grid should be equal to the number of parameters", call. = FALSE)

  ## When verbose this is more compact than previous behavior
  pb <- utils::txtProgressBar(min = 1, max = nrow(grid), style = 3)

  ## Check that the name in the grid appears somewhere in the parameter path
  for(.ii in seq_along(parm.paths)){
    is.dot.present <- grepl(".", names(grid)[.ii], fixed = TRUE)
    is.soil <- grepl("soil.profile|^Soil$", names(grid)[.ii]) ## This should be of length one

    if(is.dot.present && !is.soil){
      ## The first element should match simulation names
      fspe <- strsplit(names(grid)[.ii], ".", fixed = TRUE)[[1]]
      ## First and second parameter elements
      apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
      simulation.names <- sapply(apsimx_json$Children, FUN = function(x) x$Name)
      ippgn1 <- grepl(fspe[1], parm.paths[.ii], ignore.case = TRUE)
      if(!ippgn1){
        cat("Name in first elelment grid name:", fspe[1], "\n")
        cat("parameter name", parm.paths[.ii], "\n")
        warning("names in first element of grid object name do not match parameter path name")
      }
      ippgn2 <- grepl(fspe[2], parm.paths[.ii], ignore.case = TRUE)
      if(!ippgn2){
        cat("Name in second element grid names:", fspe[2], "\n")
        cat("parameter name", parm.paths[.ii], "\n")
        warning("names in second element of grid object name do not match parameter path name")
      }
    }else{
      ### If the intention is to replace soil profiles
      ### Then 'parm.path' can only be either 'soil.profile' or 'Soil'
      if(grepl("soil.profile|^Soil$", parm.paths[.ii])){
       ### Check that there is a column in grid that corresponds to soils
        isn <- grepl("soil.profile|^Soil$", names(grid)[.ii], ignore.case = TRUE)
        if(isFALSE(isn)){
          cat("Name in grid:", names(grid)[.ii], "\n")
          cat("parameter name", parm.paths[.ii], "\n")
          warning("soil.profile is in the parameter paths but not found in the grid")
        }
      }else{
        ippgn <- grepl(names(grid)[.ii], parm.paths[.ii], ignore.case = TRUE)
        if(!ippgn){
          cat("Name in grid:", names(grid)[.ii], "\n")
          cat("parameter name", parm.paths[.ii], "\n")
          warning("names in grid object do not match parameter path name")
        }
      }
    }
  }

  col.sim <- NULL
  start <- Sys.time()

  original.cores <- cores
  cores.last.round <- 0
  core.counter <- 1L
  sim.list <- vector("list", length = cores)

  for(.i in 1:dim(grid)[1]){

    ## Need to edit the parameters in the simulation file or replacement
    for(.j in seq_along(parm.paths)){
      ## Edit the specific parameters with the corresponding values
      if(convert[.j] <= 0){
        par.val <- grid[.i, .j]
      }else{
        ## Converting from character to numeric. Values need to be separated by spaces.
        par.val <- as.numeric(strsplit(grid[.i, .j], " ")[[1]])
      }

      if(replacement[.j]){
        pp0 <- strsplit(parm.paths[.j], ".", fixed = TRUE)[[1]]
        mpp <- paste0(pp0[-length(pp0)], collapse = ".")
        if(missing(root)){
          edit_apsimx_replacement(file = file,
                                  src.dir = src.dir,
                                  wrt.dir = src.dir,
                                  node.string = mpp,
                                  overwrite = TRUE,
                                  parm = pp0[length(pp0)],
                                  value = par.val,
                                  verbose = FALSE)
        }else{
          edit_apsimx_replacement(file = file,
                                  src.dir = src.dir,
                                  wrt.dir = src.dir,
                                  node.string = mpp,
                                  overwrite = TRUE,
                                  parm = pp0[length(pp0)],
                                  value = par.val,
                                  root = root,
                                  verbose = FALSE)
        }
      }else{
        ## cat("parameter path", parm.paths[.j], "\n")
        if(!grepl("soil.profile|Soil$", parm.paths[.j])){
          if(missing(root)){
            edit_apsimx(file = file,
                        src.dir = src.dir,
                        wrt.dir = src.dir,
                        node = "Other",
                        parm.path = parm.paths[.j],
                        overwrite = TRUE,
                        value = par.val,
                        verbose = FALSE)
          }else{
            edit_apsimx(file = file,
                        src.dir = src.dir,
                        wrt.dir = src.dir,
                        node = "Other",
                        parm.path = parm.paths[.j],
                        overwrite = TRUE,
                        value = par.val,
                        root = root,
                        verbose = FALSE)
          }
        }else{
         if(verbose > 1L) cat("Replacing the soil profile in simulation", .i, "\n")
         wspc <- grep("soil.profile|Soil$", names(grid)) ## Which one is the soil profile column

         if(length(wspc) == 0){
           cat("Result of wspc:", wspc, "\n")
           stop("'soil.profile' column not found in grid", call. = FALSE)
         }

         ## There can be more than one column with soils that need to be replaced
         for(.k in seq_along(wspc)){
           if(grepl("Soil$", names(grid)[wspc[.k]])){
             ## In this case there is a 'root'
             soil.node.name <- parse_root(names(grid)[wspc[.k]])
             if(soil.node.name[1] == "Simulations"){
               root <- soil.node.name[2]
             }else{
               root <- soil.node.name[1]
             }
           }
           soil.profile.index <- grid[.i, wspc[.k]]

           if(missing(root)){
             edit_apsimx_replace_soil_profile(file = file,
                                              src.dir = src.dir,
                                              wrt.dir = src.dir,
                                              soil.profile = soil.profiles[[soil.profile.index]],
                                              overwrite = TRUE,
                                              verbose = FALSE)
           }else{
             edit_apsimx_replace_soil_profile(file = file,
                                              src.dir = src.dir,
                                              wrt.dir = src.dir,
                                              soil.profile = soil.profiles[[soil.profile.index]],
                                              overwrite = TRUE,
                                              verbose = FALSE,
                                              root = root)
           }
         }
        }
      }
    }

    ## Run simulation
    ## The first simulation should not be parallelized
    if(cores == 1L || .i == 1){
      sim <- try(apsimx(file = file, src.dir = src.dir,
                        silent = TRUE, cleanup = TRUE, value = "report"),
                 silent = TRUE)

      if(inherits(sim, "try-error") && .i == 1){
        stop("Simulation failed for initial parameter combination")
      }

      ## Extract basic information from sim
      if(.i == 1){
        col.class.numeric <- which(sapply(sim, class) == "numeric") ## Which columns are numeric
        nms.sim <- names(sim[, col.class.numeric, drop = FALSE]) ## Names of the columns
        ncol.class.numeric <- ncol(sim[, col.class.numeric, drop = FALSE])
      }

      if(inherits(sim, "try-error") && .i > 1){
        ## This will skip to the next iteration
        ## If summary is not equal to none, then we need just one row
        ## If summary is equal to none we fill it with an empty data.frame
        ## with number of rows equal to the number in sim
        if(summary != "none"){
          mat <- matrix(ncol = ncol.class.numeric)
        }else{
          mat <- matrix(nrow = nrow(sim), ncol = ncol.class.numeric)
        }
        sim.sd <- as.data.frame(mat)
        names(sim.sd) <- nms.sim
        col.sim <- rbind(col.sim, sim.sd)
        next
      }
      sim.sd <- sens_summary(sim, summary = summary, grid = grid, i.index = .i, col.class.numeric = col.class.numeric)
    }

    if(original.cores > 1L && .i > 1){
      ## The first simulation will not be parallelized
      ## How many simulations are remaining?
      ## If the number of simulations remaining are less than the number of cores change a few things
      num.sim.rem <- dim(grid)[1] - .i

      if(verbose > 1){
        cat("Iteration:", .i, "\n")
        cat("Number of simulations remaining:", num.sim.rem, "\n")
      }

      while(core.counter <= cores){
        ## Maybe I should edit the file and then delete it
        ## to prevent conflicts with the file below
        file.to.run <- file.path(src.dir, paste0(tools::file_path_sans_ext(file),"-", core.counter, ".apsimx"))

        fcs <- file.copy(from = file.path(src.dir, file), to = file.to.run)
        if(isFALSE(fcs)){
          message("Iteration:", .i)
          message("Core counter:", core.counter)
          message("File to run": file.to.run)
          stop("Failed to copy file", call. = FALSE)
        }
          
        sim.list[[core.counter]] <- future::future(try(apsimx(file = paste0(tools::file_path_sans_ext(file), "-", core.counter, ".apsimx"),
                                                                src.dir = src.dir,
                                                                silent = TRUE,
                                                                cleanup = TRUE, value = "report"), silent = TRUE),
                                                   conditions = character(0))
        break
      }
      if(core.counter < cores){
        if(verbose > 1) cat("Core counter :", core.counter, "\n")
        core.counter <- core.counter + 1
        next
      }

      if(core.counter >= cores){
        if(verbose > 1) cat("Core counter :", core.counter, "\n")
        core.counter <- core.counter + 1
      }

      if(verbose > 1){
        print(sapply(sim.list, class))
        cat("Length sim.list:", length(sim.list), "\n")
        cat("Number of simulations remaining:", num.sim.rem, "\n")
      }

      simc <- future::value(sim.list)
      core.counter <- 1L

      if(num.sim.rem < cores && cores.last.round < 1){
        cores <- num.sim.rem
        sim.list <- vector("list", length = cores)
        cores.last.round <- 2
      }

      ## Identify which simulations failed
      sim.class <- sapply(simc, class)

      if(any(sim.class == "try-error")){
        if(verbose > 1) cat("Simulations which failed:", which(sim.class == "try-error"), "\n")
        ## How can I handle it if some simulations failed?
        sims.lst <- vector("list", length = length(simc))
        for(j in seq_len(length(simc))){
          if(inherits(simc[[j]], "try-error")){
            if(summary != "none"){
              mat <- matrix(ncol = ncol.class.numeric)
            }else{
              mat <- matrix(nrow = nrow(sim), ncol = ncol.class.numeric)
            }
            sim.sd.na <- as.data.frame(mat)
            names(sim.sd.na) <- nms.sim
            sims.lst[[j]] <- sim.sd.na
          }else{
            sims.lst[[j]] <- sens_summary(simc[[j]], summary = summary, grid = grid, i.index = .i, col.class.numeric = col.class.numeric)
          }
        }
        sim.sd <- do.call(rbind, sims.lst)
      }

      ## If all simulations are successful
      if(all(sim.class != "try-error")){
        ## Before they are merged both should be data.frames
        sims.lst <- vector("list", length = length(simc))
        for(j in seq_len(length(simc))){
          sims.lst[[j]] <- sens_summary(simc[[j]], summary = summary, grid = grid, i.index = .i, col.class.numeric = col.class.numeric)
        }
        sim.sd <- do.call(rbind, sims.lst)
      }

      for(j in seq_len(length(simc))){
        frs <- file.remove(file.path(src.dir, paste0(tools::file_path_sans_ext(file), "-", j, ".apsimx")))
        if(isFALSE(frs)){
          message("Iteration:", .i)
          message("Core counter:", core.counter)
          stop("Failed to remove file", call. = FALSE)
        }
      }
    }

    ## This combines the simulations regardless of cores argument
    col.sim <- rbind(col.sim, sim.sd)

    if(save != FALSE){
      if(isTRUE(save)){
        save.file.name <- paste0(tools::file_path_sans_ext(file), ".csv")
        utils::write.csv(col.sim,
                         file = save.file.name,
                         row.names = FALSE)
      }else{
        if(!is.character(save))
          stop("'save' argument should be a character", call. = FALSE)
        if(tools::file_ext(save) != "csv")
          stop("'save' only accepts 'csv' as an extension", call. = FALSE)
        save.file.name <- save
        utils::write.csv(col.sim,
                         file = save.file.name,
                         row.names = FALSE)
      }
    }

    if(isTRUE(verbose)){
      utils::setTxtProgressBar(pb, .i)
    }else{
      if(verbose >= 1L){
        nrow.grid <- nrow(grid)
        old.prev.div <- 0

        if(nrow.grid <= 10){
          dftm <- difftime(Sys.time(), start)
          cat("Progress:", round((.i/nrow.grid) * 100), "%. Time elapsed:", round(dftm, 2), units(dftm)," \n")
        }else{
          progress.step <- ifelse(nrow.grid <= 20, 10, 5)

          prev.div <- round((.i/nrow.grid) * 100) %/% progress.step

          if(prev.div > old.prev.div){
            dftm <- difftime(Sys.time(), start)
            cat("Progress:", round((.i/nrow(grid)) * 100), "%. Time elapsed:", round(dftm, 2), units(dftm)," \n")
            old.prev.div <- prev.div
          }
        }
      } ## End of verbose chunk
    }
  } ## End of big for loop

  close(pb)

  if(verbose && !isFALSE(save))
    cat("Saved results as:", save.file.name, "\n")

  if(summary != "none"){
    cdat <- cbind(grid, col.sim)
  }else{
    cdat <- col.sim
  }

  ### If there are no errors, which usually happen earlier
  file.remove(file.path(src.dir, file)) ## Remove the modified file
  file.rename(from = file.path(src.dir, file.backup.name),
            to = file.path(src.dir, file))
  if(verbose > 1L){
    message("Restoring the original file from backup")
  }
  
  attr(cdat, "summary") <- summary

  ans <- structure(list(grid.sims = cdat, grid = grid, parm.paths = parm.paths),
                   class = "sens_apsim")

  return(ans)
}

## Function to apply summary
sens_summary <- function(sim, summary = c("mean", "max", "var", "sd", "none"),
                         grid, i.index,
                         col.class.numeric){

  summary <- match.arg(summary)

  if(summary == "mean"){
    sim.s <- colMeans(sim[, col.class.numeric, drop = FALSE], na.rm = TRUE)
    sim.sd <- as.data.frame(t(sim.s))
  }

  if(summary == "max"){
    sim.s <- apply(sim[, col.class.numeric, drop = FALSE], 2, max, na.rm = TRUE)
    sim.sd <- as.data.frame(t(sim.s))
  }

  if(summary == "var"){
    sim.s <- apply(sim[, col.class.numeric, drop = FALSE], 2, var, na.rm = TRUE)
    sim.sd <- as.data.frame(t(sim.s))
  }

  if(summary == "sd"){
    sim.s <- apply(sim[, col.class.numeric, drop = FALSE], 2, sd, na.rm = TRUE)
    sim.sd <- as.data.frame(t(sim.s))
  }

  if(summary == "none"){
    sim.sd <- cbind(grid[i.index, , drop = FALSE], sim, row.names = NULL)
  }

  return(sim.sd)
}

#' The argument \sQuote{formula} can be as in \code{\link{lm}}. The response
#' can be omitted.
#'
#' @rdname sens_apsimx
#' @description Summary computes variance-based sensitivity indexes from an object of class \sQuote{sens_apsim}
#' @param object object of class \sQuote{sens_apsim}
#' @param ... additional arguments (none used at the moment)
#' @param formula formula to be passed to analysis of variance. See \code{\link{formula}}.
#' @param scale if all inputs are numeric it is better to scale them. The
#' default is FALSE as some inputs might be characters or factors. In this
#' case all inputs will be treated as factors in the sum of squares decomposition.
#' @param select option for selecting specific variables in the APSIM output. It will be treated as a regular expression
#' @param warning whether to issue a warning when applying this function to an object which has not been summarized
#' @param verbose whether to print to console results of summary
#' @return prints to console if verbose and returns a data frame
#' @export
#'
summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "all", warning = TRUE, verbose = TRUE){

  ## It probably does not make sense to compute this summary if the data were
  ## not previously summarized
  smmry <- attr(object$grid.sims, "summary")
  if(smmry == "none" && warning)
    warning("It is unlikey that running 'summary' will be useful on a
    sensitivity analysis output which was not previously summarized.")
  ## Here I compute sensitivity indexes based on the grid.sims object
  ## There are potentially many variables for which sensitivity analysis is relevant
  nms.resp.var <- setdiff(names(object$grid.sims), names(object$grid))

  l.nms.resp.var <- length(nms.resp.var)
  .j <- 0

  if(any(select == "all")){
    select <- nms.resp.var
  }else{
    gsel.col <- NULL
    for(i in seq_along(select)){
      gsel <- grep(select[i], nms.resp.var)
      if(length(gsel) == 0){
        cat("selection:", select[i], "\n")
        stop("not present in simulation object", call. = FALSE)
      }
      gsel.col <- c(gsel.col, gsel)
    }
    select <- nms.resp.var[unique(gsel.col)]
  }

  object$grid.sims <- subset(object$grid.sims, select = select)

  num.resp.var <- ncol(object$grid.sims) - ncol(object$grid)
  nms.resp.var <- setdiff(names(object$grid.sims), names(object$grid))
  wrn <- NULL

  for(.i in seq_along(nms.resp.var)){
    X <- object$grid
    y <- object$grid.sims[,nms.resp.var[.i]]
    if(suppressWarnings(var(y, na.rm = TRUE) == 0) || is.character(y[1]) || !is.numeric(y)) next

    if(scale){
      if(any(sapply(object$grid, function(x) is.character(x) || is.factor(x))))
        stop("Scale can only be applied to numeric inputs", call. = FALSE)
      dat <- data.frame(y = y, scale(X))
    }else{
      dat <- data.frame(y, as.data.frame(sapply(X, function(x) as.factor(as.character(x)))))
    }

    if(missing(formula) || formula == 1L){
      frml <- paste("y ~", paste(names(X), collapse = "+"))
    }else{
      if(is.numeric(formula)){
        frml0 <- paste(names(X), collapse = "+")
        if(formula == 2L){
          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^2")
        }
        if(formula == 3L){
          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^3")
        }
        if(formula == 4L){
          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^4")
        }
        if(formula != 2L && formula != 3L && formula != 4L)
          stop("When 'formula' is a number it needs to be either 2, 3 or 4", call. = FALSE)
      }else{
        if(attr(stats::terms(formula), "response") == 0){
          tt <- stats::terms(formula)
          frml <- stats::reformulate(attr(tt, "term.labels"), response = "y")
        }else{
          frml <- formula
        }
      }
    }

    fit <- stats::lm(formula = frml, data = dat, na.action = "na.omit")
    if(inherits(fit, "try-error")) next
    w <- fit$weights
    ssr <- sum(if (is.null(w)) fit$residuals^2 else w * fit$residuals^2)
    mss <- sum(if (is.null(w)) fit$fitted.values^2 else w * fit$fitted.values^2)
    if(ssr < 1e-10 * mss){
      cat("Variable:", nms.resp.var[.i], "\n")
      warning("ANOVA F-tests on an essentially perfect fit are unreliable",
              immediate. = TRUE)
    }
    sfit <- as.matrix(stats::anova(fit, ...))
    wrn <- warnings()
    kable.caption <- paste("Variable:", nms.resp.var[.i])
    pmat <- matrix(ncol = 2, nrow = length(labels(fit)) + 1)
    row.names(pmat) <- row.names(sfit)
    pmat[,1] <- sfit[,2]
    pmat[,2] <- sfit[,2] / sum(sfit[,2]) * 100
    colnames(pmat) <- c("SS", "SI (%)")
    pmatd <- as.data.frame(pmat)
    pmatd <- pmatd[order(pmatd$SS, decreasing = TRUE),]
    if(verbose){
      print(knitr::kable(pmatd, caption = kable.caption, digits = 0))
      cat("\n")
    }

    ansi <- data.frame(input = row.names(pmatd), SI = round(pmatd[, "SI (%)"], 1))
    names(ansi) <- c("input", paste(nms.resp.var[.i], "SI (%)"))
    if(.j == 0){
      ans <- ansi
    }else{
      ans <- merge(ans, ansi)
    }

    .j <- .j + 1
  }
  if(.j == 0) return("No variables reported. Are they all constant?")

  ## Need to reorder (on average)
  rmns <- rowMeans(ans[, -1, drop = FALSE])
  ans <- ans[order(rmns, decreasing = TRUE), ]
  invisible(ans)
}

#' @rdname sens_apsimx
#' @description Print method for an object of class \sQuote{sens_apsim}
#' @param x object of class \sQuote{sens_apsim}
#' @param ... additional arguments (none used at the moment)
#' @param variables whether to print APSIM output variables (default is FALSE)
#' @param summary whether to print the full summary of the grid simulations (default is FALSE)
#' @return compact printing
#' @export
#'
print.sens_apsim <- function(x, ..., variables = FALSE, summary = FALSE){

  ### Print parameters in grid
  cat("Grid dimensions (rows columns):", dim(x$grid), "\n")
  ### Grid names
  cat("Grid names:", names(x$grid), "\n")
  ### Print APSIM output variables
  output.variables <- setdiff(names(x$grid.sims), names(x$grid))
  cat("Number of output variables:", length(output.variables), "\n")
  ### Summary function
  sfun <- attr(x$grid.sims, "summary")
  cat("Applied summary function:", sfun, "\n")
  cat("Objects:", names(x), "\n")
  if(variables) print(output.variables)
  ### Print data.frame summary
  if(summary) print(summary(x$grid.sims))

}

#'
#' Simple optimization for APSIM Classic
#' 
#' * This function assumes that you want to optimize parameters which are stored in
#' an auxiliary XML file. These are typically crop or cultivar specific parameters.
#' However, it is possible to optimize parameters present in the main simulation
#' file.
#' 
#' * Only one observation per day is allowed in the data.
#' 
#' * Given how APSIM Classic works, this can only be run when the main simulation
#' file is in the current directory and the crop file (or XML) 
#' should be in the same directory as the main simulation.
#' 
#' * The initial values for the optimization should be the ones in the stored
#' crop parameter file.
#' 
#' * It is suggested that you keep a backup of the original file. This function
#' will edit and overwrite the file during the optimization. 
#' 
#' * When you use the parm.vector.index you cannot edit two separate elements of
#' a vector at the same time. This should be used to target a single element of 
#' a vector only.
#' 
#' * Internally, the optimization is done around the scaled value of the initial parameter
#' values. A value of 1 would correspond to the inital value of the parameter. 
#' The \sQuote{lower} and \sQuote{upper} (or \sQuote{ub} and \sQuote{lb}) are also scaled 
#' to the initial values of the parameters. So, for example, if your initial value is 20 
#' and you provide an upper bound of 5, it means that the actual upper value that you are allowing for is 100. 
#' 
#' @title Optimize parameters in an APSIM simulation
#' @name optim_apsim
#' @rdname optim_apsim
#' @description It is a wrapper for running APSIM and optimizing parameters using \code{\link[stats]{optim}}
#' @param file file name to be run (the extension .apsim is optional)
#' @param src.dir directory containing the .apsim file to be run (defaults to the current directory)
#' @param crop.file name of auxiliary xml file where parameters are stored. If this is missing, it is 
#'                  assumed that the parameters to be edited are in the main simulation file.
#' @param parm.paths absolute paths of the coefficients to be optimized. 
#'             It is recommended that you use \code{\link{inspect_apsim}} or \code{\link{inspect_apsim_xml}}  for this.
#' @param data data frame with the observed data. By default is assumes there is a 'Date' column for the index.
#' @param type Type of optimization. For now, \code{\link[stats]{optim}} and, if available, \code{\link[nloptr]{nloptr}} or 
#' \sQuote{mcmc} through \code{\link[BayesianTools]{runMCMC}}. Option \sQuote{ucminf} uses the \code{\link[ucminf]{ucminf}} function.
#' @param weights Weighting method or values for computing the residual sum of squares (see Note). 
#' @param index Index for filtering APSIM output. \sQuote{Date} is currently used. (I have not tested how well it works using anything other than Date).
#' @param parm.vector.index Index to optimize a specific element of a parameter vector. At the moment it is
#' possible to only edit one element at a time. This is because there is a conflict when generating multiple
#' elements in the candidate vector for the same parameter.
#' @param xml.parm optional logical vector used when optimizing parameters which are both in the .apsim file and in the \sQuote{crop.file}.
#' If \sQuote{crop.file} is missing it is assumed that the paramters to be optimized are in the .apsim file. If \sQuote{crop.file} is
#' not missing it is assumed that they are in the \sQuote{crop.file}. If the parameters are in both, this needs to be specified in 
#' this argument.
#' @param ... additional arguments to be passed to the optimization algorithm. If you want confidence intervals, then include \sQuote{hessian = TRUE}.
#' @note When computing the objective function (residual sum-of-squares) different variables are combined.
#' It is common to weight them since they are in different units. If the argument weights is not supplied
#' no weighting is applied. It can be \sQuote{mean}, \sQuote{var} or a numeric vector of appropriate length.
#' @return object of class \sQuote{optim_apsim}, but really just a list with results from optim and additional information.
#' @export
#' 

optim_apsim <- function(file, src.dir = ".", 
                        crop.file, parm.paths, data, 
                        type = c("optim", "nloptr","mcmc", "ucminf"), 
                        weights, index = "Date",
                        parm.vector.index,
                        xml.parm,
                        ...){
  
  .check_apsim_name(file)
  
  if(src.dir != ".") stop("At the moment it is not possible \n
                          to change the source directory.")

  ## This might offer suggestions in case there is a typo in 'file'
  file.names <- dir(path = src.dir, pattern = ".apsim$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsim files in the specified directory to run.")
  }
  
  file <- match.arg(file, file.names)
  
  file.name.path <- file.path(src.dir, file)
  
  ## optimization type
  type <- match.arg(type)
  
  if(type == "nloptr"){
    if(!requireNamespace("nloptr", quietly = TRUE)){
      warning("The nloptr package is required for this optimization method")
      return(NULL)
    }
  }
  
  if(type == "mcmc"){
    if(!requireNamespace("BayesianTools", quietly = TRUE)){
      warning("The BayesianTools package is required for this method.")
      return(NULL)
    }
  }
  
  if(type == "ucminf"){
    if(!requireNamespace("ucminf", quietly = TRUE)){
      warning("The ucminf package is required for this method.")
      return(NULL)
    }
  }
  
  ## Data needs to be a data.frame
  if(!inherits(data, "data.frame"))
    stop("Object 'data' should be of class 'data.frame'.", call. = FALSE)
  
  ## Index can now potentially reference two columns
  datami <- data[,-which(names(data) %in% index), drop = FALSE]
  if(any(grepl("Date", index))) data$Date <- as.Date(data$Date)
  
  ## Setting up weights
  if(missing(weights)){
    weights <- rep(1, ncol(datami))
  }else{
    if(weights == "mean"){
      weights <- abs(1 / apply(datami, 2, mean, na.rm = TRUE))  
    }else{
      if(weights == "var"){
        weights <- 1 / apply(datami, 2, var, na.rm = TRUE)    
      }else{
        if(length(weights) != ncol(datami))
          stop("Weights not of correct length")
        if(!is.numeric(weights))
          stop("Weights should be numeric")
      } 
    } 
  }
  
  if(missing(parm.vector.index)){
    parm.vector.index <- rep(-1, length(parm.paths))
  }else{
    if(length(parm.vector.index) != length(parm.paths))
      stop("parm.vector.index should have length equal to parm.paths") 
    if(!is.numeric(parm.vector.index))
      stop("parm.vector.index should be numeric")
  }
  
  ## What this does, is pick the crop.file to be edited when it is not missing
  if(!missing(crop.file)){
    aux.file <- crop.file
    if(missing(xml.parm)){
      cfile <- rep(TRUE, length(parm.paths))  
    }else{
      cfile <- xml.parm
    } 
  }else{
    aux.file <- file
    if(missing(xml.parm)){
      cfile <- rep(FALSE, length(parm.paths))
    }else{
      cfile <- xml.parm
    } 
  }
  
  if(!is.logical(cfile) || length(cfile) != length(parm.paths))
    stop("xml.parm should be a logical of length equal to parm.paths")
  
  ## Retrieve initial value vectors for the auxiliary file
  iaux.parms <- vector("list", length = length(parm.paths))
  length.aux.parms <- numeric(length(parm.paths))
  names(iaux.parms) <- parm.paths
    
  for(i in seq_along(parm.paths)){
    
    if(cfile[i]){
      aux.xml <- xml2::read_xml(file.path(src.dir, aux.file))    
    }else{
      aux.xml <- xml2::read_xml(file.path(src.dir, file))
    }
    
    xml.node <- xml2::xml_find_first(aux.xml, parm.paths[i])
    length.xml.node0 <- xml2::xml_length(xml.node)
    length.xml.node1 <- length(xml2::xml_text(xml.node))
    length.xml.node <- max(length.xml.node0, length.xml.node1)
    length.aux.parms[i] <- length.xml.node 
    if(length.xml.node == 1){
      aux.parm.text <- xml2::xml_text(xml.node)
      aux.parm.value <- as.numeric(strsplit(aux.parm.text, "\\s+")[[1]])      
    }else{
      aux.parm.text <- xml2::xml_text(xml2::xml_children(xml.node))
      aux.parm.value <- as.numeric(aux.parm.text)
    }
    iaux.parms[[i]] <- aux.parm.value
  }    
  
  obj_fun <- function(cfs, parm.paths, data, file, aux.file, 
                      iaux.parms, weights, index, 
                      parm.vector.index,
                      xml.parm){
    
    ## Need to edit the parameters in the crop file or the main simulation
    for(i in seq_along(cfs)){
      ## Retrieve the vector of current parameters
      if(parm.vector.index[i] <= 0){
        if(length.aux.parms[i] == 1){
          mparm <- paste(iaux.parms[[i]] * cfs[i], collapse = " ")    
        }else{
          mparm <- iaux.parms[[i]] * cfs[i] 
        }
      }else{
        pvi <- parm.vector.index[i]
        iaux.parms[[i]][pvi] <- iaux.parms[[i]][pvi] * cfs[i]
        mparm <- paste(iaux.parms[[i]], collapse = " ")    
      }
      
      ## Edit the specific parameters with the corresponding values
      if(xml.parm[i]){
        ## Here I'm editing an auxiliary file ending in .xml
        edit_apsim_xml(file = aux.file, 
                       src.dir = src.dir,
                       parm.path = parm.paths[i],
                       overwrite = TRUE,
                       value = as.character(mparm),
                       verbose = FALSE)        
      }else{
        ## Here I'm editing the main simulation file .apsim
        edit_apsim(file = file, 
                   node = "Other",
                   src.dir = src.dir,
                   parm.path = parm.paths[i],
                   overwrite = TRUE,
                   value = as.character(mparm),
                   verbose = FALSE)        
      }
    }
    
    ## Run simulation  
    sim <- try(apsim(file = file, src.dir = src.dir,
                     silent = TRUE, cleanup = TRUE, value = "report"),
               silent = TRUE)
    
    if(inherits(sim, "try-error")) return(NA)
    
    if(!inherits(sim, "data.frame"))
      stop("the apsim simulation should return an object of class 'data.frame'. \n 
           Check that the output does simplify to a 'data.frame' instead of a 'list'.")
    
    if("outfile" %in% names(sim) && length(index) == 1)
      stop("If you have multiple simulations, you should modify the index argument. \n
           It is possible that 'index = c('outfile', 'Date')' might work.")
    
    ## Only keep those columns with corresponding names in the data
    ## and only the dates that match those in 'data'
    if(!all(names(data) %in% names(sim))) 
      stop("names in 'data' do not match names in simulation")
    
    if(length(index) == 1 && index[1] == "Date"){
      sim.s <- subset(sim, sim$Date %in% data[[index]], select = names(data))  
      
      sim.s <- sim.s[order(sim.s[, index]),]
      data <- data[order(data[, index]),]
      
      if(!all(sim.s[[index]] == data[[index]]))
        stop(paste("simulations and data for", index, "do not match"))   
      
    }else{
      if(!is.null(data$outfile)) data$outfile <- as.factor(data$outfile)
      if(!is.null(sim$outfile)) sim$outfile <- as.factor(sim$outfile)
      sim.s0 <- merge(sim, subset(data, select = index), by = index)  
      sim.s <- subset(sim.s0, select = names(data))
      
      ## However, they need to be in the exact same order
      sim.s <- sim.s[order(sim.s[, index[1]], sim.s[ ,index[2]]),]
      data <- data[order(data[, index[1]], data[, index[2]]),]
      
      if(!all(sim.s[[index[1]]] == data[[index[1]]]))
        stop(paste("simulations and data for", index[1], "do not match"))        
      
      if(!all(sim.s[[index[2]]] == data[[index[2]]]))
        stop(paste("simulations and data for", index[2], "do not match"))   
    }

    if(ncol(sim.s) != ncol(data)){
      cat("Number of columns in data", ncol(data), "\n")
      cat("Number of columns in subset simulation", ncol(sim.s), "\n")
      stop("Number of columns in data does not equal number of columns in simulation")
    }
        
    if(nrow(sim.s) == 0L) stop("Something went wrong. No rows selected in simulations")
    ## Assuming they are aligned, get rid of the 'index' column
    sim.s <- sim.s[,-which(names(sim.s) %in% index)]
    data <- data[,-which(names(data) %in% index)]
    ## Now I need to calculate the residual sum of squares
    ## For this to work all variables should be numeric
    diffs <- as.matrix(data) - as.matrix(sim.s)
    rss <- sum(weights * colSums(diffs^2, na.rm = TRUE)) 
    return(log(rss))
  }
  
  ## Pre-optimized RSS
  pre.lrss <- obj_fun(cfs = rep(1, length(parm.paths)),
                      parm.paths = parm.paths,
                      data = data,
                      file = file,
                      aux.file = aux.file,
                      iaux.parms = iaux.parms,
                      weights = weights,
                      index = index,
                      parm.vector.index = parm.vector.index,
                      xml.parm = cfile)
  ## optimization
  if(type == "optim"){
    op <- stats::optim(par = rep(1, length(parm.paths)), 
                       fn = obj_fun, 
                       parm.paths = parm.paths, 
                       data = data, 
                       file = file,
                       aux.file = aux.file, 
                       iaux.parms = iaux.parms,
                       weights = weights,
                       index = index,
                       parm.vector.index = parm.vector.index,
                       xml.parm = cfile,
                       ...)    
  }
  
  if(type == "ucminf"){
    op <- ucminf::ucminf(par = rep(1, length(parm.paths)), 
                         fn = obj_fun, 
                         parm.paths = parm.paths, 
                         data = data, 
                         file = file,
                         aux.file = aux.file, 
                         iaux.parms = iaux.parms,
                         weights = weights,
                         index = index,
                         parm.vector.index = parm.vector.index,
                         xml.parm = cfile,
                         ...)    
  }
  
  if(type == "nloptr"){
    op <- nloptr::nloptr(x0 = rep(1, length(parm.paths)),
                         eval_f = obj_fun,
                         parm.paths = parm.paths, 
                         data = data, 
                         file = file, 
                         aux.file = aux.file, 
                         iaux.parms = iaux.parms,
                         weights = weights,
                         index = index,
                         parm.vector.index = parm.vector.index,
                         xml.parm = cfile,
                         ...)
    op$par <- op$solution
    op$value <- op$objective 
    op$convergence <- op$status
  }
  
  if(type == "mcmc"){
    ## Setting defaults
    datami.sds <- apply(datami, 2, sd)
    mcmc.args <- list(...)
    if(is.null(mcmc.args$lower)) lower <- rep(0, length(iaux.parms) + ncol(datami))
    if(is.null(mcmc.args$upper)) upper <- c(rep(2, length(iaux.parms)), datami.sds * 100)
    if(is.null(mcmc.args$sampler)) sampler <- "DEzs"
    if(is.null(mcmc.args$parallel)) mcmc.args$parallel <- FALSE
    if(is.null(mcmc.args$settings)) stop("runMCMC settings are missing with no default")

    cfs <- c(rep(1, length(iaux.parms)), apply(datami, 2, sd))
    
    ## Create environment with objects
    assign('.file', file, mcmc.apsim.env)
    assign('.aux.file', aux.file, mcmc.apsim.env)
    assign('.src.dir', src.dir, mcmc.apsim.env)
    assign('.parm.paths', parm.paths, mcmc.apsim.env)
    assign('.data', data, mcmc.apsim.env)
    assign('.iaux.parms', iaux.parms, mcmc.apsim.env)
    assign('.index', index, mcmc.apsim.env)
    assign('.parm.vector.index', parm.vector.index, mcmc.apsim.env)
    assign('.xml.parm', xml.parm, mcmc.apsim.env)
    
    ## Pre-optimized log-likelihood
    pll <- log_lik2(cfs)
    
    cat("Pre-optimized log-likelihood", pll, "\n")
    
    nms <- c(names(iaux.parms), paste0("sd_", names(datami)))
    bayes.setup <- BayesianTools::createBayesianSetup(log_lik2, 
                                                      lower = lower,
                                                      upper = upper,
                                                      names = nms,
                                                      parallel = mcmc.args$parallel)
    
    op.mcmc <- BayesianTools::runMCMC(bayes.setup, 
                                      sampler = sampler, 
                                      settings = mcmc.args$settings)
    return(op.mcmc)

  }
  
  ## pre.rss are the pre-optimized residual sum of squares
  ## op$value are the optimized weighted residual sum of squares in the log scale
  ## The next line is only true when optimizing one single variable
  ## logLik <- 0.5 * ( - N * (log(2 * pi) + 1 - log(N) + log(op$value))

  ans <- structure(list(pre.rss = exp(pre.lrss), 
                        post.rss = exp(op$value), ## This is weighted
                        weights = weights,
                        iaux.parms = iaux.parms, op = op, n = nrow(data),
                        parm.vector.index = parm.vector.index),
                   class = "optim_apsim")

  return(ans)
}

#' @rdname optim_apsim
#' @description Friendly printing of optim_apsim
#' @param x object of class \sQuote{optim_apsim}
#' @param ... additional arguments (none used at the moment)
#' @param digits number of digits to round up the output
#' @param level confidence level (default 0.95)
#' @return prints to console
#' @export
#' 
print.optim_apsim <- function(x, ..., digits = 3, level = 0.95){

  cat("Initial values: \n")
  
  for(i in seq_along(x$iaux.parms)){
    
    cat("\t Parameter path: ", names(x$iaux.parms)[i], "\n")
    if(x$parm.vector.index[i] > 0) cat("Vector index: ", x$parm.vector.index[i], "\n")
    cat("\t Values: ", x$iaux.parms[[i]], "\n")
  }   
  cat("\t Pre-optimized (weighted) RSS: ", x$pre.rss, "\n")
  cat("Optimized values: \n")
  
  for(i in seq_along(x$iaux.parms)){
    
    cat("\t Parameter path: ", names(x$iaux.parms)[i], "\n")
    if(x$parm.vector.index[i] > 0) cat("Vector index: ", x$parm.vector.index[i], "\n")
    if(x$parm.vector.index[i] <= 0){
      cat("\t Values: ", round(x$iaux.parms[[i]] * x$op$par[i], digits), "\n")  
    }else{
      pvi <- x$parm.vector.index[i]
      x$iaux.parms[[i]][pvi] <- x$iaux.parms[[i]][pvi] * x$op$par[i]
      cat("\t Values: ", round(x$iaux.parms[[i]], digits), "\n")        
    }
    
    if(!is.null(x$op$hessian)){
      ## I actually found this way of computing SE here:
      ## https://www.researchgate.net/post/In_R_how_to_estimate_confidence_intervals_from_the_Hessian_matrix
      degf <- x$n - length(x$iaux.parms) ## Degrees of freedom
      par.se <- sqrt((2 * (1 / (x$op$hessian[i,i])) * x$iaux.parms[[i]]^2) / degf)
      qTT <- -1 * stats::qt((1 - level) * 0.5, degf) ## t statistic
      cat("\t CI level: ", level, "t: ", qTT, " SE:", round(par.se * x$iaux.parms[[i]], digits),"\n")
      if(x$parm.vector.index[i] <= 0){
        cat("\t Lower: ", round((x$op$par[i] - qTT * par.se) * x$iaux.parms[[i]], digits))
        cat("\t Upper: ", round((x$op$par[i] + qTT * par.se) * x$iaux.parms[[i]], digits),"\n")  
      }else{
        pvi <- x$parm.vector.index[i]
        cat("\t Lower: ", round((x$op$par[i] - qTT * par.se) * x$iaux.parms[[i]][pvi], digits))
        cat("\t Upper: ", round((x$op$par[i] + qTT * par.se) * x$iaux.parms[[i]][pvi], digits),"\n")        
      }
    }
  }   
  cat("\t Optimized (weighted) RSS: ", x$post.rss, "\n")
  cat("Convergence:", x$op$convergence,"\n")
}
  
## Log-likelihood for mcmc method
log_lik2 <- function(.cfs){

  .file <- get('.file', envir = mcmc.apsim.env)
  .aux.file <- get('.aux.file', envir = mcmc.apsim.env)
  .src.dir <- get('.src.dir', envir = mcmc.apsim.env)
  .parm.paths <- get('.parm.paths', envir = mcmc.apsim.env)
  .data <- get('.data', envir = mcmc.apsim.env)
  .iaux.parms <- get('.iaux.parms', envir = mcmc.apsimx.env)
  .index <- get('.index', envir = mcmc.apsim.env)
  .parm.vector.index <- get('.parm.vector.index', envir = mcmc.apsim.env)
  .xml.parm <- get('.xml.parm', envir = mcmc.apsim.env)
  
  for(i in seq_along(.cfs)){
    ## Retrieve the vector of current parameters
    if(.parm.vector.index[i] <= 0){
      mparm <- paste(.iaux.parms[[i]] * .cfs[i], collapse = " ")  
    }else{
      pvi <- .parm.vector.index[i]
      .iaux.parms[[i]][pvi] <- .iaux.parms[[i]][pvi] * .cfs[i]
      mparm <- paste(.iaux.parms[[i]], collapse = " ")    
    }
    
    ## Edit the specific parameters with the corresponding values
    if(.xml.parm[i]){
      ## Here I'm editing an auxiliary file ending in .xml
      edit_apsim_xml(file = .aux.file, 
                     src.dir = .src.dir,
                     parm.path = .parm.paths[i],
                     overwrite = TRUE,
                     value = as.character(mparm),
                     verbose = FALSE)        
    }else{
      ## Here I'm editing the main simulation file .apsim
      edit_apsim(file = .aux.file, 
                 node = "Other",
                 src.dir = .src.dir,
                 parm.path = .parm.paths[i],
                 overwrite = TRUE,
                 value = as.character(mparm),
                 verbose = FALSE)        
    }
  }
  
  ## Run simulation  
  sim <- try(apsim(file = .file, src.dir = .src.dir,
                   silent = TRUE, cleanup = TRUE, value = "report"),
             silent = TRUE)
  
  if(inherits(sim, "try-error")) return(NA)
  
  ## Only keep those columns with corresponding names in the data
  ## and only the dates that match those in 'data'
  if(!all(names(.data) %in% names(sim))) 
    stop("names in 'data' do not match names in simulation")

  if(length(.index) == 1){
    sim.s <- subset(sim, sim[, .index, drop = FALSE] %in% .data[[.index]], select = names(.data))  
    
    sim.s <- sim.s[order(sim.s[, .index[1]]),]
    .data <- .data[order(.data[, .index[1]]),]
    
    if(!all(sim.s[[.index[1]]] == .data[[.index[1]]]))
      stop(paste("simulations and data for", .index[1], "do not match"))   
    
  }else{
    if(!is.null(.data$outfile)) .data$outfile <- as.factor(.data$outfile)
    if(!is.null(sim$outfile)) sim$outfile <- as.factor(sim$outfile)
    sim.s0 <- merge(sim, subset(.data, select = .index), by = .index)  
    sim.s <- subset(sim.s0, select = names(.data))
    
    ## However, they need to be in the exact same order
    sim.s <- sim.s[order(sim.s[, .index[1]], sim.s[ ,.index[2]]),]
    .data <- .data[order(.data[, .index[1]], .data[, .index[2]]),]
    
    if(!all(sim.s[[.index[1]]] == .data[[.index[1]]]))
      stop(paste("simulations and data for", .index[1], "do not match"))        
    
    if(!all(sim.s[[.index[2]]] == .data[[.index[2]]]))
      stop(paste("simulations and data for", .index[2], "do not match"))   
  }
    
  if(nrow(sim.s) == 0L) stop("no rows selected in simulations")
  ## Assuming they are aligned, get rid of the 'index' column
  sim.s <- sim.s[,-which(names(sim.s) == .index)]
  .data <- .data[,-which(names(.data) == .index)]
  ## Now I need to calculate the residual sum of squares
  ## For this to work all variables should be numeric
  diffs <- as.matrix(.data) - as.matrix(sim.s)
  if(ncol(diffs) == 1){
    lls <- stats::dnorm(diffs[,1], sd = .cfs[length(.cfs)], log = TRUE)
    return(sum(lls))
  }else{
    Sigma <- diag(.cfs[(length(.iaux.parms) + 1):length(.cfs)])
    lls <- mvtnorm::dmvnorm(diffs, sigma = Sigma, log = TRUE)
    return(sum(lls))    
  }
}

#' @rdname optim_apsim
#' @description Variance-Covariance for an \sQuote{optim_apsim} object
#' @param object object of class \sQuote{optim_apsim}
#' @param ... additional arguments (none used at the moment)
#' @param scaled for now only return the scaled matrix
#' @note This in the scale of the optimized parameters which are
#' scaled to be around 1.
#' @return it returns the variance-covariance matrix for an object of class \sQuote{optim_apsim}.
#' @export
#' 
vcov.optim_apsim <- function(object, ..., scaled = TRUE){
  
  if(is.null(object$op$hessian)) stop("Hessian matrix not found")
  ## Hessian matrix
  hess <- object$op$hessian
  ## Correlation matrix
  cor.mat <- stats::cov2cor(solve(hess))
  ## Standard deviation vector
  sd.vec <- numeric(nrow(hess))
  degf <- object$n - nrow(hess)
  for(i in 1:nrow(hess)){
    sd.vec[i] <- sqrt((2 * 1/(hess[i,i]) * object$iaux.parms[[i]]^2) / degf)
  }
  ans <- t(sd.vec * cor.mat) * sd.vec
  return(ans)
}

#' @rdname optim_apsim
#' @description Parameter estimates for an \sQuote{optim_apsim} object
#' @param object object of class \sQuote{optim_apsim}
#' @param ... additional arguments (none used at the moment)
#' @param scaled whether to return the scaled or unscaled estimates
#' (TRUE in the optimized scale, FALSE in the original scale)
#' @return a numeric vector with the value of the parameter estimates.
#' @export
#' 
coef.optim_apsim <- function(object, ..., scaled = FALSE){
  
  ans <- numeric(length(object$op$par))
  
  for(i in seq_along(ans)){
    if(object$parm.vector.index[i] <= 0){
      ans[i] <- object$op$par[i] * object$iaux.parms[[i]]
    }else{
      pvi <- object$parm.vector.index[i]
      ans[i] <- object$op$par[i] * object$iaux.parms[[i]][pvi]
    }
  }

  if(scaled) ans <- object$op$par
  
  names(ans) <- names(object$iaux.parms)
  
  return(ans)
}

#' @rdname optim_apsim
#' @description Confidence intervals for parameter estimates for an \sQuote{optim_apsim} object
#' @param object object of class \sQuote{optim_apsim}
#' @param parm parameter to select (it can be a regular expression)
#' @param level confidence level (default is 0.95)
#' @param ... additional arguments (none used at the moment)
#' @return a matrix with lower and upper limits and the point estimate (coef)
#' @export
#' 

confint.optim_apsim <- function(object, parm, level = 0.95, ...){
  
  if(is.null(object$op$hessian)) 
    stop("Hessian is needed to calculate confidence intervals", call. = FALSE)

  if(level < 0 || level > 1)
    stop("level should be between 0 and 1", call. = FALSE)
  
  x <- object
  
  ans <- matrix(ncol = 3, nrow = length(x$iaux.parms))
  colnames(ans) <- c("lwr", "coef", "upr")
  rownames(ans) <- names(x$iaux.parms)

  for(i in seq_along(x$iaux.parms)){
    
    if(x$parm.vector.index[i] <= 0){
      ans[i, 2] <- x$iaux.parms[[i]] * x$op$par[i]
    }else{
      pvi <- x$parm.vector.index[i]
      x$iaux.parms[[i]][pvi] <- x$iaux.parms[[i]][pvi] * x$op$par[i]
      ans[i, 2] <- x$iaux.parms[[i]]
    }
    
    ## I actually found this way of computing SE here:
    ## That post references the Introduction to R section 11.7.2
    degf <- x$n - length(x$iaux.parms) ## Degrees of freedom
    par.se <- sqrt((2 * (1 / (x$op$hessian[i,i])) * x$iaux.parms[[i]]^2) / degf)
    qTT <- -1 * stats::qt((1 - level) * 0.5, degf) ## t statistic
      if(x$parm.vector.index[i] <= 0){
        ans[i, 1] <- (x$op$par[i] - qTT * par.se) * x$iaux.parms[[i]]
        ans[i, 3] <- (x$op$par[i] + qTT * par.se) * x$iaux.parms[[i]]
      }else{
        pvi <- x$parm.vector.index[i]
        ans[i, 1] <- (x$op$par[i] - qTT * par.se) * x$iaux.parms[[i]][pvi]
        ans[i, 3] <- (x$op$par[i] + qTT * par.se) * x$iaux.parms[[i]][pvi]
      }
  }
  
  if(!missing(parm)){
    w.parm <- grep(parm, names(x$iaux.parms))
    if(length(w.parm) == 0)
      stop("'parm' not found", call. = FALSE)
    ans <- ans[w.parm, , drop = FALSE]
  }
  return(ans)
}

#' Create an apsim environment for MCMC
#' 
#' @title Environment to store data for apsim MCMC
#' @description Environment which stores data for MCMC
#' @return This is an environment, so nothing to return.
#' @export
#' 
mcmc.apsim.env <- new.env(parent = emptyenv())

## This test will build from simple to more complex sensitivity analysis

require(apsimx)
packageVersion("apsimx")
require(ggplot2)
apsimx_options(warn.versions = FALSE)

run.sens.apsimx <- FALSE

tmp.dir <- tempdir()
setwd(tmp.dir)

## WheatRye example
if(run.sens.apsimx){
  
  ## Test the effect of changing RUE
  extd.dir <- "~/Dropbox/apsimx/inst/extdata"
  ## extd.dir <- "C:/Users/femiguez/Dropbox/apsimx/inst/extdata"
  file.copy(file.path(extd.dir, "WheatRye.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  
  inspect_apsimx("WheatRye.apsimx", node = "Report")
  
  edit_apsimx("WheatRye.apsimx", node = "Report",
              parm = "VariableNames",
              value = "[Wheat].Leaf.Transpiration",
              overwrite = TRUE)
  
  
  sim0 <- apsimx("WheatRye.apsimx")
  
  ggplot(data = sim0, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    geom_point()
  
  ggplot(data = sim0, aes(x = Date, y = Wheat.Leaf.Transpiration)) + 
    geom_point()
  
  inspect_apsimx_replacement("WheatRye.apsimx",
                             node = "Wheat", node.child = "Leaf",
                             node.subchild = "Photosynthesis",
                             node.subsubchild = "RUE")
  
  pp <- "Wheat.Leaf.Photosynthesis.RUE"
  
  start <- Sys.time()
  rues <- seq(0, 3, 0.25)
  res.dat <- NULL
  
  for(i in seq_along(rues)){
    
    edit_apsimx_replacement("WheatRye.apsimx",
                            node.string = pp,
                            parm = "FixedValue",
                            value = rues[i],
                            overwrite = TRUE,
                            verbose = FALSE)
    
    sim1 <- apsimx("WheatRye.apsimx")
    
    res.dat <- rbind(res.dat, data.frame(rue = rues[i], 
                                         wheat.aboveground.wt = mean(sim1$Wheat.AboveGround.Wt),
                                         wheat.leaf.lai = mean(sim1$Wheat.Leaf.LAI),
                                         wheat.leaf.transpiration = mean(sim1$Wheat.Leaf.Transpiration)))
  }
  end <- Sys.time()
  
  elapsed.time <- end - start
  
  ggplot(data = res.dat, aes(x = rue, y = wheat.aboveground.wt)) + 
    geom_point() + 
    geom_smooth(se = FALSE)
  
  ggplot(data = res.dat, aes(x = rue, y = wheat.leaf.lai)) + 
    geom_point() + 
    geom_smooth(se = FALSE)

  ggplot(data = res.dat, aes(x = rue, y = wheat.leaf.transpiration)) + 
    geom_point() + 
    geom_smooth(se = FALSE)
  
  ggplot(data = sim1, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    geom_point()
 
  ggplot(data = sim1, aes(x = Date, y = Wheat.Leaf.LAI)) + 
    geom_point()
  
  file.remove("WheatRye.apsimx")
  file.remove("Ames.met")
  file.remove("WheatRye.db")
  file.remove("WheatRye.db-shm")
  file.remove("WheatRye.db-wal")
}

## MaizeSoybean example
if(run.sens.apsimx){
  
  ## Test the effect of changing RUE
  extd.dir <- "~/Dropbox/apsimx/inst/extdata"
  file.copy(file.path(extd.dir, "MaizeSoybean.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  
  inspect_apsimx("MaizeSoybean.apsimx", node = "Report",
                 root = "SimulationMaize")
  
  edit_apsimx("MaizeSoybean.apsimx",
              root = "SimulationMaize",
              node = "Report",
              parm = "VariableNames",
              value = c("[Maize].AboveGround.Wt", "[Maize].Leaf.LAI", "[Maize].Leaf.Transpiration"),
              overwrite = TRUE)
  
  sim0 <- apsimx("MaizeSoybean.apsimx")
  
  ggplot(data = sim0, aes(x = Date, y = Maize.AboveGround.Wt)) + 
    geom_point()
  
  ggplot(data = sim0, aes(x = Date, y = Maize.Leaf.LAI)) + 
    geom_point()

  ggplot(data = sim0, aes(x = Date, y = Maize.Leaf.Transpiration)) + 
    geom_point()

  pp <- "Maize.Leaf.Photosynthesis.RUE"
  
  start <- Sys.time()
  rues <- seq(0, 3, 0.25)
  res.dat <- NULL
  
  for(i in seq_along(rues)){
    
    edit_apsimx_replacement("MaizeSoybean.apsimx",
                            node.string = pp,
                            parm = "FixedValue",
                            value = rues[i],
                            overwrite = TRUE,
                            verbose = FALSE)
    
    sim1 <- apsimx("MaizeSoybean.apsimx")
    
    res.dat <- rbind(res.dat, data.frame(rue = rues[i], 
                                         maize.aboveground.wt = mean(sim1$Maize.AboveGround.Wt, na.rm = TRUE),
                                         maize.leaf.lai = mean(sim1$Maize.Leaf.LAI, na.rm = TRUE),
                                         maize.leaf.transpiration = mean(sim1$Maize.Leaf.Transpiration, na.rm = TRUE)))
  }
  end <- Sys.time()
  elapsed.time <- end - start
  
  ggplot(data = res.dat, aes(x = rue, y = maize.aboveground.wt)) + 
    geom_point() + 
    geom_smooth(se = FALSE)
  
  ggplot(data = res.dat, aes(x = rue, y = maize.leaf.lai)) + 
    geom_point() + 
    geom_smooth(se = FALSE)
  
  ggplot(data = res.dat, aes(x = rue, y = maize.leaf.transpiration)) + 
    geom_point() + 
    geom_smooth(se = FALSE) 
  
  ## For the specific simulation
  ggplot(data = sim1, aes(x = Date, y = Maize.AboveGround.Wt)) + 
    geom_point()
  
  ggplot(data = sim1, aes(x = Date, y = Maize.Leaf.LAI)) + 
    geom_point()

  ggplot(data = sim1, aes(x = Date, y = Maize.Leaf.Transpiration)) + 
    geom_point()
  
  ### Clean up
  file.remove(dir(pattern = "MaizeSoybean"))
  file.remove("Ames.met")
}

if(run.sens.apsimx){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  ## Identify a parameter of interest
  ## In this case we want to know the impact of varying the fertilizer amount
  pp <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                       node = "Manager", parm = list("SowingFertiliser", 1))
  pp <- paste0(pp, ".Amount")
  ## Do we need to edit the Report?
  edit_apsimx("Wheat.apsimx", src.dir = tmp.dir, node = "Report",
              parm = "VariableNames", value = c("max of [Wheat].Leaf.LAI from 1-Jan to 31-Dec as MaximumLAI", 
                                                "sum of [Wheat].Leaf.Transpiration from 1-Jan to 31-Dec as AnnualLeafTranspiration"),
              overwrite = TRUE)
  ## Before the analysis
  sim0 <- apsimx("Wheat.apsimx", src.dir = tmp.dir)
  
  head(sim0)
  
  ggplot(data = sim0, aes(x = Date, y = MaximumLAI)) + 
    geom_point()
  
  ggplot(data = sim0, aes(x = MaximumLAI, y = Yield)) + 
    geom_point()
  
  ## For simplicity, we create a vector of fertilizer amounts (instead of sampling)
  start <- Sys.time()
  ferts <- seq(5, 300, length.out = 10)
  col.res <- NULL
  for(i in 1:10){
    
    edit_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                node = "Other", parm.path = pp,
                value = ferts[i],
                overwrite = TRUE)
    
    sim <- apsimx("Wheat.apsimx", src.dir = tmp.dir)
    col.res <- rbind(col.res, data.frame(fertilizer.amount = ferts[i], 
                                         wheat.aboveground.wt = mean(sim$Wheat.AboveGround.Wt, na.rm = TRUE),
                                         wheat.yield = mean(sim$Yield, na.rm = TRUE),
                                         wheat.maximum.lai = mean(sim$MaximumLAI, na.rm = TRUE),
                                         wheat.annual.leaf.transpiration = mean(sim$AnnualLeafTranspiration, na.rm = TRUE)))
  }
  end <- Sys.time()
  elapsed.time <- end - start
  
  ggplot(col.res, aes(x = fertilizer.amount, y = wheat.aboveground.wt)) + 
    geom_point()
  
  ggplot(col.res, aes(x = fertilizer.amount, y = wheat.yield)) + 
    geom_point()

  ggplot(col.res, aes(x = fertilizer.amount, y = wheat.maximum.lai)) + 
    geom_point()

  ggplot(col.res, aes(x = fertilizer.amount, y = wheat.annual.leaf.transpiration)) + 
    geom_point()
  
  ### Clean up
  file.remove(dir(pattern = "Wheat"))
  file.remove("Ames.met")

}

if(run.sens.apsimx){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Wheat.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  ## Identify a parameter of interest
  ## In this case we want to know the impact of varying the fertilizer amount
  ## and the plant population
  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".", 
                       node = "Manager", parm = list("SowingFertiliser", 1))
  pp1 <- paste0(pp1, ".Amount")
  
  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                       node = "Manager", parm = list("SowingRule1", 9))
  pp2 <- paste0(pp2, ".Population")
  
  grd <- expand.grid(parm1 = c(50, 100, 150), parm2 = c(100, 200, 300))
  names(grd) <- c("Fertiliser", "Population")
  
  start <- Sys.time()
  sns <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                     parm.paths = c(pp1, pp2),
                     grid = grd,
                     cores = 2L)
  end <- Sys.time()
  ### Time elapsed with 2 cores (Mac 2017): 1.158267 (minutes)
  ### Time elapsed with 1 core (Mac 2017): 1.921198 (minutes)
  ### Time elapsed with 2 cores (Mac 2021): 55.6 (seconds)
  
  summary(sns)
  summary(sns, select = "Wheat.AboveGround.Wt")
  summary(sns, select = "AboveGround")
  ## summary(sns, select = "AboveGround", formula = 2L)

  sns.nn <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                        parm.paths = c(pp1, pp2),
                        grid = grd, summary = "none",
                        cores = 2L)
  
  summary(sns.nn, select = "Wheat.AboveGround.Wt")
  
  ## it is possible to use the package sensitivity to design an experiment
  library(sensitivity)
  X.mrrs <- morris(factors = c("Fertiliser", "Population"), r = 3, 
                     design = list(type = "oat", levels = 3, grid.jump = 1), 
                     binf = c(5, 50), bsup = c(200, 300))
  
  sns2 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                      parm.paths = c(pp1, pp2),
                      grid = X.mrrs$X,
                      cores = 2L)
  ## These are the sensitivity results for AboveGround.Wt only
  sns.res.ag <- tell(X.mrrs, sns2$grid.sims$Wheat.AboveGround.Wt)
  sns.res.ag
  plot(sns.res.ag)
  
  ### Clean up
  file.remove(dir(pattern = "Wheat"))
  file.remove(dir(pattern = "Ames.met"))
  
}

#### Create Classic example ####
if(run.sens.apsimx){
  
  apsim_options(warn.versions = FALSE)
  ex.dir <- auto_detect_apsim_examples()
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Millet.apsim"), ".")  
  file.copy(file.path(ex.dir, "MetFiles", "Goond.met"), ".")

  pp1 <- inspect_apsim("Millet.apsim", src.dir = tmp.dir, 
                        node = "Manager", 
                       parm = list("Sow", 2),
                       print.path = TRUE)
  
  pp2 <- inspect_apsim("Millet.apsim", src.dir = tmp.dir, 
                       node = "Manager", 
                       parm = list("Sow", 5), 
                       print.path = TRUE)

  grd <- expand.grid(parm1 = c("15-nov", "1-dec", "15-dec"), parm2 = c(5, 7, 9))
  names(grd) <- c("date", "density")    
  
  edit_apsim("Millet.apsim", node = "Clock",
             parm = "start_date", value = "01/01/1980",
             overwrite = TRUE)
  
  sim0 <- apsim("Millet.apsim")
  
  sns <- sens_apsim("Millet.apsim",
                     parm.paths = c(pp1, pp2),
                     grid = grd)
  
  summary(sns)
  summary(sns, select = "millet_biomass")
  
  sns2 <- sens_apsim("Millet.apsim",
                    parm.paths = c(pp1, pp2),
                    grid = grd,
                    summary = "none")
  
  sns2$grid.sims[1:5, 1:5]  
  
}

#### Testing the cores argument ####

run.sens.apsimx.cores <- FALSE

if(run.sens.apsimx.cores){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Wheat.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  ## Identify a parameter of interest
  ## In this case we want to know the impact of varying the fertilizer amount
  ## and the plant population
  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".", 
                        node = "Manager", parm = list("SowingFertiliser", 1))
  pp1 <- paste0(pp1, ".Amount")
  
  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                        node = "Manager", parm = list("SowingRule1", 9))
  pp2 <- paste0(pp2, ".Population")
  
  grd <- expand.grid(parm1 = c(50, 100, 150), parm2 = c(100, 200, 300))
  names(grd) <- c("Fertiliser", "Population")
  
  ## dim(grd) ## 9 simulations
  start <- Sys.time()
  sns <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                     parm.paths = c(pp1, pp2),
                     grid = grd)
  (Sys.time() - start)
  ## This takes 1.55 minutes (Mac)
  ## This now (Jan 2024) takes 2.14 minutes (Mac)
  ## Mac 2017 (Aug 2024): 1.800195
  ## Mac 2021 (Mar 2025): 1.5 minutes
  summary(sns)
  summary(sns, select = "Wheat.AboveGround.Wt", scale = TRUE)
  summary(sns, select = "AboveGround", scale = TRUE)
  
  start <- Sys.time()
  sns.c2 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                        parm.paths = c(pp1, pp2),
                        grid = grd, cores = 2L)
  (Sys.time() - start)
  ## This takes 1.29 minutes (Jan 2024)
  ## Windows: NA
  ## Mac 2017 (Aug 2024): 1.319598
  ## Mac 2021 (Mar 2025): 54 seconds 
  ## Are they the same?
  (diff.sns.vs.sns.c2 <- sum(colSums(sns$grid.sims - sns.c2$grid.sims)))
  
  if(abs(diff.sns.vs.sns.c2) > 0.001)
    stop("Simulations with 2 cores do not match")
  
  start <- Sys.time()
  sns.c4 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                        parm.paths = c(pp1, pp2),
                        grid = grd, cores = 4)
  (Sys.time() - start)
  ## Mac 2017 (Aug 2024) takes: 1.161401 minutes
  ## Mac 2021 (Mar 2025) takes: 33.9 seconds
  (diff.sns.vs.sns.c4 <- sum(colSums(sns$grid.sims - sns.c4$grid.sims)))
  
  if(abs(diff.sns.vs.sns.c4) > 0.001)
    stop("Simulations with 4 cores do not match")

  #### Summary = 'none' and 1L core
  start <- Sys.time()
  sns.s0 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                        parm.paths = c(pp1, pp2),
                        summary = "none",
                        grid = grd)
  (Sys.time() - start)
  
  start <- Sys.time()
  sns.s0.c2 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                           parm.paths = c(pp1, pp2),
                           summary = "none",
                           grid = grd,
                           cores = 2L)
  (Sys.time() - start)
  ### The difference in times here are: Mac 2017 (1 core) - 1.63 vs. (2 cores) - 1.1
  ### The difference in times here are: Mac 2021 (1 core) - 1.52 vs. (2 cores) - 54.4 seconds
  ## library(arsenal) 
  
  ## Data.frames are not identical, but summary results are
  ## cm1 <- arsenal::comparedf(sns.s0$grid.sims, sns.s0.c2$grid.sims)
  sns.s0$grid.sims[78, "Fertiliser"]
  sns.s0.c2$grid.sims[78, "Fertiliser"]
  ## The results are the same, but in different order
  
  summary(sns.s0, select = "AboveGround.Wt", scale = TRUE)
  summary(sns.s0.c2, select = "AboveGround.Wt", scale = TRUE)
  
  summary(sns.s0, select = "AboveGround.N", scale = TRUE)
  summary(sns.s0.c2, select = "AboveGround.N", scale = TRUE)
  
  summary(sns.s0, select = "Yield", scale = TRUE)
  summary(sns.s0.c2, select = "Yield", scale = TRUE)
  
  ### What if I reorder the simulations?
  snsd1 <- sns.s0$grid.sims
  snsd2 <- sns.s0.c2$grid.sims

  snsd1.o <- snsd1[order(snsd1$Fertiliser, snsd1$Population, snsd1$Date), ]
  snsd2.o <- snsd2[order(snsd2$Fertiliser, snsd2$Population, snsd2$Date), ]
  ##cmpd12 <- arsenal::comparedf(snsd2.o, snsd2.o)   
  
  ### Clean up
  file.remove(dir(pattern = "Wheat"))
  file.remove(dir(pattern = "Ames.met"))

}

#### Testing passing of soil profiles -----

run.sens.apsimx.soils <- FALSE

if(run.sens.apsimx.soils){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Wheat.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  ## Identify a parameter of interest
  ## In this case we want to know the impact of varying the fertilizer amount
  ## and the plant population
  pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = ".", 
                        node = "Manager", parm = list("SowingFertiliser", 1))
  pp1 <- paste0(pp1, ".Amount")
  
  pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                        node = "Manager", parm = list("SowingRule1", 9))
  pp2 <- paste0(pp2, ".Population")
  
  grd <- expand.grid(parm1 = c(50, 100, 150), parm2 = c(100, 200, 300), parm3 = c(1, 2))
  names(grd) <- c("Fertiliser", "Population", "soil.profile")
  
  inspect_apsimx("Wheat.apsimx", node = "Other", parm = list(1, 1, 5, 0), print.path = TRUE)

  ### Getting soils
  sps <- get_ssurgo_soil_profile(lonlat = c(-93, 42), nsoil = 2, fix = TRUE)
  
  plot(sps[[1]], property = "water")
  plot(sps[[1]], property = "Carbon")
  
  plot(sps[[2]], property = "water")
  plot(sps[[2]], property = "Carbon")

  ## Base simulation
  sim0 <- apsimx("Wheat.apsimx", src.dir = tmp.dir)
  
  start <- Sys.time()
  sns <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                     parm.paths = c(pp1, pp2, 'soil.profile'),
                     soil.profiles = sps,
                     grid = grd)
  (Sys.time() - start) 
  ## Taking 3.32 minutes (Mac 2017 - Aug 2024)
  ## Taking 3.1 minutes (Mac 2021 - Mar 2025)
  
  snsd <- sns$grid.sims
  
  ggplot(data = snsd, aes(x = Fertiliser, y = Yield, color = as.factor(soil.profile))) + 
    facet_wrap(~ Population) + 
    geom_line()

  ggplot(data = snsd, aes(x = Fertiliser, y = Wheat.AboveGround.Wt, color = as.factor(soil.profile))) + 
    facet_wrap(~ Population) + 
    geom_line()
  
  ### Compare soil profiles
  cmps <- compare_apsim_soil_profile(sps[[1]], sps[[2]])
  
  plot(cmps, plot.type = "depth", soil.var = c("DUL"))
  plot(cmps, plot.type = "depth", soil.var = c("LL15")) ## Should implement 'water'
  plot(cmps, plot.type = "depth", soil.var = "Carbon")

  ### Clean up
  file.remove("Wheat.apsimx")
  file.remove("Ames.met")
}

#### Multiple soils simulation ----
### I need to think more about the additional tests that I need to 
### to ensure that the feature of cycling through soils works
### Why did I think that there can be more than one column of soils that
### Need to be replaced? I think it is because of multiple simulations
### In one simulation run I could need to replace the soil of several 'roots'
### How do I do this?

run.multiple.soils.sens.apsimx <- FALSE

if(run.multiple.soils.sens.apsimx){
  
  ### Is AgPasture the best example of a multiple simulation file?
  extd.dir <- system.file("extdata", package = "apsimx")  
  file.copy(file.path(extd.dir, "MaizeSoybean.apsimx"), ".")
  file.copy(file.path(extd.dir, "Ames.met"), ".")
  
  inspect_apsimx("MaizeSoybean.apsimx", ".", node = "Other",
                 parm = 2)
  
  inspect_apsimx("MaizeSoybean.apsimx", ".", node = "Other",
                 parm = list(1, 2, 5, 0))
  
  check_apsimx("MaizeSoybean.apsimx", root = "SimulationSoybean")
  check_apsimx("MaizeSoybean.apsimx", root = "SimulationMaize")
  
  sim0 <- apsimx("MaizeSoybean.apsimx")

  dir()
  dim(sim0)
  
  ggplot(data = sim0[sim0$SimulationName == "SimulationMaize", ], 
         aes(x = Date, y = Maize.Total.Wt)) + 
    geom_line()
  
  ggplot(data = sim0[sim0$SimulationName == "SimulationSoybean", ], 
         aes(x = Date, y = Soybean.Total.Wt)) + 
    geom_line()
  
  ### Inspecting the soil
  inspect_apsimx("MaizeSoybean.apsimx", node = "Soil",
                 root = "SimulationSoybean",
                 soil.child = "Physical")
  
  inspect_apsimx("MaizeSoybean.apsimx", node = "Soil",
                 root = "SimulationMaize",
                 soil.child = "Physical")

  inspect_apsimx("MaizeSoybean.apsimx", node = "Soil",
                 root = "SimulationMaize") ### Nicollet
  inspect_apsimx("MaizeSoybean.apsimx", node = "Soil",
                 root = "SimulationSoybean") ### Nicollet
  
  #### Extracting alternative soils for this location?
  sps <- get_ssurgo_soil_profile(c(-93.76911, 42.02204), nsoil = 2, fix = TRUE)

  #### Which soils are these?
  sps[[1]]$metadata$SoilType
  sps[[2]]$metadata$SoilType
  
  plot(sps[[1]], property = "water")
  plot(sps[[2]], property = "water")
  
  #### Now I want to edit these soils for each simulation... Will it work?
  #### What if I ONLY change the soils? It does not work
  soil.paths <- c("SimulationSoybean.Soil", "SimulationMaize.Soil")
  grd <- expand.grid(soil1 = 1:2, soil2 = 1:2)
  names(grd) <- soil.paths
  
  start <- Sys.time()
  sns.soil0 <- sens_apsimx("MaizeSoybean.apsimx", 
                           parm.paths = soil.paths,
                           convert = rep(FALSE, 2),
                           replacement = rep(FALSE, 2),
                           grid = grd,
                           summary = "none",
                           soil.profiles = sps)
  (Sys.time() - start)
  
  head(sns.soil0$grid.sims[, c(1, 2, 3, 9)])

  snsM <- subset(sns.soil0$grid.sims, SimulationName == "SimulationMaize") 
  ggplot(data = snsM, 
         aes(x = Date, y = Maize.Total.Wt, color = as.factor(SimulationMaize.Soil))) + 
    geom_line() + 
    ylim(c(1000, 2000))
  
  snsS <- subset(sns.soil0$grid.sims, SimulationName == "SimulationSoybean") 
  ggplot(data = snsS, 
         aes(x = Date, y = Soybean.Total.Wt, color = as.factor(SimulationSoybean.Soil))) + 
    geom_line()
  
  ### Can I use more than one core?
  start <- Sys.time()
  sns.soil0.c2 <- sens_apsimx("MaizeSoybean.apsimx", 
                              parm.paths = soil.paths,
                              convert = rep(FALSE, 2),
                              replacement = rep(FALSE, 2),
                              grid = grd,
                              summary = "none",
                              soil.profiles = sps,
                              cores = 2L)
  (Sys.time() - start)
  #### There appears to be no benefit with such a small number of simulations,
  #### but it runs...
  
  
}

#' Edit an APSIM-X (JSON) Simulation
#' 
#' This function allows editing of an APSIM-X (JSON) simulation file.
#' 
#' The variables specified by \code{parm} within the .apsimx file specified by \code{file} 
#' in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
#' is a list that has the same number of elements as the length of the vector \code{parm}.  The current
#' .apsimx file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
#' \sQuote{file} \emph{-edited.apsimx} will be created.  If (verbose = TRUE) then the name
#'  of the written file is returned. 
#'  
#'  When node equals \sQuote{Report}, the editing allows to add variables, but not to remove them at the moment.
#' 
#'  When node equals Operations, \sQuote{parm} should have a list with two elements. The first should be the line(s) to edit and 
#'  the second should be the component(s) to edit. Either \sQuote{Date}, \sQuote{Action} or \sQuote{Line}.
#'  When more than one line is edited, \sQuote{value} should be a character vector of the same length as the number of
#'  lines to edit. It is possible to remove, say, line 10 by using \sQuote{parm = list(-10, NA)}. It is safer to remove
#'  lines at the end of \sQuote{Operations}. To remove several use the following \sQuote{parm = list(-c(10:12), NA)}. 
#'  This assumes that \sQuote{12} is the maximum number of lines present. Trying to remove lines in the middle will have 
#'  unexpected effects. It is possible to create additional lines, but only by using \sQuote{Date} first. This feature
#'  has not been tested much so use it carefully.
#'  
#' @name edit_apsimx
#' @param file file ending in .apsimx to be edited (JSON)
#' @param src.dir directory containing the .apsimx file to be edited; defaults to the current working directory
#' @param wrt.dir should be used if the destination directory is different from the src.dir
#' @param node either \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}, 
#' \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Report}, \sQuote{Operations} or \sQuote{Other} 
#' @param soil.child specific soil component to be edited
#' @param manager.child specific manager component to be edited
#' @param parm parameter to be edited. It can be a regular expression.
#' @param value new values for the parameter to be edited 
#' @param overwrite logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise
#' @param edit.tag if the file is edited a different tag from the default \sQuote{-edited} can be used.
#' @param parm.path path to the attribute to edit when node is \sQuote{Other}
#' @param root supply the node position in the case of multiple simulations such as factorials.
#' @param verbose whether to print information about successful edit
#' @return (when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
#' As a side effect this function creates a new (JSON) .apsimx file.
#' @export
#' @examples 
#' \donttest{
#' ## This example will read one of the examples distributed with APSIM-X
#' ## but write to a temporary directory
#' tmp.dir <- tempdir()
#' 
#' ## Edit Bulk density
#' extd.dir <- system.file("extdata", package = "apsimx")
#' bds <- c(1.02, 1.03, 1.09, 1.16, 1.18, 1.19, 1.20)
#' edit_apsimx("Wheat.apsimx", src.dir = extd.dir,
#'             wrt.dir = tmp.dir,
#'             node = "Soil",
#'             soil.child = "Physical", 
#'             parm = "BD", value = bds,
#'             verbose = FALSE)
#' ## Inspect file
#' inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir,
#'                 node = "Soil", soil.child = "Physical")
#' ## To delete the file...
#' file.remove(file.path(tmp.dir, "Wheat-edited.apsimx"))
#' 
#' ## Edit the fertilizer amount in 'Maize.apsimx'
#' edit_apsimx("Maize.apsimx", src.dir = extd.dir,
#'              wrt.dir = tmp.dir, node = "Manager",
#'              manager.child = "SowingFertiliser",
#'              parm = "Amount", value = 200, verbose = TRUE)
#'              
#' ## Make sure it worked
#' inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir, 
#'                node = "Manager",
#'                parm = list("SowingFertiliser", NA))
#' 
#' ## Remove the file
#' file.remove(file.path(tmp.dir, "Maize-edited.apsimx"))
#' }
#' 

edit_apsimx <- function(file, src.dir = ".", wrt.dir = NULL,
                        node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager", "Report", "Operations", "Other"),
                        soil.child = c("Metadata", "Water", "SoilWater", "Organic", "Physical", "Analysis", "Chemical", "InitialWater", "Sample", "Solute", "NO3", "NH4", "Urea", "Swim3"),
                        manager.child = NULL,
                        parm = NULL, value = NULL, 
                        overwrite = FALSE,
                        edit.tag = "-edited",
                        parm.path = NULL,
                        root = NULL,
                        verbose = TRUE){
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(src.dir)
  } 
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to edit.")
  }
  
  node <- match.arg(node)
  soil.child <- match.arg(soil.child)
  edited.child <- "none"
  
  ## For now we just edit one file at a time
  file <- match.arg(file, file.names)
  
  if(apsimx_filetype(file = file, src.dir = src.dir) != "json")
    stop("This function only edits JSON files")

  if(!is.null(parm.path) && node != "Other")
    stop("When parm.path is supplied node should be 'Other'", call. = FALSE)
  
  if(is.null(value))
    stop("'value' is missing", call. = FALSE)
  
  ## Parse apsimx file (JSON)
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))

  ## apsimx_json <<- apsimx_json
  ## Where is the 'Core' simulation?
  children.names <- sapply(apsimx_json$Children, FUN = function(x) x$`$type`)
  wcore <- grep("Core.Simulation", children.names)
  
  parm.path.0 <- paste0(".", apsimx_json$Name) ## Root
  ## When node == "Other" root should always be missing
  ## The problem is that I have to guess it
  ## We should be able to use node == "Other" even if root is not missing
  if(node == "Other" && is.null(root) && length(wcore) > 1){
    ## Process 'parm'
    if(is.null(parm.path))
      stop("parm.path is missing")
    if(!grepl(".", parm.path, fixed = TRUE))
      stop("parm.path is not a proper json path")
    cparm <- paste0(parm.path, ".", parm)
    pparm <- strsplit(cparm, split = ".", fixed = TRUE)[[1]]
    root.name.level.0 <- gsub(".", "", parm.path.0, fixed = TRUE)
    if(pparm[2] != root.name.level.0)
      stop(paste("First parm element does not match:", root.name.level.0), call. = FALSE)
    root1 <- pparm[3] 
    if(is.na(pparm[4])){
      root <- root1
    }else{
      ## Guess if 'root' is contained in the first level of names
      root.names.level.1 <- vapply(apsimx_json$Children, FUN = function(x) x$Name, 
                                   FUN.VALUE = "character")
      wroot1 <- grep(as.character(root1), root.names.level.1)    
      if(length(wroot1) == 0)
        stop(paste("Second element of parm did not match:", root.names.level.1), call. = FALSE)
      ## Need to test if the fourth element of pparm is a node
      nodes <- c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop", "Manager","Report", "Operations", "Other", "Field")
      if(pparm[4] %in% nodes){
        ## This amounts to guessing that root should be of length 1
        root <- list(pparm[3])
      }else{
        ## This amounts to guessing that pparm[4] should be the second element in 
        root.names.level.2 <- vapply(apsimx_json$Children[[wroot1]]$Children, 
                                     FUN = function(x) x$Name, 
                                     FUN.VALUE = "character")
        root2 <- pparm[4]
        wroot2 <- grep(as.character(root2), root.names.level.2)  
        if(length(wroot2) == 0)
          stop(paste("Third element of parm did not match:", root.names.level.2), call. = FALSE)
        if(pparm[5] %in% nodes){
          root <- list(pparm[3], pparm[4])
        }else{
          root.names.level.3 <- vapply(apsimx_json$Children[[wroot1]]$Children[[wroot2]]$Children, 
                                       FUN = function(x) x$Name, 
                                       FUN.VALUE = "character")
          root3 <- pparm[5]
          wroot3 <- grep(as.character(root3), root.names.level.3)    
          if(length(wroot3) == 0)
            stop(paste("Fourth element of parm did not match:", root.names.level.3), call. = FALSE)
        }
      }      
    }
  }

  if(length(wcore) > 1 || !is.null(root)){
    if(is.null(root)){
      cat("Simulation structure: \n")
      str_list(apsimx_json)
      stop("more than one simulation found and no root node label has been specified \n select one of the children names above")   
    }else{
      ## Parse root
      root <- parse_root(root)
      if(length(root) > 3)
        stop("At the moment 3 is the maximum length for root", call. = TRUE)
      if(length(root) == 1){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children
      }
      if(length(root) == 2){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root[1]), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label in position 1 found or root is not unique")
        root.node.0 <- apsimx_json$Children[[wcore1]]
        root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)  
        wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
        if(length(wcore2) == 0 || length(wcore2) > 1)
          stop("no root node label in position 2 found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children        
      }
      if(length(root) == 3){
        root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
        wcore1 <- grep(as.character(root[1]), root.node.0.names)
        if(length(wcore1) == 0 || length(wcore1) > 1)
          stop("no root node label in position 1 found or root is not unique")
        root.node.0 <- apsimx_json$Children[[wcore1]]
        root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)
        wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
        if(length(wcore2) == 0 || length(wcore2) > 1)
          stop("no root node label in position 2 found or root is not unique")
        root.node.1 <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]
        root.node.1.child.names <- sapply(root.node.1$Children, function(x) x$Name)  
        wcore3 <- grep(as.character(root[3]), root.node.1.child.names)
        if(length(wcore3) == 0 || length(wcore3) > 1)
          stop("no root node label in position 3 found or root is not unique")
        parent.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children        
      }
    }
  }else{
    parent.node <- apsimx_json$Children[[wcore]]$Children  
  }
  
  ## Edit the 'Clock'
  if(node == "Clock"){
    parm.choices <- c("Start","End")
    parm <- match.arg(parm, choices = parm.choices, several.ok = TRUE)
    ## Find what the 'Start' and 'End' are actually called
    ## Find the 'Clock'
    wlc <- function(x) grepl("Models.Clock", x$`$type`, ignore.case = TRUE)
    wlcl <- sapply(parent.node, FUN = wlc)
    
    if(sum(wlcl) < 1)
      stop("Clock not found", call. = FALSE)
    
    start <- grep("Start", names(parent.node[wlcl][[1]]), 
                  ignore.case = TRUE, value = TRUE)
    
    end <- grep("End", names(parent.node[wlcl][[1]]), 
                  ignore.case = TRUE, value = TRUE)
     
    if(length(parm) == 1){
     if(parm == "Start"){
       parent.node[wlcl][[1]][start] <- value
     }
     if(parm == "End"){
        parent.node[wlcl][[1]][end] <- value
      }
    }
    
    if(length(parm) == 2){
      if(parm[1] == "Start"){
        parent.node[wlcl][[1]][start] <- value[1]
      }
      if(parm[2] == "End"){
        parent.node[wlcl][[1]][end] <- value[2]
      }
    }
    ## The code below is not needed, I think
    # if(missing(root)){
    #   apsimx_json$Children[[1]]$Children <- parent.node  
    # }else{
    #   if(length(root) == 1){
    #     apsimx_json$Children[[wcore1]]$Children <- parent.node  
    #   }else{
    #     apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children <- parent.node
    #   }
    # }
  }
  
  ## Edit the met file
  if(node == "Weather"){
    wlw <- function(x) grepl("Models.Climate.Weather|Models.Weather", x$`$type`)
    wlwl <- sapply(parent.node, FUN = wlw)
    
    if(sum(wlwl) < 1)
      stop("Weather node not found", call. = FALSE)
    
    parent.node[wlwl][[1]]$FileName <- value
  }

  ## Extract 'Core' simulation
  wcz <- grepl("Models.Core.Zone", parent.node)
  
  if(sum(wcz) < 1)
    stop("Models.Core.Zone not found", call. = FALSE)
  
  core.zone.node <- parent.node[wcz][[1]]$Children
  
  ## Edit the soil
  if(node == "Soil"){
    ## Extract soil
    wsn <- grepl("Models.Soils.Soil", core.zone.node)
    
    if(sum(wsn) < 1)
      stop("Models.Soils.Soil not found", call. = FALSE)
    
    soil.node <- core.zone.node[wsn]
    
    soil.node0 <- soil.node[[1]]$Children
    
    if(soil.child == "Metadata"){
      edited.child <- soil.child
      ## Perhaps too many possible metadata parms to list here?
      metadata.parms <- c("RecordNumber", "ASCOrder", "ASCSubOrder", "SoilType",
                          "LocalName", "Site", "NearestTown", "Region", 
                          "State", "Country", "NaturalVegetation", "ApsoilNumber",
                          "Latitude", "Longitude", "LocationAccuracy", "DataSource",
                          "Comments")
      if(!all(parm %in% metadata.parms)) stop("parm name(s) might be wrong")
      for(i in seq_along(parm)){
        soil.node[[1]][[parm[i]]] <- value[i]
      }
    }
  
    if(soil.child == "Physical"){
      edited.child <- soil.child
      
      ## In older versions of APSIM Next Gen instead of 'Physical' it was called
      ## 'Water'. Now "InitialWater" has been renamed to simply "Water"
      ## So here we now just look for "Physical"
      wwn <- grep("Physical", sapply(soil.node[[1]]$Children, function(x) x$Name)) 
      soil.water.node <- soil.node[[1]]$Children[[wwn]]

      if(soil.water.node$Name != "Water" && soil.water.node$Name != "Physical"){
        cat("Found: ", soil.water.node$Name, "instead of Physical or Water \n")
        stop("Wrong node (Physical or Water)")
      }
      
      crop.parms <- c("XF", "KL", "LL")
      
      if(parm %in% crop.parms || any(sapply(crop.parms, function(x) grepl(x, parm))) && parm != "LL15"){
        ## Maybe we are trying to edit the parameter for a specific crop
        ## The first options matches the parameter exactly 
        if(parm %in% crop.parms){
          for(i in seq_along(soil.water.node$Children[[1]][[parm]])){
            soil.water.node$Children[[1]][[parm]][[i]] <- value[i]
          }          
        }else{
         ## This assumes that the parameter to be edited is "Wheat LL" for example
          parm0 <- strsplit(parm, " ")[[1]]
          crop.name <- parm0[1]
          sparm <- parm0[2]
          wcnp <- which(gsub("Soil", "", sapply(soil.water.node$Children, function(x) x$Name)) == crop.name)
          for(i in seq_along(soil.water.node$Children[[1]][[sparm]])){
            soil.water.node$Children[[wcnp]][[sparm]][[i]] <- value[i]  
          }
        }
      }else{
        for(i in 1:length(soil.water.node[[parm]])){
          soil.water.node[[parm]][[i]] <- value[i]
        }
      }
      soil.node[[1]]$Children[[wwn]] <- soil.water.node
    }
    
    if(soil.child == "SoilWater"){
      edited.child <- soil.child
      wswn <- grep("^SoilWater", sapply(soil.node[[1]]$Children, function(x) x$Name)) 
      soil.soilwater.node <- soil.node[[1]]$Children[[wswn]]
      
      soilwat.parms <- c("SummerDate", "SummerU", "SummerCona", "WinterDate",
                         "WinterU", "WinterCona", "DiffusConst", "DiffusSlope",
                         "Salb", "CN2Bare", "CNRed", "CNCov", "Slope", "DischargeWidth",
                         "CatchmentArea")
      
      if(parm %in% soilwat.parms){
        ## This allows for editing multiple parameters and values
        for(i in seq_along(parm)){
          soil.soilwater.node[[parm[i]]] <- value[i]
        }
      }else{
        ## This case is most likely SWCON
        if(!parm %in% c("SWCON", "KLAT")) stop("parameter is likely incorrect")
        for(i in 1:length(soil.soilwater.node[[parm]])){
          soil.soilwater.node[[parm]][[i]] <- value[i]
        }
      }
      soil.node[[1]]$Children[[wswn]] <- soil.soilwater.node
    }
    
    if(soil.child == "Nitrogen"){
      wnn <- grepl("Nitrogen", soil.node0)
      soil.nitrogen.node <- soil.node0[wnn][[1]]
      
      for(i in 1:length(soil.nitrogen.node[[parm]])){
        soil.nitrogen.node[[parm]][[i]] <- value[i]
      }
      soil.node[[1]]$Children[wnn][[1]] <- soil.nitrogen.node
    }
    
    if(soil.child == "Organic"){
      edited.child <- "Organic"
      wsomn <- grepl("Organic", soil.node0)
      soil.om.node <- soil.node0[wsomn][[1]]
      
      som.parms1 <- c("RootCN", "EnrACoeff", "EnrBCoeff")
      
      if(parm %in% som.parms1){
          soil.om.node[[parm]] <- value
      }else{
        for(i in 1:length(soil.om.node[[parm]])){
          soil.om.node[[parm]][[i]] <- value[i]
        }
      }
      soil.node[[1]]$Children[wsomn][[1]] <- soil.om.node
    }
    
    if(soil.child %in% c("Analysis", "Chemical", "Solute", "NO3", "NH4", "Urea")){
      edited.child <- soil.child
      wan <- grepl(soil.child, soil.node0)

      if(sum(wan) == 1){
        soil.analysis.node <- soil.node0[wan][[1]]  
      }else{
        soil.node0.names <- sapply(soil.node0[wan], FUN = function(x) x$Name)
        if(soil.child == "Solute"){
          if(length(parm) != 2)
            stop("When 'soil.child' is 'Solute' 'parm' should be of length = 2", call. = FALSE)
          if(!parm[[1]] %in% c("NO3", "NH4", "Urea"))
            stop("The first element of 'parm' should be one of :", soil.node0.names, call. = FALSE)
          wsan <- which(soil.node0.names == parm[[1]])
          soil.analysis.node <- soil.node0[wan][[wsan]]
        }
      }
      
      ## Check for length
      if(length(parm) == 1){
        if(length(soil.analysis.node[[parm]]) != length(value)){
          cat("Length of value:", length(value), "\n")
          cat("Length of 'node':", length(soil.analysis.node[[parm]]), "\n")
          stop("Length of 'value' should equalto length of 'node'", call. = FALSE)
        }
        if(parm %in% c("PH", "NO3", "NH4", "Urea", "Thickness", "InitialValues")){
          for(i in 1:length(soil.analysis.node[[parm]])){
            soil.analysis.node[[parm]][[i]] <- value[i]
          }
        }else{
          stop("'parm' should be one of 'PH', 'NO3', 'NH4', 'Urea', 'InitialValues' or 'Thickness'", call. = FALSE)
        }
      }
      
      if(length(parm) == 2){
        if(length(soil.analysis.node[[parm[[2]]]]) > length(value))
          stop("Length of 'value' should not be less than length of 'node'", call. = FALSE)
        if(length(soil.analysis.node[[parm[[2]]]]) < length(value))
          stop("Length of 'value' should not be greater than length of 'node'", call. = FALSE)
        if(!parm[[2]] %in% names(soil.analysis.node))
          stop("The second element of 'parm' should be one of: ", names(soil.analysis.node), call. = FALSE)
        for(i in 1:length(soil.analysis.node[[parm[[2]]]])){
          soil.analysis.node[[parm[[2]]]][[i]] <- value[i]
        }
      }
      ### Need to fix this
      if(sum(wan) == 1){
        soil.node[[1]]$Children[wan][[1]] <- soil.analysis.node  
      }else{
        soil.node[[1]]$Children[wan][[wsan]] <- soil.analysis.node
      }
    }
    
    if(soil.child == "InitialWater" || soil.child == "Water"){
      edited.child <- "InitialWater"
      soil.node0.names <- sapply(soil.node0, function(x) x$Name)
      wiwn <- grep("InitialWater", soil.node0.names)
      if(length(wiwn) == 0){
        ### Maybe find just water?
        wiwn <- grep("^Water", soil.node0.names)
        if(length(wiwn) == 0){
          wiwn <- grep("initial water", soil.node0.names, ignore.case = TRUE)
        }
      }
      if(length(wiwn) == 0)
        stop("InitialWater node not found", call. = FALSE)
      
      soil.initialwater.node <- soil.node0[wiwn][[1]]
      
      ## Only three can be edited: PercentMethod, FractionFull, DepthWetSoil
      siw.parms <- c("PercentMethod", "FractionFull", "DepthWetSoil", "Thickness", "InitialValues")
      parm <- match.arg(parm, choices = siw.parms)
      
      if(parm %in% c("Thickness", "InitialValues")){
        soil.initialwater.node.vector <- soil.initialwater.node[[parm]]
        if(length(value) != length(soil.initialwater.node.vector))
          stop("Length of 'value' should match the length of ", parm, call. = FALSE)
        for(i in seq_along(soil.initialwater.node.vector))
          soil.initialwater.node.vector[[1]][[i]] <- value[i]
        soil.initialwater.node[[parm]] <- soil.initialwater.node.vector[[1]]
        soil.node[[1]]$Children[wiwn][[1]] <- soil.initialwater.node
      }else{
        soil.initialwater.node[[parm]] <- value
        soil.node[[1]]$Children[wiwn][[1]] <- soil.initialwater.node        
      }
    }
    
    if(soil.child == "Sample"){
      edited.child <- "Sample"
      wsn <- grepl("Sample", soil.node0)
      soil.sample.node <- soil.node0[wsn][[1]]
      
      for(i in 1:length(soil.sample.node[[parm]])){
        soil.sample.node[[parm]][[i]] <- value[i]
      }
      soil.node[[1]]$Children[wsn][[1]] <- soil.sample.node
    }
    
    if(soil.child == "Swim3"){
      edited.child <- "Swim3"
      wswimn <- grepl("Swim3", soil.node0)
      soil.swim.node <- soil.node0[wswimn][[1]]
      
      for(i in 1:length(soil.swim.node[[parm]])){
        soil.swim.node[[parm]][[i]] <- value[i]
      }
      soil.node[[1]]$Children[wswimn][[1]] <- soil.swim.node
    }
    
    core.zone.node[wsn] <- soil.node
  }
  
  if(node == "SurfaceOrganicMatter"){
    wsomn <- grepl("Models.Surface.SurfaceOrganicMatter", core.zone.node)
    som.node <- core.zone.node[wsomn][[1]]
    
    if(som.node$Name != "SurfaceOrganicMatter"){
      stop("Wrong node")
    }
    som.node[[parm]] <- value
    core.zone.node[wsomn][[1]] <- som.node
  }
  
  if(node == "MicroClimate"){
    wmcn <- grepl("Models.MicroClimate", core.zone.node)
    microclimate.node <- core.zone.node[wmcn][[1]]
    
    if(microclimate.node$Name != "MicroClimate"){
      stop("Wrong node")
    }
    microclimate.node[[parm]] <- value
    core.zone.node[wmcn][[1]] <- microclimate.node
  }
  
  if(node == "Crop"){
    ## Which is 'Crop'
    wmmn <- grepl("Models.Manager", core.zone.node)
    manager.node <- core.zone.node[wmmn]
    ## Which element has the crop information?
    wcn <- grepl("CultivarName", manager.node)
    crop.node <- manager.node[wcn][[1]]$Parameters
    
    for(i in 1:length(crop.node)){
      if(crop.node[[i]]$Key == parm){
        crop.node[[i]]$Value <- value
      }
    }
    core.zone.node[wmmn][wcn][[1]]$Parameters <- crop.node
  }
  
  if(node == "Manager"){
    wmmn <- grepl("Models.Manager", core.zone.node)
    manager.node <- core.zone.node[wmmn]
    manager.node.names <- sapply(manager.node, FUN = function(x) x$Name)

    if(missing(manager.child)) stop("need to specify manager.child")
    edited.child <- manager.child
    ## Which child should we edit?
    wmc <- grep(manager.child, manager.node.names)
    ## Maybe if it is inside a folder try looking inside manager.node?
    if(length(wmc) == 0){
      manager.node.names <- sapply(manager.node[[1]]$Children, FUN = function(x) x$Name)
      wmc2 <- grep(manager.child, manager.node.names)
      manager.child.node <- manager.node[[1]]$Children[[wmc2]]$Parameters  
    }else{
      manager.child.node <- manager.node[[wmc]]$Parameters  
    }
    
    for(i in 1:length(manager.child.node)){
      if(manager.child.node[[i]]$Key == parm){
        manager.child.node[[i]]$Value <- value
      }
    }
    
    if(length(wmc) == 0){
      manager.node[[1]]$Children[[wmc2]]$Parameters <- manager.child.node 
    }else{
      manager.node[[wmc]]$Parameters <- manager.child.node  
    }
    core.zone.node[wmmn] <- manager.node
  }
  
  if(node == "Operations"){
    won <- grepl("Models.Operations", core.zone.node)
    operations.node <- core.zone.node[won]
    
    if(length(operations.node) > 1)
      stop("Not ready to handle multiple 'Operations'", call. = FALSE)
    
    if(is.null(operations.node[[1]]$Operation))
      stop("'Operation' child node not found", call. = FALSE)
    
    ## Here the assumption is that 'parm' is a line and a component to edit
    if(length(parm) != 2)
      stop("'parm' should be a list of length 2. The first should be the line and the second should be the component", call. = FALSE)
    
    if(all(parm[[1]] > 0)){
      if(length(parm[[1]]) != length(value))
        stop("lenght of the first 'parm' element should be equal to the length of 'value'", call. = FALSE)      
    }

    length.operation <- length(operations.node[[1]]$Operation)
    
    if(all(parm[[1]] > 0)){
      if(parm[[2]] == "Date"){
        for(i in seq_along(value)){
          date.exists <- try(operations.node[[1]]$Operation[[parm[[1]][i]]]$Date, silent = TRUE)
          if(!inherits(date.exists, 'try-error')){
            operations.node[[1]]$Operation[[parm[[1]][i]]]$Date <- value[i]      
          }else{
            ## Need to add Date if not present 
            ##stop("Adding a row is not available yet", call. = FALSE)
            if(verbose) cat("Added a new 'Date' element in position", parm[[1]][i], "\n")
            input.list <- vector("list", length = 1) ## Create empty list
            list.elements <- operations.node[[1]]$Operation[[1]] ## Copying the first one
            list.elements$Date <- value[i] ## replacing Date
            list.elements$Action <- "" 
            list.elements$Line <- ""
            input.list[[1]] <- list.elements
            operations.node[[1]]$Operation <- append(operations.node[[1]]$Operation, input.list) 
          }
        }
      }
      
      if(parm[[2]] == "Action"){
        for(i in seq_along(value)){
          action.exists <- try(operations.node[[1]]$Operation[[parm[[1]][i]]]$Action, silent = TRUE)
          if(!inherits(action.exists, 'try-error')){
            operations.node[[1]]$Operation[[parm[[1]][i]]]$Action <- value[i]      
          }else{
            stop("Trying to edit an 'Action' item but it is not present. Add 'Date' first.", call. = FALSE)
          }
        }
      }
      
      if(parm[[2]] == "Line"){
        for(i in seq_along(value)){
          line.exists <- try(operations.node[[1]]$Operation[[parm[[1]][i]]]$Date, silent = TRUE)
          if(!inherits(line.exists, 'try-error')){
            operations.node[[1]]$Operation[[parm[[1]][i]]]$Line <- value[i]      
          }else{
            stop("Trying to edit a 'Line' item but it is not present. Add 'Date' first.", call. = FALSE)
          }
        }
      }      
    }else{
      ### In this case the format should be parm = list(-10, 'NA')
      if(!is.na(parm[[2]][1]))
        stop("Second element of the 'parm' list should be 'NA'", call. = FALSE)
      
      for(i in rev(seq_along(parm[[1]]))){
        idx.rm <- abs(parm[[1]][i])
        operations.node[[1]]$Operation[[idx.rm]] <- NULL      
      }
    }
      
    if(all(parm[[1]] > 0)){
      if(!parm[[2]] %in% c("Date", "Action", "Line"))
        stop("The second 'parm' component should be either 'Date', 'Action', or 'Line'", call. = FALSE)      
    }

    core.zone.node[won][[1]]$Operation <- operations.node[[1]]$Operation
    parm <- unlist(parm)
  }
  
  if(node == "Report"){
    wrn <- grepl("Models.Report", core.zone.node)
    report.node <- core.zone.node[wrn]
    report.node.names <- sapply(report.node, FUN = function(x) x$Name)
    
    if(missing(parm))
      stop("parm argument is missing")
    ## There is only one report in this case
    if(!is.list(parm)){
      if(length(report.node.names) > 1)
        stop("More than one Report is present. Use a list to choose one.")
      if(!grepl(parm, "VariableNames") && !grepl(parm, "EventNames"))
        stop("parm should contain either VariableNames or EventNames")
      if(grepl(parm, "VariableNames")){
        lvn <- length(report.node[[1]]$VariableNames)
        for(i in seq_along(value)){
          vnindx <- lvn + i
          report.node[[1]]$VariableNames[[vnindx]] <- value[i]
        }
      }else{
        evn <- length(report.node[[1]]$EventNames)
        for(i in seq_along(value)){
          evnindx <- evn + i
          report.node[[1]]$EventNames[[evnindx]] <- value[i]
        }
      }  
    }else{
      if(length(parm) < 2)
        stop("parm should be a list of length greater than 1")
      wrn <- grep(parm[[1]], report.node.names)  
      if(length(wrn) == 0L)
        stop("Report Name not found")
      if(!grepl(parm[[2]], "VariableNames") && !grepl(parm[[2]], "EventNames"))
        stop("parm should contain either VariableNames or EventNames")
      if(grepl(parm[[2]], "VariableNames")){
        lvn <- length(report.node[[wrn]]$VariableNames)
        for(i in seq_along(value)){
          vnindx <- lvn + i
          report.node[[wrn]]$VariableNames[[vnindx]] <- value[i]
        }
      }else{
        evn <- length(report.node[[wrn]]$EventNames)
        for(i in seq_along(value)){
          evnindx <- evn + i
          report.node[[wrn]]$EventNames[[evnindx]] <- value[i]
        }
      }
    }
    core.zone.node[wrn] <- report.node 
  }
  
  if(node == "Other"){
    ## Note: this strategy might not always work well
    ## because cultivar parameters are under Command
    ## which I think behaves differently
    ## Here the path should be to the full parameter path
    ## Unpack the parm.path
    ## root node starts at .
    
    if(!missing(parm))
      parm.path <- paste0(parm.path, ".", parm)
    
    if(missing(parm.path))
      stop("When node = 'Other' parm.path should be provided")
    
    ## Strip dollar sign if present
    if(substr(parm.path, 1, 1) == "$") 
      parm.path <- substr(parm.path, 2, nchar(parm.path))
    
    upp <- strsplit(parm.path, ".", fixed = TRUE)[[1]]
    upp.lngth <- length(upp)
    if(upp.lngth < 5) stop("Parameter path too short?")
    if(upp.lngth > 10) stop("Cannot handle this yet")
    ## upp[2] is typically "Simulations"
    if(apsimx_json$Name != upp[2])
      stop("Simulation root name does not match")
    wl3 <- which(upp[3] == sapply(apsimx_json$Children, function(x) x$Name))
    if(length(wl3) == 0) stop("Could not find parameter at level 3")
    ## At this level I select among simulation children
    ## upp[3] is typically "Simulation"
    n3 <- apsimx_json$Children[[wl3]]
    ## Look for the first reasonable parameter
    wl4 <- which(upp[4] == sapply(n3$Children, function(x) x$Name))
    if(length(wl4) == 0) stop("Could not find parameter at level 4")
    ## This is super dumb but I do not know how to do it otherwise
    if(upp.lngth == 5){
      if(upp[5] %in% names(n3$Children[[wl4]])){
        apsimx_json$Children[[wl3]]$Children[[wl4]][[upp[5]]] <- value
      }else{
        wl5 <- which(upp[5] == sapply(n3$Children[[wl4]]$Children, function(x) x$Name))
        if(length(wl5) == 0) stop("Could not find parameter at level 5")
        apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]][[upp[5]]] <- value
      }
    }
    ## Handling level 6
    if(upp.lngth == 6){
      n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
      wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
      if(length(wl5) == 0) stop("Could not find parameter at level 5")
      if(upp[6] %in% names(n4$Children[[wl5]])){
        apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]][[upp[6]]] <- value
      }else{
        if("Parameters" %in% names(n4$Children[[wl5]])){
          wp <- grep(upp[6], n4$Children[[wl5]]$Parameters)
          if(length(wp) == 0) stop("Could not find parameter at level 6 (Parameter)")
          apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Parameters[[wp]]$Value <- value
        }else{
          wl6 <- which(upp[6] == sapply(n4$Children[[wl5]]$Children, function(x) x$Name))
          if(length(wl6) == 0) stop("Could not find parameter at level 6")
          apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[6]]] <- value         
        }
      }
    }
    ## Handling level 7, I don't think this works. Need to test.
    if(upp.lngth == 7){
      n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
      wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
      n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
      wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[7]]] <- value
    }
    if(upp.lngth == 8){
      n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
      wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
      n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
      wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
      n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
      wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]][[upp[8]]] <- value
    }
    if(upp.lngth == 9){
      n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
      wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
      n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
      wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
      n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
      wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
      n7 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]
      wl8 <- which(upp[8] == sapply(n7$Children, function(x) x$Name))
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]][[upp[9]]] <- value
    }
    if(upp.lngth == 10){
      n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
      wl5 <- which(upp[5] == sapply(n4$Children, function(x) x$Name))
      n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
      wl6 <- which(upp[6] == sapply(n5$Children, function(x) x$Name))
      n6 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]
      wl7 <- which(upp[7] == sapply(n6$Children, function(x) x$Name))
      n7 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]
      wl8 <- which(upp[8] == sapply(n7$Children, function(x) x$Name))
      n8 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]
      wl9 <- which(upp[9] == sapply(n8$Children, function(x) x$Name))
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]]$Children[[wl8]]$Children[[wl9]][[upp[10]]] <- value
    }
  }
  
  if(node != "Other"){
    parent.node[wcz][[1]]$Children <- core.zone.node
    
    if(length(wcore) > 1 || !is.null(root)){
      ## I have to assume that root was supplied 
      ## otherwise an error would have been triggered before
      if(length(root) == 1){
        apsimx_json$Children[[wcore1]]$Children <- parent.node 
      }
      if(length(root) == 2){
        apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children <- parent.node 
      }
      if(length(root) == 3){
        apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children[[wcore3]]$Children <- parent.node 
      }
    }else{
      apsimx_json$Children[[wcore]]$Children <- parent.node 
    }    
  }

  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir, "/",
                      tools::file_path_sans_ext(file),
                      edit.tag, ".apsimx")
  }else{
    wr.path <- paste0(wrt.dir, "/", file)
  }
  
  jsonlite::write_json(apsimx_json, path = wr.path, 
                       pretty = TRUE, digits = NA, 
                       auto_unbox = TRUE, null = "null")
  
  if(verbose){
    cat("Edited (node): ", node, "\n")
    cat("Edited (child): ", edited.child,"\n")
    cat("Edited parameters: ", parm, "\n")
    cat("New values: ", value, "\n")
    cat("Created: ", wr.path,"\n")
  }
}

## This function is not exported at the moment
## It is not ready to be used yet.
edit_apsimx_json <- function(file, src.dir = ".", wrt.dir = NULL,
                             parm.path = NULL, value = NULL, 
                             overwrite = FALSE, edit.tag = "-edited",
                             verbose = TRUE){
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(src.dir)
  } 
  
  if(missing(wrt.dir)) wrt.dir <- src.dir
  
  file.names.apsimx <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
  file.names.json <- dir(path = src.dir, pattern=".json$", ignore.case=TRUE)
  file.names <- c(file.names.apsimx, file.names.json)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx or.json files in the specified directory to edit.")
  }
  
  file <- match.arg(file, file.names)
  
  ## Parse apsimx file (JSON)
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
  
  ## Should add code to deal with scoped paths
  if(substr(parm.path, 1, 1) == "$") 
    parm.path <- substr(parm.path, 2, nchar(parm.path))
    
  upp <- strsplit(parm.path, ".", fixed = TRUE)[[1]]
  upp.lngth <- length(upp)
  keep.going <- TRUE
  
  if(upp.lngth == 2){
    wp2p <- which(names(apsimx_json) == upp[2])
    if(length(wp2p) == 0)
      stop("Parameter not found")
    apsimx_json[[wp2p]] <- value
  }
  
  if(upp.lngth >= 3 && keep.going){
    if(apsimx_json$Name != upp[2])
      stop("First position name does not match")    
    wl3 <- which(upp[3] == sapply(apsimx_json$Children, function(x) x$Name))
    if(length(wl3) == 0) stop("Could not find parameter at level 3")
    if(upp.lngth == 3){
      apsimx_json$Children[[wl3]] <- value
      keep.going <- FALSE
    }
  }
  
  if(upp.lngth >= 4 && keep.going){
    n3 <- apsimx_json$Children[[wl3]]
    ## Look for the first reasonable parameter
    wl4 <- which(upp[4] == sapply(n3$Children, function(x) x$Name))
    if(length(wl4) == 0) stop("Could not find parameter at level 4")
    if(upp.lngth == 4){
      apsimx_json$Children[[wl3]]$Children[[wl4]] <- value
      keep.going <- FALSE
    }
  }
  
  if(upp.lngth >= 5 && keep.going){
    
    if(upp[5] %in% names(n3$Children[[wl4]])){
      apsimx_json$Children[[wl3]]$Children[[wl4]][[upp[5]]] <- value
      keep.going <- FALSE
    }else{
      wl5 <- which(upp[5] == sapply(n3$Children[[wl4]]$Children, function(x) x$Name))
      if(length(wl5) == 0) stop("Could not find parameter at level 5")
      if(upp.lngth == 5){
        apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]][[upp[5]]] <- value
        keep.going <- FALSE
      }
    }
  }
  ## Handling level 6
  if(upp.lngth >= 6 && keep.going){
    n4 <- apsimx_json$Children[[wl3]]$Children[[wl4]]
    if(upp[6] %in% names(n4$Children[[wl5]])){
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]][[upp[6]]] <- value
      keep.going <- FALSE
    }else{
      if("Parameters" %in% names(n4$Children[[wl5]])){
        wp <- grep(upp[6], n4$Children[[wl5]]$Parameters)
        if(length(wp) == 0) stop("Could not find parameter at level 6 (Parameter)")
        apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Parameters[[wp]]$Value <- value
        keep.going <- FALSE
      }else{
        wl6 <- which(upp[6] == sapply(n4$Children[[wl5]]$Children, function(x) x$Name))
        if(length(wl6) == 0) stop("Could not find parameter at level 6")
        if(upp.lngth == 6){
          apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[6]]] <- value         
          keep.going <- FALSE
        }
      }
    }
  }
  
  ## Handling level 7
  if(upp.lngth >= 7 && keep.going){
    n5 <- apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]
    if(upp[7] %in% names(n5$Children[[wl6]])){
      apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]][[upp[7]]] <- value
      keep.going <- FALSE
    }else{
      if("Parameters" %in% names(n5$Children[[wl6]])){
        wp <- grep(upp[7], n5$Children[[wl6]]$Parameters)
        if(length(wp) == 0) stop("Could not find parameter at level 7 (Parameter)")
        apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Parameters[[wp]]$Value <- value
        keep.going <- FALSE
      }else{
        wl7 <- which(upp[7] == sapply(n5$Children[[wl6]]$Children, function(x) x$Name))
        if(length(wl6) == 0) stop("Could not find parameter at level 7")
        if(upp.lngth == 7){
          apsimx_json$Children[[wl3]]$Children[[wl4]]$Children[[wl5]]$Children[[wl6]]$Children[[wl7]][[upp[7]]] <- value
          keep.going <- FALSE
        }
      }
    }
  }

  if(overwrite == FALSE){
    wr.path <- paste0(wrt.dir, "/",
                      tools::file_path_sans_ext(file),
                      edit.tag, ".apsimx")
  }else{
    wr.path <- paste0(wrt.dir, "/", file)
  }
  
  jsonlite::write_json(apsimx_json, path = wr.path, 
                       pretty = TRUE, digits = NA, 
                       auto_unbox = TRUE, null = "null")
  
  if(verbose){
    cat("Edited parameters: ", parm.path, "\n")
    cat("New values: ", value, "\n")
    cat("Created: ", wr.path,"\n")
  }
}
  
  
  
  
  

## A few steps
require(apsimx)
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

## Only write to a temp directory
tmp.dir <- tempdir()

## These tests can only be run locally
run.factorial.example <- get(".run.local.tests", envir = apsimx.options)

if(run.factorial.example){

  ex.dir <- auto_detect_apsimx_examples()
  
  ## Copy 'Factorial' example to current directory
  fact.ex <- paste0(ex.dir, "/Factorial.apsimx")
  file.exists(fact.ex)
  file.copy(from = fact.ex,
            to = tmp.dir, copy.mode = FALSE)
  
  ## I ran these commands at the command line
  ## sed -i'.bak' -e 's/\bWeatherFiles/%root%\\\\\\\\Examples\\\\\\\\WeatherFiles/g *

  inspect_apsimx_replacement("Factorial", src.dir = tmp.dir,
                             root = list("Experiment",1),
                             node = "Base",
                             node.child = "Weather")
  for(i in 1:5){
    edit_apsimx_replacement("Factorial", 
                            src.dir = tmp.dir,
                            wrt.dir = tmp.dir,
                            root = list("Experiment",i),
                            node = "Base", 
                            node.child = "Weather",
                            parm = "FileName",
                            overwrite = TRUE,
                            verbose = TRUE,
                            value = paste0(ex.dir,"/WeatherFiles/lincoln.met"))
  }

  inspect_apsimx_replacement("Factorial", src.dir = tmp.dir,
                             root = list("Experiment",2),
                             node = "Base",
                             node.child = "Weather")

  fact <- apsimx("Factorial.apsimx", src.dir = tmp.dir)
  file.remove(paste0(tmp.dir, "/Factorial.apsimx"))
  file.remove(paste0(tmp.dir, "/Factorial.db"))
}

run.factorial.example2.inspect <- get(".run.local.tests", envir = apsimx.options)

if(run.factorial.example2.inspect){

  ex.dir <- auto_detect_apsimx_examples()

  ## Inspect factorial
  inspect_apsimx("Factorial", src.dir = ex.dir,
                 root = c("RangeExperiment","Base"),
                 node = "Weather")

  inspect_apsimx("Factorial", src.dir = ex.dir,
                 root = c("RangeExperiment","Base"),
                 node = "Clock")
  
  
}

run.factorial.example2.edit <- get(".run.local.tests", envir = apsimx.options)

if(run.factorial.example2.edit){
  
  ex.dir <- auto_detect_apsimx_examples()
  ## Copy 'Factorial' example to current directory
  fact.ex <- paste0(ex.dir,"/Factorial.apsimx")
  file.exists(fact.ex)
  file.copy(from = fact.ex,
            to = tmp.dir, copy.mode = FALSE)
  
  factor.label <- c("PropertyReplacement", "RangeExperiment", "CompositeFactor", 
                    "ModelReplacement", "Permutation")
  # bases <- c("Base","Base2","Base3","Base4","Base5")
  
  for(i in 1:5){
    edit_apsimx("Factorial.apsimx", 
                src.dir = tmp.dir, 
                wrt.dir = tmp.dir,
                node = "Weather",
                overwrite = TRUE,
                root = c(factor.label[i], "Base"), 
                value = paste0(ex.dir,"/WeatherFiles/lincoln.met"))
  }
  
  fact2 <- apsimx("Factorial.apsimx", src.dir = tmp.dir)
  file.remove(paste0(tmp.dir, "/Factorial.apsimx"))
  file.remove(paste0(tmp.dir, "/Factorial.db"))
  
}

## Testing apsim_met suite of functions
## Some observations:
##
## 1. check should never fail. It should always return warnings
## 2. napad should work for many different tricky situations
## 3. plotting and different combinations

require(apsimx)
require(mgcv)
require(ggplot2)
packageVersion("apsimx")
packageVersion("GSODR")
packageVersion("data.table")

run.apsim.met <- get(".run.local.tests", envir = apsimx.options)

if(.Platform$OS.type == "unix"){
  internet <- !is.null(nsl("google.com"))
}else{
  internet <- FALSE  
}
run.apsim.met <- FALSE  ## This does not work because GSODR does not work

if(run.apsim.met && internet){
  
  ## Testing napad and impute
  ## Buenos Aires 34d 36 12, 58 22 54 W
  bsas.lat <- unit_conv("34d 36' 12\" W", from = "degrees", to = "decimal")
  bsas.lon <- unit_conv("58d 22' 54\" S", from = "degrees", to = "decimal")
  
  pwr <- get_power_apsim_met(lonlat = c(bsas.lon, bsas.lat), dates = c("2010-01-01", "2015-12-31"))
  dim(pwr)
  check_apsim_met(pwr)
  gsd <- get_gsod_apsim_met(lonlat = c(bsas.lon, bsas.lat), 
                            dates = c("2010-01-01", "2015-12-31"))
  dim(gsd)
  check_apsim_met(gsd)
  gsd$radn <- pwr$radn
  check_apsim_met(gsd)
  gsd.impt <- impute_apsim_met(gsd)
  check_apsim_met(gsd.impt)
  
  ## Test for Brazil (Sao Paulo)
  ## 23 33′S 46 38′W
  sp.lat <- unit_conv("23d 33' 0\" W", from = "degrees", to = "decimal")
  sp.lon <- unit_conv("46d 38' 0\" S", from = "degrees", to = "decimal")
  
  pwr <- get_power_apsim_met(lonlat = c(sp.lon, sp.lat), dates = c("2010-01-01", "2015-12-31"))
  check_apsim_met(pwr)
  
  plot(pwr, met.var = "rain", years = 2010:2015, cumulative = TRUE, climatology = TRUE)
  
  gsd <- get_gsod_apsim_met(lonlat = c(sp.lon, sp.lat), dates = c("2010-01-01", "2015-12-31"), fillin.radn = TRUE)
  check_apsim_met(gsd)
  gsd.np <- napad_apsim_met(gsd)
  gsd.imp <- impute_apsim_met(gsd.np)
  check_apsim_met(gsd.imp) ## Rain variable is missing
  plot(gsd.imp, met.var = "radn", years = 2010:2015, cumulative = TRUE, climatology = TRUE)
  
  ## Test for Daymet data
  dmt <- get_daymet2_apsim_met(lonlat = c(-93, 42), years = c(2000, 2020))
  check_apsim_met(dmt)
  dmt.np <- napad_apsim_met(dmt)  
  dmt.imp <- impute_apsim_met(dmt.np)
  check_apsim_met(dmt.imp)
  
  summary(dmt.imp)
  plot(dmt.imp)
  plot(dmt.imp, met.var = "rain", years = 2010:2015, cumulative = TRUE, climatology = TRUE)
}

## Testing the feature for adding a column to a met file
if(run.apsim.met){
  
  ## Testing adding a column
  extd.dir <- system.file("extdata", package = "apsimx")
  ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
  
  ## ames$vp <- 5 This does not work and it is not supposed to 
  
  vp <- data.frame(vp = abs(rnorm(nrow(ames), 2)))
  attr(vp, "units") <- "(hPa)"
  
  ames$vp <- vp ## This works now (2026-6-18)
  ames[["vp"]] <- vp ## This works now (2026-6-18)

  val <- abs(rnorm(nrow(ames), 2))
  nm <- "vp"
  ames <- add_column_apsim_met(ames, value = val, name = "vp", units = "(hPa)")
  
  ## Extra tests
  ames <- ames[ames$year < 2018, ]
  ames <- apsimx:::pp_apsim_met(ames)
  ames <- tt_apsim_met(ames, dates = c("01-05", "30-10"), method = "Classic_TT")
  
  plot(ames, met.var = "photoperiod")
  plot(ames, met.var = "Classic_TT")
    
}

if(run.apsim.met && internet){
 
  iem <- get_iem_apsim_met(lonlat = c(-93.77, 42.02), 
                           dates = c("2013-01-01","2013-12-31"))
  
  pwr <- get_power_apsim_met(lonlat = c(-93.77, 42.02), 
                           dates = c("2013-01-01","2013-12-31"))
  
  dmt <- get_daymet_apsim_met(lonlat = c(-93.77, 42.02), 
                              years = 2013)
  
  cmp <- compare_apsim_met(iem, pwr[, 1:6], dmt[, 1:6], 
                           labels = c("IEM", "POWER", "DAYMET"))
  
  cmp <- compare_apsim_met(iem, pwr[, 1:6], dmt[, 1:6], 
                           met.var = "rain",
                           labels = c("IEM", "POWER", "DAYMET"))

  plot(cmp)
  plot(cmp, plot.type = "ts")
  plot(cmp, plot.type = "ts", met.var = "rain", cumulative = TRUE)

}

#### Different graph tests ----
if(run.apsim.met && internet){
  
  ## Testing the graph functionality
  pwr <- get_power_apsim_met(lonlat = c(-93.77, 42.02), 
                             dates = c("1990-01-01","2020-12-31"))
  
  ### default plotting options
  plot(pwr)
  plot(pwr, climatology = TRUE)
  plot(pwr, climatology = TRUE, years = 2015:2018)
  plot(pwr, climatology = TRUE, cumulative = TRUE)
  plot(pwr, climatology = TRUE, years = 2015:2018, cumulative = TRUE)
  
  plot(pwr, summary = TRUE)
  plot(pwr, summary = TRUE, climatology = TRUE)
  plot(pwr, summary = TRUE, met.var = "rain", climatology = TRUE)
  
  ## Plotting using the density option ----
  plot(pwr, plot.type = "density")
  plot(pwr, plot.type = "density", met.var = "radn")
  plot(pwr, plot.type = "density", met.var = "rain")
  
  ## Plotting using the density option
  plot(pwr, plot.type = "density", climatology = TRUE)
  plot(pwr, plot.type = "density", met.var = "radn", climatology = TRUE)
  plot(pwr, plot.type = "density", met.var = "rain", climatology = TRUE)
  
  plot(pwr, plot.type = "density", climatology = TRUE, years = c(2012, 2019))
  plot(pwr, plot.type = "density", met.var = "radn", climatology = TRUE, years = c(2012, 2019))
  plot(pwr, plot.type = "density", met.var = "rain", climatology = TRUE, years = c(2012, 2019))

  plot(pwr, plot.type = "density", summary = TRUE)
  plot(pwr, plot.type = "density", met.var = "radn_sum", summary = TRUE)
  plot(pwr, plot.type = "density", met.var = "rain", summary = TRUE)
  
  ## Testing the options with individual years
  plot(pwr, plot.type = "density", summary = TRUE, years = c(2012:2015))
  plot(pwr, plot.type = "density", met.var = "radn", summary = TRUE, years = c(2012, 2019))
  plot(pwr, plot.type = "density", met.var = "rain", summary = TRUE, years = c(2015, 2017))
  
  ## Testing with climatology and years
  plot(pwr, plot.type = "density", met.var = "rain", summary = TRUE, climatology = TRUE, years = c(2012, 2019))
  plot(pwr, plot.type = "density", met.var = "radn_sum", summary = TRUE, climatology = TRUE, years = c(2012, 2019))
  plot(pwr, plot.type = "density", met.var = "avg_maxt", summary = TRUE, climatology = TRUE, years = c(2012, 2019))

}


#### Testing anomaly features ----
if(run.apsim.met && internet){
  
  ## Testing the graph functionality
  pwr <- get_power_apsim_met(lonlat = c(-93.77, 42.02), 
                             dates = c("1990-01-01","2020-12-31"))
  
  apwr1 <- summary(pwr, anomaly = TRUE)
  apwr2 <- summary(pwr, anomaly = "rain_sum")
  apwr3 <- summary(pwr, anomaly = "rain")
  apwr4 <- summary(pwr, anomaly = "maxt")
  apwr5 <- summary(pwr, anomaly = c("maxt", "rain"))
  
  plot(pwr, plot.type = "anomaly")
  plot(pwr, plot.type = "anomaly", summary = TRUE, met.var = "rain")
  plot(pwr, plot.type = "anomaly", summary = TRUE, years = 2012:2015)
  
  plot(pwr, plot.type = "anomaly", summary = TRUE, met.var = c("avg_maxt", "rain_sum"))
  plot(pwr, plot.type = "anomaly", summary = TRUE, years = 2012:2015, met.var = c("avg_maxt", "rain_sum"))
  plot(pwr, plot.type = "anomaly", summary = TRUE, years = 2012:2015, met.var = c("rain_sum", "avg_maxt"))
  
  pwr <- tt_apsim_met(pwr, dates = c("01-05", "30-10"), method = "Classic_TT")
  
  summary(pwr)
  summary(pwr, compute.frost = TRUE)
  
  plot(pwr, met.var = "Classic_TT")
  plot(pwr, met.var = "Classic_TT", plot.type = "density")
  plot(pwr, met.var = "Classic_TT", plot.type = "anomaly", years = 2012:2019)
  
  plot(pwr, plot.type = "anomaly", summary = TRUE, met.var = c("rain", "high_classic_tt"))
  plot(pwr, plot.type = "anomaly", summary = TRUE, met.var = c("rain", "high_classic_tt"),
       years = 2012:2016)
}

if(run.apsim.met && internet){
  
  #### Will use this code to identify the potential issue with 
  #### modifying a iem.met object
  iem.met <- get_iem_apsim_met(lonlat = c(-93.77, 42.02), 
                                dates = c("2000-01-01","2020-12-31"))
  
  iem.met <- tt_apsim_met(iem.met, method = "Classic_TT", dates = c("1-5", "31-10"))
  
  names(iem.met)
  ### This works as expected now and it removes the units properly
  iem.met$Classic_TT <- NULL 
    
  iem.met <- tt_apsim_met(iem.met, method = "Classic_TT", dates = c("1-5", "31-10"))
    
  iem.met <- remove_column_apsim_met(iem.met, "Classic_TT")

  ### This should return an error  
  try(iem.met$hola <- 100, silent = TRUE)
  
}

### Next test should be for creating the 'by' argument

#### Comparing met files with different year ranges ----

if(run.apsim.met){
  
  pwr1 <- get_power_apsim_met(lonlat = c(-93, 42), dates = c("2000-01-01", "2010-12-31"))
  pwr2 <- get_power_apsim_met(lonlat = c(-93, 42), dates = c("2011-01-01", "2024-12-31"))
  
  ### This has a better error message
  merge.test1 <- try(compare_apsim_met(pwr1, pwr2), silent = TRUE)
  
  ### What about one year in common?
  
  pwr3 <- get_power_apsim_met(lonlat = c(-93, 42), dates = c("2000-01-01", "2011-12-31"))
  
  ### This should work but it should give a message
  merge.test2 <- compare_apsim_met(pwr2, pwr3)
  
  dmt <- get_daymet_apsim_met(lonlat = c(-93, 42), years = c(2000, 2023))
  
  merge.test3 <- try(compare_apsim_met(pwr3, dmt, labels = c("POWER", "DAYMET")), silent = TRUE)
  ### The number of columns is the same
  ### This gives an error because some column names are different
  
  merge.test4 <- compare_apsim_met(pwr3[, 1:6], dmt[, 1:6], labels = c("POWER", "DAYMET"))
  plot(merge.test4, met.var = "rain", plot.type = "ts", cumulative = TRUE)
  plot(merge.test4, plot.type = "ts")
  
  #### How can I fix the previous gam fit?
  mm4 <- merge.test4$met.mrg
  
  ggplot(data = mm4, aes(x = dates, y = radn.1)) + 
    geom_point() + 
    geom_smooth(formula = y ~ s(x, bs = "cs", k = 70, fx = TRUE))
  
  
  merge.test5 <- compare_apsim_met(pwr3[, 1:6], dmt[, 1:6], labels = c("POWER", "DAYMET"),
                                   check = FALSE)
  
  plot(merge.test5, met.var = "rain", plot.type = "ts", cumulative = TRUE)
  
  ### Scramble the order of columns
  dmt.attr <- attributes(dmt)
  dmt2 <- subset(dmt, select = names(dmt)[8:1])
  attr(dmt2, "filename") <- dmt.attr$filename
  attr(dmt2, "site") <- dmt.attr$site
  attr(dmt2, "latitude") <- dmt.attr$latitude
  attr(dmt2, "longitude") <- dmt.attr$longitude
  attr(dmt2, "tav") <- dmt.attr$tav
  attr(dmt2, "colnames") <- rev(dmt.attr$colnames)
  attr(dmt2, "units") <- dmt.attr$units
  attr(dmt2, "comments") <- dmt.attr$comments
  attr(dmt2, "class") <- dmt.attr$class
  attr(dmt2, "amp") <- dmt.attr$amp
  head(dmt2)

  merge.test6 <- try(compare_apsim_met(pwr3, dmt2, labels = c("POWER", "DAYMET")), silent = TRUE)
  ### this results in an error as it should

  merge.test7 <- compare_apsim_met(pwr3[, 1:6], dmt2[, 3:8], labels = c("POWER", "DAYMET"))  
  plot(merge.test7)
  plot(merge.test7, met.var = "rain", plot.type = "ts", cumulative = TRUE)
  
  summary(pwr3)
  summary(dmt2)
  
  plot(pwr3, summary = TRUE, climatology = TRUE)
  plot(dmt2, summary = TRUE, climatology = TRUE)

  plot(pwr3, met.var = "rain", summary = TRUE, climatology = TRUE)
  plot(dmt2, met.var = "rain", summary = TRUE, climatology = TRUE)
  
  ### Testing the extractor operation, this works now
  ### DO I need to create other methods?
  ### I have created `[<-.met` and `[[<-.met`
  ### which basically send an error message and ask instead to use 'add_column_apsim_met'
  dmt5 <- dmt[, 1:4]
  class(dmt5)
  dmt6 <- dmt[1:365, 1:4]
  class(dmt6)
  
  summary(dmt6)
  
}

if(FALSE){
  
  ### How the heck does the 'Extract' work in 'met' files?
  ## Testing adding a column
  extd.dir <- system.file("extdata", package = "apsimx")
  ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
  
  ## ames$vp <- 5 This does not work and it is not supposed to 
  
  vp <- data.frame(vp = abs(rnorm(nrow(ames), 2)))
  attr(vp, "units") <- "(hPa)"
  
  ames$vp <- vp ## This works now (2026-6-18)
  ames[["vp"]] <- vp ## This works now (2026-6-18)
  
  
}

#' 
#' ### Maybe insert information about regex at some point
#' 
#' @title Inspect a replacement component in an .apsimx (JSON) file
#' @name inspect_apsimx_replacement
#' @description inspect the replacement componenet of an JSON apsimx file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
#' @param file file ending in .apsimx to be inspected (JSON)
#' @param src.dir directory containing the .apsimx file to be inspected; defaults to the current working directory
#' @param node specific node to be inspected
#' @param node.child specific node child component to be inspected.
#' @param node.subchild specific node sub-child to be inspected.
#' @param node.subsubchild specific node sub-subchild to be inspected.
#' @param node.sub3child specific node sub3child to be inspected.
#' @param node.sub4child specific node sub4child to be inspected.
#' @param node.sub5child specific node sub5child to be inspected.
#' @param node.string passing of a string instead of the node hierarchy.
#' Do not use this and also the other node arguments. This argument will
#' overwrite the other node specifications.
#' @param root \sQuote{root} for the inspection of a replacement file (it gives flexibility to inspect other types of files).
#' In previous versions of APSIM (before mid 2023) this was \sQuote{Models.Core.Replacement}. In more recent versions, it needs
#' to be \sQuote{Models.Core.Folder}.
#' @param parm specific parameter to display. It can be a regular expression.
#' @param display.available logical. Whether to display available components to be inspected (default = FALSE)
#' @param digits number of decimals to print (default 3)
#' @param print.path print the path to the inspected parameter (default FALSE)
#' @param verbose whether to print additional information, default: TRUE
#' @param grep.options Additional options for grep. To be passed as a list. At the moment these are: \sQuote{fixed}, \sQuote{ignore.case} and \sQuote{exact}.
#' The option \sQuote{exact} is not a grep option, but it can be used to use exact matching (effectively not using grep).
#' This option will be ignored at the level of \sQuote{Command}.
#' @details This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
#' @note I need to make some changes in order to be able to handle multiple parameters. At this point, it
#' might work but it will generate warnings.
#' @return table with inspected parameters and values (and \sQuote{parm path} when \sQuote{print.path} = TRUE).
#' @export
#' @examples 
#' \donttest{
#' extd.dir <- system.file("extdata", package = "apsimx")
#' inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
#'                            node = "Maize", node.child = "Phenology",
#'                            node.subchild = "ThermalTime", 
#'                            node.subsubchild = "BaseThermalTime",
#'                            node.sub3child = "Response") 
#' ## For Wheat
#' ## getting down to 'XYPairs'
#' inspect_apsimx_replacement("WheatRye.apsimx", 
#'                            src.dir = extd.dir,
#'                            node = "Wheat",
#'                            node.child = "Structure",
#'                            node.subchild = "BranchingRate",
#'                            node.subsubchild = "PotentialBranchingRate",
#'                            node.sub3child = "Vegetative",
#'                            node.sub4child = "PotentialBranchingRate",
#'                            node.sub5child = "XYPairs")
#'}


inspect_apsimx_replacement <- function(file = "", src.dir = ".", 
                                       node = NULL, node.child = NULL,
                                       node.subchild = NULL, node.subsubchild = NULL,
                                       node.sub3child = NULL, node.sub4child = NULL,
                                       node.sub5child = NULL, node.string = NULL,
                                       root = list("Models.Core.Replacements", NA),
                                       parm = NULL, display.available = FALSE,
                                       digits = 3, print.path = FALSE,
                                       verbose = TRUE,
                                       grep.options){
  
  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
    .check_apsim_name(file)
    .check_apsim_name(normalizePath(src.dir))
  }

  file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case = TRUE)
  
  if(length(file.names) == 0){
    stop("There are no .apsimx files in the specified directory to inspect.")
  }
  
  ## This matches the specified file from a list of files
  ## Notice that the .apsimx extension will be added here
  file <- match.arg(file, file.names)
  
  apsimx_json <- jsonlite::read_json(file.path(src.dir, file))
  
  parm.path.0 <- paste0(".", apsimx_json$Name)
  ans <- parm.path.0
  ## Select Replacements node
  frn <- grep(root[[1]], apsimx_json$Children, fixed = TRUE)
  
  if(length(frn) == 0){
    ### Here we try a different root
    root <- list('Models.Core.Folder', NA)
    frn <- grep(root[[1]], apsimx_json$Children, fixed = TRUE)  
    if(length(frn) == 0)
      stop("Could not find 'root' for replacements", call. = FALSE)
  }
  
  if(!missing(grep.options)){
    gfixed <- grep.options$fixed
    gignore.case <- grep.options$ignore.case
    gexact <- grep.options$exact ### This makes matches exact, effectively not using grep
  }else{
    gfixed <- FALSE
    gignore.case <- FALSE
    gexact <- FALSE ### Default is FALSE meaning: use grep
  }

  if(length(frn) == 0) stop(paste0(root," not found"))
  
  if(length(frn) > 1){
    if(length(root) == 1){
      cat("root matched multiple positions", frn, "\n")
      stop("Change it so that it is unique", call. = FALSE)
    }else{
      if(is.na(root[[2]])){
        cat("These positions matched ", root[[1]], " ", frn, "\n")
        stop("Multiple root nodes found. Please provide a position", call. = FALSE)
      }else{
        if(length(root) == 2){
          if(is.numeric(root[[2]])){
            replacements.node <- apsimx_json$Children[[frn[root[[2]]]]]    
          }else{
            root.node.0.names <- sapply(apsimx_json$Children, function(x) x$Name)
            wcore1 <- grep(as.character(root[1]), root.node.0.names)
            if(length(wcore1) == 0 || length(wcore1) > 1)
              stop("no root node label in position 1 found or root is not unique")
            root.node.0 <- apsimx_json$Children[[wcore1]]
            root.node.0.child.names <- sapply(root.node.0$Children, function(x) x$Name)  
            wcore2 <- grep(as.character(root[2]), root.node.0.child.names)
            if(length(wcore2) == 0 || length(wcore2) > 1)
              stop("no root node label in position 2 found or root is not unique")
            replacements.node <- apsimx_json$Children[[wcore1]]$Children[[wcore2]]$Children 
            parm.path.0 <- paste0(parm.path.0, ".", apsimx_json$Children[[wcore1]]$Children[[wcore2]])
          }
        }
      }  
    }
  }else{
    replacements.node <- apsimx_json$Children[[frn]]
  }
  
  if(verbose){
    if(!grepl("replace", replacements.node$Name, ignore.case = TRUE))
      warning("Node does not appear to be a 'Replacements' node")
  }
  
  parm.path.0.1 <- paste0(parm.path.0, ".", replacements.node$Name)
  ## Print names of replacements
  replacements.node.names <- vapply(replacements.node$Children, function(x) x$Name, 
                                    FUN.VALUE = "character")
  if(verbose) cat("Replacements: ", replacements.node.names, "\n")
  
  if(!missing(node.string)){
    ## strip the dot automatically if it is included
    if(strsplit(node.string, "")[[1]][1] == "."){
      node.string <- substring(node.string, 2)
    }
    nodes <- strsplit(node.string, ".", fixed = TRUE)[[1]]
    node <- nodes[1]
    if(!is.na(nodes[2])) node.child <- nodes[2]
    if(!is.na(nodes[3])) node.subchild <- nodes[3]
    if(!is.na(nodes[4])) node.subsubchild <- nodes[4]
    if(!is.na(nodes[5])) node.sub3child <- nodes[5]
    if(!is.na(nodes[6])) node.sub4child <- nodes[6]
    if(!is.na(nodes[7])) node.sub5child <- nodes[7]
  }
  ## This handles a missing node 'gracefully'
  if(missing(node)){
    parm.path <- parm.path.0.1
    if(!is.null(parm)) parm.path <- paste0(parm.path, ".", parm)
    if(print.path) cat("Parm path:", parm.path, "\n") 
    if(verbose) cat("Please provide a node \n")
    return(invisible(parm.path))
  } 
  
  if(verbose) cat("node:", node, "\n")

  wrn <- grep(node, replacements.node.names)
  if(length(wrn) == 0) stop("node not found")
  if(length(wrn) > 1) stop("More than one node found. Make it unique (see regular expressions)")
  rep.node <- replacements.node$Children[[wrn]] ## Is this robust enough?
  ## This last object is a list with Children
  
  parm.path.0.1.1 <- paste0(parm.path.0.1, ".", rep.node$Name)
  
  if(!is.null(rep.node$CropType) && verbose) cat("CropType", rep.node$CropType, "\n")
  
  ## This tries to handle the fact that ther paramter might be at this
  ## high of a level
  if(!missing(parm) && any(parm %in% names(rep.node))){
    unpack_node(rep.node, parm = parm, display.available = display.available)
  }
  
  ## Available node children
  rep.node.children.names <- vapply(rep.node$Children, function(x) x$Name,
                                    FUN.VALUE = "character")
  if(display.available){
    cat("Level: node \n")
    try(str_list(rep.node), silent = TRUE)
  } 
  
  ## If node.child is missing try to handle it gracefully
  if(missing(node.child)){
    parm.path <- parm.path.0.1.1
    if(!is.null(parm)) parm.path <- paste0(parm.path, ".", parm)
    if(print.path) cat("Parm path: ", parm.path, "\n") 
    if(verbose) cat("missing node.child \n")
    return(invisible(parm.path))
  } 
  
  if(verbose) cat("node child:", node.child, "\n")
  
  wrnc <- grep(node.child, rep.node.children.names)
  if(length(wrnc) == 0) stop("node.child not found")
  if(length(wrnc) > 1) stop("More than one child found. Make it unique (see regular expressions)")
  rep.node.child <- rep.node$Children[[wrnc]]
  
  parm.path.0.1.1.1 <- paste0(parm.path.0.1.1, ".", rep.node.child$Name)
  
  ## If children are missing display data at this level
  ## Conditions for stopping here:
  ## 1. There are no children OR parm is not null AND
  ## 2. node.subchild is missing
  if((length(rep.node.child$Children) == 0 || !is.null(parm)) && missing(node.subchild)){
    unpack_node(rep.node.child, parm = parm, display.available = display.available)
    parm.path <- parm.path.0.1.1.1
    if(print.path) cat("Parm path: ", format_parm_path(parm.path, parm), "\n")
    if(verbose) cat("no node sub-children available or parm not equal to null \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }else{
    if(display.available) try(str_list(rep.node.child), silent = TRUE)
  }
  
  if(verbose) cat("node subchild:", node.subchild, "\n")
  ## This is intended to be used to handle a missing node.subchild gracefully
  if(missing(node.subchild)){
    parm.path <- parm.path.0.1.1.1
    if(print.path) cat("Parm path: ", format_parm_path(parm.path, parm), "\n") 
    if(verbose) cat("missing node.subchild \n")
    return(invisible(format_parm_path(parm.path, parm)))
  } 
  
  rep.node.subchildren.names <- vapply(rep.node.child$Children, function(x) x$Name,
                                       FUN.VALUE = "character")
  if(isFALSE(gexact)){
    wrnsc <- grep(node.subchild, rep.node.subchildren.names, fixed = gfixed, ignore.case = gignore.case)  
  }else{
    wrnsc <- which(node.subchild == rep.node.subchildren.names)
  }
  if(length(wrnsc) == 0) stop("node.subchild not found")
  if(length(wrnsc) > 1) stop("More than one subchild found. Make it unique (see regular expressions)")
  rep.node.subchild <- rep.node.child$Children[[wrnsc]]
  
  parm.path.0.1.1.1.1 <- paste0(parm.path.0.1.1.1, ".", rep.node.subchild$Name)
  ## Let's just print this information somehow
  if(verbose) cat("Subchild Name: ", rep.node.subchild$Name, "\n")
  
  if((length(rep.node.subchild$Children) == 0 || !is.null(parm)) && missing(node.subsubchild)){
    unpack_node(rep.node.subchild, parm = parm, display.available = display.available)
    parm.path <- parm.path.0.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n")
    if(verbose) cat("no node sub-sub-children available or parm is not null \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }else{
    ## The problem here is that rep.node.subchild can either be
    ## named or nameless
    if(display.available) try(str_list(rep.node.subchild), silent = TRUE)
  }
  
  ## This is intended to be used to handle a missing node.subsubchild gracefully
  if(missing(node.subsubchild)){
    parm.path <- parm.path.0.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    if(verbose) cat("missing node.subsubchild \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  ## The thing here is that I need to be able to handle an object with either
  ## named Children or
  ## unnamed Children
  rep.node.subsubchildren.names <- vapply(rep.node.subchild$Children, function(x) x$Name,
                                          FUN.VALUE = "character")
  if(isFALSE(gexact)){
    wrnssc <- grep(node.subsubchild, rep.node.subsubchildren.names, fixed = gfixed, ignore.case = gignore.case)  
  }else{
    wrnssc <- which(node.subsubchild == rep.node.subsubchildren.names)
  }
  if(length(wrnssc) == 0) stop("node.subsubchild not found")
  if(length(wrnssc) > 1) stop("More than one subsubchild found. Make it unique (see regular expressions)")
  rep.node.subsubchild <- rep.node.subchild$Children[[wrnssc]]
  
  if(verbose) cat("Name sub-sub-child: ", rep.node.subsubchild$Name, "\n")
  parm.path.0.1.1.1.1.1 <- paste0(parm.path.0.1.1.1.1,".",rep.node.subsubchild$Name)
  
  rep.node.subsubchild.names <- names(rep.node.subsubchild)

  ## I also need to check that parameter is not in 'Command'
  ## ispic: Is Parameter In Command?
  if(length(rep.node.subsubchild$Command) > 0 && !is.null(parm)){
    ispic <- any(grepl(parm, unlist(rep.node.subsubchild$Command)))
  }else{
    ispic <- FALSE
  }
  
  if((length(rep.node.subsubchild$Children) == 0 || !is.null(parm)) && !ispic && missing(node.sub3child)){
      unpack_node(rep.node.subsubchild, parm = parm, display.available = display.available)
      ## It might be that 'unpack' is not always the best choice and simply cat_parm might be better
      parm.path <- parm.path.0.1.1.1.1.1
      if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
      if(verbose) cat("no node sub-sub-sub-children available or parm is not null \n")
      return(invisible(format_parm_path(parm.path, parm)))
  }
    
  if(is.null(parm) && is.null(node.sub3child)){ 
    if(length(rep.node.subsubchild.names) == 0){
        rep.node.sub3child <- rep.node.subsubchild$Children[[1]]
    }else{
        rep.node.sub3child <- rep.node.subsubchild$Children
        if(is.null(rep.node.sub3child) && !is.null(rep.node.subsubchild$Children)){
          rep.node.sub3child <- rep.node.subsubchild$Children[[1]]
        } 
        if(is.null(rep.node.sub3child)) stop("Something went wrong: rep.node.sub3child is null")
    }
    if(is.null(node.sub3child)){
      unpack_node(rep.node.sub3child, parm = NULL, display.available = display.available)
    }
  }
  
  if(!is.null(parm) && any(parm %in% rep.node.subsubchild.names)){
    if(isFALSE(gexact)){
      wrnsspc <- grep(parm, rep.node.subsubchild.names, fixed = gfixed, ignore.case = gignore.case)
    }else{
      wrnsspc <- which(parm == rep.node.subsubchild.names)  
    }
    rep.node.sub3child <- rep.node.subsubchild[wrnsspc]
    cat_parm(rep.node.sub3child, parm = parm)
  }else{
    if(!is.null(parm) && any(grepl(parm, unlist(rep.node.subsubchild$Command)))){
      if(verbose && gexact) message("'exact' is ignored when 'parm' is in 'Command'")
      wcp <- grep(parm, unlist(rep.node.subsubchild$Command), fixed = gfixed, ignore.case = gignore.case)
      cat(unlist(rep.node.subsubchild$Command)[wcp], "\n")
    }else{
      if(!is.null(parm) && missing(node.sub3child)) stop("Parameter not found")
    }
  }
  
  ## At this point it seems that this should work
  ## Node at the third level should go here
  ## This is intended to be used to handle a missing node.subsubchild gracefully
  if(missing(node.sub3child)){
    parm.path <- parm.path.0.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    if(verbose) cat("missing node.sub3child \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  rep.node.sub3children.names <- vapply(rep.node.subsubchild$Children, function(x) x$Name,
                                       FUN.VALUE = "character")
  if(isFALSE(gexact)){
    wrnsssc <- grep(node.sub3child, rep.node.sub3children.names, fixed = gfixed, ignore.case = gignore.case)  
  }else{
    wrnsssc <- which(node.sub3child == rep.node.sub3children.names)
  }
  if(length(wrnsssc) == 0) stop("node.sub3child not found")
  if(length(wrnsssc) > 1) stop("More than one sub3child found. Make it unique (see regular expressions)")
  rep.node.sub4child <- rep.node.subsubchild$Children[[wrnsssc]]
  
  if(verbose) cat("Name sub-sub-sub-child: ", rep.node.sub4child$Name, "\n")
  parm.path.0.1.1.1.1.1.1 <- paste0(parm.path.0.1.1.1.1.1, ".", rep.node.sub4child$Name)

  ## Why does this work instead of the extraction through x$Name?
  rep.node.sub4child.names <- names(rep.node.sub4child)
  
  ## I also need to check that parameter is not in 'Command'
  ## ispic: Is Parameter In Command?
  if(length(rep.node.sub4child$Command) > 0 && !is.null(parm)){
    ispic <- any(grepl(parm, unlist(rep.node.sub4child$Command)))
  }else{
    ispic <- FALSE
  }
  
  if((length(rep.node.sub4child$Children) == 0 || !is.null(parm)) && !ispic){
    ## unpack_node(rep.node.sub4child, parm = parm, display.available = display.available)
    cat_parm(rep.node.sub4child, parm = parm)
    parm.path <- parm.path.0.1.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    ## if(verbose) cat("no node sub-sub-sub-sub-children available or parm is not null \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  if(is.null(parm) && is.null(node.sub4child)){ 
    if(length(rep.node.sub4child.names) == 0){
      rep.node.sub5child <- rep.node.sub4child$Children[[1]]
    }else{
      rep.node.sub5child <- rep.node.sub4child$Children  
      if(is.null(rep.node.sub5child) && !is.null(rep.node.sub5child$Children)){
        rep.node.sub5child <- rep.node.sub4child$Children[[1]]
      } 
      if(is.null(rep.node.sub5child)) stop("Something went wrong: rep.node.sub5child is null")
    }
    unpack_node(rep.node.sub5child, parm = NULL, display.available = display.available)
  }
  
  if(!is.null(parm) && any(grepl(parm, rep.node.sub4child.names))){
    if(isFALSE(gexact)){
      wrnssspc <- grep(parm, rep.node.sub4child.names, fixed = gfixed, ignore.case = gignore.case)  
    }else{
      wrnssspc <- which(parm == rep.node.sub4child.names)
    }
    
    rep.node.sub4child <- rep.node.sub4child[wrnssspc]
    cat_parm(rep.node.sub4child, parm = parm) ## This might be wrong, without parm argument
  }else{
    if(!is.null(parm) && any(grepl(parm, unlist(rep.node.sub4child$Command)))){
      if(verbose && gexact) message("'exact' is ignored when 'parm' is in 'Command'")
      wcp <- grep(parm, unlist(rep.node.sub4child$Command), fixed = gfixed, ignore.case = gignore.case)
      cat(unlist(rep.node.sub4child$Command)[wcp])
    }else{
      if(!is.null(parm)) stop("Parameter not found")
    }
  }

  if(missing(node.sub4child)){
    parm.path <- parm.path.0.1.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    if(verbose) cat("missing node.sub4child \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  #### Start of code chunck for node.sub4child ----
  rep.node.sub4children.names <- vapply(rep.node.sub4child$Children, function(x) x$Name,
                                        FUN.VALUE = "character")
  if(isFALSE(gexact)){
    wrnssssc <- grep(node.sub4child, rep.node.sub4children.names, fixed = gfixed, ignore.case = gignore.case)  
  }else{
    wrnssssc <- which(node.sub4child == rep.node.sub4children.names)
  }
  if(length(wrnssssc) == 0) stop("node.sub4child not found")
  if(length(wrnssssc) > 1) stop("More than one sub4child found. Make it unique (see regular expressions)")
  rep.node.sub5child <- rep.node.sub4child$Children[[wrnssssc]]
  
  if(verbose) cat("Name sub-sub-sub-subchild: ", rep.node.sub5child$Name, "\n")
  parm.path.0.1.1.1.1.1.1.1 <- paste0(parm.path.0.1.1.1.1.1.1,".",rep.node.sub5child$Name)

  ## Why does this work instead of the extraction through x$Name?
  rep.node.sub5child.names <- names(rep.node.sub5child)

  ## I also need to check that parameter is not in 'Command'
  ## ispic: Is Parameter In Command?
  if(length(rep.node.sub5child$Command) > 0 && !is.null(parm)){
    ispic <- any(grepl(parm, unlist(rep.node.sub5child$Command), fixed = gfixed, ignore.case = gignore.case))
  }else{
    ispic <- FALSE
  }
  
  if((length(rep.node.sub5child$Children) == 0 || !is.null(parm)) && !ispic){
    ## unpack_node(rep.node.sub4child, parm = parm, display.available = display.available)
    cat_parm(rep.node.sub5child, parm = parm)
    parm.path <- parm.path.0.1.1.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    return(invisible(format_parm_path(parm.path, parm)))
  }

  if(is.null(parm) && is.null(node.sub5child)){ 
    if(length(rep.node.sub5child.names) == 0){
      rep.node.sub6child <- rep.node.sub5child$Children[[1]]
    }else{
      rep.node.sub6child <- rep.node.sub5child$Children  
      if(is.null(rep.node.sub6child) && !is.null(rep.node.sub6child$Children)){
        rep.node.sub6child <- rep.node.sub5child$Children[[1]]
      } 
      if(is.null(rep.node.sub6child)) stop("Something went wrong: rep.node.sub6child is null")
    }
    unpack_node(rep.node.sub6child, parm = NULL, display.available = display.available)
  }  

  if(!is.null(parm) && any(grepl(parm, rep.node.sub5child.names))){
    if(isFALSE(gexact)){
      wrnsssspc <- grep(parm, rep.node.sub5child.names, fixed = gfixed, ignore.case = gignore.case)  
    }else{
      wrnsssspc <- which(parm == rep.node.sub5child.names)
    }
    rep.node.sub5child <- rep.node.sub4child[wrnsssspc]
    cat_parm(rep.node.sub5child, parm = parm) ## This might be wrong, without parm argument
  }else{
    if(!is.null(parm) && any(grepl(parm, unlist(rep.node.sub5child$Command)))){
      if(verbose && gexact) message("'exact' is ignored when 'parm' is in 'Command'")
      wcp <- grep(parm, unlist(rep.node.sub5child$Command), fixed = gfixed, ignore.case = gignore.case)
      cat(unlist(rep.node.sub5child$Command)[wcp])
    }else{
      if(!is.null(parm)) stop("Parameter not found")
    }
  }
  
  if(missing(node.sub5child)){
    parm.path <- parm.path.0.1.1.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    if(verbose) cat("missing node.sub5child \n")
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  #### End of node.sub4child code chunk ----
  
  #### Start of code chunck for node.sub5child ----
  rep.node.sub5children.names <- vapply(rep.node.sub5child$Children, function(x) x$Name,
                                        FUN.VALUE = "character")
  if(isFALSE(gexact)){
    wrnsssssc <- grep(node.sub5child, rep.node.sub5children.names, fixed = gfixed, ignore.case = gignore.case)
  }else{
    wrnsssssc <- which(node.sub5child == rep.node.sub5children.names)  
  }
  if(length(wrnsssssc) == 0) stop("node.sub5child not found")
  if(length(wrnsssssc) > 1) stop("More than one sub5child found. Make it unique (see regular expressions)")
  rep.node.sub6child <- rep.node.sub5child$Children[[wrnsssssc]]
  
  if(verbose) cat("Name Sub-sub-sub-sub-subchild: ", rep.node.sub6child$Name, "\n")
  parm.path.0.1.1.1.1.1.1.1.1 <- paste0(parm.path.0.1.1.1.1.1.1.1, ".", rep.node.sub6child$Name)
  
  ## Why does this work instead of the extraction through x$Name?
  rep.node.sub6child.names <- names(rep.node.sub6child)
  
  ## I also need to check that parameter is not in 'Command'
  ## ispic: Is Parameter In Command?
  if(length(rep.node.sub6child$Command) > 0 && !is.null(parm)){
    ispic <- any(grepl(parm, unlist(rep.node.sub6child$Command), fixed = gfixed, ignore.case = gignore.case))
  }else{
    ispic <- FALSE
  }
  
  if((length(rep.node.sub6child$Children) == 0 || !is.null(parm)) && !ispic){
    ## unpack_node(rep.node.sub4child, parm = parm, display.available = display.available)
    cat_parm(rep.node.sub6child, parm = parm)
    parm.path <- parm.path.0.1.1.1.1.1.1.1.1
    if(print.path) cat("Parm path:", format_parm_path(parm.path, parm), "\n") 
    return(invisible(format_parm_path(parm.path, parm)))
  }
  
  node.sub6child <- NULL
  if(is.null(parm) && is.null(node.sub6child)){ 
    if(length(rep.node.sub6child.names) == 0){
      rep.node.sub7child <- rep.node.sub6child$Children[[1]]
    }else{
      rep.node.sub7child <- rep.node.sub6child$Children  
      if(is.null(rep.node.sub7child) && !is.null(rep.node.sub7child$Children)){
        rep.node.sub7child <- rep.node.sub6child$Children[[1]]
      } 
      if(is.null(rep.node.sub7child)) stop("Something went wrong: rep.node.sub7child is null")
    }
    unpack_node(rep.node.sub7child, parm = NULL, display.available = display.available)
  }  
  
  if(!is.null(parm) && any(grepl(parm, rep.node.sub6child.names))){
    if(isFALSE(gexact)){
      wrnssssspc <- grep(parm, rep.node.sub6child.names, fixed = gfixed, ignore.case = gignore.case)  
    }else{
      wrnssssspc <- which(parm == rep.node.sub6child.names)
    }
    rep.node.sub6child <- rep.node.sub5child[wrnssssspc]
    cat_parm(rep.node.sub6child, parm = parm) ## This might be wrong, without parm argument
  }else{
    if(!is.null(parm) && any(grepl(parm, unlist(rep.node.sub6child$Command)))){
      if(verbose && gexact) message("'exact' is ignored when 'parm' is in 'Command'")
      wcp <- grep(parm, unlist(rep.node.sub6child$Command), fixed = gfixed, ignore.case = gignore.case)
      cat(unlist(rep.node.sub6child$Command)[wcp])
    }else{
      if(!is.null(parm)) stop("Parameter not found")
    }
  }
  
  if(is.null(node.sub6child)) parm.path <- parm.path.0.1.1.1.1.1.1.1.1
  
  #### End of node.sub4child code chunk ----
      
  if(print.path){
    if(is.null(parm)){
      ans <- parm.path
    }else{
      if(length(parm) == 1){
        ans <- paste0(parm.path, ".", parm)
      }else{
        ans <- paste0(parm.path, ".", parm[[1]])
      }
    }
    cat("Parm path:", parm.path, "\n")
  }
  invisible(ans)
}


## I will use this function to unpack a node when it is time to print
## Not exported
unpack_node <- function(x, parm = NULL, display.available = FALSE){
  
  if(!is.list(x)) stop("x should be a list")
  ## I will do just three levels of unpacking
  ## x is a node
  lnode <- length(x)
  
  node.names <- names(x)
  if(display.available) cat("Available node children (unpack_node): ", node.names, "\n")
  
  ## Let's try to handle the different elements that x can be
  ## If it is a list of length one and just one element, just cat
  ## the key, value pair
  if(is.list(x) & lnode == 1 & length(x[[1]]) == 1){
    return(cat("Key: ", names(x), "; Value: ", x[[1]], "\n"))
  }
  
  if(all(sapply(x, is.atomic)) || (!is.null(parm) && parm %in% names(x))){
    ## This is for a list which has all atomic elements
    ## If one of the elements is an empty list
    ## This will fail
    return(cat_parm(x, parm = parm))
  }
  
  for(i in seq_len(lnode)){
    ## The components can either have multiple elements such as
    ## Command, or have children, in either case I need to unpack
    node.child <- x[i]
    ## If x is a list node.child will be a list
    ## If it has just one element, then 
    if(is.list(node.child) && 
       length(node.child$Children) == 0 &&
       length(node.child) == 1 && 
       length(node.child[[1]]) == 1){
          cat_parm(node.child, parm = parm)
    }else{
      ## Let's assume this is a list with multiple elements
      ## Shouldn't 'cat_parm' be able to handle this?
      if(length(node.child) == 1 & length(x[[i]]) > 1){
        ## This is an element such as 'Command'
        tcp <- try(cat_parm(x[[i]], parm = parm), silent = TRUE)
        if(!inherits(tcp, "try-error")){
          cat(names(node.child), "\n")
          tcp
        }else{
          print(unlist(x[[i]]))
          print(parm)
          stop("There was an error in cat_parm", call. = FALSE)
        }
      }
      ## Let's handle 'Children' now
      if(length(node.child$Children) != 0){
        for(j in seq_along(node.child$Children)){
          if(names(node.child) == 0 || !is.null(node.child$Children[[1]])){
            node.subchild <- node.child$Children[[1]]
          }else{
            node.subchild <- node.child$Children
          }
          try(cat_parm(node.subchild, parm = parm), silent = TRUE)
        }
      }
    }
  }
}
      
        
cat_parm <- function(x, parm = NULL){

  ## This will print an element or multiple elements 
  ## When x is a simple list structure, with no 
  ## Children
  lx <- length(x)
  x.nms <- names(x)
  for(i in seq_len(lx)){
    if(is.null(parm)){
      cat(x.nms[i], ":", unlist(x[i]), "\n")
    }else{
      ## First case is name is in parm
      if(!is.null(x.nms[i])){
        if(grepl(parm, x.nms[i]))
           cat(x.nms[i], ":", unlist(x[i]), "\n")
      }else{
        ## Second case is
        ulx <- unlist(x[i])
        if(any(sapply(parm, function(x1) grepl(x1, ulx)))){
          cat(x.nms[i], ":", unlist(x[i]), "\n")  
        }
      }
    }
  }
}

## list structure
str_list <- function(x){
  cNms <- NA
  ## List name
  cat("list Name:",x$Name,"\n")
  ## Number of elements
  ln <- length(x)
  cat("list length:",ln,"\n")
  ## What are the names of the elements
  lnms <- names(x)
  cat("list names:",lnms,"\n")
  ## Are Children present?
  if(!is.null(x$Children)){
    cat("Children: Yes \n")
    cln <- length(x$Children)
    cat("Children length:",cln,"\n")
    cnms <- names(x$Children)
    if(length(cnms) != 0) cat("Children names:",cnms,"\n")
    if(length(cnms) == 0 && cln > 0){
      cNms <- sapply(x$Children, function(x) x$Name)
      if(length(cNms) > 0) cat("Children Names:",cNms,"\n")
    }
  }
  invisible(list(ln=ln,lnms=lnms,cln=cln,cnms=cnms,cNms=cNms))
}

format_parm_path <- function(x, parm = NULL){
  if(is.null(parm)){
    ans <- x
  }else{
    if(length(parm) == 1){
      ans <- paste0(x,".",parm)
    }else{
      ans <- paste0(x,".",parm[[1]])
    }    
  }
 ans
}

## Note: In the package I distribute some files which have additional 'replacements'
## The specific replacement needs to be incorporated by copying code from
## ApsimX/Models/Resources/
## Or: https://github.com/APSIMInitiative/ApsimX/tree/master/Models/Resources

#'
#' This function requires the \CRANpkg{nasapower} package version 4.0.0.
#' 
#' It looks like the earliest year you can request data for is 1984.
#'
#' @title Get NASA-POWER data for an APSIM met file
#' @description Uses \code{\link[nasapower]{get_power}} from the \CRANpkg{nasapower} package to download data to create an APSIM met file.
#' @name get_power_apsim_met
#' @param lonlat Longitude and latitude vector
#' @param dates date ranges
#' @param wrt.dir write directory
#' @param filename file name for writing out to disk
#' @return returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
#' @details If the filename is not provided it will not write the file to disk, 
#' but it will return an object of class \sQuote{met}. This is useful in case manipulation
#' is required before writing to disk.
#' @export
#' @examples 
#' \dontrun{
#' require(nasapower)
#' ## This will not write a file to disk
#' pwr <- get_power_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
#' ## Let's insert a missing value
#' pwr[100, "radn"] <- NA
#' summary(pwr)
#' ## Check the met file 
#' check_apsim_met(pwr)
#' ## Impute using linear interpolation
#' pwr.imptd <- impute_apsim_met(pwr, verbose = TRUE)
#' summary(pwr.imptd)
#' check_apsim_met(pwr.imptd)
#' }
#' 

get_power_apsim_met <- function(lonlat, dates, wrt.dir = ".", filename = NULL){
  
  if(!requireNamespace("nasapower", quietly = TRUE)){
    warning("The nasapower package is required for this function")
    return(NULL)
  }
  
  if(packageVersion("nasapower") <= '3.0.1'){
   stop("Please upgrade the 'nasapower' package to the latest version", call. = FALSE) 
  }
  
  if(missing(filename)) filename <- "noname.met"
   
  if(!grepl(".met", filename, fixed = TRUE)) stop("filename should end in .met")
  
  pwr <- nasapower::get_power(community = "AG",
                                pars = c("T2M_MAX",
                                         "T2M_MIN",
                                         "ALLSKY_SFC_SW_DWN",
                                         "PRECTOTCORR",
                                         "RH2M",
                                         "WS2M"),
                                dates = dates,
                                lonlat = lonlat,
                                temporal_api = "daily")    

  pwr <- subset(as.data.frame(pwr), select = c("YEAR", "DOY",
                                               "ALLSKY_SFC_SW_DWN",
                                               "T2M_MAX", "T2M_MIN",
                                               "PRECTOTCORR", "RH2M", "WS2M"))
  
  names(pwr) <- c("year", "day", "radn", "maxt", "mint", "rain", "rh", "windspeed")
  units <- c("()", "()", "(MJ/m2/day)", "(oC)", "(oC)", "(mm)", "(%)", "(m/s)")
  
  comments <- paste("!data from nasapower R package. retrieved: ", Sys.time())
    
  ## Calculating annual amplitude in mean monthly temperature

  attr(pwr, "filename") <- filename
  attr(pwr, "site") <- paste("site =", sub(".met", "", filename, fixed = TRUE))
  attr(pwr, "latitude") <- paste("latitude =", lonlat[2])
  attr(pwr, "longitude") <- paste("longitude =", lonlat[1])
  attr(pwr, "colnames") <- names(pwr)
  attr(pwr, "units") <- units
  attr(pwr, "comments") <- comments
  ## No constants
  class(pwr) <- c("met", "data.frame")
  
  pwr <- amp_apsim_met(pwr)
  pwr <- tav_apsim_met(pwr)
  
  if(filename != "noname.met"){
    write_apsim_met(pwr, wrt.dir = wrt.dir, filename = filename)
  }
  return(invisible(pwr))
}

## Testing the get_daymet_apsim_met function
## Note (2020-02-09)
## I've ran this test several times now on Mac and Windows
## I won't run it again until it is needed
## However, note that the source of weather data can
## have a large impact on final yield estimates
## the difference is 171.1547 g per meter squared
## with higher values for power over daymet
## Note (2020-06-15): This takes for ever, I'm not running it for a while
require(apsimx)
apsimx_options(warn.versions = FALSE)

run.test.daymet0 <- FALSE

run.test.daymet <- get(".run.local.tests", envir = apsimx.options) & run.test.daymet0

username <- Sys.info()[["user"]]

tmp.dir <- tempdir()

if(run.test.daymet && username == "fernandomiguez"){

  lonlat <- c(-93, 42)
  
  dmet <- get_daymet_apsim_met(lonlat = lonlat, 
                               years = 2015:2016,
                               width.height = c(1e-5,1e-5),
                               raw.dir = paste0(tmp.dir,"/RAW/DAYMET"),
                               extraction.dir = paste0(tmp.dir, "/EXTRACTIONS/CIA/DAYMET"),
                               label = "CIA",
                               filename = "cia-daymet.met",
                               cleanup = FALSE)  
  
  dmet2 <- get_daymet2_apsim_met(lonlat = lonlat, years = 2015:2016)
  
  cia.daymet <- read_apsim_met("/cia-daymet.met", src.dir = tmp.dir)
  
  check_apsim_met(cia.daymet)
}

## Does the file produced by DayMet work in APSIM?

run.test.maize.daymet <- get(".run.local.tests", envir = apsimx.options) & run.test.daymet0

if(run.test.maize.daymet && username == "fernandomiguez"){

  extd.dir <- system.file("extdata", package = "apsimx")

  edit_apsimx("Maize.apsimx", 
              node = "Clock", 
              src.dir = extd.dir, 
              wrt.dir = tmp.dir,
              parm = c("Start","End"),
              value = c("2015-01-01","2016-12-30"),
              overwrite = FALSE)

  inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir, node = "Clock")

  edit_apsimx("Maize-edited.apsimx", 
              src.dir = tmp.dir, wrt.dir = tmp.dir,
              node = "Weather", 
              value = "cia-daymet.met",
              overwrite = TRUE)

  inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir, node = "Weather")

  maize.daymet <- apsimx("Maize-edited.apsimx", src.dir = tmp.dir, value = "report")

}

## What if I get data from POWER?

run.test.power <- get(".run.local.tests", envir = apsimx.options) & run.test.daymet0

if(run.test.power && username == "fernandomiguez"){

  lonlat <- c(-93, 42)

  pwr <- get_power_apsim_met(lonlat = lonlat, wrt.dir = tmp.dir,
                             dates = c("2015-01-01","2016-12-31"),
                             filename = "cia-power.met")
  
  pwr.met <- read_apsim_met("cia-power.met", src.dir = tmp.dir)
  
  check_apsim_met(pwr.met)
  
  pwr.met$radn <- ifelse(pwr.met$radn == -99, NA, pwr.met$radn)
    
  pwr.met2 <- impute_apsim_met(pwr.met, verbose = TRUE)
  
  check_apsim_met(pwr.met2)
  
  write_apsim_met(pwr.met2, wrt.dir = tmp.dir, filename = "cia-power2.met")
}

run.test.maize.power <- get(".run.local.tests", envir = apsimx.options) & run.test.daymet0

if(run.test.maize.daymet && username == "fernandomiguez"){
  
  edit_apsimx("Maize-edited.apsimx", node = "Weather", 
              value = "cia-power2.met", src.dir = tmp.dir,
              edit.tag = "-power")
  
  maize.power <- apsimx("Maize-edited-power.apsimx", 
                        src.dir = tmp.dir, value = "report")
  
  yld.diff <- maize.power$Maize.Grain.Wt - maize.daymet$Maize.Grain.Wt
}

## Testing the edit function with a variety of files
require(apsimx)
packageVersion("apsimx")
apsimx_options(warn.versions = FALSE, allow.path.spaces = TRUE)

extd.dir <- system.file("extdata", package = "apsimx")

run.apsimx.edit <- get(".run.local.tests", envir = apsimx.options)

tmp.dir <- tempdir()

if(run.apsimx.edit){
  
  ## Inspect manager, Let's look at fertiliser first
  inspect_apsimx("maize-manager.apsimx", src.dir = extd.dir, 
                 node = "Manager", parm = list("SowingFert", NA))
  ## Inspect the fertilizer
  inspect_apsimx("maize-manager.apsimx", src.dir = extd.dir, 
                 node = "Manager", parm = list("SowingFert", 1))
  ## Generate the path
  pp <- inspect_apsimx("maize-manager.apsimx", src.dir = extd.dir, 
                       node = "Manager", parm = list("SowingFert",1),
                       print.path = TRUE)
  ## Edit by moving the file to the current directory
  edit_apsimx("maize-manager.apsimx", src.dir = extd.dir, 
              wrt.dir = tmp.dir, overwrite = TRUE,
              node = "Manager", manager.child = "SowingFertiliser",
              parm = "Amount", value = 150)
  ## Inspect the new file to make sure it worked
  inspect_apsimx("maize-manager.apsimx", src.dir = tmp.dir, 
                 node = "Manager", parm = list("SowingFert", 1))
  
  ## Test the maize manager folder
  edit_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir, 
              wrt.dir = tmp.dir, overwrite = TRUE,
              node = "Manager", manager.child = "SowingFertiliser",
              parm = "Amount", value = 150)
  
}

run.apsim.edit.millet <- get(".run.local.tests", envir = apsimx.options)

if(run.apsim.edit.millet){
  
  ## Inspect
  pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Manager",
                  parm = list("Sow on a fixed date",5), print.path = TRUE)
  ## Edit
  edit_apsim("Millet.apsim", src.dir = extd.dir, 
             node = "Other", wrt.dir = tmp.dir,
             overwrite = TRUE,
             parm.path = pp,
             value = 8)
  
  ## Try using manager.child
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Manager")
  ## This prints available manager childs
  inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Manager",
                parm = list("Sow on a fixed date",NA))
  
  edit_apsim("Millet.apsim", src.dir = extd.dir, 
              wrt.dir = tmp.dir, node = "Manager",
             manager.child = "Sow on a fixed date",
             edit.tag = "-smurf",
             parm = "cultivar",
             value = "smurfs")
  
  ## Testing with Maize factorial
  pmf <- inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
                       root = "IA-CC_Canisteo_Cover",
                       node = "Weather", print.path = TRUE)

  ## Testing with the maize-factorial.apsim file
  inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
                node = "Weather",
                root = "IA-CC_Canisteo_Cover")

  inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
                node = "Weather",
                root = "IA-CC_Canisteo_No-Cover")

  edit_apsim("maize-factorial.apsim", 
             src.dir = extd.dir,
             wrt.dir = tmp.dir,
             node = "Weather", 
             root = "IA-CC_Canisteo_Cover",
             value = "Ames.met")
  
  inspect_apsim("maize-factorial-edited.apsim", 
                src.dir = tmp.dir, 
                node = "Weather",
                root = "IA-CC_Canisteo_Cover")
  
  edit_apsim("maize-factorial-edited.apsim", 
             src.dir = tmp.dir,
             overwrite = TRUE,
             node = "Weather", 
             root = "IA-CC_Canisteo_No-Cover",
             value = "Boone.met")
  
  inspect_apsim("maize-factorial-edited.apsim", 
                src.dir = tmp.dir, 
                node = "Weather",
                root = "IA-CC_Canisteo_No-Cover")
}

run.apsimx.edit.maize.soil <- get(".run.local.tests", envir = apsimx.options)

if(run.apsimx.edit.maize.soil){
  
  inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
                 node = "Soil", parm = "Site", print.path = TRUE)
  
  edit_apsimx("Wheat.apsimx", 
              src.dir = extd.dir, wrt.dir = tmp.dir,
              node = "Soil", soil.child = "Metadata", 
              parm = "Site", value = "Ames")
  
  inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, 
                 node = "Soil")
  
}

run.apsimx.edit.soil.soilwat <- get(".run.local.tests", envir = apsimx.options)

if(run.apsimx.edit.soil.soilwat){

  inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
                 node = "Soil", soil.child = "SoilWater", 
                 parm = "SummerCona")
    
  edit_apsimx("Wheat.apsimx", 
              src.dir = extd.dir, wrt.dir = tmp.dir,
              node = "Soil", soil.child = "SoilWater", 
              parm = "SummerCona", value = 6)
  
  inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, 
                 node = "Soil", soil.child = "SoilWater", 
                 parm = "SummerCona")

  pp <- inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir,
                       node = "Soil", soil.child = "SoilWater",
                       parm = "SummerCona", print.path = TRUE)  
  
  edit_apsimx("Wheat-edited.apsimx", 
              src.dir = tmp.dir, wrt.dir = tmp.dir,
              node = "Other", parm.path = pp, value = 7,
              overwrite = TRUE)
  
  inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, 
                 node = "Soil", soil.child = "SoilWater", 
                 parm = "SummerCona")
  
}

## Testing Report feature
if(run.apsimx.edit){
  
 ex.dir <- auto_detect_apsimx_examples()
 
 inspect_apsimx("Wheat.apsimx", src.dir = ex.dir, node = "Report")
 
 edit_apsimx("Wheat.apsimx", src.dir = ex.dir,
             node = "Report", wrt.dir = tmp.dir,
             parm = "VariableNames",
             value = "[Soil].esw")
 
 inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, node = "Report")
 
 edit_apsimx("Wheat.apsimx", src.dir = ex.dir,
             node = "Report", wrt.dir = tmp.dir,
             parm = "EventNames",
             value = "[Wheat].EndOfDay")
 
 inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir, node = "Report")
 
 ## Running soil carbon
 edit_apsimx("Wheat.apsimx", src.dir = ex.dir,
             node = "Report", wrt.dir = tmp.dir,
             parm = "VariableNames",
             value = "[Soil].Nutrient.Organic.C",
             edit.tag = "-carbon")
 
 inspect_apsimx("Wheat-carbon.apsimx", src.dir = tmp.dir, node = "Report")
 
 sim.o <- apsimx("Wheat-carbon.apsimx", src.dir = tmp.dir)
 
 edit_apsimx("Soybean.apsimx", src.dir = ex.dir,
             node = "Report", wrt.dir = tmp.dir,
             parm = "VariableNames",
             value = "[Soil].Nutrient.Organic.C",
             edit.tag = "-carbon")
 
 inspect_apsimx("Soybean-carbon.apsimx", src.dir = tmp.dir, node = "Report")
 
 sim.o <- apsimx("Soybean-carbon.apsimx", src.dir = tmp.dir)
  
}

#### Testing edit when node = "Other" ----

if(run.apsimx.edit){
  
  extd.dir <- system.file("extdata", package = "apsimx")
  
  pp <- inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager",
                       parm = list("Fert", NA))
  
  edit_apsimx("Wheat.apsimx", node = "Other",
              src.dir = extd.dir, wrt.dir = tmp.dir,
              parm.path = pp, parm = "Amount", value = 5,
              overwrite = TRUE)

  inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, node = "Manager",
                 parm = list("Fert", NA))  
  
  file.copy(file.path(tmp.dir, "Wheat.apsimx"), ".")
  
  #### Testing edit when node == "Other" and there are multiple simulations
  ex.dir <- auto_detect_apsimx_examples()
  file.copy(file.path(ex.dir, "AgPasture.apsimx"), tmp.dir)
  
  inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir, node = "Other")
  inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir, node = "Other", parm = 2)
  
  edit_apsimx("AgPasture.apsimx", src.dir = tmp.dir, 
              node = "Other",
              parm.path = ".Simulations.AgPastureExample.Clock.Start",
              value = "2024-12-31T00:00:00")
  
  inspect_apsimx("AgPasture-edited.apsimx", src.dir = tmp.dir, 
                 root = list("AgPastureExample"))
  ### It might have worked?
  file.remove(file.path(tmp.dir, "AgPasture-edited.apsimx"))
  
  
}

## Testing changing variables in the outputfile
if(run.apsimx.edit){
 
  extd.dir <- system.file("extdata", package = "apsimx")
  file.copy(file.path(extd.dir, "Millet.apsim"), tmp.dir)
  
  inspect_apsim("Millet.apsim", src.dir = tmp.dir, node = "Outputfile", parm = "title")

  edit_apsim("Millet.apsim", src.dir = tmp.dir,
             node = "Outputfile", parm = "variables", 
             value = "surfaceom_wt")
    
  edit_apsim("Millet.apsim", src.dir = tmp.dir,
             node = "Outputfile",
             parm = "variables", value = "surfaceom_wt")
  
  inspect_apsim("Millet-edited.apsim", src.dir = tmp.dir, 
                node = "Outputfile", parm = "variables")
  
}

#### Testing editing of solute ----

if(run.apsimx.edit){
  
  ex.dir <- auto_detect_apsimx_examples()
  
  ex.to.test <- dir(ex.dir, pattern = "apsimx$")
  
  ### 
  ### ex2test <- ex.to.test[c(17, 18, 19, 25, 31, 32)]
  ex2test <- c("Maize.apsimx", "Mungbean.apsimx", "Oats.apsimx", "RedClover.apsimx", "Sorghum.apsimx", "Soybean.apsimx") 
  
  for(i in ex2test){
    
    cat("Example:", i, "\n")

    if(!file.exists(file.path(tmp.dir, i)))
      file.copy(file.path(ex.dir, i), tmp.dir)
    
    inspect_apsimx(i, src.dir = tmp.dir, node = "Soil",
                   soil.child = "Solute")
    
    edf.solute <- extract_data_apsimx(i, src.dir = tmp.dir, node = "Soil",
                   soil.child = "Solute")
    
    initialvalues.length <- nrow(edf.solute$NO3$first)
    
    edit_apsimx(i, src.dir = tmp.dir, wrt.dir = tmp.dir,
                node = "Soil", soil.child = "NO3",
                parm = "InitialValues", 
                value = rep(2, initialvalues.length))
    
    ii <- paste0(tools::file_path_sans_ext(i), "-edited.apsimx")
    inspect_apsimx(ii, src.dir = tmp.dir, node = "Soil",
                   soil.child = "Solute", parm = "NO3")
    
    ### The parameters in the second table cannot be edited ATM
    # edit_apsimx(i, src.dir = tmp.dir, wrt.dir = tmp.dir,
    #             node = "Soil", soil.child = "NO3",
    #             parm = "DepthConstant", 
    #             value = 1,
    #             edit.tag = "-DepthConstant")
  }
}

#### Testing edit.tag feature ----

if(run.apsimx.edit){
  
  ex.dir <- auto_detect_apsimx_examples()
  
  ex.to.test <- dir(ex.dir, pattern = "apsimx$")
  
  ex2test <- c("Maize.apsimx", "Mungbean.apsimx", "Oats.apsimx", "RedClover.apsimx", "Sorghum.apsimx", "Soybean.apsimx") 
  
  for(i in ex2test){
    
    cat("Example:", i, "\n")
    
    if(!file.exists(file.path(tmp.dir, i)))
      file.copy(file.path(ex.dir, i), tmp.dir)   
    
    edit_apsimx(i, src.dir = tmp.dir, wrt.dir = tmp.dir,
                node = "Weather",
                value = "temp.met",
                edit.tag = "-tempmet")
  }
}

#### Testing editing vectors -----
if(run.apsimx.edit){
  
  dir(extd.dir, pattern = "apsimx$")
  
  setwd(tmp.dir)
  
  file.copy(file.path(extd.dir, "Wheat-opt-ex.apsimx"), tmp.dir)
  
  inspect_apsimx_replacement("Wheat-opt-ex.apsimx",
                             node = "Wheat",
                             node.child = "Phenology",
                             node.subchild = "ThermalTime",
                             node.subsubchild = "Response",
                             parm = "X",
                             display.available = FALSE)
  
  pp1 <- inspect_apsimx_replacement("Wheat-opt-ex.apsimx",
                                    node = "Wheat",
                                    node.child = "Phenology",
                                    node.subchild = "ThermalTime",
                                    node.subsubchild = "Response",
                                    parm = "X",
                                    print.path = TRUE,
                                    display.available = FALSE)

  edit_apsimx("Wheat-opt-ex.apsimx",
              node = "Other",
              parm.path = pp1,
              value = "0 25 37")  
  
  inspect_apsimx_replacement("Wheat-opt-ex-edited.apsimx",
                             node = "Wheat",
                             node.child = "Phenology",
                             node.subchild = "ThermalTime",
                             node.subsubchild = "Response",
                             parm = "X",
                             display.available = FALSE)
  
  extract_values_apsimx("Wheat-opt-ex-edited.apsimx", src.dir = ".", parm = pp1)
  
  ## extract_data_apsimx("Wheat-opt-ex-edited.apsimx", node = "Other", parm = pp1)
  
  tmps <- expand.grid(tmp1 = 18:30, tmp2 = 33:45)
  card.temp.vector <- paste0("0 ", tmps$tmp1, " ", tmps$tmp2)
  grd <- data.frame(X = card.temp.vector)
    
  # sns <- sens_apsimx("Wheat-opt-ex-edited.apsimx",
  #                    parm.paths = )
  
  
}

#### Testing 'duplication' of APSIM nodes ----
if(run.apsimx.edit){
  
  dir(extd.dir, pattern = "apsimx$")
  
  setwd(tmp.dir)
  
  file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
  
  inspect_apsimx("Wheat.apsimx",
                  node = "Soil",
                  soil.child = "Physical")
  
  pp <- inspect_apsimx("Wheat.apsimx",
                       node = "Soil",
                       soil.child = "Physical",
                       parm = "Wheat LL",
                       print.path = TRUE)
  
  edit_apsimx("Wheat.apsimx",
              node = "Soil",
              soil.child = "Physical",
              parm = "Wheat LL",
              value = rep(0.25, 7))
  
  inspect_apsimx("Wheat-edited.apsimx",
                  node = "Soil",
                  soil.child = "Physical",
                  parm = "Wheat LL")
  
  
}

---
title: "Parameter Optimization"
author: "Fernando Miguez"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Parameter Optimization}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)
apsimx_options(warn.versions = FALSE)
tmp.dir <- tempdir()
```

# Introduction

Often the simulations from a model like APSIM will not be close enough to the observed data. APSIM (Classic and Next Generation) are very complex models with many moving pieces and unexpected interactions among parameters. If you decide that you need to optimize parameters I suggest answering these questions:

1. Why do you need to perform parameter optimization?

2. Which parameters do you need to optimize?

3. Are the observed measurements appropriate for the parameters you want to optimize?

I like to emphasize here the importance of thinking deeply about whether parameter optimization is needed in a given application. Before you use an optimization routine consider the following:

1. Did you control the quality of the weather data ('met' file)?

2. Are the soil properties appropriate for the location being simulated?

3. Is the management information accurate?

Clearly, if there are any problems, issues or inaccuracies which are derived from these three sources of information, then calibrating or tweaking crop parameters to compensate for those errors will not result in a useful model. In concrete terms, as an example: Are you modifying the radiation use efficiency parameter to compensate for incorrect planting date or fertilizer inputs? Running an optimization algorithm is easier than having to think hard about the scientific questions, the mechanisms and the structure of the model.

Given this disclaimer, I provide simple functions which connect APSIM with optimization routines. These are  **optim_apsim** and **optim_apsimx**. These functions work best for situations when the function being optimized is smooth (i.e. small changes in the inputs result in small changes in the output without jumps or discontinuities) and the parameters being optimized are continuous. This is: that both input parameters and outputs can take any real value. However, several of the optimization algorithms, such as the default 'Nelder-Mead' work for non-differentiable functions. An alternative is to use method 'grid' and provide a grid of parameters values at which to evaluate the model. This can be especially useful as a preliminary step.

I have been testing them and they work as expected. This means that very often they give you the right answer to the wrong question! For example, the optimized value for a parameter might not be within a physical or biological meaningful range if you are not careful. Let's look at the arguments:

## optim_apsim

* file = .apsim file
* src.dir = source directory. Should be the current directory for both this file and the crop file.
* crop.file = typically something like 'Wheat.xml', 'Maize.xml' or similar. You should have also placed an 'ini' under the crop in the .apsim file and point to this file.
* parm.paths = character vector with the full path to the parameters to be optimized. This can be obtained by using **inspect_apsim** or **inspect_apsim_xml**. The length of the parameter vector should be equal to the number of parameters being optimized (sometimes less is more). For example:

```{r inspect-apsimx-xml-maize}
extd.dir <- system.file("extdata", package = "apsimx")
rue.pth <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir, parm = "rue")
ext.pth <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir, parm = "y_extinct_coef")
## To pass them to optim_apsim, combine them
pp <- c(rue.pth, ext.pth)
```
Generates the paths for the radiation use efficiency and extinction coefficient paths. 

* data = data frame with the observed measurement which will be compared to APSIM output. It is important that the names match. This is the names given in the output of an APSIM simulation should have the same names as the column names in this data frame so that they can be aligned. Only one observation is allowed per Date.

* type = type of optimization. The default is to use **optim**. It is also possible to use the **nloptr** package, **mcmc** via the **BayesianTools** package, or **ucminf**. More recently, (since version 2.7.7) there is an option for **grid** which requires an input grid of parameter
values.

* weights = weights to apply when multiple variables are being optimized. See help file for more details.

* index = the default is 'Date' this allows the alignment of APSIM output and observed measurements. If you have multiple simulations or 'sites' that you want to optimize, the index might need to be c("outfile", "Date"). In this way both the different simulations and observations for a particular 'Date' can be aligned. However, you need to construct the file with observed data carefully so that they can be aligned. 

* parm.vector.index = some paths point to a vector of parameters, but sometimes only one element of that vector needs to be optimized. In this case, provide the index for which element of the vector needs to be optimized. If some of the parameters need this index while others do not, include a zero for those parameter vectors which are not indexed. 

* grid = a grid with columns as the parameters that will vary from simulation to simulation.
Rows should contain different parameter value combinations.

* ... = additional arguments to be passed to **optim**. For example, you can include lower and upper bounds and change the optimization algorithm (in this case 'L-BFGS-B'). Also, with 'hessian = TRUE' you can obtain confidence intervals. See **optim** help file and documentation. Other options for either **nloptr** or **mcmc**. Argument **verbose** can be passed as well.

## optim_apsimx

This function has a very similar structure to **optim_apsim**. The main difference is that at the moment it is required to supply the initial values for the parameters. It is also required to indicate whether each parameter is present in the 'replacement' component. As explained before, use **inspect_apsimx** or **inspect_apsimx_replacement** to find the parameter paths.

# Wheat Example

As a simple example, let's imagine we have collected some data on phenology, LAI and biomass of wheat. The data are already in the package and we just need to load it. These data were actually simulated with known parameter values.

## Observed data

```{r obsWheat}
data(obsWheat)
## See the structure
head(obsWheat)
## Only 10 observations
dim(obsWheat)
## Visualize the data
ggplot(obsWheat, aes(Date, Wheat.AboveGround.Wt)) + 
  geom_line() + 
  ggtitle("Biomass (g/m2)")
  
ggplot(obsWheat, aes(Date, Wheat.Leaf.LAI)) + 
  geom_line() +
  ggtitle("LAI")
  
ggplot(obsWheat, aes(Date, Wheat.Phenology.Stage)) + 
  geom_line() +
  ggtitle("Phenology Stages")
```

Notice that the names in the *obsWheat* data frame are identical to the names from the APSIM-X output and this is important later in the optimization so that we can match the variables.

## Before optimization

```{r wheat-sim-b4-opt, echo = FALSE}
sim0 <- read.csv(file.path(extd.dir, "wheat-sim-b4-opt.csv"))
sim0$Date <- as.Date(sim0$Date)
```

```{r sim0-wheat-sim, eval = FALSE}
sim0 <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir, value = "report")
```

```{r sim0-viz}
## Select 
sim0.s <- subset(sim0, 
                 Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))
## Visualize before optimization
## phenology
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Stage)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.Phenology.Stage)) + 
  ggtitle("Phenology")
## LAI
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
  ggtitle("LAI")
## Biomass
ggplot() + 
  geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
  geom_line(data = sim0.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")
```

## Setting up the optimization

We have observed data and we have ran the model. We notice that the agreement between observed and simulated could be better. For this example we will only be optimizing two parameters: RUE and phyllochron. We could use the function **inspect_apsimx_replacement** to find them. Then we construct the paths.

```{r inspect}
## Finding RUE
inspect_apsimx_replacement("Wheat-opt-ex.apsimx", src.dir = extd.dir,
                           node = "Wheat", 
                           node.child = "Leaf",
                           node.subchild = "Photosynthesis",
                           node.subsubchild = "RUE", 
                           parm = "FixedValue",
                           verbose = FALSE)
## Finding BasePhyllochron
inspect_apsimx_replacement("Wheat-opt-ex.apsimx", src.dir = extd.dir,
                           node = "Wheat", 
                           node.child = "Cultivars",
                           node.subchild = "USA",
                           node.subsubchild = "Yecora", 
                           verbose = FALSE)
## Constructing the paths is straight-forward
pp1 <- "Wheat.Leaf.Photosynthesis.RUE.FixedValue"
pp2 <- "Wheat.Cultivars.USA.Yecora.BasePhyllochron"
```

## Running the optimization

Once we have set up these pieces we can run the optimization. Here, the first argument is the name of the file, the second the directory where it is (*src.dir*), then the paths indicating the parameters to optimize, the observed data (*data*), the weighting method (here = *mean*), whether the parameters are in the replacement part of the simulation and the initial values.

```{r optim-apsimx, eval = FALSE}
## wop is for wheat optimization
wop <- optim_apsimx("Wheat-opt-ex.apsimx", 
                    src.dir = extd.dir, 
                    parm.paths = c(pp1, pp2),
                    data = obsWheat, 
                    weights = "mean",
                    replacement = c(TRUE, TRUE),
                    initial.values = c(1.2, 120))
```

This optimization ran for about 7 minutes (on my laptop). This is the result:

```{r load-wop, echo = FALSE}
data("wheat-opt-ex", package = "apsimx")
```

```{r wop-result}
wop
```

We can see that the optimized values are very close to the known values which were used to simulate the data. The known value for RUE was 1.5 $g \; MJ^{-1}$ and for BasePhyllochron it was 90 GDD.

In addition, it is good practice to compute the Hessian matrix, since it provides information about standard errors and confidence intervals.

```{r optim-apsimx-hessian, eval = FALSE}
## wop is for wheat optimization
wop.h <- optim_apsimx("Wheat-opt-ex.apsimx", 
                      src.dir = extd.dir, 
                      parm.paths = c(pp1, pp2),
                      data = obsWheat, 
                      weights = "mean",
                      replacement = c(TRUE, TRUE),
                      initial.values = c(1.2, 120),
                      hessian = TRUE)
```

This ran for about 8 minutes and it allows for calculation of confidence intervals and standard errors.

```{r wop-result-hessian}
wop.h
```

After optimization there is good agreement between observed and simulated.

```{r wop-result-opt, eval = FALSE}
## We re-run the model because the Wheat-opt-ex.apsimx file 
## is already edited
sim.opt <- apsimx("Wheat-opt-ex.apsimx", src.dir = extd.dir, value = "report")  
sim.opt.s <- subset(sim.opt, 
                    Date > as.Date("2016-09-30") & Date < as.Date("2017-07-01"))
```

```{r import-wop-result, echo = FALSE}
sim.opt.s <- read.csv(file.path(extd.dir, "wheat-sim-opt.csv"))
sim.opt.s$Date <- as.Date(sim.opt.s$Date)
```

```{r wop-result-opt-visualize}
  ## phenology
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Phenology.Stage)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Phenology.Stage)) + 
    ggtitle("Phenology")
  ## LAI
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.Leaf.LAI)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.Leaf.LAI)) + 
    ggtitle("LAI")
  ## Biomass
  ggplot() + 
    geom_point(data = obsWheat, aes(x = Date, y = Wheat.AboveGround.Wt)) +
    geom_line(data = sim.opt.s, aes(x = Date, y = Wheat.AboveGround.Wt)) + 
    ggtitle("Biomass (g/m2)")
```

## Multi-simulation optimization

In both APSIM Classic and Next Generation it might be common to perform simulations (for example for different sites or years). In this case the optimization might be done over those multiple simulations. The main considerations when doing this is that for Classic the observations should be organized with 'outfile' for the first column. In this column there should be a string which matches the simulation in APSIM that corresponds to this specific simulation (i.e. site/year). The second column should be the 'Date' and the variables to be optimized should match those names in the APSIM simulation. Also, the argument 'index' in 'optim_apsim' should be c('outfile', 'Date'). For Next Generation, the considerations are very similar except that the observed data should be organized with 'report' as the first column and 'Date' as the second column. Likewise, the 'index' argument in 'optim_apsimx' should be c('report', 'Date'). In this way, for both types of optimizations the simulations in APSIM will be able to be matched with the observations. 

### Additional considerations and potential issues

The main additional consideration is that it can be a good idea to set up lower and upper bounds on the parameters in terms of how much higher and lower than the default values can be. In a model such as APSIM usually plus or minus 50 percent is enough of a range and even a lower range can often suffice. These can be included as 0.5 and 1.5. With *optim* it is necessary to use method = "L-BFGS-B". There are also similar options when using **nloptr**. This package has many available algorithms to choose from with more modern implementations than the ones available in **optim**. However, one disadvantage of the **nloptr** function is that it does not seem to be able to compute the Hessian numerically. 

* One issue which I have not fully tracked down is an error which appears which indicates a conflict with disk I/O. Again, I need to check but this may be when trying to read and/or write an apsimx file which is also opened in the APSIM GUI application. It is possible that this issue goes away when the APSIM GUI is closed.

---
title: "apsimx: An R package for APSIM Next Generation (and Classic)"
author: "Fernando Miguez"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{apsimx: An R package for APSIM (Next Generation and Classic)}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)
```

```{r apsimx-options, echo = FALSE, eval = FALSE}
apsimx_options(warn.versions = FALSE)
ava <- apsim_version(verbose = FALSE)
aiu <- apsim_version(which = "inuse")
##if(is.na(ava[2,2])) stop("Need APSIM-X to create this vignette")
```

```{r create-temp-dir, echo = TRUE, eval = TRUE}
## Will only write file to a temporary directory for these Examples
tmp.dir <- tempdir()
```

# Introduction

Are you a user of APSIM-X ("Next Generation")? Are you a frequent user of R? Are you hoping for APSIM-X and R to interact with each other? Then the answer is the **apsimx** package, which allows you to inspect, edit, run and read files created by APSIM-X -- .apsim(x) files. New .apsimx files are based on the JSON (JavaScript Object Notation https://www.json.org/) format and it is used to communicate information to the APSIM-X engine. By using the 'jsonlite' R package **apsimx** can read and write ('inspect' and 'edit') files in this format. APSIM 'Classic' used an XML format and this type is now fully supported ('inspect', 'edit', 'run' and 'read').  For 'inspect' and 'edit' in the APSIM 'Classic' version I use the 'xml2' package. To run simulations the *apsimx* function (*apsim* for 'Classic') is available and results from the simulation can be imported as an R object with a dedicated function (*read_apsimx* 'NG' and *read_apsim* for 'Classic'). If you need to write scripts (regardless of whether you use the **apsimx** package or not) the vignette 'writing scripts' might be useful. The package is currently being developed and available at: https://github.com/femiguez/apsimx. 
If you have any questions write to: femiguez *at* iastate.edu. 

In part this package is inspired by previous work by Bryan Stanfill on the 'apsimr' package (previously available from CRAN and github: https://github.com/stanfill/apsimr). That package made it possible to not only run APSIM from R, but also to perform sensitivity analysis and model emulation using 'GAMS'. However, that package has not been maintained in the last five years and it does not work for the new APSIM-X framework. Another useful R package used to be 'APSIM', but it has been removed from CRAN (as of 2020-10-19) but as with 'apsimr' it has not been updated and it has somewhat of a different functionality. The 'APSIM' package was especially useful for creating weather ('.met') files. The functionality in *apsimx* goes beyond that in the removed 'APSIM' package. There is also a package 'APSIMBatch', but it has limited functionality.

## Background information

To run the code in this vignette, you need to have APSIM 'Next Generation' installed. However, if you have not used APSIM it is recommended that you become familiar with it before hand.

See the following for details

[APSIM](https://www.apsim.info)

[APSIM-X](https://apsimnextgeneration.netlify.app/)

Holzworth et al. (2018). APSIM Next Generation: Overcoming challenges in modernising a farming systems model, Environmental Modelling & Software.

This package has been tested on (latest version):

**Mac**: ApsimX version (2025-03) 7382 (R 4.3.0)

**Debian**: ApsimX version (2024-02) 7382 (R 4.1.2)

**Windows**: ApsimX version (2025-03) 7676 and APSIM Classic 7.10-r4221 (R 4.3.3)

## Installing APSIM (Next Generation)

For most functions in this package you need to have a version of APSIM-X (or 'Next Generation') installed (or APSIM 'Classic'). For Mac and Linux, for versions older than Sep 2021, first you need to install mono (https://www.mono-project.com/download/stable/) and then download and install APSIM (https://www.apsim.info/download-apsim/downloads/). **NOTE**: If you are using a version of APSIM Next Gen released after Sep 2021, you do **not** need to install mono. You will need the .NET Runtime 8.0 for Linux or Mac (go to dotnet microsoft), -> download -> dotnet -> 8.0. The **apsimx** package will try to detect both mono and APSIM (Classic and Next Generation). 

**Mac**: For Mac it is also assumed that APSIM has been moved to 'Applications'.

**Windows**: In Windows the default install location is in 'Program Files' for APSIM (Next Generation) and Program Files (x86) for APSIM 'Classic'.

**Linux (Debian)**: It assumed that APSIM is in '/usr/local/lib/apsim/'.

If you have your own flavor of APSIM, then you need to set the path manually using the function 'apsim_options' for Classic and 'apsimx_options' for Next Generation.

## R functions for APSIM (Next Generation)

The main functions in the package are (for now):

* **inspect_apsimx** simple inspecting of an .apsimx file. (It does not replace the GUI.)
* **edit_apsimx** edit/modify an .apsimx file.
* **apsimx** run an .apsimx simulation.
* **read_apsimx** read files created by an .apsimx simulation run (.db files)
* **read_apsimx_all** read all files created by one or more .apsimx simulation runs in a directory (.db files)
* **inspect_apsimx_replacement** inspect a replacement component of an .apsimx file.
* **apsimx_options** set global options: path for executable, examples and warnings.

There are many other useful function described below.

## R functions for APSIM 'Classic' (7.x) - Only for Windows

The main functions relevant for 'Classic' are:

* **inspect_apsim** allows for inspection of .apsim file.
* **edit_apsim** edit an .apsim file.
* **apsim** runs an .apsim file.
* **read_apsim** reads files (.out and .sim) generated by apsim into a data frame.
* **apsim_example** runs (only a few) of the built-in examples.
* **apsim_options** set global options: path for executable, examples and warnings.
* **edit_apsim_xml** edit an XML file such as Maize.xml or Soil.xml

See also 'optim' vignette for optimization options and 'sensitivity' vignette for sensitivity analysis.

# Using 'apsimx'

## Running a built-in example with 'apsimx_example'

To get started you can run one of the examples distributed with APSIM-X. This function does the following: it detects where the APSIM examples are located, it runs the example indicated and it returns the 'report' as a data.frame. After that you can do the usual manipulations, including visualization.

```{r apsimx-example, eval = FALSE}
maize <- apsimx_example("Maize")
```

After we run this example we can treat it as a standard data frame in R.

```{r read-in-maize-example, eval = TRUE, echo = FALSE}
extd.dir <- system.file("extdata", package = "apsimx")
## maize <- read_apsimx("Maize.db", src.dir = extd.dir)
maize <- read.csv(paste0(extd.dir, "/Maize.csv"))
maize$Date <- as.Date(maize$Date)
```

```{r summary-ggplot-maize}
summary(maize)
## Simple data plotting
ggplot(data = maize , aes(x = Date, y = Maize.AboveGround.Wt)) + 
  geom_point()
```

## Inspecting a file with 'inspect_apsimx': Why?

Being able to inspect a file from within R is a functionality which was not considered in previous R packages. My first motivation for doing this was to be able to verify from within R that if I edited an APSIM(X) file, the editing worked as expected. The second use of 'inspect' is to be able to understand the structure of 'apsim' files better. A third useful aspect of 'inspect' is that it can be used to supply the path for a parameter that will be later edited. Another benefit of the inspect functions is that if you are running APSIM in a Linux cluster, you do not have access to the GUI, so these functions allow you to inspect your files in that environment. In addition, having this functionality within R allows to make these tasks repeatable. For complex inspections and manipulations of an .apsim(x) file the GUI is recommended. This package was originally designed to work with relatively simple .apsim(x) files, but functionality has evolved and it can handle more
complex simulations.

## Inspecting and Editing the 'Maize' example

Here I'm using the 'Maize' example distributed with APSIM-X but stored in the package and the specific task is: we want to make sure that the intended component was edited as expected. Let's look at the weather file used.

```{r inspect-apsimx}
extd.dir <- system.file("extdata", package = "apsimx")
inspect_apsimx("Maize.apsimx", src.dir = extd.dir, node = "Weather")
```

Editing a file has the side effect of creating a new file with '-edited' added to the name to avoid conflict or unintended consequences, but it is also possible to overwrite it. This follows the same style as in the 'apsimr' package. One difference is that it is possible to supply a different 'edit.tag'. This code does not run an APSIM simulation, it only edits the file. 

Let's say we want to modify the values of 'Organic Carbon' in the soil in the 'Maize.apsimx' example. First we should inspect the current values.

```{r inspect-maize-oc}
inspect_apsimx("Maize.apsimx", src.dir = extd.dir, 
               node = "Soil", soil.child = "Organic")
```

The function 'inspect_apsimx', in this example, displays various information about the organic matter in the soil. (See the help page - ?inspect_apsimx - for more options). In the next step, we 'edit' the values.

```{r edit-maize-oc}
ocs <- c(1.5, 1.4, 1.3, 1.2, 1.1, 1.0, 0.9)
edit_apsimx("Maize.apsimx", 
            src.dir = extd.dir,
            wrt.dir = ".",
            node = "Soil",
            soil.child = "Organic", 
            parm = "Carbon", value = ocs)
```

Let's make sure that we have changed the values successfully.

```{r inspect-maize-oc-check}
inspect_apsimx("Maize-edited.apsimx", src.dir = ".", 
               node = "Soil", soil.child = "Organic")
```

There is another function that might be useful **inspect_apsimx_json** which can be more flexible and a variation of grep **grep_json_list**.

# Proposed workflow

It is recommended that you start by creating an .apsim(x) file using the GUI. In following steps the goal could be, for example, to either perform sensitivity or uncertainty analysis using R. The functions included in this package are aimed at simplifying the process of running a simulation, changing the value of a variable/parameter and running the model again iteratively. Writing complex scripts requires a high level knowledge of both APSIM and R, but it is made possible with these tools. This is the proposed workflow:

1. Create a simulation using the APSIM GUI
2. Identify variables/parameters that you want to modify (they need to be exposed in the .apsimx (JSON) or .apsim (XML) file)
3. Edit the file using 'edit_apsim(x)'
4. Run the simulation using 'apsim(x)'
5. Repeat steps 3 and 4 as needed
6. Read in and/or manipulate the results, possibly using 'read_apsim(x)'
7. Perform visual and statistical analysis

Another proposed more advanced workflow would involve calibrating or optimizing variables given observed data. This is considered in more detail below.

## Wheat Example (APSIM-X)

The 'Wheat' example is distributed with the 'apsimx' package.

```{r apsimx-wheat-no-run, eval = FALSE}
sim <- apsimx("Wheat.apsimx", src.dir = extd.dir, value = "report")
```

If you have APSIM-X installed you can also run it in this way.
```{r apsimx-wheat-no-run-two, eval = FALSE}
ex.dir <- auto_detect_apsimx_examples()
## Copy 'Wheat' file to a temporary directory
## (or change as needed)
tmp.dir2 <- tempdir()
file.copy(paste0(ex.dir, "/", "Wheat.apsimx"), tmp.dir2)
sim <- apsimx("Wheat.apsimx", src.dir = tmp.dir2, value = "report")
```

```{r apsimx-wheat-read, echo = FALSE}
## sim <- read_apsimx("Wheat.db", src.dir = extd.dir)
sim <- read.csv(paste0(extd.dir, "/Wheat.csv"))
sim$Date <- as.Date(sim$Date)
```

```{r apsimx-wheat-summary}
## Calcualte summary statistics on all variables
summary(sim)
## Plot data
ggplot(data = sim, aes(x = Date, y = Yield)) + geom_point()
## Inspect the Wheat .apsimx file
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop")
## This only displays the available 'Manager' components
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager")
## Looking more in-depth into 'SowingRule1'
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", NA))
```

If you only want to display the Wheat cultivar choose the sixth element in that table.

```{r apsimx-wheat-cultivar-only}
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", 6))
## We can print and store the path to this parameter
pp <- inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, 
               node = "Manager", parm = list("SowingRule1", 6),
               print.path = TRUE)
```

## Millet example (APSIM 'Classic')

One of the difficulties with working with APSIM(X) in scripting efforts is that the names of the parameters that we might want to edit can be buried deep in the .apsim(x) file and that there might be multiple instances of those parameters. To overcome this (to some extent) the 'inspect' functions can provide the path to the parameter to edit. For example,

```{r Millet}
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager")
```

We can see that we have 'Sow on a fixed date' as one of the options. Let's say that we want to edit the planting density

```{r Millet-sow}
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager",
              parm = list("Sow on a fixed date", NA))
```

Planting density is the fifth element in that table. We can obtain the full path by using the 'print.path' option. It makes sense to use this option once you have found the parameter you are looking for. 

```{r Millet-sow-plt-dens}
inspect_apsim("Millet.apsim", src.dir = extd.dir, node  = "Manager",
              parm = list("Sow on a fixed date",5), print.path = TRUE)
## Or store it in an object for later editing
pp <- inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Manager",
              parm = list("Sow on a fixed date", 5), print.path = TRUE)
```

Once we have identified the parameter we want to edit, we can do so by using the 'edit_apsim' function. When using the 'parm.path' argument, node should be "Other".

```{r Millet-edit}
edit_apsim("Millet.apsim", src.dir = extd.dir, wrt.dir = tmp.dir,
           node = "Other", parm.path = pp, value = 8, 
           edit.tag = "-pp")
```

Verify that it was edited correctly.

```{r Millet-edit-inspect}
inspect_apsim("Millet-pp.apsim", src.dir = tmp.dir,
              node = "Manager",
              parm = list("Sow on a fixed date", NA))
```

```{r removing-Millet-pp, echo = FALSE, eval = TRUE}
## Apparently this is not needed
file.remove(paste0(tmp.dir, "/Millet-pp.apsim"))
```

# Advanced Examples

## Inspecting a 'Replacement' component

Many additional options in an APSIM-X simulation can be realized through the use of 'Replacements'. For this, there is a function that allows inspection. In this case, it is important to know the structure of the 'replacement' and explore the available components. For example, if we want to display the xy pair for the thermal time in the phenology component of the 'Maize' replacement we might go through the following steps:

State the node ("Maize") and display available components:

```{r inspect-replacement-node}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", display.available = TRUE)
```

Choose 'Phenology' as a 'child' of 'Maize':

```{r inspect-replacement-node-child}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           display.available = TRUE)
```

Choose 'ThermalTime' as a 'sub.child' of 'Phenology':

```{r inspect-replacement-node-subchild}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime",
                           display.available = TRUE)
```

Now we need to pick from within 'ThermalTime'. There is a component named "BaseThermalTime".

```{r inspect-replacement-node-subchild-subsubchild}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           display.available = TRUE) 
```

'BaseThermalTime' is the only child inside 'ThermalTime', but within 'BaseThermalTime' there are two Children: 'Response' and 'InterpolationMethod'. There is not much to edit for 'InterpolationMethod', so we choose 'Response'.

```{r inspect-replacement-node-subchild-subsubchild-temp-response}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response") 
```

Choose the 'Y' parameter within 'Response'.

```{r inspect-replacement-node-subchild-subsubchild-temp-response-parm}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response",
                           parm = "Y") 
```

The function shows the available replacements ("Maize" and "Soybean"), the 'CropType' if available, the subchild name and the 'Y' vector in this example. If parm is not specified all elements will be displayed.

Let's say we want to inspect details of soybean cultivars in the same file:

```{r inspect-replacement-soybean-cultivar-node}
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Soybean", display.available = TRUE) 
```

The cultivars are at the node.child level.

```{r inspect-replacement-soybean-cultivar-node-child}
inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                           src.dir = extd.dir,
                           node = "Soybean", 
                           node.child = "Cultivars",
                           node.subchild = "USA",
                           node.subsubchild = "PioneerP22T61_MG22",
                           display.available = FALSE) 
```

The function **inspect_apsimx_replacement** can also be used to inspect more complex file structures such as the 'Factorial' example distributed with APSIM-X. See '?inspect_apsimx_replacement'.

## Editing a 'Replacement' 

After using the 'inspect' version of the file, editing should be much more straight forward.

```{r edit-replacement}
edit_apsimx_replacement("MaizeSoybean.apsimx", 
                        src.dir = extd.dir, wrt.dir = tmp.dir,
                        node = "Maize", node.child = "Phenology",
                        node.subchild = "ThermalTime", 
                        node.subsubchild = "BaseThermalTime",
                        node.sub3child = "Response",
                        parm = "Y", value = c(0, 12, 20, 28, 0))
```

Let's inspect the edited file

```{r inspect-edit-replacement}
inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", 
                           src.dir = tmp.dir,
                           node = "Maize", 
                           node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response",
                           parm = "Y") 
```

## Inspecting and editing a Factorial

The complexity of APSIM-X files can be mind-boggling and it makes it difficult to write robust functions across all tasks that APSIM is able to accomplish. I wrote the 'inspect_apsimx_replacement' and 'edit_apsimx_replacement' specifically to edit replacements but these functions have much more flexibility than 'inspect_apsimx' and 'edit_apsimx'. The function 'apsimx_example' is limited in the number of examples that it is allowed to run, but this does not mean that you cannot run the other examples which are distributed with APSIM-X. Let's take the 'Factorial' example:

If we want to inspect and edit this file, we need the 'inspect_apsimx_replacement' function.

```{r inspect-factorial-0, eval = FALSE}
## There are multiple 'Experiments' so we need to pick one
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", NA))
##These positions matched  Experiment   1 2 3 4 5 
##Error in inspect_apsimx_replacement("Factorial", src.dir = ex.dir, root = ##list("Experiment", : Multiple root nodes found. Please provide a position
```

```{r inspect-factorial}
## There are multiple 'Experiments' so we need to pick one
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1))
## We need to provide a node
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1), 
                           node = "Base", display.available = TRUE)
## We need to provide a node child
inspect_apsimx_replacement("Factorial", src.dir = extd.dir,
                           root = list("Experiment", 1), 
                           node = "Base", node.child = "Clock",
                           display.available = TRUE)
```

Here we stop at this level but it is possible to dig deeper using 'node.subchild' and 'node.subsubchild'. It is not possible at this point to dig deeper than 'node.sub5child' in the hierarchy level.

The **inspect_apsimx** function can also handle factorials. 

```{r inspect-apsimx-factorial-1, eval = FALSE}
inspect_apsimx("Factorial.apsimx", src.dir = extd.dir)
## Simulation structure: 
## list Name: Simulations 
# list length: 8 
# list names: $type ExplorerWidth Version Name Children IncludeInDocumentation Enabled ReadOnly 
# Children: Yes 
# Children length: 6 
# Children Names: Experiment RangeExperiment OperationsExpt Compound ManagerExpt DataStore 
# Error in inspect_apsimx("Factorial", src.dir = ex.dir) : 
#   more than one simulation found and no root node label has been specified 
#  select one of the children names above
```

We need to pick one out of the possible factorials. If you check this file with the APSIM GUI, you'll see that within 'RangeExperiment' there is 'Factors' and 'Base2', but the core simulation is within 'Base2'. The 'root' argument provides the specific factorial and the node 'Base2' within that factorial.

```{r inspect-apsimx-factorial-2}
inspect_apsimx("Factorial.apsimx", src.dir = extd.dir,
               root = c("RangeExperiment", "Base2"),
               node = "Weather")
```

Following this structure, if we want to edit the path to the weather file we can use the following example.

```{r edit-apsimx-factorial-2}
edit_apsimx("Factorial.apsimx", 
            src.dir = extd.dir, wrt.dir = tmp.dir,
            root = c("RangeExperiment", "Base2"),
            node = "Weather", 
            value = "Ames.met")
```

I won't be running the 'Factorial' example here, but if you are interested this is done in the folder 'tests' in the pacakge source. 

# APSIM Inputs: Weather and Soil

## Obtaining weather data

One of the easiest ways of generating weather data (".met" files) is by using the  'nasapower' (https://github.com/ropensci/nasapower) or 'GSODR' (https://github.com/ropensci/GSODR) packages. For the US, some options are 'daymetr' (https://github.com/bluegreen-labs/daymetr) or through the Iowa
Environmental Mesonet (https://mesonet.agron.iastate.edu/). In addition, this package provides the following utilities:

* **read_apsim_met**: read an APSIM met file into R with S3 class 'met'.
* **write_apsim_met**: write an APSIM met file from an object of class 'met'.
* **check_apsim_met**: check for reasonable values for met variables.
* **impute_apsim_met**: impute missing values using linear interpolation, splines or the mean.
* **napad_apsim_met**: Fill in date gaps with NAs in a 'met' object. Useful for imputation.
* **compare_apsim_met**: utility for quick comparison of 'met' objects. There is also a plot method.
* **get_power_apsim_met**: (requires the 'nasapower' package). Fetch data and create a 'met' R object and (optionally) write a met file to disk.
* **get_daymet2_apsim_met**: (requires the 'daymetr' package). Fetch data and create a 'met' R object and (optionally) write a met file to disk.
* **get_gsod_apsim_met**: (requires the 'GSODR' package). Fetch data and create a 'met' R object and (optionally) write a met file to disk.
* **get_iem_apsim_met**: Fetch data from Iowa Environmental Mesonet
* **get_iemre_apsim_met**: Fetch data from Iowa Environmental Mesonet Reanalysis
* **get_worldmodeler_apsim_met**: Fetch data from CSIRO Worldmodeler
* **plot** plotting method for objects of class 'met'.
* **summary** summary method for objects of class 'met'.

Note that some utilities were available in the 'APSIM' package (loadMet, prepareMet, checkMet, writeMetFile) for reading, preparing, checking and writing a met file. However, that package has been removed from CRAN.

## Obtaining soil data

The function **get_isric_soil_profile** will retrieve data from the ISRIC Soil Grid global dataset (https://www.isric.org/) and return an object of class *soil_profile* (see also *apsimx_soil_profile*). To derive some of the properties needed by APSIM pedotransfer functions from Saxton and Rawls (2006) were used. This is a work in progress and improvements and suggestions are welcome.

In the USA, the function **get_ssurgo_soil_profile** will retrieve data from the SSURGO database through the **soilDB** R pacakge and will always return a list with objects of class *soil_profile*. Internally, it uses the **ssurgo2sp** function, which can be used to create an APSIM soil profile in two steps. The first step would be to obtain soil data through the 'FedData' R package by using the function **get_ssurgo** or use the **get_ssurgo_tables** in apsimx. The function **ssurgo2sp** can be used for conversion from **SSURGO** csv files to an object of class **soil_profile** and **apsimx_soil_profile** for generating a soil profile that can be used in APSIM-X. Finally, the function **edit_apsimx_replace_soil_profile** will batch replace the soil profile created by **apsimx_soil_profile** in a designated '.apsimx' file. Similarly, 'edit_apsim_replace_soil_profile' will batch replace the soil profile in a designated '.apsim' (Classic) file. I will be working on a more concrete tutorial at some point.

In addition, there are is a function to compare soil profiles **compare_apsim_soil_profile**, which can be used to compare, numerically, one or more soil profiles and a **plot** method to visually compare this object.

# File Type (or File Format) Details

APSIM has traditionally generated files that produce simulations using an XML schema. This has been used with APSIM versions of 7.x, 'Classic'. With APSIM-X a new format based on JSON files has been implemented and apparently both formats will be supported for the forseeable future. JSON files are less verbose and they are gaining widespread adoption. All distributed APSIM-X examples are 'JSON' now.

The apsimx package supports inspection and editing of both file types, but XML is for 'Classic' only and JSON is for APSIM-X. In most instances the parameters that are likely to need inspection and editing are supported, but more features will be added in the future as needed. 

# Additional miscellaneous functions

**apsim_version**: displays which version(s) of APSIM(X) are available. For example,

```{r apsim-verions-tail, eval = FALSE}
ava <- apsim_version()
aiu <- apsim_version(which = "inuse")
```

**edit_apsimx_batch**: Uses APSIM-X built-in functionality to edit parameters. In testing, it is slower than the other 'edit_apsimx*' functions.

**doy2date**: Utility function for converting from day of the year (1-366) to 'Date' and performing the inverse conversion.

**compare_apsim**: Utility function for comparing simulations or observed and simulated data.

**unit_conv**: Unit conversion utility

**carbon_stocks**: function to calcualte carbon stocks.

**check_apsim_met**: check an object of class 'met'

**check_apsimx_soil_profile**: check an object of class 'soil_profile'

# Appendix

Files in 'inst/extdata' and how they are used (ordered alphabetically):

* **Ames.met**: used to run the Soybean.apsimx.
* **Factorial.apsimx**: This file IS needed for the vignette.
* **ISUAG** (folder): SSURGO files to test the ssurgo2sp function.
* **maize-factorial.apsim** file used to test the ability to inspect and edit factorial .apsim files (see 'inspect_apsim').
* **maize-manager-folder.apsimx**: file used to test whether I need to modify inspect and edit functions when the management is inside a folder.
* **maize-soybean-rotation.apsim**: example file with a maize and soybean rotation used in testing and in the 'inspect_apsim' file.
* **maize-swim.apsim**: example file with the SWIM module for testing (test_apsim_swim.R).
* **Maize.apsimx**: This file is needed for the vignette.
* **Maize.csv**: This file is needed for the vignette.
* **Maize.out/sum**: This file is needed for the 'read_apsim' example.
* **Maize75.xml**: This file is used in 'inspect_apsim_xml' and optim vignette.
* **MaizeSoybean.apsimx**: This file is needed for the vignette and for testing in 'test_replacements.R'
* **Millet.apsim**: Millet example from APSIM 'Classic' for testing 'inspect_apsim' and used in the vignette.
* **Millet.out/sum**: result from the Millet simulation. Used in the 'read_apsim' example.
* **Soybean.apsimx**: Soybean example for APSIM-X. Used in 'test_replacements.R'.
* **Wheat-opt-ex.apsimx**: This file is used in the 'optim-apsim' vignette.
* **wheat-sim-b4-opt.csv**: This file is used in the 'optim-apsim' vignette.
* **wheat-sim-opt.csv**: This file is used in the 'optim-apsim' vignette.
* **Wheat.apsimx**: This file is used in the vignette.
* **Wheat.csv**: This file is used in the vignette.
* **WheatRye.apsimx**: This file is used in 'test_replacements.R'.

---
title: "Sensitivity Analysis"
author: "Fernando Miguez"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Sensitivity Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(apsimx)
library(ggplot2)
apsimx_options(warn.versions = FALSE)
tmp.dir <- tempdir()
```

# Introduction

Sensitivity analysis of crop models such as APSIM is a valuable task which
allows us to develop an intuition of the model response to changing a variety
of inputs. These inputs can generally be included in the following categories:

* weather 
* soil profiles
* crop/genetics 
* management

The weather component can depend on different sources of data or manipulations
to simulate different conditions and stresses. The soil profiles can be from different
locations or specifically manipulated to understand the impact of particular soil properties.
Crop properties or cultivar specific parameters can also be investigated as a source of 
model behavior. Finally, examples of management usually include planting date, fertilizer application, tillage or residue management and irrigation. All of these components can be sources of uncertainty and can be evaluated in a sensitivity analysis.

When multiple parameters are evaluated performing model runs for all combinations can become prohibitive. For this reason, there are some combinations which can be more efficient and provide nearly the same amount of information. The suggestion here is to build complexity in the sensitivity analysis task gradually. For example, first choose one parameter and select 5-10 reasonable values, run the model and evaluate the model output. This is a one parameter at a time approach and it can be of great value in developing an intuition for the model behavior.

For a general introduction to sensitivity analysis see Saltelli et al. 2008. Global sensitivity analysis. The Primer.

Here they define sensitivity analysis as:

*The study of how uncertainty in the output of a model (numerical or otherwise) can be apportioned to different sources of uncertainty in the model input*

A preliminary step in sensitivity analysis often involves uncertainty analysis and, as an example, it can involve computing the output from a model based on various samples from the relevant inputs. Usually, this involves identifying parameters, choosing appropriate distributions which we sample from and creating candidate values which are then used in the model.

## Uncertainty Analysis using for loops

The steps needed to perform a uncertainty analysis can involve simple use of **for** loops.

1. Identify a parameter of interest
2. Sample (or select) a new value for the parameter of interest
3. Edit the APSIM file with the new value
4. Run the model
5. Collect the results and repeat as needed

A simple example

```{r sens0, eval = FALSE}
## Create a temporary directory and copy the Wheat.apsimx file
tmp.dir <- tempdir()
extd.dir <- system.file("extdata", package = "apsimx")
file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
## Identify a parameter of interest
## In this case we want to know the impact of varying the fertilizer amount
pp <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                      node = "Manager", parm = list("SowingFertiliser", 1))
## For simplicity, we create a vector of fertilizer amounts (instead of sampling)
ferts <- seq(5, 200, length.out = 7)
col.res <- NULL
for(i in seq_along(ferts)){
  
  edit_apsimx("Wheat.apsimx", src.dir = tmp.dir,
              node = "Other",
              parm.path = pp, parm = "Amount",
              value = ferts[i])
  
  sim <- apsimx("Wheat.apsimx", src.dir = tmp.dir)
  col.res <- rbind(col.res, data.frame(fertilizer.amount = ferts[i], 
                                       wheat.aboveground.wt = mean(sim$Wheat.AboveGround.Wt, na.rm = TRUE)))
}
```

The code here is shown for illustration so that the basic strategy can be replicated. The results collected in object *col.res* can be further processed to better understand the impact of the chosen parameter (input) on the output.

There is a function in the package which simplifies this task. The one below is for APSIM Next Gen and there is a similar one for APSIM Classic.

## sens_apsimx

* file = .apsimx file
* src.dir = source directory where the APSIM file is located.
* parm.paths = character vector with the full path to the parameters to be evaluated. This can be obtained by using **inspect_apsimx** or **inspect_apsim_json**. The length of the parameter vector should be equal to the number of parameters being evaluated (sometimes less is more). 

* convert = whether to convert strings to numeric

* replacement = logical indicating whether each parameter is in the replacement component or not.

* grid = data frame with combinations of parameters to be evaluated.

* soil.profiles = soil profiles, which can be passed as a list

* summary = optional summary function to apply if the APSIM output returns multiple rows. The mean is used by default.

* root = indicate the root simulation in case it is not the default one.

* verbose = printing progress and other details

* cores = number of cores to use. The package 'future' is currently used.

* save = whether to save intermediate (and final) results.

* ... = additional arguments (none used at the moment).

## Example using sens_apsimx

The example below shows the following steps.

1. Copy the example 'Wheat' file to at temporary directory
2. Identify parameters of interest (using inspect_apsimx)
3. Create a simple grid for evaluating the model

Here the grid is created using *expand.grid* and it has 9 rows. This means that 
this will require 9 simulation runs of the APSIM model. This takes a couple of minutes
to run. The resulting object is of class *sens_apsim* and it can be investigated 
using the *summary* function (for now).

```{r sens_apsimx, eval = FALSE}
tmp.dir <- tempdir()
extd.dir <- system.file("extdata", package = "apsimx")
file.copy(file.path(extd.dir, "Wheat.apsimx"), tmp.dir)
## Identify a parameter of interest
## In this case we want to know the impact of varying the fertilizer amount
## and the plant population
pp1 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                      node = "Manager", parm = list("SowingFertiliser", 1))
pp1 <- paste0(pp1, ".Amount")

pp2 <- inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, 
                      node = "Manager", parm = list("SowingRule1", 9))
pp2 <- paste0(pp2, ".Population")

## The names in the grid should (partially) match the parameter path names
grd <- expand.grid(Fertiliser = c(50, 100, 150), Population = c(100, 200, 300))
 
## This takes 2-3 minutes
sns <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                    parm.paths = c(pp1, pp2),
                    grid = grd)
```

## Creating grids

There are many ways of creating grids. The example above using function *expand.gird*
returned all possible combinations of the parameter values. This might not be desirable 
or feasible and other methods are available. For this task, the package **sensitivity** is 
especially useful.

### First option: parameterSets

```{r sensitivity-parameterSets}
library(sensitivity)
## Simple example: see documentation for other options
X.grid <- parameterSets(par.ranges = list(Fertiliser = c(1, 300), 
                                          Population = c(1, 300)),
                        samples = c(3,3), method = "grid")
X.grid
```


## Second option: Morris

A second option is to use the function *morris*.

```{r sensitivity-morris}
X.mrrs <- morris(factors = c("Fertiliser", "Population"),
                   r = 3, design = list(type = "oat", levels = 3, grid.jump = 1),
                   binf = c(0, 5), bsup = c(200, 300))
X.mrrs$X
```

The sensitivity package allows for decoupling the simulation runs and the analysis of the results.
For example:

```{r sensitivity-decoupling, eval = FALSE}
## This takes 2-3 minutes
sns2 <- sens_apsimx("Wheat.apsimx", src.dir = tmp.dir,
                     parm.paths = c(pp1, pp2),
                     grid = X.mrrs$X)
## These are the sensitivity results for AboveGround.Wt only
sns.res.ag <- tell(X.mrrs, sns2$grid.sims$Wheat.AboveGround.Wt)
sns.res.ag
## Call: morris(factors = c("Fertiliser", "Population"), 
## r = 3, design = list(type = "oat", levels = 3, grid.jump = 1), 
## binf = c(0, 5), bsup = c(200, 300))
## 
## Model runs: 9 
##                 mu  mu.star    sigma
## Fertiliser 916.6674 916.6674 662.8798
## Population 448.4530 448.4530 640.6807
plot(sns.res.ag)
```

I'm not running the code in this vignette to save time, but hopefully this is sufficient
information to illustrate performing sensitivity analysis using package **apsimx** and **sensitivity**.

---
title: "apsimx: writing scripts"
author: "Fernando Miguez"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{apsimx: writing scripts}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
##require(apsimx)
```

# Writing scripts and running APSIM-X at the command line

The reason for running APSIM without the use of the GUI is to be able to perform a variety of more complex analysis, but the diversity of problems that a user might want to solve are not likely to be addressed by this package alone. Since it is possible to run APSIM-X from the command line it is also possible to write custom scripts in R, Python or other languages. Within APSIM-X it is also possible to write C# and VB.NET scripts to control field and management operations. 

Even though this package is meant to simplify writing scripts in R that require the use of APSIM "Next Generation", it is likely that a user might decide not to use this package, but still the information below will be useful in writing scripts. Other users are likely to run into problems when trying to run APSIM-X at the command line and the material below is meant to try to help with this process. 

APSIM-X is an effort at modernizing and improving the previous versions of APSIM 7.x. The program is primarily written in C# and it uses the Mono Framework and GTK# (https://apsimnextgeneration.netlify.app/). Note that in the previous website there is also information about using Cloud Services and Microsoft Azure.

If you are using a previous version of APSIM (7.x) you might be interested in either the 'APSIM' or 'apsimr' R packages. However, as of 2021-02-25 they have been removed from CRAN. The **apsimx** provides support for APSIM 'Classic'.

Let's see what is needed to run APSIM-X at the command line.

## Windows

In Windows, if you use the current (`r Sys.Date()`) version of the installer the program will be, most likely, in:

**C:\\Program Files\\APSIM2024.2.7382.0\\bin\\Models.exe**

Notice that for each new build of APSIM-X the path will change. After 'APSIM' is the year and month, and version separated by a period. In this case '2021.08.12.6707'. In the folder 'bin' (for 'binary') there are many libraries ('.dll files') and some executables. The 'Models.exe' is the one that can run .apsimx files.

When trying to invoke that line at the command line, the space after 'Program' is a problem. There are some possible solutions. One is to add quotations (for example see 'shQuote' function in R). Another option is to use Windows short name paths, which in this case, for 'Program Files' is 'PROGRA~1' (but this can change). If running the program from R, for example, the separator needs to be either a double backslash '\\\\' or a forward slash '/'. So one example would be:

**C:/PROGRA~1/APSIM2024.2.7382.0/bin/Models.exe** or

**C:\\\\PROGRA~1\\\\APSIM2024.2.7382.0\\\\bin\\\\Models.exe**

The backslash ("\\") is an escape character in R, so you can't use it within strings.

After this, the name of the file that you want to run needs to be included. For example,

**C:/PROGRA~1/APSIM2024.2.7382.0/bin/Models.exe ./Wheat.apsimx**.

Notice the "./", without this, it will likely not work under Windows. (update: this issue has been addressed in current version of APSIM Next Gen). However, the previous string cannot be passed directly to a Windows shell. It needs to be translated with the forward slashes ("/") converted to backslashes ("\\").

So to run the previous command in Windows from within R this is one way:

```{r apsimx-windows, eval = FALSE}
run.strng <- "C:/PROGRA~1/APSIM2024.2.7382.0/bin/Models.exe ./Wheat.apsimx"
shell(cmd = run.strng, translate = TRUE, intern = TRUE)
```

<!-- Note: it seems that, in Windows, it is only possible to run files in the current directory. So, for example, if the file 'Wheat.apsimx' was in subdirectory './myApsimXsimulations' and you want to replace the './Wheat.apsimx' above with './myApsimXsimulations/Wheat.apsimx' this does not work. I do not know yet if there is a possible workaround. This issue was fixed at one point. It is now possible to run from a different directory. -->

## Unix (Mac) or Linux (Debian)

For current versions of APSIM Next Gen, this command should be able to run from the command line:

**/Applications/APSIM2024.2.7382.0.app/Contents/Resources/bin/Models ./Wheat.apsimx**

**Mono is not required.**

### Pre Sep 2021 (mono required)

In Unix (Mac) or Linux (Debian) to be able to run APSIM-X the Mono Framework needs to be installed (https://www.mono-project.com/download/stable/) - (pre Sep 2021). This is important because the command 'mono' needs to be available in order to run APSIM-X at the command line. The Mono Framework should be available to install APSIM-X in the first place, but it is important to be aware of this at this stage.

To find out if 'mono' is present you can type 'which mono' at the Terminal.

**which mono**

For Mac:

/Library/Frameworks/Mono.framework/Versions/Current/Commands/mono

For Debian:

/usr/bin/mono

In R you can also use 'Sys.which("mono")'.

A current version of Mono (4.3 or higher) is needed and you can find out which version you are running:

**mono --version**

Which in my case is 6.12.0.122. (Mac)  and 6.0.0.313 (Debian).

To run an example from the command line it is best to move to a directory where you are doing your work and have read and write permissions. APSIM-X will write an SQLite database (.db) with the results and it needs permissions to do this in the current directory. For example, you might want to move to:

Mac example:

**/Users/fernandomiguez/Documents/myApsimWork**

Debian example:

**/home/femiguez/Documents/myApsimWork**

Then find out where APSIM-X is installed, but this will change with each new version. For example, I have these versions installed now:

Mac:

**/Applications/APSIM2024.2.7382.0.app**

Debian:

apsim lib: **/usr/local/lib/apsim/2024.2.7382.0**

The apsim binary is in: **/usr/local/bin/apsim**

Finally, to run an example at the command line:

Mac:

**mono /Applications/APSIM2024.2.7382.0.app/Contents/Resources/bin/Models.exe ./Wheat.apsimx**

Debian:

**mono /usr/local/lib/apsim/2024.2.7382.0/bin/Models.exe ./Wheat.apsimx**

The previous two commands assume that the file "Wheat.apsimx" is present in the current working directory. A detail that seems to have changed with recent versions of 'mono' (not sure) is that if you just use the name of a file, it will return an error saying that empty paths are not valid. So this did not work on Debian:

**mono /usr/local/lib/apsim/2019.10.04.4236/Bin/Models.exe Wheat.apsimx**

but the line above which points to the current directory ("./") did work.

So to run this from within R, the following is one possibility

```{r run-apsimx, eval = FALSE}
## Mac
run.string.mac <- "mono /Applications/APSIM2021.01.14.6010.app/Contents/Resources/Bin/Models.exe ./Wheat.apsimx"
system(command = run.string.mac, intern = TRUE)
## Debian
run.string.debian <- "mono /usr/local/lib/apsim/2019.10.04.4236/Bin/Models.exe ./Wheat.apsimx"
system(command = run.string.debian, intern = TRUE)
```

As mentioned before, if the simulation is successful, a database will be written to the current directory (where the file "Wheat.apsimx" is located in the previous example). In this case it will be a file with the name 'Wheat.db'.

## Functions in the apsimx package

Having to change scripts every time you install a new version of APSIM-X or when you change operating system is certainly problematic and for this reason apsimx tries to 'guess' where APSIM-X is installed and where the 'Examples' are located. The functions to do this are:

Executable: **auto_detect_apsimx()**

Examples: **auto_detect_apsimx_examples()**

Which will return the path to the executable and the directory where the examples are located. These functions are supposed to work whether you are using Windows, Mac or Linux. The function 'auto_detect_apsimx' is not exported, so in order to expose it, you need 'apsimx:::auto_detect_apsimx()'. This function is meant to work with the current binary distributions (as opposed to source code). (I'm guessing that if you are building APSIM-X from source code, you do not need this information.) 

If multiple versions of APSIM-X are installed, apsimx will choose the newest one, but it will issue a warning. If you want to supress this warning change this setting using the 'apsimx_options'. If APSIM-X is installed in an alternative location which make the above functions fail it is possible to set these paths manually using the 'apsimx_options' function. This information applies to APSIM 'Classic' (use apsim_options instead).

In addition, when loaded, the apsimx package will try to detect the presence of APSIM-X and, if not found, it will issue a warning with instructions to set the path manually using 'apsimx_options'. If you have a previous version of APSIM installed (7.x) or multiple versions of both the 'old' and 'new' this might be potentially problematic, but it is hard for me now to consider these endless possibilities.

### Unix and Windows paths

[Windows paths](https://learn.microsoft.com/en-us/dotnet/standard/io/file-path-formats)

To look at paths in Windows:

**path**
**set**
**set path**
**echo %path%**
**echo %path:;=&echo.%**

[Editing PATH in Windows Example](https://docs.telerik.com/teststudio/features/test-runners/add-path-environment-variables)

[Editing PATH in Windows 2](https://www.architectryan.com/2018/03/17/add-to-the-path-on-windows-10/)

Unix/Linux paths

I used to have links here but CRAN does not like them. Search for unix paths.


### Session information

Displaying below specific information related to when the vignette was built.

```{r sisd, eval = FALSE}
sessionInfo()
Sys.info()[["sysname"]]
Sys.info()[["release"]]
Sys.info()[["version"]]
if(.Platform$OS.type == "unix") system(command = "mono --version", intern = TRUE)
R.version
```

### Links

There are some links on the APSIM forums related to this, but I have found the information in those links to be incomplete or outdated. This document is meant to be a comprehensive summary of this topic.

<!-- Running APSIM-X command line (Mac): https://www.apsim.info/Support/tabid/254/forumid/1/threadid/2312/scope/posts/Default.aspx This link does not exist anymore -->

<!-- Here it is mentioned that 'Modelx.exe' can work (Windows): -->
<!-- https://www.apsim.info/Support/tabid/254/forumid/1/threadid/2037/scope/posts/Default.aspx -->

<!-- For previous version of APSIM (7.x): -->
<!-- https://www.apsim.info/Support/tabid/254/forumid/1/postid/2088/scope/posts/Default.aspx -->

I believe this information is outdated:
https://www.apsim.info/documentation/model-documentation/technical-documentation/run-apsim-from-command-line-or-from-scripts/

Here it is mentioned that 'Models.exe Wheat.apsimx' should be able to run APSIM-X at the command line:
https://github.com/APSIMInitiative/ApsimX/issues/2199

<!-- This information could be useful: -->
<!-- http://www.apsim.info/Documentation/TrainingManualsandResources/APSIMTraining(SIMLESA)/Climatechangeprojections.aspx -->

<!-- Large scale simulations and climate change: -->
<!-- http://www.apsim.info/Portals/0/Documentation/SIMLESA/Apsim-Climate-change-2010.pdf -->

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_data_apsimx.R
\name{extract_data_apsimx}
\alias{extract_data_apsimx}
\title{Extract data from an .apsimx (JSON) file}
\usage{
extract_data_apsimx(
  file = "",
  src.dir = ".",
  node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop",
    "Manager", "Report", "Operations", "Other"),
  soil.child = c("Metadata", "Water", "InitialWater", "Chemical", "Physical", "Analysis",
    "SoilWater", "InitialN", "CERESSoilTemperature", "Sample", "Solute", "NO3", "NH4",
    "Urea", "Nutrient", "Organic", "Swim3"),
  parm = NULL,
  digits = 3,
  root = NULL
)
}
\arguments{
\item{file}{file ending in .apsimx to be inspected (JSON)}

\item{src.dir}{directory containing the .apsimx file to be inspected; defaults to the current working directory}

\item{node}{specific node to be used either \sQuote{Clock}, \sQuote{Weather},
\sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop},
 \sQuote{Manager}, \sQuote{Operations} or \sQuote{Other}}

\item{soil.child}{specific soil component to be inspected. The options vary depending on what is available (see \link{inspect_apsimx})}

\item{parm}{parameter to refine the extraction of the \sQuote{manager} list(\sQuote{parm},\sQuote{position}), use \sQuote{NA} for all the positions. \sQuote{parm} can be a regular expression for partial matching.}

\item{digits}{number of decimals to print (default 3). Not used now because everything is a character.}

\item{root}{root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.}
}
\value{
a \link{data.frame} or a \link{list}. It does not return a path.
}
\description{
Extract data from a JSON apsimx file.
}
\details{
This function does not print anything (compared to inspect_apsimx). The purpose is to return data
contained in the APSIM simulation as a data.frame. It will return a \sQuote{list} when a data frame
does not naturally accommodate the result. For example, the complete manager node does not naturally
fit into a data frame structure. In some cases, multiple data frames are returned as part of lists.

Have not written this section yet
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Clock"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Weather"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "Metadata"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "Physical"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "SoilWater"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "Organic"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "Chemical"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "InitialWater"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil",
soil.child = "InitialN"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "SurfaceOrganicMatter"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "MicroClimate"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager"))
(edf <- extract_data_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Report"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_version.R
\name{apsim_version}
\alias{apsim_version}
\title{Display available APSIM \sQuote{Classic} and APSIM-X versions}
\usage{
apsim_version(
  which = c("all", "inuse"),
  version = c("number", "full"),
  verbose = TRUE
)
}
\arguments{
\item{which}{either \sQuote{all} or \sQuote{inuse}}

\item{version}{either \sQuote{number} for a compact display or \sQuote{full}}

\item{verbose}{whether to print the information to standard output}
}
\value{
a data frame (all) or a vector (inuse) with APSIM-X and/or APSIM versions
}
\description{
Display available APSIM \sQuote{Classic} and APSIM-X versions
}
\examples{
\dontrun{
## Check which APSIM versions are available
ava <- apsim_version(verbose = TRUE)

## How to get APSIM 'help'
## apc <- apsimx:::auto_detect_apsimx()
## system(paste(apc, "--help"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{edit_apsim_replace_soil_profile}
\alias{edit_apsim_replace_soil_profile}
\title{Edit APSIM \sQuote{Classic} file with a replaced soil profile}
\usage{
edit_apsim_replace_soil_profile(
  file = "",
  src.dir = ".",
  wrt.dir = NULL,
  soil.profile = NULL,
  swim = NULL,
  soilwat = NULL,
  initialwater = NULL,
  edit.tag = "-edited",
  overwrite = FALSE,
  verbose = TRUE,
  root
)
}
\arguments{
\item{file}{name of the .apsim file to be edited}

\item{src.dir}{source directory}

\item{wrt.dir}{writing directory}

\item{soil.profile}{a soil profile object with class \sQuote{soil_profile}}

\item{swim}{list with SWIM specific parameters}

\item{soilwat}{list with SoilWat specific parameters}

\item{initialwater}{list with InitialWater specific parameters}

\item{edit.tag}{default edit tag \sQuote{-edited}}

\item{overwrite}{default FALSE}

\item{verbose}{default TRUE. Will print messages indicating what was done.}

\item{root}{supply the node postion in the case of multiple simulations such as factorials.}
}
\value{
writes an APSIM file to disk with the supplied soil profile
}
\description{
Edits an APSIM Classic simulation by replacing the soil profile
}
\details{
This function is designed to batch replace the whole soil in an APSIM simulation.
}
\note{
There is no such thing as a default soil, carefully build the profile for each simulation.
This function replaces values and it can grow an XML node, but it cannot edit a property
which is not present in the original file.
}
\examples{
\donttest{
sp <- apsimx_soil_profile(nlayers = 20,
                          crops = c("Barley", "Chickpea", "Lucerne", 
                          "Maize", "Perennial Grass", "Sorghum", 
                          "Wheat", "Millet"))

extd.dir <- system.file("extdata", package = "apsimx")

## Writing to a temp directory
tmp.dir <- tempdir()
edit_apsim_replace_soil_profile("Millet.apsim", soil.profile = sp, 
                                edit.tag = "-newsoil",
                                src.dir = extd.dir, 
                                wrt.dir = tmp.dir)
                                
inspect_apsim("Millet-newsoil.apsim", src.dir = tmp.dir,
              node = "Soil", soil.child = "Water")
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_iemre_apsim_met.R
\name{get_iem_apsim_met}
\alias{get_iem_apsim_met}
\title{Get weather data from Iowa Environmental Ag Weather Stations}
\usage{
get_iem_apsim_met(lonlat, dates, wrt.dir = ".", state, station, filename)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (optional)}

\item{dates}{date ranges}

\item{wrt.dir}{write directory}

\item{state}{state which you choose climate data from}

\item{station}{station which you choose cliamte data from}

\item{filename}{file name for writing out to disk}
}
\value{
returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Retrieves weather data from Iowa Environmental Mesonet (AgWeather) into an APSIM met file
}
\details{
The original data can be obtained from: (mesonet.agron.iastate.edu/request/coop)

If the filename is not provided it will not write the file to disk, 
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk. For this function either provide the longitude 
and latitude or the state and station, but not both. In fact, \sQuote{state} and
\sQuote{station} will be ignored if \sQuote{lonlat} is supplied.
}
\examples{
\dontrun{
## This will not write a file to disk
iem.met <- get_iem_apsim_met(state = "IA", 
                             station = "IA0200", 
                             dates = c("2012-01-01","2012-12-31"))

summary(iem.met)

## Alternatively, coordinates can be used
## This should be equivalent to the previous request
iem.met2 <- get_iem_apsim_met(lonlat = c(-93.77, 42.02), 
                              dates = c("2012-01-01","2012-12-31"))

summary(iem.met2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_filetype.R
\name{apsimx_filetype}
\alias{apsimx_filetype}
\title{Test file format for .apsimx files}
\usage{
apsimx_filetype(file = "", src.dir = ".")
}
\arguments{
\item{file}{file ending in .apsimx to be tested}

\item{src.dir}{directory containing the .apsimx file to be tested; defaults to the current working directory}
}
\value{
\sQuote{xml}, \sQuote{json} or \sQuote{unknown}
}
\description{
Test whether an .apsimx file is XML or json
}
\note{
Minimal function which reads only the first line in a file and tries to guess whether it is
      an \sQuote{xml} or \sQuote{json} file type.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
apsimx_filetype("Wheat.apsimx", src.dir = extd.dir) 
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_chirps_apsim_met.R
\name{get_chirps_apsim_met}
\alias{get_chirps_apsim_met}
\title{Get CHIRPS data for an APSIM met file}
\usage{
get_chirps_apsim_met(
  lonlat,
  dates,
  wrt.dir = ".",
  filename = NULL,
  fillin.radn = TRUE,
  silent = FALSE
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector}

\item{dates}{date ranges}

\item{wrt.dir}{write directory}

\item{filename}{file name for writing out to disk}

\item{fillin.radn}{whether to fill in radiation data using the nasapower pacakge. Default is TRUE.}

\item{silent}{defaut is FALSE. Changing it will not do anything at the moment. A future feature.}
}
\value{
returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Uses \code{\link[chirps]{get_chirps}} from the \CRANpkg{chirps} package to download data to create an APSIM met file.
}
\details{
This function requires the \CRANpkg{chirps} package, version 0.1.4.

If the filename is not provided it will not write the file to disk,
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk.
}
\examples{
\dontrun{
require(chirps)
## This will not write a file to disk
chrp <- get_chirps_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_apsim_met.R
\name{summary.met}
\alias{summary.met}
\title{Summary for an APSIM met file}
\usage{
\method{summary}{met}(
  object,
  ...,
  years,
  months,
  days,
  julian.days,
  by = c("year", "month"),
  compute.frost = FALSE,
  frost.temperature = 0,
  anomaly,
  anomaly.units = c("percent", "standard.deviation", "absolute"),
  check = FALSE,
  verbose = FALSE,
  na.rm = FALSE,
  digits = 2
)
}
\arguments{
\item{object}{object of class \sQuote{met}}

\item{...}{optional argument (none used at the moment)}

\item{years}{optional argument to subset years}

\item{months}{optional argument to subset by months. If an integer, it should
be between 1 and 12. If a character, it can be in the format, for example, 
\sQuote{jan} or \sQuote{Jan}.}

\item{days}{optional argument to subset by days. It should be an integer
between 1 and 31.}

\item{julian.days}{optional argument to subset by julian days. It
should be a vector of integers between 1 and 365. Either use \sQuote{days} or
\sQuote{julian.days} but not both.}

\item{by}{by default the summary is performed by \sQuote{year}. This allows
the use of \sQuote{month}.}

\item{compute.frost}{logical (default FALSE). Whether to compute
frost statistics.}

\item{frost.temperature}{value to use for the calculation of the frost 
period (default is zero).}

\item{anomaly}{whether to compute the anomaly. Default is FALSE. 
It could be TRUE (for all variables) or a character vector for a specific set of variables.}

\item{anomaly.units}{what units to use for anomaly calculations. Default is \sQuote{percent}.
Other options are \sQuote{standard.devaition} and \sQuote{absolute} - this means the difference, 
which will be in the original units.}

\item{check}{logical (default FALSE). Whether to \sQuote{check} the \sQuote{met} object.}

\item{verbose}{whether to print additional infomation to the console}

\item{na.rm}{whether to remove missing values. Passed to \sQuote{aggregate}}

\item{digits}{digits for rounding (default is 2).}
}
\value{
an object of class \sQuote{data.frame} with attributes
}
\description{
Create a data.frame summarizing an object of class \sQuote{met}
}
\details{
The frost free period is computed by first spliting each year (or year interval)
in two halves. The first and last frosts in the first and second period are found.
For the Northern hemisphere calendar days are used (1-365).
For the Southern hemisphere the year is split in two halfs, but the second half of
the year is used as the first part of the growing season.
If frost is not found a zero is returned.
}
\examples{

extd.dir <- system.file("extdata", package = "apsimx")
ames <- read_apsim_met("Ames.met", src.dir = extd.dir)

summary(ames, years = 2014:2016)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optim_apsimx.R
\name{mcmc.apsimx.env}
\alias{mcmc.apsimx.env}
\title{Environment to store data for apsimx MCMC}
\usage{
mcmc.apsimx.env
}
\value{
This is an environment, so nothing to return.
}
\description{
Environment which stores data for MCMC
}
\details{
Create an apsimx environment for MCMC
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_soil_profile.R
\name{apsimx_soil_profile}
\alias{apsimx_soil_profile}
\alias{plot.soil_profile}
\alias{check_apsimx_soil_profile}
\title{Create APSIM-X Soil Profiles}
\usage{
apsimx_soil_profile(
  nlayers = 10,
  Depth = NULL,
  Thickness = NULL,
  BD = NULL,
  AirDry = NULL,
  LL15 = NULL,
  DUL = NULL,
  SAT = NULL,
  KS = NULL,
  crop.LL = NULL,
  crop.KL = NULL,
  crop.XF = NULL,
  Carbon = NULL,
  SoilCNRatio = NULL,
  FOM = NULL,
  FOM.CN = NULL,
  FBiom = NULL,
  FInert = NULL,
  NO3N = NULL,
  NH4N = NULL,
  CACO3 = NULL,
  PH = NULL,
  ParticleSizeClay = NULL,
  ParticleSizeSilt = NULL,
  ParticleSizeSand = NULL,
  soil.bottom = 150,
  water.table = 200,
  soil.type = 0,
  crops = c("Maize", "Soybean", "Wheat"),
  metadata = NULL,
  soilwat = NA,
  swim = NA,
  initialwater = NA,
  solutes = NA,
  soilorganicmatter = NA,
  dist.parms = list(a = 0, b = 0.2),
  check = TRUE
)

\method{plot}{soil_profile}(
  x,
  ...,
  property = c("all", "water", "initialwater", "BD", "AirDry", "LL15", "DUL", "SAT",
    "KS", "Carbon", "SoilCNRatio", "FOM", "FOM.CN", "FBiom", "FInert", "NO3N", "NH4N",
    "CACO3", "PH", "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand", "texture")
)

check_apsimx_soil_profile(x, particle.density = 2.65)
}
\arguments{
\item{nlayers}{Number of soil layers (default = 10)}

\item{Depth}{specific depths for each soil layer (cm)}

\item{Thickness}{thickness for each soil layer (mm)}

\item{BD}{bulk density for each soil layer (g/cc) -- \sQuote{cc} is cubic cm}

\item{AirDry}{air dry for each soil layer (mm/mm)}

\item{LL15}{lower limit (15 bar) for each soil layer (mm/mm)}

\item{DUL}{drainage upper limit (0.33 bar) for each soil layer (mm/mm)}

\item{SAT}{saturation (0 bar) for each soil layer (mm/mm)}

\item{KS}{saturated hydraulic conductivity (mm/day)}

\item{crop.LL}{lower limit for a specific crop}

\item{crop.KL}{root ability to extract water for a specific crop}

\item{crop.XF}{soil root exploration for a specific crop}

\item{Carbon}{organic carbon (percent)}

\item{SoilCNRatio}{organic carbon C:N ratio}

\item{FOM}{fresh organic matter (kg/ha)}

\item{FOM.CN}{fresh organic matter C:N ratio}

\item{FBiom}{Fraction of microbial biomass (0-1)}

\item{FInert}{Fraction of inert carbon (0-1)}

\item{NO3N}{nitrate nitrogen (Chemical) (ppm)}

\item{NH4N}{ammonium nitrogen (Chemical) (ppm)}

\item{CACO3}{carbonates (chemical) (ppm). Not used by APSIM currently.}

\item{PH}{soil pH}

\item{ParticleSizeClay}{particle size clay (in percent)}

\item{ParticleSizeSilt}{particle size silt (in percent)}

\item{ParticleSizeSand}{particle size sand (in percent)}

\item{soil.bottom}{bottom of the soil profile (cm)}

\item{water.table}{water table level (not used at the moment) (cm)}

\item{soil.type}{might use it in the future for auto filling missing information}

\item{crops}{name of crops being grown}

\item{metadata}{list with soil metadata. For possible parameters and values see an example of \code{\link{inspect_apsimx}} with soil.child = \dQuote{Metadata}.}

\item{soilwat}{optional \sQuote{list} of class \sQuote{soilwat_parms}}

\item{swim}{optional \sQuote{list} of class \sQuote{swim_parms}}

\item{initialwater}{optional \sQuote{list} of class \sQuote{initialsoilwater_parms}}

\item{solutes}{optional \sQuote{list} of class \sQuote{solutes_parms}}

\item{soilorganicmatter}{optional \sQuote{list} of class \sQuote{soilorganicmatter_parms}}

\item{dist.parms}{parameter values for creating a profile. If a == 0 and b == 0 then \cr
a constant value of 1 is used. If a == 0 and b != 0, then an exponential decay is used. \cr
If a != 0 and b != 0 then the equation is \code{a*soil.layer*exp(-b*soil.layer)}.}

\item{check}{whether to check for reasonable values using \code{\link{check_apsimx_soil_profile}}}

\item{x}{object of class \sQuote{soil_profile} or the \sQuote{soil} 
component within an object of class \sQuote{soil_profile}.}

\item{...}{additional plotting arguments (none use at the moment).}

\item{property}{\dQuote{all} for plotting all soil properties, \dQuote{water} for just SAT, DUL and LL15}

\item{particle.density}{default value for soil particle density (2.65 g/cm3)}
}
\value{
a soil profile with class \sQuote{soil_profile} with elements \sQuote{soil}, \sQuote{crops}, \sQuote{metadata},
\sQuote{soilwat} and \sQuote{swim}.

it produces a plot

It does not produce output unless potential issues are found. Only warnings
are produced and it returns an object of class \sQuote{soil_profile}.
}
\description{
Generates a soil profile that can then replace the existing one in an \sQuote{.apsimx} or \sQuote{.apsim} simulation file

plotting function for a soil profile, it requires \sQuote{ggplot2}

checking an apsimx soil profile for reasonable values
}
\details{
Soil Profiles

Real soils might have discontinuities, but for APSIM it might be beneficial to be able to create 
a soil profile with an arbitrary number of layers and have flexibility in the 
distribution of soil physical and chemical properties. Steps:
 
 1. \code{\link{apsimx_soil_profile}} is a function which can create a soil matrix with many layers \cr
 2. It allows for creating a smooth distribution for Physical (or Water), Chemical, InitialWater, Analysis, InitialN, Organic or SoilOrganicMatter \cr
 3. The distribution can be specified with the \sQuote{a} and \sQuote{c} parameter of an exponential decay function, using a list. E.g. DUL = list(0.35, 0, -0.1).
 This means that the top value for DUL will be 0.35 and it will decay with a rate of -0.1. \cr
 4. If an increase and then a decay is needed the Ricker function can be used. See \sQuote{SSricker} in the \sQuote{nlraa} package. \cr

The value of soil particle density (2.65 g/cm3) is hard coded in APSIM. 
https://en.wikipedia.org/wiki/Bulk_density
}
\examples{
\donttest{
 sp <- apsimx_soil_profile()
 require(ggplot2)
 plot(sp)
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{obsWheat}
\alias{obsWheat}
\title{Observed wheat phenology, LAI and biomass}
\format{
A data frame with 10 rows and 4 variables:
\describe{
  \item{Date}{ -date- date starting Oct 1 2016 and ending June 6 2017}
  \item{Wheat.Phenology.Stage}{ -numeric- phenology stage of wheat}
  \item{Wheat.Leaf.LAI}{ -numeric- Leaf Area Index}
  \item{Wheat.AboveGround.Wt}{-numeric- above ground biomass (g/m2)}
}
}
\source{
These are simulated data. For details see the APSIM documentation
}
\usage{
obsWheat
}
\description{
Artificial observed data for Wheat
}
\details{
A dataset containing the Date, phenology stage, LAI and above ground biomass for Wheat
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsod_apsim_met.R
\name{get_gsod_apsim_met}
\alias{get_gsod_apsim_met}
\title{Get GSOD data for an APSIM met file}
\usage{
get_gsod_apsim_met(
  lonlat,
  dates,
  wrt.dir = ".",
  filename = NULL,
  distance = 100,
  station,
  fillin.radn = FALSE
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector}

\item{dates}{date ranges}

\item{wrt.dir}{write directory}

\item{filename}{file name for writing out to disk}

\item{distance}{distance in kilometers for the nearest station}

\item{station}{choose the station either by index or character}

\item{fillin.radn}{whether to fill in radiation data using the nasapower pacakge. Default is FALSE.}
}
\value{
returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Uses \code{\link[GSODR]{get_GSOD}} from the \CRANpkg{GSODR} package to download data to create an APSIM met file.
}
\details{
This function requires the \CRANpkg{GSODR} package.

If the filename is not provided it will not write the file to disk, 
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk.
}
\note{
This source of data does not provide solar radiation. If \sQuote{fillin.radn} is
TRUE it fill in radiation data using the nasapower package.
}
\examples{
\dontrun{
require(GSODR)
## This will not write a file to disk
gsd <- get_gsod_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"),
                          fillin.radn = TRUE)
summary(gsd)
## Check for reasonable ranges
check_apsim_met(gsd)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{write_apsim_met}
\alias{write_apsim_met}
\title{Write an APSIM met file}
\usage{
write_apsim_met(met, wrt.dir = NULL, filename = NULL)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{wrt.dir}{directory where the file will be written}

\item{filename}{optional alternative filename}
}
\value{
does not create an R object, it only writes to disk
}
\description{
Write an object of class \sQuote{met} to disk
}
\details{
Write a met file to disk. It takes an object of class \sQuote{met}

at the moment the read-write cycle will strip comments
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
ames.met <- read_apsim_met("Ames.met", src.dir = extd.dir)
ames.met
tmp.dir <- tempdir()
write_apsim_met(ames.met, wrt.dir = tmp.dir, filename = "Ames.met")
## Here I write to a temporary directory, but change this to where
## you want to write to
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{swim_parms}
\alias{swim_parms}
\title{Helper function to supply SWIM parameters}
\usage{
swim_parms(
  Salb = NA,
  CN2Bare = NA,
  CNRed = NA,
  CNCov = NA,
  KDul = NA,
  PSIDul = NA,
  VC = NA,
  DTmin = NA,
  DTmax = NA,
  MaxWaterIncrement = NA,
  SpaceWeightingFactor = NA,
  SoluteSpaceWeightingFactor = NA,
  Diagnostics = NA,
  SwimWaterTable_WaterTableDepth = NA,
  SwimSubsurfaceDrain_DrainDepth = NA,
  SwimSubsurfaceDrain_DrainSpacing = NA,
  SwimSubsurfaceDrain_DrainRadius = NA,
  SwimSubsurfaceDrain_Klat = NA,
  SwimSubsurfaceDrain_ImpermDepth = NA
)
}
\arguments{
\item{Salb}{see APSIM documentation}

\item{CN2Bare}{see APSIM documentation}

\item{CNRed}{see APSIM documentation}

\item{CNCov}{see APSIM documentation}

\item{KDul}{see APSIM documentation}

\item{PSIDul}{see APSIM documentation}

\item{VC}{see APSIM documentation}

\item{DTmin}{see APSIM documentation}

\item{DTmax}{see APSIM documentation}

\item{MaxWaterIncrement}{see APSIM documentation}

\item{SpaceWeightingFactor}{see APSIM documentation}

\item{SoluteSpaceWeightingFactor}{see APSIM documentation}

\item{Diagnostics}{see APSIM documentation}

\item{SwimWaterTable_WaterTableDepth}{see APSIM documentation}

\item{SwimSubsurfaceDrain_DrainDepth}{see APSIM documentation}

\item{SwimSubsurfaceDrain_DrainSpacing}{see APSIM documentation}

\item{SwimSubsurfaceDrain_DrainRadius}{see APSIM documentation}

\item{SwimSubsurfaceDrain_Klat}{see APSIM documentation}

\item{SwimSubsurfaceDrain_ImpermDepth}{see APSIM documentation}
}
\value{
a \sQuote{list} with class \sQuote{swim_parms}
}
\description{
Creates a list with specific components for the SWIM model
}
\details{
current documentation for APSIM 7.10 \url{https://www.apsim.info/documentation/model-documentation/soil-modules-documentation/swim3/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{apsim.options}
\alias{apsim.options}
\title{Environment which stores APSIM options}
\usage{
apsim.options
}
\value{
This is an environment, so nothing to return.
}
\description{
Environment which can store the path to the executable and where examples are located.
             Creating an environment avoids the use of global variables or other similar practices
             which would have possible undesriable consequences.
}
\details{
Environment which stores APSIM options
}
\examples{
\dontrun{
names(apsim.options)
apsim_options(exe.path = "some-new-path-to-executable")
apsim.options$exe.path
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{auto_detect_apsim_examples}
\alias{auto_detect_apsim_examples}
\title{Auto detect where apsim examples are located}
\usage{
auto_detect_apsim_examples()
}
\value{
will create a directory pointing to APSIM \sQuote{Classic} distributed examples
}
\description{
simple function to detect where APSIM \sQuote{Classic} examples are located
}
\details{
Auto detect where APSIM (7.x) \sQuote{Classic} examples are located
}
\examples{
\dontrun{
ex.dir <- auto_detect_apsim_examples()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{apsimx.options}
\alias{apsimx.options}
\title{Environment which stores APSIM-X options}
\usage{
apsimx.options
}
\value{
This is an environment, not a function, so nothing is returned.
}
\description{
Environment which can store the path to the executable, warning settings and 
             where examples are located.
             Creating an environment avoids the use of global variables or other similar practices
             which would have possible undesriable consequences.
}
\details{
Environment which stores APSIM-X options
}
\examples{
\donttest{
names(apsimx.options)
apsimx_options(exe.path = "some-new-path-to-executable")
apsimx.options$exe.path
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsimx_replacement.R
\name{inspect_apsimx_replacement}
\alias{inspect_apsimx_replacement}
\title{Inspect a replacement component in an .apsimx (JSON) file}
\usage{
inspect_apsimx_replacement(
  file = "",
  src.dir = ".",
  node = NULL,
  node.child = NULL,
  node.subchild = NULL,
  node.subsubchild = NULL,
  node.sub3child = NULL,
  node.sub4child = NULL,
  node.sub5child = NULL,
  node.string = NULL,
  root = list("Models.Core.Replacements", NA),
  parm = NULL,
  display.available = FALSE,
  digits = 3,
  print.path = FALSE,
  verbose = TRUE,
  grep.options
)
}
\arguments{
\item{file}{file ending in .apsimx to be inspected (JSON)}

\item{src.dir}{directory containing the .apsimx file to be inspected; defaults to the current working directory}

\item{node}{specific node to be inspected}

\item{node.child}{specific node child component to be inspected.}

\item{node.subchild}{specific node sub-child to be inspected.}

\item{node.subsubchild}{specific node sub-subchild to be inspected.}

\item{node.sub3child}{specific node sub3child to be inspected.}

\item{node.sub4child}{specific node sub4child to be inspected.}

\item{node.sub5child}{specific node sub5child to be inspected.}

\item{node.string}{passing of a string instead of the node hierarchy.
Do not use this and also the other node arguments. This argument will
overwrite the other node specifications.}

\item{root}{\sQuote{root} for the inspection of a replacement file (it gives flexibility to inspect other types of files).
In previous versions of APSIM (before mid 2023) this was \sQuote{Models.Core.Replacement}. In more recent versions, it needs
to be \sQuote{Models.Core.Folder}.}

\item{parm}{specific parameter to display. It can be a regular expression.}

\item{display.available}{logical. Whether to display available components to be inspected (default = FALSE)}

\item{digits}{number of decimals to print (default 3)}

\item{print.path}{print the path to the inspected parameter (default FALSE)}

\item{verbose}{whether to print additional information, default: TRUE}

\item{grep.options}{Additional options for grep. To be passed as a list. At the moment these are: \sQuote{fixed}, \sQuote{ignore.case} and \sQuote{exact}.
The option \sQuote{exact} is not a grep option, but it can be used to use exact matching (effectively not using grep).
This option will be ignored at the level of \sQuote{Command}.}
}
\value{
table with inspected parameters and values (and \sQuote{parm path} when \sQuote{print.path} = TRUE).
}
\description{
inspect the replacement componenet of an JSON apsimx file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
}
\details{
### Maybe insert information about regex at some point

This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
}
\note{
I need to make some changes in order to be able to handle multiple parameters. At this point, it
might work but it will generate warnings.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
inspect_apsimx_replacement("MaizeSoybean.apsimx", src.dir = extd.dir,
                           node = "Maize", node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response") 
## For Wheat
## getting down to 'XYPairs'
inspect_apsimx_replacement("WheatRye.apsimx", 
                           src.dir = extd.dir,
                           node = "Wheat",
                           node.child = "Structure",
                           node.subchild = "BranchingRate",
                           node.subsubchild = "PotentialBranchingRate",
                           node.sub3child = "Vegetative",
                           node.sub4child = "PotentialBranchingRate",
                           node.sub5child = "XYPairs")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_soil_profile.R
\name{plot.carbon.stocks}
\alias{plot.carbon.stocks}
\title{Plot carbon stocks (function in progress)}
\usage{
\method{plot}{carbon.stocks}(x, ...)
}
\arguments{
\item{x}{object of class \sQuote{soil_profile}}

\item{...}{optional argument, not used at the moment}
}
\value{
returns a graph
}
\description{
Plotting of carbon stocks based on an object of class \sQuote{soil_profile}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_soil_profile.R
\name{compare_apsim_soil_profile}
\alias{compare_apsim_soil_profile}
\alias{print.soil_profile_mrg}
\alias{plot.soil_profile_mrg}
\title{Compare two or more soil profiles}
\usage{
compare_apsim_soil_profile(
  ...,
  soil.var = c("all", "Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS", "Carbon",
    "SoilCNRatio", "FOM", "FOM.CN", "FBiom", "FInert", "NO3N", "NH4N", "CACO3", "PH",
    "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand"),
  property,
  labels,
  merge.wide = TRUE,
  check = FALSE,
  verbose = FALSE
)

\method{print}{soil_profile_mrg}(x, ..., format = c("wide", "long"), digits = 2)

\method{plot}{soil_profile_mrg}(
  x,
  ...,
  plot.type = c("depth", "vs", "diff", "density"),
  pairs = NULL,
  soil.var = c("all", "Thickness", "BD", "AirDry", "LL15", "DUL", "SAT", "KS", "Carbon",
    "SoilCNRatio", "FOM", "FOM.CN", "FBiom", "FInert", "NO3N", "NH4N", "CACO3", "PH",
    "ParticleSizeClay", "ParticleSizeSilt", "ParticleSizeSand"),
  property,
  span = 0.75
)
}
\arguments{
\item{...}{\sQuote{soil_profile} objects. Should be of class \sQuote{soil_profile}}

\item{soil.var}{soil variable to plot}

\item{property}{same as soil.var}

\item{labels}{labels for plotting and identification of \sQuote{soil_profile} objects.}

\item{merge.wide}{whether to attempt to merge soils in \sQuote{wide} format.}

\item{check}{whether to check \sQuote{soil_profile} objects using \sQuote{check_apsimx_soil_profile}.}

\item{verbose}{whether to print agreement values (default is FALSE).}

\item{x}{object of class \sQuote{soil_profile_mrg}}

\item{format}{either \sQuote{wide} or \sQuote{long}, which depends on the merging.}

\item{digits}{number of digits to print (default is 2)}

\item{plot.type}{either \sQuote{depth}, \sQuote{vs}, \sQuote{diff} or \sQuote{density}}

\item{pairs}{pair of objects to compare, defaults to 1 and 2 but others are possible}

\item{span}{argument to be passed to \sQuote{geom_smooth}}
}
\value{
object of class \sQuote{soil_profile_mrg}, which can be used for further plotting

a table with indexes for the soil profiles

it produces a plot
}
\description{
Helper function which allows for a simple comparison among soil_profile objects

print method for \sQuote{soil_profile_mrg}

plotting function for compare_apsim_soil_profile, it requires ggplot2
}
\note{
I have only tested this for 2 or 3 objects. The code is set up to be able to 
compare more, but I'm not sure that would be all that useful.
}
\examples{
\dontrun{
require(soilDB)
require(sp)
require(sf)
require(spData)
# Get two soil profiles
sp1 <- get_ssurgo_soil_profile(lonlat = c(-93, 42))
sp2 <- get_ssurgo_soil_profile(lonlat = c(-92, 41))
# Compare them
cmp <- compare_apsim_soil_profile(sp1[[1]], sp2[[1]], labels = c("sp1", "sp2"))
# Plot the variables
plot(cmp)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_soil_profile.R
\name{available_water_content}
\alias{available_water_content}
\title{Calculate available water content}
\usage{
available_water_content(
  x,
  depth,
  area = c("m", "m2", "ha"),
  method = c("linear", "constant"),
  weights,
  ...
)
}
\arguments{
\item{x}{object of class \sQuote{soil_profile}}

\item{depth}{soil depth (in meters). If missing then the whole soil profile is used.}

\item{area}{either \sQuote{m} meter, \sQuote{m2} meter squared or \sQuote{ha}.}

\item{method}{interpolation method. Either \sQuote{linear} or \sQuote{constant}.}

\item{weights}{optional weights}

\item{...}{additional arguments passed to internal functions (none used at the moment).}
}
\value{
returns a value with attribute \sQuote{units} and \sQuote{depth}
}
\description{
Calculation of available water content based on an object of class \sQuote{soil_profile}
}
\details{
Function to calculate available water content. The output units depend on the choice of area.
If \sQuote{m} is used, then the output units will be \sQuote{mm}. If the \sQuote{area} is \sQuote{m2},
then the output units will be in \sQuote{m3}. If the \sQuote{area} is \sQuote{ha}, then the output units will be \sQuote{kg/ha}.
}
\examples{
\dontrun{
sp <- apsimx_soil_profile()
available_water_content(sp)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{[.met}
\alias{[.met}
\title{Extract or Replace Part of a \sQuote{met} object}
\usage{
\method{[}{met}(x, i, j, ..., drop = TRUE)
}
\arguments{
\item{x}{object of class \sQuote{met}}

\item{i}{index for rows}

\item{j}{index for columns}

\item{...}{additional arguments}

\item{drop}{result is coerced to lowest possible dimension if TRUE.}
}
\description{
Extract or Replace Part of a \sQuote{met} object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_apsim.R
\name{compare_apsim}
\alias{compare_apsim}
\alias{print.out_mrg}
\alias{plot.out_mrg}
\title{Compare two or more apsim output objects}
\usage{
compare_apsim(
  ...,
  variable,
  index = "Date",
  by,
  labels,
  cRSS = FALSE,
  weights,
  verbose = FALSE
)

\method{print}{out_mrg}(x, ..., digits = 2)

\method{plot}{out_mrg}(
  x,
  ...,
  plot.type = c("vs", "diff", "resid", "ts", "density"),
  pairs = c(1, 2),
  cumulative = FALSE,
  variable,
  id,
  id.label,
  by,
  facet = FALSE,
  span = 0.75,
  dodge.width = NULL
)
}
\arguments{
\item{...}{data frames with APSIM output or observed data.}

\item{variable}{variable to plot}

\item{index}{index for merging objects. Default is \sQuote{Date}}

\item{by}{variable in \sQuote{index} used for plotting}

\item{labels}{labels for plotting and identification of objects.}

\item{cRSS}{compute (weighted) combined residual sum of squares using some or all variables}

\item{weights}{optional weights for computing the (weighted) combined sum of squares}

\item{verbose}{whether to print indexes to console (default is FALSE).}

\item{x}{object of class \sQuote{out_mrg}}

\item{digits}{digits to print (default is 2)}

\item{plot.type}{either \sQuote{vs}, \sQuote{diff}, \sQuote{ts} - for time series or \sQuote{density}}

\item{pairs}{pair of objects to compare, defaults to 1 and 2 but others are possible}

\item{cumulative}{whether to plot cumulative values (default FALSE)}

\item{id}{identification. Useful for finding extreme values. If this values is equal to 
1 and no id.label is provided all observations are labeled by the row number. If it is 
less than one points are labeled if their probability is equal or less than the id value.
For example, a value of 0.05 will label values that have a probability of 0.05 (or less)
under a normal distribution.}

\item{id.label}{optional label for the id}

\item{facet}{whether to facet or use color for the by variable (default is FALSE, meaning \sQuote{color})}

\item{span}{argument passed to \sQuote{geom_smooth}}

\item{dodge.width}{optional argument to control the \sQuote{dodge} for the \sQuote{id.label}}
}
\value{
object of class \sQuote{out_mrg}, which can be used for further plotting

it prints the index.table data.frame

it produces a plot
}
\description{
Function which allows for a simple comparison between APSIM output objects

print method for \sQuote{out_mrg}

plotting function for compare_apsim, it requires ggplot2
}
\details{
There are different metrics with different interpretations produced by this function.
Many of them take one object (assuming that is the observed data) and another object
(assuming that is the simulated or predicted) object. Some of these metrics
are a result from the regression.

- metrics that do not rely on regression (obs vs. pred): bias, mean bias, rss, rmse,
corr, concorr, mod.eff

- metrics that do depend on a regression model (Reg): intercept, slope, rsigma, R2. In this case,
a simple linear regression is first adjusted to the two variables and the parameters and
statistics are derived from that model. 

\sQuote{rsigma} is the regression residual standard deviation obtained by using the function
\code{\link[stats]{sigma}} to the regression object.

Plotting function for observed and simulated data
}
\note{
\sQuote{Con Corr} is the concordance correlation coefficient (https://en.wikipedia.org/wiki/Concordance_correlation_coefficient);
\sQuote{ME} is the model efficiency (https://en.wikipedia.org/wiki/Nash-Sutcliffe_model_efficiency_coefficient)
}
\examples{
\donttest{
## Directory with files
extd.dir <- system.file("extdata", package = "apsimx")
## Comparing observed and simulated for Wheat
data(obsWheat)
sim.opt <- read.csv(file.path(extd.dir, "wheat-sim-opt.csv"))
sim.opt$Date <- as.Date(sim.opt$Date)

cap <- compare_apsim(obsWheat, sim.opt, labels = c("obs", "sim"))

plot(cap)
plot(cap, plot.type = "diff")
plot(cap, plot.type = "resid")
plot(cap, plot.type = "ts")

plot(cap, variable = "AboveGround")
plot(cap, variable = "AboveGround", plot.type = "diff")
plot(cap, variable = "AboveGround", plot.type = "ts")

## Selecting a variable
cap <- compare_apsim(obsWheat, sim.opt, variable = "AboveGround", 
                     labels = c("obs", "sim"))
                     
## Using id
plot(cap, variable = "AboveGround", id = 0.05)
                     
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{soilwat_parms}
\alias{soilwat_parms}
\title{Helper function to supply SoilWat parameters}
\usage{
soilwat_parms(
  SummerCona = NA,
  SummerU = NA,
  SummerDate = NA,
  WinterCona = NA,
  WinterU = NA,
  WinterDate = NA,
  DiffusConst = NA,
  DiffusSlope = NA,
  Salb = NA,
  CN2Bare = NA,
  CNRed = NA,
  CNCov = NA,
  Slope = NA,
  DischargeWidth = NA,
  CatchmentArea = NA,
  MaxPond = NA,
  SWCON = NA,
  Thickness = NA
)
}
\arguments{
\item{SummerCona}{see APSIM documentation}

\item{SummerU}{see APSIM documentation}

\item{SummerDate}{see APSIM documentation}

\item{WinterCona}{see APSIM documentation}

\item{WinterU}{see APSIM documentation}

\item{WinterDate}{see APSIM documentation}

\item{DiffusConst}{see APSIM documentation}

\item{DiffusSlope}{see APSIM documentation}

\item{Salb}{soil albedo (see APSIM documentation)}

\item{CN2Bare}{see APSIM documentation}

\item{CNRed}{see APSIM documentation}

\item{CNCov}{see APSIM documentation}

\item{Slope}{see APSIM documentation}

\item{DischargeWidth}{see APSIM documentation}

\item{CatchmentArea}{see APSIM documentation}

\item{MaxPond}{see APSIM documentation}

\item{SWCON}{see APSIM documentation}

\item{Thickness}{provide the corresponding thickness layer}
}
\value{
a \sQuote{list} with class \sQuote{soilwat_parms}
}
\description{
Creates a list with specific components for the SoilWat model
}
\details{
current documentation for APSIM 7.10 \url{https://www.apsim.info/documentation/model-documentation/soil-modules-documentation/soilwat/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{xargs_apsimx}
\alias{xargs_apsimx}
\title{Provide extra arguments for APSIM-X}
\usage{
xargs_apsimx(
  verbose = FALSE,
  csv = FALSE,
  merge.db.files = FALSE,
  list.simulations = FALSE,
  list.referenced.filenames = FALSE,
  single.threaded = FALSE,
  cpu.count = -1L,
  simulation.names = FALSE,
  dotnet = FALSE,
  mono = FALSE,
  exe.path = NA
)
}
\arguments{
\item{verbose}{Write detailed messages to stdout when a simulation starts/finishes.}

\item{csv}{Export all reports to .csv files.}

\item{merge.db.files}{Merge multiple .db files into a single .db file.}

\item{list.simulations}{List simulation names without running them.}

\item{list.referenced.filenames}{List all files that are referenced by an .apsimx file(s).}

\item{single.threaded}{Run all simulations sequentially on a single thread.}

\item{cpu.count}{(Default: -1) Maximum number of threads/processes to spawn for running simulations.}

\item{simulation.names}{Only run simulations if their names match this regular expression.}

\item{dotnet}{Logical. There is a global option for this argument, but this will override it. This 
can be useful if the goal is to compare an old version of Next Gen (before Sept 2021) with a more
recent version in the same script. This might be needed if you have your own compiled version of APSIM Next Gen.}

\item{mono}{Logical. Should be set to TRUE if running a version of APSIM Next Gen from Aug 2021 or older on Mac or Linux.}

\item{exe.path}{executable path. This can be useful for having both a global option through \sQuote{apsimx.options} and
a local option that will override that. This option will take precedence.}
}
\value{
it returns a character vector with the extra arguments.
}
\description{
This provides additinoal command line arguments when running the model
}
\details{
Extra arguments for running APSIM-X
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{read_apsim_all}
\alias{read_apsim_all}
\title{Read all APSIM generated .out files in a directory}
\usage{
read_apsim_all(
  filenames,
  src.dir = ".",
  value = "report",
  date.format = "\%d/\%m/\%Y",
  simplify = TRUE,
  silent = FALSE
)
}
\arguments{
\item{filenames}{names of files to be read}

\item{src.dir}{source directory where files are located}

\item{value}{either \sQuote{report}, \sQuote{user-defined} or \sQuote{all} (not implemented at the moment)}

\item{date.format}{format for adding \sQuote{Date} column}

\item{simplify}{whether to return a single data frame or a list.}

\item{silent}{whether to issue warnings or suppress them}
}
\value{
returns a data frame or a list depending on the argument \sQuote{simplify} above.
}
\description{
Like \code{\link{read_apsim}}, but it can read many .out files in a directory. 
It will read all of them unless these are filtered using a regular expression as an argument
to \sQuote{value}.
}
\details{
Read all APSIM generated .out files in a directory
}
\note{
Warning: very simple function at the moment, not optimized for memory or speed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unit_conv.R
\name{doy2date}
\alias{doy2date}
\alias{date2doy}
\title{Converts from doy to date}
\usage{
doy2date(x, year = 2001, inverse = FALSE)

date2doy(x, year = 2001, inverse = FALSE)
}
\arguments{
\item{x}{either an integer 1-366 or a \sQuote{Date}}

\item{year}{year}

\item{inverse}{if TRUE it goes from doy to \sQuote{Date}}
}
\value{
an object of class \sQuote{Date} or a numeric if inverse equals TRUE.

an numeric or an object of class \sQuote{Date} if inverse equals TRUE.
}
\description{
Given a day of the year as julian (1-366) it converts to \sQuote{Date}

Given a \sQuote{Date} it converts to julian day (1-366) or day of the year
}
\examples{
doy2date(120)
date2doy("04-30")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{check_apsim_met}
\alias{check_apsim_met}
\title{Check a met file for possible errors}
\usage{
check_apsim_met(met)
}
\arguments{
\item{met}{object of class \sQuote{met}}
}
\value{
does not return anything unless possible errors are found
}
\description{
Takes in an object of class \sQuote{met} and checks for missing/valid/reasonable values
}
\details{
It will only check for missing values and reasonable (within range) values for:
 \sQuote{year}: range (1500 to 3000); \cr
 \sQuote{day}: range (1 to 366); \cr 
 \sQuote{maxt}: range (-60 to 60) -- units (C); \cr
 \sQuote{mint}: range (-60 to 40) -- units (C); \cr
 \sQuote{radn}: range (0 to 40) -- units (MJ/m2/day);  \cr
 \sQuote{rain}: range (0 to 100) -- units (mm/day)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optim_apsim.R
\name{optim_apsim}
\alias{optim_apsim}
\alias{print.optim_apsim}
\alias{vcov.optim_apsim}
\alias{coef.optim_apsim}
\alias{confint.optim_apsim}
\title{Optimize parameters in an APSIM simulation}
\usage{
optim_apsim(
  file,
  src.dir = ".",
  crop.file,
  parm.paths,
  data,
  type = c("optim", "nloptr", "mcmc", "ucminf"),
  weights,
  index = "Date",
  parm.vector.index,
  xml.parm,
  ...
)

\method{print}{optim_apsim}(x, ..., digits = 3, level = 0.95)

\method{vcov}{optim_apsim}(object, ..., scaled = TRUE)

\method{coef}{optim_apsim}(object, ..., scaled = FALSE)

\method{confint}{optim_apsim}(object, parm, level = 0.95, ...)
}
\arguments{
\item{file}{file name to be run (the extension .apsim is optional)}

\item{src.dir}{directory containing the .apsim file to be run (defaults to the current directory)}

\item{crop.file}{name of auxiliary xml file where parameters are stored. If this is missing, it is 
assumed that the parameters to be edited are in the main simulation file.}

\item{parm.paths}{absolute paths of the coefficients to be optimized. 
It is recommended that you use \code{\link{inspect_apsim}} or \code{\link{inspect_apsim_xml}}  for this.}

\item{data}{data frame with the observed data. By default is assumes there is a 'Date' column for the index.}

\item{type}{Type of optimization. For now, \code{\link[stats]{optim}} and, if available, \code{\link[nloptr]{nloptr}} or 
\sQuote{mcmc} through \code{\link[BayesianTools]{runMCMC}}. Option \sQuote{ucminf} uses the \code{\link[ucminf]{ucminf}} function.}

\item{weights}{Weighting method or values for computing the residual sum of squares (see Note).}

\item{index}{Index for filtering APSIM output. \sQuote{Date} is currently used. (I have not tested how well it works using anything other than Date).}

\item{parm.vector.index}{Index to optimize a specific element of a parameter vector. At the moment it is
possible to only edit one element at a time. This is because there is a conflict when generating multiple
elements in the candidate vector for the same parameter.}

\item{xml.parm}{optional logical vector used when optimizing parameters which are both in the .apsim file and in the \sQuote{crop.file}.
If \sQuote{crop.file} is missing it is assumed that the paramters to be optimized are in the .apsim file. If \sQuote{crop.file} is
not missing it is assumed that they are in the \sQuote{crop.file}. If the parameters are in both, this needs to be specified in 
this argument.}

\item{...}{additional arguments (none used at the moment)}

\item{x}{object of class \sQuote{optim_apsim}}

\item{digits}{number of digits to round up the output}

\item{level}{confidence level (default is 0.95)}

\item{object}{object of class \sQuote{optim_apsim}}

\item{scaled}{whether to return the scaled or unscaled estimates
(TRUE in the optimized scale, FALSE in the original scale)}

\item{parm}{parameter to select (it can be a regular expression)}
}
\value{
object of class \sQuote{optim_apsim}, but really just a list with results from optim and additional information.

prints to console

it returns the variance-covariance matrix for an object of class \sQuote{optim_apsim}.

a numeric vector with the value of the parameter estimates.

a matrix with lower and upper limits and the point estimate (coef)
}
\description{
It is a wrapper for running APSIM and optimizing parameters using \code{\link[stats]{optim}}

Friendly printing of optim_apsim

Variance-Covariance for an \sQuote{optim_apsim} object

Parameter estimates for an \sQuote{optim_apsim} object

Confidence intervals for parameter estimates for an \sQuote{optim_apsim} object
}
\details{
Simple optimization for APSIM Classic

* This function assumes that you want to optimize parameters which are stored in
an auxiliary XML file. These are typically crop or cultivar specific parameters.
However, it is possible to optimize parameters present in the main simulation
file.

* Only one observation per day is allowed in the data.

* Given how APSIM Classic works, this can only be run when the main simulation
file is in the current directory and the crop file (or XML) 
should be in the same directory as the main simulation.

* The initial values for the optimization should be the ones in the stored
crop parameter file.

* It is suggested that you keep a backup of the original file. This function
will edit and overwrite the file during the optimization. 

* When you use the parm.vector.index you cannot edit two separate elements of
a vector at the same time. This should be used to target a single element of 
a vector only.

* Internally, the optimization is done around the scaled value of the initial parameter
values. A value of 1 would correspond to the inital value of the parameter. 
The \sQuote{lower} and \sQuote{upper} (or \sQuote{ub} and \sQuote{lb}) are also scaled 
to the initial values of the parameters. So, for example, if your initial value is 20 
and you provide an upper bound of 5, it means that the actual upper value that you are allowing for is 100.
}
\note{
When computing the objective function (residual sum-of-squares) different variables are combined.
It is common to weight them since they are in different units. If the argument weights is not supplied
no weighting is applied. It can be \sQuote{mean}, \sQuote{var} or a numeric vector of appropriate length.

This in the scale of the optimized parameters which are
scaled to be around 1.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{apsimx_options}
\alias{apsimx_options}
\title{Setting some options for the package}
\usage{
apsimx_options(
  exe.path = NA,
  dotnet = FALSE,
  mono = FALSE,
  examples.path = NA,
  warn.versions = TRUE,
  warn.find.apsimx = TRUE,
  allow.path.spaces = FALSE,
  system.intern = FALSE
)
}
\arguments{
\item{exe.path}{path to apsim executable. White spaces are not allowed.}

\item{dotnet}{logical indicating if APSIM should be run through the dotnet command}

\item{mono}{logical indicating if the mono command should be used when running APSIM. This is for versions 
for Mac/Linux older than Sept 2021.}

\item{examples.path}{path to apsim examples}

\item{warn.versions}{logical. warning if multiple versions of APSIM-X are detected.}

\item{warn.find.apsimx}{logical. By default a warning will be thrown if APSIM-X is not found. 
If \sQuote{exe.path} is \sQuote{NA} an error will be thrown instead.}

\item{allow.path.spaces}{logical. By default spaces are not allowed in paths or in the run command.}

\item{system.intern}{logical. Default is FALSE. \sQuote{intern} argument passed to \code{\link{system}}.}
}
\value{
as a side effect it modifies the \sQuote{apsimx.options} environment.
}
\description{
Set the path to the APSIM-X executable, examples and warning suppression.
}
\details{
Set apsimx options
}
\note{
It is possible that APSIM-X is installed in some alternative location other than the 
      defaults ones. Guessing this can be difficult and then the auto_detect functions might
      fail. Also, if multiple versions of APSIM-X are installed apsimx will choose the newest
      one but it will issue a warning. Suppress the warning by setting warn.versions = FLASE.
}
\examples{
\donttest{
names(apsimx.options)
apsimx_options(exe.path = "some-new-path-to-executable")
apsimx.options$exe.path
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_apsim_met.R
\name{compare_apsim_met}
\alias{compare_apsim_met}
\alias{print.met_mrg}
\alias{plot.met_mrg}
\title{Compare two or more \sQuote{met} files}
\usage{
compare_apsim_met(
  ...,
  met.var = c("all", "radn", "maxt", "mint", "rain", "rh", "wind_speed", "vp"),
  labels,
  check = FALSE,
  verbose = FALSE
)

\method{print}{met_mrg}(x, ..., digits = 2)

\method{plot}{met_mrg}(
  x,
  ...,
  plot.type = c("vs", "diff", "ts", "density"),
  pairs = c(1, 2),
  cumulative = FALSE,
  met.var = c("radn", "maxt", "mint", "rain"),
  id,
  span = 0.75
)
}
\arguments{
\item{...}{met file objects. Should be of class \sQuote{met}}

\item{met.var}{meteorological variable to plot}

\item{labels}{labels for plotting and identification of \sQuote{met} objects.}

\item{check}{whether to check \sQuote{met} objects using \sQuote{check_apsim_met}.}

\item{verbose}{whether to print agreement stats to console (default is FALSE).}

\item{x}{object of class \sQuote{met_mrg}}

\item{digits}{digits to print (default is 2)}

\item{plot.type}{either \sQuote{vs}, \sQuote{diff}, \sQuote{ts} - for time series or \sQuote{density}}

\item{pairs}{pair of objects to compare, defaults to 1 and 2 but others are possible}

\item{cumulative}{whether to plot cumulative values (default FALSE)}

\item{id}{identification (not implemented yet)}

\item{span}{argument to be passed to \sQuote{geom_smooth}}
}
\value{
object of class \sQuote{met_mrg}, which can be used for further plotting

it prints the index.table data.frame

it produces a plot
}
\description{
Helper function which allows for a simple comparison among \sQuote{met} objects

print method for \sQuote{met_mrg}

plotting function for compare_apsim_met, it requires ggplot2
}
\details{
If the \sQuote{met} files do not have the same number of columns or the same
names for the data frame, then an error will be returned. If the years are not
the same a message will be returned with the different years and the years 
in common.
}
\note{
I have only tested this for 2 or 3 objects. The code is set up to be able to 
compare more, but I'm not sure that would be all that useful.
}
\examples{
\dontrun{
require(nasapower)
## Specify the location
lonlat <- c(-93, 42)
## dates
dts <- c("2017-01-01","2017-12-31")
## Get pwr
pwr <- get_power_apsim_met(lonlat = lonlat, dates = dts)
## Get data from IEM
iem <- get_iem_apsim_met(lonlat = lonlat, dates = dts)
## Compare them
cmet <- compare_apsim_met(pwr[,1:6], iem, labels = c("pwr","iem"))
## Visualize radiation
plot(cmet, met.var = "radn")
plot(cmet, plot.type = "diff")
plot(cmet, plot.type = "ts")
## Visualize maxt
plot(cmet, met.var = "maxt")
plot(cmet, met.var = "maxt", plot.type = "diff")
plot(cmet, met.var = "maxt", plot.type = "ts")
## Cumulative rain
plot(cmet, met.var = "rain", plot.type = "ts", cumulative = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{amp_apsim_met}
\alias{amp_apsim_met}
\title{Calculates attribute amp for an object of class \sQuote{met}}
\usage{
amp_apsim_met(met, by.year = TRUE)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{by.year}{whether to perform calculations by year (default is TRUE)}
}
\value{
an object of class \sQuote{met} with a recalculation of annual amplitude in mean monthly temperature
}
\description{
This function can re-calculate annual mean monthly amplitude
for an object of class \sQuote{met}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsimx_replacement.R
\name{edit_apsimx_replacement}
\alias{edit_apsimx_replacement}
\title{Edit a replacement component in an .apsimx (JSON) file}
\usage{
edit_apsimx_replacement(
  file = "",
  src.dir = ".",
  wrt.dir = ".",
  node = NULL,
  node.child = NULL,
  node.subchild = NULL,
  node.subsubchild = NULL,
  node.sub3child = NULL,
  node.sub4child = NULL,
  node.sub5child = NULL,
  node.string = NULL,
  root = list("Models.Core.Replacements", NA),
  parm = NULL,
  value = NULL,
  overwrite = FALSE,
  edit.tag = "-edited",
  verbose = TRUE,
  grep.options
)
}
\arguments{
\item{file}{file ending in .apsimx to edit (JSON)}

\item{src.dir}{directory containing the .apsimx file; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{node}{specific node to edit}

\item{node.child}{specific node child component to edit.}

\item{node.subchild}{specific node sub-child to edit.}

\item{node.subsubchild}{specific node sub-subchild to edit.}

\item{node.sub3child}{specific node sub-sub-subchild to edit.}

\item{node.sub4child}{specific node sub-sub-sub-subchild to edit.}

\item{node.sub5child}{specific node sub-sub-sub-sub-subchild to edit.}

\item{node.string}{passing of a string instead of the node hierarchy. It can either start with a dot or not.
However, the \sQuote{best} form is not to start with a dot as it should be a more convenient form of passing
the nodes and their childs and not a real \sQuote{jsonpath}.}

\item{root}{\sQuote{root} node to explore (default = \dQuote{Models.Core.Replacements})}

\item{parm}{specific parameter to edit}

\item{value}{new values for the parameter}

\item{overwrite}{logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise}

\item{edit.tag}{if the file is edited a different tag from the default \sQuote{-edited} can be used.}

\item{verbose}{whether to print information about successful edit}

\item{grep.options}{Additional options for grep. To be passed as a list.}
}
\value{
(when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
As a side effect this function creates a new (JSON) .apsimx file.
}
\description{
edit the replacement componenet of an JSON apsimx file. It does not replace the GUI, but it can save time by quickly editing parameters and values.
}
\details{
This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
}
\note{
The components that can be edited are restricted becuase this is better in preventing
errors of editing unintended parts of the file.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
## Writing to a temp directory, but change as needed
tmp.dir <- tempdir()

## Inspect original values
inspect_apsimx_replacement("MaizeSoybean.apsimx", 
                           src.dir = extd.dir,
                           node = "Maize", 
                           node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response")

edit_apsimx_replacement("MaizeSoybean.apsimx", 
                        src.dir = extd.dir, wrt.dir = tmp.dir,
                        node = "Maize", 
                        node.child = "Phenology",
                        node.subchild = "ThermalTime", 
                        node.subsubchild = "BaseThermalTime",
                        node.sub3child = "Response",
                        parm = "X",
                        value = c(10, 20, 30, 40, 50)) 
## inspect it
inspect_apsimx_replacement("MaizeSoybean-edited.apsimx", 
                           src.dir = tmp.dir,
                           node = "Maize", 
                           node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response")

## Illustrating using 'node.string'
## Equivalent to the code to edit above

edit_apsimx_replacement("MaizeSoybean-edited.apsimx", 
                        src.dir = tmp.dir, wrt.dir = tmp.dir,
                        node.string = "Maize.Phenology.ThermalTime.BaseThermalTime.Response",
                        parm = "X",
                        value = c(11, 21, 31, 41, 51),
                        edit.tag = "-ns") 
                        
inspect_apsimx_replacement("MaizeSoybean-edited-ns.apsimx", 
                           src.dir = tmp.dir,
                           node = "Maize", 
                           node.child = "Phenology",
                           node.subchild = "ThermalTime", 
                           node.subsubchild = "BaseThermalTime",
                           node.sub3child = "Response")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_slga_soil_profile.R
\name{get_slga_soil_profile}
\alias{get_slga_soil_profile}
\title{Generate a synthetic APSIM soil profile from the SLGA soil database}
\usage{
get_slga_soil_profile(
  lonlat,
  statistic = c("mean", "Q0.5"),
  soil.profile,
  find.location.name = TRUE,
  fix = FALSE,
  verbose = TRUE,
  check = TRUE,
  physical = c("default", "SR"),
  xargs = NULL
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(151.8306, -27.4969)).}

\item{statistic}{default is the mean}

\item{soil.profile}{a soil profile to fill in in case the default one is not appropriate}

\item{find.location.name}{default is TRUE. Use either maps package or photon API to find Country/State.
If you are running this function many times it might be better to set this to FALSE.}

\item{fix}{whether to fix compatibility between saturation and bulk density (default is FALSE).}

\item{verbose}{argument passed to the fix function.}

\item{check}{whether to check the soil profile (default is TRUE)}

\item{physical}{whether soil physical properties are obtained from the data base or through \sQuote{SR}, Saxton and Rawls pedotransfer functions.}

\item{xargs}{additional arguments passed to \code{\link{apsimx_soil_profile}} or \sQuote{apsimx:::approx_soil_variable} function. At the moment these are:
\sQuote{soil.bottom}, \sQuote{method} and \sQuote{nlayers}.}
}
\value{
it generates an object of class \sQuote{soil_profile}.
}
\description{
Retrieves soil data from the SLGA database (Australia) and converts it to an APSIM soil_profile object
}
\details{
Source: https://esoil.io/TERNLandscapes/Public/Pages/SLGA/index.html
}
\examples{
\dontrun{
## Get soil profile properties for a single point
  sp1 <- get_slga_soil_profile(lonlat = c(151.8306, -27.4969), fix = TRUE, verbose = FALSE)
  ## Visualize
  plot(sp1)
  plot(sp1, property = "water")
}
}
\seealso{
\code{\link{apsimx_soil_profile}}, \code{\link{edit_apsim_replace_soil_profile}}, \code{\link{edit_apsimx_replace_soil_profile}}.
}
\author{
Fernando E. Miguez, Chloe (Yunru Lai), Eric Zurcher (CSIRO) and Andrew Moore (CSIRO)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{add_column_apsim_met}
\alias{add_column_apsim_met}
\alias{$<-.met}
\alias{[<-.met}
\alias{[[<-.met}
\alias{remove_column_apsim_met}
\title{Add a column to an object of class \sQuote{met}}
\usage{
add_column_apsim_met(met, value, name, units)

\method{$}{met}(x, name) <- value

\method{[}{met}(x, i, j) <- value

\method{[[}{met}(x, i, j) <- value

remove_column_apsim_met(met, name)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{value}{value for the data.frame. It could be an integer, double or vector of length equal to the number of rows in x.}

\item{name}{name of the variable to be removed}

\item{units}{units for the new column (required)}

\item{x}{object of class \sQuote{met}}

\item{i}{indices specifying elements to extract or replace.}

\item{j}{indices specifying elements to extract or replace.}
}
\value{
an object of class \sQuote{met} with the additional column

an object of class \sQuote{met} without the variable (column) in \sQuote{name}
}
\description{
The usual way of adding a column to a data frame might
not work for an object of class \sQuote{met}, so this method is recommended
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
ames <- read_apsim_met("Ames.met", src.dir = extd.dir)

## The recommended method is
val <- abs(rnorm(nrow(ames), 10))
ames <- add_column_apsim_met(ames, value = val, name = "vp", units = "(hPa)")

## This is also possible
vp <- data.frame(vp = abs(rnorm(nrow(ames), 10)))
attr(vp, "units") <- "(hPa)"
ames$vp <- vp$vp

## or
ames[["vp"]] <- vp

## This is needed to ensure that units and related attributes are also removed
ames <- remove_column_apsim_met(ames, "vp")
## However, ames$vp <- NULL will also work
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsimx_json.R
\name{inspect_apsimx_json}
\alias{inspect_apsimx_json}
\title{Inspect an .apsimx or .json (JSON) file}
\usage{
inspect_apsimx_json(
  file = "",
  src.dir = ".",
  parm,
  search.depth = 15,
  print.path = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{file}{file ending in .apsimx or .json to be inspected (JSON)}

\item{src.dir}{directory containing the .apsimx or .json file to be inspected; defaults to the current working directory}

\item{parm}{string or regular expression for partial matching. It can be two strings separated by a period to search within a node (child).}

\item{search.depth}{default is 15. How deep should the algorithm explore the structure of the list.}

\item{print.path}{whether to print the parameter path (default is FALSE)}

\item{verbose}{whether to print additional information (mostly used for debugging)}
}
\value{
prints a table with inspected parameters and values (and the path when \sQuote{print.path} = TRUE).
}
\description{
inspect an .apsimx or .json (JSON) file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
}
\details{
This function is a work in progress. There are many instances for which it will not work.

 It will probably only find the first instance that matches.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
## It seems to work for simple search
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Version")
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Simulations")
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Clock")
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Weather")
## Does return soil components
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "DUL")
## Or cultivar
inspect_apsimx_json("Wheat.apsimx", src.dir = extd.dir, parm = "Hartog")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{impute_apsim_met}
\alias{impute_apsim_met}
\title{Perform imputation for missing data in a met file}
\usage{
impute_apsim_met(
  met,
  method = c("approx", "spline", "mean"),
  verbose = FALSE,
  ...
)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{method}{method for imputation, \sQuote{approx} (\code{\link[stats]{approxfun}}),
\sQuote{spline} (\code{\link[stats]{splinefun}}) or \sQuote{mean} (\code{\link{mean}}).}

\item{verbose}{whether to print missing data to the console, default = FALSE}

\item{...}{additional arguments to be passed to imputation method}
}
\value{
an object of class \sQuote{met} with attributes
}
\description{
Takes in an object of class \sQuote{met} and imputes values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_soils.R
\name{read_apsim_soils}
\alias{read_apsim_soils}
\title{Read in a soils (XML) file into a list of \sQuote{soil_profile} objects}
\usage{
read_apsim_soils(file, src.dir = ".", verbose = TRUE)
}
\arguments{
\item{file}{name of the file (the extension sohuld be .soils)}

\item{src.dir}{directory containing the .soils file (defaults to the current directory)}

\item{verbose}{whether to print additional information about the progress of reading 
the individual soils in.}
}
\description{
APSIM soils can be stored as XML files (soils) and reading them in
converts them into a list of individual objects of class \sQuote{soil_profile}
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")

sls <- read_apsim_soils("Clarion.soils", src.dir = extd.dir)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{tav_apsim_met}
\alias{tav_apsim_met}
\title{Calculates attribute amp for an object of class \sQuote{met}}
\usage{
tav_apsim_met(met, by.year = TRUE, na.rm = TRUE)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{by.year}{whether to compute tav for each year and then average (default is TRUE)}

\item{na.rm}{whether to remove missing values (NAs). Default is TRUE.}
}
\value{
an object of class \sQuote{met} with a recalculation of annual mean temperature amplitude
}
\description{
This function can re-calculate annual mean temperature
for an object of class \sQuote{met}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view_apsimx.R
\name{view_apsim}
\alias{view_apsim}
\title{Viewing an APSIM Classic file interactively}
\usage{
view_apsim(file, src.dir, viewer = c("json", "react"), ...)
}
\arguments{
\item{file}{a file ending in .apsim to be inspected (XML)}

\item{src.dir}{directory containing the .apsim file to be inspected; defaults to the current working directory}

\item{viewer}{either \dQuote{json} or \dQuote{react}.}

\item{...}{additional arguments passed to either \sQuote{jsonedit} or \sQuote{reactjson}.
These are functions in package \CRANpkg{listviewer}.}
}
\value{
a display with the APSIM file structure.
}
\description{
Generate an interactive viewer for an APSIM file
}
\note{
I do not know how to edit an APSIM file using this method yet.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
## View the structure of the APSIM-X simulation file
view_apsim("Millet.apsim", src.dir = extd.dir)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{wop.h}
\alias{wop.h}
\title{Wheat example optimization results plus Hessian}
\format{
An object of class 'optim_apsim' 
\describe{
  \item{ wop.h }{ wheat optimization results plus Hessian}
}
}
\source{
Result of running the examples in Parameter Optimization vignette with the added Hessian
}
\usage{
wop.h
}
\description{
Results from Wheat optimization example plus the Hessian
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sens_apsim.R
\name{sens_apsim}
\alias{sens_apsim}
\title{Sensitivity Analysis for APSIM Next Generation simulation}
\usage{
sens_apsim(
  file,
  src.dir = ".",
  crop.file,
  parm.paths,
  parm.vector.index,
  xml.parm,
  grid,
  summary = c("mean", "max", "var", "sd", "none"),
  root,
  verbose = TRUE,
  cores = 1L,
  save,
  ...
)
}
\arguments{
\item{file}{file name to be run (with extension .apsim)}

\item{src.dir}{directory containing the .apsim file to be run (defaults to the current directory)}

\item{crop.file}{name of auxiliary xml file where parameters are stored. If this is missing, it is 
assumed that the parameters to be edited are in the main simulation file.}

\item{parm.paths}{absolute or relative paths of the coefficients to be evaluated. 
It is recommended that you use \code{\link{inspect_apsim}} for this}

\item{parm.vector.index}{Index to evaluate a specific element of a parameter vector.  At the moment it is
possible to only edit one element at a time. This is because there is a conflict when generating multiple
elements in the candidate vector for the same parameter.}

\item{xml.parm}{TRUE or FALSE for each parameter. Indicating whether it is part of 
an xml file. Its length should be equal to the length or \sQuote{parm.paths}.}

\item{grid}{grid of parameter values for the evaluation. It can be a data.frame.}

\item{summary}{function name to use to summarize the output to be a sinlge row (default is the mean).}

\item{root}{root argument for \code{\link{edit_apsim}}}

\item{verbose}{whether to print progress in percent and elapsed time.}

\item{cores}{number of cores to use for parallel evaluation}

\item{save}{whether to save intermediate results. By default they will be saved as a
\sQuote{csv} file using the name of the apsim file. This will replace \sQuote{apsim} with \sQuote{csv}.
It is also possible to provide the file name here (for example: \sQuote{Some_results.csv}).}

\item{...}{additional arguments (none used at the moment).}
}
\value{
object of class \sQuote{sens_apsim}, but really just a list with results from the evaluations.
}
\description{
It is a wrapper for running APSIM and evaluating different parameters values
}
\note{
The summary function is stored as an attribute of the data frame \sQuote{grid.sims}.
}
\examples{
\donttest{
## See the vignette for examples
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{as_apsim_met}
\alias{as_apsim_met}
\title{Conversion from data frame to met object}
\usage{
as_apsim_met(
  x,
  filename = "noname.met",
  site = "nosite",
  latitude = 0,
  longitude = 0,
  tav = NA,
  amp = NA,
  colnames = c("year", "day", "radn", "maxt", "mint", "rain"),
  units = c("()", "()", "(MJ/m2/day)", "(oC)", "(oC)", "(mm)"),
  constants = NA,
  comments = NA,
  check = TRUE
)
}
\arguments{
\item{x}{object of class \sQuote{data frame}}

\item{filename}{default \sQuote{noname.met}}

\item{site}{default \sQuote{nosite}}

\item{latitude}{default is zero (0)}

\item{longitude}{default is zero (0)}

\item{tav}{average temperature (calculated if not supplied)}

\item{amp}{temperature amplitude (calculated if not supplied)}

\item{colnames}{default are \dQuote{year}, \dQuote{day}, \dQuote{radn}, 
\dQuote{maxt}, \dQuote{mint}, \dQuote{rain}}

\item{units}{default are \dQuote{()}, \dQuote{()}, \dQuote{(MJ/m2/day)},
\dQuote{(oC)}, \dQuote{(oC)}, \dQuote{(mm)}}

\item{constants}{default is \dQuote{NA}}

\item{comments}{default is \dQuote{NA}}

\item{check}{whether to check the resulting met file using \code{\link{check_apsim_met}}.
default is TRUE.}
}
\value{
it returns an object of class \sQuote{met}.
}
\description{
It makes minimum assumptions about the data so it is recommended to change defaults
}
\details{
Simple utility for converting a data frame to an object of class met
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{napad_apsim_met}
\alias{napad_apsim_met}
\title{Pad a met file with NAs when there are date discontinuities}
\usage{
napad_apsim_met(met)
}
\arguments{
\item{met}{object of class \sQuote{met}}
}
\value{
It returns an object of class \sQuote{met} with padded NAs.
}
\description{
It will fill in or \sQuote{pad} a met object with NAs
}
\details{
Fill in with missing data date discontinuities in a met file
}
\note{
The purpose of this function is to allow for imputation using \code{\link{impute_apsim_met}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sens_apsimx.R
\name{sens_apsimx}
\alias{sens_apsimx}
\alias{summary.sens_apsim}
\alias{print.sens_apsim}
\title{Sensitivity Analysis for APSIM Next Generation simulation}
\usage{
sens_apsimx(
  file,
  src.dir = ".",
  parm.paths,
  convert,
  replacement,
  grid,
  soil.profiles = NULL,
  summary = c("mean", "max", "var", "sd", "none"),
  root,
  verbose = TRUE,
  cores = 1L,
  save = FALSE,
  ...
)

\method{summary}{sens_apsim}(
  object,
  ...,
  formula,
  scale = FALSE,
  select = "all",
  warning = TRUE,
  verbose = TRUE
)

\method{print}{sens_apsim}(x, ..., variables = FALSE, summary = FALSE)
}
\arguments{
\item{file}{file name to be run (the extension .apsimx is optional)}

\item{src.dir}{directory containing the .apsimx file to be run (defaults to the current directory)}

\item{parm.paths}{absolute or relative paths of the coefficients to be evaluated.
It is recommended that you use \code{\link{inspect_apsimx}} for this}

\item{convert}{(logical) This argument is needed if there is a need to pass a vector instead of a single value.
The vector can be passed as a character string (separated by spaces) and it will be converted to a
numeric vector. It should be either TRUE or FALSE for each parameter.}

\item{replacement}{TRUE or FALSE for each parameter. Indicating whether it is part of
the \sQuote{replacement} component. Its length should be equal to the length or \sQuote{parm.paths}.}

\item{grid}{grid of parameter values for the evaluation. It can be a data.frame.}

\item{soil.profiles}{list with soil profiles for replacement (see details.)}

\item{summary}{whether to print the full summary of the grid simulations (default is FALSE)}

\item{root}{root argument for \code{\link{edit_apsimx_replacement}}}

\item{verbose}{whether to print to console results of summary}

\item{cores}{number of cores to use for parallel evaluation}

\item{save}{whether to save intermediate results. By default they will be saved as a
\sQuote{csv} file using the name of the apsim file. This will replace \sQuote{apsimx} with \sQuote{csv}.
It is also possible to provide the file name here (for example: \sQuote{Some_results.csv}).}

\item{...}{additional arguments (none used at the moment)}

\item{object}{object of class \sQuote{sens_apsim}}

\item{formula}{formula to be passed to analysis of variance. See \code{\link{formula}}.}

\item{scale}{if all inputs are numeric it is better to scale them. The
default is FALSE as some inputs might be characters or factors. In this
case all inputs will be treated as factors in the sum of squares decomposition.}

\item{select}{option for selecting specific variables in the APSIM output. It will be treated as a regular expression}

\item{warning}{whether to issue a warning when applying this function to an object which has not been summarized}

\item{x}{object of class \sQuote{sens_apsim}}

\item{variables}{whether to print APSIM output variables (default is FALSE)}
}
\value{
object of class \sQuote{sens_apsim}, but really just a list with results from the evaluations.

prints to console if verbose and returns a data frame

compact printing
}
\description{
It is a wrapper for running APSIM-X and evaluating different parameters values

Summary computes variance-based sensitivity indexes from an object of class \sQuote{sens_apsim}

Print method for an object of class \sQuote{sens_apsim}
}
\details{
It is possible to provide a list of soil profiles for replacement in the simulations. In this
case, the parameter path can be simply \sQuote{soil.profile} or \sQuote{soil_profile} if there
is one single simulation. It can also be the path to \sQuote{Soil}. In this case, the path should
be something such as \sQuote{Simulations.SimulationName.Soil}. \sQuote{SimulationName} should
be replaced with the appropriate string.

In the grid, the column with name \sQuote{soil.profile} should contain integers that will be used to
pick from the list of provided soil profiles. In this case it is possible to re-use them.
For example, the values could be 1, 2, 3, etc. to select the corresponding soil profiles from
the \sQuote{soil.profiles} list.

If the \sQuote{cores} argument is greater than 1, then the package \CRANpkg{future} is required.
It will first search for a future plan under options and if nothing is found it will chose an OS-appropriate plan
(\sQuote{multisession}) and it uses the chosen number of cores for execution. Errors, messages and warnings are normally 
suppressed during parallel execution, so it is important to ensure that the simulations are constructed properly.
In testing, cloud services (Box, Dropbox, etc.) do not work well as they seem to interfere with the syncing
of apsim database files. It is suggested that they are turned off when running simulations.

In version 2.8.0 and earlier the original file was changed after it was edited by the function.
In newer versions, the function creates a backup file and then restores it after the code is
executed, if there are no errors.

Suggested reading on the topic of sensitivity analysis:

Pianosa et al (2016). Sensitivity analysis of environmental models: A systematic review with practical workflow.
\doi{10.1016/j.envsoft.2016.02.008}

Saltelli et al. . Global Sensitivity Analysis.
}
\note{
The summary function is stored as an attribute of the data frame \sQuote{grid.sims}.
}
\examples{
\donttest{
## See the vignette for examples
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsimx_json.R
\name{inspect_apsimx}
\alias{inspect_apsimx}
\title{Inspect an .apsimx (JSON) file}
\usage{
inspect_apsimx(
  file = "",
  src.dir = ".",
  node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop",
    "Manager", "Report", "Operations", "Other"),
  soil.child = c("Metadata", "Water", "InitialWater", "Chemical", "Physical", "Analysis",
    "SoilWater", "InitialN", "CERESSoilTemperature", "SoilTemperature", "Sample",
    "Solute", "NO3", "NH4", "Urea", "Nutrient", "Organic", "Swim3"),
  parm = NULL,
  digits = 3,
  print.path = FALSE,
  root
)
}
\arguments{
\item{file}{file ending in .apsimx to be inspected (JSON)}

\item{src.dir}{directory containing the .apsimx file to be inspected; defaults to the current working directory}

\item{node}{specific node to be inspected either \sQuote{Clock}, \sQuote{Weather},
\sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop},
 \sQuote{Manager}, \sQuote{Operations} or \sQuote{Other}}

\item{soil.child}{specific soil component to be inspected. The options vary depending on what is available (see details)}

\item{parm}{parameter to refine the inspection of the \sQuote{manager} list(\sQuote{parm},\sQuote{position}), use \sQuote{NA} for all the positions. \sQuote{parm} can be a regular expression for partial matching.}

\item{digits}{number of decimals to print (default 3). Not used now because everything is a character.}

\item{print.path}{whether to print the path to the specific parameter. Useful to give the later editing. (Also returned as \sQuote{invisible})}

\item{root}{root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.}
}
\value{
prints a table with inspected parameters and values (and \sQuote{parm path} when \sQuote{print.path} = TRUE).
}
\description{
inspect a JSON apsimx file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
}
\details{
In general, this function is used to inspect one parameter at a time. There are some exceptions. \cr

When node equals \sQuote{Other} there are several options. If \sQuote{parm} is not specified the structure
of the simulation file will be returned. In this case, the parameter to print is typically just \sQuote{Simulations}.
This option is useful when the intention is to show the simulation structure to pick a root presumably. \sQuote{parm}
can be set as 0, 1, 2 or 3 for different levels.
\sQuote{parm} can also be a list with integers, such as \sQuote{list(1, 2, 3)}. If zero is included, available elements
If a parameter is specified the function will try to \sQuote{guess} the root elements from the parameter path supplied.

This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsimx file.
         To investigate the available \sQuote{soil.childs} specify \sQuote{Soil} for \sQuote{node} and do not specify the \sQuote{soil.child}.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Clock")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Weather")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Metadata")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Physical")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "SoilWater")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Organic")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "Chemical")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "InitialWater")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Soil", soil.child = "InitialN")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "SurfaceOrganicMatter")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "MicroClimate")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Crop")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager")
inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Report")

## Examples of using node = "Other"
extd.dir <- system.file("extdata", package = "apsimx")

## When parm is not provided
inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir, node = "Other")
## When parm = 2
inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
               node = "Other", parm = 2)
## When parm = 3
inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
               node = "Other", parm = 3)
## When parm is a path
inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
               node = "Other", parm = ".Simulations.Simulation")
## When parm is a list with numbers (integers)
pp <- inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
                     node = "Other", parm = list(1, 1, 5),
                     print.path = TRUE)
## Same as above, but with zero prints possible options
inspect_apsimx("maize-manager-folder.apsimx", src.dir = extd.dir,
                node = "Other", parm = list(1, 1, 5, 0))

## It is possible to look into folders using this method
inspect_apsimx("maize-manager-folder.apsimx", node = "Other", src.dir = extd.dir,
               parm = list("Manager", "Fertiliser", "Amount"))



}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{soilorganicmatter_parms}
\alias{soilorganicmatter_parms}
\title{Helper function to supply additional Soil Organic Matter parameters}
\usage{
soilorganicmatter_parms(
  RootCN = NA,
  RootWt = NA,
  EnrACoeff = NA,
  EnrBCoeff = NA,
  OCUnits = NA
)
}
\arguments{
\item{RootCN}{Root Carbon:Nitrogen ratio (see APSIM documentation)}

\item{RootWt}{Root weight (see APSIM documentation)}

\item{EnrACoeff}{(see APSIM documentation)}

\item{EnrBCoeff}{(see APSIM documentation)}

\item{OCUnits}{Organic Carbon Units}
}
\description{
Creates a list with specific components for the Soil Organic Matter module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_iemre_apsim_met.R
\name{get_iemre_apsim_met}
\alias{get_iemre_apsim_met}
\title{Get weather data from Iowa Environmental Mesonet Reanalysis}
\usage{
get_iemre_apsim_met(
  lonlat,
  dates,
  wrt.dir = ".",
  filename = NULL,
  fillin.radn = FALSE
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector}

\item{dates}{date ranges}

\item{wrt.dir}{write directory}

\item{filename}{file name for writing out to disk}

\item{fillin.radn}{whether to fill in radiation data using the nasapower pacakge. Default is FALSE.}
}
\value{
returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Retrieves weather data from Iowa Environmental Mesonet Reanalysis into an APSIM met file
}
\details{
The original data can be obtained from: (mesonet.agron.iastate.edu/iemre)

If the filename is not provided it will not write the file to disk, 
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk.
}
\note{
Multi-year query is not supported for this product.
}
\examples{
\dontrun{
## This will not write a file to disk
iemre <- get_iemre_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
## Note that solar radiation is not available, but can be filled in
## using the nasapower package
iemre2 <- get_iemre_apsim_met(lonlat = c(-93,42), 
                             dates = c("2012-01-01","2012-12-31"), 
                             fillin.radn = TRUE)
summary(iemre)
summary(iemre2)

## Still it is important to check this object
check_apsim_met(iemre2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optim_apsim.R
\name{mcmc.apsim.env}
\alias{mcmc.apsim.env}
\title{Environment to store data for apsim MCMC}
\usage{
mcmc.apsim.env
}
\value{
This is an environment, so nothing to return.
}
\description{
Environment which stores data for MCMC
}
\details{
Create an apsim environment for MCMC
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unit_conv.R
\name{unit_conv}
\alias{unit_conv}
\title{performs common unit conversions}
\usage{
unit_conv(x, from, to, ...)
}
\arguments{
\item{x}{input variable}

\item{from}{original units}

\item{to}{target units}

\item{...}{additional arguments passed to specific conversions}
}
\value{
value of the input variable with new units
}
\description{
This function is slowly getting better. Adding more unit conversions as I need them.
}
\details{
Function which performs common unit conversions

At the moment possible conversions are:
\itemize{

\item \sQuote{g/m2} to \sQuote{kg/ha}
\item \sQuote{kg/ha} to \sQuote{g/m2}
\item \sQuote{lb} to \sQuote{kg}
\item \sQuote{kg} to \sQuote{lb}
\item \sQuote{maize bu} to \sQuote{kg}
\item \sQuote{kg} to \sQuote{maize bu}
\item \sQuote{soy bu} to \sQuote{kg}
\item \sQuote{kg} to \sQuote{soy bu}
\item \sQuote{maize bu/ac} to \sQuote{kg/ha}
\item \sQuote{maize bu/ac} to \sQuote{g/m2}
\item \sQuote{kg/ha} to \sQuote{maize bu/ac}
\item \sQuote{g/m2} to \sQuote{maize bu/ac}
\item \sQuote{soy bu/ac} to \sQuote{kg/ha}
\item \sQuote{soy bu/ac} to \sQuote{g/m2}
\item \sQuote{kg/ha} to \sQuote{soy bu/ac}
\item \sQuote{g/m2} to \sQuote{soy bu/ac}
\item \sQuote{mm} to \sQuote{inches}
\item \sQuote{inches} to \sQuote{mm}
\item \sQuote{lb/ac} to \sQuote{kg/ha}
\item \sQuote{kg/ha} to \sQuote{lb/ac}
\item \sQuote{lb/ac} to \sQuote{g/m2}
\item \sQuote{g/m2} to \sQuote{lb/ac}
\item \sQuote{decimal} to \sQuote{degrees}
\item \sQuote{degrees} to \sQuote{decimal}
\item \sQuote{Fahrenheit} to \sQuote{Celsius}
\item \sQuote{Celsius} to \sQuote{Fahrenheit} 
}

This is for metric and Imperial conversions
Source: https://www.extension.iastate.edu/agdm/wholefarm/html/c6-80.html
}
\examples{
\donttest{
grain.yield.gm2 <- 600
grain.yield.kgha <- unit_conv(grain.yield.gm2, from = "g/m2", to = "kg/ha")
grain.yield.kgha
## Converting coordinates
require(sp)
unit_conv("42d 0' 0\" N", from = "degrees", to = "decimal")
unit_conv(42, from = "decimal", to = "degrees") ## EW by default
unit_conv(42, from = "decimal", to = "degrees", NS = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_xml.R
\name{edit_apsim_xml}
\alias{edit_apsim_xml}
\title{Edit an APSIM (Classic) Simulation auxiliary xml file}
\usage{
edit_apsim_xml(
  file,
  src.dir = ".",
  wrt.dir = NULL,
  parm.path = NULL,
  value = NULL,
  overwrite = FALSE,
  edit.tag = "-edited",
  verbose = TRUE
)
}
\arguments{
\item{file}{file ending in .xml to be edited}

\item{src.dir}{directory containing the .xml file to be edited; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{parm.path}{parameter path to be edited (see example)}

\item{value}{new values for the parameter to be edited}

\item{overwrite}{logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise}

\item{edit.tag}{if the file is edited a different tag from the default \sQuote{-edited} can be used.}

\item{verbose}{whether to print information about successful edit}
}
\value{
(when verbose=TRUE) complete file path to edited .xml file is returned as a character string.
As a side effect this function creates a new XML file.
}
\description{
This function allows editing of an APSIM (Classic) simulation xml file.
}
\details{
The variables specified by \code{parm} within the .apsim file specified by \code{file} 
in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
is a list that has the same number of elements as the length of the vector \code{parm}.  The current
.xml file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
\sQuote{file} \emph{-edited.xml} will be created.  If (verbose = TRUE) then the name
 of the written file is returned. The function is similar to the edit_sim_file functin in the \sQuote{apsimr}
 package, but with the difference that here the xml2 package is used instead.
}
\note{
This function cannot check whether replacement is of the correct length. Also, there is
an inspect equivalent. It is more flexible than \sQuote{edit_apsim} and (perhaps) similar to 
\sQuote{apsimr::edit_sim_file}.
}
\examples{
\donttest{
## This example changes the RUE values

extd.dir <- system.file("extdata", package = "apsimx")

values <- paste(rep(1.7, 12), collapse = " ")

## Writing to a temp directory, but replace as needed
tmp.dir <- tempdir()

edit_apsim_xml("Maize75.xml", 
               src.dir = extd.dir,
               wrt.dir = tmp.dir,
               parm.path = ".//Model/rue",
               value = values)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx_soil_profile.R
\name{carbon_stocks}
\alias{carbon_stocks}
\title{Calculate soil carbon stocks}
\usage{
carbon_stocks(
  x,
  depth = NULL,
  area = c("m2", "ha"),
  method = c("linear", "constant"),
  value = c("numeric", "carbon.stocks"),
  ...
)
}
\arguments{
\item{x}{object of class \sQuote{soil_profile}}

\item{depth}{soil depth (in meters). If missing then the whole soil profile is used.}

\item{area}{either \sQuote{m2} meter squared or \sQuote{ha}.}

\item{method}{interpolation method. Either \sQuote{linear} or \sQuote{constant}.}

\item{value}{whether to return a \sQuote{numeric} or an object of class \sQuote{carbon.stocks}.}

\item{...}{additional arguments passed to internal functions (none used at the moment).}
}
\value{
returns a value with attribute \sQuote{units} and \sQuote{depth}
}
\description{
Calculation of carbon stocks based on an object of class \sQuote{soil_profile}
}
\details{
Function to calculate carbon stocks. The output units depend on the choice of area.
If \sQuote{m2} is used, then the output units will be \sQuote{kg/m2}. If the \sQuote{area}
is \sQuote{ha}, then the output units will be \sQuote{Mg/ha}.

Note that the bulk density (which is needed in the calculation) is
available as part of the \sQuote{soil_profile} object.
}
\examples{
\dontrun{
sp <- apsimx_soil_profile()
carbon_stocks(sp)
carbon_stocks(sp, depth = 0.1)
carbon_stocks(sp, depth = 0.2)
carbon_stocks(sp, depth = 0.3)
carbon_stocks(sp, depth = 0.4)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{read_apsim_met}
\alias{read_apsim_met}
\title{Read in an APSIM met file}
\usage{
read_apsim_met(file, src.dir = ".", verbose = TRUE)
}
\arguments{
\item{file}{path to met file}

\item{src.dir}{optional source directory}

\item{verbose}{whether to suppress all messages and warnings}
}
\value{
an object of class \sQuote{met} with attributes
}
\description{
Read into R a met file and return an object of class \sQuote{met}
}
\details{
Read a met file into R

This function uses S3 classes and stores the additional information as attributes \cr
I use a more strict format than \acronym{APSIM} and reading and writing will not \cr
preserve all the details. For example, at this moment comments are lost through \cr
the process of read and write unless they are added back in manually. \cr
Also, empty lines are ignored so these will be lost as well in the read and write process.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
ames.met <- read_apsim_met("Ames.met", src.dir = extd.dir)
ames.met
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{apsimx}
\alias{apsimx}
\title{Run an APSIM-X simulation}
\usage{
apsimx(
  file = "",
  src.dir = ".",
  silent = FALSE,
  intern = FALSE,
  value = "report",
  cleanup = FALSE,
  simplify = TRUE,
  xargs
)
}
\arguments{
\item{file}{file name to be run (the extension .apsimx is optional)}

\item{src.dir}{directory containing the .apsimx file to be run (defaults to the current directory)}

\item{silent}{whether to print messages from apsim simulation. This is passed to argument \sQuote{ignore.stdout} for
function \code{\link{system}}.}

\item{intern}{This is passed to argument \sQuote{intern} for function \code{\link{system}}.}

\item{value}{how much output to return: \cr
option \sQuote{report} returns only the \sQuote{main} report component; \cr
option \sQuote{all} returns all components of the simulation; \cr
option \sQuote{none} does not create a data.frame but it generates the databases; \cr
option \sQuote{user-defined} should be the name of a specific table}

\item{cleanup}{logical. Whether to delete the .db file generated by APSIM-X. Default is FALSE}

\item{simplify}{whether to return a single data frame when multiple reports are present. If FALSE it will return a list.}

\item{xargs}{extra arguments to be passed to the APSIM-X run. Use function xargs_apsimx.}
}
\value{
a data frame with the \sQuote{Report} from the APSIM-X simulation. The return value depends on the argument \sQuote{value} above.
}
\description{
Run apsimx from R. It uses \code{\link{system}} (unix) or \code{\link{shell}} (windows) and it attempts to be platform independent.
}
\details{
Run an APSIM-X Simulation

A valid apsimx file can be run from within R. The main goal is to make running APSIM-X
simple, especially for large scale simulations or parameter optimization
}
\examples{
\donttest{
## See function 'apsimx_example' and vignette 'apsimx'
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ssurgo_soil_profile.R
\name{get_ssurgo_soil_profile}
\alias{get_ssurgo_soil_profile}
\title{Retrieve soil profile data and convert it to an object of class \sQuote{soil_profile}}
\usage{
get_ssurgo_soil_profile(
  lonlat,
  shift = -1,
  nmapunit = 1,
  nsoil = 1,
  xout = NULL,
  soil.bottom = 200,
  method = c("constant", "linear"),
  nlayers = 10,
  check = TRUE,
  fix = FALSE,
  verbose = FALSE,
  xargs = NULL
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(-93, 42))}

\item{shift}{simple mechanism for creating an area of interest by displacing the point indicated in 
lonlat by some amount of distance (e.g. 300 - in meters)}

\item{nmapunit}{number of mapunits to select (see \code{\link{ssurgo2sp}})}

\item{nsoil}{number of soils to select (see \code{\link{ssurgo2sp}}). If the 
number of soils is negative or NA it will fetch all the soils in the mapunit}

\item{xout}{see \code{\link{ssurgo2sp}}}

\item{soil.bottom}{see \code{\link{ssurgo2sp}}}

\item{method}{interpolation method see \code{\link{ssurgo2sp}}}

\item{nlayers}{number for layer for the new soil profile}

\item{check}{whether to check for reasonable values using \code{\link{check_apsimx_soil_profile}}.
TRUE by default. If \sQuote{fix} is TRUE, it will be applied only after the fix attempt.}

\item{fix}{whether to fix compatibility between saturation and bulk density (default is FALSE).}

\item{verbose}{default FALSE. Whether to print messages.}

\item{xargs}{additional arguments passed to \code{\link{apsimx_soil_profile}} function.}
}
\value{
this function will always return a list. Each element of the list will
be an object of class \sQuote{soil_profile}
}
\description{
Generate a synthetic soil profile based on the information in SSURGO database
}
\details{
Data source is USDA-NRCS Soil Data Access. See package soilDB for more details
}
\examples{
\dontrun{
require(soilDB)
require(sp)
require(sf)
require(spData)
require(ggplot2)
## Soil inforation for a single point
sp <- get_ssurgo_soil_profile(lonlat = c(-93, 42))
## The initial attempt throws warnings, so better to use 'fix'
sp <- get_ssurgo_soil_profile(lonlat = c(-93, 42), fix = TRUE)
plot(sp[[1]])
plot(sp[[1]], property = "water")
## Add initial water
iwat <- initialwater_parms(Thickness = sp[[1]]$soil$Thickness,
                           InitialValues = sp[[1]]$soil$DUL * 0.8)
sp[[1]]$initialwater <- iwat
plot(sp[[1]], property = "initialwater")
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_isric_soil_profile.R
\name{get_isric_soil_profile}
\alias{get_isric_soil_profile}
\title{Generate a synthetic APSIM soil profile from the ISRIC soil database}
\usage{
get_isric_soil_profile(
  lonlat,
  statistic = c("mean", "Q0.5"),
  soil.profile,
  find.location.name = TRUE,
  fix = FALSE,
  verbose = TRUE,
  check = TRUE,
  physical = c("default", "SR"),
  xargs = NULL
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(-93, 42)).}

\item{statistic}{default is the mean}

\item{soil.profile}{a soil profile to fill in in case the default one is not appropriate}

\item{find.location.name}{default is TRUE. Use either maps package or photon API to find Country/State.
If you are running this function many times it might be better to set this to FALSE.}

\item{fix}{whether to fix compatibility between saturation and bulk density (default is FALSE).}

\item{verbose}{argument passed to the fix function.}

\item{check}{whether to check the soil profile (default is TRUE)}

\item{physical}{whether soil physical properties are obtained from the data base or through \sQuote{SR}, Saxton and Rawls pedotransfer functions.}

\item{xargs}{additional arguments passed to \code{\link{apsimx_soil_profile}} or \sQuote{apsimx:::approx_soil_variable} function. At the moment these are:
\sQuote{soil.bottom}, \sQuote{crops}, and \sQuote{nlayers} for the first function and \sQuote{method} for the second function.}
}
\value{
it generates an object of class \sQuote{soil_profile}.
}
\description{
Retrieves soil data from the ISRIC global database and converts it to an APSIM soil_profile object
}
\details{
Source: https://isric.org/ \cr
Details: https://docs.isric.org/globaldata/soilgrids/SoilGrids_faqs.html \cr

Pedotransfer functions: Saxton and Rawls, 2006. Soil Water Characteristic Estimates by Texture and Organic Matter for Hydrologic Solutions.
Soil Sci. Soc. Am. J. 70:1569–1578. \cr

TODO: need to look into how this is done in APSIM NG
https://github.com/APSIMInitiative/ApsimX/pull/3994/files \cr

NOTE: Eric Zurcher provided help by sending me an R file originally written by
Andrew Moore. It provides a bit of context for how some of the decisions
were made for constructing the synthetic soil profiles in APSIM. (email from june 3 2021).

Variable which are directly retrieved and a simple unit conversion is performed: \cr
* Bulk density - bdod \cr
* Carbon - soc \cr
* Clay - clay \cr
* Sand - sand \cr
* PH - phh2o \cr
* Nitrogen - nitrogen \cr
Variables which are optionally estimated using pedotransfer functions: \cr
LL15, DUL, SAT, KS, AirDry \cr
TO-DO: \cr
What do I do with nitrogen? \cr
Can I use CEC? \cr
How can I have a guess at FBiom and Finert? \cr
FBiom does not depend on any soil property at the moment, should it? \cr
}
\examples{
\dontrun{
## Only run this if rest.isric.org is working
rest.isric.on <- suppressWarnings(try(readLines("http://rest.isric.org", 
n = 1, warn = FALSE), silent = TRUE))

## Get soil profile properties for a single point
if(!inherits(rest.isric.on, "try-error")){
  sp1 <- get_isric_soil_profile(lonlat = c(-93, 42), fix = TRUE, verbose = FALSE)
  ## Visualize
  plot(sp1)
  plot(sp1, property = "water")
 }
}
}
\seealso{
\code{\link{apsimx_soil_profile}}, \code{\link{edit_apsim_replace_soil_profile}}, \code{\link{edit_apsimx_replace_soil_profile}}.
}
\author{
Fernando E. Miguez, Eric Zurcher (CSIRO) and Andrew Moore (CSIRO)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_worldmodeler_soil_profile.R
\name{get_worldmodeler_soil_profile}
\alias{get_worldmodeler_soil_profile}
\title{Obtain a synthetic APSIM soil profile from the World Modeler database}
\usage{
get_worldmodeler_soil_profile(
  lonlat,
  soil.name,
  wrt.dir,
  filename,
  verbose = FALSE
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(-93, 42)) or matrix.}

\item{soil.name}{optional soil name}

\item{wrt.dir}{optional directory where to save a file with \sQuote{soils} extension.
If missing it will be written to a temporary directory.}

\item{filename}{optional name to be used when saving the file}

\item{verbose}{verbose argument passed to \sQuote{read_apsim_soils}}
}
\value{
it returns a list with objects of class \sQuote{soil_profile}. If
\sQuote{filename} is specified it also creates a file with extension \sQuote{soils}, 
which can be read using function \code{\link{read_apsim_soils}}.
}
\description{
Retrieves soil data from the World Modeler global database and (optionally) saves it to a soils file
}
\examples{
\dontrun{
## Get soil profile properties for a single point
  sp1 <- get_worldmodeler_soil_profile(lonlat = c(-93, 42))
  
  if(inherits(sp1[[1]], 'soil_profile')){
    plot(sp1[[1]], property = "Carbon")
  }
}
}
\author{
Brian Collins (University of Southern Queensland) and Fernando Miguez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{read_apsimx_all}
\alias{read_apsimx_all}
\title{Read all APSIM-X generated .db files in a directory}
\usage{
read_apsimx_all(src.dir = ".", value = "report")
}
\arguments{
\item{src.dir}{source directory where files are located}

\item{value}{either \sQuote{report} or \sQuote{all} (only \sQuote{report} implemented at the moment)}
}
\value{
it returns a data frame or a list if \sQuote{value} equals \sQuote{all}.
}
\description{
Like \code{\link{read_apsimx}}, but it reads all .db files in a directory.
}
\details{
Read all APSIM-X generated .db files in a directory
}
\note{
Warning: very simple function at the moment, not optimized for memory or speed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{print.met}
\alias{print.met}
\title{Printer-friendly version of a metfile}
\usage{
\method{print}{met}(x, ...)
}
\arguments{
\item{x}{an R object of class \sQuote{met}}

\item{...}{additional printing arguments}
}
\value{
It prints to console. Not used to return an R object.
}
\description{
Print a met file in a friendly way
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssurgo2sp.R
\name{ssurgo2sp}
\alias{ssurgo2sp}
\title{Take in \acronym{SSURGO} csv files and create a soil profile}
\usage{
ssurgo2sp(
  mapunit = NULL,
  component = NULL,
  chorizon = NULL,
  mapunit.shp = NULL,
  nmapunit = 1,
  nsoil = 1,
  xout = NULL,
  soil.bottom = 200,
  method = c("constant", "linear"),
  nlayers = 10,
  carbon.to.om = 1.724,
  pedotransfer.method = c("Saxton-Rawls"),
  verbose = FALSE
)
}
\arguments{
\item{mapunit}{mapunit \acronym{SSURGO} file}

\item{component}{component \acronym{SSURGO} file}

\item{chorizon}{chorizon \acronym{SSURGO} file}

\item{mapunit.shp}{mapunit shapefile for creating metadata}

\item{nmapunit}{number of mapunits to select}

\item{nsoil}{number of soil components (within a mapunit) to consider}

\item{xout}{vector for interpolation and extrapolation}

\item{soil.bottom}{bottom of the soil profile}

\item{method}{method used for interpolation (see \code{\link{approx}})}

\item{nlayers}{number of soil layers to generate}

\item{carbon.to.om}{conversion factor from carbon to organic matter (default is 1.724). This is known as the \sQuote{von Bemmelen factor}.}

\item{pedotransfer.method}{\sQuote{Saxton-Rawls}. Only option for now.}

\item{verbose}{whether to print details of the process}
}
\value{
a list with soil profile matrices with length equal to nsoil
}
\description{
Utility function to convert \acronym{SSURGO} data to soil profile
}
\details{
Some of the conversions use pedotrasnfer equations from Saxton and Rawls.
 Soil Water Characteristic Estimates by Texture and Organic Matter for Hydrologic Solutions.
 Soil Sci. Soc. Am. J. 70:1569–1578 (2006).

Download the data from \acronym{SSURGO} using the \sQuote{FedData} package \cr
This will generate csv files \sQuote{chorizon}, \sQuote{component} and \sQuote{mapunit}, \cr
but also many other files which are not needed for creating a soil profile.
}
\examples{
\donttest{
require(ggplot2)
require(sf)
extd.dir <- system.file("extdata", package = "apsimx")

chorizon <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_chorizon.csv"))
component <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_component.csv"))
mapunit <- read.csv(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_mapunit.csv"))
mapunit.shp <- st_read(paste0(extd.dir,"/ISUAG/SSURGO/ISUAG_SSURGO_Mapunits.shp"), quiet = TRUE)

## Using default 'constant' method
sp.c <- ssurgo2sp(mapunit = mapunit, 
                 component = component, 
                 chorizon = chorizon, 
                 mapunit.shp = mapunit.shp)

sp.c <- sp.c[[1]]

ggplot(data = sp.c, aes(y = -Depth, x = Carbon)) + 
geom_point() + 
  geom_path() + 
  ylab("Soil Depth (cm)") + xlab("Organic Matter (percent)") +
  ggtitle("method = constant")
  
  
## Using 'linear' method
sp.l <- ssurgo2sp(mapunit = mapunit, 
                 component = component, 
                 chorizon = chorizon, 
                 mapunit.shp = mapunit.shp,
                 method = "linear")
                 
sp.l <- sp.l[[1]]
                 
ggplot(data = sp.l, aes(y = -Depth, x = Carbon)) + 
geom_point() + 
  geom_path() + 
  ylab("Soil Depth (cm)") + xlab("Organic Matter (percent)") +
  ggtitle("Method linear")
}

\dontrun{
## Method using get_ssurgo_tables

require(soilDB)
require(sp)
require(sf)
require(spData)
## retrieve data from lon -93, lat = 42
stbls <- get_ssurgo_tables(lonlat = c(-93, 42))

sp2.c <- ssurgo2sp(mapunit = stbls$mapunit, 
                 component = stbls$component, 
                 chorizon = stbls$chorizon, 
                 mapunit.shp = stbls$mapunit.shp)  
names(sp2.c)

metadata <- attributes(sp2.c[[1]])
metadata$names <- NULL; metadata$class <- NULL; metadata$row.names <- NULL

## Convert to an APSIM soil profile
asp2.c <- apsimx_soil_profile(nlayers = 10,
                              Thickness = sp2.c[[1]]$Thickness * 10,
                              BD = sp2.c[[1]]$BD,
                              AirDry = sp2.c[[1]]$AirDry,
                              LL15 = sp2.c[[1]]$LL15,
                              DUL = sp2.c[[1]]$DUL,
                              SAT = sp2.c[[1]]$SAT,
                              KS = sp2.c[[1]]$KS,
                              Carbon = sp2.c[[1]]$Carbon,
                              PH = sp2.c[[1]]$PH,
                              ParticleSizeClay = sp2.c[[1]]$ParticleSizeClay,
                              ParticleSizeSilt = sp2.c[[1]]$ParticleSizeSilt,
                              ParticleSizeSand = sp2.c[[1]]$ParticleSizeSand,
                              metadata = metadata)
                              
plot(asp2.c)
plot(asp2.c, property = "water")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ssurgo_tables.R
\name{get_ssurgo_tables}
\alias{get_ssurgo_tables}
\title{Retrieve soil profile data and return a (list) with data frames (tables)}
\usage{
get_ssurgo_tables(lonlat, shift = -1, aoi, verbose = FALSE, ...)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(-93, 42))}

\item{shift}{simple mechanism for creating an area of interest by displacing the point indicated in 
lonlat by some amount of distance (e.g. 300 - in meters)}

\item{aoi}{area of interest, if supplied the lonlat and shift arguments will be ignored. Should be
of class \sQuote{sp::SpatialPolygons} or \sQuote{sf}.}

\item{verbose}{whether to print messages and warnings to the console default FALSE}

\item{...}{additional arguments to be passed to \sQuote{soilDB} functions.}
}
\value{
a list with elements: mapunit, component, chorizon and mapunit.shp
}
\description{
This function does partially what get_ssurgo_soil_profile does, but it 
returns a list with tables for mapunit, component, chorizon and mapunit.shp (object of class sf)
}
\details{
Data source is USDA-NRCS Soil Data Access. See package soilDB for more details

* If a point is requested then an object of class \sQuote{sf} is returned (for mapunit.shp) 
with the MUKEY and AREASYMBOL with GEOMETRY type: POINT. 

* If a the request is for a spatial polygon, then an object of class \sQuote{sf} is returned
with gid, mukey and area_ac with GEOMETRY type: POLYGON.
}
\examples{
\dontrun{
require(soilDB)
require(sp)
require(sf)
require(spData)
## retrieve data from lon -93, lat = 42
stbls <- get_ssurgo_tables(lonlat = c(-93, 42)) 

stbls2 <- get_ssurgo_tables(lonlat = c(-93, 42), shift = 200)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsim_xml.R
\name{inspect_apsim_xml}
\alias{inspect_apsim_xml}
\title{Inspect an APSIM Classic auxiliary (XML) file}
\usage{
inspect_apsim_xml(
  file = "",
  src.dir = ".",
  parm,
  verbose = TRUE,
  print.path = TRUE
)
}
\arguments{
\item{file}{file ending in .xml to be inspected.}

\item{src.dir}{directory containing the .xml file to be inspected; defaults to the current working directory}

\item{parm}{parameter to inspect.}

\item{verbose}{Whether to print to standard output}

\item{print.path}{Whether to print the parameter path}
}
\value{
it returns an absolute parameter path(s)
}
\description{
inspect an auxiliary XML apsim file.
}
\note{
the behavior has changed from previous verions (earlier than 1.977). Before,
if more than match was found it would return an error. Now it returns a list with all
possible matches. This can be useful when trying to find a parameter.
}
\examples{
 
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")

inspect_apsim_xml("Maize75.xml", src.dir = extd.dir,
                  parm = "leaf_no_rate_change")
                  
pp <- inspect_apsim_xml("Maize75.xml", src.dir = extd.dir,
                        parm = "leaf_no_rate_change",
                        verbose = FALSE, 
                        print.path = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{apsim_example}
\alias{apsim_example}
\title{Access Example APSIM Simulations}
\usage{
apsim_example(example = "Millet", silent = FALSE, tmp.dir = NULL)
}
\arguments{
\item{example}{run an example from built-in APSIM. Options are all of the ones included with the APSIM distribution, except \sQuote{Graph}.}

\item{silent}{whether to print standard output from the APSIM execution}

\item{tmp.dir}{temporary directory where to write files}
}
\value{
This function returns a data frame with APSIM output
}
\description{
simple function to run some of the built-in APSIM examples
}
\details{
This function creates a temporary copy of the example file distributed with APSIM to avoid writing a .out file 
         to the directory where the \sQuote{Examples} are located. It is not a good practice and there is no guarantee that 
         the user has read/write permissions in that directory.
}
\note{
This function creates a new column \sQuote{Date} which is in the R \sQuote{Date} format which is convenient for graphics.
}
\examples{
\dontrun{
## Only run these if you have APSIM 'Classic' installed (Windows only)
millet <- apsim_example("Millet")
potato <- apsim_example("Potato")
sugar <- apsim_example("Sugar")
## The 'Date' column is created by this function, based on apsim output.
require(ggplot2)
ggplot(data = millet , aes(x = Date, y = millet_biomass)) + 
  geom_line()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsimx_batch.R
\name{edit_apsimx_batch}
\alias{edit_apsimx_batch}
\title{Edit an APSIM-X (JSON) Simulation in Batch mode}
\usage{
edit_apsimx_batch(
  file,
  src.dir = ".",
  wrt.dir = NULL,
  parms = NULL,
  silent = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{file}{file ending in .apsimx to be edited (JSON)}

\item{src.dir}{directory containing the .apsimx file to be edited; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{parms}{parameter to be edited in the for of \sQuote{key = value}}

\item{silent}{controls the output of running APSIM at the command line}

\item{verbose}{whether to print information about successful edit}
}
\value{
(when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
As a side effect this function creates a new (JSON) .apsimx file.
}
\description{
This function allows editing of an APSIM-X (JSON) simulation file in batch mode.
}
\details{
from hol430

This allows the user to specify an .apsimx file and a config file when running Models.exe. The .apsimx file will not be run but instead, the changes listed in the config file will be applied to the .apsimx file, which will then be written to disk under the same filename.

The config file should contain lines of the form \sQuote{path = value}

e.g.

[Clock].StartDate = 2019-1-20
.Simulations.Sim1.Name = SimulationVariant35
.Simulations.Sim2.Enabled = false
.Simulations.Sim1.Paddock.Soil.Thickness[1] = 50
Notes:
  
  Command line arguments should look like:
 Models.exe file.apsimx /Edit /path/to/config/file.conf

Relative paths will be resolved to the first match. ie
[Clock].StartDate will match the first clock found in the file.

Dates can be specified as yyyy-mm-dd or mm/dd/yyyy.

Strings should not be quoted

Array indices will be interpted as 1-indexed (mad face). So the first
element in the array should have index 1 in the config file.

The file will be upgraded to the latest file version as part of this
process.
}
\examples{
\donttest{
## This example will read one of the examples distributed with APSIM-X
## but write to a temporary directory

tmp.dir <- tempdir()

## Edit InitialResidueMass
extd.dir <- system.file("extdata", package = "apsimx")
parms <- list(`.Simulations.Simulation.Field.SurfaceOrganicMatter.InitialResidueMass` = 600)
edit_apsimx_batch("Wheat.apsimx", src.dir = extd.dir, wrt.dir = tmp.dir, parms = parms)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsimx_replace_soil_profile.R
\name{edit_apsimx_replace_soil_profile}
\alias{edit_apsimx_replace_soil_profile}
\title{Edit APSIM-X file with a replaced soil profile}
\usage{
edit_apsimx_replace_soil_profile(
  file = "",
  src.dir = ".",
  wrt.dir = NULL,
  soil.profile = NULL,
  edit.tag = "-edited",
  overwrite = FALSE,
  verbose = TRUE,
  root = NULL
)
}
\arguments{
\item{file}{name of the .apsimx file to be edited}

\item{src.dir}{source directory}

\item{wrt.dir}{writing directory}

\item{soil.profile}{a soil profile object with class \sQuote{soil_profile}}

\item{edit.tag}{default edit tag \sQuote{-edited}}

\item{overwrite}{default FALSE}

\item{verbose}{default TRUE and it will print messages to console}

\item{root}{supply the node position in the case of multiple simulations such as factorials.}
}
\value{
writes a file to disk with the supplied soil profile
}
\description{
Edits an APSIM-X simulation by replacing the soil profile
}
\details{
This function is designed to batch replace the whole soil in an APSIM simulation file.
}
\note{
There is no such thing as a default soil, carefully build the profile for each simulation.
}
\examples{
\donttest{
sp <- apsimx_soil_profile()
extd.dir <- system.file("extdata", package = "apsimx")

## I write to a temp directory but replace as needed
tmp.dir <- tempdir()

edit_apsimx_replace_soil_profile("Maize.apsimx", soil.profile = sp, 
                                  src.dir = extd.dir, wrt.dir = tmp.dir)
inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir,
                 node = "Soil")
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_daymet_apsim_met.R
\name{get_daymet_apsim_met}
\alias{get_daymet_apsim_met}
\title{Get DAYMET data for an APSIM met file}
\source{
The data is retrieved using the \CRANpkg{daymetr} package. For the original
source see: https://daymet.ornl.gov/
}
\usage{
get_daymet_apsim_met(lonlat, years, wrt.dir = ".", filename, silent = FALSE)
}
\arguments{
\item{lonlat}{Longitude and latitude vector}

\item{years}{a numeric vector of years to extract (c(start, end)). For example, f you need 2012 through 2015, use c(2012, 2015).}

\item{wrt.dir}{write directory (default is the current directory)}

\item{filename}{file name for writing out to disk}

\item{silent}{argument passed to \code{\link[daymetr]{download_daymet}}}
}
\value{
It returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Uses \code{\link[daymetr]{download_daymet}} from the \CRANpkg{daymetr} package to download data to create an APSIM met file.
}
\details{
This function requires the \CRANpkg{daymetr} package. This function should replace 
the \code{\link{get_daymet_apsim_met}} function.

If the filename is not provided it will not write the file to disk, 
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk. The variable \sQuote{srad} as downloaded from
daymet is average solar radiation, so it is converted to total. 
Daily total radiation (MJ/m2/day) can be calculated as 
follows: ((srad (W/m2) * dayl (s/day)) / 1,000,000) \cr
Vapor Pressure Deficit (vp) should be in hecto Pascals
}
\examples{
\dontrun{
require(daymetr)
## I write to a temp directory but replace as needed
dmet12 <- get_daymet_apsim_met(lonlat = c(-93,42), years = 2012)
summary(dmet12)
## Check for reasonable ranges 
check_apsim_met(dmet12)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_worldmodeler_soil_profile.R
\name{get_worldmodeler_apsim_met}
\alias{get_worldmodeler_apsim_met}
\title{Obtain a weather APSIM met from the World Modeler database}
\usage{
get_worldmodeler_apsim_met(
  lonlat,
  dates,
  wrt.dir,
  filenames,
  check = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(-93, 42)) or matrix.}

\item{dates}{date range (see example for format)}

\item{wrt.dir}{optional directory where to save a file with \sQuote{met} extension.
If missing it will be written to a temporary directory.}

\item{filenames}{optional name(s) to be used when saving the file. It should be equal to the
number of rows of the input matrix.}

\item{check}{whether to check the met file}

\item{verbose}{argument passed to read_apsim_met}
}
\value{
it creates a list with objects of class \sQuote{met}. 
If it fails, then the objects will be of class \sQuote{try-error}.
}
\description{
Retrieves met data from the World Modeler global database and (optionally) saves it to a file
}
\examples{
\dontrun{
## Get soil profile properties for a single point
  am1 <- get_worldmodeler_apsim_met(lonlat = c(-93, 42), 
                                    dates = c("2010-01-01", "2013-12-31"))
  if(inherits(am1, 'met')){
     plot(am1[[1]], met.var = "rain", cumulative = TRUE)
  }
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{auto_detect_apsimx_examples}
\alias{auto_detect_apsimx_examples}
\title{Auto detect where apsimx examples are located}
\usage{
auto_detect_apsimx_examples()
}
\value{
will create a directory (character string) pointing to APSIM-X distributed examples
}
\description{
simple function to detect where APSIM-X examples are located
}
\details{
Auto detect where apsimx examples are located
}
\examples{
\dontrun{
ex.dir <- auto_detect_apsimx_examples()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsim_xml.R
\name{inspect_apsim}
\alias{inspect_apsim}
\title{Inspect an .apsim (XML) file}
\usage{
inspect_apsim(
  file = "",
  src.dir = ".",
  node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "Crop", "Manager",
    "Outputfile", "Other"),
  soil.child = c("Metadata", "Water", "OrganicMatter", "Nitrogen", "Analysis",
    "InitialWater", "Sample", "SWIM"),
  parm = NULL,
  digits = 3,
  print.path = FALSE,
  root
)
}
\arguments{
\item{file}{file ending in .apsim (Classic) to be inspected (XML)}

\item{src.dir}{directory containing the .apsim file to be inspected; defaults to the current working directory}

\item{node}{either \sQuote{Weather}, \sQuote{Soil}, \sQuote{SurfaceOrganicMatter}, 
\sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other}}

\item{soil.child}{specific soil component to be inspected}

\item{parm}{parameter to inspect when node = \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other}}

\item{digits}{number of decimals to print (default 3)}

\item{print.path}{whether to print the parameter path (default = FALSE)}

\item{root}{root node label. In simulation structures such as factorials there will be multiple possible nodes. This can be specified by supplying an appropriate character.}
}
\value{
It returns the parameter path (when print.path equals TRUE) and table with inspected parameters and values
}
\description{
inspect an XML apsim file. It does not replace the GUI, but it can save time by quickly checking parameters and values.
}
\details{
This is simply a script that prints the relevant parameters which are likely to need editing. It does not print all information from an .apsim file.
         For \sQuote{Crop}, \sQuote{Manager} and \sQuote{Other}, \sQuote{parm} should be indicated with a first element to look for and a second with the relative position in case there are
         multiple results.
}
\note{
When multiple folders are present as it is the case when there are factorials. Inspect will find
      the instance in the first folder unless 'root' is supplied. By providing the name of the folder to root
      (or a regular expression), the appropriate node can be selected. In this case the printed path will
      be absolute instead of relative.
}
\examples{
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
## Testing using 'Millet'
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Clock")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Weather") 
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Metadata")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "OrganicMatter")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Analysis")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "InitialWater")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", soil.child = "Sample")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "SurfaceOrganicMatter")
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",NA)) 
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Crop", parm = list("sow",7))

## when soil.child = "Water" there are potentially many crops to chose from 
## This selects LL, KL and XF for Barley
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
              soil.child = "Water", parm = "Barley")
## This selects LL for all the crops
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
              soil.child = "Water", parm = "LL")
## To print the parm.path the selection needs to be unique
## but still there will be multiple soil layers
## 'parm' can be a list or a character vector of length equal to two
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Soil", 
              soil.child = "Water", parm = list("Barley", "LL"),
              print.path = TRUE)

## Inspect outputfile
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Outputfile",
              parm = "filename")
 
inspect_apsim("Millet.apsim", src.dir = extd.dir, node = "Outputfile",
              parm = "variables")

## Testing with maize-soybean-rotation.apsim
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Clock")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Weather")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil",
              soil.child = "Metadata")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
               soil.child = "OrganicMatter")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
               soil.child = "Analysis")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
               soil.child = "InitialWater")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Soil", 
               soil.child = "Sample")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
               node = "SurfaceOrganicMatter")
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, node = "Crop")
## This has many options and a complex structure
## It is possible to select unique managements, but not non-unique ones
## The first element in parm can be a regular expression
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
               node = "Manager", parm = list("rotat",NA))
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
               node = "Manager", 
               parm = list("sow on a fixed date - maize",NA))
## Select an individual row by position
inspect_apsim("maize-soybean-rotation.apsim", src.dir = extd.dir, 
              node = "Manager", 
              parm = list("sow on a fixed date - maize",7))
              
## Illustrating the 'print.path' feature.
inspect_apsim("Millet.apsim", src.dir = extd.dir, 
               node = "Soil", soil.child = "Water", 
               parm = "DUL", print.path = TRUE)
## But the path can also be returned as a string
## Which is useful for later editing
pp <-  inspect_apsim("Millet.apsim", src.dir = extd.dir, 
               node = "Soil", soil.child = "Water", 
               parm = "DUL", print.path = TRUE)
               
## Inspecting a factorial
## (or simply a simulation with multiple folders)
## No cover
inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
               root = "IA-CC_Canisteo_No-Cover")
               
## Cover
inspect_apsim("maize-factorial.apsim", src.dir = extd.dir, 
               root = "IA-CC_Canisteo_Cover")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsimx_json.R
\name{edit_apsimx}
\alias{edit_apsimx}
\title{Edit an APSIM-X (JSON) Simulation}
\usage{
edit_apsimx(
  file,
  src.dir = ".",
  wrt.dir = NULL,
  node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop",
    "Manager", "Report", "Operations", "Other"),
  soil.child = c("Metadata", "Water", "SoilWater", "Organic", "Physical", "Analysis",
    "Chemical", "InitialWater", "Sample", "Solute", "NO3", "NH4", "Urea", "Swim3"),
  manager.child = NULL,
  parm = NULL,
  value = NULL,
  overwrite = FALSE,
  edit.tag = "-edited",
  parm.path = NULL,
  root = NULL,
  verbose = TRUE
)
}
\arguments{
\item{file}{file ending in .apsimx to be edited (JSON)}

\item{src.dir}{directory containing the .apsimx file to be edited; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{node}{either \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}, 
\sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Report}, \sQuote{Operations} or \sQuote{Other}}

\item{soil.child}{specific soil component to be edited}

\item{manager.child}{specific manager component to be edited}

\item{parm}{parameter to be edited. It can be a regular expression.}

\item{value}{new values for the parameter to be edited}

\item{overwrite}{logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise}

\item{edit.tag}{if the file is edited a different tag from the default \sQuote{-edited} can be used.}

\item{parm.path}{path to the attribute to edit when node is \sQuote{Other}}

\item{root}{supply the node position in the case of multiple simulations such as factorials.}

\item{verbose}{whether to print information about successful edit}
}
\value{
(when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
As a side effect this function creates a new (JSON) .apsimx file.
}
\description{
This function allows editing of an APSIM-X (JSON) simulation file.
}
\details{
The variables specified by \code{parm} within the .apsimx file specified by \code{file} 
in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
is a list that has the same number of elements as the length of the vector \code{parm}.  The current
.apsimx file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
\sQuote{file} \emph{-edited.apsimx} will be created.  If (verbose = TRUE) then the name
 of the written file is returned. 
 
 When node equals \sQuote{Report}, the editing allows to add variables, but not to remove them at the moment.

 When node equals Operations, \sQuote{parm} should have a list with two elements. The first should be the line(s) to edit and 
 the second should be the component(s) to edit. Either \sQuote{Date}, \sQuote{Action} or \sQuote{Line}.
 When more than one line is edited, \sQuote{value} should be a character vector of the same length as the number of
 lines to edit. It is possible to remove, say, line 10 by using \sQuote{parm = list(-10, NA)}. It is safer to remove
 lines at the end of \sQuote{Operations}. To remove several use the following \sQuote{parm = list(-c(10:12), NA)}. 
 This assumes that \sQuote{12} is the maximum number of lines present. Trying to remove lines in the middle will have 
 unexpected effects. It is possible to create additional lines, but only by using \sQuote{Date} first. This feature
 has not been tested much so use it carefully.
}
\examples{
\donttest{
## This example will read one of the examples distributed with APSIM-X
## but write to a temporary directory
tmp.dir <- tempdir()

## Edit Bulk density
extd.dir <- system.file("extdata", package = "apsimx")
bds <- c(1.02, 1.03, 1.09, 1.16, 1.18, 1.19, 1.20)
edit_apsimx("Wheat.apsimx", src.dir = extd.dir,
            wrt.dir = tmp.dir,
            node = "Soil",
            soil.child = "Physical", 
            parm = "BD", value = bds,
            verbose = FALSE)
## Inspect file
inspect_apsimx("Wheat-edited.apsimx", src.dir = tmp.dir,
                node = "Soil", soil.child = "Physical")
## To delete the file...
file.remove(file.path(tmp.dir, "Wheat-edited.apsimx"))

## Edit the fertilizer amount in 'Maize.apsimx'
edit_apsimx("Maize.apsimx", src.dir = extd.dir,
             wrt.dir = tmp.dir, node = "Manager",
             manager.child = "SowingFertiliser",
             parm = "Amount", value = 200, verbose = TRUE)
             
## Make sure it worked
inspect_apsimx("Maize-edited.apsimx", src.dir = tmp.dir, 
               node = "Manager",
               parm = list("SowingFertiliser", NA))

## Remove the file
file.remove(file.path(tmp.dir, "Maize-edited.apsimx"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wget_apsimx_json.R
\name{get_apsimx_json}
\alias{get_apsimx_json}
\title{fetches the json file for a specific model from APSIMX github}
\usage{
get_apsimx_json(model = "Wheat", wrt.dir = ".", cleanup = FALSE)
}
\arguments{
\item{model}{a model (e.g. \sQuote{Wheat} or \sQuote{Maize})}

\item{wrt.dir}{directory to save the JSON file (default is the current directory)}

\item{cleanup}{whether to delete the JSON file}
}
\value{
a list read through the jsonlite package
}
\description{
Retrieves the json replacement file for a specific model
}
\details{
Get APSIM-X Model Replacement from github
}
\examples{
\donttest{
tmp.dir <- tempdir()
### try to download without generating an error
wheat <- try(get_apsimx_json(model = "Wheat", wrt.dir = tmp.dir), silent = TRUE)
}

}
\seealso{
\code{\link{insert_replacement_node}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsim_xml.R
\name{view_apsim_xml}
\alias{view_apsim_xml}
\title{View an APSIM Classic auxiliary (XML) file}
\usage{
view_apsim_xml(file, src.dir, viewer = c("json", "react"), ...)
}
\arguments{
\item{file}{file ending in .xml to be viewed.}

\item{src.dir}{directory containing the .xml file to be viewed; defaults to the current working directory}

\item{viewer}{either \dQuote{json} or \dQuote{react}.}

\item{...}{additional arguments passed to either \sQuote{jsonedit} or \sQuote{reactjson}.}
}
\value{
It does not return an object but it produces a tree display of the APSIM file.
}
\description{
view an auxilliary XML apsim file.
}
\details{
view APSIM XML file
}
\examples{
 
\donttest{
extd.dir <- system.file("extdata", package = "apsimx")
view_apsim_xml("Maize75.xml", src.dir = extd.dir)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{check_apsimx}
\alias{check_apsimx}
\title{Partial checking of an apsimx file for possible issues.}
\usage{
check_apsimx(
  file = "",
  src.dir = ".",
  node = c("all", "Clock", "Weather", "Soil"),
  soil.child = c("all", "Physical", "InitialWater", "SoilWater", "Solute", "Organic"),
  check.apsim.met = FALSE,
  root = NULL,
  verbose = TRUE
)
}
\arguments{
\item{file}{file ending in .apsimx to be edited (JSON)}

\item{src.dir}{directory containing the .apsimx file to be checked; defaults to the current working directory}

\item{node}{either \sQuote{all}, \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}}

\item{soil.child}{specific soil component to be checked.}

\item{check.apsim.met}{whether to check the \sQuote{met} file. Default is FALSE.}

\item{root}{supply the node position in the case of multiple simulations such as factorials.}

\item{verbose}{whether to print information}
}
\value{
It does not return an object, but it prints messages useful for diagnosing issues.
}
\description{
Partial checking of an apsimx file for possible issues.
}
\examples{
\donttest{

## Check file distributed with the package
extd.dir <- system.file("extdata", package = "apsimx")

check_apsimx("Wheat.apsimx", src.dir = extd.dir)
## This throws warnings but it should not produce errors
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{solutes_parms}
\alias{solutes_parms}
\title{Helper function to supply additional Solute parameters}
\usage{
solutes_parms(
  Depth = NA,
  Thickness = NA,
  Solutes = NA,
  InitialValues = NA,
  InitialValuesUnits = NA,
  WaterTableConcentration = NA,
  D0 = NA,
  Exco = NA,
  FIP = NA,
  DepthConstant = NA,
  MaxDepthSoluteAccessible = NA,
  RunoffEffectivenessAtMovingSolute = NA,
  MaxEffectiveRunoff = NA
)
}
\arguments{
\item{Depth}{depth for soil layers (see APSIM documentation)}

\item{Thickness}{soil thickness for layers (either enter Depth or Thickness, but not both). 
Thickness will be recycled if more than one Solute is passed.}

\item{Solutes}{Solutes supplied (for now this can be one or more of: \sQuote{NO3}, \sQuote{NH4} or \sQuote{Urea})}

\item{InitialValues}{initial values of solutes}

\item{InitialValuesUnits}{passed to Solutes}

\item{WaterTableConcentration}{passed to Solutes}

\item{D0}{passed to Solutes}

\item{Exco}{passed to Solutes}

\item{FIP}{passed to Solutes}

\item{DepthConstant}{passed to Solutes}

\item{MaxDepthSoluteAccessible}{passed to Solutes}

\item{RunoffEffectivenessAtMovingSolute}{passed to Solutes}

\item{MaxEffectiveRunoff}{passed to Solutes}
}
\description{
Creates a list with specific components for the Solutes module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_data_apsimx.R
\name{extract_values_apsimx}
\alias{extract_values_apsimx}
\title{Extract values from a parameter path}
\usage{
extract_values_apsimx(file, src.dir, parm.path)
}
\arguments{
\item{file}{file name to be run (the extension .apsimx is optional)}

\item{src.dir}{directory containing the .apsimx file to be run (defaults to the current directory)}

\item{parm.path}{parameter path either use inspect_apsimx or see example below}
}
\value{
a vector with extracted parameter values from an APSIM file.
}
\description{
Extract initial values from a parameter path
}
\examples{
\donttest{
## Find examples
extd.dir <- system.file("extdata", package = "apsimx")
## Extract parameter path
pp <- inspect_apsimx("Maize.apsimx", src.dir = extd.dir,
                     node = "Manager", parm = list("Fert", 1))
ppa <- paste0(pp, ".Amount")
## Extract value
extract_values_apsimx("Maize.apsimx", src.dir = extd.dir, parm.path = ppa)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{apsim}
\alias{apsim}
\title{Run an APSIM (7.x) \sQuote{Classic} simulation}
\usage{
apsim(
  file = "",
  src.dir = ".",
  silent = FALSE,
  value = "report",
  cleanup = FALSE,
  simplify = TRUE
)
}
\arguments{
\item{file}{file name to be run (the extension .apsim is optional)}

\item{src.dir}{directory containing the .apsim file to be run (defaults to the current directory)}

\item{silent}{whether to print messages from apsim simulation}

\item{value}{how much output to return: \cr
option \sQuote{report} returns only the \sQuote{main} report component;
option \sQuote{all} returns all components of the simulation; \cr
option \sQuote{none} runs simulation but does not return a data frame; \cr
option \sQuote{user-defined} should be the name of a specific output file.}

\item{cleanup}{logical. Whether to delete the .out and .sum files generated by APSIM. Default is FALSE.}

\item{simplify}{whether to return a single data frame when multiple simulations are present. If FALSE it will return a list.}
}
\value{
This function returns a data frame with APSIM output, but it depends on the argument \sQuote{value} above.
}
\description{
Run apsim from R. It's for Windows only. It uses \sQuote{shell}.
}
\details{
Run an APSIM (7.x) \sQuote{Classic} Simulation

A valid apsim file can be run from within R. The main goal is to make running APSIM-X
simple, especially for large scale simulations or parameter optimization
}
\examples{
\donttest{
## See function 'apsim_example' 
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{apsim_options}
\alias{apsim_options}
\title{Setting some options specific to APSIM (7.x) \sQuote{Classic}}
\usage{
apsim_options(exe.path = NA, examples.path = NA, warn.versions = TRUE)
}
\arguments{
\item{exe.path}{path to apsim executable}

\item{examples.path}{path to apsim examples}

\item{warn.versions}{logical. warning if multiple versions of APSIM are detected.}
}
\value{
It modifies the \sQuote{apsim.options} environment as a side effect.
}
\description{
Set the path to the APSIM executable, examples and warning suppression.
}
\details{
Set apsim options
}
\note{
It is possible that APSIM 7.x \sQuote{Classic} is installed in some alternative location other than the 
      defaults ones. Guessing this can be difficult and then the auto_detect functions might
      fail. Also, if multiple versions of APSIM are installed apsim will choose the newest
      one but it will issue a warning. Suppress the warning by setting warn.versions = FLASE.
}
\examples{
\dontrun{
names(apsim.options)
apsim_options(exe.path = "some-new-path-to-executable")
apsim.options$exe.path
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{read_apsimx}
\alias{read_apsimx}
\title{Read APSIM-X generated .db files}
\usage{
read_apsimx(file = "", src.dir = ".", value = "report", simplify = TRUE)
}
\arguments{
\item{file}{file name}

\item{src.dir}{source directory where file is located}

\item{value}{either \sQuote{report}, \sQuote{all} (list) or user-defined for a specific report}

\item{simplify}{if TRUE will attempt to simplify multiple reports into a single data.frame. 
If FALSE it will return a list.}
}
\value{
normally it returns a data frame, but it depends on the argument \sQuote{value} above
}
\description{
read SQLite databases created by APSIM-X runs. One file at a time.
}
\details{
Read APSIM-X generated .db files
}
\note{
if there is one single report it will return a data.frame. 
If there are multiple reports, it will attempt to merge them into a data frame. 
If not possible it will return a list with names corresponding to the 
table report names. It is also possible to select a specific report from several
available by selecting \sQuote{value = ReportName}, where \sQuote{ReportName} is the name
of the specific report that should be returned.
If you select \sQuote{all} it will return all the components in the data base also as a list.
}
\seealso{
\code{\link{read_apsimx_all}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_classic.R
\name{read_apsim}
\alias{read_apsim}
\title{Read APSIM generated .out files}
\usage{
read_apsim(
  file = "",
  src.dir = ".",
  value = c("report", "all"),
  date.format = "\%d/\%m/\%Y",
  silent = FALSE
)
}
\arguments{
\item{file}{file name}

\item{src.dir}{source directory where file is located}

\item{value}{either \sQuote{report} (data.frame), \sQuote{user-defined} or \sQuote{all} (list)}

\item{date.format}{format for adding \sQuote{Date} column}

\item{silent}{whether to issue warnings or suppress them}
}
\value{
This function returns a data frame with APSIM output or a list if value equals \sQuote{all}
}
\description{
read \sQuote{output} databases created by APSIM runs (.out and .sim). One file at a time.
}
\details{
Read APSIM generated .out files
}
\examples{
\dontrun{
extd.dir <- system.file("extdata", package = "apsimx")
maize.out <- read_apsim("Maize", src.dir = extd.dir, value = "report")
millet.out <- read_apsim("Millet", src.dir = extd.dir, value = "report")
}

}
\seealso{
\code{\link{read_apsim_all}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsimx.R
\name{apsimx_example}
\alias{apsimx_example}
\title{Access Example APSIM-X Simulations}
\usage{
apsimx_example(example = "Wheat", silent = FALSE)
}
\arguments{
\item{example}{run an example from built-in APSIM-X. Options are all of the ones included with the APSIM-X distribution, except \sQuote{Graph}.}

\item{silent}{whether to print standard output from the APSIM-X execution}
}
\value{
It returns a data frame
}
\description{
simple function to run some of the built-in APSIM-X examples
}
\details{
This function creates a temporary copy of the example file distributed with APSIM-X to avoid writing a .db file 
         to the directory where the \sQuote{Examples} are located. It is not a good practice and there is no guarantee that 
         the user has read/write permissions in that directory.
}
\note{
This function creates a new column \sQuote{Date} which is in the R \sQuote{Date} format which is convenient for graphics.
}
\examples{
\dontrun{
wheat <- apsimx_example("Wheat")
maize <- apsimx_example("Maize")
barley <- apsimx_example("Barley")
## The 'Date' column is created by this function, based on apsim output.
require(ggplot2)
ggplot(data = wheat , aes(x = Date, y = Yield)) + 
  geom_point()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim_replace_soil_profile.R
\name{initialwater_parms}
\alias{initialwater_parms}
\title{Helper function to supply additional Initial Soil Water parameters}
\usage{
initialwater_parms(
  Depth = NA,
  Thickness = NA,
  InitialValues = NA,
  InitialPAWmm = NA,
  PercentFull = NA,
  RelativeTo = NA,
  FilledFromTop = NA,
  DepthWetSoil = NA
)
}
\arguments{
\item{Depth}{depth for soil layers (see APSIM documentation)}

\item{Thickness}{soil thickness for layers (either enter Depth or Thickness, but not both)}

\item{InitialValues}{initial values of soil water}

\item{InitialPAWmm}{Initial Plant Available Water in mm}

\item{PercentFull}{Percent full (0 - 100)}

\item{RelativeTo}{usually LL15}

\item{FilledFromTop}{either true or false}

\item{DepthWetSoil}{depth of wet soil in mm}
}
\description{
Creates a list with specific components for the Initial Soil Water module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_power_apsim_met.R
\name{get_power_apsim_met}
\alias{get_power_apsim_met}
\title{Get NASA-POWER data for an APSIM met file}
\usage{
get_power_apsim_met(lonlat, dates, wrt.dir = ".", filename = NULL)
}
\arguments{
\item{lonlat}{Longitude and latitude vector}

\item{dates}{date ranges}

\item{wrt.dir}{write directory}

\item{filename}{file name for writing out to disk}
}
\value{
returns an object of class \sQuote{met} and writes a file to disk when filename is supplied.
}
\description{
Uses \code{\link[nasapower]{get_power}} from the \CRANpkg{nasapower} package to download data to create an APSIM met file.
}
\details{
This function requires the \CRANpkg{nasapower} package version 4.0.0.

It looks like the earliest year you can request data for is 1984.

If the filename is not provided it will not write the file to disk, 
but it will return an object of class \sQuote{met}. This is useful in case manipulation
is required before writing to disk.
}
\examples{
\dontrun{
require(nasapower)
## This will not write a file to disk
pwr <- get_power_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2012-12-31"))
## Let's insert a missing value
pwr[100, "radn"] <- NA
summary(pwr)
## Check the met file 
check_apsim_met(pwr)
## Impute using linear interpolation
pwr.imptd <- impute_apsim_met(pwr, verbose = TRUE)
summary(pwr.imptd)
check_apsim_met(pwr.imptd)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_slga_soil_profile.R
\name{get_slga_soil}
\alias{get_slga_soil}
\title{Retrieve soil profile data from SLGA (Soils for Australia)}
\usage{
get_slga_soil(lonlat)
}
\arguments{
\item{lonlat}{Longitude and latitude vector (e.g. c(151.8306, -27.4969))}
}
\value{
a data.frame with elements: depth (midpoint in cm), depths (as character in cm), thickness (cm), clay, sand, silt, wv1500, wv0033, bdod, nitrogen, phh2o, cec, soc
}
\description{
This function gets a soil profile for the Australia extent
}
\details{
The data comes from https://esoil.io/TERNLandscapes/Public/Pages/SLGA/index.html
}
\examples{
\dontrun{
## retrieve data from longitude and latitude 151.8305805675806 and -27.496873026858598
## Note: This can take a couple of minutes
slga.soil <- get_slga_soil(lonlat = c(151.8306, -27.4969)) 

}
}
\author{
Chloe (Yunru Lai) and Fernando E. Miguez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optim_apsimx.R
\name{optim_apsimx}
\alias{optim_apsimx}
\title{Optimize parameters in an APSIM Next Generation simulation}
\usage{
optim_apsimx(
  file,
  src.dir = ".",
  parm.paths,
  data,
  type = c("optim", "nloptr", "mcmc", "ucminf", "grid"),
  weights,
  index = "Date",
  parm.vector.index,
  replacement,
  root,
  initial.values,
  grid,
  ...
)
}
\arguments{
\item{file}{file name to be run (the extension .apsimx is optional)}

\item{src.dir}{directory containing the .apsimx file to be run (defaults to the current directory)}

\item{parm.paths}{absolute or relative paths of the coefficients to be optimized.
It is recommended that you use \code{\link{inspect_apsimx}} for this}

\item{data}{data frame with the observed data. By default is assumes there is a 'Date' column for the index.}

\item{type}{Type of optimization. For now, \code{\link[stats]{optim}}, and, if available, \code{\link[nloptr]{nloptr}} or
\sQuote{mcmc} through \code{\link[BayesianTools]{runMCMC}}. Option \sQuote{ucminf} uses the \code{\link[ucminf]{ucminf}} function.
If \sQuote{type} is \sQuote{grid}, then a grid can be passed and no optimization will be performed.}

\item{weights}{Weighting method or values for computing the residual sum of squares.}

\item{index}{Index for filtering APSIM output. Typically, \dQuote{Date}, but it can be c(\dQuote{report}, \dQuote{Date}) for
multiple simulations}

\item{parm.vector.index}{Index to optimize a specific element of a parameter vector.  At the moment it is
possible to only edit one element at a time. This is because there is a conflict when generating multiple
elements in the candidate vector for the same parameter.}

\item{replacement}{TRUE or FALSE for each parameter. Indicating whether it is part of
the \sQuote{replacement} component. Its length should be equal to the length or \sQuote{parm.paths}.}

\item{root}{root argument for \code{\link{edit_apsimx_replacement}}}

\item{initial.values}{(required) supply the initial values of the parameters. (Working on fixing this...).
If the parameters to be optimized correspond to a single value, then a simple numeric vector can be supplied.
If one or more of the parameters represent a vector in APSIM, then the initial values should be passed as a
list. At the moment, it is not possible to check if these are appropriate (correct name and length).}

\item{grid}{grid used when \sQuote{type = grid}. Columns should be parameters and rows different values for
those parameters.}

\item{...}{additional arguments to be passed to the optimization algorithm. See \code{\link[stats]{optim}}}
}
\value{
object of class \sQuote{optim_apsim}, but really just a list with results from optim and additional information.
}
\description{
It is a wrapper for running APSIM-X and optimizing parameters using \code{\link[stats]{optim}}
}
\details{
Simple optimization for APSIM Next Generation

* At the moment it is required to provide starting values for the parameters of interest.

* It is suggested that you keep a backup of the original file. This function
will edit and overwrite the file during the optimization.

* When you use the parm.vector.index you cannot edit two separate elements of
a vector at the same time. This should be used to target a single element of
a vector only. (I can add this feature in the future if it is justified.)

* Internally, the optimization is done around the scaled value of the initial parameter
values. A value of 1 would correspond to the inital value of the parameter.
The \sQuote{lower} and \sQuote{upper} (or \sQuote{ub} and \sQuote{lb}) are also scaled
to the initial values of the parameters. So, for example, if your initial value is 20
and you provide an upper bound of 5, it means that the actual upper value that you are allowing for is 100.

* I have tested other optimizers and packages, but I think these are enough for most purposes. I tried
function stats::nlm (but it does not support bounds and it can fail), package 'optimx' is a bit messy and it
does not provide sufficient additional functionality. Package 'ucminf' seems like a good alternative, but it
did not perform better than the other ones.
}
\note{
When computing the objective function (residual sum-of-squares) different variables are combined.
It is common to weight them since they are in different units. If the argument weights is not supplied
no weighting is applied. It can be 'mean', 'variance' or a numeric vector of appropriate length.
}
\examples{
\donttest{
## See the vignette for examples
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{plot.met}
\alias{plot.met}
\title{Plot method for object of class \sQuote{met}}
\usage{
\method{plot}{met}(
  x,
  ...,
  years,
  met.var,
  plot.type = c("ts", "area", "col", "density", "anomaly"),
  cumulative = FALSE,
  facet = FALSE,
  climatology = FALSE,
  summary = FALSE,
  by = c("year", "month")
)
}
\arguments{
\item{x}{object of class \sQuote{met}}

\item{...}{additional arguments. None used at the moment.}

\item{years}{optional argument to subset years}

\item{met.var}{optional argument to choose a certain variable. By default,
temperature (min and max) is displayed}

\item{plot.type}{type of plot, default is \sQuote{ts} or time-series. 
The options \sQuote{area} and \sQuote{col} are only available when summary = TRUE.
Option \sQuote{density} produces a simple plot. Option \sQuote{anomaly} ignores
argument cumulative is treated as TRUE regardless.}

\item{cumulative}{default is FALSE. Especially useful for \sQuote{rain}.}

\item{facet}{whether to display the years in in different panels (facets). Not implemented yet.}

\item{climatology}{logical (default FALSE). Whether to display the \sQuote{climatology}
which would be the average of the data. 
Ideally, there are at least 20 years in the \sQuote{met} object.}

\item{summary}{whether to plot \sQuote{summary} data. (default FALSE).}

\item{by}{whether to perform summary by \sQuote{year} or \sQuote{month}.}
}
\description{
Some plots are similar to APSIM, others are different
and more useful in some respects
}
\examples{
\donttest{
## Read in and plot a met file
extd.dir <- system.file("extdata", package = "apsimx")
ames <- read_apsim_met("Ames.met", src.dir = extd.dir)
plot(ames, years = 2012:2015)
## Perhaps more informative
plot(ames, years = 2012:2015, cumulative = TRUE)
## for rain
plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE)
plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE, climatology = TRUE)
plot(ames, met.var = "rain", years = 2012:2015, plot.type = "anomaly")
## It is possible to add ggplot elements
library(ggplot2)
p1 <- plot(ames, met.var = "rain", years = 2012:2015, cumulative = TRUE)
p1 + ggtitle("Cumulative rain for 2012-2015")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apsim_met.R
\name{tt_apsim_met}
\alias{tt_apsim_met}
\title{Calculates Thermal Time taking a \sQuote{met} object}
\usage{
tt_apsim_met(
  met,
  dates,
  method = c("Classic_TT", "HeatStress_TT", "CropHeatUnit_TT", "APSIM_TT", "CERES_TT",
    "all"),
  x_temp = c(0, 26, 34),
  y_tt = c(0, 26, 0),
  base_temp = 0,
  max_temp = 30,
  dates.format = c("\%d-\%m")
)
}
\arguments{
\item{met}{object of class \sQuote{met}}

\item{dates}{when the calculation starts and when it ends. At the moment
it needs to be a character vector (e.g. c(\sQuote{01-05}, \sQuote{10-10})). It will
use the same dates every year for multiple years.}

\item{method}{one of \sQuote{Classic_TT}, \sQuote{HeatStress_TT}, \sQuote{ASPIM_TT},
\sQuote{CERES_TT} and \sQuote{all}}

\item{x_temp}{cardinal temperatures (base, optimal and maximum)}

\item{y_tt}{thermal time accumulation for cardinal temperatures}

\item{base_temp}{base temperature for Classic TT calculation}

\item{max_temp}{maximum temperature for Classic TT calculation}

\item{dates.format}{default is \sQuote{\%d-\%m} which means day and month}
}
\value{
it returns an object of class \sQuote{met} with additional columns
\sQuote{Date} and the corresponding TT calculation
}
\description{
Calculates Thermal Time using the \sQuote{Classic} formula, 
Heat Stress, Crop Heat Unit and other methods
}
\details{
Calculating Thermal Time using a variety of methods.
The function will fail if the method is not selected.
Also, it does not work if each year does not have at least
365 days.
}
\examples{
\dontrun{
require(nasapower)
require(ggplot2)

pwr <- get_power_apsim_met(lonlat = c(-93,42), dates = c("2012-01-01","2015-12-31"))
check_apsim_met(pwr)
pwr <- impute_apsim_met(pwr)

pwr2 <- tt_apsim_met(pwr, dates = c("01-05", "30-10"), method = c("Classic", "Heat"))

ggplot(data = pwr2, aes(x = Date, y = Classic_TT)) + geom_point()

ggplot(data = pwr2, aes(x = Date, y = HeatStress_TT)) + geom_point()

}
}
\references{
Abendroth, L.J., Miguez, F.E., Castellano, M.J. and Hatfield, J.L. (2019),
 Climate Warming Trends in the U.S. Midwest Using Four Thermal Models. 
 Agron. J., 111: 3230-3243. (doi:10.2134/agronj2019.02.0118)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_apsim.R
\name{edit_apsim}
\alias{edit_apsim}
\title{Edit an APSIM (Classic) Simulation}
\usage{
edit_apsim(
  file,
  src.dir = ".",
  wrt.dir = NULL,
  node = c("Clock", "Weather", "Soil", "SurfaceOrganicMatter", "MicroClimate", "Crop",
    "Manager", "Outputfile", "Other"),
  soil.child = c("Metadata", "Water", "Physical", "OrganicMatter", "Chemical",
    "Analysis", "InitialWater", "Sample", "SWIM"),
  manager.child = NULL,
  parm = NULL,
  value = NULL,
  overwrite = FALSE,
  edit.tag = "-edited",
  parm.path = NULL,
  root,
  verbose = TRUE,
  check.length = TRUE
)
}
\arguments{
\item{file}{file ending in .apsim to be edited}

\item{src.dir}{directory containing the .apsim file to be edited; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{node}{either \sQuote{Clock}, \sQuote{Weather}, \sQuote{Soil}, 
\sQuote{SurfaceOrganicMatter}, \sQuote{MicroClimate}, \sQuote{Crop}, \sQuote{Manager}, \sQuote{Outputfile} or \sQuote{Other}}

\item{soil.child}{specific soil component to be edited}

\item{manager.child}{specific manager component to be edited (not implemented yet)}

\item{parm}{parameter to be edited}

\item{value}{new values for the parameter to be edited}

\item{overwrite}{logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise}

\item{edit.tag}{if the file is edited a different tag from the default \sQuote{-edited} can be used.}

\item{parm.path}{path to the attribute to edit when node is \sQuote{Other}}

\item{root}{supply the node postion in the case of multiple simulations such as factorials.}

\item{verbose}{whether to print information about successful edit}

\item{check.length}{check whether vectors are of the correct length}
}
\value{
(when verbose=TRUE) complete file path to edited .apsimx file is returned as a character string.
As a side effect this function creates a new (XML) .apsimx file.
}
\description{
This function allows editing of an APSIM (Classic) simulation file.
}
\details{
The variables specified by \code{parm} within the .apsim file specified by \code{file} 
in the source directory \code{src.dir} are edited. The old values are replaced with \code{value}, which
is a list that has the same number of elements as the length of the vector \code{parm}.  The current
.apsim file will be overwritten if \code{overwrite} is set to \code{TRUE}; otherwise the file
\sQuote{file} \emph{-edited.apsim} will be created.  If (verbose = TRUE) then the name
 of the written file is returned. 
 
 When node equals Outputfile, the editing allows to add variables, but not to remove them at the moment.
}
\note{
The components that can be edited are restricted becuase this is better in preventing
errors of editing unintended parts of the file. The disadvantage is that there is less flexibility
compared to the similar function in the 'apsimr' package.
}
\examples{
\donttest{
## This example will read one of the examples distributed with APSIM
## but write to a temporary directory

tmp.dir <- tempdir()

extd.dir <- system.file("extdata", package = "apsimx")
edit_apsim("Millet", src.dir = extd.dir, wrt.dir = tmp.dir, 
           node = "Clock",
           parm = "start_date", value = "01/02/1940")

## Editing all of the KL values for Millet
pp.KL <- inspect_apsim_xml("Millet.apsim", src.dir = extd.dir, 
                  parm = "SoilCrop[8]/KL")
                 
kls <- seq(0.08, 0.2, length.out = 11)

edit_apsim("Millet.apsim", 
           src.dir = extd.dir,
           wrt.dir = tmp.dir,
           node = "Other", 
           parm.path = pp.KL,
           value = kls)
           
## Check that it was properly edited

inspect_apsim("Millet-edited.apsim", 
              src.dir = tmp.dir,
              node = "Soil",
              soil.child = "Water",
              parm = "KL")

              
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_apsimx_json.R
\name{grep_json_list}
\alias{grep_json_list}
\title{grep but for json list}
\usage{
grep_json_list(
  pattern,
  x,
  ignore.case = FALSE,
  search.depth = 10,
  how = c("unlist", "replace", "list")
)
}
\arguments{
\item{pattern}{as in grep}

\item{x}{object (a list)}

\item{ignore.case}{as in grep}

\item{search.depth}{search depth for the list (to prevent endless search)}

\item{how}{argument passed to \link{rapply}}
}
\value{
It returns a list with the found object, the json path and the positions in the list.
}
\description{
recursive grep adapted for a json list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{wop}
\alias{wop}
\title{Wheat example optimization results}
\format{
An object of class 'optim_apsim' 
\describe{
  \item{ wop }{ wheat optimization results }
}
}
\source{
Result of running the examples in Parameter Optimization vignette
}
\usage{
wop
}
\description{
Results from Wheat optimization example
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wget_apsimx_json.R
\name{insert_replacement_node}
\alias{insert_replacement_node}
\title{Inserts a replacement node in a simple apsimx simulation file}
\usage{
insert_replacement_node(
  file,
  src.dir,
  wrt.dir,
  rep.node,
  edit.tag = "-edited",
  overwrite = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{file}{file ending in .apsimx to be edited (JSON)}

\item{src.dir}{directory containing the .apsimx file to be edited; defaults to the current working directory}

\item{wrt.dir}{should be used if the destination directory is different from the src.dir}

\item{rep.node}{replacement node as obtained by the \code{\link{get_apsimx_json}} function}

\item{edit.tag}{if the file is edited a different tag from the default \sQuote{-edited} can be used.}

\item{overwrite}{logical; if \code{TRUE} the old file is overwritten, a new file is written otherwise}

\item{verbose}{whether to print information about successful edit}
}
\value{
it does not return an R object but it writes an apsimx file to disk
}
\description{
Inserts a replacement node in a simple apsimx simulation file
}
\examples{
\dontrun{
## It is not trivial to produce a reproducible example
## because the model and file versions need to align.
## The steps are:
## 1. Get model: 
##    wheat <- get_apsimx_json(model = "Wheat", wrt.dir = tmp.dir)
## 2. Create file that matches current model version
## 3. Edit the file by inserting the 'replacements' node
##    insert_replacement_node("Wheat.apsimx", rep.node = wheat)
                        
}

}
